﻿using System;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000015 RID: 21
	public static class AbilityTool
	{
		// Token: 0x060000D3 RID: 211 RVA: 0x00006B2C File Offset: 0x00004D2C
		public static void CheckAddPsylink(this Pawn p, int level = -1)
		{
			bool royaltyActive = ModsConfig.RoyaltyActive;
			if (royaltyActive)
			{
				Hediff_Psylink mainPsylinkSource = p.GetMainPsylinkSource();
				bool flag = p.psychicEntropy != null && p.psychicEntropy.Psylink == null;
				if (flag)
				{
					bool flag2 = mainPsylinkSource == null;
					if (flag2)
					{
						p.ChangePsylinkLevel((level > 0) ? level : 1, true);
						p.abilities.abilities.Clear();
						p.psychicEntropy.Notify_GainedPsylink();
						p.psychicEntropy.PsychicEntropyTrackerTick();
					}
					else
					{
						bool flag3 = level >= 0;
						if (flag3)
						{
							p.ChangePsylinkLevel(level - mainPsylinkSource.level, true);
						}
					}
				}
				else
				{
					bool flag4 = level >= 0;
					if (flag4)
					{
						p.ChangePsylinkLevel(level - mainPsylinkSource.level, true);
					}
				}
				mainPsylinkSource = p.GetMainPsylinkSource();
				bool flag5 = mainPsylinkSource != null;
				if (flag5)
				{
					p.health.Notify_HediffChanged(mainPsylinkSource);
				}
			}
		}

		// Token: 0x060000D4 RID: 212 RVA: 0x00006C11 File Offset: 0x00004E11
		public static float GetPsyfocus(this Pawn p)
		{
			return p.HasPsylink ? p.psychicEntropy.CurrentPsyfocus : 0f;
		}

		// Token: 0x060000D5 RID: 213 RVA: 0x00006C2D File Offset: 0x00004E2D
		public static float GetEntropy(this Pawn p)
		{
			return p.HasPsylink ? p.psychicEntropy.EntropyValue : 0f;
		}

		// Token: 0x060000D6 RID: 214 RVA: 0x00006C4C File Offset: 0x00004E4C
		public static void SetEntropy(this Pawn p, float val)
		{
			bool flag = !p.HasPsylink;
			if (!flag)
			{
				p.psychicEntropy.SetMemberValue("currentEntropy", val);
			}
		}

		// Token: 0x060000D7 RID: 215 RVA: 0x00006C80 File Offset: 0x00004E80
		public static void SetPsyfocus(this Pawn p, float val)
		{
			bool flag = !p.HasPsylink;
			if (!flag)
			{
				p.psychicEntropy.SetMemberValue("currentPsyfocus", val);
			}
		}

		// Token: 0x060000D8 RID: 216 RVA: 0x00006CB4 File Offset: 0x00004EB4
		public static string GetPsyAbilitiesAsSeparatedString(this Pawn p)
		{
			bool flag = !p.HasAbilityTracker();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				foreach (Ability ability in p.abilities.abilities)
				{
					text = text + ability.def.defName + ":";
				}
				text = text.SubstringRemoveLast();
				result = text;
			}
			return result;
		}

		// Token: 0x060000D9 RID: 217 RVA: 0x00006D4C File Offset: 0x00004F4C
		public static void SetPsyAbilitiesFromSeparatedString(this Pawn p, string s)
		{
			bool flag = !p.HasAbilityTracker();
			if (!flag)
			{
				p.abilities.abilities.Clear();
				string[] array = s.Split(new string[]
				{
					":"
				}, StringSplitOptions.None);
				foreach (string defName in array)
				{
					AbilityDef abilityDef = DefTool.AbilityDef(defName);
					bool flag2 = abilityDef != null;
					if (flag2)
					{
						p.abilities.GainAbility(abilityDef);
					}
				}
				p.abilities.Notify_TemporaryAbilitiesChanged();
			}
		}
	}
}
