﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  // Token: 0x02000020 RID: 32
  public static class ApparelTool
  {
    // Token: 0x06000134 RID: 308 RVA: 0x00009419 File Offset: 0x00007619
    public static bool IsApparelForNeck(this ThingDef a)
    {
      return a != null && a.apparel.layers != null && a.apparel.layers.Contains(ApparelLayerDefOf.Overhead) && a.HasBodyPartGroupDefName("Neck");
    }

    // Token: 0x06000135 RID: 309 RVA: 0x00009450 File Offset: 0x00007650
    public static bool IsForNeck(this Apparel a)
    {
      return a != null && a.def.apparel.layers != null && a.def.apparel.layers.Contains(ApparelLayerDefOf.Overhead) && a.def.HasBodyPartGroupDefName("Neck");
    }

    // Token: 0x06000136 RID: 310 RVA: 0x000094A1 File Offset: 0x000076A1
    public static bool IsForLegs(this Apparel a)
    {
      return a != null && !a.def.apparel.bodyPartGroups.NullOrEmpty<BodyPartGroupDef>() && a.def.apparel.bodyPartGroups.Contains(BodyPartGroupDefOf.Legs);
    }

    // Token: 0x06000137 RID: 311 RVA: 0x000094DA File Offset: 0x000076DA
    public static bool IsForEyes(this Apparel a)
    {
      return a != null && a.def.apparel.layers != null && a.def.apparel.layers.Contains(ApparelLayerDefOf.EyeCover);
    }

    // Token: 0x06000138 RID: 312 RVA: 0x0000950E File Offset: 0x0000770E
    public static bool HasAnyApparel(this Pawn pawn)
    {
      return pawn.HasApparelTracker() && !pawn.apparel.WornApparel.NullOrEmpty<Apparel>();
    }

    // Token: 0x06000139 RID: 313 RVA: 0x0000952E File Offset: 0x0000772E
    public static bool IsWornByPawn(this Pawn pawn, Apparel a)
    {
      return pawn.HasAnyApparel() && pawn.apparel.WornApparel.Contains(a);
    }

    // Token: 0x0600013A RID: 314 RVA: 0x0000954C File Offset: 0x0000774C
    public static bool HasThisApparelDef(this Pawn pawn, ThingDef t)
    {
      bool flag = pawn.HasAnyApparel();
      if (flag)
      {
        foreach (Apparel apparel in pawn.apparel.WornApparel)
        {
          bool flag2 = apparel.def == t;
          if (flag2)
          {
            return true;
          }
        }
      }
      return false;
    }

    // Token: 0x0600013B RID: 315 RVA: 0x000095C8 File Offset: 0x000077C8
    public static Apparel RandomWornApparel(this Pawn pawn)
    {
      return pawn.HasAnyApparel() ? pawn.apparel.WornApparel.RandomElement<Apparel>() : null;
    }

    // Token: 0x0600013C RID: 316 RVA: 0x000095E8 File Offset: 0x000077E8
    public static Apparel ThisOrFirstWornApparel(this Pawn pawn, Apparel apparel)
    {
      bool flag = pawn == null || pawn.apparel == null || pawn.apparel.WornApparel.NullOrEmpty<Apparel>();
      Apparel result;
      if (flag)
      {
        result = null;
      }
      else
      {
        bool flag2 = apparel == null || !pawn.apparel.WornApparel.Contains(apparel);
        if (flag2)
        {
          result = pawn.apparel.WornApparel.FirstOrFallback(null);
        }
        else
        {
          result = apparel;
        }
      }
      return result;
    }

    // Token: 0x0600013D RID: 317 RVA: 0x00009654 File Offset: 0x00007854
    public static string GetAllApparelAsSeparatedString(this Pawn p)
    {
      bool flag = !p.HasApparelTracker() || p.apparel.WornApparel.NullOrEmpty<Apparel>();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        string text = "";
        foreach (Apparel t in p.apparel.WornApparel)
        {
          text += t.GetAsSeparatedString();
          text += ":";
        }
        text = text.SubstringRemoveLast();
        result = text;
      }
      return result;
    }

    // Token: 0x0600013E RID: 318 RVA: 0x00009700 File Offset: 0x00007900
    public static void SetApparelFromSeparatedString(this Pawn p, string s)
    {
      bool flag = !p.HasApparelTracker();
      if (!flag)
      {
        p.apparel.DestroyAll(DestroyMode.Vanish);
        bool flag2 = s.NullOrEmpty();
        if (!flag2)
        {
          string[] array = s.SplitNo(":");
          foreach (string s2 in array)
          {
            string[] array3 = s2.SplitNo("|");
            bool flag3 = array3.Length >= 6;
            if (flag3)
            {
              string styledefname = (array3.Length >= 7) ? array3[6] : "";
              Apparel apparel = ApparelTool.GenerateApparel(Selected.ByName(array3[0], array3[1], styledefname, array3[2].HexStringToColor(), array3[3].AsInt32(), array3[4].AsInt32()));
              bool flag4 = apparel != null;
              if (flag4)
              {
                apparel.HitPoints = array3[5].AsInt32();
                p.WearThatApparel(apparel);
              }
            }
          }
        }
      }
    }

    // Token: 0x0600013F RID: 319 RVA: 0x000097F8 File Offset: 0x000079F8
    public static void TestAllApparel(this Pawn pawn)
    {
      bool flag = pawn.HasApparelTracker() && pawn.HasAnyApparel();
      if (flag)
      {
        List<Apparel> list = pawn.apparel.WornApparel.ToList<Apparel>();
        for (int i = list.Count - 1; i >= 0; i--)
        {
          Apparel apparel = pawn.apparel.WornApparel[i];
          bool flag2 = !pawn.ApparelGraphicTest(apparel, false, false);
          if (flag2)
          {
            pawn.TransferToInventory(apparel);
          }
        }
      }
    }

    // Token: 0x06000140 RID: 320 RVA: 0x00009878 File Offset: 0x00007A78
    public static void DestroyAllApparel(this Pawn pawn)
    {
      bool flag = !pawn.HasApparelTracker();
      if (!flag)
      {
        pawn.apparel.DestroyAll(DestroyMode.Vanish);
      }
    }

    // Token: 0x06000141 RID: 321 RVA: 0x000098A4 File Offset: 0x00007AA4
    public static void DestroyApparel(this Pawn pawn, Apparel apparel)
    {
      bool flag = !pawn.HasApparelTracker();
      if (!flag)
      {
        pawn.outfits.forcedHandler.SetForced(apparel, false);
        pawn.apparel.WornApparel.Remove(apparel);
        apparel.Destroy(DestroyMode.Vanish);
      }
    }

    // Token: 0x06000142 RID: 322 RVA: 0x000098EE File Offset: 0x00007AEE
    public static List<Apparel> ListOfCopyOutfits(this Pawn pawn)
    {
      return pawn.HasAnyApparel() ? pawn.apparel.WornApparel.ListFullCopy<Apparel>() : null;
    }

    // Token: 0x06000143 RID: 323 RVA: 0x0000990C File Offset: 0x00007B0C
    public static void PasteCopyOutfits(this Pawn pawn, List<Apparel> l)
    {
      bool flag = pawn.HasApparelTracker();
      if (flag)
      {
        pawn.apparel.WornApparel.Clear();
        bool flag2 = !l.NullOrEmpty<Apparel>();
        if (flag2)
        {
          foreach (Apparel thing in l)
          {
            pawn.CreateAndWearApparel(Selected.ByThing(thing));
          }
        }
      }
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x06000144 RID: 324 RVA: 0x0000999C File Offset: 0x00007B9C
    public static List<BodyPartGroupDef> GetCoveredBodyPartGroupDefs(this Pawn pawn)
    {
      List<BodyPartGroupDef> list = new List<BodyPartGroupDef>();
      foreach (Apparel apparel in pawn.apparel.WornApparel)
      {
        list.AddFromList(apparel.def.apparel.bodyPartGroups);
      }
      return list;
    }

    // Token: 0x06000145 RID: 325 RVA: 0x00009A14 File Offset: 0x00007C14
    public static List<Apparel> GetApparelWithThisLayer(this Pawn pawn, ApparelLayerDef ald)
    {
      return (from td in pawn.apparel.WornApparel
              where td.def.apparel.layers.Contains(ald)
              select td).ToList<Apparel>();
    }

    // Token: 0x06000146 RID: 326 RVA: 0x00009A54 File Offset: 0x00007C54
    public static int CountApparelThatWillBeReplacedByThisApparel(this Pawn pawn, ThingDef apparelDef)
    {
      int num = 0;
      try
      {
        for (int i = 0; i < pawn.apparel.WornApparel.Count; i++)
        {
          bool flag = !ApparelUtility.CanWearTogether(apparelDef, pawn.apparel.WornApparel[i].def, pawn.RaceProps.body);
          if (flag)
          {
            num++;
          }
        }
      }
      catch
      {
      }
      return num;
    }

    // Token: 0x06000147 RID: 327 RVA: 0x00009AD8 File Offset: 0x00007CD8
    public static bool CreateAndWearApparel(this Pawn pawn, Selected s)
    {
      Apparel apparel;
      return pawn.CreateAndWearApparel(s, out apparel, false);
    }

    // Token: 0x06000148 RID: 328 RVA: 0x00009AF0 File Offset: 0x00007CF0
    public static bool CreateAndWearApparel(this Pawn pawn, Selected s, out Apparel a, bool showError)
    {
      a = null;
      bool flag = pawn.HasApparelTracker();
      if (flag)
      {
        a = ApparelTool.GenerateApparel(s);
        bool flag2 = pawn.CanWearApparel(a, showError);
        if (flag2)
        {
          pawn.WearThatApparel(a);
          return true;
        }
      }
      return false;
    }

    // Token: 0x06000149 RID: 329 RVA: 0x00009B38 File Offset: 0x00007D38
    public static void ReplaceAndWearRandomApparel(this Pawn pawn, Apparel a, ApparelLayerDef ald = null, bool pawnSpecific = false)
    {
      bool flag = pawn.HasApparelTracker();
      if (flag)
      {
        ApparelLayerDef apparelLayerDef = (a != null) ? a.def.apparel.layers.FirstOrDefault<ApparelLayerDef>() : ald;
        BodyPartGroupDef bodyPartGroupDef = (a != null) ? a.def.apparel.bodyPartGroups.FirstOrDefault<BodyPartGroupDef>() : null;
        bool flag2 = pawn.GetApparelWithThisLayer(apparelLayerDef).CountAllowNull<Apparel>() == 1;
        if (flag2)
        {
          string text = (bodyPartGroupDef != null && bodyPartGroupDef.defName != null) ? bodyPartGroupDef.defName.ToLower() : "";
          bool flag3 = !text.StartsWith("left") && !text.StartsWith("right") && !text.EndsWith("left") && !text.EndsWith("right");
          if (flag3)
          {
            bodyPartGroupDef = null;
          }
        }
        List<ThingDef> hashSet = ApparelTool.ListOfApparel(apparelLayerDef, null, bodyPartGroupDef);
        bool hasApparelTags = pawn.kindDef.apparelTags != null;
        if (pawnSpecific)
        {
          hashSet = (from td in hashSet
                     where td.IsFromMod(pawn.kindDef.GetModName()) || (hasApparelTags && td.apparel.tags != null && (from t in td.apparel.tags
                                                                                                                     select pawn.kindDef.apparelTags.Contains(t)) != null)

                     orderby td.label
                     select td).ToList<ThingDef>();
        }
        Selected selected = null;
        int num = 10;
        bool flag4 = hashSet.Count <= 1 && ald == ApparelLayerDefOf.EyeCover;
        if (!flag4)
        {
          do
          {
            for (int i = 0; i < 5; i++)
            {
              selected = Selected.Random(hashSet, Event.current.alt, false);
              bool flag5 = pawn.CountApparelThatWillBeReplacedByThisApparel(selected.thingDef) <= 1;
              if (flag5)
              {
                break;
              }
            }
            num--;
          }
          while (!pawn.CreateAndWearApparel(selected) && num > 0);
        }
      }
    }

    // Token: 0x0600014A RID: 330 RVA: 0x00009D20 File Offset: 0x00007F20
    public static bool CanWearApparel(this Pawn pawn, Apparel a, bool showError)
    {
      bool result;
      try
      {
        bool flag = pawn.HasApparelTracker() && a != null && ApparelUtility.HasPartsToWear(pawn, a.def);
        if (flag)
        {
          bool flag2 = pawn.IsAnimal();
          if (flag2)
          {
            result = true;
          }
          else
          {
            result = pawn.ApparelGraphicTest(a, showError, false);
          }
        }
        else
        {
          result = false;
        }
      }
      catch
      {
        result = pawn.apparel.CanWearWithoutDroppingAnything(a.def);
      }
      return result;
    }

    // Token: 0x0600014B RID: 331 RVA: 0x00009D94 File Offset: 0x00007F94
    public static void ReplaceThatApparel(this Pawn pawn, Apparel old, Apparel neu)
    {
      bool flag = pawn.HasApparelTracker() && pawn.story != null && neu != null && ApparelUtility.HasPartsToWear(pawn, neu.def);
      if (flag)
      {
        pawn.AllowAllApparel(neu);
        pawn.apparel.Remove(old);
        ((ThingOwner<Apparel>)pawn.apparel.GetMemberValue("wornApparel")).TryAdd(neu, true);
      }
    }

    // Token: 0x0600014C RID: 332 RVA: 0x00009DFC File Offset: 0x00007FFC
    public static void WearThatApparel(this Pawn pawn, Apparel a)
    {
      bool flag = pawn.HasApparelTracker() && a != null && ApparelUtility.HasPartsToWear(pawn, a.def);
      if (flag)
      {
        pawn.AllowAllApparel(a);
        pawn.apparel.Wear(a, false, false);
      }
    }

    // Token: 0x0600014D RID: 333 RVA: 0x00009E44 File Offset: 0x00008044
    public static void AskToWearIncompatibleApparel(this Pawn pawn, Apparel a)
    {
      bool flag = !pawn.HasStoryTracker();
      if (!flag)
      {
        bool flag2 = (a.def.IsFromCoreMod() || a.def.IsFromMod("Royalty")) && !pawn.story.bodyType.IsFromCoreMod() && !pawn.story.bodyType.IsFromMod("Royalty");
        if (flag2)
        {
          WindowTool.Open(Dialog_MessageBox.CreateConfirmation("core or royalty apparel that is not compatible to the current body type found - try to wear it anyway?\n\nif you do, you will get texture error messages on savegame load. but they should be harmless.", delegate
          {
            ApparelGraphicRecord apparelGraphicRecord;
            ApparelGraphicRecordGetter.TryGetGraphicApparel(a, pawn.story.bodyType, out apparelGraphicRecord);
            pawn.AllowAllApparel(a);
            pawn.apparel.Wear(a, false, false);
            CEditor.API.UpdateGraphics();
          }, false, null, WindowLayer.Dialog));
        }
      }
    }

    // Token: 0x0600014E RID: 334 RVA: 0x00009F0C File Offset: 0x0000810C
    public static Apparel GenerateRandomApparel(ApparelLayerDef ald = null, bool originalColors = true, string modname = null, BodyPartGroupDef bpd = null)
    {
      HashSet<ThingDef> hashSet = CEditor.API.Get<Dictionary<ApparelLayerDef, List<ThingDef>>>(EType.DicApparel).GetValue(ald).ToHashSet<ThingDef>();
      bool flag = modname != null || bpd != null;
      if (flag)
      {
        hashSet = (from td in hashSet
                   where (modname == null || td.IsFromMod(modname)) && (bpd == null || td.apparel.bodyPartGroups.Contains(bpd))
                   select td).ToHashSet<ThingDef>();
      }
      return ApparelTool.GenerateApparel(Selected.Random(hashSet, originalColors, false));
    }

    // Token: 0x0600014F RID: 335 RVA: 0x00009F88 File Offset: 0x00008188
    public static Apparel GenerateApparel(Selected s)
    {
      bool flag = s == null || s.thingDef == null;
      Apparel result;
      if (flag)
      {
        result = null;
      }
      else
      {
        ThingDef thingDef = DefTool.ThingDef(s.thingDef.defName);
        bool flag2 = thingDef == null || !thingDef.IsApparel;
        if (flag2)
        {
          result = null;
        }
        else
        {
          s.Stuff = s.thingDef.ThisOrDefaultStuff(s.Stuff);
          bool flag3 = !s.thingDef.MadeFromStuff;
          if (flag3)
          {
            s.Stuff = null;
          }
          Apparel apparel = (Apparel)ThingMaker.MakeThing(s.thingDef, s.Stuff);
          apparel.HitPoints = apparel.MaxHitPoints;
          apparel.SetQuality(s.quality);
          apparel.SetDrawColor(s.DrawColor);
          apparel.stackCount = s.stackVal;
          bool flag4 = s.Style != null;
          if (flag4)
          {
            apparel.StyleDef = s.Style;
            apparel.StyleDef.color = s.Style.color;
          }
          result = apparel;
        }
      }
      return result;
    }

    // Token: 0x06000150 RID: 336 RVA: 0x0000A094 File Offset: 0x00008294
    public static List<Apparel> GetConflictedApparelList(this Pawn pawn, ThingDef apparelToCheck)
    {
      List<Apparel> list = new List<Apparel>();
      try
      {
        foreach (BodyPartGroupDef bodyPartGroupDef in apparelToCheck.apparel.bodyPartGroups)
        {
          foreach (Apparel apparel in CEditor.API.Pawn.apparel.WornApparel)
          {
            bool flag = !ApparelUtility.CanWearTogether(apparel.def, apparelToCheck, CEditor.API.Pawn.RaceProps.body);
            if (flag)
            {
              bool flag2 = !list.Contains(apparel);
              if (flag2)
              {
                list.Add(apparel);
              }
            }
          }
        }
      }
      catch
      {
      }
      return list;
    }

    // Token: 0x06000151 RID: 337 RVA: 0x0000A1A8 File Offset: 0x000083A8
    public static bool RenderAsPack(ThingDef a)
    {
      bool flag = !a.apparel.LastLayer.IsUtilityLayer;
      return !flag && (a.apparel.wornGraphicData == null || a.apparel.wornGraphicData.renderUtilityAsPack);
    }

    // Token: 0x06000152 RID: 338 RVA: 0x0000A1F8 File Offset: 0x000083F8
    public static string GetApparelPath(ThingDef a, Pawn p)
    {
      string text = "";
      bool flag = a != null && a.apparel != null;
      if (flag)
      {
        bool flag2 = false;
        bool flag3 = a.apparel.layers != null && a.apparel.layers.Contains(ApparelLayerDefOf.Overhead);
        if (flag3)
        {
          text = a.apparel.wornGraphicPath;
          flag2 = true;
        }
        bool flag4 = string.IsNullOrEmpty(text) && a.apparel.wornGraphicPath.NullOrEmpty();
        if (flag4)
        {
          text = a.apparel.wornGraphicPath;
          flag2 = true;
        }
        bool flag5 = string.IsNullOrEmpty(text) && ApparelTool.RenderAsPack(a);
        if (flag5)
        {
          text = a.apparel.wornGraphicPath;
          flag2 = true;
        }
        bool flag6 = string.IsNullOrEmpty(text) && a.apparel.wornGraphicPath == BaseContent.PlaceholderImagePath;
        if (flag6)
        {
          text = a.apparel.wornGraphicPath;
          flag2 = true;
        }
        bool flag7 = string.IsNullOrEmpty(text) && !flag2 && p.story != null && p.story.bodyType != null && p.story.bodyType.defName != null;
        if (flag7)
        {
          text = a.apparel.wornGraphicPath + "_" + p.story.bodyType.defName;
        }
      }
      return text;
    }

    // Token: 0x06000153 RID: 339 RVA: 0x0000A358 File Offset: 0x00008558
    public static bool ApparalGraphicTest2(this Pawn pawn, ThingDef a, bool showError)
    {
      bool flag = false;
      string path = "";
      bool flag2 = pawn.HasApparelTracker() && a != null && ApparelUtility.HasPartsToWear(pawn, a);
      if (flag2)
      {
        bool flag3 = a.apparel.layers.Contains(ApparelLayerDefOf.Overhead) || a.apparel.layers.Contains(ApparelLayerDefOf.EyeCover) || a.apparel.layers.Contains(ApparelLayerDefOf.Belt);
        if (flag3)
        {
          return true;
        }
        bool flag4 = (a.IsFromCoreMod() || a.IsFromMod("Royalty") || a.IsFromMod("Ideology")) && (pawn.story.bodyType.IsFromCoreMod() || pawn.story.bodyType.IsFromMod("Royalty") || pawn.story.bodyType.IsFromMod("Ideology"));
        if (flag4)
        {
          flag = true;
        }
        else
        {
          path = ApparelTool.GetApparelPath(a, pawn);
          try
          {
            HashSet<string> source = CEditor.API.Get<HashSet<string>>(EType.GraphicPaths);
            List<string> list = (from td in source
                                 where td == path
                                 select td).ToList<string>();
            bool flag5 = !list.NullOrEmpty<string>();
            if (flag5)
            {
              flag = true;
            }
            else
            {
              string value = (a.apparel.wornGraphicPath.Contains(a.defName) ? a.defName : "") + "_" + pawn.story.bodyType.defName;
              List<Texture2D> list2 = ContentFinder<Texture2D>.GetAllInFolder(a.apparel.wornGraphicPath).ToList<Texture2D>();
              foreach (Texture2D texture2D in list2)
              {
                bool flag6 = texture2D.name.Contains(value);
                if (flag6)
                {
                  flag = true;
                  break;
                }
              }
            }
            bool flag7 = !flag && a.apparel.layers.Contains(ApparelLayerDefOf.Overhead) && a.apparel.layers.Count == 1;
            if (flag7)
            {
              flag = true;
            }
          }
          catch
          {
          }
        }
      }
      bool flag8 = !flag && showError;
      if (flag8)
      {
        MessageTool.Show(Label.NOT_COMPATIBLE_APPAREL + ", missing texture=" + path, MessageTypeDefOf.RejectInput);
      }
      return flag;
    }

    // Token: 0x06000154 RID: 340 RVA: 0x0000A600 File Offset: 0x00008800
    public static bool ApparelGraphicTest(this Pawn pawn, Apparel a, bool showError, bool force = false)
    {
      bool flag = !force && !CEditor.API.GetO(OptionB.DOAPPARELCHECK);
      return flag || pawn.ApparalGraphicTest2(a.def, showError);
    }

    // Token: 0x06000155 RID: 341 RVA: 0x0000A63C File Offset: 0x0000883C
    public static bool HasSameCover(this ThingDef a, ThingDef b)
    {
      bool flag = a == null || b == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        foreach (BodyPartGroupDef bodyPartGroupDef in a.apparel.bodyPartGroups)
        {
          foreach (BodyPartGroupDef bodyPartGroupDef2 in b.apparel.bodyPartGroups)
          {
            bool flag2 = bodyPartGroupDef == bodyPartGroupDef2;
            if (flag2)
            {
              return true;
            }
          }
        }
        result = false;
      }
      return result;
    }

    // Token: 0x06000156 RID: 342 RVA: 0x0000A704 File Offset: 0x00008904
    public static bool HasSameApparelLayer(this ThingDef a, ThingDef b)
    {
      bool flag = a == null || b == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        foreach (ApparelLayerDef apparelLayerDef in a.apparel.layers)
        {
          foreach (ApparelLayerDef apparelLayerDef2 in b.apparel.layers)
          {
            bool flag2 = apparelLayerDef == apparelLayerDef2;
            if (flag2)
            {
              return true;
            }
          }
        }
        result = false;
      }
      return result;
    }

    // Token: 0x06000157 RID: 343 RVA: 0x0000A7CC File Offset: 0x000089CC
    public static void PasteApparelLayer(this ThingDef t, List<ApparelLayerDef> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.apparel.layers == null;
        if (flag2)
        {
          t.apparel.layers = new List<ApparelLayerDef>();
        }
        foreach (ApparelLayerDef apparelLayerDef in l)
        {
          t.AddApparelLayer(apparelLayerDef);
        }
      }
    }

    // Token: 0x06000158 RID: 344 RVA: 0x0000A850 File Offset: 0x00008A50
    public static bool HasApparelLayer(this ThingDef t, ApparelLayerDef apparelLayerDef)
    {
      bool flag = t != null && t.apparel.layers != null;
      return flag && t.apparel.layers.Contains(apparelLayerDef);
    }

    // Token: 0x06000159 RID: 345 RVA: 0x0000A890 File Offset: 0x00008A90
    public static void AddApparelLayer(this ThingDef t, ApparelLayerDef apparelLayerDef)
    {
      bool flag = t != null && apparelLayerDef != null;
      if (flag)
      {
        bool flag2 = t.apparel.layers == null;
        if (flag2)
        {
          t.apparel.layers = new List<ApparelLayerDef>();
        }
        bool flag3 = !t.HasApparelLayer(apparelLayerDef);
        if (flag3)
        {
          t.apparel.layers.Add(apparelLayerDef);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x0600015A RID: 346 RVA: 0x0000A8F8 File Offset: 0x00008AF8
    public static void RemoveApparelLayer(this ThingDef t, ApparelLayerDef apparelLayerDef)
    {
      bool flag = t != null && t.apparel.layers != null;
      if (flag)
      {
        foreach (ApparelLayerDef apparelLayerDef2 in t.apparel.layers)
        {
          bool flag2 = apparelLayerDef2 == apparelLayerDef;
          if (flag2)
          {
            t.apparel.layers.Remove(apparelLayerDef2);
            break;
          }
        }
        bool flag3 = t.apparel.layers.Count == 0;
        if (flag3)
        {
          t.apparel.layers = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x0600015B RID: 347 RVA: 0x0000A9B4 File Offset: 0x00008BB4
    public static void PasteBodyPartGroup(this ThingDef t, List<BodyPartGroupDef> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.apparel.bodyPartGroups == null;
        if (flag2)
        {
          t.apparel.bodyPartGroups = new List<BodyPartGroupDef>();
        }
        foreach (BodyPartGroupDef bodyPartGroupDef in l)
        {
          t.AddBodyPart(bodyPartGroupDef);
        }
      }
    }

    // Token: 0x0600015C RID: 348 RVA: 0x0000AA38 File Offset: 0x00008C38
    public static bool HasBodyPartGroupDefName(this ThingDef t, string defName)
    {
      bool flag = t != null && t.apparel.bodyPartGroups != null;
      if (flag)
      {
        foreach (BodyPartGroupDef bodyPartGroupDef in t.apparel.bodyPartGroups)
        {
          bool flag2 = bodyPartGroupDef.defName == defName;
          if (flag2)
          {
            return true;
          }
        }
      }
      return false;
    }

    // Token: 0x0600015D RID: 349 RVA: 0x0000AAC4 File Offset: 0x00008CC4
    public static bool HasBodyPartGroup(this ThingDef t, BodyPartGroupDef bodyPartGroupDef)
    {
      bool flag = t != null && t.apparel.bodyPartGroups != null;
      return flag && t.apparel.bodyPartGroups.Contains(bodyPartGroupDef);
    }

    // Token: 0x0600015E RID: 350 RVA: 0x0000AB04 File Offset: 0x00008D04
    public static void AddBodyPart(this ThingDef t, BodyPartGroupDef bodyPartGroupDef)
    {
      bool flag = t != null && bodyPartGroupDef != null;
      if (flag)
      {
        bool flag2 = t.apparel.bodyPartGroups == null;
        if (flag2)
        {
          t.apparel.bodyPartGroups = new List<BodyPartGroupDef>();
        }
        bool flag3 = !t.HasBodyPartGroup(bodyPartGroupDef);
        if (flag3)
        {
          t.apparel.bodyPartGroups.Add(bodyPartGroupDef);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x0600015F RID: 351 RVA: 0x0000AB6C File Offset: 0x00008D6C
    public static void RemoveBodyPartGroup(this ThingDef t, BodyPartGroupDef bodyPartGroupDef)
    {
      bool flag = t != null && t.apparel.bodyPartGroups != null;
      if (flag)
      {
        foreach (BodyPartGroupDef bodyPartGroupDef2 in t.apparel.bodyPartGroups)
        {
          bool flag2 = bodyPartGroupDef2 == bodyPartGroupDef;
          if (flag2)
          {
            t.apparel.bodyPartGroups.Remove(bodyPartGroupDef2);
            break;
          }
        }
        bool flag3 = t.apparel.bodyPartGroups.Count == 0;
        if (flag3)
        {
          t.apparel.bodyPartGroups = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000160 RID: 352 RVA: 0x0000AC28 File Offset: 0x00008E28
    public static void PasteApparelTag(this ThingDef t, List<string> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.apparel.tags == null;
        if (flag2)
        {
          t.apparel.tags = new List<string>();
        }
        foreach (string tag in l)
        {
          t.AddApparelTag(tag);
        }
      }
    }

    // Token: 0x06000161 RID: 353 RVA: 0x0000ACAC File Offset: 0x00008EAC
    public static bool HasApparelTag(this ThingDef t, string tag)
    {
      bool flag = t != null && t.apparel.tags != null;
      return flag && t.apparel.tags.Contains(tag);
    }

    // Token: 0x06000162 RID: 354 RVA: 0x0000ACEC File Offset: 0x00008EEC
    public static void AddApparelTag(this ThingDef t, string tag)
    {
      bool flag = t != null && tag != null;
      if (flag)
      {
        bool flag2 = t.apparel.tags == null;
        if (flag2)
        {
          t.apparel.tags = new List<string>();
        }
        bool flag3 = !t.HasApparelTag(tag);
        if (flag3)
        {
          t.apparel.tags.Add(tag);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000163 RID: 355 RVA: 0x0000AD54 File Offset: 0x00008F54
    public static void RemoveApparelTag(this ThingDef t, string tag)
    {
      bool flag = t != null && t.apparel.tags != null;
      if (flag)
      {
        foreach (string text in t.apparel.tags)
        {
          bool flag2 = text == tag;
          if (flag2)
          {
            t.apparel.tags.Remove(text);
            break;
          }
        }
        bool flag3 = t.apparel.tags.Count == 0;
        if (flag3)
        {
          t.apparel.tags = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000164 RID: 356 RVA: 0x0000AE14 File Offset: 0x00009014
    public static void PasteOutfitTag(this ThingDef t, List<string> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.apparel.defaultOutfitTags == null;
        if (flag2)
        {
          t.apparel.defaultOutfitTags = new List<string>();
        }
        foreach (string tag in l)
        {
          t.AddOutfitTag(tag);
        }
      }
    }

    // Token: 0x06000165 RID: 357 RVA: 0x0000AE98 File Offset: 0x00009098
    public static bool HasOutfitTag(this ThingDef t, string tag)
    {
      bool flag = t != null && t.apparel.defaultOutfitTags != null;
      return flag && t.apparel.defaultOutfitTags.Contains(tag);
    }

    // Token: 0x06000166 RID: 358 RVA: 0x0000AED8 File Offset: 0x000090D8
    public static void AddOutfitTag(this ThingDef t, string tag)
    {
      bool flag = t != null && tag != null;
      if (flag)
      {
        bool flag2 = t.apparel.defaultOutfitTags == null;
        if (flag2)
        {
          t.apparel.defaultOutfitTags = new List<string>();
        }
        bool flag3 = !t.HasOutfitTag(tag);
        if (flag3)
        {
          t.apparel.defaultOutfitTags.Add(tag);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000167 RID: 359 RVA: 0x0000AF40 File Offset: 0x00009140
    public static void RemoveOutfitTag(this ThingDef t, string tag)
    {
      bool flag = t != null && t.apparel.defaultOutfitTags != null;
      if (flag)
      {
        foreach (string text in t.apparel.defaultOutfitTags)
        {
          bool flag2 = text == tag;
          if (flag2)
          {
            t.apparel.defaultOutfitTags.Remove(text);
            break;
          }
        }
        bool flag3 = t.apparel.defaultOutfitTags.Count == 0;
        if (flag3)
        {
          t.apparel.defaultOutfitTags = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000168 RID: 360 RVA: 0x0000B000 File Offset: 0x00009200
    public static bool RenderAsPack(Apparel apparel)
    {
      bool flag = apparel.def.apparel.LastLayer != ApparelLayerDefOf.Belt;
      return !flag && (apparel.def.apparel.wornGraphicData == null || apparel.def.apparel.wornGraphicData.renderUtilityAsPack);
    }

    // Token: 0x06000169 RID: 361 RVA: 0x0000B060 File Offset: 0x00009260
    public static void FixForBelts(ThingDef t, StatDef stat)
    {
      bool flag = t == null || t.apparel == null;
      if (!flag)
      {
        bool flag2 = stat == StatDefOf.EnergyShieldEnergyMax || stat == StatDefOf.EnergyShieldRechargeRate;
        if (flag2)
        {
          MessageTypeDef mt = t.apparel.layers.Contains(ApparelLayerDefOf.Belt) ? MessageTypeDefOf.SilentInput : MessageTypeDefOf.RejectInput;
          MessageTool.Show(Label.ONLYFORSHIELD, mt);
          //t.thingClass = typeof(ShieldBelt);
          t.comps.AddDistinct<CompProperties>(new CompProperties_Shield());
          t.ResolveReferences();
          t.PostLoad();
        }
      }
    }

    // Token: 0x0600016A RID: 362 RVA: 0x0000B0EC File Offset: 0x000092EC
    public static List<ApparelLayerDef> ListOfRandomApparelLayerDefs(int numberOfLayers = -1)
    {
      List<ApparelLayerDef> list = ApparelTool.ListOfApparelLayerDefs(false);
      numberOfLayers = ((numberOfLayers < 0) ? CEditor.zufallswert.Next(2, list.Count) : numberOfLayers);
      numberOfLayers = ((numberOfLayers > 8) ? 8 : numberOfLayers);
      return list.TakeRandom(numberOfLayers).ToList<ApparelLayerDef>();
    }

    // Token: 0x0600016B RID: 363 RVA: 0x0000B138 File Offset: 0x00009338
    public static void Redress(this Pawn pawn, Selected selected, bool originalColors, int numberOfLayers = 1, bool pawnSpecific = false)
    {
      bool flag = !pawn.HasApparelTracker();
      if (!flag)
      {
        bool flag2 = selected == null;
        if (flag2)
        {
          pawn.DestroyAllApparel();
          List<ApparelLayerDef> list = ApparelTool.ListOfRandomApparelLayerDefs(numberOfLayers);
          if (pawnSpecific)
          {
            bool flag3 = !list.Contains(ApparelLayerDefOf.OnSkin);
            if (flag3)
            {
              list.Add(ApparelLayerDefOf.OnSkin);
            }
            bool flag4 = !list.Contains(ApparelLayerDefOf.Shell);
            if (flag4)
            {
              list.Add(ApparelLayerDefOf.Shell);
            }
          }
          Event.current.alt = originalColors;
          foreach (ApparelLayerDef ald in list)
          {
            pawn.ReplaceAndWearRandomApparel(null, ald, pawnSpecific);
          }
          Event.current.alt = false;
        }
        else
        {
          pawn.CreateAndWearApparel(selected);
        }
      }
    }

    // Token: 0x0600016C RID: 364 RVA: 0x0000B22C File Offset: 0x0000942C
    public static void AllowAllApparel(this Pawn pawn, Apparel apparel = null)
    {
      bool flag = pawn == null;
      if (!flag)
      {
        bool flag2 = pawn.outfits == null;
        if (flag2)
        {
          pawn.outfits = new Pawn_OutfitTracker(pawn);
        }
        bool flag3 = pawn.outfits.CurrentOutfit == null;
        if (flag3)
        {
          pawn.outfits.CurrentOutfit = new Outfit();
        }
        bool flag4 = pawn.apparel == null;
        if (flag4)
        {
          pawn.apparel = new Pawn_ApparelTracker(pawn);
        }
        bool flag5 = apparel != null;
        if (flag5)
        {
          pawn.outfits.CurrentOutfit.filter.SetAllow(apparel.def, true);
        }
        else
        {
          bool flag6 = !pawn.apparel.WornApparel.NullOrEmpty<Apparel>();
          if (flag6)
          {
            foreach (Apparel apparel2 in pawn.apparel.WornApparel)
            {
              pawn.outfits.CurrentOutfit.filter.SetAllow(apparel2.def, true);
            }
          }
        }
      }
    }

    // Token: 0x0600016D RID: 365 RVA: 0x0000B34C File Offset: 0x0000954C
    public static Dictionary<ApparelLayerDef, List<ThingDef>> CreateDicOfApparel()
    {
      Dictionary<ApparelLayerDef, List<ThingDef>> dictionary = new Dictionary<ApparelLayerDef, List<ThingDef>>();
      List<ApparelLayerDef> list = ApparelTool.ListOfApparelLayerDefs(false);
      foreach (ApparelLayerDef apparelLayerDef in list)
      {
        List<ThingDef> value = ApparelTool.ListOfApparel(apparelLayerDef, null, null);
        dictionary.Add(apparelLayerDef, value);
      }
      return dictionary;
    }

    // Token: 0x0600016E RID: 366 RVA: 0x0000B3C0 File Offset: 0x000095C0
    public static HashSet<string> CreateListOfGraphicPaths()
    {
      Dictionary<GraphicRequest, Graphic> source = (Dictionary<GraphicRequest, Graphic>)typeof(GraphicDatabase).GetMemberValue("allGraphics");
      return source.Select(delegate (KeyValuePair<GraphicRequest, Graphic> td)
      {
        KeyValuePair<GraphicRequest, Graphic> keyValuePair = td;
        return keyValuePair.Value.path;
      }).ToHashSet<string>();
    }

    // Token: 0x0600016F RID: 367 RVA: 0x0000B418 File Offset: 0x00009618
    public static List<ThingDef> GetAllOnSkn()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsApparel && td.apparel.LastLayer == ApparelLayerDefOf.OnSkin
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000170 RID: 368 RVA: 0x0000B47C File Offset: 0x0000967C
    public static List<ThingDef> GetAllBelt()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsApparel && td.apparel.LastLayer == ApparelLayerDefOf.Belt
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000171 RID: 369 RVA: 0x0000B4E0 File Offset: 0x000096E0
    public static List<ThingDef> GetAllMiddle()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsApparel && td.apparel.LastLayer == ApparelLayerDefOf.Middle
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000172 RID: 370 RVA: 0x0000B544 File Offset: 0x00009744
    public static List<ThingDef> GetAllOverhead()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsApparel && td.apparel.LastLayer == ApparelLayerDefOf.Overhead
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000173 RID: 371 RVA: 0x0000B5A8 File Offset: 0x000097A8
    public static List<ThingDef> GetAllShell()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsApparel && td.apparel.LastLayer == ApparelLayerDefOf.Shell
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000174 RID: 372 RVA: 0x0000B60C File Offset: 0x0000980C
    public static List<ApparelLayerDef> ListOfApparelLayerDefs(bool insertNull)
    {
      List<ApparelLayerDef> list = (from td in DefDatabase<ApparelLayerDef>.AllDefs
                                    where td.IsFromCoreMod()
                                    orderby td.label
                                    select td).ToList<ApparelLayerDef>();
      list.AddRange((from td in DefDatabase<ApparelLayerDef>.AllDefs
                     where !td.IsFromCoreMod()
                     orderby td.label
                     select td).ToList<ApparelLayerDef>());
      List<ApparelLayerDef> list2 = new List<ApparelLayerDef>();
      foreach (ApparelLayerDef apparelLayerDef in list)
      {
        bool flag = !ApparelTool.ListOfApparel(apparelLayerDef, null, null).NullOrEmpty<ThingDef>();
        if (flag)
        {
          list2.Add(apparelLayerDef);
        }
      }
      if (insertNull)
      {
        list2.Insert(0, null);
      }
      return list2;
    }

    // Token: 0x06000175 RID: 373 RVA: 0x0000B740 File Offset: 0x00009940
    public static List<ThingDef> ListOfApparel(ApparelLayerDef ld, string modname, BodyPartGroupDef bpd)
    {
      bool bAll1 = modname.NullOrEmpty();
      bool bAll2 = ld == null;
      bool bAll3 = bpd == null;
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsApparel && !td.defName.NullOrEmpty() && !td.label.NullOrEmpty() && (bAll1 || td.IsFromMod(modname)) && (bAll2 || (!td.apparel.layers.NullOrEmpty<ApparelLayerDef>() && td.apparel.layers.Contains(ld))) && (bAll3 || (!td.apparel.bodyPartGroups.NullOrEmpty<BodyPartGroupDef>() && td.apparel.bodyPartGroups.Contains(bpd)))
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000176 RID: 374 RVA: 0x0000B7E0 File Offset: 0x000099E0
    public static void MoveDressToInv(this Pawn p, ApparelLayerDef layerOnly)
    {
      bool flag = !CEditor.API.Pawn.HasApparelTracker() || !CEditor.API.Pawn.HasInventoryTracker();
      if (!flag)
      {
        int num = CEditor.API.Pawn.HasApparelTracker() ? CEditor.API.Pawn.apparel.WornApparel.CountAllowNull<Apparel>() : 0;
        bool flag2 = num > 0;
        if (flag2)
        {
          for (int i = num - 1; i >= 0; i--)
          {
            Thing thing = CEditor.API.Pawn.apparel.WornApparel[i];
            bool flag3 = thing.def.IsApparel && (layerOnly == null || thing.def.apparel.layers.Contains(layerOnly));
            if (flag3)
            {
              CEditor.API.Pawn.TransferToInventory(thing);
            }
          }
        }
      }
    }

    // Token: 0x06000177 RID: 375 RVA: 0x0000B8D0 File Offset: 0x00009AD0
    public static void MoveDressFromInv(this Pawn p, ApparelLayerDef layerOnly)
    {
      bool flag = !CEditor.API.Pawn.HasApparelTracker() || !CEditor.API.Pawn.HasInventoryTracker();
      if (!flag)
      {
        p.MoveDressToInv(layerOnly);
        int num = CEditor.API.Pawn.inventory.innerContainer.CountAllowNull<Thing>();
        bool flag2 = num > 0;
        if (flag2)
        {
          for (int i = num - 1; i >= 0; i--)
          {
            Thing thing = CEditor.API.Pawn.inventory.innerContainer[i];
            bool isApparel = thing.def.IsApparel;
            if (isApparel)
            {
              bool flag3 = (layerOnly == null && CEditor.API.Pawn.apparel.CanWearWithoutDroppingAnything(thing.def)) || (layerOnly != null && thing.def.apparel.layers.Contains(layerOnly));
              if (flag3)
              {
                CEditor.API.Pawn.TransferFromInventory(thing);
                bool flag4 = layerOnly != null;
                if (flag4)
                {
                  break;
                }
              }
            }
          }
        }
      }
    }

    // Token: 0x06000178 RID: 376 RVA: 0x0000B9F4 File Offset: 0x00009BF4
    public static void AllowApparelToBeColorable()
    {
      try
      {
        ////Log.Message("allowing apparel to be colorable...");
        List<ThingDef> list = DefTool.ListAll<ThingDef>();
        foreach (ThingDef thingDef in list)
        {
          bool flag = thingDef != null && thingDef.defName != null && thingDef.IsApparel && thingDef.apparel.LastLayer != ApparelLayerDefOf.Belt;
          if (flag)
          {
            bool flag2 = thingDef.colorGenerator == null;
            if (flag2)
            {
              ColorGenerator_Options colorGenerator_Options = new ColorGenerator_Options();
              ColorOption colorOption = new ColorOption();
              colorOption.weight = 10f;
              colorOption.only = Color.white;
              colorGenerator_Options.options.Add(colorOption);
              thingDef.colorGenerator = colorGenerator_Options;
              thingDef.apparel.useWornGraphicMask = false;
            }
            bool flag3 = !thingDef.HasComp(typeof(CompColorable));
            if (flag3)
            {
              thingDef.AddCompColorable();
            }
            thingDef.ResolveReferences();
            thingDef.PostLoad();
          }
        }
      }
      catch (Exception ex)
      {
        Log.Error(ex.StackTrace);
      }
    }
  }
}
