﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;
using HarmonyLib;
using System.Reflection;
using System.Reflection.Emit;

namespace CharacterEditor
{
  // Token: 0x02000023 RID: 35
  public static class BackstoryTool
  {
    // Token: 0x06000186 RID: 390 RVA: 0x0000C40C File Offset: 0x0000A60C
    public static List<BackstoryDef> ListOfBackstories(bool isChildhood, bool notDisabling)
    {
      //return (from td in BackstoryDatabase.allBackstories.Values
      return (from td in DefDatabase<BackstoryDef>.AllDefsListForReading
              where td != null && td.slot == (isChildhood ? BackstorySlot.Childhood : BackstorySlot.Adulthood) && !string.IsNullOrEmpty(td.title) && (!notDisabling || td.DisabledWorkTypes.Count<WorkTypeDef>() == 0)
              orderby td.title
              select td).ToList<BackstoryDef>();
    }

    // Token: 0x06000187 RID: 391 RVA: 0x0000C478 File Offset: 0x0000A678
    public static BackstoryDef GetBackstoryDef(string defNameOrLegacyIdentifier)
    {
      ////Log.Message("GetBackstory, start, defNameOrLegacyIdentifier = " + defNameOrLegacyIdentifier);
      //bool flag = s.NullOrEmpty();
      BackstoryDef result;
      ////Log.Message("GetBackstory, step 1");
      if (defNameOrLegacyIdentifier.NullOrEmpty())
      {
        return null;
      }
      ////Log.Message("GetBackstory, step 2, looking for BackstoryDef.defName = " + defNameOrLegacyIdentifier);
      TryGetWithIdentifier(defNameOrLegacyIdentifier, out BackstoryDef backstory);
      //Backstory backstory;
      //BackstoryDatabase.TryGetWithIdentifier(s, ref backstory, false);
      //BackstoryDatabase.TryGetWithIdentifier(s, out BackstoryDef backstory, false);

      //BackstoryDef backstory = DefDatabase<BackstoryDef>.AllDefs.First(def => def.identifier == s);

      ////Log.Message("GetBackstory, step 3");
      result = backstory;

      ////Log.Message("GetBackstory, end");
      return backstory;
    }

    private static void TryGetWithIdentifier(string identifier, out BackstoryDef backstory, bool closestMatchWarning = true)
    {
      backstory = DefDatabase<BackstoryDef>.AllDefs.FirstOrDefault(b => b.defName == identifier);
      if (backstory == null)
      {
        string strippedDefName = BackstoryTranslationUtility.StripNumericSuffix(identifier);
        backstory = DefDatabase<BackstoryDef>.AllDefs.FirstOrDefault(b => BackstoryTranslationUtility.StripNumericSuffix(b.defName) == strippedDefName);
        if (backstory == null)
        {
          Log.Error("Couldn't find any backstory with identifier = " + identifier + ", choosing random element");
          backstory = DefDatabase<BackstoryDef>.GetRandom();
          return;
        }
        if (closestMatchWarning)
        {
          Log.Warning("Couldn't find exact match for backstory " + identifier + ", using closest match " + backstory.identifier);
        }
      }

    }

    // Token: 0x06000188 RID: 392 RVA: 0x0000C4A4 File Offset: 0x0000A6A4
    //public static string GetBackstrory(this Pawn pawn, bool isChildhood)
    //{
    //  return pawn.HasStoryTracker() ? (isChildhood ? ((pawn.story.Childhood == null) ? "" : pawn.story.Childhood.identifier) : ((pawn.story.Adulthood == null) ? "" : pawn.story.Adulthood.identifier)) : "";
    //}

    public static string GetChildhoodDefname(this Pawn pawn)
    {
      if (pawn?.story?.Childhood?.defName == null)
      {
        ////Log.Message("GetChildhoodDefname, pawn " + pawn + " has null childhood");
        return "";
      }
      string defName = pawn.story.Childhood.defName;
      ////Log.Message("GetChildhoodDefname, pawn " + pawn + ", Childhood.defName " + defName);
      return defName;
    }

    public static string GetAdulthoodDefname(this Pawn pawn)
    {
      if (pawn?.story?.Adulthood?.defName == null)
      {
        ////Log.Message("GetAdulthoodDefname, pawn " + pawn + " has null adulthood");
        return "";
      }
      string defName = pawn.story.Adulthood.defName;
      ////Log.Message("GetAdulthoodDefname, pawn " + pawn + ", Adulthood.defName " + defName);
      return defName;
    }

    // Token: 0x06000189 RID: 393 RVA: 0x0000C510 File Offset: 0x0000A710
    public static void SetBackstory(this Pawn pawn, bool next, bool random, bool isChildhood, bool notDisabled)
    {
      if (pawn?.story != null)
      {
        List<BackstoryDef> list = (from td in DefDatabase<BackstoryDef>.AllDefs
                                   where td != null
                                   && td.slot == (isChildhood ? BackstorySlot.Childhood : BackstorySlot.Adulthood)
                                   && !string.IsNullOrEmpty(td.title)
                                   && (!notDisabled || td.DisabledWorkTypes.Count<WorkTypeDef>() == 0)
                                   orderby td.title
                                   select td).ToList<BackstoryDef>();
        int index = list.IndexOf(isChildhood ? pawn.story.Childhood : pawn.story.Adulthood);
        index = list.NextOrPrevIndex(index, next, random);
        BackstoryDef backstory;
        if (!isChildhood)
        {
          Pawn_StoryTracker story = pawn.story;
          backstory = story != null ? story.Childhood : null;
        }
        else
        {
          backstory = list[index];
        }
        BackstoryDef childhood = backstory;
        BackstoryDef backstory2;
        if (isChildhood)
        {
          Pawn_StoryTracker story2 = pawn.story;
          backstory2 = ((story2 != null) ? story2.Adulthood : null);
        }
        else
        {
          backstory2 = list[index];
        }
        BackstoryDef adulthood = backstory2;
        pawn.SetBackstory(childhood, adulthood);
      }
    }

    // Token: 0x0600018A RID: 394 RVA: 0x0000C618 File Offset: 0x0000A818
    public static void SetBackstory(this Pawn p, BackstoryDef childhood, BackstoryDef adulthood)
    {
      if (p?.story == null)
      {
        return;
      }
      p.story.Childhood = childhood;
      p.story.Adulthood = adulthood;
      BackstoryDef childhood2 = p.story.Childhood;
      BackstoryDef adulthood2 = p.story.Adulthood;
      if (p.skills == null)
      {
        return;
      }
      if (p.skills.skills != null)
      {
        p.EnableAllSkills();
        foreach (SkillRecord skillRecord in p.skills.skills)
        {
          if (p.story.Childhood != null)
          {
            if (skillRecord.def.IsDisabled(p.story.Childhood.workDisables, p.story.Childhood.DisabledWorkTypes))
            {
              p.SkillDisable(skillRecord.def, BoolUnknown.True);
            }
          }
          if (p.story.Adulthood != null)
          {
            if (skillRecord.def.IsDisabled(p.story.Adulthood.workDisables, p.story.Adulthood.DisabledWorkTypes))
            {
              p.SkillDisable(skillRecord.def, BoolUnknown.True);
            }
          }
        }
      }
      p.skills.Notify_SkillDisablesChanged();
      p.workSettings.Notify_DisabledWorkTypesChanged();
      MainTabWindowUtility.NotifyAllPawnTables_PawnsChanged();
      MeditationFocusTypeAvailabilityCache.ClearFor(p);
      StatsReportUtility.Reset();

    }
  }
}
