﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000027 RID: 39
	public static class BodyTool
	{
		// Token: 0x060001A0 RID: 416 RVA: 0x0000CEC8 File Offset: 0x0000B0C8
		public static List<BodyPartGroupDef> ListAllBodyPartGroupDefs(bool insertNull)
		{
			List<BodyPartGroupDef> list = (from td in DefDatabase<BodyPartGroupDef>.AllDefs
			where td.modContentPack.IsCoreMod
			orderby td.label
			select td).ToList<BodyPartGroupDef>();
			list.AddRange((from td in DefDatabase<BodyPartGroupDef>.AllDefs
			where !td.modContentPack.IsCoreMod
			orderby td.label
			select td).ToList<BodyPartGroupDef>());
			if (insertNull)
			{
				list.Insert(0, null);
			}
			return list;
		}

		// Token: 0x060001A1 RID: 417 RVA: 0x0000CF98 File Offset: 0x0000B198
		public static List<BodyTypeDef> GetBodyList(this Pawn pawn)
		{
			bool flag = pawn == null || pawn.story == null;
			List<BodyTypeDef> result;
			if (flag)
			{
				result = null;
			}
			else
			{
				BodyTypeDef bodyType = pawn.story.bodyType;
				string a;
				if (bodyType == null)
				{
					a = null;
				}
				else
				{
					ModContentPack modContentPack = bodyType.modContentPack;
					a = ((modContentPack != null) ? modContentPack.Name : null);
				}
				bool flag2 = a == "Alien Vs Predator";
				List<BodyTypeDef> list = CEditor.API.ListOf<BodyTypeDef>(EType.Bodies);
				List<BodyTypeDef> list2 = new List<BodyTypeDef>();
				foreach (BodyTypeDef bodyTypeDef in list)
				{
					try
					{
						string text = pawn.Drawer.renderer.graphics.nakedGraphic.path;
						bool flag3 = text.Contains("/Naked");
						if (flag3)
						{
							text = text.SubstringTo("/Naked_", false) + bodyTypeDef.defName + "_south";
						}
						else
						{
							bool flag4 = text.Contains("_Naked_");
							if (flag4)
							{
								text = text.SubstringTo("_Naked_", false) + bodyTypeDef.defName + "_south";
							}
							else
							{
								bool flag5 = text.Contains("Naked_");
								if (flag5)
								{
									text = text.SubstringBackwardTo("Naked_", true) + "Naked_" + bodyTypeDef.defName + "_south";
								}
							}
						}
						bool flag6 = TextureTool.TestTexturePath(text, false);
						bool flag7 = flag2;
						if (flag7)
						{
							flag6 = (pawn.kindDef.modContentPack == bodyTypeDef.modContentPack && flag6);
						}
						bool flag8 = flag6;
						if (flag8)
						{
							list2.Add(bodyTypeDef);
						}
					}
					catch
					{
					}
				}
				result = list2;
			}
			return result;
		}

    public static string GetBodyTypeDefName(this Pawn p)
    {
      return p?.story?.bodyType?.defName != null ? p?.story?.bodyType?.defName : "";
    }

    public static string GetBodyTypeDefLabel(this Pawn p, BodyTypeDef bodyTypeDef = null)
		{
			if (!p.HasStoryTracker())
      {
				return "";
      }
			if (bodyTypeDef == null)
      {
        bodyTypeDef = p.story.bodyType;
      }
			return bodyTypeDef.ConstructLabelFromDef();
		}

		// Token: 0x060001A3 RID: 419 RVA: 0x0000D1A8 File Offset: 0x0000B3A8
		//public static string GetBodyTypeName(this Pawn pawn)
		//{
		//	return pawn.GetBodyTypeDefName();
  //    //TaggedString? taggedString;
  //    //if (pawn == null)
  //    //{
  //    //	taggedString = null;
  //    //}
  //    //else
  //    //{
  //    //	Pawn_StoryTracker story = pawn.story;
  //    //	if (story == null)
  //    //	{
  //    //		taggedString = null;
  //    //	}
  //    //	else
  //    //	{
  //    //		BodyTypeDef bodyType = story.bodyType;
  //    //       //taggedString = ((bodyType != null) ? new TaggedString?(bodyType.defName.Translate()) : null);
  //    //       taggedString = ((bodyType != null) ? new TaggedString?(bodyType.LabelCap) : null);
  //    //     }
  //    //}
  //    //TaggedString? taggedString2 = taggedString;
  //    //string text = (taggedString2 != null) ? taggedString2.GetValueOrDefault() : null;
  //    //return text ?? "";
  //  }

		public static string RemoveRotEndings(this string t)
		{
			return t.Replace("_south", "").Replace("_east", "").Replace("_north", "").Replace("_west", "");
		}

		// Token: 0x060001A5 RID: 421 RVA: 0x0000D264 File Offset: 0x0000B464
		public static void SetBody(this Pawn p, BodyTypeDef b)
		{
			bool flag = !p.HasStoryTracker() || b == null;
			if (!flag)
			{
				p.story.bodyType = b;
			}
		}

		// Token: 0x060001A6 RID: 422 RVA: 0x0000D294 File Offset: 0x0000B494
		public static void SetBody(this Pawn pawn, bool next, bool random)
		{
			bool flag = pawn == null || pawn.story == null;
			if (!flag)
			{
				List<BodyTypeDef> bodyList = pawn.GetBodyList();
				int index = bodyList.IndexOf(pawn.story.bodyType);
				index = bodyList.NextOrPrevIndex(index, next, random);
				bool flag2 = !bodyList.NullOrEmpty<BodyTypeDef>();
				if (flag2)
				{
					pawn.story.bodyType = bodyList[index];
				}
			}
		}
	}
}
