﻿using System;
using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200000C RID: 12
	public class Capturer
	{
		// Token: 0x17000023 RID: 35
		// (get) Token: 0x0600005D RID: 93 RVA: 0x00002FCA File Offset: 0x000011CA
		// (set) Token: 0x0600005E RID: 94 RVA: 0x00002FD2 File Offset: 0x000011D2
		internal Pawn Pawn { get; set; }

		// Token: 0x0600005F RID: 95 RVA: 0x00002FDC File Offset: 0x000011DC
		public Capturer()
		{
			this.image = null;
			this.lRotation = new List<Rot4>
			{
				Rot4.South,
				Rot4.West,
				Rot4.North,
				Rot4.East
			};
			this.iCurrentRotation = 0;
			this.bNude = true;
			this.bHats = true;
			this.bUpdateGraphics = false;
		}

		// Token: 0x06000060 RID: 96 RVA: 0x00003081 File Offset: 0x00001281
		internal void RotateAndCapture(Pawn pawn)
		{
			this.iCurrentRotation = this.lRotation.NextOrPrevIndex(this.iCurrentRotation, true, false);
			this.UpdatePawnGraphic(pawn);
		}

		// Token: 0x06000061 RID: 97 RVA: 0x000030A5 File Offset: 0x000012A5
		internal void ToggleNudeAndCapture(Pawn pawn)
		{
			this.bNude = !this.bNude;
			this.UpdatePawnGraphic(pawn);
		}

		// Token: 0x06000062 RID: 98 RVA: 0x000030BF File Offset: 0x000012BF
		internal void ToggleHatAndCapture(Pawn pawn)
		{
			this.bHats = !this.bHats;
			this.UpdatePawnGraphic(pawn);
		}

		// Token: 0x06000063 RID: 99 RVA: 0x000030D9 File Offset: 0x000012D9
		internal void UpdatePawnGraphic(Pawn pawn)
		{
			this.bUpdateGraphics = true;
			this.image = this.GetRenderTexture(pawn, false);
		}

		// Token: 0x06000064 RID: 100 RVA: 0x000030F4 File Offset: 0x000012F4
		internal RenderTexture GetRenderTexture(Pawn pawn, bool fromCache)
		{
			bool flag = pawn == null;
			RenderTexture result;
			if (flag)
			{
				result = null;
			}
			else if (fromCache)
			{
				result = PortraitsCache.Get(pawn, this.v2, this.lRotation[this.iCurrentRotation], default(Vector3), 1f, true, true, true, true, null, null, false);
			}
			else
			{
				bool flag2 = this.image == null;
				if (flag2)
				{
					this.image = new RenderTexture(this.renderW, this.renderH, 32, this.renderTextureFormat);
				}
				bool flag3 = this.bUpdateGraphics;
				if (flag3)
				{
					this.PrepareForRender(pawn);
					this.Render(pawn);
					this.bUpdateGraphics = false;
				}
				result = this.image;
			}
			return result;
		}

		// Token: 0x06000065 RID: 101 RVA: 0x000031B4 File Offset: 0x000013B4
		internal void ChangeRenderTextureParamter(int resolution, bool isARGB)
		{
			this.renderW = resolution;
			this.renderH = (int)((double)this.renderW * 1.4);
			this.renderTextureFormat = (isARGB ? RenderTextureFormat.ARGB32 : RenderTextureFormat.DefaultHDR);
			bool flag = this.image != null;
			if (flag)
			{
				this.image.Release();
			}
			this.image = new RenderTexture(this.renderW, this.renderH, 32, this.renderTextureFormat);
		}

		// Token: 0x06000066 RID: 102 RVA: 0x0000322C File Offset: 0x0000142C
		internal void PrepareForRender(Pawn pawn)
		{
			try
			{
				PortraitsCache.SetDirty(pawn);
				bool flag = pawn.kindDef.defName == "Zombie";
				if (flag)
				{
					pawn.Drawer.renderer.graphics.hairGraphic = GraphicDatabase.Get<Graphic_Multi>(pawn.story.hairDef.texPath, ShaderDatabase.Cutout, Vector2.one, pawn.story.HairColor);
					pawn.Drawer.renderer.graphics.ResolveApparelGraphics();
				}
				else
				{
					PawnGraphicSet graphics = pawn.Drawer.renderer.graphics;
					if (graphics != null)
					{
						graphics.ResolveAllGraphics();
					}
				}
			}
			catch
			{
			}
		}

		// Token: 0x06000067 RID: 103 RVA: 0x000032EC File Offset: 0x000014EC
		internal void Render(Pawn pawn)
		{
			float num = 0f;
			Vector3 vector = default(Vector3);
			bool flag = pawn.Dead || pawn.Downed;
			if (flag)
			{
				num = 85f;
				vector.x -= 0.18f;
				vector.z -= 0.18f;
			}
			try
			{
				PawnCacheCameraManager.PawnCacheRenderer.RenderPawn(pawn, this.image, Vector3.zero, 1f, num, this.lRotation[this.iCurrentRotation], true, true, true, true, true, vector, null, null, false);
			}
			catch (Exception ex)
			{
				Log.Error(ex.StackTrace);
			}
		}

		// Token: 0x06000068 RID: 104 RVA: 0x000033AC File Offset: 0x000015AC
		internal void DrawPawnImage(Pawn pawn, int x, int y)
		{
			this.image = this.GetRenderTexture(pawn, false);
			bool flag = this.image != null;
			if (flag)
			{
				GUI.DrawTexture(new Rect((float)x, (float)y, 200f, 280f), this.image, ScaleMode.ScaleToFit);
			}
		}

		// Token: 0x04000073 RID: 115
		internal bool bUpdateGraphics;

		// Token: 0x04000074 RID: 116
		public bool bNude;

		// Token: 0x04000075 RID: 117
		public bool bHats;

		// Token: 0x04000076 RID: 118
		private int iCurrentRotation;

		// Token: 0x04000077 RID: 119
		private List<Rot4> lRotation;

		// Token: 0x04000078 RID: 120
		private RenderTexture image;

		// Token: 0x04000079 RID: 121
		private const int imageW = 200;

		// Token: 0x0400007A RID: 122
		private const int imageH = 280;

		// Token: 0x0400007B RID: 123
		public int renderW = 500;

		// Token: 0x0400007C RID: 124
		public int renderH = 700;

		// Token: 0x0400007D RID: 125
		private Vector2 v2 = new Vector2(200f, 280f);

		// Token: 0x0400007E RID: 126
		public RenderTextureFormat renderTextureFormat = RenderTextureFormat.ARGB32;
	}
}
