﻿using System;
using System.Collections.Generic;
using RimWorld;
using Verse;
using Verse.AI;
using Verse.Sound;

namespace CharacterEditor
{
	// Token: 0x0200000B RID: 11
	public class CharacterEditorGrave : Building_CryptosleepCasket
	{
		// Token: 0x06000057 RID: 87 RVA: 0x00002CE0 File Offset: 0x00000EE0
		public CharacterEditorGrave()
		{
			this.innerPawn = null;
			this.cachedGraphicFull = null;
		}

		// Token: 0x17000022 RID: 34
		// (get) Token: 0x06000058 RID: 88 RVA: 0x00002CF8 File Offset: 0x00000EF8
		public override Graphic Graphic
		{
			get
			{
				bool flag = this.innerPawn == null && this.innerContainer.Count == 0;
				Graphic graphic;
				if (flag)
				{
					graphic = base.Graphic;
				}
				else
				{
					this.cachedGraphicFull = this.def.building.fullGraveGraphicData.GraphicColoredFor(this);
					bool flag2 = this.def.building.fullGraveGraphicData == null;
					if (flag2)
					{
						graphic = base.Graphic;
					}
					else
					{
						bool flag3 = this.cachedGraphicFull == null;
						if (flag3)
						{
							this.cachedGraphicFull = this.def.building.fullGraveGraphicData.GraphicColoredFor(this);
						}
						graphic = this.cachedGraphicFull;
					}
				}
				return graphic;
			}
		}

		// Token: 0x06000059 RID: 89 RVA: 0x00002DA1 File Offset: 0x00000FA1
		public override IEnumerable<FloatMenuOption> GetFloatMenuOptions(Pawn myPawn)
		{
			bool flag = this.innerContainer.Count == 0;
			if (flag)
			{
				bool flag2 = !myPawn.CanReach(this, PathEndMode.InteractionCell, Danger.Deadly, false, false, TraverseMode.ByPawn);
				if (flag2)
				{
					FloatMenuOption failer = new FloatMenuOption("CannotUseNoPath".Translate(), null, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0);
					yield return failer;
					failer = null;
				}
				else
				{
					string jobStr = CharacterEditor.Label.ENTER_ZOMBGRELLA;
					Action jobAction = delegate()
					{
						JobDef jobDef = DefTool.JobDef("EnterZGrave");
						bool flag3 = jobDef != null;
						if (flag3)
						{
							Job job = new Job(jobDef, this);
							myPawn.jobs.TryTakeOrderedJob(job, new JobTag?(JobTag.Misc), false);
						}
					};
					yield return FloatMenuUtility.DecoratePrioritizedTask(new FloatMenuOption(jobStr, jobAction, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0), myPawn, this, "ReservedBy");
					jobStr = null;
					jobAction = null;
				}
			}
			yield break;
		}

		// Token: 0x0600005A RID: 90 RVA: 0x00002DB8 File Offset: 0x00000FB8
		public override void EjectContents()
		{
			ThingDef filth_Slime = ThingDefOf.Filth_Slime;
			foreach (Thing thing in ((IEnumerable<Thing>)this.innerContainer))
			{
				Pawn pawn = thing as Pawn;
				bool flag = pawn != null;
				if (flag)
				{
					PawnComponentsUtility.AddComponentsForSpawn(pawn);
					pawn.filth.GainFilth(filth_Slime);
					this.innerPawn = null;
				}
			}
			bool flag2 = !base.Destroyed;
			if (flag2)
			{
				SoundDef soundDef = DefTool.SoundDef("Bridge_CollapseWater");
				bool flag3 = soundDef != null;
				if (flag3)
				{
					soundDef.PlayOneShot(SoundInfo.InMap(new TargetInfo(base.Position, base.Map, false), MaintenanceType.None));
				}
			}
			this.innerContainer.TryDropAll(this.InteractionCell, base.Map, ThingPlaceMode.Near, null, null, true);
			this.contentsKnown = true;
		}

		// Token: 0x0600005B RID: 91 RVA: 0x00002EA4 File Offset: 0x000010A4
		public bool TryToEnter(Thing thing)
		{
			bool flag = !this.Accepts(thing);
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				bool flag2 = thing.holdingOwner != null;
				bool flag3;
				if (flag2)
				{
					thing.holdingOwner.TryTransferToContainer(thing, this.innerContainer, thing.stackCount, true);
					flag3 = true;
				}
				else
				{
					flag3 = this.innerContainer.TryAdd(thing, true);
				}
				bool flag4 = flag3;
				if (flag4)
				{
					bool flag5 = thing.Faction != null && thing.Faction.IsPlayer;
					if (flag5)
					{
						this.contentsKnown = true;
					}
					result = true;
				}
				else
				{
					result = false;
				}
			}
			return result;
		}

		// Token: 0x0600005C RID: 92 RVA: 0x00002F38 File Offset: 0x00001138
		public override bool TryAcceptThing(Thing thing, bool allowSpecialEffects = true)
		{
			bool flag = this.TryToEnter(thing);
			bool result;
			if (flag)
			{
				SoundDefOf.Corpse_Drop.PlayOneShot(new TargetInfo(base.Position, base.Map, false));
				this.cachedGraphicFull = this.def.building.fullGraveGraphicData.GraphicColoredFor(this);
				this.innerPawn = (thing as Pawn);
				CEditor.API.Get<Dictionary<int, Building_CryptosleepCasket>>(EType.UIContainers)[0] = this;
				CEditor.API.StartEditor(this.innerPawn);
				result = true;
			}
			else
			{
				result = false;
			}
			return result;
		}

		// Token: 0x04000071 RID: 113
		private Pawn innerPawn;

		// Token: 0x04000072 RID: 114
		private Graphic cachedGraphicFull;
	}
}
