﻿using System;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200000D RID: 13
	public class CustomDialog : Window
	{
		// Token: 0x17000024 RID: 36
		// (get) Token: 0x06000069 RID: 105 RVA: 0x000033F8 File Offset: 0x000015F8
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(640f, 460f);
			}
		}

		// Token: 0x17000025 RID: 37
		// (get) Token: 0x0600006A RID: 106 RVA: 0x00003409 File Offset: 0x00001609
		private float TimeUntilInteractive
		{
			get
			{
				return this.interactionDelay - (Time.realtimeSinceStartup - this.creationRealTime);
			}
		}

		// Token: 0x17000026 RID: 38
		// (get) Token: 0x0600006B RID: 107 RVA: 0x0000341E File Offset: 0x0000161E
		private bool InteractionDelayExpired
		{
			get
			{
				return (double)this.TimeUntilInteractive <= 0.0;
			}
		}

		// Token: 0x0600006C RID: 108 RVA: 0x00003438 File Offset: 0x00001638
		public CustomDialog(string text, string title, Action onAbort, Action onConfirm, Action onNext)
		{
			this.title = title;
			this.text = text;
			this.buttonAbortAction = onAbort;
			this.buttonAcceptAction = onConfirm;
			this.buttonNextAction = onNext;
			this.layer = CEditor.Layer;
			this.forcePause = true;
			this.absorbInputAroundWindow = true;
			this.creationRealTime = RealTime.LastRealTime;
			this.onlyOneOfTypeAllowed = false;
			this.closeOnAccept = true;
			this.closeOnCancel = true;
		}

		// Token: 0x0600006D RID: 109 RVA: 0x000034C4 File Offset: 0x000016C4
		public override void DoWindowContents(Rect inRect)
		{
			float num = inRect.y;
			bool flag = !this.title.NullOrEmpty();
			if (flag)
			{
				Text.Font = GameFont.Medium;
				Widgets.Label(new Rect(0f, num, inRect.width, 42f), this.title);
				num += 42f;
			}
			Text.Font = GameFont.Small;
			Rect outRect = new Rect(inRect.x, num, inRect.width, (float)((double)inRect.height - 35.0 - 5.0) - num);
			float width = outRect.width - 16f;
			Rect viewRect = new Rect(0f, 0f, width, Text.CalcHeight(this.text, width));
			Widgets.BeginScrollView(outRect, ref this.scrollPosition, viewRect, true);
			Widgets.Label(new Rect(0f, 0f, viewRect.width, viewRect.height), this.text);
			Widgets.EndScrollView();
			GUI.color = Color.white;
			float num2 = this.InitialSize.y - 70f;
			SZWidgets.CheckBoxOnChange(new Rect(inRect.x + 420f, num2 - 30f, 180f, 30f), Label.ALWAYS_SKIP, CEditor.DontAsk, new Action<bool>(this.ASetAlwaysSkip));
			SZWidgets.ButtonText(new Rect(inRect.x, num2, 180f, 30f), "Cancel".Translate(), new Action(this.AOnAbort), "");
			SZWidgets.ButtonText(new Rect(inRect.x + (float)((this.buttonNextAction == null) ? 420 : 210), num2, 180f, 30f), "Confirm".Translate(), new Action(this.AOnAccept), "");
			bool flag2 = this.buttonNextAction != null;
			if (flag2)
			{
				SZWidgets.ButtonText(new Rect(inRect.x + 420f, num2, 180f, 30f), Label.SKIP, new Action(this.AOnNext), "");
			}
		}

		// Token: 0x0600006E RID: 110 RVA: 0x00003701 File Offset: 0x00001901
		private void ASetAlwaysSkip(bool val)
		{
			CEditor.DontAsk = val;
		}

		// Token: 0x0600006F RID: 111 RVA: 0x0000370C File Offset: 0x0000190C
		private void AOnAbort()
		{
			bool flag = this.buttonAbortAction != null;
			if (flag)
			{
				this.buttonAbortAction();
			}
			this.Close(true);
		}

		// Token: 0x06000070 RID: 112 RVA: 0x0000373C File Offset: 0x0000193C
		private void AOnAccept()
		{
			bool flag = this.buttonAcceptAction != null;
			if (flag)
			{
				this.buttonAcceptAction();
			}
			this.Close(true);
		}

		// Token: 0x06000071 RID: 113 RVA: 0x0000376C File Offset: 0x0000196C
		private void AOnNext()
		{
			bool flag = this.buttonNextAction != null;
			if (flag)
			{
				this.buttonNextAction();
			}
			this.Close(true);
		}

		// Token: 0x06000072 RID: 114 RVA: 0x0000379B File Offset: 0x0000199B
		public override void OnCancelKeyPressed()
		{
			this.AOnAbort();
		}

		// Token: 0x06000073 RID: 115 RVA: 0x000037A5 File Offset: 0x000019A5
		public override void OnAcceptKeyPressed()
		{
			this.AOnAccept();
		}

		// Token: 0x04000080 RID: 128
		public string text;

		// Token: 0x04000081 RID: 129
		public string title;

		// Token: 0x04000082 RID: 130
		public string buttonAbortText;

		// Token: 0x04000083 RID: 131
		public string buttonAcceptText;

		// Token: 0x04000084 RID: 132
		public string buttonNextText;

		// Token: 0x04000085 RID: 133
		public Action buttonAbortAction;

		// Token: 0x04000086 RID: 134
		public Action buttonAcceptAction;

		// Token: 0x04000087 RID: 135
		public Action buttonNextAction;

		// Token: 0x04000088 RID: 136
		public float interactionDelay;

		// Token: 0x04000089 RID: 137
		private Vector2 scrollPosition = Vector2.zero;

		// Token: 0x0400008A RID: 138
		private float creationRealTime = -1f;

		// Token: 0x0400008B RID: 139
		private const float TitleHeight = 42f;

		// Token: 0x0400008C RID: 140
		protected const float ButtonHeight = 35f;
	}
}
