﻿using System;
using System.Reflection;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;
using HarmonyLib;

namespace CharacterEditor
{
  // Token: 0x02000029 RID: 41
  public static class DefTool
  {
    // Token: 0x060001AF RID: 431 RVA: 0x0000DEB8 File Offset: 0x0000C0B8
    public static KeyBindingDef TryCreateKeyBinding(string defName, string keyCode, string label, string desc)
    {
      //////Log.Message("TryCreateKeyBinding, start");
      KeyBindingDef keyBindingDef = null;
      try
      {
        //////Log.Message("TryCreateKeyBinding, step 1");
        bool flag = !keyCode.NullOrEmpty();
        //////Log.Message("TryCreateKeyBinding, step 2");
        if (flag)
        {
          //////Log.Message("TryCreateKeyBinding, step 3");
          keyBindingDef = new KeyBindingDef();
          //////Log.Message("TryCreateKeyBinding, step 4");
          keyBindingDef.category = KeyBindingCategoryDefOf.MainTabs;
          //////Log.Message("TryCreateKeyBinding, step 5");
          keyBindingDef.defaultKeyCodeA = keyCode.AsKeyCode();
          //////Log.Message("TryCreateKeyBinding, step 6");
          keyBindingDef.defaultKeyCodeB = KeyCode.None;
          //////Log.Message("TryCreateKeyBinding, step 7");
          keyBindingDef.defName = defName;
          //////Log.Message("TryCreateKeyBinding, step 8");
          keyBindingDef.description = desc;
          //////Log.Message("TryCreateKeyBinding, step 9");
          keyBindingDef.label = label;
          //////Log.Message("TryCreateKeyBinding, step 10");
          keyBindingDef.ResolveReferences();
          //////Log.Message("TryCreateKeyBinding, step 11");
          keyBindingDef.PostLoad();
          //////Log.Message("TryCreateKeyBinding, step 12");
          FieldInfo takenHashesFieldInfo = AccessTools.Field(typeof(ShortHashGiver), "takenHashesPerDeftype");
          //////Log.Message("TryCreateKeyBinding, step 13");
          Dictionary<Type, HashSet<ushort>> takenHashesDict = (Dictionary<Type, HashSet<ushort>>)takenHashesFieldInfo.GetValue(null);
          //////Log.Message("TryCreateKeyBinding, step 14");
          MethodInfo methodInfo = AccessTools.Method(typeof(ShortHashGiver), "GiveShortHash");
          //////Log.Message("TryCreateKeyBinding, step 15");
          methodInfo.CallMethod(new object[] { keyBindingDef, typeof(KeyBindingDef), takenHashesDict[typeof(KeyBindingDef)] });
          //////Log.Message("TryCreateKeyBinding, step 16");
          DefDatabase<Verse.KeyBindingDef>.Add(keyBindingDef);
          //////Log.Message("TryCreateKeyBinding, step 17");
          KeyPrefs.KeyPrefsData.keyPrefs.Add(keyBindingDef, new KeyBindingData(keyBindingDef.defaultKeyCodeA, keyBindingDef.defaultKeyCodeB));
          //////Log.Message("TryCreateKeyBinding, step 18");
        }
      }
      catch (Exception ex)
      {
        Log.Error("TryCreateKeyBinding, " + ex);
      }
      //////Log.Message("TryCreateKeyBinding, end");
      return keyBindingDef;
    }

    // Token: 0x060001B0 RID: 432 RVA: 0x0000DFA0 File Offset: 0x0000C1A0
    public static MainButtonDef GetCreateMainButton(string defName, string label, string desc, Type typeClass, ModContentPack pack, string keyCode, bool isVisible)
    {
      //////Log.Message("GetCreateMainButton, start");
      MainButtonDef mainButtonDef = DefTool.MainButtonDef(defName);
      if (mainButtonDef == null)
      {
        //////Log.Message("GetCreateMainButton, step 1");
        KeyBindingDef hotKey = DefTool.TryCreateKeyBinding(defName, keyCode, label, desc);
        mainButtonDef = new MainButtonDef();
        mainButtonDef.buttonVisible = isVisible;
        mainButtonDef.defName = defName;
        mainButtonDef.description = desc;
        mainButtonDef.label = label;
        mainButtonDef.tabWindowClass = typeClass;
        mainButtonDef.order = MainButtonDefOf.Menu.order - 1;
        mainButtonDef.hotKey = hotKey;
        mainButtonDef.validWithoutMap = false;
        mainButtonDef.modContentPack = pack;
        mainButtonDef.ResolveReferences();
        mainButtonDef.PostLoad();
        //////Log.Message("GetCreateMainButton, step 2");
        FieldInfo takenHashesFieldInfo = AccessTools.Field(typeof(ShortHashGiver), "takenHashesPerDeftype");
        //////Log.Message("GetCreateMainButton, step 3");
        Dictionary<Type, HashSet<ushort>> takenHashesDict = (Dictionary<Type, HashSet<ushort>>)takenHashesFieldInfo.GetValue(null);
        //////Log.Message("GetCreateMainButton, step 4");
        MethodInfo methodInfo = AccessTools.Method(typeof(ShortHashGiver), "GiveShortHash");
        //////Log.Message("GetCreateMainButton, step 5");
        methodInfo.CallMethod(new object[] { mainButtonDef, typeof(KeyBindingDef), takenHashesDict[typeof(KeyBindingDef)] });
        //////Log.Message("GetCreateMainButton, step 6");
        //typeof(ShortHashGiver).CallMethod("GiveShortHash", new object[]
        //{
        //  mainButtonDef,
        //  typeof(ThingDef)
        //});
        DefDatabase<RimWorld.MainButtonDef>.Add(mainButtonDef);
        //////Log.Message("GetCreateMainButton, step 7");
      }
      //////Log.Message("GetCreateMainButton, end");
      return mainButtonDef;
    }

    // Token: 0x060001B1 RID: 433 RVA: 0x0000E068 File Offset: 0x0000C268
    public static T GetDef<T>(string defName) where T : Def
    {
      return (!defName.NullOrEmpty()) ? DefDatabase<T>.GetNamed(defName, false) : default(T);
    }

    // Token: 0x060001B2 RID: 434 RVA: 0x0000E094 File Offset: 0x0000C294
    public static MainButtonDef MainButtonDef(string defName)
    {
      return DefTool.GetDef<MainButtonDef>(defName);
    }

    // Token: 0x060001B3 RID: 435 RVA: 0x0000E09C File Offset: 0x0000C29C
    public static PawnKindDef PawnKindDef(string defName)
    {
      return DefTool.GetDef<PawnKindDef>(defName);
    }

    // Token: 0x060001B4 RID: 436 RVA: 0x0000E0A4 File Offset: 0x0000C2A4
    public static PawnKindDef PawnKindDef(string defName, PawnKindDef fallback)
    {
      return DefTool.PawnKindDef(defName) ?? fallback;
    }

    // Token: 0x060001B5 RID: 437 RVA: 0x0000E0B1 File Offset: 0x0000C2B1
    public static PawnRelationDef PawnRelationDef(string defName)
    {
      return DefTool.GetDef<PawnRelationDef>(defName);
    }

    // Token: 0x060001B6 RID: 438 RVA: 0x0000E0B9 File Offset: 0x0000C2B9
    public static BodyTypeDef BodyTypeDef(string defName)
    {
      return DefTool.GetDef<BodyTypeDef>(defName);
    }

    // Token: 0x060001B7 RID: 439 RVA: 0x0000E0C1 File Offset: 0x0000C2C1
    public static BeardDef BeardDef(string defName)
    {
      return DefTool.GetDef<BeardDef>(defName);
    }

    // Token: 0x060001B8 RID: 440 RVA: 0x0000E0C9 File Offset: 0x0000C2C9
    public static TattooDef TattooDef(string defName)
    {
      return DefTool.GetDef<TattooDef>(defName);
    }

    // Token: 0x060001B9 RID: 441 RVA: 0x0000E0D1 File Offset: 0x0000C2D1
    public static AbilityDef AbilityDef(string defName)
    {
      return DefTool.GetDef<AbilityDef>(defName);
    }

    // Token: 0x060001BA RID: 442 RVA: 0x0000E0D9 File Offset: 0x0000C2D9
    public static HairDef HairDef(string defName)
    {
      return DefTool.GetDef<HairDef>(defName);
    }

    // Token: 0x060001BB RID: 443 RVA: 0x0000E0E1 File Offset: 0x0000C2E1
    public static HediffDef HediffDef(string defName)
    {
      return DefTool.GetDef<HediffDef>(defName);
    }

    // Token: 0x060001BC RID: 444 RVA: 0x0000E0E9 File Offset: 0x0000C2E9
    public static RecordDef RecordDef(string defName)
    {
      return DefTool.GetDef<RecordDef>(defName);
    }

    // Token: 0x060001BD RID: 445 RVA: 0x0000E0F1 File Offset: 0x0000C2F1
    public static SkillDef SkillDef(string defName)
    {
      return DefTool.GetDef<SkillDef>(defName);
    }

    // Token: 0x060001BE RID: 446 RVA: 0x0000E0F9 File Offset: 0x0000C2F9
    public static ScenPartDef ScenPartDef(string defName)
    {
      return DefTool.GetDef<ScenPartDef>(defName);
    }

    // Token: 0x060001BF RID: 447 RVA: 0x0000E101 File Offset: 0x0000C301
    public static ThingDef ThingDef(string defName)
    {
      return DefTool.GetDef<ThingDef>(defName);
    }

    // Token: 0x060001C0 RID: 448 RVA: 0x0000E109 File Offset: 0x0000C309
    public static ThingStyleDef ThingStyleDef(string defName)
    {
      return DefTool.GetDef<ThingStyleDef>(defName);
    }

    // Token: 0x060001C1 RID: 449 RVA: 0x0000E111 File Offset: 0x0000C311
    public static ThingDef ThingDef(string defName, ThingDef fallback)
    {
      return DefTool.ThingDef(defName) ?? fallback;
    }

    // Token: 0x060001C2 RID: 450 RVA: 0x0000E11E File Offset: 0x0000C31E
    public static NeedDef NeedDef(string defName)
    {
      return DefTool.GetDef<NeedDef>(defName);
    }

    // Token: 0x060001C3 RID: 451 RVA: 0x0000E126 File Offset: 0x0000C326
    public static ThoughtDef ThoughtDef(string defName)
    {
      return DefTool.GetDef<ThoughtDef>(defName);
    }

    // Token: 0x060001C4 RID: 452 RVA: 0x0000E12E File Offset: 0x0000C32E
    public static TraitDef TraitDef(string defName)
    {
      return DefTool.GetDef<TraitDef>(defName);
    }

    // Token: 0x060001C5 RID: 453 RVA: 0x0000E136 File Offset: 0x0000C336
    public static WorkTypeDef WorkTypeDef(string defName)
    {
      return DefTool.GetDef<WorkTypeDef>(defName);
    }

    // Token: 0x060001C6 RID: 454 RVA: 0x0000E13E File Offset: 0x0000C33E
    public static StatDef StatDef(string defName)
    {
      return DefTool.GetDef<StatDef>(defName);
    }

    // Token: 0x060001C7 RID: 455 RVA: 0x0000E146 File Offset: 0x0000C346
    public static JobDef JobDef(string defName)
    {
      return DefTool.GetDef<JobDef>(defName);
    }

    // Token: 0x060001C8 RID: 456 RVA: 0x0000E14E File Offset: 0x0000C34E
    public static SoundDef SoundDef(string defName)
    {
      return DefTool.GetDef<SoundDef>(defName);
    }

    // Token: 0x060001C9 RID: 457 RVA: 0x0000E156 File Offset: 0x0000C356
    public static StuffCategoryDef StuffCategoryDef(string defName)
    {
      return DefTool.GetDef<StuffCategoryDef>(defName);
    }

    // Token: 0x060001CA RID: 458 RVA: 0x0000E15E File Offset: 0x0000C35E
    public static BodyPartGroupDef BodyPartGroupDef(string defName)
    {
      return DefTool.GetDef<BodyPartGroupDef>(defName);
    }

    // Token: 0x060001CB RID: 459 RVA: 0x0000E166 File Offset: 0x0000C366
    public static ApparelLayerDef ApparelLayerDef(string defName)
    {
      return DefTool.GetDef<ApparelLayerDef>(defName);
    }

    // Token: 0x060001CC RID: 460 RVA: 0x0000E16E File Offset: 0x0000C36E
    public static DesignatorDropdownGroupDef DesignatorDropdownGroupDef(string defName)
    {
      return DefTool.GetDef<DesignatorDropdownGroupDef>(defName);
    }

    // Token: 0x060001CD RID: 461 RVA: 0x0000E176 File Offset: 0x0000C376
    public static DesignationCategoryDef DesignationCategoryDef(string defName)
    {
      return DefTool.GetDef<DesignationCategoryDef>(defName);
    }

    // Token: 0x060001CE RID: 462 RVA: 0x0000E17E File Offset: 0x0000C37E
    public static KeyBindingDef KeyBindingDef(string defName)
    {
      return DefTool.GetDef<KeyBindingDef>(defName);
    }

    // Token: 0x060001CF RID: 463 RVA: 0x0000E186 File Offset: 0x0000C386
    public static bool IsFromMod(this Def d, string modname)
    {
      return d != null && d.modContentPack != null && d.modContentPack.Name == modname;
    }

    // Token: 0x060001D0 RID: 464 RVA: 0x0000E1A7 File Offset: 0x0000C3A7
    public static bool IsFromCoreMod(this Def d)
    {
      return d != null && d.modContentPack != null && d.modContentPack.IsCoreMod;
    }

    // Token: 0x060001D1 RID: 465 RVA: 0x0000E1C2 File Offset: 0x0000C3C2
    public static string GetModName(this Def d)
    {
      return (d != null && d.modContentPack != null) ? d.modContentPack.Name : "";
    }

    // Token: 0x060001D2 RID: 466 RVA: 0x0000E1E1 File Offset: 0x0000C3E1
    public static bool IsNullOrEmpty(this Def d)
    {
      return d == null || d.defName.NullOrEmpty();
    }

    // Token: 0x060001D3 RID: 467 RVA: 0x0000E1F4 File Offset: 0x0000C3F4
    public static List<T> ListAll<T>() where T : Def
    {
      return (from x in DefDatabase<T>.AllDefs
              orderby x.label
              select x).ToList<T>();
    }

    // Token: 0x060001D4 RID: 468 RVA: 0x0000E224 File Offset: 0x0000C424
    public static List<string> ListModnamesWithNull<T>(Func<T, bool> condition = null) where T : Def
    {
      List<string> hashSet = new List<string>();
      hashSet.Add(null);
      hashSet.AddRange((condition == null) ? DefTool.ListModnames<T>() : DefTool.ListModnames<T>(condition));
      return hashSet;
    }

    // Token: 0x060001D5 RID: 469 RVA: 0x0000E25C File Offset: 0x0000C45C
    public static List<string> ListModnames<T>(Func<T, bool> condition) where T : Def
    {
      return (from x in DefDatabase<T>.AllDefs
              where x.modContentPack != null && condition(x)
              orderby x.modContentPack.Name
              select x.modContentPack.Name).ToList<string>();
    }

    // Token: 0x060001D6 RID: 470 RVA: 0x0000E2DC File Offset: 0x0000C4DC
    public static List<string> ListModnames<T>() where T : Def
    {
      return (from x in DefDatabase<T>.AllDefs
              where x.modContentPack != null
              orderby x.modContentPack.Name
              select x.modContentPack.Name).ToList<string>();
    }

    // Token: 0x060001D7 RID: 471 RVA: 0x0000E360 File Offset: 0x0000C560
    public static List<T> ListByModWithNull<T>(string name, Func<T, bool> condition = null) where T : Def
    {
      List<T> hashSet = new List<T>();
      hashSet.Add(default(T));
      hashSet.AddRange((condition == null) ? DefTool.ListByMod<T>(name) : DefTool.ListByMod<T>(name, condition));
      hashSet.OrderBy(x => x.label);
      return hashSet;
    }

    // Token: 0x060001D8 RID: 472 RVA: 0x0000E3A4 File Offset: 0x0000C5A4
    public static List<T> ListByMod<T>(string name, Func<T, bool> condition) where T : Def
    {
      return (from x in DefDatabase<T>.AllDefs
              where !x.IsNullOrEmpty() && (name.NullOrEmpty() || x.IsFromMod(name)) && condition(x)
              orderby x.label
              select x).ToList<T>();
    }

    // Token: 0x060001D9 RID: 473 RVA: 0x0000E404 File Offset: 0x0000C604
    public static List<T> ListByMod<T>(string name) where T : Def
    {
      return (from x in DefDatabase<T>.AllDefs
              where !x.IsNullOrEmpty() && (name.NullOrEmpty() || x.IsFromMod(name))
              orderby x.label
              select x).ToList<T>();
    }

    // Token: 0x060001DA RID: 474 RVA: 0x0000E460 File Offset: 0x0000C660
    public static List<T> ListBy<T>(Func<T, bool> condition) where T : Def
    {
      return (from x in DefDatabase<T>.AllDefs
              where condition(x)
              orderby x.label
              select x).ToList<T>();
    }

    // Token: 0x1700003F RID: 63
    // (get) Token: 0x060001DB RID: 475 RVA: 0x0000E4B9 File Offset: 0x0000C6B9
    public static Func<ThingCategoryDef, bool> CONDITION_NO_CORPSE
    {
      get
      {
        return (ThingCategoryDef t) => !t.defName.ToLower().Contains("corpse");
      }
    }

    // Token: 0x060001DC RID: 476 RVA: 0x0000E4DC File Offset: 0x0000C6DC
    public static Func<ThingDef, bool> CONDITION_IS_ITEM(ThingCategoryDef tc, string name)
    {
      return (ThingDef t) => !t.label.NullOrEmpty() && !t.IsBlueprint && !t.IsCorpse && !t.IsFrame && !t.isFrameInt && !t.IsBullet() && t.race == null && t.mote == null && t.graphicData != null && !t.graphicData.texPath.NullOrEmpty() && (name.NullOrEmpty() || t.IsFromMod(name)) && (tc == null || (!t.thingCategories.NullOrEmpty<ThingCategoryDef>() && t.thingCategories.Contains(tc)));
    }

    // Token: 0x060001DD RID: 477 RVA: 0x0000E50C File Offset: 0x0000C70C
    public static Func<StatDef, bool> CONDITION_STATDEFS_APPAREL(List<StatCategoryDef> lnok)
    {
      return (StatDef t) => !lnok.Contains(t.category) || t.category == StatCategoryDefOf.Apparel;
    }

    // Token: 0x060001DE RID: 478 RVA: 0x0000E534 File Offset: 0x0000C734
    public static Func<StatDef, bool> CONDITION_STATDEFS_WEAPON(List<StatCategoryDef> lnok)
    {
      return (StatDef t) => !lnok.Contains(t.category) || t.category == StatCategoryDefOf.Weapon;
    }

    // Token: 0x060001DF RID: 479 RVA: 0x0000E55A File Offset: 0x0000C75A
    public static string SLabel(this ThoughtStage t)
    {
      return t.label.NullOrEmpty() ? "" : t.label;
    }

    // Token: 0x060001E0 RID: 480 RVA: 0x0000E576 File Offset: 0x0000C776
    public static string SLabel(this Def d)
    {
      return d.label.NullOrEmpty() ? "" : d.label;
    }

    // Token: 0x060001E1 RID: 481 RVA: 0x0000E592 File Offset: 0x0000C792
    public static string SDefname(this Def d)
    {
      return (d == null || d.defName.NullOrEmpty()) ? "" : d.defName;
    }
  }
}
