﻿using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  public class DialogAddAnimal : Window
  {

    private DialogCapsuleUI CapsuleUI;
    private Vector2 scrollPos;
    private List<PawnKindDef> lOfAnimals;
    private PawnKindDef selectedAnimal;
    //private int stackVal;
    //private int oldStackVal;
    private EntryInt entryInt;
    private int idxList;
    private int iChangeTick0;
    private int maxSliderVal;
    private Func<PawnKindDef, PawnKindDef, bool> FAnimalComparator;

    public override Vector2 InitialSize => new Vector2(300f, 745f);

    private bool IsSameAnimal(PawnKindDef a, PawnKindDef b)
    {
      return a == b;
    }

    public DialogAddAnimal(DialogCapsuleUI _CapsuleUI)
    {
      this.CapsuleUI = _CapsuleUI;
      this.scrollPos = default(Vector2);
      this.iChangeTick0 = 0;
      this.idxList = 8;
      SearchTool.Update(this.idxList);
      this.lOfAnimals = (from td in DefDatabase<PawnKindDef>.AllDefs
                         where td.race != null && td.race.label != null && td.RaceProps.Animal
                         orderby td.race.label
                         select td).ToList<PawnKindDef>();
      this.selectedAnimal = this.lOfAnimals.First<PawnKindDef>();
      this.maxSliderVal = ((CEditor.API != null) ? CEditor.API.MaxSliderVal : 100);
      //this.stackVal = 1;
      //this.oldStackVal = 1;
      this.entryInt = new EntryInt(1);
      this.FAnimalComparator = new Func<PawnKindDef, PawnKindDef, bool>(this.IsSameAnimal);
      this.doCloseX = true;
      this.absorbInputAroundWindow = true;
      this.draggable = true;
      this.layer = CEditor.Layer;
    }

    public override void DoWindowContents(Rect inRect)
    {
      float num = 0f;
      float num2 = this.InitialSize.x - 40f;
      this.DrawEditLabel(0f, num, num2, 30f);
      num += 30f;
      this.DrawTexture(35f, num, num2, num2);
      num += 230f;
      this.DrawList(0f, num, num2, this.InitialSize.y - num - 84f);
      if (Widgets.ButtonText(new Rect(this.InitialSize.x - 136f, this.InitialSize.y - 75f, 100f, 40f), "OK".Translate(), true, true, true))
      {
        this.DoAndClose();
      }
    }

    // Token: 0x060002B7 RID: 695 RVA: 0x00015F5E File Offset: 0x0001415E
    public override void OnAcceptKeyPressed()
    {
      base.OnAcceptKeyPressed();
      this.DoAndClose();
    }

    // Token: 0x060002B8 RID: 696 RVA: 0x00015F70 File Offset: 0x00014170
    private void DoAndClose()
    {
      if (this.CapsuleUI != null)
      {
        //this.CapsuleUI.AddAnimal(this.selectedAnimal, this.stackVal);
        this.CapsuleUI.AddAnimal(this.selectedAnimal, this.entryInt.valInt);
      }
      SearchTool.Save(this.idxList);
      this.Close(true);
    }

    // Token: 0x060002B9 RID: 697 RVA: 0x00015FBC File Offset: 0x000141BC
    private void DrawTexture(float x, float y, float w, float h)
    {
      bool flag = this.selectedAnimal == null || this.selectedAnimal.race == null;
      if (!flag)
      {
        Texture2D uiIcon = this.selectedAnimal.race.uiIcon;
        Rect position = new Rect(x, y, 200f, 200f);
        GUI.color = this.selectedAnimal.race.uiIconColor;
        GUI.DrawTexture(position, uiIcon, ScaleMode.ScaleAndCrop, true);
        y += 200f;
        GUI.color = Color.white;

        this.entryInt.NumericTextField(0.5f * w - 35f, y - 25f, 70f, 25f, 1, this.maxSliderVal);
        string sliderLabel = this.entryInt.Open ? null : this.entryInt.valInt.ToString();
        this.entryInt.HorizontalSlider(new Rect(0f, y, w, 25f), 1, this.maxSliderVal, label: sliderLabel);

        //this.oldStackVal = this.stackVal;
        //this.stackVal = (int)Widgets.HorizontalSlider(new Rect(0f, y, w, 25f), (float)this.stackVal, 1f, (float)this.maxSliderVal, false, this.stackVal.ToString(), null, null, -1f);
        //bool flag2 = this.oldStackVal != this.stackVal;
        //if (flag2)
        //{
        //  this.oldStackVal = this.stackVal;
        //}
      }
    }

    // Token: 0x060002BA RID: 698 RVA: 0x000160B4 File Offset: 0x000142B4
    private void DrawEditLabel(float x, float y, float w, float h)
    {
      bool flag = this.selectedAnimal == null || this.selectedAnimal.race == null || this.selectedAnimal.race.label == null;
      if (!flag)
      {
        Text.Font = GameFont.Medium;
        Rect rect = new Rect(x, y, w, h);
        bool flag2 = this.iChangeTick0 <= 0;
        if (flag2)
        {
          bool flag3 = Widgets.ButtonInvisible(rect, true);
          if (flag3)
          {
            this.iChangeTick0 = 1000;
          }
          Widgets.Label(rect, this.selectedAnimal.race.label);
        }
        else
        {
          this.selectedAnimal.race.label = Widgets.TextField(rect, this.selectedAnimal.race.label);
          this.iChangeTick0--;
        }
      }
    }

    // Token: 0x060002BB RID: 699 RVA: 0x00016180 File Offset: 0x00014380
    private void DrawList(float x, float y, float w, float h)
    {
      SZWidgets.ListView<PawnKindDef>(x, y, w, h, this.lOfAnimals, CEditor.PKDNAME, CEditor.PKDTOOLTIP, this.FAnimalComparator, ref this.selectedAnimal, ref this.scrollPos, false, new Action<PawnKindDef>(this.AAnimalSelected), true, true, false, false);
    }

    // Token: 0x060002BC RID: 700 RVA: 0x000161CC File Offset: 0x000143CC
    private void AAnimalSelected(PawnKindDef pkd)
    {
      if (pkd != null)
      {
        this.selectedAnimal = pkd;
      }
    }
  }
}
