﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000059 RID: 89
	public class DialogAddHediff : Window, IPawnable, IPartable
	{
    private Pawn selPawn;
    private BodyPartRecord selPart;
    private bool isPartSelectionDone = false;
    private bool isPawnSelectionDone = false;
    private bool bNeedsPawn = false;
    private bool bNeedsPart = false;
    private int idxList;
    private Vector2 scrollPos;
    private HediffDef selectedHediff;
    private HediffDef oldSelectedHediff;

    private float selectedSeverity;
    private int selectedDuration;
    private int selectedPain;
    private int selectedLevel;
    private bool inEditMode;
    private bool isAdjustable;
    private bool isPermanent;
    private bool isOverridden;
    private bool doOnce;
    private List<string> lOfFilter1;
    private Dictionary<string, BodyPartRecord> DicOfFilter2;
    private List<HediffDef> lOfHediffs;
    private Hediff example;
    private HediffComp_Disappears hDisappears;
    private HediffComp_GetsPermanent hPermanent;
    private string paramName;
    private string extraTipString;
    private SearchTool search;
    private Func<HediffDef, string> FHediffLabel = (HediffDef h) => h.LabelCap;
    private Func<HediffDef, string> FHediffTooltip = delegate (HediffDef h)
    {
      ModContentPack modContentPack = h.modContentPack;
      return ((modContentPack != null) ? modContentPack.Name : null) + "\n" + h.description;
    };
    private Func<HediffDef, HediffDef, bool> FHediffComparator = (HediffDef h1, HediffDef h2) => h1 == h2;
    private int yCalc = 0;
    private float oldSeverity;

    public Pawn SelectedPawn
		{
			get
			{
				return this.selPawn;
			}
			set
			{
				this.selPawn = value;
				this.isPawnSelectionDone = true;
				this.CheckIsReady();
			}
		}
		public Pawn SelectedPawn2 { get; set; }
		public Pawn SelectedPawn3 { get; set; }
		public Pawn SelectedPawn4 { get; set; }
		public BodyPartRecord SelectedPart
		{
			get
			{
				return this.selPart;
			}
			set
			{
				this.selPart = value;
				this.isPartSelectionDone = true;
				this.CheckIsReady();
			}
		}
		public override Vector2 InitialSize => new Vector2(300f, 710f);

		public DialogAddHediff(Hediff toSelect = null)
		{
			this.scrollPos = default(Vector2);
			this.selectedHediff = null;
			this.oldSelectedHediff = null;
			this.isAdjustable = false;
			this.isPermanent = false;
			this.isOverridden = false;
			this.selectedSeverity = 0.01f;
			this.selectedDuration = -1;
			this.selectedLevel = -1;
			this.selectedPain = -1;
			this.paramName = "";
			this.selPawn = null;
			this.selPart = null;
			this.isPartSelectionDone = false;
			this.isPawnSelectionDone = false;
			this.bNeedsPart = true;
			this.bNeedsPawn = false;
			this.extraTipString = "";
			this.doOnce = true;
			this.idxList = 0;
			this.search = SearchTool.Update(this.idxList);
			this.lOfFilter1 = new List<string>();
			this.lOfFilter1.Add(Label.ALL);
			this.lOfFilter1.Add(Label.HB_ALLIMPLANTS);
			this.lOfFilter1.Add(Label.HB_ALLADDICTIONS);
			this.lOfFilter1.Add(Label.HB_ALLDISEASES);
			this.lOfFilter1.Add(Label.HB_ALLINJURIES);
			this.lOfFilter1.Add(Label.HB_ALLTIME);
			this.lOfFilter1.Add(Label.HB_WITHLEVEL);
			this.DicOfFilter2 = new Dictionary<string, BodyPartRecord>();
			this.DicOfFilter2.Add(Label.ALL, null);
			this.DicOfFilter2.Add(Label.HB_WholeBody, null);
			foreach (BodyPartRecord bodyPartRecord in CEditor.API.Pawn.RaceProps.body.AllParts)
			{
				this.DicOfFilter2.AddLabeled(bodyPartRecord.Label.CapitalizeFirst(), bodyPartRecord);
			}
			this.UpdateHediffList();
			this.selectedHediff = ((toSelect != null) ? toSelect.def : null);
			this.CheckSelectionChanged(toSelect);
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x06000554 RID: 1364 RVA: 0x00036F00 File Offset: 0x00035100
		public override void DoWindowContents(Rect inRect)
		{
			bool flag = this.doOnce;
			if (flag)
			{
				this.doOnce = false;
				this.windowRect.position = new Vector2(this.windowRect.position.x - 80f, this.windowRect.position.y);
			}
			int num = (int)this.InitialSize.x - 40;
			int num2 = (int)this.InitialSize.y - 115;
			int num3 = 0;
			int num4 = 0;
			this.DrawTitle(num4, num3, num, 30);
			num3 += 30;
			this.GetExtraTipString();
			Text.Font = GameFont.Small;
			this.DrawDropdown(num4, num3, num, 30);
			num3 += 92;
			SZWidgets.ListView<HediffDef>((float)num4, (float)num3, (float)num, (float)(num2 - 110 - this.yCalc), this.lOfHediffs, this.FHediffLabel, this.FHediffTooltip, this.FHediffComparator, ref this.selectedHediff, ref this.scrollPos, false, null, true, false, false, false);
			num3 += num2 - 110 - this.yCalc;
			num3 += 10;
			this.CheckSelectionChanged(null);
			this.yCalc = 0;
			this.yCalc += this.DrawAdjustableSeverity(num4, ref num3, num, 45);
			this.yCalc += this.DrawAdjustableLevel(num4, ref num3, num, 45);
			this.yCalc += this.DrawAdjustablePain(num4, ref num3, num, 45);
			this.yCalc += this.DrawAdjustableTime(num4, ref num3, num, 45);
			this.yCalc += this.DrawPermanent(num4, ref num3, num, 25);
			Rect rect = new Rect(this.InitialSize.x - 140f, this.InitialSize.y - 70f, 100f, 30f);
			bool flag2 = !this.inEditMode;
			if (flag2)
			{
				bool flag3 = Widgets.ButtonText(rect, "OK".Translate(), true, true, true);
				if (flag3)
				{
					bool flag4 = this.selectedHediff != null;
					if (flag4)
					{
						this.CheckIsReady();
					}
					else
					{
						SearchTool.Save(this.idxList);
						this.Close(true);
					}
				}
			}
			SZWidgets.ButtonImage(new Rect((float)(num - 30), 0f, 30f, 30f), "bresurrect", new Action(this.AToggleOverride), "override", this.isOverridden);
		}

		// Token: 0x06000555 RID: 1365 RVA: 0x0003715C File Offset: 0x0003535C
		private void CheckIsReady()
		{
			bool flag = this.selectedHediff != null;
			if (flag)
			{
				bool flag2 = true;
				bool flag3 = false;
				bool flag4 = false;
				bool flag5 = this.bNeedsPart && !this.isPartSelectionDone;
				if (flag5)
				{
					flag2 = false;
					flag3 = true;
				}
				bool flag6 = this.bNeedsPawn && !this.isPawnSelectionDone;
				if (flag6)
				{
					flag2 = false;
					flag4 = true;
				}
				bool flag7 = flag2;
				if (flag7)
				{
					this.DoAndClose();
				}
				else
				{
					bool flag8 = flag3;
					if (flag8)
					{
						List<BodyPartRecord> listOfAllowedBodyPartRecords = CEditor.API.Pawn.GetListOfAllowedBodyPartRecords(this.selectedHediff);
						bool flag9 = listOfAllowedBodyPartRecords.CountAllowNull<BodyPartRecord>() <= 1;
						if (flag9)
						{
							this.SelectedPart = listOfAllowedBodyPartRecords.FirstOrDefault<BodyPartRecord>();
						}
						else
						{
							WindowTool.Open(new DialogChoosePart(this, this.selectedHediff));
						}
					}
					else
					{
						bool flag10 = flag4;
						if (flag10)
						{
							WindowTool.Open(new DialogChoosePawn(this, 1, Gender.None));
						}
					}
				}
			}
		}

		// Token: 0x06000556 RID: 1366 RVA: 0x00037244 File Offset: 0x00035444
		private void GetExtraTipString()
		{
			try
			{
				this.extraTipString = ((this.example != null) ? this.example.TipStringExtra : "");
			}
			catch
			{
				this.extraTipString = "";
			}
		}

		// Token: 0x06000557 RID: 1367 RVA: 0x00037298 File Offset: 0x00035498
		private void CheckSelectionChanged(Hediff toSelect = null)
		{
			bool flag = !this.FHediffComparator(this.oldSelectedHediff, this.selectedHediff);
			if (flag)
			{
				this.oldSelectedHediff = this.selectedHediff;
				this.selPawn = null;
				this.isPartSelectionDone = (this.search.filter2 != null && !(this.search.filter2 == Label.ALL));
				this.selPart = (this.isPartSelectionDone ? this.DicOfFilter2.GetValue(this.search.filter2) : null);
				this.isPawnSelectionDone = false;
				this.hDisappears = null;
				this.hPermanent = null;
				this.isAdjustable = HealthTool.IsAdjustableSeverity(this.selectedHediff);
				List<BodyPartRecord> listOfAllowedBodyPartRecords = CEditor.API.Pawn.GetListOfAllowedBodyPartRecords(this.selectedHediff);
				this.bNeedsPart = (listOfAllowedBodyPartRecords.CountAllowNull<BodyPartRecord>() > 0);
				this.bNeedsPawn = (this.selectedHediff.IsHediffWithTarget() || this.selectedHediff.IsHediffLink());
				this.inEditMode = (toSelect != null);
				this.example = ((toSelect != null) ? toSelect : HediffMaker.MakeHediff(this.selectedHediff, CEditor.API.Pawn, listOfAllowedBodyPartRecords.FirstOrDefault<BodyPartRecord>()));
				this.selectedSeverity = this.example.Severity;
				this.selectedLevel = this.example.GetLevel();
				this.selectedPain = HealthTool.ConvertPainCategoryToSliderVal((PainCategory)this.example.GetPainValue());
				this.selectedDuration = this.example.GetDuration();
				this.hDisappears = this.example.TryGetComp<HediffComp_Disappears>();
				this.hPermanent = this.example.TryGetComp<HediffComp_GetsPermanent>();
				bool flag2 = this.hDisappears != null;
				if (flag2)
				{
					this.hDisappears.Props.showRemainingTime = true;
				}
				bool flag3 = this.hPermanent != null;
				if (flag3)
				{
					this.isPermanent = this.hPermanent.IsPermanent;
				}
				this.example.ShowDebugInfo();
			}
		}

		// Token: 0x06000558 RID: 1368 RVA: 0x00037481 File Offset: 0x00035681
		private void AToggleOverride()
		{
			this.isOverridden = !this.isOverridden;
		}

		// Token: 0x06000559 RID: 1369 RVA: 0x00037493 File Offset: 0x00035693
		private void DrawTitle(int x, int y, int w, int h)
		{
			Text.Font = GameFont.Medium;
			Widgets.Label(new Rect((float)x, (float)y, (float)(w - 40), (float)h), this.inEditMode ? this.example.LabelCap : Label.ADD_HEDIFF);
		}

		// Token: 0x0600055A RID: 1370 RVA: 0x000374D0 File Offset: 0x000356D0
		private int DrawPermanent(int x, ref int y, int w, int h)
		{
			bool flag = this.selectedHediff != null && (this.isAdjustable || this.isOverridden);
			if (flag)
			{
				bool flag2 = this.selectedHediff.injuryProps != null || this.isOverridden;
				if (flag2)
				{
					SZWidgets.CheckBoxOnChange(new Rect((float)x, (float)(y + 10), (float)w, 24f), "Permanent".Translate(), this.isPermanent, new Action<bool>(this.AChangePermanent));
					y += h;
					return h;
				}
			}
			return 0;
		}

		// Token: 0x0600055B RID: 1371 RVA: 0x00037568 File Offset: 0x00035768
		private void AChangePermanent(bool perma)
		{
			this.isPermanent = perma;
			bool flag = this.hPermanent != null;
			if (flag)
			{
				this.hPermanent.IsPermanent = perma;
			}
		}

		// Token: 0x0600055C RID: 1372 RVA: 0x00037598 File Offset: 0x00035798
		private int DrawAdjustableLevel(int x, ref int y, int w, int h)
		{
			bool flag = this.selectedHediff != null && this.example != null && this.selectedHediff.IsHediffWithLevel();
			int result;
			if (flag)
			{
				Text.Font = GameFont.Small;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(new Rect((float)x, (float)y, (float)w, (float)h));

				listing_X.AddIntSection("Level".Translate(), "", ref this.paramName, ref this.selectedLevel, (int)this.selectedHediff.minSeverity, (int)this.selectedHediff.maxSeverity, true, "");
				listing_X.End();
				this.example.SetLevel(this.selectedLevel);
				this.example.Severity = (float)this.selectedLevel;
				this.selectedSeverity = (float)this.selectedLevel;
				y += h;
				result = h;
			}
			else
			{
				result = 0;
			}
			return result;
		}

		// Token: 0x0600055D RID: 1373 RVA: 0x00037680 File Offset: 0x00035880
		private int DrawAdjustablePain(int x, ref int y, int w, int h)
		{
			bool flag = this.hPermanent != null && this.selectedHediff != null;
			int result;
			if (flag)
			{
				Text.Font = GameFont.Small;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(new Rect((float)x, (float)y, (float)w, (float)h));
				listing_X.AddIntSection(" ", "pain", ref this.paramName, ref this.selectedPain, 0, 3, true, "");
				listing_X.End();
				this.hPermanent.SetPainCategory(HealthTool.ConvertSliderToPainCategory(this.selectedPain));
				y += h;
				result = h;
			}
			else
			{
				result = 0;
			}
			return result;
		}

		// Token: 0x0600055E RID: 1374 RVA: 0x00037720 File Offset: 0x00035920
		private int DrawAdjustableTime(int x, ref int y, int w, int h)
		{
			bool flag = this.hDisappears != null && this.selectedHediff != null;
			int result;
			if (flag)
			{
				Text.Font = GameFont.Small;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(new Rect((float)x, (float)y, (float)w, (float)h));
				string format = "dauer" + this.hDisappears.CompLabelInBracketsExtra;
				listing_X.AddIntSection(Label.DAUER, format, ref this.paramName, ref this.selectedDuration, 0, 220000, true, this.extraTipString);
        listing_X.End();
				this.hDisappears.ticksToDisappear = this.selectedDuration;
				y += h;
				result = h;
			}
			else
			{
				result = 0;
			}
			return result;
		}

		// Token: 0x0600055F RID: 1375 RVA: 0x000377D4 File Offset: 0x000359D4
		private int DrawAdjustableSeverity(int x, ref int y, int w, int h)
		{
			bool flag = this.selectedHediff != null && (this.isAdjustable || this.isOverridden);
			int result;
			if (flag)
			{
				Text.Font = GameFont.Small;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(new Rect((float)x, (float)y, (float)w, (float)h));
				float maxSeverity = HealthTool.GetMaxSeverity(this.selectedHediff);
				this.selectedSeverity = ((this.selectedSeverity > maxSeverity) ? maxSeverity : this.selectedSeverity);
				this.oldSeverity = this.selectedSeverity;
				bool flag2 = this.selectedHediff.lethalSeverity >= 0f && this.selectedSeverity >= this.selectedHediff.lethalSeverity;
				if (flag2)
				{
					GUI.color = Color.red;
				}
				else
				{
					GUI.color = Color.white;
				}
				this.example.Severity = this.selectedSeverity;
				string text = this.example.LabelInBrackets;
				if (text.NullOrEmpty())
				{
					text = (this.selectedHediff.IsHediffWithLevel() ? "int" : (this.selectedHediff.IsAddiction ? "addict" : ""));
				}
				else
				{
					text = "comp" + text;
				}
				listing_X.AddSection("ConfigurableSeverity".Translate().ToString().SubstringTo(":", true), text, ref this.paramName, ref this.selectedSeverity, this.selectedHediff.minSeverity, maxSeverity, true, this.extraTipString);
				listing_X.End();
				if (this.oldSeverity != this.selectedSeverity)
				{
					CEditor.API.UpdateGraphics();
				}
				y += h;
				result = h;
			}
			else
			{
				result = 0;
			}
			return result;
		}

		// Token: 0x06000560 RID: 1376 RVA: 0x00037994 File Offset: 0x00035B94
		private void DrawDropdown(int x, int y, int w, int h)
		{
			Rect rect = new Rect((float)x, (float)y, (float)w, (float)h);
			SZWidgets.FloatMenuOnButtonText<string>(rect, this.search.SelectedModName, CEditor.API.Get<List<string>>(EType.ModsHediffDef), (string s) => s ?? Label.ALL, new Action<string>(this.ASelectedModName), "");
			Rect rect2 = new Rect(rect);
			rect2.y += 30f;
			SZWidgets.FloatMenuOnButtonText<string>(rect2, this.search.SelectedFilter1, this.lOfFilter1, (string s) => s, new Action<string>(this.ASelectFilter1));
			Rect rect3 = new Rect(rect2);
			rect3.y += 30f;
			SZWidgets.FloatMenuOnButtonText<string>(rect3, this.search.SelectedFilter2, this.DicOfFilter2.Keys.ToList<string>(), (string s) => s, new Action<string>(this.ASelectBodyPartRecord));
		}

		// Token: 0x06000561 RID: 1377 RVA: 0x00037AC8 File Offset: 0x00035CC8
		private void ASelectedModName(string val)
		{
			this.search.modName = val;
			this.UpdateHediffList();
		}

		// Token: 0x06000562 RID: 1378 RVA: 0x00037ADE File Offset: 0x00035CDE
		private void ASelectFilter1(string val)
		{
			this.search.filter1 = val;
			this.UpdateHediffList();
		}

		// Token: 0x06000563 RID: 1379 RVA: 0x00037AF4 File Offset: 0x00035CF4
		private void ASelectBodyPartRecord(string val)
		{
			this.search.filter2 = val;
			this.selPart = this.DicOfFilter2.GetValue(val);
			this.isPartSelectionDone = (val != Label.ALL);
			this.UpdateHediffList();
		}

		// Token: 0x06000564 RID: 1380 RVA: 0x00037B30 File Offset: 0x00035D30
		private void UpdateHediffList()
		{
			this.lOfHediffs = HealthTool.ListOfHediffDef(this.search.modName, CEditor.API.Pawn, this.DicOfFilter2.GetValue(this.search.SelectedFilter2), this.search.SelectedFilter1, this.search.SelectedFilter2 == Label.HB_WholeBody);
		}

		// Token: 0x06000565 RID: 1381 RVA: 0x00037B94 File Offset: 0x00035D94
		private void DoAndClose()
		{
			bool flag = this.selectedHediff != null;
			if (flag)
			{
				CEditor.API.Pawn.AddHediff2(false, this.selectedHediff, this.selectedSeverity, this.SelectedPart, this.isPermanent, this.selectedLevel, (int)((this.selectedPain < 0) ? ((PainCategory)(-1)) : HealthTool.ConvertSliderToPainCategory(this.selectedPain)), this.selectedDuration, this.SelectedPawn);
				StatsReportUtility.Reset();
				CEditor.API.UpdateGraphics();
			}
			SearchTool.Save(this.idxList);
			this.Close(true);
		}

		// Token: 0x06000566 RID: 1382 RVA: 0x00037C28 File Offset: 0x00035E28
		public override void OnAcceptKeyPressed()
		{
			base.OnAcceptKeyPressed();
			this.DoAndClose();
		}

		
	}
}
