﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  // Token: 0x02000012 RID: 18
  public class DialogAddThought : Window, IPawnable
  {
    private Vector2 scrollPos;
    private int idxList;
    private string selectedModName;
    private string selectedType;
    private string selectedStage;
    private RoyalTitleDef selectedTitle;
    private ThoughtDef selectedThought;
    private List<string> lModnames;
    private List<string> lListType;
    private List<ThoughtDef> lListThoughts;
    private List<ThoughtDef> lTMemory;
    private List<ThoughtDef> lTMemorySocial;
    private List<ThoughtDef> lTSituational;
    private List<ThoughtDef> lTSituationalSocial;
    private List<ThoughtDef> lTUnsupported;
    private List<ThoughtDef> lTAll;
    private List<RoyalTitleDef> lRoyalTitles;

    // Token: 0x040000C3 RID: 195
    private Dictionary<int, string> dicStages;

    // Token: 0x040000C4 RID: 196
    private float baseMoodOffset;

    // Token: 0x040000C5 RID: 197
    private float selectedMoodOffset;

    // Token: 0x040000C6 RID: 198
    private float selectedOpinionOffset;

    // Token: 0x040000C7 RID: 199
    private bool doOnce;

    // Token: 0x040000C8 RID: 200
    private bool bNeedOtherPawn = false;

    // Token: 0x040000C9 RID: 201
    private bool bNeedTitle = false;

    // Token: 0x040000CA RID: 202
    private bool bNeedStage = false;

    // Token: 0x040000CB RID: 203
    private bool bAllOk = false;

    // Token: 0x1700002F RID: 47
    // (get) Token: 0x0600009B RID: 155 RVA: 0x000047D7 File Offset: 0x000029D7
    // (set) Token: 0x0600009C RID: 156 RVA: 0x000047DF File Offset: 0x000029DF
    public Pawn SelectedPawn { get; set; }

    // Token: 0x17000030 RID: 48
    // (get) Token: 0x0600009D RID: 157 RVA: 0x000047E8 File Offset: 0x000029E8
    // (set) Token: 0x0600009E RID: 158 RVA: 0x000047F0 File Offset: 0x000029F0
    public Pawn SelectedPawn2 { get; set; }

    // Token: 0x17000031 RID: 49
    // (get) Token: 0x0600009F RID: 159 RVA: 0x000047F9 File Offset: 0x000029F9
    // (set) Token: 0x060000A0 RID: 160 RVA: 0x00004801 File Offset: 0x00002A01
    public Pawn SelectedPawn3 { get; set; }

    // Token: 0x17000032 RID: 50
    // (get) Token: 0x060000A1 RID: 161 RVA: 0x0000480A File Offset: 0x00002A0A
    // (set) Token: 0x060000A2 RID: 162 RVA: 0x00004812 File Offset: 0x00002A12
    public Pawn SelectedPawn4 { get; set; }

    // Token: 0x17000033 RID: 51
    // (get) Token: 0x060000A3 RID: 163 RVA: 0x0000481B File Offset: 0x00002A1B
    private Func<ThoughtDef, string> FGetTooltip
    {
      get
      {
        return (ThoughtDef t) => this.GetTooltipForThought(t);
      }
    }

    // Token: 0x17000034 RID: 52
    // (get) Token: 0x060000A4 RID: 164 RVA: 0x00004829 File Offset: 0x00002A29
    private Func<ThoughtDef, string> FGetThoughtLabel
    {
      get
      {
        return (ThoughtDef t) => this.GetLabelForThought(t);
      }
    }

    // Token: 0x17000035 RID: 53
    // (get) Token: 0x060000A5 RID: 165 RVA: 0x00004837 File Offset: 0x00002A37
    private Func<RoyalTitleDef, string> FGetRoyalTitleLabel
    {
      get
      {
        return (RoyalTitleDef r) => (r == null) ? Label.NONE : r.LabelCap.ToString();
      }
    }

    // Token: 0x17000036 RID: 54
    // (get) Token: 0x060000A6 RID: 166 RVA: 0x00004858 File Offset: 0x00002A58
    private Func<Pawn, string> FGetPawnName
    {
      get
      {
        return (Pawn p) => (p == null) ? Label.NONE : p.LabelShort;
      }
    }

    // Token: 0x17000037 RID: 55
    // (get) Token: 0x060000A7 RID: 167 RVA: 0x0000487C File Offset: 0x00002A7C
    public override Vector2 InitialSize
    {
      get
      {
        return new Vector2(300f, 800f);
      }
    }

    // Token: 0x060000A8 RID: 168 RVA: 0x000048A0 File Offset: 0x00002AA0
    private string GetLabelForThought(ThoughtDef t)
    {
      bool flag = t == null;
      string result;
      if (flag)
      {
        result = Label.NONE;
      }
      else
      {
        bool flag2 = t.defName == "DeadMansApparel";
        if (flag2)
        {
          result = t.defName;
        }
        else
        {
          result = t.GetThoughtLabel(0, CEditor.API.Pawn);
        }
      }
      return result;
    }

    // Token: 0x060000A9 RID: 169 RVA: 0x000048F0 File Offset: 0x00002AF0
    private string GetTooltipForThought(ThoughtDef t)
    {
      string text = t.SDefname();
      bool flag = t != null;
      if (flag)
      {
        text = text + "\n\n" + t.GetThoughtDescription(0, null);
        text += "\n";
        bool flag2 = !t.stages.NullOrEmpty<ThoughtStage>();
        if (flag2)
        {
          for (int i = 0; i < t.stages.Count; i++)
          {
            try
            {
              int num = (int)t.stages[i].baseMoodEffect;
              int num2 = (int)t.stages[i].baseOpinionOffset;
              text = string.Concat(new string[]
              {
                text,
                "\nStage ",
                i.ToString(),
                " BaseMood: ",
                num.ToString().Colorize((num == 0) ? ColorLibrary.Grey : ((num > 0) ? ColorLibrary.Green : ColorLibrary.Red))
              });
              text = text + " BaseOpinion: " + num2.ToString().Colorize((num2 == 0) ? ColorLibrary.Grey : ((num2 > 0) ? ColorLibrary.Green : ColorLibrary.Red));
            }
            catch
            {
            }
          }
        }
        text = text + "\n" + t.ThoughtClass.ToString();
        bool flag3 = t.IsTypeOf<Thought_PsychicHarmonizer>();
        if (flag3)
        {
          text += ("\nrequires hediff: " + HediffDefOf.PsychicHarmonizer.LabelCap.ToString()).Colorize(ColorLibrary.Purple);
        }
        bool flag4 = t.IsTypeOf<Thought_WeaponTrait>();
        if (flag4)
        {
          text += "\nrequires weapon".Colorize(ColorLibrary.Purple);
        }
        bool flag5 = t.IsTypeOf<ThoughtWorker_WantToSleepWithSpouseOrLover>() || t.IsTypeOf<Thought_OpinionOfMyLover>();
        if (flag5)
        {
          text += "\nrequires love relation".Colorize(ColorLibrary.Purple);
        }
        bool flag6 = !t.requiredTraits.NullOrEmpty<TraitDef>();
        if (flag6)
        {
          text += ("\nrequires trait: " + t.requiredTraits.First<TraitDef>().SDefname()).Colorize(ColorLibrary.Purple);
        }
      }
      return text;
    }

    // Token: 0x060000AA RID: 170 RVA: 0x00004B24 File Offset: 0x00002D24
    public DialogAddThought(string _selectedType = null)
    {
      this.scrollPos = default(Vector2);
      this.idxList = 9;
      SearchTool.Update(this.idxList);
      this.baseMoodOffset = 0f;
      this.selectedMoodOffset = 1f;
      this.selectedOpinionOffset = 1f;
      this.selectedModName = Label.ALL;
      this.selectedType = (_selectedType ?? Label.ALL);
      this.SelectedPawn = null;
      this.selectedStage = null;
      this.selectedThought = null;
      this.selectedTitle = null;
      this.lModnames = DefTool.ListModnamesWithNull<ThoughtDef>((ThoughtDef x) => true);
      this.lListType = new List<string>
      {
        Label.ALL,
        Label.TH_MEMORY,
        Label.TH_SOCIAL,
        Label.TH_SITUATIONAL,
        Label.TH_SITUATIONAL + Label.TH_SOCIAL,
        Label.TH_UNSUPPORTED
      };
      this.lTMemory = CEditor.API.ListOf<ThoughtDef>(EType.ThoughtMemory);
      this.lTMemorySocial = CEditor.API.ListOf<ThoughtDef>(EType.ThoughtMemorySocial);
      this.lTSituational = CEditor.API.ListOf<ThoughtDef>(EType.ThoughtSituational);
      this.lTSituationalSocial = CEditor.API.ListOf<ThoughtDef>(EType.ThoughtSituationalSocial);
      this.lTUnsupported = CEditor.API.ListOf<ThoughtDef>(EType.ThoughtUnsupported);
      this.lTAll = CEditor.API.ListOf<ThoughtDef>(EType.ThoughtsAll);
      this.lListThoughts = this.lTAll;
      this.lRoyalTitles = DefTool.ListBy<RoyalTitleDef>((RoyalTitleDef x) => true);
      this.dicStages = new Dictionary<int, string>();
      this.AThoughtTypeSelected(this.selectedType);
      this.doOnce = true;
      this.doCloseX = true;
      this.absorbInputAroundWindow = true;
      this.draggable = true;
      this.layer = CEditor.Layer;
    }

    // Token: 0x060000AB RID: 171 RVA: 0x00004D38 File Offset: 0x00002F38
    public override void DoWindowContents(Rect inRect)
    {
      int w = (int)this.InitialSize.x - 40;
      int num = 0;
      int x = 0;
      bool flag = this.doOnce;
      if (flag)
      {
        this.doOnce = false;
        this.windowRect.position = new Vector2(this.windowRect.position.x - 100f, this.windowRect.position.y);
      }
      this.DrawTitle(x, ref num, w, 30);
      this.DrawUpperDropdowns(x, ref num, w, 30);
      this.DrawThoughtList(x, ref num, w, 500);
      this.DrawLowerDropdowns(x, ref num, w, 30);
      this.DrawSlider(x, ref num, w, 30);
      this.DrawAccept(x, ref num, w, 30);
    }

    // Token: 0x060000AC RID: 172 RVA: 0x00004DF8 File Offset: 0x00002FF8
    private void DrawTitle(int x, ref int y, int w, int h)
    {
      Text.Font = GameFont.Medium;
      SZWidgets.Label(new Rect((float)x, (float)y, (float)w, (float)h), Label.ADD_THOUGHT, null, "");
      GUI.DrawTexture(new Rect((float)(x + w - 32), (float)y, 30f, 30f), ContentFinder<Texture2D>.Get("bmemory", true));
      y += 30;
    }

    // Token: 0x060000AD RID: 173 RVA: 0x00004E60 File Offset: 0x00003060
    private void DrawUpperDropdowns(int x, ref int y, int w, int h)
    {
      Text.Font = GameFont.Small;
      SZWidgets.FloatMenuOnButtonText<string>(new Rect((float)x, (float)y, (float)w, (float)h), this.selectedModName ?? Label.ALL + " mods", this.lModnames, (string s) => s ?? Label.ALL, new Action<string>(this.AModnameSelected), "");
      y += 30;
      SZWidgets.FloatMenuOnButtonText<string>(new Rect((float)x, (float)y, (float)w, (float)h), this.selectedType ?? Label.ALL + " types", this.lListType, (string s) => s ?? Label.ALL, new Action<string>(this.AThoughtTypeSelected), "");
      y += 32;
    }

    // Token: 0x060000AE RID: 174 RVA: 0x00004F38 File Offset: 0x00003138
    private void AModnameSelected(string val)
    {
      this.selectedModName = val;
      bool flag = this.selectedModName == null || val == Label.ALL;
      if (flag)
      {
        this.lTMemory = CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtMemory);
        this.lTMemorySocial = CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtMemorySocial);
        this.lTSituational = CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtSituational);
        this.lTSituationalSocial = CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtSituationalSocial);
        this.lTUnsupported = CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtUnsupported);
        this.lTAll = CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtsAll);
      }
      else
      {
        this.lTMemory = (from t in CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtMemory)
                         where t.IsFromMod(this.selectedModName)
                         orderby t.label
                         select t).ToList<ThoughtDef>();
        this.lTMemorySocial = (from t in CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtMemorySocial)
                               where t.IsFromMod(this.selectedModName)
                               select t).ToList<ThoughtDef>();
        this.lTSituational = (from t in CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtSituational)
                              where t.IsFromMod(this.selectedModName)
                              select t).ToList<ThoughtDef>();
        this.lTSituationalSocial = (from t in CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtSituationalSocial)
                                    where t.IsFromMod(this.selectedModName)
                                    select t).ToList<ThoughtDef>();
        this.lTUnsupported = (from t in CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtUnsupported)
                              where t.IsFromMod(this.selectedModName)
                              select t).ToList<ThoughtDef>();
        this.lTAll = (from t in CEditor.API.Get<List<ThoughtDef>>(EType.ThoughtsAll)
                      where t.IsFromMod(this.selectedModName)
                      select t).ToList<ThoughtDef>();
      }
      this.AThoughtTypeSelected(this.selectedType);
    }

    // Token: 0x060000AF RID: 175 RVA: 0x000050DC File Offset: 0x000032DC
    private void AThoughtTypeSelected(string val)
    {
      this.selectedType = val;
      this.SelectedPawn = null;
      this.selectedStage = null;
      this.selectedThought = null;
      this.selectedTitle = null;
      this.dicStages.Clear();
      bool flag = val == Label.TH_MEMORY;
      if (flag)
      {
        this.lListThoughts = this.lTMemory;
      }
      else
      {
        bool flag2 = val == Label.TH_SOCIAL;
        if (flag2)
        {
          this.lListThoughts = this.lTMemorySocial;
        }
        else
        {
          bool flag3 = val == Label.TH_SITUATIONAL;
          if (flag3)
          {
            this.lListThoughts = this.lTSituational;
          }
          else
          {
            bool flag4 = val == Label.TH_SITUATIONAL + Label.TH_SOCIAL;
            if (flag4)
            {
              this.lListThoughts = this.lTSituationalSocial;
            }
            else
            {
              bool flag5 = val == Label.TH_UNSUPPORTED;
              if (flag5)
              {
                this.lListThoughts = this.lTUnsupported;
              }
              else
              {
                this.lListThoughts = this.lTAll;
              }
            }
          }
        }
      }
    }

    // Token: 0x060000B0 RID: 176 RVA: 0x000051C8 File Offset: 0x000033C8
    private void DrawThoughtList(int x, ref int y, int w, int h)
    {
      SZWidgets.ListView<ThoughtDef>(x, y, w, h, this.lListThoughts, this.FGetThoughtLabel, this.FGetTooltip, (ThoughtDef t1, ThoughtDef t2) => t1 == t2, ref this.selectedThought, ref this.scrollPos, false, new Action<ThoughtDef>(this.AThoughtSelected), true, false, false, false);
      y += 510;
    }

    // Token: 0x060000B1 RID: 177 RVA: 0x00005244 File Offset: 0x00003444
    private void AThoughtSelected(ThoughtDef t)
    {
      this.selectedThought = t;
      this.SelectedPawn = null;
      this.selectedStage = null;
      this.selectedTitle = null;
      this.dicStages.Clear();
      bool flag = t == null;
      if (!flag)
      {
        this.selectedMoodOffset = 1f;
        int num = this.selectedThought.stages.CountAllowNull<ThoughtStage>();
        bool flag2 = !this.selectedThought.stages.NullOrEmpty<ThoughtStage>();
        if (flag2)
        {
          for (int i = 0; i < num; i++)
          {
            string thoughtLabel = t.GetThoughtLabel(i, null);
            bool flag3 = thoughtLabel == t.defName;
            if (flag3)
            {
              this.dicStages.Add(i, "");
            }
            else
            {
              this.dicStages.Add(i, thoughtLabel);
            }
          }
          this.selectedStage = this.dicStages.First<KeyValuePair<int, string>>().Value;
          this.baseMoodOffset = (float)((int)t.stages[this.dicStages.KeyByValue(this.selectedStage)].baseMoodEffect);
          this.selectedOpinionOffset = t.stages[this.dicStages.KeyByValue(this.selectedStage)].baseOpinionOffset;
        }
        else
        {
          this.selectedStage = null;
        }
      }
    }

    // Token: 0x060000B2 RID: 178 RVA: 0x00005388 File Offset: 0x00003588
    private void DrawLowerDropdowns(int x, ref int y, int w, int h)
    {
      Text.Font = GameFont.Small;
      bool flag = this.selectedThought.HasOtherPawnMember();
      if (flag)
      {
        this.bNeedOtherPawn = true;
        SZWidgets.ButtonText(new Rect((float)x, (float)y, (float)w, (float)h), this.FGetPawnName(this.SelectedPawn), new Action(this.ASelectPawn), "");
        y += 30;
      }
      else
      {
        this.bNeedOtherPawn = false;
        this.SelectedPawn = null;
      }
      bool flag2 = this.selectedThought != null && this.selectedThought.stages.CountAllowNull<ThoughtStage>() > 1;
      if (flag2)
      {
        this.bNeedStage = true;
        SZWidgets.FloatMenuOnButtonText<string>(new Rect((float)x, (float)y, (float)w, (float)h), this.selectedStage, this.dicStages.Values.ToList<string>(), (string s) => s, new Action<string>(this.AStageSelected));
        y += 30;
      }
      else
      {
        this.bNeedStage = false;
        this.selectedStage = null;
      }
      bool flag3 = this.selectedThought != null && this.selectedThought.IsForTitle();
      if (flag3)
      {
        this.bNeedTitle = true;
        SZWidgets.FloatMenuOnButtonText<RoyalTitleDef>(new Rect((float)x, (float)y, (float)w, (float)h), this.FGetRoyalTitleLabel(this.selectedTitle), this.lRoyalTitles, this.FGetRoyalTitleLabel, new Action<RoyalTitleDef>(this.ARoyalTitleSelected), "");
        y += 30;
      }
      else
      {
        this.bNeedTitle = false;
        this.selectedTitle = null;
      }
    }

    // Token: 0x060000B3 RID: 179 RVA: 0x0000551F File Offset: 0x0000371F
    private void ASelectPawn()
    {
      WindowTool.Open(new DialogChoosePawn(this, 1, Gender.None));
    }

    // Token: 0x060000B4 RID: 180 RVA: 0x00005530 File Offset: 0x00003730
    private void AStageSelected(string val)
    {
      this.selectedStage = val;
      bool flag = !this.selectedThought.stages.NullOrEmpty<ThoughtStage>();
      if (flag)
      {
        this.baseMoodOffset = (float)((int)this.selectedThought.stages[this.dicStages.KeyByValue(this.selectedStage)].baseMoodEffect);
        this.selectedOpinionOffset = this.selectedThought.stages[this.dicStages.KeyByValue(this.selectedStage)].baseOpinionOffset;
      }
    }

    // Token: 0x060000B5 RID: 181 RVA: 0x000055B8 File Offset: 0x000037B8
    private void ARoyalTitleSelected(RoyalTitleDef val)
    {
      this.selectedTitle = val;
    }

    // Token: 0x060000B6 RID: 182 RVA: 0x000055C4 File Offset: 0x000037C4
    private void DrawSlider(int x, ref int y, int w, int h)
    {
      bool flag = this.selectedThought == null;
      if (!flag)
      {
        Listing_X listing_X = new Listing_X();
        bool flag2 = this.baseMoodOffset != 0f;
        if (flag2)
        {
          bool flag3 = this.selectedThought.IsTypeOf<Thought_Memory>();
          if (flag3)
          {
            SZWidgets.SimpleMultiplierSlider(new Rect((float)x, (float)y, (float)w, 45f), "mood", "int", false, this.baseMoodOffset, ref this.selectedMoodOffset, -4f, 5f);
            y += 45;
          }
          else
          {
            int num = (int)(this.selectedMoodOffset * this.baseMoodOffset);
            string s = "mood [" + num.ToString() + "]";
            SZWidgets.Label(new Rect((float)x, (float)y, (float)w, 30f), s.Colorize((num > 0) ? Color.green : ((num == 0) ? Color.grey : Color.red)), null, "");
            y += 30;
          }
        }
        bool flag4 = this.selectedThought.IsTypeOf<Thought_MemorySocial>();
        if (flag4)
        {
          SZWidgets.SimpleMultiplierSlider(new Rect((float)x, (float)y, (float)w, 45f), "opinion", "int", false, 1f, ref this.selectedOpinionOffset, -100f, 100f);
          y += 45;
        }
        else
        {
          bool flag5 = this.selectedOpinionOffset != 0f || this.SelectedPawn != null;
          if (flag5)
          {
            SZWidgets.Label(new Rect((float)x, (float)y, (float)w, 30f), "opinion [" + ((int)this.selectedOpinionOffset).ToString() + "]", null, "");
            y += 30;
          }
        }
      }
    }

    // Token: 0x060000B7 RID: 183 RVA: 0x00005780 File Offset: 0x00003980
    private void DrawAccept(int x, ref int y, int w, int h)
    {
      Rect rect = new Rect(this.InitialSize.x - 140f, this.InitialSize.y - 70f, 100f, 30f);
      this.bAllOk = (this.selectedThought != null && (!this.bNeedOtherPawn || this.SelectedPawn != null) && (!this.bNeedTitle || this.selectedTitle != null) && (!this.bNeedStage || this.selectedStage != null));
      GUI.color = ((!this.bAllOk || (this.bNeedStage && this.selectedStage.NullOrEmpty())) ? Color.red : Color.green);
      bool flag = this.bAllOk;
      if (flag)
      {
        SZWidgets.ButtonText(rect, "OK".Translate(), new Action(this.DoAndClose), "");
      }
    }

    // Token: 0x060000B8 RID: 184 RVA: 0x00005868 File Offset: 0x00003A68
    private void DoAndClose()
    {
      int stage = 0;
      bool flag = this.selectedStage != null;
      if (flag)
      {
        stage = this.dicStages.KeyByValue(this.selectedStage);
      }
      CEditor.API.Pawn.AddThought(this.selectedThought, this.SelectedPawn, stage, this.selectedTitle.SDefname(), this.selectedMoodOffset, this.selectedOpinionOffset, 0);
      SearchTool.Save(this.idxList);
      this.Close(true);
    }


  }
}
