﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000058 RID: 88
	public class DialogAddTrait : Window
	{
		// Token: 0x17000067 RID: 103
		// (get) Token: 0x0600053A RID: 1338 RVA: 0x0003631C File Offset: 0x0003451C
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(300f, 788f);
			}
		}

		// Token: 0x0600053B RID: 1339 RVA: 0x00036340 File Offset: 0x00034540
		public DialogAddTrait(Trait _trait = null)
		{
			this.oldTrait = _trait;
			this.scrollPos = default(Vector2);
			this.doOnce = true;
			this.idxList = 2;
			this.search = SearchTool.Update(this.idxList);
			this.lOfSM = TraitTool.ListOfTraitStatModifier(this.search.modName, true);
			this.lOfFilters = new List<string>();
			this.lOfFilters.Add(null);
			this.lOfFilters.Add(Label.STAT);
			this.lOfFilters.Add(Label.MENTAL);
			this.lOfFilters.Add(Label.THOUGHTS);
			this.lOfFilters.Add(Label.INSPIRATIONS);
			this.lOfFilters.Add(Label.FOCUS);
			this.lOfFilters.Add(Label.SKILLGAINS);
			this.lOfFilters.Add(Label.ABILITIES);
			this.lOfFilters.Add(Label.NEEDS);
			this.lOfFilters.Add(Label.INGESTIBLEMOD);
			this.lOfTraits = TraitTool.ListOfTraitsKeyValuePair(this.search.modName, (StatModifier)this.search.ofilter1, this.search.filter1);
			this.UpdateDicTooltip();
			this.selectedTrait = this.lOfTraits.FirstOrDefault<KeyValuePair<TraitDef, TraitDegreeData>>();
			this.oldSelectedTrait = this.selectedTrait;
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x0600053C RID: 1340 RVA: 0x00036558 File Offset: 0x00034758
		private void UpdateDicTooltip()
		{
			DialogAddTrait.dicDesc.Clear();
			bool flag = !this.lOfTraits.NullOrEmpty<KeyValuePair<TraitDef, TraitDegreeData>>();
			if (flag)
			{
				foreach (KeyValuePair<TraitDef, TraitDegreeData> key in this.lOfTraits)
				{
					bool flag2 = key.Key != null;
					if (flag2)
					{
						string text = key.Key.GetModName() + "\n";
						string text2 = (key.Value != null) ? key.Value.description : key.Key.description;
						bool flag3 = !text2.NullOrEmpty();
						if (flag3)
						{
							try
							{
								Trait trait = new Trait(key.Key, (key.Value != null) ? key.Value.degree : 0, false);
								text2 = trait.TipString(CEditor.API.Pawn);
							}
							catch
							{
							}
						}
						text += text2;
						DialogAddTrait.dicDesc.Add(key, text);
					}
				}
			}
		}

		// Token: 0x0600053D RID: 1341 RVA: 0x0003669C File Offset: 0x0003489C
		public override void DoWindowContents(Rect inRect)
		{
			bool flag = this.doOnce;
			if (flag)
			{
				this.doOnce = false;
				this.windowRect.position = new Vector2(this.windowRect.position.x - 380f, this.windowRect.position.y);
			}
			int h = (int)this.InitialSize.y - 115;
			int num = (int)this.InitialSize.x - 40;
			int x = 0;
			int y = 0;
			SZWidgets.ButtonImage((float)(num - 25), 0f, 25f, 25f, "brandom", new Action(this.ARandomTrait), "");
			y = this.DrawTitle(x, y, num, 30);
			y = this.DrawDropdown(x, y, num);
			y = this.DrawList(x, y, num, h);
			Rect rect = new Rect(this.InitialSize.x - 140f, this.InitialSize.y - 70f, 100f, 30f);
			bool flag2 = Widgets.ButtonText(rect, "OK".Translate(), true, true, true);
			if (flag2)
			{
				this.DoAndClose();
			}
		}

		// Token: 0x0600053E RID: 1342 RVA: 0x000367C8 File Offset: 0x000349C8
		private int DrawTitle(int x, int y, int w, int h)
		{
			Text.Font = GameFont.Medium;
			Widgets.Label(new Rect((float)x, (float)y, (float)w, (float)h), Label.ADD_TRAIT);
			return h;
		}

		// Token: 0x0600053F RID: 1343 RVA: 0x000367FC File Offset: 0x000349FC
		private int DrawDropdown(int x, int y, int w)
		{
			Text.Font = GameFont.Small;
			Rect rect = new Rect((float)x, (float)y, (float)w, 30f);
			SZWidgets.FloatMenuOnButtonText<string>(rect, this.search.SelectedModName, CEditor.API.ListOf<string>(EType.ModsTraitDef), (string s) => s ?? Label.ALL, new Action<string>(this.AChangedModName), "");
			Rect rect2 = new Rect(rect)
			{
				y = rect.y + 30f
			};
			SZWidgets.FloatMenuOnButtonText<StatModifier>(rect2, this.FStatLabel((StatModifier)this.search.ofilter1), this.lOfSM, this.FStatLabel, new Action<StatModifier>(this.AChangedSM), "");
			SZWidgets.FloatMenuOnButtonText<string>(new Rect(rect2)
			{
				y = rect2.y + 30f
			}, this.search.SelectedFilter1, this.lOfFilters, (string s) => (s == null) ? Label.ALL : s, new Action<string>(this.AChangedCategory));
			return y + 90;
		}

		// Token: 0x06000540 RID: 1344 RVA: 0x00036938 File Offset: 0x00034B38
		private int DrawList(int x, int y, int w, int h)
		{
			Text.Font = GameFont.Small;
			SZWidgets.ListView(x, y, w, h - y + 30, this.lOfTraits, this.FTraitLabel, this.FTraitTooltip, this.FTraitComparator, ref this.selectedTrait, ref this.scrollPos, false, null, true, false, false, false);
			bool flag = !this.FTraitComparator(this.oldSelectedTrait, this.selectedTrait);
			if (flag)
			{
				this.oldSelectedTrait = this.selectedTrait;
				bool devMode = Prefs.DevMode;
				if (devMode)
				{
					MessageTool.Show(this.selectedTrait.Key.defName, null);
				}
			}
			return h - y;
		}

		// Token: 0x06000541 RID: 1345 RVA: 0x000369E0 File Offset: 0x00034BE0
		private void AChangedModName(string val)
		{
			this.search.modName = val;
			this.lOfTraits = TraitTool.ListOfTraitsKeyValuePair(this.search.modName, (StatModifier)this.search.ofilter1, this.search.filter1);
			this.UpdateDicTooltip();
		}

		// Token: 0x06000542 RID: 1346 RVA: 0x00036A34 File Offset: 0x00034C34
		private void AChangedSM(StatModifier val)
		{
			this.search.ofilter1 = val;
			this.lOfTraits = TraitTool.ListOfTraitsKeyValuePair(this.search.modName, (StatModifier)this.search.ofilter1, this.search.filter1);
			this.UpdateDicTooltip();
		}

		// Token: 0x06000543 RID: 1347 RVA: 0x00036A88 File Offset: 0x00034C88
		private void AChangedCategory(string val)
		{
			this.search.filter1 = val;
			this.lOfTraits = TraitTool.ListOfTraitsKeyValuePair(this.search.modName, (StatModifier)this.search.ofilter1, this.search.filter1);
			this.UpdateDicTooltip();
		}

		// Token: 0x06000544 RID: 1348 RVA: 0x00036ADA File Offset: 0x00034CDA
		private void ARandomTrait()
		{
			SZWidgets.sFind = this.FTraitLabel(this.lOfTraits.RandomElement<KeyValuePair<TraitDef, TraitDegreeData>>());
			SZWidgets.sFindTemp = SZWidgets.sFind;
			SZWidgets.waitTimer = 120;
		}

		// Token: 0x06000545 RID: 1349 RVA: 0x00036B0C File Offset: 0x00034D0C
		private void DoAndClose()
		{
			bool flag = this.selectedTrait.Key != null;
			if (flag)
			{
				CEditor.API.Pawn.AddTrait(this.selectedTrait.Key, this.selectedTrait.Value, false, true, this.oldTrait);
			}
			SearchTool.Save(this.idxList);
			this.Close(true);
		}

		// Token: 0x06000546 RID: 1350 RVA: 0x00036B70 File Offset: 0x00034D70
		public override void OnAcceptKeyPressed()
		{
			base.OnAcceptKeyPressed();
			this.DoAndClose();
		}

		// Token: 0x040004C8 RID: 1224
		private int idxList;

		// Token: 0x040004C9 RID: 1225
		private List<KeyValuePair<TraitDef, TraitDegreeData>> lOfTraits;

		// Token: 0x040004CA RID: 1226
		private static Dictionary<KeyValuePair<TraitDef, TraitDegreeData>, string> dicDesc = new Dictionary<KeyValuePair<TraitDef, TraitDegreeData>, string>();

		// Token: 0x040004CB RID: 1227
		private Vector2 scrollPos;

		// Token: 0x040004CC RID: 1228
		private KeyValuePair<TraitDef, TraitDegreeData> selectedTrait;

		// Token: 0x040004CD RID: 1229
		private KeyValuePair<TraitDef, TraitDegreeData> oldSelectedTrait;

		// Token: 0x040004CE RID: 1230
		private bool doOnce;

		// Token: 0x040004CF RID: 1231
		private List<StatModifier> lOfSM;

		// Token: 0x040004D0 RID: 1232
		private List<string> lOfFilters;

		// Token: 0x040004D1 RID: 1233
		private SearchTool search;

		// Token: 0x040004D2 RID: 1234
		private Trait oldTrait;

		// Token: 0x040004D3 RID: 1235
		private Func<KeyValuePair<TraitDef, TraitDegreeData>, string> FTraitLabel = delegate(KeyValuePair<TraitDef, TraitDegreeData> t)
		{
			string result;
			if (t.Value == null)
			{
				result = t.Key.LabelCap.RawText;
			}
			else if ((result = t.Value.LabelCap) == null)
			{
				result = (t.Value.label ?? t.Key.label);
			}
			return result;
		};

		// Token: 0x040004D4 RID: 1236
		private Func<KeyValuePair<TraitDef, TraitDegreeData>, string> FTraitTooltip = (KeyValuePair<TraitDef, TraitDegreeData> t) => DialogAddTrait.dicDesc.GetValue(t);

		// Token: 0x040004D5 RID: 1237
		private Func<KeyValuePair<TraitDef, TraitDegreeData>, KeyValuePair<TraitDef, TraitDegreeData>, bool> FTraitComparator = (KeyValuePair<TraitDef, TraitDegreeData> t1, KeyValuePair<TraitDef, TraitDegreeData> t2) => t1.Key == t2.Key && t1.Value == t2.Value;

		// Token: 0x040004D6 RID: 1238
		private Func<StatModifier, string> FStatLabel = (StatModifier t) => (t == null) ? Label.ALL : t.stat.LabelCap.ToString();
	}
}
