﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace CharacterEditor
{
	// Token: 0x02000043 RID: 67
	public class DialogArmory : Window
	{
		// Token: 0x17000056 RID: 86
		// (get) Token: 0x060003A7 RID: 935 RVA: 0x0001DE90 File Offset: 0x0001C090
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(this.MAXW, 800f);
			}
		}

		// Token: 0x060003A8 RID: 936 RVA: 0x0001DEB4 File Offset: 0x0001C0B4
		private string GetThingStyleDefLabel(ThingStyleDef t)
		{
			return (t == null) ? Label.NONE : (t.label.NullOrEmpty() ? t.defName : t.label.CapitalizeFirst());
		}

		// Token: 0x060003A9 RID: 937 RVA: 0x0001DEF0 File Offset: 0x0001C0F0
		private string GetStatLabel(StatDef s)
		{
			return (s == null) ? Label.NONE : s.label;
		}

		// Token: 0x060003AA RID: 938 RVA: 0x0001DF14 File Offset: 0x0001C114
		private string GetTradeabilityLabel(Tradeability t)
		{
			return Enum.GetName(typeof(Tradeability), t);
		}

		// Token: 0x060003AB RID: 939 RVA: 0x0001DF3C File Offset: 0x0001C13C
		private string GetTechLevelLabel(TechLevel t)
		{
			return Enum.GetName(typeof(TechLevel), t);
		}

		// Token: 0x060003AC RID: 940 RVA: 0x0001DF64 File Offset: 0x0001C164
		private string GetTagLabel(string s)
		{
			return s ?? Label.NONE;
		}

		// Token: 0x060003AD RID: 941 RVA: 0x0001DF80 File Offset: 0x0001C180
		private string GetApparelLayerLabel(ApparelLayerDef a)
		{
			return (a == null) ? Label.ALL : a.label.CapitalizeFirst();
		}

		// Token: 0x060003AE RID: 942 RVA: 0x0001DFA8 File Offset: 0x0001C1A8
		private string GetBodyPartGroupLabel(BodyPartGroupDef b)
		{
			return (b == null) ? Label.ALL : b.label.CapitalizeFirst();
		}

		// Token: 0x060003AF RID: 943 RVA: 0x0001DFD0 File Offset: 0x0001C1D0
		private string GetStuffCategoryLabel(StuffCategoryDef stuffCategoryDef)
		{
			return stuffCategoryDef.label;
		}

		// Token: 0x060003B0 RID: 944 RVA: 0x0001DFE8 File Offset: 0x0001C1E8
		private string GetApparelLabel(ThingDef thingDef)
		{
			string label = thingDef.label;
			return (label != null) ? label.CapitalizeFirst() : null;
		}

		// Token: 0x060003B1 RID: 945 RVA: 0x0001E00C File Offset: 0x0001C20C
		private string GetApparelTooltip(ThingDef thingDef)
		{
			return thingDef.GetModName() + "\n" + thingDef.description;
		}

		// Token: 0x060003B2 RID: 946 RVA: 0x0001E034 File Offset: 0x0001C234
		private bool IsSameApparel(ThingDef a, ThingDef b)
		{
			return a == b;
		}

		// Token: 0x060003B3 RID: 947 RVA: 0x0001E04C File Offset: 0x0001C24C
		public DialogArmory(Thing toSelect = null)
		{
			this.idxList = 6;
			bool flag = toSelect != null;
			if (flag)
			{
				SearchTool.ClearSearch(this.idxList);
			}
			this.search = SearchTool.Update(this.idxList);
			this.scrollPos = default(Vector2);
			this.scrollPosParam = default(Vector2);
			this.lOfApparel = new List<ThingDef>();
			this.lOfQuality = new List<int>();
			foreach (QualityCategory item in QualityUtility.AllQualityCategories)
			{
				this.lOfQuality.Add((int)item);
			}
			this.lOfAllLayers = ApparelTool.ListOfApparelLayerDefs(true);
			this.lCopyStat = new List<StatModifier>();
			this.lCopyEquipStat = new List<StatModifier>();
			this.lCopyStuff = new List<StuffCategoryDef>();
			this.lCopyCost = new List<ThingDefCountClass>();
			this.lCopyBodyPartGroup = new List<BodyPartGroupDef>();
			this.lCopyApparelLayer = new List<ApparelLayerDef>();
			this.lCopyApparelTag = new List<string>();
			this.lCopyOutfitTag = new List<string>();
			this.iChangeTick0 = 0;
			this.iTick_AllowRemoveStat = 0;
			this.previousColor = Color.white;
			this.allowRemove = false;
			this.doOnce = true;
			this.toggleText = ">>";
			this.tempName = "";
			this.FApparelComparator = new Func<ThingDef, ThingDef, bool>(this.IsSameApparel);
			this.FApparelLabel = new Func<ThingDef, string>(this.GetApparelLabel);
			this.FApparelTooltip = new Func<ThingDef, string>(this.GetApparelTooltip);
			this.FGetApparelLayerLabel = new Func<ApparelLayerDef, string>(this.GetApparelLayerLabel);
			this.FGetBodyPartGroupLabel = new Func<BodyPartGroupDef, string>(this.GetBodyPartGroupLabel);
			this.FGetStuffCategoryLabel = new Func<StuffCategoryDef, string>(this.GetStuffCategoryLabel);
			this.FGetTagLabel = new Func<string, string>(this.GetTagLabel);
			this.FGetTechLevelLabel = new Func<TechLevel, string>(this.GetTechLevelLabel);
			this.FGetTradeabilityLabel = new Func<Tradeability, string>(this.GetTradeabilityLabel);
			this.FGetStatLabel = new Func<StatDef, string>(this.GetStatLabel);
			this.FGetThingStyleDefLabel = new Func<ThingStyleDef, string>(this.GetThingStyleDefLabel);
			this.SELECTED = Selected.ByThingDef(null);
			this.search.ofilter1 = ((toSelect != null) ? toSelect.def.apparel.layers.FirstOrDefault<ApparelLayerDef>() : this.search.ofilter1);
			this.search.ofilter2 = ((toSelect != null) ? toSelect.def.apparel.bodyPartGroups.FirstOrDefault<BodyPartGroupDef>() : this.search.ofilter2);
			this.lOfApparel = ApparelTool.ListOfApparel((ApparelLayerDef)this.search.ofilter1, this.search.modName, (BodyPartGroupDef)this.search.ofilter2);
			this.ASelectApparelFromSelector(toSelect);
			this.firstPreselection = (toSelect != null);
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x060003B4 RID: 948 RVA: 0x0001E364 File Offset: 0x0001C564
		public override void DoWindowContents(Rect inRect)
		{
			float num = this.InitialSize.y - 104f;
			this.MAXW = this.W_APPA + this.W_PARAMS + 10f;
			this.windowRect.width = (CEditor.IsExtendedUI ? this.MAXW : (this.W_APPA + 36f));
			this.toggleText = (CEditor.IsExtendedUI ? "<<" : ">>");
			bool flag = this.doOnce;
			if (flag)
			{
				this.doOnce = false;
				this.windowRect.position = new Vector2((float)(CEditor.API.EditorPosX + 420), this.windowRect.y);
			}
			float num2 = 0f;
			float num3 = 0f;
			this.SELECTED.tooltip = Label.PRICE + this.SELECTED.buyPrice.ToString();
			this.DrawEditLabel(num3, num2, this.W_APPA, 30f);
			num2 += 30f;
			this.DrawTexture(num3 + 50f, ref num2, this.W_APPA, this.W_APPA);
			this.DrawDropdown(num3, num2, this.W_APPA, 30f);
			num2 += 92f;
			this.DrawList(num3, num2, this.W_APPA, num - num2 + 20f);
			num2 += num - num2 + 30f;
			bool isExtendedUI = CEditor.IsExtendedUI;
			if (isExtendedUI)
			{
				this.DrawParameter((int)(this.MAXW - this.W_PARAMS), 0, (int)(this.W_PARAMS - 40f), (int)num + 20);
			}
			bool isExtendedUI2 = CEditor.IsExtendedUI;
			if (isExtendedUI2)
			{
				this.DrawLowerButtons(num3, num2, this.MAXW - this.W_APPA - 40f, 40f);
			}
			SZWidgets.ButtonText(240f, this.InitialSize.y - 75f, 30f, 40f, this.toggleText, new Action(this.AToggleStats), "");
			bool flag2 = this.SELECTED != null && this.SELECTED.thingDef != null && Widgets.ButtonText(new Rect(140f, this.InitialSize.y - 75f, 100f, 40f), "OK".Translate(), true, true, true);
			if (flag2)
			{
				this.DoAndClose();
			}
		}

		// Token: 0x060003B5 RID: 949 RVA: 0x0001E5C5 File Offset: 0x0001C7C5
		public override void OnAcceptKeyPressed()
		{
			base.OnAcceptKeyPressed();
			this.DoAndClose();
		}

		// Token: 0x060003B6 RID: 950 RVA: 0x0001E5D8 File Offset: 0x0001C7D8
		public void DoAndClose()
		{
			bool flag = this.SELECTED.Stuff != null && !this.SELECTED.thingDef.MadeFromStuff;
			if (flag)
			{
				this.SELECTED.Stuff = null;
			}
			SearchTool.Save(this.idxList);
			bool flag2 = this.SELECTED.thingDef != null;
			if (flag2)
			{
				List<Apparel> conflictedApparelList = CEditor.API.Pawn.GetConflictedApparelList(this.SELECTED.thingDef);
				bool flag3 = conflictedApparelList.Count > 1;
				if (flag3)
				{
					MessageTool.ShowCustomDialog(conflictedApparelList.ListToString<Apparel>(), this.SELECTED.thingDef.label.CapitalizeFirst() + " " + Label.WILL_REPLACE, null, new Action(this.Confirmed), null);
				}
				else
				{
					this.Confirmed();
				}
			}
			this.Close(true);
		}

		// Token: 0x060003B7 RID: 951 RVA: 0x0001E6B0 File Offset: 0x0001C8B0
		private void Confirmed()
		{
			Apparel a;
			bool flag = !CEditor.API.Pawn.CreateAndWearApparel(this.SELECTED, out a, true);
			if (flag)
			{
				CEditor.API.Pawn.AskToWearIncompatibleApparel(a);
			}
			CEditor.API.UpdateGraphics();
		}

		// Token: 0x060003B8 RID: 952 RVA: 0x0001E6FC File Offset: 0x0001C8FC
		private void DrawDropdown(float x, float y, float w, float h)
		{
			Rect rect = new Rect(x, y, w, h);
			SZWidgets.FloatMenuOnButtonText<string>(rect, this.search.SelectedModName, CEditor.API.Get<List<string>>(EType.ModsApparel), (string s) => s ?? Label.ALL, new Action<string>(this.ASelectedModName), "");
			Rect rect2 = new Rect(rect);
			rect2.y += 30f;
			SZWidgets.FloatMenuOnButtonText<ApparelLayerDef>(rect2, this.GetApparelLayerLabel((ApparelLayerDef)this.search.ofilter1), this.lOfAllLayers, this.FGetApparelLayerLabel, new Action<ApparelLayerDef>(this.ASelectLayerCat));
			Rect rect3 = new Rect(rect2);
			rect3.y += 30f;
			SZWidgets.FloatMenuOnButtonText<BodyPartGroupDef>(rect3, this.GetBodyPartGroupLabel((BodyPartGroupDef)this.search.ofilter2), BodyTool.ListAllBodyPartGroupDefs(true), this.FGetBodyPartGroupLabel, new Action<BodyPartGroupDef>(this.ASelectedBodyPart));
		}

		// Token: 0x060003B9 RID: 953 RVA: 0x0001E806 File Offset: 0x0001CA06
		private void ASelectedModName(string val)
		{
			this.search.modName = val;
			this.ASelectLayerCat(null);
		}

		// Token: 0x060003BA RID: 954 RVA: 0x0001E820 File Offset: 0x0001CA20
		private void ASelectLayerCat(ApparelLayerDef val)
		{
			this.search.ofilter1 = val;
			this.lOfApparel = ApparelTool.ListOfApparel(val, this.search.modName, (BodyPartGroupDef)this.search.ofilter2);
			this.SELECTED.Init(this.lOfApparel.FirstOrFallback(null), true, null, null);
		}

		// Token: 0x060003BB RID: 955 RVA: 0x0001E87C File Offset: 0x0001CA7C
		private void ASelectedBodyPart(BodyPartGroupDef val)
		{
			this.search.ofilter2 = val;
			this.lOfApparel = ApparelTool.ListOfApparel((ApparelLayerDef)this.search.ofilter1, this.search.modName, val);
			this.SELECTED.Init(this.lOfApparel.FirstOrFallback(null), true, null, null);
		}

		// Token: 0x060003BC RID: 956 RVA: 0x0001E8D8 File Offset: 0x0001CAD8
		private void DrawEditLabel(float x, float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				Text.Font = GameFont.Medium;
				Rect rect = new Rect(x, y, w, h);
				bool flag2 = this.iChangeTick0 <= 0;
				if (flag2)
				{
					SZWidgets.Label(rect, this.SELECTED.thingDef.label, delegate()
					{
						this.iChangeTick0 = 300;
					}, "");
				}
				else
				{
					this.tempName = Widgets.TextField(rect, this.SELECTED.currentApparel.label);
					bool flag3 = !this.SELECTED.thingDef.label.Equals(this.tempName);
					if (flag3)
					{
						this.SELECTED.currentApparel.label = this.tempName;
						this.iChangeTick0 = 300;
					}
					this.iChangeTick0--;
				}
			}
		}

		// Token: 0x060003BD RID: 957 RVA: 0x0001E9C0 File Offset: 0x0001CBC0
		private void DrawList(float x, float y, float w, float h)
		{
			SZWidgets.ListView<ThingDef>(x, y, this.W_APPA, h, this.lOfApparel, this.FApparelLabel, this.FApparelTooltip, this.FApparelComparator, ref this.SELECTED.thingDef, ref this.scrollPos, false, new Action<ThingDef>(this.AApparelSelected), true, true, true, false);
		}

		// Token: 0x060003BE RID: 958 RVA: 0x0001EA18 File Offset: 0x0001CC18
		private void DrawLowerButtons(float x, float y, float w, float h)
		{
			float num = (float)((int)(w / 3f));
			x += this.W_APPA;
			SZWidgets.ButtonText(x, y, num, h, Label.RESET, new Action(this.AResetOne), "");
			x += num;
			SZWidgets.ButtonText(x, y, num, h, Label.RESETALL, new Action(this.AResetAll), "");
			x += num;
			SZWidgets.ButtonText(x, y, num, h, Label.SAVE, new Action(this.ASave), "");
		}

		// Token: 0x060003BF RID: 959 RVA: 0x0001EAA8 File Offset: 0x0001CCA8
		private void DrawTexture(float x, ref float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				Texture2D texture = this.SELECTED.thingDef.GetTexture(1, this.SELECTED.Style, default(Rot4));
				GUI.color = this.SELECTED.DrawColor;
				Rect rect = new Rect(39f, y, 192f, 192f);
				GUI.DrawTexture(rect, texture, ScaleMode.ScaleAndCrop, true, 1f);
				y += 192f;
				y = SZWidgets.NavSelectorQuality(new Rect(0f, y, w, 25f), this.SELECTED, this.lOfQuality);
				y = SZWidgets.NavSelectorStuff(new Rect(0f, y, w, 25f), this.SELECTED);
				y = SZWidgets.NavSelectorStyle(new Rect(0f, y, w, 25f), this.SELECTED);
				TooltipHandler.TipRegion(rect, this.SELECTED.tip);
				bool flag2 = !CEditor.InStartingScreen;
				if (flag2)
				{
					SZWidgets.ButtonInvisible(rect, new Action(this.ABuyApparel), "");
				}
			}
		}

		// Token: 0x060003C0 RID: 960 RVA: 0x0001EBD0 File Offset: 0x0001CDD0
		private void DrawParameter(int x, int y, int w, int h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				this.CalcHSCROLL();
				bool flag2 = this.iTick_AllowRemoveStat > 0;
				if (flag2)
				{
					this.iTick_AllowRemoveStat--;
				}
				this.allowRemove = (this.iTick_AllowRemoveStat > 0);
				Rect outRect = new Rect((float)x, (float)y, (float)w, (float)h);
				Rect rect = new Rect(0f, 0f, outRect.width - 16f, this.HSCROLLPARAM);
				Widgets.BeginScrollView(outRect, ref this.scrollPosParam, rect, true);
				Rect rect2 = rect.ContractedBy(4f);
				rect2.y -= 4f;
				rect2.height = this.HSCROLLPARAM;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(rect2);
				listing_X.verticalSpacing = 30f;
				this.DrawStatProperties(listing_X, w);
				this.DrawEquipStatProperties(listing_X, w);
				this.DrawStuffProperties(listing_X, w);
				this.DrawCosts(listing_X, w);
				this.DrawBodyParts(listing_X, w);
				this.DrawLayers(listing_X, w);
				this.DrawTags(listing_X, w);
				this.DrawOutfits(listing_X, w);
				this.DrawExtras(listing_X);
				listing_X.End();
				Widgets.EndScrollView();
			}
		}

		// Token: 0x060003C1 RID: 961 RVA: 0x0001ED14 File Offset: 0x0001CF14
		private void DrawStatProperties(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.STATS);
			Text.Font = GameFont.Small;
			view.ButtonImage((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.ADialogAddStat));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyStat));
			bool flag = !this.lCopyStat.NullOrEmpty<StatModifier>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteStat));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.statBases != null;
			if (flag2)
			{
				try
				{
					foreach (StatModifier statModifier in this.SELECTED.thingDef.statBases)
					{
						bool flag3 = view.ListviewSM(400f, statModifier, statModifier.stat.defName == this.SELECTED.statDefName, this.allowRemove, new Action<string>(this.ARemoveStat));
						if (flag3)
						{
							this.SELECTED.statDefName = ((this.SELECTED.statDefName != null) ? null : statModifier.stat.defName);
						}
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C2 RID: 962 RVA: 0x0001EF14 File Offset: 0x0001D114
		private void DrawEquipStatProperties(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.EQUIPSTATS);
			Text.Font = GameFont.Small;
			view.ButtonImage((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.ADialogAddEquipStat));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyEquipStat));
			bool flag = !this.lCopyEquipStat.NullOrEmpty<StatModifier>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteEquipStat));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.equippedStatOffsets != null;
			if (flag2)
			{
				try
				{
					foreach (StatModifier statModifier in this.SELECTED.thingDef.equippedStatOffsets)
					{
						bool flag3 = view.ListviewSM(400f, statModifier, statModifier.stat.defName == this.SELECTED.equipStatDefName, this.allowRemove, new Action<string>(this.ARemoveEquipStat));
						if (flag3)
						{
							this.SELECTED.equipStatDefName = ((this.SELECTED.equipStatDefName != null) ? null : statModifier.stat.defName);
						}
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C3 RID: 963 RVA: 0x0001F114 File Offset: 0x0001D314
		private void DrawStuffProperties(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.STUFFPROPS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<StuffCategoryDef>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<StuffCategoryDef>(EType.StuffCategory), this.FGetStuffCategoryLabel, new Action<StuffCategoryDef>(this.AAddStuffCategory));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyStuff));
			bool flag = !this.lCopyStuff.NullOrEmpty<StuffCategoryDef>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteStuff));
			}
			view.Gap(30f);
			bool madeFromStuff = this.SELECTED.thingDef.MadeFromStuff;
			if (madeFromStuff)
			{
				view.LabelSimple(Label.COSTSTUFFCOUNT + this.SELECTED.thingDef.costStuffCount.ToString(), 0f, view.CurY, 300f, 25f, null);
				view.Gap(20f);
				this.SELECTED.thingDef.costStuffCount = (int)view.Slider((float)this.SELECTED.thingDef.costStuffCount, 1f, 200f, 300f);
				bool flag2 = this.SELECTED.OldStuffCount != this.SELECTED.thingDef.costStuffCount;
				if (flag2)
				{
					this.SELECTED.OldStuffCount = this.SELECTED.thingDef.costStuffCount;
					this.SELECTED.thingDef.UpdateCostStuffCount();
				}
				try
				{
					foreach (StuffCategoryDef stuffCategoryDef in this.SELECTED.thingDef.stuffCategories)
					{
						view.Listview(400f, stuffCategoryDef.defName, stuffCategoryDef.label, stuffCategoryDef.description, this.allowRemove, new Action<string>(this.ARemoveStuffCategory));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C4 RID: 964 RVA: 0x0001F3C0 File Offset: 0x0001D5C0
		private void DrawCosts(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.COSTS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<ThingDef>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<ThingDef>(EType.CostItems), this.FApparelLabel, new Action<ThingDef>(this.AAddCosts));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyCosts));
			bool flag = !this.lCopyCost.NullOrEmpty<ThingDefCountClass>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteCosts));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.costList != null;
			if (flag2)
			{
				try
				{
					foreach (ThingDefCountClass thingDefCountClass in this.SELECTED.thingDef.costList)
					{
						bool flag3 = view.ListviewTDC(400f, thingDefCountClass, thingDefCountClass.thingDef.defName == this.SELECTED.tdcDefName, this.allowRemove, new Action<string>(this.ARemoveCosts), this.SELECTED.thingDef);
						if (flag3)
						{
							this.SELECTED.tdcDefName = ((this.SELECTED.tdcDefName != null) ? null : thingDefCountClass.thingDef.defName);
						}
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C5 RID: 965 RVA: 0x0001F5E0 File Offset: 0x0001D7E0
		private void DrawBodyParts(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.BODYPARTGROUPS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<BodyPartGroupDef>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", BodyTool.ListAllBodyPartGroupDefs(false), this.FGetBodyPartGroupLabel, new Action<BodyPartGroupDef>(this.AAddBodyPartGroup));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyBodyPartGroup));
			bool flag = !this.lCopyBodyPartGroup.NullOrEmpty<BodyPartGroupDef>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteBodyPartGroup));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.apparel.bodyPartGroups != null;
			if (flag2)
			{
				try
				{
					foreach (BodyPartGroupDef bodyPartGroupDef in this.SELECTED.thingDef.apparel.bodyPartGroups)
					{
						view.Listview(400f, bodyPartGroupDef.defName, bodyPartGroupDef.label, bodyPartGroupDef.description, this.allowRemove, new Action<string>(this.ARemoveBodyPartGroup));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C6 RID: 966 RVA: 0x0001F7BC File Offset: 0x0001D9BC
		private void DrawLayers(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.APPARELLAYER);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<ApparelLayerDef>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", this.lOfAllLayers, this.FGetApparelLayerLabel, new Action<ApparelLayerDef>(this.AAddApparelLayer));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyApparelLayer));
			bool flag = !this.lCopyApparelLayer.NullOrEmpty<ApparelLayerDef>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteApparelLayer));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.apparel.layers != null;
			if (flag2)
			{
				try
				{
					foreach (ApparelLayerDef apparelLayerDef in this.SELECTED.thingDef.apparel.layers)
					{
						view.Listview(400f, apparelLayerDef.defName, apparelLayerDef.label, apparelLayerDef.description, this.allowRemove, new Action<string>(this.ARemoveApparelLayer));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C7 RID: 967 RVA: 0x0001F998 File Offset: 0x0001DB98
		private void DrawTags(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.TAGS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<string>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<string>(EType.ApparelTags), this.FGetTagLabel, new Action<string>(this.AAddApparelTag));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyApparelTag));
			bool flag = !this.lCopyApparelTag.NullOrEmpty<string>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteApparelTag));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.apparel.tags != null;
			if (flag2)
			{
				try
				{
					foreach (string text in this.SELECTED.thingDef.apparel.tags)
					{
						view.Listview(400f, text, text, null, this.allowRemove, new Action<string>(this.ARemoveApparelTag));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C8 RID: 968 RVA: 0x0001FB68 File Offset: 0x0001DD68
		private void DrawOutfits(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.OUTFITTAGS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<string>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<string>(EType.OutfitTags), this.FGetTagLabel, new Action<string>(this.AAddOutfitTag));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyOutfitTag));
			bool flag = !this.lCopyOutfitTag.NullOrEmpty<string>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteOutfitTag));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.apparel.defaultOutfitTags != null;
			if (flag2)
			{
				try
				{
					foreach (string text in this.SELECTED.thingDef.apparel.defaultOutfitTags)
					{
						view.Listview(400f, text, text, null, this.allowRemove, new Action<string>(this.ARemoveOutfitTag));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x060003C9 RID: 969 RVA: 0x0001FD3C File Offset: 0x0001DF3C
		private void DrawExtras(Listing_X view)
		{
			Text.Font = GameFont.Medium;
			view.Label(Label.EXTRAS, 300f, 0f, -1f, null);
			Text.Font = GameFont.Small;
			view.FloatMenuButtonWithLabel<TechLevel>(Label.TECHLEVEL, 100f, 200f, Enum.GetName(typeof(TechLevel), this.SELECTED.thingDef.techLevel), CEditor.API.ListOf<TechLevel>(EType.TechLevel), this.FGetTechLevelLabel, new Action<TechLevel>(this.ASetTechLevel));
			view.FloatMenuButtonWithLabel<Tradeability>(Label.TRADEABILITY, 100f, 200f, Enum.GetName(typeof(Tradeability), this.SELECTED.thingDef.tradeability), CEditor.API.ListOf<Tradeability>(EType.Tradeability), this.FGetTradeabilityLabel, new Action<Tradeability>(this.ASetTradeability));
			view.CheckboxLabeled(Label.STEALABLE, 2f, 120f, ref this.SELECTED.thingDef.stealable, null);
			Text.Font = GameFont.Medium;
			view.Label(Label.GRAPHICS, 300f, 0f, -1f, null);
			Text.Font = GameFont.Tiny;
			Text.WordWrap = false;
			view.Label(Label.TEXPATH + this.SELECTED.thingDef.graphicData.texPath, 1000f, 0f, -1f, null);
			view.Label(Label.WORNPATH + this.SELECTED.thingDef.apparel.wornGraphicPath, 1000f, 0f, -1f, null);
			view.Label(Label.DRAWSIZE + this.SELECTED.thingDef.graphicData.drawSize.ToString(), 1000f, 0f, -1f, null);
			Text.WordWrap = true;
			Text.Font = GameFont.Small;
		}

		// Token: 0x060003CA RID: 970 RVA: 0x0001FF37 File Offset: 0x0001E137
		private void AToggleStats()
		{
			CEditor.IsExtendedUI = !CEditor.IsExtendedUI;
		}

		// Token: 0x060003CB RID: 971 RVA: 0x0001FF48 File Offset: 0x0001E148
		private void ASelectApparelFromSelector(Thing toSelect = null)
		{
			bool flag = !this.lOfApparel.NullOrEmpty<ThingDef>();
			if (flag)
			{
				bool flag2 = toSelect == null;
				if (flag2)
				{
					this.SELECTED.Init(this.lOfApparel.FirstOrFallback(null), true, null, null);
				}
				else
				{
					this.SELECTED.Init(toSelect.def, false, toSelect, null);
				}
			}
		}

		// Token: 0x060003CC RID: 972 RVA: 0x0001FFA4 File Offset: 0x0001E1A4
		private void AApparelSelected(ThingDef thingDef)
		{
			bool flag = this.firstPreselection;
			if (flag)
			{
				this.firstPreselection = false;
			}
			else
			{
				bool flag2 = thingDef != null;
				if (flag2)
				{
					bool flag3 = !CEditor.InStartingScreen && thingDef.soundInteract != null;
					if (flag3)
					{
						thingDef.soundInteract.PlayOneShot(new TargetInfo(UI.MouseMapPosition().ToIntVec3(), Find.CurrentMap, false));
					}
					this.SELECTED.Init(thingDef, true, null, null);
				}
			}
		}

		// Token: 0x060003CD RID: 973 RVA: 0x00020020 File Offset: 0x0001E220
		private void AResetAll()
		{
			foreach (PresetApparel presetApparel in CEditor.API.Get<Dictionary<string, PresetApparel>>(EType.ApparelPreset).Values)
			{
				presetApparel.UpdateArmor();
			}
			CEditor.API.SetCustom(OptionS.CUSTOMARMOR, "", "");
			this.SELECTED.Init(this.lOfApparel.FirstOrFallback(null), true, null, null);
			MessageTool.Show("reset all apparel done", null);
		}

		// Token: 0x060003CE RID: 974 RVA: 0x000200C0 File Offset: 0x0001E2C0
		private void AResetOne()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				PresetApparel.ResetToDefault(this.SELECTED.thingDef.defName);
				this.SELECTED.Init(this.lOfApparel.FirstOrFallback(null), true, null, null);
				MessageTool.Show("reset apparel " + this.SELECTED.thingDef.defName + " done", null);
			}
		}

		// Token: 0x060003CF RID: 975 RVA: 0x0002013C File Offset: 0x0001E33C
		private void ASave()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				PresetApparel presetApparel = new PresetApparel(this.SELECTED.thingDef);
				presetApparel.UpdateArmor();
				presetApparel.SaveCustom();
				MessageTool.Show(presetApparel.label + " " + Label.SETTINGSSAVED, null);
			}
		}

		// Token: 0x060003D0 RID: 976 RVA: 0x0002019C File Offset: 0x0001E39C
		private void CalcHSCROLL()
		{
			int num = this.SELECTED.thingDef.stuffCategories.CountAllowNull<StuffCategoryDef>();
			int num2 = this.SELECTED.thingDef.statBases.CountAllowNull<StatModifier>();
			int num3 = this.SELECTED.thingDef.equippedStatOffsets.CountAllowNull<StatModifier>();
			int num4 = this.SELECTED.thingDef.apparel.bodyPartGroups.CountAllowNull<BodyPartGroupDef>();
			int num5 = this.SELECTED.thingDef.apparel.layers.CountAllowNull<ApparelLayerDef>();
			int num6 = this.SELECTED.thingDef.apparel.tags.CountAllowNull<string>();
			int num7 = this.SELECTED.thingDef.apparel.defaultOutfitTags.CountAllowNull<string>();
			int num8 = num + num2 + num3 + num4 + num5 + num6 + num7;
			this.HSCROLLPARAM = (float)(900 + num8 * 28);
		}

		// Token: 0x060003D1 RID: 977 RVA: 0x00020280 File Offset: 0x0001E480
		private void AAddStat(StatDef stat)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddStat(stat, 0f);
				ApparelTool.FixForBelts(this.SELECTED.thingDef, stat);
			}
		}

		// Token: 0x060003D2 RID: 978 RVA: 0x000202CC File Offset: 0x0001E4CC
		private void ADialogAddStat()
		{
      List<StatDef> hashSet = DefTool.ListBy<StatDef>(DefTool.CONDITION_STATDEFS_APPAREL(ThingTool.GetAllStatCategoriesOnEquip()));
			for (int i = 0; i < hashSet.Count; i++)
			{
				bool flag = this.SELECTED.thingDef.HasStat(hashSet.At(i));
				if (flag)
				{
					hashSet.Remove(hashSet.At(i));
					i--;
				}
			}
			SZWidgets.FloatMenuOnRect<StatDef>(hashSet, this.FGetStatLabel, new Action<StatDef>(this.AAddStat));
		}

		// Token: 0x060003D3 RID: 979 RVA: 0x0002034C File Offset: 0x0001E54C
		private void AAllowRemoveStat()
		{
			bool flag = this.iTick_AllowRemoveStat > 0;
			if (flag)
			{
				this.iTick_AllowRemoveStat = 0;
			}
			else
			{
				this.iTick_AllowRemoveStat = 1000;
			}
		}

		// Token: 0x060003D4 RID: 980 RVA: 0x0002037C File Offset: 0x0001E57C
		private void ACopyStat()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.statBases.NullOrEmpty<StatModifier>();
			if (flag)
			{
				this.lCopyStat = new List<StatModifier>();
				foreach (StatModifier item in this.SELECTED.thingDef.statBases)
				{
					this.lCopyStat.Add(item);
				}
			}
		}

		// Token: 0x060003D5 RID: 981 RVA: 0x0002041C File Offset: 0x0001E61C
		private void APasteStat()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteStats(this.lCopyStat);
			}
		}

		// Token: 0x060003D6 RID: 982 RVA: 0x00020454 File Offset: 0x0001E654
		private void ARemoveStat(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveStat(DefTool.StatDef(defName));
			}
		}

		// Token: 0x060003D7 RID: 983 RVA: 0x0002048C File Offset: 0x0001E68C
		private void AAddEquipStat(StatDef stat)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddEquipStat(stat, 0f);
				ApparelTool.FixForBelts(this.SELECTED.thingDef, stat);
			}
		}

		// Token: 0x060003D8 RID: 984 RVA: 0x000204D8 File Offset: 0x0001E6D8
		private void ADialogAddEquipStat()
		{
			List<StatDef> list = CEditor.API.ListOfStatDef(null, false, true);
			for (int i = 0; i < list.Count; i++)
			{
				bool flag = this.SELECTED.thingDef.HasStat(list[i]);
				if (flag)
				{
					list.Remove(list[i]);
					i--;
				}
			}
			SZWidgets.FloatMenuOnRect<StatDef>(list, this.FGetStatLabel, new Action<StatDef>(this.AAddEquipStat));
		}

		// Token: 0x060003D9 RID: 985 RVA: 0x00020554 File Offset: 0x0001E754
		private void ACopyEquipStat()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.equippedStatOffsets.NullOrEmpty<StatModifier>();
			if (flag)
			{
				this.lCopyEquipStat = new List<StatModifier>();
				foreach (StatModifier item in this.SELECTED.thingDef.equippedStatOffsets)
				{
					this.lCopyEquipStat.Add(item);
				}
			}
		}

		// Token: 0x060003DA RID: 986 RVA: 0x000205F4 File Offset: 0x0001E7F4
		private void APasteEquipStat()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteEquipStats(this.lCopyEquipStat);
			}
		}

		// Token: 0x060003DB RID: 987 RVA: 0x0002062C File Offset: 0x0001E82C
		private void ARemoveEquipStat(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveEquipStat(DefTool.StatDef(defName));
			}
		}

		// Token: 0x060003DC RID: 988 RVA: 0x00020664 File Offset: 0x0001E864
		private void AAddCosts(ThingDef thingDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddCost(thingDef, 0);
			}
		}

		// Token: 0x060003DD RID: 989 RVA: 0x00020698 File Offset: 0x0001E898
		private void ACopyCosts()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.costList.NullOrEmpty<ThingDefCountClass>();
			if (flag)
			{
				this.lCopyCost = new List<ThingDefCountClass>();
				foreach (ThingDefCountClass item in this.SELECTED.thingDef.costList)
				{
					this.lCopyCost.Add(item);
				}
			}
		}

		// Token: 0x060003DE RID: 990 RVA: 0x00020738 File Offset: 0x0001E938
		private void APasteCosts()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteCost(this.lCopyCost);
			}
		}

		// Token: 0x060003DF RID: 991 RVA: 0x00020770 File Offset: 0x0001E970
		private void ARemoveCosts(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveCost(DefTool.ThingDef(defName));
			}
		}

		// Token: 0x060003E0 RID: 992 RVA: 0x000207A8 File Offset: 0x0001E9A8
		private void AAddStuffCategory(StuffCategoryDef stuffCategoryDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddStuff(stuffCategoryDef);
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x060003E1 RID: 993 RVA: 0x000207E8 File Offset: 0x0001E9E8
		private void ACopyStuff()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.stuffCategories.NullOrEmpty<StuffCategoryDef>();
			if (flag)
			{
				this.lCopyStuff = new List<StuffCategoryDef>();
				foreach (StuffCategoryDef item in this.SELECTED.thingDef.stuffCategories)
				{
					this.lCopyStuff.Add(item);
				}
			}
		}

		// Token: 0x060003E2 RID: 994 RVA: 0x00020888 File Offset: 0x0001EA88
		private void APasteStuff()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteStuff(this.lCopyStuff);
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x060003E3 RID: 995 RVA: 0x000208D0 File Offset: 0x0001EAD0
		private void ARemoveStuffCategory(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveStuff(DefTool.StuffCategoryDef(defName));
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x060003E4 RID: 996 RVA: 0x00020918 File Offset: 0x0001EB18
		private void AAddBodyPartGroup(BodyPartGroupDef bodyPartGroupDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddBodyPart(bodyPartGroupDef);
			}
		}

		// Token: 0x060003E5 RID: 997 RVA: 0x0002094C File Offset: 0x0001EB4C
		private void ACopyBodyPartGroup()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.apparel.bodyPartGroups.NullOrEmpty<BodyPartGroupDef>();
			if (flag)
			{
				this.lCopyBodyPartGroup = new List<BodyPartGroupDef>();
				foreach (BodyPartGroupDef item in this.SELECTED.thingDef.apparel.bodyPartGroups)
				{
					this.lCopyBodyPartGroup.Add(item);
				}
			}
		}

		// Token: 0x060003E6 RID: 998 RVA: 0x000209F8 File Offset: 0x0001EBF8
		private void APasteBodyPartGroup()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteBodyPartGroup(this.lCopyBodyPartGroup);
			}
		}

		// Token: 0x060003E7 RID: 999 RVA: 0x00020A30 File Offset: 0x0001EC30
		private void ARemoveBodyPartGroup(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveBodyPartGroup(DefTool.BodyPartGroupDef(defName));
			}
		}

		// Token: 0x060003E8 RID: 1000 RVA: 0x00020A68 File Offset: 0x0001EC68
		private void AAddApparelLayer(ApparelLayerDef apparelLayerDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddApparelLayer(apparelLayerDef);
			}
		}

		// Token: 0x060003E9 RID: 1001 RVA: 0x00020A9C File Offset: 0x0001EC9C
		private void ACopyApparelLayer()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.apparel.layers.NullOrEmpty<ApparelLayerDef>();
			if (flag)
			{
				this.lCopyApparelLayer = new List<ApparelLayerDef>();
				foreach (ApparelLayerDef item in this.SELECTED.thingDef.apparel.layers)
				{
					this.lCopyApparelLayer.Add(item);
				}
			}
		}

		// Token: 0x060003EA RID: 1002 RVA: 0x00020B48 File Offset: 0x0001ED48
		private void APasteApparelLayer()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteApparelLayer(this.lCopyApparelLayer);
			}
		}

		// Token: 0x060003EB RID: 1003 RVA: 0x00020B80 File Offset: 0x0001ED80
		private void ARemoveApparelLayer(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				bool flag2 = this.SELECTED.thingDef.apparel.layers.Count == 1;
				if (flag2)
				{
					MessageTool.Show(Label.CANTREMOVELAYER, MessageTypeDefOf.RejectInput);
				}
				else
				{
					this.SELECTED.thingDef.RemoveApparelLayer(DefTool.ApparelLayerDef(defName));
				}
			}
		}

		// Token: 0x060003EC RID: 1004 RVA: 0x00020BEC File Offset: 0x0001EDEC
		private void AAddApparelTag(string tag)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddApparelTag(tag);
			}
		}

		// Token: 0x060003ED RID: 1005 RVA: 0x00020C20 File Offset: 0x0001EE20
		private void ACopyApparelTag()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.apparel.tags.NullOrEmpty<string>();
			if (flag)
			{
				this.lCopyApparelTag = new List<string>();
				foreach (string item in this.SELECTED.thingDef.apparel.tags)
				{
					this.lCopyApparelTag.Add(item);
				}
			}
		}

		// Token: 0x060003EE RID: 1006 RVA: 0x00020CCC File Offset: 0x0001EECC
		private void APasteApparelTag()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteApparelTag(this.lCopyApparelTag);
			}
		}

		// Token: 0x060003EF RID: 1007 RVA: 0x00020D04 File Offset: 0x0001EF04
		private void ARemoveApparelTag(string tagName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveApparelTag(tagName);
			}
		}

		// Token: 0x060003F0 RID: 1008 RVA: 0x00020D38 File Offset: 0x0001EF38
		private void AAddOutfitTag(string tag)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddOutfitTag(tag);
			}
		}

		// Token: 0x060003F1 RID: 1009 RVA: 0x00020D6C File Offset: 0x0001EF6C
		private void ACopyOutfitTag()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.apparel.defaultOutfitTags.NullOrEmpty<string>();
			if (flag)
			{
				this.lCopyOutfitTag = new List<string>();
				foreach (string item in this.SELECTED.thingDef.apparel.defaultOutfitTags)
				{
					this.lCopyOutfitTag.Add(item);
				}
			}
		}

		// Token: 0x060003F2 RID: 1010 RVA: 0x00020E18 File Offset: 0x0001F018
		private void APasteOutfitTag()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteOutfitTag(this.lCopyOutfitTag);
			}
		}

		// Token: 0x060003F3 RID: 1011 RVA: 0x00020E50 File Offset: 0x0001F050
		private void ARemoveOutfitTag(string tagName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveOutfitTag(tagName);
			}
		}

		// Token: 0x060003F4 RID: 1012 RVA: 0x00020E84 File Offset: 0x0001F084
		private void ASetTechLevel(TechLevel techLevel)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.techLevel = techLevel;
				this.SELECTED.thingDef.ResolveReferences();
			}
		}

		// Token: 0x060003F5 RID: 1013 RVA: 0x00020EC8 File Offset: 0x0001F0C8
		private void ASetTradeability(Tradeability tradeability)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.tradeability = tradeability;
				this.SELECTED.thingDef.ResolveReferences();
			}
		}

		// Token: 0x060003F6 RID: 1014 RVA: 0x00020F0C File Offset: 0x0001F10C
		private void ABuyApparel()
		{
			ThingTool.BeginBuyItem(this.SELECTED);
			CEditor.API.CloseEditor();
			this.Close(true);
		}

		// Token: 0x040001D7 RID: 471
		private Vector2 scrollPos;

		// Token: 0x040001D8 RID: 472
		private Vector2 scrollPosParam;

		// Token: 0x040001D9 RID: 473
		private Selected SELECTED;

		// Token: 0x040001DA RID: 474
		private Color previousColor;

		// Token: 0x040001DB RID: 475
		private List<ThingDef> lOfApparel;

		// Token: 0x040001DC RID: 476
		private List<int> lOfQuality;

		// Token: 0x040001DD RID: 477
		private List<StatModifier> lCopyStat;

		// Token: 0x040001DE RID: 478
		private List<StatModifier> lCopyEquipStat;

		// Token: 0x040001DF RID: 479
		private List<StuffCategoryDef> lCopyStuff;

		// Token: 0x040001E0 RID: 480
		private List<ThingDefCountClass> lCopyCost;

		// Token: 0x040001E1 RID: 481
		private List<BodyPartGroupDef> lCopyBodyPartGroup;

		// Token: 0x040001E2 RID: 482
		private List<ApparelLayerDef> lCopyApparelLayer;

		// Token: 0x040001E3 RID: 483
		private List<ApparelLayerDef> lOfAllLayers;

		// Token: 0x040001E4 RID: 484
		private List<string> lCopyApparelTag;

		// Token: 0x040001E5 RID: 485
		private List<string> lCopyOutfitTag;

		// Token: 0x040001E6 RID: 486
		private int iTick_AllowRemoveStat;

		// Token: 0x040001E7 RID: 487
		private int iChangeTick0;

		// Token: 0x040001E8 RID: 488
		private int idxList;

		// Token: 0x040001E9 RID: 489
		private bool allowRemove;

		// Token: 0x040001EA RID: 490
		private bool doOnce;

		// Token: 0x040001EB RID: 491
		private bool firstPreselection;

		// Token: 0x040001EC RID: 492
		private string toggleText;

		// Token: 0x040001ED RID: 493
		private string tempName;

		// Token: 0x040001EE RID: 494
		private const float WTITLE = 300f;

		// Token: 0x040001EF RID: 495
		private float MAXW = 850f;

		// Token: 0x040001F0 RID: 496
		private float W_APPA = 270f;

		// Token: 0x040001F1 RID: 497
		private float W_PARAMS = 530f;

		// Token: 0x040001F2 RID: 498
		private float HSCROLLPARAM;

		// Token: 0x040001F3 RID: 499
		private const int TEXTURESIZE = 192;

		// Token: 0x040001F4 RID: 500
		private SearchTool search;

		// Token: 0x040001F5 RID: 501
		private Func<ApparelLayerDef, string> FGetApparelLayerLabel;

		// Token: 0x040001F6 RID: 502
		private Func<BodyPartGroupDef, string> FGetBodyPartGroupLabel;

		// Token: 0x040001F7 RID: 503
		private Func<string, string> FGetTagLabel;

		// Token: 0x040001F8 RID: 504
		private Func<StuffCategoryDef, string> FGetStuffCategoryLabel;

		// Token: 0x040001F9 RID: 505
		private Func<ThingDef, string> FApparelLabel;

		// Token: 0x040001FA RID: 506
		private Func<ThingDef, string> FApparelTooltip;

		// Token: 0x040001FB RID: 507
		private Func<ThingDef, ThingDef, bool> FApparelComparator;

		// Token: 0x040001FC RID: 508
		private Func<TechLevel, string> FGetTechLevelLabel;

		// Token: 0x040001FD RID: 509
		private Func<Tradeability, string> FGetTradeabilityLabel;

		// Token: 0x040001FE RID: 510
		private Func<StatDef, string> FGetStatLabel;

		// Token: 0x040001FF RID: 511
		private Func<ThingStyleDef, string> FGetThingStyleDefLabel;
	}
}
