﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	public class DialogCapsuleUI : Window
	{
    private Vector2 scrollPos;
    private Vector2 scrollPos2;
    private Vector2 scrollPos3;
    //private List<ScenPart> lParts;
    //private List<ScenPart> lPartsTaken;
    //private List<ScenPart> lPartsOnMap;
    //private List<ScenPart> lPartsAnimal;
    private Dictionary<int, string> dicSlots;
    //private ScenPart selectedPart;
    private bool bShowRemove;
    private bool addToTaken;
    private bool doOnce;
    private float WTITLE = 300f;
    private float HITEM = 32f;
    private int astronautPic;
    private string SLOTPATH;

    public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(1120f, 780f);
			}
		}

    public override void PostClose()
    {
      //ScenarioTool.SetScenarioParts();
      ScenarioTool.ScenarioParts = ScenarioTool.CustomScenarioParts;
      base.PostClose();
    }

    // Token: 0x060002BE RID: 702 RVA: 0x00016210 File Offset: 0x00014410
    public DialogCapsuleUI()
		{
			this.scrollPos = default(Vector2);
			this.scrollPos2 = default(Vector2);
			this.scrollPos3 = default(Vector2);
			//this.selectedPart = null;
			this.bShowRemove = false;
			this.doOnce = true;
			this.astronautPic = 1;
      ScenarioTool.CustomScenarioParts = ScenarioTool.ScenarioParts;
      this.InitSlots();
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x060002BF RID: 703 RVA: 0x000162A9 File Offset: 0x000144A9
		private string FULLSLOTPATH(int index)
		{
			return this.SLOTPATH + index.ToString() + ".txt";
		}

		// Token: 0x060002C0 RID: 704 RVA: 0x000162C4 File Offset: 0x000144C4
		private void InitSlots()
		{
			string text = GenFilePaths.ConfigFolderPath.Replace("Config", "CharacterEditor");
			FileIO.CheckOrCreateDir(text);
			this.SLOTPATH = text + Path.DirectorySeparatorChar.ToString() + "capsulesetup";
			this.dicSlots = new Dictionary<int, string>();
			int numCapsuleSlots = CEditor.API.NumCapsuleSlots;
			for (int i = 0; i < numCapsuleSlots; i++)
			{
				this.dicSlots.Add(i, "");
			}
			this.UpdateSlots();
		}

		// Token: 0x060002C1 RID: 705 RVA: 0x00016350 File Offset: 0x00014550
		private void UpdateSlots()
		{
			int numCapsuleSlots = CEditor.API.NumCapsuleSlots;
			for (int i = 0; i < numCapsuleSlots; i++)
			{
				bool flag = FileIO.Exists(this.FULLSLOTPATH(i));
				if (flag)
				{
					string text = FileIO.ReadFile(this.FULLSLOTPATH(i)).AsString(Encoding.UTF8);
					bool flag2 = !text.NullOrEmpty();
					if (flag2)
					{
						string[] array = text.SplitNo("\n");
						bool flag3 = array.Length > 1;
						if (flag3)
						{
							this.dicSlots[i] = array[0];
						}
					}
				}
			}
		}

		// Token: 0x060002C2 RID: 706 RVA: 0x000163E4 File Offset: 0x000145E4
		public override void DoWindowContents(Rect inRect)
		{
			bool flag = this.doOnce;
			if (flag)
			{
				this.doOnce = false;
				this.windowRect.position = new Vector2((float)(CEditor.API.EditorPosX + 105), this.windowRect.y);
			}
			Text.Font = GameFont.Medium;
			Widgets.Label(new Rect(0f, 0f, this.WTITLE, 30f), Label.RETTUNGSKAPSEL);
			GUI.DrawTexture(new Rect(0f, 0f, inRect.width, inRect.height), ContentFinder<Texture2D>.Get("bcapsule_back", true), ScaleMode.ScaleToFit);
			this.DrawUIObjects();
      //this.DrawContainer(13, 428, 392, 256);
      //this.DrawWreckage(673, 428, 392, 256);
      //this.DrawAnimals(848, 148, 197, 190);
      this.DrawScenPartList(13, 428, 392, 256, ref scrollPos, ScenarioTool.takenPartEntryList, new Action(this.AAddPartsTaken), new Action<Pair<ScenPart, EntryInt>>(this.ARemovePartTaken));
      this.DrawScenPartList(673, 148, 392, 190, ref scrollPos3, ScenarioTool.animalPartEntryList, new Action(this.AAddPartsAnimal), new Action<Pair<ScenPart, EntryInt>>(this.ARemovePartAnimal));
      this.DrawScenPartList(673, 428, 392, 256, ref scrollPos2, ScenarioTool.onMapPartEntryList, new Action(this.AAddPartsMap), new Action<Pair<ScenPart, EntryInt>>(this.ARemovePartMap));
      

      this.DrawButtons(0, (int)this.InitialSize.y - 70, (int)this.InitialSize.x - 32, 30);
			Text.Font = GameFont.Small;
			// Changed this button from "OK" to "Back"
			SZWidgets.ButtonText(new Rect(this.InitialSize.x - 136f, this.InitialSize.y - 72f, 100f, 30f), "Back".Translate(), new Action(this.ACommit), "");
		}

		// Token: 0x060002C3 RID: 707 RVA: 0x00016568 File Offset: 0x00014768
		private void DrawUIObjects()
		{
			bool inStartingScreen = CEditor.InStartingScreen;
			if (inStartingScreen)
			{
				Widgets.DrawBoxSolid(new Rect(480f, 50f, 120f, 30f), new Color(0.8f, 0.2f, 0.2f, 0.5f));
				Text.Anchor = TextAnchor.MiddleCenter;
				Widgets.Label(new Rect(480f, 50f, 120f, 30f), Label.START);
				SZWidgets.ButtonInvisible(new Rect(480f, 50f, 120f, 30f), new Action(this.AStart), "");
				Text.Anchor = TextAnchor.UpperLeft;
			}
			Widgets.DrawBoxSolid(new Rect(15f, 50f, 160f, 160f), new Color(0.2f, 0.2f, 0.2f, 0.5f));
			GUI.DrawTexture(new Rect(15f, 50f, 160f, 160f), ContentFinder<Texture2D>.Get("bastronaut" + this.astronautPic.ToString(), true));
			SZWidgets.ButtonInvisible(new Rect(15f, 50f, 160f, 160f), new Action(this.AOnAstronaut), "");
			Widgets.Label(new Rect(150f, 176f, 30f, 30f), ScenarioTool.CurrentTakenPawnCount.ToString());

      Widgets.DrawBoxSolid(new Rect(10f, 390f, 400f, 320f), new Color(0.2f, 0.2f, 0.2f, 0.5f));
      Widgets.Label(new Rect(15f, 395f, 395f, 30f), Label.KAPSELCONTAINER);

      //Widgets.DrawBoxSolid(new Rect(845f, 110f, 205f, 250f), new Color(0.2f, 0.2f, 0.2f, 0.5f));
      //Widgets.Label(new Rect(850f, 115f, 200f, 30f), Label.TIERKAPSEL);
      Widgets.DrawBoxSolid(new Rect(670f, 110f, 400f, 250f), new Color(0.2f, 0.2f, 0.2f, 0.5f));
      Widgets.Label(new Rect(675f, 115f, 395f, 30f), Label.TIERKAPSEL);

      Widgets.DrawBoxSolid(new Rect(670f, 390f, 400f, 320f), new Color(0.2f, 0.2f, 0.2f, 0.5f));
			Widgets.Label(new Rect(675f, 395f, 395f, 30f), Label.TEILEIMSCHIFFSWRACK);
		}

		// Token: 0x17000047 RID: 71
		// (get) Token: 0x060002C4 RID: 708 RVA: 0x00016800 File Offset: 0x00014A00
		private Rect rectLoad
		{
			get
			{
				return new Rect(0f, (float)((int)this.InitialSize.y - 70), 120f, 30f);
			}
		}

		// Token: 0x17000048 RID: 72
		// (get) Token: 0x060002C5 RID: 709 RVA: 0x00016826 File Offset: 0x00014A26
		private Rect rectSave
		{
			get
			{
				return new Rect(120f, (float)((int)this.InitialSize.y - 70), 120f, 30f);
			}
		}

		// Token: 0x060002C6 RID: 710 RVA: 0x0001684C File Offset: 0x00014A4C
		private void DrawButtons(int x, int y, int w, int h)
		{
			SZWidgets.FloatMenuOnButtonText<int>(this.rectLoad, "Load".Translate(), this.dicSlots.Keys.ToList<int>(), (int s) => Label.LOADSLOT + s.ToString() + "\n" + this.dicSlots[s], new Action<int>(this.AOnLoadSlot));
			SZWidgets.FloatMenuOnButtonText<int>(this.rectSave, "Save".Translate(), this.dicSlots.Keys.ToList<int>(), (int s) => Label.SAVESLOT + s.ToString() + "\n" + this.dicSlots[s], new Action<int>(this.AOnSaveSlot));
		}

		// Token: 0x060002C7 RID: 711 RVA: 0x000168E0 File Offset: 0x00014AE0
		private void AOnLoadSlot(int index)
		{
			ScenarioTool.LoadCapsuleSetup(this.FULLSLOTPATH(index));
      //ScenarioTool.UpdateLists();
		}

		// Token: 0x060002C8 RID: 712 RVA: 0x000168F7 File Offset: 0x00014AF7
		private void AOnSaveSlot(int index)
		{
			ScenarioTool.SaveCapsuleSetup(this.FULLSLOTPATH(index));
			this.UpdateSlots();
		}

    private void DrawScenPartList(int x, int y, int w, int h, ref Vector2 scrollPos, List<Pair<ScenPart, EntryInt>> entryPartList, Action addPart, Action<Pair<ScenPart, EntryInt>> removePart)
    {
      SZWidgets.ButtonImage(x + w - 25f, y - 30f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(addPart), "");
      SZWidgets.ButtonImage(x + w - 50f, y - 30f, 24f, 24f, "bminus", new Action(this.AAllowRemoveParts), "");
      float height = entryPartList.CountAllowNull<Pair<ScenPart, EntryInt>>() * this.HITEM;
      Rect outRect = new Rect(x, y, w, h);
      Rect rect = new Rect(0f, 0f, outRect.width - 16f, height);
      Widgets.BeginScrollView(outRect, ref scrollPos, rect, true);
      Rect rect2 = rect.ContractedBy(4f);
      rect2.y -= 4f;
      rect2.height = height;
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect2);
      listing_X.verticalSpacing = 30f;
      Text.Font = GameFont.Small;
      try
      {
        foreach (Pair<ScenPart, EntryInt> pair in entryPartList)
        {
          listing_X.ListviewPart(w, this.HITEM, pair, this.bShowRemove, removePart);
        }
      }
      catch
      {
      }
      listing_X.End();
      Widgets.EndScrollView();
    }

  //  private void DrawContainer(int x, int y, int w, int h)
		//{
		//	SZWidgets.ButtonImage((float)(x + w - 25), (float)(y - 30), 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.AAddParts), "");
		//	SZWidgets.ButtonImage((float)(x + w - 50), (float)(y - 30), 24f, 24f, "bminus", new Action(this.AAllowRemoveParts), "");
		//	float height = ScenarioTool.lPartsTaken.CountAllowNull<Pair<ScenPart, EntryInt>>() * this.HITEM;
		//	Rect outRect = new Rect((float)x, (float)y, (float)w, (float)h);
		//	Rect rect = new Rect(0f, 0f, outRect.width - 16f, height);
		//	Widgets.BeginScrollView(outRect, ref this.scrollPos, rect, true);
		//	Rect rect2 = rect.ContractedBy(4f);
		//	rect2.y -= 4f;
		//	rect2.height = height;
		//	Listing_X listing_X = new Listing_X();
		//	listing_X.Begin(rect2);
		//	listing_X.verticalSpacing = 30f;
		//	Text.Font = GameFont.Small;
		//	try
		//	{
		//		foreach (Pair<ScenPart, EntryInt> pair in ScenarioTool.lPartsTaken)
		//		{
  //        listing_X.ListviewPart((float)w, this.HITEM, pair, this.bShowRemove, new Action<Pair<ScenPart,EntryInt>>(this.ARemovePart));
		//		}
		//	}
		//	catch
		//	{
		//	}
		//	listing_X.End();
		//	Widgets.EndScrollView();
		//}

		//private void DrawWreckage(int x, int y, int w, int h)
		//{
		//	SZWidgets.ButtonImage((float)(x + w - 25), (float)(y - 30), 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.AAddPartsMap), "");
		//	SZWidgets.ButtonImage((float)(x + w - 50), (float)(y - 30), 24f, 24f, "bminus", new Action(this.AAllowRemoveParts), "");
		//	float height = ScenarioTool.lPartsOnMap.CountAllowNull<Pair<ScenPart, EntryInt>>() * this.HITEM;
		//	Rect outRect = new Rect((float)x, (float)y, (float)w, (float)h);
		//	Rect rect = new Rect(0f, 0f, outRect.width - 16f, height);
		//	Widgets.BeginScrollView(outRect, ref this.scrollPos2, rect, true);
		//	Rect rect2 = rect.ContractedBy(4f);
		//	rect2.y -= 4f;
		//	rect2.height = height;
		//	Listing_X listing_X = new Listing_X();
		//	listing_X.Begin(rect2);
		//	listing_X.verticalSpacing = 30f;
		//	Text.Font = GameFont.Small;
		//	try
		//	{
		//		foreach (Pair<ScenPart, EntryInt> entryPair in ScenarioTool.lPartsOnMap)
		//		{
		//			listing_X.ListviewPart((float)w, this.HITEM, entryPair, this.bShowRemove, new Action<Pair<ScenPart, EntryInt>>(this.ARemovePartMap));
		//		}
		//	}
		//	catch
		//	{
		//	}
		//	listing_X.End();
		//	Widgets.EndScrollView();
		//}

		//private void DrawAnimals(int x, int y, int w, int h)
		//{
		//	SZWidgets.ButtonImage((float)(x + w - 25), (float)(y - 30), 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.AAddPartsAnimal), "");
		//	SZWidgets.ButtonImage((float)(x + w - 50), (float)(y - 30), 24f, 24f, "bminus", new Action(this.AAllowRemoveParts), "");
		//	float height = ScenarioTool.lPartsAnimal.CountAllowNull<Pair<ScenPart, EntryInt>>() * this.HITEM;
		//	Rect outRect = new Rect((float)x, (float)y, (float)w, (float)h);
		//	Rect rect = new Rect(0f, 0f, outRect.width - 16f, height);
		//	Widgets.BeginScrollView(outRect, ref this.scrollPos3, rect, true);
		//	Rect rect2 = rect.ContractedBy(4f);
		//	rect2.y -= 4f;
		//	rect2.height = height;
		//	Listing_X listing_X = new Listing_X();
		//	listing_X.Begin(rect2);
		//	listing_X.verticalSpacing = 30f;
		//	Text.Font = GameFont.Small;
		//	try
		//	{
		//		foreach (Pair<ScenPart, EntryInt> entryPair in ScenarioTool.lPartsAnimal)
		//		{
		//			listing_X.ListviewPart((float)w, this.HITEM, entryPair, this.bShowRemove, new Action<Pair<ScenPart, EntryInt>>(this.ARemovePartAnimal));
		//		}
		//	}
		//	catch
		//	{
		//	}
		//	listing_X.End();
		//	Widgets.EndScrollView();
		//}

		private void AStart()
		{
			//ScenarioTool.SetScenarioParts();
			ScenarioTool.ScenarioParts = ScenarioTool.CustomScenarioParts;
			Window windowOf = WindowTool.GetWindowOf<Page_ConfigureStartingPawns>();
			if (windowOf != null)
			{
				((Page_ConfigureStartingPawns)windowOf).nextAct();
			}
			this.Close(false);
			CEditor.API.CloseEditor();
		}

		private void ACommit()
		{
      ScenarioTool.ScenarioParts = ScenarioTool.CustomScenarioParts;
      this.Close(false);
		}

		private void AOnAstronaut()
		{
			if (this.astronautPic == 1)
			{
				this.astronautPic = 2;
			}
			else
			{
				this.astronautPic = 1;
			}
		}

		// Token: 0x060002D0 RID: 720 RVA: 0x00016FB8 File Offset: 0x000151B8
		public void AddThingDef(ThingDef thingDef, ThingDef stuffDef, ThingStyleDef styleDef, int count)
		{
			if (this.addToTaken)
			{
				ScenarioTool.AddScenarioPartTaken(thingDef, stuffDef, styleDef, count);
			}
			else
			{
				ScenarioTool.AddScenarioPartMap(thingDef, stuffDef, styleDef, count);
			}
			//this.UpdateLists();
		}

		// Token: 0x060002D1 RID: 721 RVA: 0x00016FEF File Offset: 0x000151EF
		public void AddAnimal(PawnKindDef pkd, int count)
		{
			ScenarioTool.AddScenarioPartAnimal(pkd, count);
			//this.UpdateLists();
		}

		// Token: 0x060002D2 RID: 722 RVA: 0x00017001 File Offset: 0x00015201
		private void AAddPartsTaken()
		{
			this.addToTaken = true;
			WindowTool.Open(new DialogItemry(this, null));
		}

		// Token: 0x060002D3 RID: 723 RVA: 0x00017018 File Offset: 0x00015218
		private void AAddPartsMap()
		{
			this.addToTaken = false;
			WindowTool.Open(new DialogItemry(this, null));
		}

		// Token: 0x060002D4 RID: 724 RVA: 0x0001702F File Offset: 0x0001522F
		private void AAddPartsAnimal()
		{
			WindowTool.Open(new DialogAddAnimal(this));
		}

		// Token: 0x060002D5 RID: 725 RVA: 0x0001703E File Offset: 0x0001523E
		private void AAllowRemoveParts()
		{
			this.bShowRemove = !this.bShowRemove;
		}

		// Token: 0x060002D6 RID: 726 RVA: 0x00017050 File Offset: 0x00015250
		private void ARemovePartTaken(Pair<ScenPart, EntryInt> entryPair)
		{
      ScenarioTool.takenPartEntryList.Remove(entryPair);
      //ScenarioTool.lParts.Remove(entryPair.First);
		}

		// Token: 0x060002D7 RID: 727 RVA: 0x0001706D File Offset: 0x0001526D
		private void ARemovePartMap(Pair<ScenPart, EntryInt> entryPair)
    {
      ScenarioTool.onMapPartEntryList.Remove(entryPair);
      //ScenarioTool.lParts.Remove(entryPair.First);
		}

		// Token: 0x060002D8 RID: 728 RVA: 0x0001708A File Offset: 0x0001528A
		private void ARemovePartAnimal(Pair<ScenPart, EntryInt> entryPair)
    {
      ScenarioTool.animalPartEntryList.Remove(entryPair);
      //ScenarioTool.lParts.Remove(entryPair.First);
		}

	}
}
