﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  // Token: 0x02000053 RID: 83
  public class DialogChangeBackstory : Window
  {
    // Token: 0x17000062 RID: 98
    // (get) Token: 0x06000513 RID: 1299 RVA: 0x0003495C File Offset: 0x00032B5C
    public override Vector2 InitialSize
    {
      get
      {
        return new Vector2(300f, 788f);
      }
    }

    // Token: 0x06000514 RID: 1300 RVA: 0x0003497D File Offset: 0x00032B7D
    public void RememberOldBackstory()
    {
      this.oBackAdult = CEditor.API.Pawn.story.GetBackstory(BackstorySlot.Adulthood);
      this.oBackChild = CEditor.API.Pawn.story.GetBackstory(BackstorySlot.Childhood);
    }

    // Token: 0x06000515 RID: 1301 RVA: 0x000349B8 File Offset: 0x00032BB8
    public void RecalcSkills()
    {
      BackstoryDef backstory = CEditor.API.Pawn.story.GetBackstory(BackstorySlot.Adulthood);
      BackstoryDef backstory2 = CEditor.API.Pawn.story.GetBackstory(BackstorySlot.Childhood);
      foreach (SkillRecord skillRecord in CEditor.API.Pawn.skills.skills)
      {
        skillRecord.Level -= ((this.oBackAdult == null) ? 0 : this.oBackAdult.skillGains.TryGetValue(skillRecord.def, 0));
        skillRecord.Level -= ((this.oBackChild == null) ? 0 : this.oBackChild.skillGains.TryGetValue(skillRecord.def, 0));
        skillRecord.Level += ((backstory == null) ? 0 : backstory.skillGains.TryGetValue(skillRecord.def, 0));
        skillRecord.Level += ((backstory2 == null) ? 0 : backstory2.skillGains.TryGetValue(skillRecord.def, 0));
      }
    }

    // Token: 0x06000516 RID: 1302 RVA: 0x00034AF8 File Offset: 0x00032CF8
    public DialogChangeBackstory(bool _isChildhood)
    {
      this.RememberOldBackstory();
      this.scrollPos = default(Vector2);
      this.isChildhood = _isChildhood;
      this.selectedBackstory = (this.isChildhood ? CEditor.API.Pawn.story.Childhood : CEditor.API.Pawn.story.Adulthood);
      this.selectedCategory = Label.ALL;
      this.selectedFilter = new KeyValuePair<string, SkillDef>(Label.ALL, null);
      this.selectedFilter2 = Label.ALL;
      this.isFiltered = false;
      this.isFilteredOld = false;
      this.doOnce = true;
      this.gender = CEditor.API.Pawn.gender;
      this.UpdateDictionary();
      this.idxList = 4;
      SearchTool.Update(this.idxList);
      List<List<string>> list = (from td in this.dicBackstory.Keys
                                 select td.spawnCategories).ToList<List<string>>();
      this.lOfCategories = new List<string>();
      this.lOfCategories.Add(Label.ALL);
      foreach (List<string> list2 in list)
      {
        foreach (string item in list2)
        {
          this.lOfCategories.Add(item);
        }
      }
      this.dicOfFilters = new Dictionary<string, SkillDef>();
      this.dicOfFilters.Add(Label.ALL, null);
      foreach (SkillRecord skillRecord in CEditor.API.Pawn.skills.skills)
      {
        this.dicOfFilters.Add(skillRecord.def.LabelCap.ToString() + " +", skillRecord.def);
        this.dicOfFilters.Add(skillRecord.def.LabelCap.ToString() + " ++", skillRecord.def);
        this.dicOfFilters.Add(skillRecord.def.LabelCap.ToString() + " +++", skillRecord.def);
      }
      this.lOfFilter2 = new List<string>();
      this.lOfFilter2.Add(Label.ALL);
      this.lOfFilter2.Add(Label.POSITIVE_ONLY);
      this.lOfFilter2.Add(Label.SUMME + " > 3");
      this.lOfFilter2.Add(Label.SUMME + " > 4");
      this.lOfFilter2.Add(Label.SUMME + " > 5");
      this.lOfFilter2.Add(Label.SUMME + " > 6");
      this.lOfFilter2.Add(Label.SUMME + " > 7");
      this.lOfFilter2.Add(Label.SUMME + " > 8");
      this.lOfFilter2.Add(Label.SUMME + " > 9");
      this.lOfFilter2.Add(Label.SUMME + " > 10");
      this.lOfFilter2.Add(Label.SUMME + " > 11");
      this.lOfFilter2.Add(Label.SUMME + " > 12");
      this.FBackstoryLabel = new Func<BackstoryDef, string>(this.GetBackstoryLabel);
      this.FBackstoryTooltip = new Func<BackstoryDef, string>(this.GetBackstoryTooltip);
      this.FBackstoryComparator = new Func<BackstoryDef, BackstoryDef, bool>(this.IsSameBackstory);
      this.doCloseX = true;
      this.absorbInputAroundWindow = true;
      this.draggable = true;
      this.layer = CEditor.Layer;
    }

    // Token: 0x06000517 RID: 1303 RVA: 0x00034F4C File Offset: 0x0003314C
    private string GetBackstoryLabel(BackstoryDef backstory)
    {
      return backstory.TitleCapFor(this.gender);
    }

    // Token: 0x06000518 RID: 1304 RVA: 0x00034F6C File Offset: 0x0003316C
    private string GetBackstoryTooltip(BackstoryDef backstory)
    {
      return this.dicBackstory[backstory];
    }

    // Token: 0x06000519 RID: 1305 RVA: 0x00034F8C File Offset: 0x0003318C
    private bool IsSameBackstory(BackstoryDef a, BackstoryDef b)
    {
      return a == b;
    }

    // Token: 0x0600051A RID: 1306 RVA: 0x00034FA4 File Offset: 0x000331A4
    private void UpdateDictionary()
    {
      this.dicBackstory = new Dictionary<BackstoryDef, string>();
      List<BackstoryDef> list = (from td in DefDatabase<BackstoryDef>.AllDefs
                                 where td != null && td.slot == (this.isChildhood ? BackstorySlot.Childhood : BackstorySlot.Adulthood) && !string.IsNullOrEmpty(td.title) && (!this.isFiltered || td.DisabledWorkTypes.Count<WorkTypeDef>() == 0)
                                 orderby td.title
                                 select td).ToList<BackstoryDef>();
      bool flag = this.selectedCategory == null || this.selectedCategory == Label.ALL;
      bool flag2 = this.selectedFilter.Key == null || this.selectedFilter.Key == Label.ALL;
      bool flag3 = this.selectedFilter2 == null || this.selectedFilter2 == Label.ALL;
      foreach (BackstoryDef backstory in list)
      {
        bool flag4 = false;
        bool flag5 = flag || backstory.spawnCategories.Contains(this.selectedCategory);
        if (flag5)
        {
          bool flag6 = flag2;
          if (flag6)
          {
            flag4 = true;
          }
          else
          {
            bool flag7 = this.selectedFilter.Value == null || backstory.skillGains.ContainsKey(this.selectedFilter.Value);
            if (flag7)
            {
              flag4 = false;
              int num = (this.selectedFilter.Value == null) ? 0 : backstory.skillGains[this.selectedFilter.Value];
              bool flag8 = this.selectedFilter.Key.EndsWith("+++");
              if (flag8)
              {
                bool flag9 = num > 3;
                if (flag9)
                {
                  flag4 = true;
                }
              }
              else
              {
                bool flag10 = this.selectedFilter.Key.EndsWith("++");
                if (flag10)
                {
                  bool flag11 = num > 2;
                  if (flag11)
                  {
                    flag4 = true;
                  }
                }
                else
                {
                  bool flag12 = this.selectedFilter.Key.EndsWith("+");
                  if (flag12)
                  {
                    bool flag13 = num > 0;
                    if (flag13)
                    {
                      flag4 = true;
                    }
                  }
                }
              }
            }
          }
          bool flag14 = flag4;
          if (flag14)
          {
            bool flag15 = flag3;
            if (flag15)
            {
              flag4 = true;
            }
            else
            {
              bool flag16 = this.selectedFilter2.Contains(" > ");
              if (flag16)
              {
                flag4 = false;
                int num2 = 0;
                int num3 = this.selectedFilter2.SubstringFrom(">", true).Trim().AsInt32();
                foreach (SkillDef key in backstory.skillGains.Keys)
                {
                  num2 += backstory.skillGains[key];
                }
                bool flag17 = num2 > num3;
                if (flag17)
                {
                  flag4 = true;
                }
              }
              else
              {
                bool flag18 = this.selectedFilter2 == Label.POSITIVE_ONLY;
                if (flag18)
                {
                  flag4 = true;
                  foreach (SkillDef key2 in backstory.skillGains.Keys)
                  {
                    flag4 &= (backstory.skillGains[key2] > 0);
                  }
                }
              }
            }
          }
          bool flag19 = flag4;
          if (flag19)
          {
            this.dicBackstory.Add(backstory, backstory.FullDescriptionFor(CEditor.API.Pawn).Resolve());
          }
        }
      }
      this.lOfBackstories = this.dicBackstory.Keys.ToList<BackstoryDef>();
    }

    // Token: 0x0600051B RID: 1307 RVA: 0x00035368 File Offset: 0x00033568
    public override void DoWindowContents(Rect inRect)
    {
      bool flag = this.doOnce;
      if (flag)
      {
        this.doOnce = false;
        this.windowRect.position = new Vector2(this.windowRect.position.x - 380f, this.windowRect.position.y);
      }
      float num = this.InitialSize.x - 40f;
      float num2 = this.InitialSize.y - 150f;
      Text.Font = GameFont.Medium;
      Widgets.Label(new Rect(0f, 0f, num, 30f), this.isChildhood ? "Childhood".Translate() : "Adulthood".Translate());
      SZWidgets.ButtonImage(num - 25f, 0f, 25f, 25f, "brandom", new Action(this.ARandomBackstory), "");
      Text.Font = GameFont.Small;
      Widgets.CheckboxLabeled(new Rect(0f, 30f, num, 30f), Label.NO_BLOCKING_SKILLS, ref this.isFiltered, false, null, null, false);
      bool flag2 = this.isFiltered != this.isFilteredOld;
      if (flag2)
      {
        this.isFilteredOld = this.isFiltered;
        this.UpdateDictionary();
      }
      SZWidgets.FloatMenuOnButtonText<string>(new Rect(0f, 60f, num, 30f), this.selectedCategory ?? Label.ALL, this.lOfCategories, (string s) => s, new Action<string>(this.ACategoryChanged), "");
      SZWidgets.FloatMenuOnButtonText<string>(new Rect(0f, 90f, num, 30f), this.selectedFilter.Key, this.dicOfFilters.Keys.ToList<string>(), (string s) => s, new Action<string>(this.AFilterChanged));
      SZWidgets.FloatMenuOnButtonText<string>(new Rect(0f, 120f, num, 30f), this.selectedFilter2, this.lOfFilter2, (string s) => s, new Action<string>(this.AFilter2Changed));
      SZWidgets.ListView<BackstoryDef>(0f, 150f, num, num2 - 85f, this.lOfBackstories, this.FBackstoryLabel, this.FBackstoryTooltip, this.FBackstoryComparator, ref this.selectedBackstory, ref this.scrollPos, false, null, true, false, false, false);
      SZWidgets.ButtonText(new Rect(160f, this.InitialSize.y - 70f, 100f, 30f), "OK".Translate(), new Action(this.DoAndClose), "");
      SZWidgets.ButtonText(new Rect(0f, this.InitialSize.y - 70f, 100f, 30f), "Remove".Translate(), new Action(this.DoRemoveAndClose), "");
    }

    // Token: 0x0600051C RID: 1308 RVA: 0x0003569C File Offset: 0x0003389C
    private void ACategoryChanged(string val)
    {
      this.selectedCategory = val;
      this.UpdateDictionary();
    }

    // Token: 0x0600051D RID: 1309 RVA: 0x000356AD File Offset: 0x000338AD
    private void AFilterChanged(string val)
    {
      this.selectedFilter = new KeyValuePair<string, SkillDef>(val, this.dicOfFilters[val]);
      this.UpdateDictionary();
    }

    // Token: 0x0600051E RID: 1310 RVA: 0x000356CF File Offset: 0x000338CF
    private void AFilter2Changed(string val)
    {
      this.selectedFilter2 = val;
      this.UpdateDictionary();
    }

    // Token: 0x0600051F RID: 1311 RVA: 0x000356E0 File Offset: 0x000338E0
    private void DoRemoveAndClose()
    {
      this.selectedBackstory = null;
      this.DoAndClose();
    }

    // Token: 0x06000520 RID: 1312 RVA: 0x000356F4 File Offset: 0x000338F4
    private void DoAndClose()
    {
      BackstoryDef backstory;
      if (!this.isChildhood)
      {
        Pawn_StoryTracker story = CEditor.API.Pawn.story;
        backstory = ((story != null) ? story.Childhood : null);
      }
      else
      {
        backstory = this.selectedBackstory;
      }
      BackstoryDef childhood = backstory;
      BackstoryDef backstory2;
      if (this.isChildhood)
      {
        Pawn_StoryTracker story2 = CEditor.API.Pawn.story;
        backstory2 = ((story2 != null) ? story2.Adulthood : null);
      }
      else
      {
        backstory2 = this.selectedBackstory;
      }
      BackstoryDef adulthood = backstory2;
      CEditor.API.Pawn.SetBackstory(childhood, adulthood);
      this.RecalcSkills();
      SearchTool.Save(this.idxList);
      this.Close(true);
    }

    // Token: 0x06000521 RID: 1313 RVA: 0x00035787 File Offset: 0x00033987
    public override void OnAcceptKeyPressed()
    {
      base.OnAcceptKeyPressed();
      this.DoAndClose();
    }

    // Token: 0x06000522 RID: 1314 RVA: 0x00035798 File Offset: 0x00033998
    private void ARandomBackstory()
    {
      SZWidgets.sFind = this.lOfBackstories.RandomElement<BackstoryDef>().TitleCapFor(CEditor.API.Pawn.gender);
      SZWidgets.sFindTemp = SZWidgets.sFind;
      SZWidgets.waitTimer = 120;
    }

    // Token: 0x040004A3 RID: 1187
    private int idxList;

    // Token: 0x040004A4 RID: 1188
    private Dictionary<BackstoryDef, string> dicBackstory;

    // Token: 0x040004A5 RID: 1189
    private bool isChildhood;

    // Token: 0x040004A6 RID: 1190
    private bool isFiltered;

    // Token: 0x040004A7 RID: 1191
    private bool isFilteredOld;

    // Token: 0x040004A8 RID: 1192
    private Vector2 scrollPos;

    // Token: 0x040004A9 RID: 1193
    private BackstoryDef selectedBackstory;

    // Token: 0x040004AA RID: 1194
    private string selectedCategory;

    // Token: 0x040004AB RID: 1195
    private KeyValuePair<string, SkillDef> selectedFilter;

    // Token: 0x040004AC RID: 1196
    private string selectedFilter2;

    // Token: 0x040004AD RID: 1197
    private bool doOnce;

    // Token: 0x040004AE RID: 1198
    private Gender gender;

    // Token: 0x040004AF RID: 1199
    private List<string> lOfCategories;

    // Token: 0x040004B0 RID: 1200
    private Dictionary<string, SkillDef> dicOfFilters;

    // Token: 0x040004B1 RID: 1201
    private List<string> lOfFilter2;

    // Token: 0x040004B2 RID: 1202
    private List<BackstoryDef> lOfBackstories;

    // Token: 0x040004B3 RID: 1203
    private BackstoryDef oBackAdult;

    // Token: 0x040004B4 RID: 1204
    private BackstoryDef oBackChild;

    // Token: 0x040004B5 RID: 1205
    private Func<BackstoryDef, string> FBackstoryLabel;

    // Token: 0x040004B6 RID: 1206
    private Func<BackstoryDef, string> FBackstoryTooltip;

    // Token: 0x040004B7 RID: 1207
    private Func<BackstoryDef, BackstoryDef, bool> FBackstoryComparator;
  }
}
