﻿using System;
using UnityEngine;
using Verse;
using RimWorld;

namespace CharacterEditor
{
  // Token: 0x02000060 RID: 96
  public class DialogChangeBirthday : Window, IPawnable
  {

    private float longitude;
    private Vector2 scrollPos;
    private int iMaxLifestage;

    //private string selectedBirthHour;
    //private string selectedBirthDay;
    //private string selectedBirthQuad;
    //private string selectedBirthYear;

    private EntryInt entryBirthHour = new EntryInt();
    private EntryInt entryBirthDay = new EntryInt();
    private EntryInt entryBirthQuad = new EntryInt();
    private EntryInt entryBirthYear = new EntryInt();

    //private string selectedBioHour;
    //private string selectedBioDay;
    //private string selectedBioQuadrum;
    //private string selectedBioYear;

    private EntryInt entryBioHour = new EntryInt();
    private EntryInt entryBioDay = new EntryInt();
    private EntryInt entryBioQuadrum = new EntryInt();
    private EntryInt entryBioYear = new EntryInt();

    private string selectedLifestage;
    private int iSelectedLifestage;

    //private int iSelectedHour;
    //private int iSelectedDay;
    //private int iSelectedQuadrum;
    //private int iSelectedYear;
    //private int iSelectedBioDay;
    //private int iSelectedBioHour;
    //private int iSelectedBioQuadrum;
    //private int iSelectedBioYear;

    //private long startTick;
    //private long startBioTick;
    //private long ticksPerDay = 60000L;
    //private long ticksPerHour = 2500L;
    //private long ticksPerQuadrum = 900000L;
    //private long ticksPerYear = 3600000L;


    // Token: 0x17000070 RID: 112
    // (get) Token: 0x06000671 RID: 1649 RVA: 0x00042C10 File Offset: 0x00040E10
    public override Vector2 InitialSize => new Vector2(350f, 680f);

    public Pawn SelectedPawn { get; set; }
    public Pawn SelectedPawn2 { get; set; }
    public Pawn SelectedPawn3 { get; set; }
    public Pawn SelectedPawn4 { get; set; }

    private long BirthAbsTicks
    {
      get
      {
        return TicksFromPeriod(entryBirthYear.valInt - GenDate.DefaultStartingYear, entryBirthQuad.valInt - 1, entryBirthDay.valInt - 1, entryBirthHour.valInt);
      }
      set
      {
        entryBirthYear.UpdateValueAndBuffer(GenDate.Year(value, this.longitude));
        entryBirthQuad.UpdateValueAndBuffer((int)GenDate.Quadrum(value, this.longitude) + 1);
        entryBirthDay.UpdateValueAndBuffer(GenDate.DayOfQuadrum(value, this.longitude) + 1);
        entryBirthHour.UpdateValueAndBuffer(GenDate.HourInteger(value, this.longitude));
      }
    }

    private long AgeChrTicks
    {
      get
      {
        return GenTicks.TicksAbs - BirthAbsTicks;
      }
      set
      {
        BirthAbsTicks = GenTicks.TicksAbs - value;
      }
    }

    private long AgeBioTicks
    {
      get
      {
        return TicksFromPeriod(entryBioYear.valInt, entryBioQuadrum.valInt, entryBioDay.valInt, entryBioHour.valInt);
      }
      set
      {
        AbsPeriodFromTicksSilent(value, out int year, out int quad, out int day, out int hour);
        entryBioYear.UpdateValueAndBuffer(year);
        entryBioQuadrum.UpdateValueAndBuffer(quad);
        entryBioDay.UpdateValueAndBuffer(day);
        entryBioHour.UpdateValueAndBuffer(hour);
        //iSelectedBioQuadrum.UpdateValueAndBuffer((int)(GenMath.PositiveMod(value, GenDate.TicksPerYear) / GenDate.TicksPerQuadrum));
        //iSelectedBioDay.UpdateValueAndBuffer((int)(GenMath.PositiveMod(value, GenDate.TicksPerQuadrum) / GenDate.TicksPerDay));
        //iSelectedBioHour.UpdateValueAndBuffer((int)(GenMath.PositiveMod(value, GenDate.TicksPerDay) / GenDate.TicksPerHour));
      }
    }

    private string ChronologicalYearsString => AdjustedDecimalString(DivideBigLongs(AgeChrTicks, GenDate.TicksPerYear));
    private string BiologicalYearsString => AdjustedDecimalString(DivideBigLongs(AgeBioTicks, GenDate.TicksPerYear));
    private string LifeExpectancyString => this.SelectedPawn?.RaceProps?.lifeExpectancy != null ? this.SelectedPawn.RaceProps.lifeExpectancy.ToString("#.##") : "Indeterminant";

    private static decimal DivideBigLongs(long num, long den)
    {
      long remainder = GenMath.PositiveMod(num, den);
      long integerPart = (num - remainder) / den;
      decimal result = (decimal)integerPart;
      result += ((decimal)remainder) / ((decimal)den);
      return result;
    }

    private static string AdjustedDecimalString(decimal num)
    {
      return num.ToString(num > 10 ? "#.00" : "#.000");
    }



    public DialogChangeBirthday(Pawn p)
    {
      Map map = p.Map;
      if (map == null)
      {
        map = (Map)typeof(DateNotifier).CallMethod("FindPlayerHomeWithMinTimezone", null, silent: true);
      }
      this.longitude = map != null ? Find.WorldGrid.LongLatOf(map.Tile).x : 0f;

      this.scrollPos = default(Vector2);
      this.SelectedPawn = p;

      if (!p.RaceProps.lifeStageAges.NullOrEmpty<LifeStageAge>())
      {
        //this.iSelectedLifestage.UpdateValueAndBuffer(p.ageTracker.CurLifeStageIndex);
        this.iSelectedLifestage = p.ageTracker.CurLifeStageIndex;
        this.iMaxLifestage = p.RaceProps.lifeStageAges.Count - 1;
      }
      this.BirthAbsTicks = p.ageTracker.BirthAbsTicks;
      this.AgeBioTicks = p.ageTracker.AgeBiologicalTicks;
      //this.startTick = -19800059000L;
      //this.entryBirthYear.UpdateValueAndBuffer(this.SelectedPawn.ageTracker.BirthYear);
      //this.entryBirthQuad.UpdateValueAndBuffer((int)this.SelectedPawn.ageTracker.BirthQuadrum + 1);
      //this.entryBirthDay.UpdateValueAndBuffer(this.SelectedPawn.ageTracker.BirthDayOfSeasonZeroBased + 1);
      //this.entryBirthHour.UpdateValueAndBuffer(GenDate.HourInteger(this.SelectedPawn.ageTracker.BirthAbsTicks, 0f));
      //this.iSelectedDay.UpdateValueAndBuffer((this.SelectedPawn.ageTracker.BirthDayOfYear + 1) % 15);
      //if (this.iSelectedDay.valInt == 0)
      //{
      //	this.iSelectedDay.valInt = 15;
      //}
      //long num = this.SelectedPawn.ageTracker.BirthAbsTicks - this.startTick;
      //long num = this.SelectedPawn.ageTracker.BirthAbsTicks - this.startTick;
      //num %= this.ticksPerYear;
      //num %= this.ticksPerQuadrum;
      //num %= this.ticksPerDay;
      //this.iSelectedHour.UpdateValueAndBuffer(Math.Abs((int)(num / this.ticksPerHour)));
      //num %= this.ticksPerHour;
      //this.startBioTick = 0L;
      //long num2 = this.startBioTick - this.SelectedPawn.ageTracker.AgeBiologicalTicks;
      //long ageBioTicks = this.SelectedPawn.ageTracker.AgeBiologicalTicks;
      //this.entryBioYear.UpdateValueAndBuffer(this.SelectedPawn.ageTracker.AgeBiologicalYears);
      //this.entryBioQuadrum.UpdateValueAndBuffer((int)GenDate.Quadrum(ageBioTicks, 0f));
      //this.entryBioDay.UpdateValueAndBuffer(GenDate.DayOfSeason(ageBioTicks, 0f));
      //this.entryBioHour.UpdateValueAndBuffer(GenDate.HourInteger(ageBioTicks, 0f));
      //   num2 %= this.ticksPerYear;
      //this.iSelectedBioQuadrum.UpdateValueAndBuffer((int)Math.Abs(num2 / this.ticksPerQuadrum));
      //num2 %= this.ticksPerQuadrum;
      //this.iSelectedBioDay.UpdateValueAndBuffer((int)Math.Abs(num2 / this.ticksPerDay));
      //num2 %= this.ticksPerDay;
      //this.iSelectedBioHour.UpdateValueAndBuffer((int)Math.Abs(num2 / this.ticksPerHour));
      //num2 %= this.ticksPerHour;

      this.doCloseX = true;
      this.absorbInputAroundWindow = true;
      this.draggable = true;
      this.layer = CEditor.Layer;
    }

    // Token: 0x0600067B RID: 1659 RVA: 0x00042E84 File Offset: 0x00041084
    public override void DoWindowContents(Rect inRect)
    {
      int num = 0;
      int num2 = 0;
      int num3 = (int)this.InitialSize.x - 16;
      int num4 = (int)this.InitialSize.y - 16 - 60;
      Rect outRect = new Rect((float)num, (float)num2, (float)num3, (float)num4);
      Rect rect = new Rect(0f, 0f, outRect.width - 16f, 400f);
      Widgets.BeginScrollView(outRect, ref this.scrollPos, rect, true);
      Rect rect2 = rect.ContractedBy(4f);
      rect2.y -= 4f;
      rect2.height = 600f;
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect2);
      listing_X.verticalSpacing = 30f;
      Text.Font = GameFont.Medium;
      listing_X.Label(Label.BIRTHDAY, 200f, 0f, -1f, null);

      listing_X.GapLine(12f);
      long oldChrTicks = AgeChrTicks;
      //listing_X.AddIntEntrySection(Label.YEAR, "", ref this.selectedBirthYear, this.entryBirthYear, -1000000 + GenDate.DefaultStartingYear, GenDate.DefaultStartingYear, -9999, GenDate.DefaultStartingYear, true, "");
      //listing_X.AddIntEntrySection(Label.QUADRUM, "quadrum", ref this.selectedBirthQuad, this.entryBirthQuad, 0, 3, 0, 3, true, "");
      //listing_X.AddIntEntrySection(Label.DAY, "", ref this.selectedBirthDay, this.entryBirthDay, 1, 15, 1, 15, true, "");
      //listing_X.AddIntEntrySection(Label.HOUR, "", ref this.selectedBirthHour, this.entryBirthHour, 0, 23, 0, 23, true, "");
      listing_X.AddIntEntrySection(Label.YEAR, "", this.entryBirthYear, -1000000, GenDate.DefaultStartingYear, 1900, GenDate.DefaultStartingYear, true, "Enter year of birth. Note that the text field can accept larger values than the slider.");
      listing_X.AddIntEntrySection(Label.QUADRUM, "quadrum", this.entryBirthQuad, 1, 4, 1, 4, true, "Enter quadrum of birth. 1 = Aprimay, 2 = Jugust, 3 = Septober, 4 = Decembary");
      listing_X.AddIntEntrySection(Label.DAY, "", this.entryBirthDay, 1, 15, 1, 15, true, "");
      listing_X.AddIntEntrySection(Label.HOUR, "", this.entryBirthHour, 0, 23, 0, 23, true, "");
      if (AgeChrTicks != oldChrTicks && AgeChrTicks < AgeBioTicks)
      {
        AgeBioTicks = AgeChrTicks;
      }
      listing_X.Label(Label.CHRONOAGE + " [" + ChronologicalYearsString + "]", -1f, 0f, -1f, null);

      listing_X.Gap(20f);
      listing_X.Label(Label.BIOAGE + " [" + BiologicalYearsString + "]", -1f, 0f, -1f, null);

      listing_X.GapLine(12f);
      long oldBioTicks = AgeBioTicks;
      string bioYearsTooltip = "Enter biological age. Note that the text field can accept larger values than the slider. The life expectancy for this species is " + this.SelectedPawn.RaceProps.lifeExpectancy.ToString("#.0") + " years.";
      listing_X.AddIntEntrySection(Label.YEARS, "", this.entryBioYear, 0, 1000000, 0, 150, true, bioYearsTooltip);
      listing_X.AddIntEntrySection(Label.QUARTALS, "", this.entryBioQuadrum, 0, 3, 0, 3, true, "");
      listing_X.AddIntEntrySection(Label.DAYS, "", this.entryBioDay, 0, 14, 0, 14, true, "");
      listing_X.AddIntEntrySection(Label.HOURS, "", this.entryBioHour, 0, 23, 0, 23, true, "");
      if (AgeBioTicks != oldBioTicks && AgeChrTicks < AgeBioTicks)
      {
        AgeChrTicks = AgeBioTicks;
      }

      listing_X.Gap(12f);
      if (this.iMaxLifestage > 0)
      {
        try
        {
          for (int i = this.iMaxLifestage; i >= 0; i--)
          {
            if (this.SelectedPawn.RaceProps.lifeStageAges[i].minAge * GenDate.TicksPerYear <= this.AgeBioTicks)
            {
              //this.iSelectedLifestage.UpdateValueAndBuffer(i);
              this.iSelectedLifestage = i;
              break;
            }
          }
          listing_X.AddIntSection(Label.LIFESTAGE, "DEF" + this.SelectedPawn.RaceProps.lifeStageAges[this.iSelectedLifestage].def.LabelCap, ref this.selectedLifestage, ref this.iSelectedLifestage, 0, this.iMaxLifestage, true, "");
        }
        catch
        {
        }
      }
      listing_X.End();
      Widgets.EndScrollView();
      Rect rect3 = new Rect(this.InitialSize.x - 140f, this.InitialSize.y - 70f, 100f, 30f);
      if (Widgets.ButtonText(rect3, "OK".Translate(), true, true, true))
      {
        this.SelectedPawn.ageTracker.BirthAbsTicks = this.BirthAbsTicks;
        this.SelectedPawn.SetAgeTicks(AgeBioTicks);
        if (!this.SelectedPawn.ageTracker.Adult && this.SelectedPawn.HasStoryTracker())
        {
          this.SelectedPawn.SetBackstory(this.SelectedPawn.story.Childhood, null);
        }
        this.Close(true);
      }
    }

    private long TicksFromPeriod(int year, int quad, int day, int hour)
    {
      long num = (long)year * GenDate.TicksPerYear;
      num += (long)quad * GenDate.TicksPerQuadrum;
      num += (long)day * GenDate.TicksPerDay;
      return num + (long)hour * GenDate.TicksPerHour;
    }

    private void AbsPeriodFromTicksSilent(long ticks, out int year, out int quad, out int day, out int hour)
    {
      long ticksYearRemainer = GenMath.PositiveMod(ticks, GenDate.TicksPerYear);
      long yearLong = (ticks - ticksYearRemainer) / GenDate.TicksPerYear;
      long ticksQuadRemainder = GenMath.PositiveMod(ticksYearRemainer, GenDate.TicksPerQuadrum);
      long quadLong = (ticksYearRemainer - ticksQuadRemainder) / GenDate.TicksPerQuadrum;
      long ticksDayRemainder = GenMath.PositiveMod(ticksYearRemainer, GenDate.TicksPerDay);
      long dayLong = (ticksQuadRemainder - ticksDayRemainder) / GenDate.TicksPerDay;
      long ticksHourRemainder = GenMath.PositiveMod(ticksYearRemainer, GenDate.TicksPerHour);
      long hourLong = (ticksDayRemainder - ticksHourRemainder) / GenDate.TicksPerHour;

      year = (int)yearLong;
      quad = (int)quadLong;
      day = (int)dayLong;
      hour = (int)hourLong;
    }

  }
}
