﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000011 RID: 17
	public class DialogChoosePawn : Window
	{
		// Token: 0x1700002D RID: 45
		// (get) Token: 0x06000091 RID: 145 RVA: 0x000041FE File Offset: 0x000023FE
		private Dictionary<string, Faction> DicFactions
		{
			get
			{
				return CEditor.API.Get<Dictionary<string, Faction>>(EType.Factions);
			}
		}

		// Token: 0x1700002E RID: 46
		// (get) Token: 0x06000092 RID: 146 RVA: 0x0000420C File Offset: 0x0000240C
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(300f, 800f);
			}
		}

		// Token: 0x06000093 RID: 147 RVA: 0x00004230 File Offset: 0x00002430
		public DialogChoosePawn(IPawnable _callback, int _id = 1, Gender gender = Gender.None)
		{
			this.scrollPos = default(Vector2);
			this.id = _id;
			this.idxList = 10;
			SearchTool.Update(this.idxList);
			this.choosenGender = gender;
			this.lOfPawns = new List<Pawn>();
			this.DicFactions.Clear();
			this.DicFactions.Merge(FactionTool.GetDicOfFactions(true, true, true));
			this.callback = _callback;
			this.selectedPawn = null;
			this.AFactionSelected(this.DicFactions.First<KeyValuePair<string, Faction>>().Key);
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x06000094 RID: 148 RVA: 0x000042E8 File Offset: 0x000024E8
		public override void DoWindowContents(Rect inRect)
		{
			int w = (int)this.InitialSize.x - 40;
			int num = 0;
			int x = 0;
			this.DrawTitle(x, ref num, w, 30);
			this.DrawDropdowns(x, ref num, w, 30);
			this.DrawPawnList(x, ref num, w, 700);
			this.DrawAccept(x, ref num, w, 30);
		}

		// Token: 0x06000095 RID: 149 RVA: 0x00004344 File Offset: 0x00002544
		private void DrawTitle(int x, ref int y, int w, int h)
		{
			Text.Font = GameFont.Medium;
			Widgets.Label(new Rect((float)x, (float)y, (float)w, (float)h), Label.CHOOSE_PAWN);
			bool flag = this.choosenGender > Gender.None;
			if (flag)
			{
				SZWidgets.Image(new Rect((float)(x + w - 20), (float)(y + 8), 20f, 20f), (this.choosenGender == Gender.Male) ? "bmale" : "bfemale");
			}
			y += 30;
		}

		// Token: 0x06000096 RID: 150 RVA: 0x000043C0 File Offset: 0x000025C0
		public void DrawDropdowns(int x, ref int y, int w, int h)
		{
			Text.Font = GameFont.Small;
			SZWidgets.FloatMenuOnButtonText<string>(new Rect((float)x, (float)y, (float)w, (float)h), this.selectedListname, this.DicFactions.Keys.ToList<string>(), (string s) => s, new Action<string>(this.AFactionSelected));
			y += 32;
		}

		// Token: 0x06000097 RID: 151 RVA: 0x00004434 File Offset: 0x00002634
		private void AFactionSelected(string listname)
		{
			this.selectedListname = listname;
			this.lOfPawns.Clear();
			List<Pawn> pawnList = PawnxTool.GetPawnList(this.selectedListname, false, this.DicFactions.GetValue(this.selectedListname));
			bool flag = this.choosenGender > Gender.None;
			if (flag)
			{
				foreach (Pawn pawn in pawnList)
				{
					bool flag2 = pawn.gender == this.choosenGender;
					if (flag2)
					{
						this.lOfPawns.Add(pawn);
					}
				}
			}
			else
			{
				this.lOfPawns.AddRange(pawnList);
			}
			this.selectedPawn = this.lOfPawns.FirstOrFallback(null);
		}

		// Token: 0x06000098 RID: 152 RVA: 0x00004504 File Offset: 0x00002704
		private void DrawPawnList(int x, ref int y, int w, int h)
		{
			Text.Font = GameFont.Small;
			Rect outRect = new Rect(0f, (float)y, (float)w, (float)(h - y));
			Rect rect = new Rect(0f, (float)y, outRect.width - 16f, (float)(this.lOfPawns.Count * 112));
			Widgets.BeginScrollView(outRect, ref this.scrollPos, rect, true);
			Rect rect2 = rect.ContractedBy(4f);
			rect2.height = rect.height;
			Listing_X listing_X = new Listing_X();
			listing_X.Begin(rect2);
			try
			{
				using (List<Pawn>.Enumerator enumerator = this.lOfPawns.GetEnumerator())
				{
					while (enumerator.MoveNext())
					{
						int num = listing_X.Selectable(enumerator.Current.GetPawnName(true), this.selectedPawn == enumerator.Current, "", PortraitsCache.Get(enumerator.Current, new Vector2(128f, 180f), Rot4.South, default(Vector3), 1f, true, true, true, true, null, null, false), null, null, default(Vector2), false, 100f, (enumerator.Current.Faction == null) ? Color.white : enumerator.Current.Faction.Color, ColorTool.colLightGray, true);
						bool flag = num == 1;
						if (flag)
						{
							this.selectedPawn = enumerator.Current;
						}
					}
				}
			}
			catch
			{
			}
			listing_X.End();
			Widgets.EndScrollView();
			y += 700;
		}

		// Token: 0x06000099 RID: 153 RVA: 0x000046C8 File Offset: 0x000028C8
		private void DrawAccept(int x, ref int y, int w, int h)
		{
			Rect rect = new Rect(this.InitialSize.x - 140f, this.InitialSize.y - 70f, 100f, 30f);
			SZWidgets.ButtonText(rect, "OK".Translate(), new Action(this.DoAndClose), "");
		}

		// Token: 0x0600009A RID: 154 RVA: 0x00004730 File Offset: 0x00002930
		private void DoAndClose()
		{
			bool flag = this.id <= 1;
			if (flag)
			{
				this.callback.SelectedPawn = this.selectedPawn;
			}
			else
			{
				bool flag2 = this.id == 2;
				if (flag2)
				{
					this.callback.SelectedPawn2 = this.selectedPawn;
				}
				else
				{
					bool flag3 = this.id == 3;
					if (flag3)
					{
						this.callback.SelectedPawn3 = this.selectedPawn;
					}
					else
					{
						bool flag4 = this.id == 4;
						if (flag4)
						{
							this.callback.SelectedPawn4 = this.selectedPawn;
						}
					}
				}
			}
			SearchTool.Save(this.idxList);
			this.Close(true);
		}

		// Token: 0x040000A6 RID: 166
		private Vector2 scrollPos;

		// Token: 0x040000A7 RID: 167
		private int idxList;

		// Token: 0x040000A8 RID: 168
		private int id;

		// Token: 0x040000A9 RID: 169
		private List<Pawn> lOfPawns;

		// Token: 0x040000AA RID: 170
		private Pawn selectedPawn;

		// Token: 0x040000AB RID: 171
		private string selectedListname;

		// Token: 0x040000AC RID: 172
		private IPawnable callback;

		// Token: 0x040000AD RID: 173
		private Gender choosenGender;
	}
}
