﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  // Token: 0x02000050 RID: 80
  public class DialogColorPicker : Window
  {
    // Token: 0x1700005F RID: 95
    // (get) Token: 0x060004EC RID: 1260 RVA: 0x00032184 File Offset: 0x00030384
    public override Vector2 InitialSize
    {
      get
      {
        return new Vector2(500f, 800f);
      }
    }

    // Token: 0x060004ED RID: 1261 RVA: 0x000321A8 File Offset: 0x000303A8
    public DialogColorPicker(ColorType _colorType, bool _primaryColor = true, Apparel a = null, ThingWithComps w = null)
    {
      this.doCloseX = true;
      this.absorbInputAroundWindow = false;
      this.draggable = true;
      this.layer = CEditor.Layer;
      this.isPrimaryColor = _primaryColor;
      this.isColor1Choosen = this.isPrimaryColor;
      this.regexColor = new Regex("^[0-9,]*");
      this.regexHex = new Regex("^[0-9A-F]*");
      this.scrollPos = default(Vector2);
      this.selectedApparel = null;
      this.selectedDef = null;
      this.selectedHair = null;
      this.selectedModName = null;
      this.lOfHairDefs = null;
      this.colorType = _colorType;
      this.Xwidth = ((this.colorType == ColorType.SkinColor || this.colorType == ColorType.FavColor || this.colorType == ColorType.EyeColor) ? 268 : 500);
      this.bInstantClose = false;
      this.doOnce = true;
      bool flag = !this.Preselect(a, w);
      if (flag)
      {
        this.bInstantClose = true;
      }
      else
      {
        this.bInstantClose = false;
        this.fMinBright = 0f;
        this.fMaxBright = ColorTool.FMAX;
        this.fRed = this.selectedColor.r;
        this.fBlue = this.selectedColor.b;
        this.fGreen = this.selectedColor.g;
        this.fAlpha = this.selectedColor.a;
        this.part = ColorTool.DMAX / ColorTool.DMAXB;
        this.TextValuesFromSelectedColor();
        this.oldSelected = this.selectedColor;
      }
    }

    // Token: 0x060004EE RID: 1262 RVA: 0x00032324 File Offset: 0x00030524
    public bool Preselect(Apparel a, ThingWithComps w)
    {
      bool flag = this.colorType == ColorType.SkinColor;
      if (flag)
      {
        bool flag2 = !CEditor.API.Pawn.HasStoryTracker();
        if (flag2)
        {
          return false;
        }
        this.selectedColor = CEditor.API.Pawn.GetSkinColor(this.isPrimaryColor);
      }
      else
      {
        bool flag3 = this.colorType == ColorType.HairColor;
        if (flag3)
        {
          bool flag4 = !CEditor.API.Pawn.HasStoryTracker();
          if (flag4)
          {
            return false;
          }
          this.lOfHairDefs = DefTool.ListByMod<HairDef>(this.selectedModName);
          this.selectedHair = CEditor.API.Pawn.story.hairDef;
          this.selectedColor = CEditor.API.Pawn.GetHairColor(this.isPrimaryColor);
        }
        else
        {
          bool flag5 = this.colorType == ColorType.FavColor;
          if (flag5)
          {
            bool flag6 = !CEditor.API.Pawn.HasStoryTracker();
            if (flag6)
            {
              return false;
            }
            this.selectedColor = CEditor.API.Pawn.story.favoriteColor.GetValueOrDefault();
          }
          else
          {
            bool flag7 = this.colorType == ColorType.ApparelColor;
            if (flag7)
            {
              bool flag8 = !CEditor.API.Pawn.HasApparelTracker();
              if (flag8)
              {
                return false;
              }
              this.selectedApparel = ((a != null) ? a : CEditor.API.Pawn.apparel.WornApparel.FirstOrFallback(null));
              Apparel apparel = this.selectedApparel;
              this.selectedDef = ((apparel != null) ? apparel.def : null);
              this.selectedColor = ((this.selectedApparel == null) ? Color.white : this.selectedApparel.DrawColor);
            }
            else
            {
              bool flag9 = this.colorType == ColorType.WeaponColor;
              if (flag9)
              {
                bool flag10 = !CEditor.API.Pawn.HasEquipmentTracker();
                if (flag10)
                {
                  return false;
                }
                this.selectedWeapon = ((w != null) ? w : CEditor.API.Pawn.equipment.Primary);
                ThingWithComps thingWithComps = this.selectedWeapon;
                this.selectedDef = ((thingWithComps != null) ? thingWithComps.def : null);
                this.selectedColor = ((this.selectedWeapon == null) ? Color.white : this.selectedWeapon.DrawColor);
              }
              else
              {
                bool flag11 = this.colorType == ColorType.EyeColor;
                if (flag11)
                {
                  this.selectedColor = CEditor.API.Pawn.GetEyeColor();
                }
                else
                {
                  this.selectedColor = Color.white;
                }
              }
            }
          }
        }
      }
      return true;
    }

    // Token: 0x060004EF RID: 1263 RVA: 0x000325A8 File Offset: 0x000307A8
    public override void DoWindowContents(Rect inRect)
    {
      bool flag = this.bInstantClose;
      if (flag)
      {
        this.Close(true);
      }
      else
      {
        bool flag2 = this.doOnce;
        if (flag2)
        {
          this.doOnce = false;
          this.windowRect.width = (float)this.Xwidth;
          this.windowRect.position = new Vector2((float)(CEditor.API.EditorPosX + 378), this.windowRect.y);
        }
        int num = 0;
        this.DrawTitle(0, num, 300, 30);
        num += 40;
        this.DrawColorTable(0, num, 57, 21);
        num += 313;
        this.DrawDerivedColors(0, num, 21);
        num += 40;
        this.DrawColorSlider(0, num, 232, 230);
        num += 215;
        this.DrawTextFields(0, num, 232, 110);
        num += 85;
        this.DrawRadioButtons(0, num);
        bool flag3 = this.colorType == ColorType.HairColor;
        if (flag3)
        {
          this.DrawDropdown(249, 4, 214, 30);
        }
        this.DrawLists(249, 40, 214, (int)this.InitialSize.y - 110);
        this.UpdatePawnColors();
        Rect rect = new Rect(0f, this.InitialSize.y - 65f, 80f, 25f);
        bool flag4 = Widgets.ButtonText(rect, "OK".Translate(), true, true, true);
        if (flag4)
        {
          this.Close(true);
        }
      }
    }

    // Token: 0x060004F0 RID: 1264 RVA: 0x00032734 File Offset: 0x00030934
    private void DrawDropdown(int x, int y, int w, int h)
    {
      Rect rect = new Rect((float)x, (float)y, (float)w, (float)h);
      SZWidgets.FloatMenuOnButtonText<string>(rect, this.selectedModName ?? Label.ALL, CEditor.API.ListOf<string>(EType.ModsHairDef), (string s) => s ?? Label.ALL, new Action<string>(this.ASelectedModName), "");
    }

    // Token: 0x060004F1 RID: 1265 RVA: 0x000327A4 File Offset: 0x000309A4
    private void ASelectedModName(string val)
    {
      this.selectedModName = val;
      this.lOfHairDefs = DefTool.ListByMod<HairDef>(this.selectedModName);
    }

    // Token: 0x060004F2 RID: 1266 RVA: 0x000327C0 File Offset: 0x000309C0
    private void DrawTitle(int x, int y, int w, int h)
    {
      Text.Font = GameFont.Medium;
      Widgets.Label(new Rect((float)x, (float)y, (float)w, (float)h), Label.SELECT_COLOR);
      bool flag = this.colorType == ColorType.FavColor || !CEditor.API.Pawn.HasStoryTracker() || CEditor.API.Pawn.story.favoriteColor == null;
      if (!flag)
      {
        SZWidgets.ButtonTextureTextHighlight2(new Rect(200f, (float)y, 30f, 30f), "", "bfavcolor", CEditor.API.Pawn.story.favoriteColor.GetValueOrDefault(), new Action(this.AFromFavColor), CompatibilityTool.GetFavoriteColorTooltip(CEditor.API.Pawn), true);
      }
    }

    // Token: 0x060004F3 RID: 1267 RVA: 0x00032888 File Offset: 0x00030A88
    private void AFromFavColor()
    {
      this.AColorSelected(CEditor.API.Pawn.story.favoriteColor.GetValueOrDefault());
    }

    // Token: 0x060004F4 RID: 1268 RVA: 0x000328AC File Offset: 0x00030AAC
    private void DrawColorTable(int x, int y, int w, int h)
    {
      Widgets.DrawBoxSolid(new Rect((float)x, (float)y, 231f, 26f), this.selectedColor);
      y += 27;
      int num = 0;
      for (int i = 0; i < 4; i++)
      {
        for (int j = 0; j < 13; j++)
        {
          SZWidgets.ButtonImageCol((float)x, (float)y, (float)w, (float)h, "bwhite", new Action<Color>(this.AColorSelected), ColorTool.ListOfColors.ElementAtOrDefault(num), "");
          y += h + 1;
          num++;
        }
        x += w + 1;
        y -= 13 * (h + 1);
      }
    }

    // Token: 0x060004F5 RID: 1269 RVA: 0x00032958 File Offset: 0x00030B58
    private void DrawDerivedColors(int x, int y, int h)
    {
      float num = 0.15f;
      for (int i = 0; i < 11; i++)
      {
        SZWidgets.ButtonImageCol((float)x, (float)y, (float)h, (float)h, "bwhite", new Action<Color>(this.AColorSelected), ColorTool.GetDerivedColor(this.selectedColor, num), "");
        x += h;
        num -= 0.03f;
      }
    }

    // Token: 0x060004F6 RID: 1270 RVA: 0x000329BC File Offset: 0x00030BBC
    private void DrawColorSlider(int x, int y, int w, int h)
    {
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(new Rect((float)x, (float)y, (float)w, (float)h));
      this.fRed = listing_X.Slider(this.selectedColor.r, 0f, (float)ColorTool.IMAX, Color.red);
      this.fGreen = listing_X.Slider(this.selectedColor.g, 0f, (float)ColorTool.IMAX, Color.green);
      this.fBlue = listing_X.Slider(this.selectedColor.b, 0f, (float)ColorTool.IMAX, Color.blue);
      this.fAlpha = listing_X.Slider(this.selectedColor.a, 0f, (float)ColorTool.IMAX, Color.white);
      this.selectedColor = new Color(this.fRed, this.fGreen, this.fBlue, this.fAlpha);
      listing_X.Gap(20f);
      GUI.color = Color.gray;
      listing_X.Label(Label.MIN_RANDOM_BRIGHTNESS, -1f, -1f, -1f, null);
      listing_X.CurY -= 5f;
      this.fMinBright = listing_X.Slider(this.fMinBright, 0f, ColorTool.FMAX);
      listing_X.CurY -= 5f;
      listing_X.Label(Label.MAX_RANDOM_BRIGHTNESS, -1f, -1f, -1f, null);
      listing_X.CurY -= 5f;
      bool flag = this.fMaxBright < this.fMinBright;
      if (flag)
      {
        this.fMaxBright = this.fMinBright;
      }
      this.fMaxBright = listing_X.Slider(this.fMaxBright, 0f, ColorTool.FMAX);
      listing_X.End();
    }

    // Token: 0x060004F7 RID: 1271 RVA: 0x00032B8C File Offset: 0x00030D8C
    private void DrawTextFields(int x, int y, int w, int h)
    {
      bool flag = this.oldSelected != this.selectedColor;
      if (flag)
      {
        this.TextValuesFromSelectedColor();
      }
      this.sRGB = Widgets.TextField(new Rect((float)x, (float)y, (float)(w - 40), 24f), this.sRGB, 15, this.regexColor);
      SZWidgets.ButtonImage((float)(x + w - 30), (float)y, 25f, 25f, "brandom", new Action(this.ARandomColor), Label.TIP_DLG_RANDOM_COLOR);
      y += 40;
      GUI.color = Color.gray;
      this.sHex = Widgets.TextField(new Rect((float)x, (float)y, (float)(w - 40), 24f), this.sHex, 15, this.regexHex);
      bool flag2 = this.colorType == ColorType.HairColor;
      if (flag2)
      {
        SZWidgets.ButtonImage((float)(x + w - 30), (float)y, 25f, 25f, "brandom", new Action(this.ARandomizeHairAndColor), Label.TIP_DLG_RANDOMIZE_HAIRANDCOLOR);
      }
      bool flag3 = !string.IsNullOrEmpty(this.sRGB) && this.oldsRGB != this.sRGB;
      if (flag3)
      {
        this.RGBTextToSelectedColor();
      }
      bool flag4 = !string.IsNullOrEmpty(this.sHex) && this.oldHex != this.sHex;
      if (flag4)
      {
        this.HEXTextToSelectedColor();
      }
    }

    // Token: 0x060004F8 RID: 1272 RVA: 0x00032CE4 File Offset: 0x00030EE4
    private void DrawRadioButtons(int x, int y)
    {
      bool flag = Widgets.RadioButtonLabeled(new Rect((float)x, (float)y, 90f, 30f), Label.COLORA, this.isColor1Choosen);
      if (flag)
      {
        this.isColor1Choosen = true;
        bool flag2 = this.colorType == ColorType.HairColor;
        if (flag2)
        {
          this.selectedColor = CEditor.API.Pawn.story.HairColor;
        }
        else
        {
          bool flag3 = this.colorType == ColorType.SkinColor;
          if (flag3)
          {
            this.selectedColor = CEditor.API.Pawn.GetSkinColor(true);
          }
        }
      }
      x += 100;
      bool flag4 = this.colorType == ColorType.SkinColor || (CEditor.IsGradientHairActive && this.colorType == ColorType.HairColor);
      if (flag4)
      {
        bool flag5 = Widgets.RadioButtonLabeled(new Rect((float)x, (float)y, 90f, 30f), Label.COLORB, !this.isColor1Choosen);
        if (flag5)
        {
          this.isColor1Choosen = false;
          bool flag6 = this.colorType == ColorType.HairColor;
          if (flag6)
          {
            this.selectedColor = CEditor.API.Pawn.GetHairColor(false);
          }
          else
          {
            bool flag7 = this.colorType == ColorType.SkinColor;
            if (flag7)
            {
              this.selectedColor = CEditor.API.Pawn.GetSkinColor(false);
            }
          }
        }
      }
    }

    // Token: 0x060004F9 RID: 1273 RVA: 0x00032E28 File Offset: 0x00031028
    private void DrawLists(int x, int y, int w, int h)
    {
      Text.Font = GameFont.Small;
      //Rect rect = new Rect((float)x, (float)y, (float)w, (float)h);
      bool flag = this.colorType == ColorType.HairColor;
      if (flag)
      {
        //SZWidgets.ListView<HairDef>(rect, this.lOfHairDefs, (HairDef hd) => hd.LabelCap, (HairDef hd) => hd.description, (HairDef hairA, HairDef hairB) => hairA == hairB, ref this.selectedHair, ref this.scrollPos, false, new Action<HairDef>(this.AHairSelected), true, false, false, false);
        SZWidgets.ListView<HairDef>(x, y, w, h, this.lOfHairDefs, (HairDef hd) => hd.LabelCap, (HairDef hd) => hd.description, (HairDef hairA, HairDef hairB) => hairA == hairB, ref this.selectedHair, ref this.scrollPos, false, new Action<HairDef>(this.AHairSelected), true, false, false, false);
      }
      else
      {
        bool flag2 = this.colorType == ColorType.ApparelColor;
        if (flag2)
        {
          List<ThingDef> l = (from td in CEditor.API.Pawn.apparel.WornApparel
                              select td.def).ToList<ThingDef>();
          //SZWidgets.ListView<ThingDef>(rect, l, (ThingDef apparel) => apparel.label, (ThingDef apparel) => apparel.DescriptionDetailed, (ThingDef appA, ThingDef appB) => appA == appB, ref this.selectedDef, ref this.scrollPos, false, new Action<ThingDef>(this.AApparelSelected), true, false, false, false);
          SZWidgets.ListView<ThingDef>(x, y, w, h, l, (ThingDef apparel) => apparel.label, (ThingDef apparel) => apparel.DescriptionDetailed, (ThingDef appA, ThingDef appB) => appA == appB, ref this.selectedDef, ref this.scrollPos, false, new Action<ThingDef>(this.AApparelSelected), true, false, false, false);
        }
        else
        {
          bool flag3 = this.colorType == ColorType.WeaponColor;
          if (flag3)
          {
            List<ThingDef> l2 = (from td in CEditor.API.Pawn.equipment.AllEquipmentListForReading
                                 select td.def).ToList<ThingDef>();
            //SZWidgets.ListView<ThingDef>(rect, l2, (ThingDef weapon) => weapon.label, (ThingDef weapon) => weapon.DescriptionDetailed, (ThingDef wA, ThingDef wB) => wA == wB, ref this.selectedDef, ref this.scrollPos, false, new Action<ThingDef>(this.AWeaponSelected), true, false, false, false);
            SZWidgets.ListView<ThingDef>(x, y, w, h, l2, (ThingDef weapon) => weapon.label, (ThingDef weapon) => weapon.DescriptionDetailed, (ThingDef wA, ThingDef wB) => wA == wB, ref this.selectedDef, ref this.scrollPos, false, new Action<ThingDef>(this.AWeaponSelected), true, false, false, false);
          }
        }
      }
    }

    // Token: 0x060004FA RID: 1274 RVA: 0x00033098 File Offset: 0x00031298
    private void TextValuesFromSelectedColor()
    {
      double num = (double)this.selectedColor.r / this.part;
      double num2 = (double)this.selectedColor.g / this.part;
      double num3 = (double)this.selectedColor.b / this.part;
      double num4 = (double)this.selectedColor.a / this.part;
      this.iRed = (int)num;
      this.iGreen = (int)num2;
      this.iBlue = (int)num3;
      this.iAlpha = (int)num4;
      this.sRGB = string.Concat(new string[]
      {
        this.iRed.ToString(),
        ",",
        this.iGreen.ToString(),
        ",",
        this.iBlue.ToString(),
        ",",
        this.iAlpha.ToString()
      });
      this.sHex = string.Concat(new string[]
      {
        this.iRed.ToString("X2"),
        ",",
        this.iGreen.ToString("X2"),
        ",",
        this.iBlue.ToString("X2"),
        ",",
        this.iAlpha.ToString("X2")
      });
      this.oldsRGB = this.sRGB;
      this.oldHex = this.sHex;
    }

    // Token: 0x060004FB RID: 1275 RVA: 0x00033204 File Offset: 0x00031404
    private void RGBTextToSelectedColor()
    {
      this.oldsRGB = this.sRGB;
      string[] array = this.sRGB.Split(new string[]
      {
        ","
      }, StringSplitOptions.RemoveEmptyEntries);
      bool flag = array.Length > 3;
      if (flag)
      {
        string s = array[0];
        string s2 = array[1];
        string s3 = array[2];
        string s4 = array[3];
        bool flag2 = false;
        bool flag3 = false;
        bool flag4 = false;
        bool flag5 = false;
        bool flag6 = int.TryParse(s, out this.iRed);
        if (flag6)
        {
          flag2 = true;
        }
        bool flag7 = int.TryParse(s2, out this.iGreen);
        if (flag7)
        {
          flag3 = true;
        }
        bool flag8 = int.TryParse(s3, out this.iBlue);
        if (flag8)
        {
          flag4 = true;
        }
        bool flag9 = int.TryParse(s4, out this.iAlpha);
        if (flag9)
        {
          flag5 = true;
        }
        bool flag10 = this.iRed > ColorTool.IMAXB;
        if (flag10)
        {
          this.iRed = ColorTool.IMAXB;
        }
        bool flag11 = this.iGreen > ColorTool.IMAXB;
        if (flag11)
        {
          this.iGreen = ColorTool.IMAXB;
        }
        bool flag12 = this.iBlue > ColorTool.IMAXB;
        if (flag12)
        {
          this.iBlue = ColorTool.IMAXB;
        }
        bool flag13 = this.iAlpha > ColorTool.IMAXB;
        if (flag13)
        {
          this.iAlpha = ColorTool.IMAXB;
        }
        bool flag14 = flag2 && flag3 && flag4 && flag5;
        if (flag14)
        {
          this.sHex = string.Concat(new string[]
          {
            this.iRed.ToString("X2"),
            ",",
            this.iGreen.ToString("X2"),
            ",",
            this.iBlue.ToString("X2"),
            ",",
            this.iAlpha.ToString("X2")
          });
          this.oldHex = this.sHex;
          this.selectedColor = new Color((float)(this.part * (double)this.iRed), (float)(this.part * (double)this.iGreen), (float)(this.part * (double)this.iBlue), (float)(this.part * (double)this.iAlpha));
        }
      }
    }

    // Token: 0x060004FC RID: 1276 RVA: 0x00033414 File Offset: 0x00031614
    private void HEXTextToSelectedColor()
    {
      this.oldHex = this.sHex;
      string value = this.sHex.SubstringTo(",", true);
      string value2 = this.sHex.SubstringTo(",", 2).SubstringFrom(",", true);
      string value3 = this.sHex.SubstringTo(",", 3).SubstringFrom(",", 2);
      string value4 = this.sHex.SubstringFrom(",", 3);
      bool flag = true;
      bool flag2 = true;
      bool flag3 = true;
      bool flag4 = true;
      try
      {
        this.iRed = (int)Convert.ToByte(value, 16);
      }
      catch
      {
        flag = false;
      }
      try
      {
        this.iGreen = (int)Convert.ToByte(value2, 16);
      }
      catch
      {
        flag2 = false;
      }
      try
      {
        this.iBlue = (int)Convert.ToByte(value3, 16);
      }
      catch
      {
        flag3 = false;
      }
      try
      {
        this.iAlpha = (int)Convert.ToByte(value4, 16);
      }
      catch
      {
        flag4 = false;
      }
      bool flag5 = flag && flag2 && flag3 && flag4;
      if (flag5)
      {
        this.sRGB = string.Concat(new string[]
        {
          this.iRed.ToString(),
          ",",
          this.iGreen.ToString(),
          ",",
          this.iBlue.ToString(),
          ",",
          this.iAlpha.ToString()
        });
        this.oldsRGB = this.sRGB;
        this.selectedColor = new Color((float)(this.part * (double)this.iRed), (float)(this.part * (double)this.iGreen), (float)(this.part * (double)this.iBlue), (float)(this.part * (double)this.iAlpha));
      }
    }

    // Token: 0x060004FD RID: 1277 RVA: 0x00033604 File Offset: 0x00031804
    private void UpdatePawnColors()
    {
      Color color = this.selectedColor;
      bool flag = true;
      if (flag)
      {
        bool flag2 = this.colorType == ColorType.HairColor;
        if (flag2)
        {
          bool flag3 = this.selectedHair != CEditor.API.Pawn.story.hairDef;
          if (flag3)
          {
            CEditor.API.Pawn.SetHair(this.selectedHair);
          }
          CEditor.API.Pawn.SetHairColor(this.isColor1Choosen, this.selectedColor);
        }
        else
        {
          bool flag4 = this.colorType == ColorType.ApparelColor;
          if (flag4)
          {
            bool flag5 = this.selectedApparel != null;
            if (flag5)
            {
              this.selectedApparel.DrawColor = this.selectedColor;
            }
          }
          else
          {
            bool flag6 = this.colorType == ColorType.WeaponColor;
            if (flag6)
            {
              bool flag7 = this.selectedWeapon != null;
              if (flag7)
              {
                this.selectedWeapon.DrawColor = this.selectedColor;
              }
            }
            else
            {
              bool flag8 = this.colorType == ColorType.SkinColor;
              if (flag8)
              {
                CEditor.API.Pawn.SetSkinColor(this.isColor1Choosen, this.selectedColor);
              }
              else
              {
                bool flag9 = this.colorType == ColorType.FavColor;
                if (flag9)
                {
                  CEditor.API.Pawn.story.favoriteColor = new Color?(this.selectedColor);
                }
                else
                {
                  bool flag10 = this.colorType == ColorType.EyeColor;
                  if (flag10)
                  {
                    CEditor.API.Pawn.SetEyeColor(this.selectedColor);
                  }
                }
              }
            }
          }
        }
      }
      bool flag11 = this.oldSelected != this.selectedColor;
      if (flag11)
      {
        this.oldSelected = this.selectedColor;
        CEditor.API.UpdateGraphics();
      }
    }

    // Token: 0x060004FE RID: 1278 RVA: 0x000337B0 File Offset: 0x000319B0
    private void AColorSelected(Color color)
    {
      this.selectedColor = color;
    }

    // Token: 0x060004FF RID: 1279 RVA: 0x000337BC File Offset: 0x000319BC
    private void ARandomColor()
    {
      bool control = Event.current.control;
      if (control)
      {
        this.selectedColor = ColorTool.RandomAlphaColor;
      }
      else
      {
        this.selectedColor = ColorTool.GetRandomColor(this.fMinBright, this.fMaxBright, false);
      }
    }

    // Token: 0x06000500 RID: 1280 RVA: 0x00033800 File Offset: 0x00031A00
    private void ARandomizeHairAndColor()
    {
      bool flag = CEditor.API.Pawn == null || CEditor.API.Pawn.story == null;
      if (!flag)
      {
        CEditor.API.Pawn.SetHair(true, true, this.selectedModName);
        this.selectedHair = CEditor.API.Pawn.story.hairDef;
        Color color = Event.current.control ? ColorTool.RandomAlphaColor : ColorTool.GetRandomColor(this.fMinBright, this.fMaxBright, false);
        CEditor.API.Pawn.SetHairColor(true, color);
        Color color2 = Event.current.control ? ColorTool.RandomAlphaColor : ColorTool.GetRandomColor(this.fMinBright, this.fMaxBright, false);
        CEditor.API.Pawn.SetHairColor(false, color2);
        this.selectedColor = (this.isColor1Choosen ? color : color2);
        CEditor.API.UpdateGraphics();
      }
    }

    // Token: 0x06000501 RID: 1281 RVA: 0x000338F7 File Offset: 0x00031AF7
    private void AHairSelected(HairDef hairDef)
    {
      CEditor.API.Pawn.SetHair(hairDef);
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x06000502 RID: 1282 RVA: 0x00033918 File Offset: 0x00031B18
    private void AApparelSelected(ThingDef apparelDef)
    {
      this.selectedApparel = (from td in CEditor.API.Pawn.apparel.WornApparel
                              where td.def == apparelDef
                              select td).First<Apparel>();
      this.selectedColor = this.selectedApparel.DrawColor;
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x06000503 RID: 1283 RVA: 0x00033980 File Offset: 0x00031B80
    private void AWeaponSelected(ThingDef weaponDef)
    {
      this.selectedWeapon = (from td in CEditor.API.Pawn.equipment.AllEquipmentListForReading
                             where td.def == weaponDef
                             select td).First<ThingWithComps>();
      this.selectedColor = this.selectedWeapon.DrawColor;
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x04000475 RID: 1141
    private Regex regexColor;

    // Token: 0x04000476 RID: 1142
    private Regex regexHex;

    // Token: 0x04000477 RID: 1143
    private Vector2 scrollPos;

    // Token: 0x04000478 RID: 1144
    private Color selectedColor;

    // Token: 0x04000479 RID: 1145
    private Color oldSelected;

    // Token: 0x0400047A RID: 1146
    private Apparel selectedApparel;

    // Token: 0x0400047B RID: 1147
    private ThingWithComps selectedWeapon;

    // Token: 0x0400047C RID: 1148
    private ThingDef selectedDef;

    // Token: 0x0400047D RID: 1149
    private HairDef selectedHair;

    // Token: 0x0400047E RID: 1150
    private List<HairDef> lOfHairDefs;

    // Token: 0x0400047F RID: 1151
    private string selectedModName;

    // Token: 0x04000480 RID: 1152
    private ColorType colorType;

    // Token: 0x04000481 RID: 1153
    private bool isColor1Choosen;

    // Token: 0x04000482 RID: 1154
    private bool bInstantClose;

    // Token: 0x04000483 RID: 1155
    private bool doOnce;

    // Token: 0x04000484 RID: 1156
    private float fRed;

    // Token: 0x04000485 RID: 1157
    private float fBlue;

    // Token: 0x04000486 RID: 1158
    private float fGreen;

    // Token: 0x04000487 RID: 1159
    private float fAlpha;

    // Token: 0x04000488 RID: 1160
    private float fMaxBright;

    // Token: 0x04000489 RID: 1161
    private float fMinBright;

    // Token: 0x0400048A RID: 1162
    private int iRed;

    // Token: 0x0400048B RID: 1163
    private int iGreen;

    // Token: 0x0400048C RID: 1164
    private int iBlue;

    // Token: 0x0400048D RID: 1165
    private int iAlpha;

    // Token: 0x0400048E RID: 1166
    private string sRGB;

    // Token: 0x0400048F RID: 1167
    private string oldsRGB;

    // Token: 0x04000490 RID: 1168
    private string sHex;

    // Token: 0x04000491 RID: 1169
    private string oldHex;

    // Token: 0x04000492 RID: 1170
    private bool isPrimaryColor;

    // Token: 0x04000493 RID: 1171
    private double part;

    // Token: 0x04000494 RID: 1172
    private int Xwidth;
  }
}
