﻿using System;
using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace CharacterEditor
{
	// Token: 0x02000045 RID: 69
	public class DialogItemry : Window
	{
		// Token: 0x17000058 RID: 88
		// (get) Token: 0x060003FC RID: 1020 RVA: 0x000212D8 File Offset: 0x0001F4D8
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(this.MAXW, 800f);
			}
		}

		// Token: 0x060003FD RID: 1021 RVA: 0x000212FC File Offset: 0x0001F4FC
		private string GetLaberForSelectedCategory(ThingCategoryDef val)
		{
			return (val == null) ? Label.ALL : val.label.CapitalizeFirst();
		}

		// Token: 0x060003FE RID: 1022 RVA: 0x00021324 File Offset: 0x0001F524
		private string GetThingStyleDefLabel(ThingStyleDef t)
		{
			return (t == null) ? Label.NONE : (t.label.NullOrEmpty() ? t.defName : t.label.CapitalizeFirst());
		}

		// Token: 0x060003FF RID: 1023 RVA: 0x00021360 File Offset: 0x0001F560
		private string GetStatLabel(StatDef s)
		{
			return (s == null) ? Label.NONE : s.label;
		}

		// Token: 0x06000400 RID: 1024 RVA: 0x00021384 File Offset: 0x0001F584
		private string GetTradeabilityLabel(Tradeability t)
		{
			return Enum.GetName(typeof(Tradeability), t);
		}

		// Token: 0x06000401 RID: 1025 RVA: 0x000213AC File Offset: 0x0001F5AC
		private string GetTechLevelLabel(TechLevel t)
		{
			return Enum.GetName(typeof(TechLevel), t);
		}

		// Token: 0x06000402 RID: 1026 RVA: 0x000213D4 File Offset: 0x0001F5D4
		private string GetStuffCategoryLabel(StuffCategoryDef stuffCategoryDef)
		{
			return stuffCategoryDef.label;
		}

		// Token: 0x06000403 RID: 1027 RVA: 0x000213EC File Offset: 0x0001F5EC
		private string GetItemLabel(ThingDef thingDef)
		{
			string label = thingDef.label;
			return (label != null) ? label.CapitalizeFirst() : null;
		}

		// Token: 0x06000404 RID: 1028 RVA: 0x00021410 File Offset: 0x0001F610
		private string GetItemTooltip(ThingDef thingDef)
		{
			return thingDef.GetModName() + "\n" + thingDef.description;
		}

		// Token: 0x06000405 RID: 1029 RVA: 0x00021438 File Offset: 0x0001F638
		private bool IsSameItem(ThingDef a, ThingDef b)
		{
			return a == b;
		}

		// Token: 0x06000406 RID: 1030 RVA: 0x00021450 File Offset: 0x0001F650
		public DialogItemry(DialogCapsuleUI _CapsuleUI = null, Thing toSelect = null)
		{
			this.idxList = 5;
			bool flag = toSelect != null;
			if (flag)
			{
				SearchTool.ClearSearch(this.idxList);
			}
			this.search = SearchTool.Update(this.idxList);
			this.CapsuleUI = _CapsuleUI;
			this.scrollPos = default(Vector2);
			this.scrollPosParam = default(Vector2);
			PlacingTool.rotation = Rot4.South;
			this.lOfModnames = CEditor.API.ListOf<string>(EType.ModsThingDef);
			this.lOfItems = new List<ThingDef>();
			this.lOfThingCategories = new List<ThingCategoryDef>();
			this.lOfFilter = new List<string>();
			this.lOfFilter.Add(Label.ALL);
			this.lOfFilter.Add(ThingCategory.Item.ToString());
			this.lOfFilter.Add(ThingCategory.Building.ToString());
			this.lOfFilter.Add("Mineable");
			this.lOfFilter.Add(ThingCategory.Plant.ToString());
			this.lOfFilter.Add(ThingCategory.Filth.ToString());
			this.lOfFilter.Add(ThingCategory.Gas.ToString());
			this.lOfFilter.Add(ThingCategory.Attachment.ToString());
			this.lOfFilter.Add(ThingCategory.Ethereal.ToString());
			this.lOfFilter.Add(ThingCategory.PsychicEmitter.ToString());
			this.lOfQuality = new List<int>();
			foreach (QualityCategory item in QualityUtility.AllQualityCategories)
			{
				this.lOfQuality.Add((int)item);
			}
			this.lCopyStat = new List<StatModifier>();
			this.lCopyEquipStat = new List<StatModifier>();
			this.lCopyStuff = new List<StuffCategoryDef>();
			this.lCopyCost = new List<ThingDefCountClass>();
			this.iChangeTick0 = 0;
			this.iTick_AllowRemoveStat = 0;
			this.maxSliderVal = ((CEditor.API != null) ? CEditor.API.MaxSliderVal : 100);
			this.previousColor = Color.white;
			this.bAllowRemove = false;
			this.bDoOnce = true;
			this.toggleText = ">>";
			this.tempName = "";
			this.FGetStuffCategoryLabel = new Func<StuffCategoryDef, string>(this.GetStuffCategoryLabel);
			this.FGetTechLevelLabel = new Func<TechLevel, string>(this.GetTechLevelLabel);
			this.FGetTradeabilityLabel = new Func<Tradeability, string>(this.GetTradeabilityLabel);
			this.FGetStatLabel = new Func<StatDef, string>(this.GetStatLabel);
			this.FItemLabel = new Func<ThingDef, string>(this.GetItemLabel);
			this.FItemTooltip = new Func<ThingDef, string>(this.GetItemTooltip);
			this.FItemComparator = new Func<ThingDef, ThingDef, bool>(this.IsSameItem);
			this.FGetThingStyleDefLabel = new Func<ThingStyleDef, string>(this.GetThingStyleDefLabel);
			this.SELECTED = Selected.ByThingDef(null);
			this.UpdateLTC();
			this.search.ofilter1 = ((toSelect != null) ? toSelect.def.FirstThingCategory : this.search.ofilter1);
			this.search.filter2 = this.search.SelectedFilter2;
			this.lOfItems = ThingTool.ListOfItems((ThingCategoryDef)this.search.ofilter1, this.search.modName);
			this.ASelectItemFromSelector(toSelect);
			this.firstPreselection = (toSelect != null);
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x06000407 RID: 1031 RVA: 0x00021824 File Offset: 0x0001FA24
		public override void DoWindowContents(Rect inRect)
		{
			float num = this.InitialSize.y - 104f;
			this.MAXW = this.W_APPA + this.W_PARAMS + 10f;
			this.windowRect.width = (CEditor.IsExtendedUI ? this.MAXW : (this.W_APPA + 36f));
			this.toggleText = (CEditor.IsExtendedUI ? "<<" : ">>");
			bool flag = this.bDoOnce;
			if (flag)
			{
				this.bDoOnce = false;
				this.windowRect.position = new Vector2((float)(CEditor.API.EditorPosX + 70), this.windowRect.position.y);
			}
			float num2 = 0f;
			float num3 = 0f;
			this.SELECTED.tooltip = Label.PRICE + this.SELECTED.buyPrice.ToString();
			this.DrawEditLabel(num3, num2, this.W_APPA, 30f);
			num2 += 30f;
			this.DrawTexture(num3 + 50f, ref num2, this.W_APPA, this.W_APPA);
			this.DrawDropdown(num3, num2, this.W_APPA, 30f);
			num2 += 92f;
			this.DrawList(num3, num2, this.W_APPA, num - num2 + 20f);
			num2 += num - num2 + 30f;
			this.DrawParameter(this.MAXW - this.W_PARAMS, 0f, this.W_PARAMS - 40f, num + 20f);
			this.DrawLowerButtons(num3, num2, this.MAXW - this.W_APPA - 40f, 40f);
			bool flag2 = !this.bInPlacingMode;
			if (flag2)
			{
				SZWidgets.ButtonText(240f, this.InitialSize.y - 75f, 30f, 40f, this.toggleText, new Action(this.AToggleStats), "");
			}
			bool flag3 = this.SELECTED != null && this.SELECTED.thingDef != null && Widgets.ButtonText(new Rect(140f, this.InitialSize.y - 75f, (float)(this.bInPlacingMode ? 130 : 100), 40f), "OK".Translate(), true, true, true);
			if (flag3)
			{
				this.DoAndClose();
			}
			bool flag4 = !CEditor.InStartingScreen;
			if (flag4)
			{
				bool flag5 = !this.bInPlacingMode;
				if (flag5)
				{
					SZWidgets.ButtonImage(new Rect(this.W_APPA - 30f, 0f, 30f, 30f), "bnone", new Action(this.APlacingMode), "activate placing mode");
				}
				else
				{
					SZWidgets.ButtonImage(new Rect(this.W_APPA - 30f, 0f, 30f, 30f), "brotate", new Action(this.ARotate), "rotation" + TextureTool.Rot4ToString(PlacingTool.rotation));
				}
			}
			bool flag6 = this.bInPlacingMode;
			if (flag6)
			{
				SZWidgets.ButtonText(new Rect(0f, this.InitialSize.y - 75f, 130f, 40f), "DESTROY", new Action(this.ADestroy), "");
			}
		}

		// Token: 0x06000408 RID: 1032 RVA: 0x00021B8F File Offset: 0x0001FD8F
		private void ADestroy()
		{
			PlacingTool.Destroy();
		}

		// Token: 0x06000409 RID: 1033 RVA: 0x00021B98 File Offset: 0x0001FD98
		private void ARotate()
		{
			bool flag = PlacingTool.rotation == Rot4.North;
			if (flag)
			{
				PlacingTool.rotation = Rot4.East;
			}
			else
			{
				bool flag2 = PlacingTool.rotation == Rot4.East;
				if (flag2)
				{
					PlacingTool.rotation = Rot4.South;
				}
				else
				{
					bool flag3 = PlacingTool.rotation == Rot4.South;
					if (flag3)
					{
						PlacingTool.rotation = Rot4.West;
					}
					else
					{
						bool flag4 = PlacingTool.rotation == Rot4.West;
						if (flag4)
						{
							PlacingTool.rotation = Rot4.North;
						}
						else
						{
							PlacingTool.rotation = Rot4.North;
						}
					}
				}
			}
		}

		// Token: 0x0600040A RID: 1034 RVA: 0x00021C2C File Offset: 0x0001FE2C
		private void APlacingMode()
		{
			this.doWindowBackground = true;
			this.absorbInputAroundWindow = false;
			this.preventCameraMotion = false;
			this.preventDrawTutor = false;
			this.draggable = false;
			this.windowRect.position = new Vector2(0f, 0f);
			PlacingTool.rotation = Rot4.North;
			this.bInPlacingMode = true;
			CEditor.API.CloseEditor();
		}

		// Token: 0x0600040B RID: 1035 RVA: 0x00021C94 File Offset: 0x0001FE94
		public override void OnAcceptKeyPressed()
		{
			base.OnAcceptKeyPressed();
			this.DoAndClose();
		}

		// Token: 0x0600040C RID: 1036 RVA: 0x00021CA8 File Offset: 0x0001FEA8
		public void DoAndClose()
		{
			bool flag = this.SELECTED.Stuff != null && !this.SELECTED.thingDef.MadeFromStuff;
			if (flag)
			{
				this.SELECTED.Stuff = null;
			}
			if (this.bInPlacingMode)
			{
				PlacingTool.PlaceInCustomPosition(this.SELECTED);
			}
			else
			{
				if (this.SELECTED.thingDef != null)
				{
					SearchTool.Save(this.idxList);
					bool flag4 = this.CapsuleUI != null;
					if (flag4)
					{
						this.CapsuleUI.AddThingDef(this.SELECTED.thingDef, this.SELECTED.Stuff, this.SELECTED.Style, this.SELECTED.stackVal);
					}
					else
					{
						CEditor.API.Pawn.Reinvent(this.SELECTED, 1);
					}
					this.Close(true);
				}
			}
		}

		// Token: 0x0600040D RID: 1037 RVA: 0x00021D90 File Offset: 0x0001FF90
		private void DrawDropdown(float x, float y, float w, float h)
		{
			Rect rect = new Rect(x, y, w, h);
			SZWidgets.FloatMenuOnButtonText<string>(rect, this.search.SelectedModName, this.lOfModnames, (string s) => s ?? Label.ALL, new Action<string>(this.ASelectedModName), "");
			Rect rect2 = new Rect(rect);
			rect2.y += 30f;
			SZWidgets.FloatMenuOnButtonText<ThingCategoryDef>(rect2, this.GetLaberForSelectedCategory((ThingCategoryDef)this.search.ofilter1), this.lOfThingCategories, (ThingCategoryDef tcd) => this.GetLaberForSelectedCategory(tcd), new Action<ThingCategoryDef>(this.ASelectThingCategory), "");
			Rect rect3 = new Rect(rect2);
			rect3.y += 30f;
			SZWidgets.FloatMenuOnButtonText<string>(rect3, this.search.filter2, this.lOfFilter, (string s) => s, new Action<string>(this.ASelectFilter));
		}

		// Token: 0x0600040E RID: 1038 RVA: 0x00021EAD File Offset: 0x000200AD
		private void ASelectedModName(string val)
		{
			this.search.modName = val;
			this.ASelectThingCategory(null);
		}

		// Token: 0x0600040F RID: 1039 RVA: 0x00021EC4 File Offset: 0x000200C4
		private void UpdateLTC()
		{
			this.lOfThingCategories = new List<ThingCategoryDef>();
			this.lOfThingCategories.Add(null);
			foreach (ThingCategoryDef thingCategoryDef in DefDatabase<ThingCategoryDef>.AllDefs)
			{
				bool flag = thingCategoryDef != null && thingCategoryDef.defName != null && thingCategoryDef.childThingDefs != null && thingCategoryDef.childThingDefs.Count > 0 && !thingCategoryDef.defName.Contains("Corpse") && thingCategoryDef.defName != "Animals";
				if (flag)
				{
					this.lOfThingCategories.Add(thingCategoryDef);
				}
			}
		}

		// Token: 0x06000410 RID: 1040 RVA: 0x00021F80 File Offset: 0x00020180
		private void ASelectThingCategory(ThingCategoryDef val)
		{
			this.search.ofilter1 = val;
			this.ASelectFilter(Label.ALL);
		}

		// Token: 0x06000411 RID: 1041 RVA: 0x00021F9C File Offset: 0x0002019C
		private void ASelectFilter(string val)
		{
			this.search.filter2 = val;
			this.lOfItems = ThingTool.ListOfItems((ThingCategoryDef)this.search.ofilter1, this.search.modName);
      List<ThingDef> hashSet = new List<ThingDef>();
			bool flag = this.search.filter2 != Label.ALL;
			if (flag)
			{
				bool flag2 = this.search.filter2 == "Mineable";
				if (flag2)
				{
					foreach (ThingDef thingDef in this.lOfItems)
					{
						bool flag3 = thingDef.IsMineableRock() || thingDef.IsMineableMineral();
						if (flag3)
						{
							hashSet.Add(thingDef);
						}
					}
				}
				else
				{
					ThingCategory thingCategory = (ThingCategory)Enum.Parse(typeof(ThingCategory), this.search.filter2);
					foreach (ThingDef thingDef2 in this.lOfItems)
					{
						bool flag4 = thingDef2.category == thingCategory;
						if (flag4)
						{
							hashSet.Add(thingDef2);
						}
					}
				}
				this.lOfItems = hashSet;
			}
			this.SELECTED.Init(this.lOfItems.FirstOrFallback(null), true, null, null);
		}

		// Token: 0x06000412 RID: 1042 RVA: 0x00022124 File Offset: 0x00020324
		private void DrawEditLabel(float x, float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				Text.Font = GameFont.Medium;
				Rect rect = new Rect(x, y, w - 40f, h);
				bool flag2 = this.iChangeTick0 <= 0;
				if (flag2)
				{
					SZWidgets.Label(rect, this.SELECTED.thingDef.label, delegate()
					{
						this.iChangeTick0 = 300;
					}, "");
				}
				else
				{
					this.tempName = Widgets.TextField(rect, this.SELECTED.thingDef.label);
					bool flag3 = !this.SELECTED.thingDef.label.Equals(this.tempName);
					if (flag3)
					{
						this.SELECTED.thingDef.label = this.tempName;
						this.iChangeTick0 = 300;
					}
					this.iChangeTick0--;
				}
			}
		}

		// Token: 0x06000413 RID: 1043 RVA: 0x00022210 File Offset: 0x00020410
		private void DrawList(float x, float y, float w, float h)
		{
			SZWidgets.ListView<ThingDef>(x, y, this.W_APPA, h, this.lOfItems, this.FItemLabel, this.FItemTooltip, this.FItemComparator, ref this.SELECTED.thingDef, ref this.scrollPos, false, new Action<ThingDef>(this.AItemSelected), true, true, true, false);
		}

		// Token: 0x06000414 RID: 1044 RVA: 0x00022268 File Offset: 0x00020468
		private void DrawLowerButtons(float x, float y, float w, float h)
		{
			float num = (float)((int)(w / 3f));
			x += this.W_APPA;
			SZWidgets.ButtonText(x, y, num, h, Label.RESET, new Action(this.AResetOne), "");
			x += num;
			SZWidgets.ButtonText(x, y, num, h, Label.RESETALL, new Action(this.AResetAll), "");
			x += num;
			SZWidgets.ButtonText(x, y, num, h, Label.SAVE, new Action(this.ASave), "");
		}

		// Token: 0x06000415 RID: 1045 RVA: 0x000222F8 File Offset: 0x000204F8
		private Texture2D GetTexture()
		{
			bool flag = this.SELECTED.thingDef.IsTurret();
			Texture2D result;
			if (flag)
			{
				result = this.SELECTED.thingDef.uiIcon;
			}
			else
			{
				bool flag2 = this.SELECTED.thingDef.mineable && this.SELECTED.Stuff == null;
				if (flag2)
				{
					GUI.color = this.SELECTED.thingDef.uiIconColor;
					result = this.SELECTED.thingDef.GetTexture(this.SELECTED.stackVal, null, default(Rot4));
				}
				else
				{
					result = this.SELECTED.thingDef.GetTexture(this.SELECTED.stackVal, this.SELECTED.Style, PlacingTool.rotation);
				}
			}
			return result;
		}

		// Token: 0x06000416 RID: 1046 RVA: 0x000223CC File Offset: 0x000205CC
		private void DrawTexture(float x, ref float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				GUI.color = this.SELECTED.DrawColor;
				int stackCount = (this.SELECTED.stackVal > this.SELECTED.thingDef.stackLimit) ? this.SELECTED.thingDef.stackLimit : this.SELECTED.stackVal;
				Rot4 rot = this.bInPlacingMode ? PlacingTool.rotation : Rot4.North;
				Texture2D texture = this.SELECTED.thingDef.GetTexture(stackCount, this.SELECTED.Style, rot);
				Rect rect = new Rect(39f, y, 192f, 192f);
				bool flag2 = this.SELECTED.thingDef.graphicData != null && (this.SELECTED.thingDef.graphicData.graphicClass == typeof(Graphic_Single) || this.SELECTED.thingDef.graphicData.graphicClass == typeof(Graphic_Random));
				if (flag2)
				{
					float angle = (rot == Rot4.South) ? 180f : ((rot == Rot4.West) ? 270f : ((rot == Rot4.East) ? 90f : 0f));
					Matrix4x4 matrix = GUI.matrix;
					Vector2 center = rect.center;
					center.x = (this.windowRect.position.x + 192f) / 2f + 39f;
					center.y = (this.windowRect.position.y + 192f) / 2f + y;
					UI.RotateAroundPivot(angle, center);
					GUI.DrawTexture(rect, texture, ScaleMode.ScaleToFit, true);
					GUI.matrix = matrix;
				}
				else
				{
					GUI.DrawTexture(rect, texture, ScaleMode.ScaleToFit);
				}
				y += 200f;
				y = SZWidgets.NavSelectorQuality(new Rect(0f, y, w, 25f), this.SELECTED, this.lOfQuality);
				y = SZWidgets.NavSelectorStuff(new Rect(0f, y, w, 25f), this.SELECTED);
				y = SZWidgets.NavSelectorStyle(new Rect(0f, y, w, 25f), this.SELECTED);
				y = SZWidgets.NavSelectorCount(new Rect(0f, y, w, 25f), this.SELECTED, this.maxSliderVal);
				TooltipHandler.TipRegion(rect, this.SELECTED.tip);
				bool flag3 = !CEditor.InStartingScreen;
				if (flag3)
				{
					SZWidgets.ButtonInvisible(rect, new Action(this.ABuyItem), "");
				}
			}
		}

		// Token: 0x06000417 RID: 1047 RVA: 0x0002268C File Offset: 0x0002088C
		private void DrawParameter(float x, float y, float w, float h)
		{
			bool flag = !CEditor.IsExtendedUI || this.SELECTED.thingDef == null;
			if (!flag)
			{
				float xOff = w - 60f;
				float xOff2 = w - 85f;
				float xOff3 = w - 110f;
				float xOff4 = w - 130f;
				this.CalcHSCROLL();
				bool flag2 = this.iTick_AllowRemoveStat > 0;
				if (flag2)
				{
					this.iTick_AllowRemoveStat--;
				}
				this.bAllowRemove = (this.iTick_AllowRemoveStat > 0);
				Rect outRect = new Rect(x, y, w, h);
				Rect rect = new Rect(0f, 0f, outRect.width - 16f, this.HSCROLLPARAM);
				Widgets.BeginScrollView(outRect, ref this.scrollPosParam, rect, true);
				Rect rect2 = rect.ContractedBy(4f);
				rect2.y -= 4f;
				rect2.height = this.HSCROLLPARAM;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(rect2);
				listing_X.verticalSpacing = 30f;
				Text.Font = GameFont.Medium;
				listing_X.Label(0f, 0f, 300f, 30f, Label.STATS);
				Text.Font = GameFont.Small;
				listing_X.ButtonImage(xOff, 5f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.ADialogAddStat));
				listing_X.ButtonImage(xOff2, 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
				listing_X.ButtonImage(xOff3, 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyStat));
				bool flag3 = !this.lCopyStat.NullOrEmpty<StatModifier>();
				if (flag3)
				{
					listing_X.ButtonImage(xOff4, 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteStat));
				}
				listing_X.Gap(30f);
				bool flag4 = this.SELECTED.thingDef.statBases != null;
				if (flag4)
				{
					try
					{
						foreach (StatModifier statModifier in this.SELECTED.thingDef.statBases)
						{
							bool flag5 = listing_X.ListviewSM(400f, statModifier, statModifier.stat.defName == this.SELECTED.statDefName, this.bAllowRemove, new Action<string>(this.ARemoveStat));
							if (flag5)
							{
								this.SELECTED.statDefName = ((this.SELECTED.statDefName != null) ? null : statModifier.stat.defName);
							}
						}
					}
					catch
					{
					}
				}
				listing_X.Gap(25f);
				Text.Font = GameFont.Medium;
				listing_X.Label(0f, 0f, 300f, 30f, Label.EQUIPSTATS);
				Text.Font = GameFont.Small;
				listing_X.ButtonImage(xOff, 5f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.ADialogAddEquipStat));
				listing_X.ButtonImage(xOff2, 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
				listing_X.ButtonImage(xOff3, 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyEquipStat));
				bool flag6 = !this.lCopyEquipStat.NullOrEmpty<StatModifier>();
				if (flag6)
				{
					listing_X.ButtonImage(xOff4, 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteEquipStat));
				}
				listing_X.Gap(30f);
				bool flag7 = this.SELECTED.thingDef.equippedStatOffsets != null;
				if (flag7)
				{
					try
					{
						foreach (StatModifier statModifier2 in this.SELECTED.thingDef.equippedStatOffsets)
						{
							bool flag8 = listing_X.ListviewSM(400f, statModifier2, statModifier2.stat.defName == this.SELECTED.equipStatDefName, this.bAllowRemove, new Action<string>(this.ARemoveEquipStat));
							if (flag8)
							{
								this.SELECTED.equipStatDefName = ((this.SELECTED.equipStatDefName != null) ? null : statModifier2.stat.defName);
							}
						}
					}
					catch
					{
					}
				}
				listing_X.Gap(25f);
				Text.Font = GameFont.Medium;
				listing_X.Label(0f, 0f, 300f, 30f, Label.STUFFPROPS);
				Text.Font = GameFont.Small;
				listing_X.FloatMenuOnButtonImage<StuffCategoryDef>(xOff, 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<StuffCategoryDef>(EType.StuffCategory), this.FGetStuffCategoryLabel, new Action<StuffCategoryDef>(this.AAddStuffCategory));
				listing_X.ButtonImage(xOff2, 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
				listing_X.ButtonImage(xOff3, 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyStuff));
				bool flag9 = !this.lCopyStuff.NullOrEmpty<StuffCategoryDef>();
				if (flag9)
				{
					listing_X.ButtonImage(xOff4, 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteStuff));
				}
				listing_X.Gap(30f);
				bool madeFromStuff = this.SELECTED.thingDef.MadeFromStuff;
				if (madeFromStuff)
				{
					listing_X.LabelSimple(Label.COSTSTUFFCOUNT + this.SELECTED.thingDef.costStuffCount.ToString(), 0f, listing_X.CurY, rect.width, 25f, null);
					listing_X.Gap(20f);
					this.SELECTED.thingDef.costStuffCount = (int)listing_X.Slider((float)this.SELECTED.thingDef.costStuffCount, 1f, 200f, 300f);
					bool flag10 = this.SELECTED.OldStuffCount != this.SELECTED.thingDef.costStuffCount;
					if (flag10)
					{
						this.SELECTED.OldStuffCount = this.SELECTED.thingDef.costStuffCount;
						this.SELECTED.thingDef.UpdateCostStuffCount();
					}
					try
					{
						foreach (StuffCategoryDef stuffCategoryDef in this.SELECTED.thingDef.stuffCategories)
						{
							listing_X.Listview(400f, stuffCategoryDef.defName, stuffCategoryDef.label, stuffCategoryDef.description, this.bAllowRemove, new Action<string>(this.ARemoveStuffCategory));
						}
					}
					catch
					{
					}
				}
				listing_X.Gap(25f);
				Text.Font = GameFont.Medium;
				listing_X.Label(0f, 0f, 300f, 30f, Label.COSTS);
				Text.Font = GameFont.Small;
				listing_X.FloatMenuOnButtonImage<ThingDef>(xOff, 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<ThingDef>(EType.CostItems), this.FItemLabel, new Action<ThingDef>(this.AAddCosts));
				listing_X.ButtonImage(xOff2, 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
				listing_X.ButtonImage(xOff3, 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyCosts));
				bool flag11 = !this.lCopyCost.NullOrEmpty<ThingDefCountClass>();
				if (flag11)
				{
					listing_X.ButtonImage(xOff4, 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteCosts));
				}
				listing_X.Gap(30f);
				bool flag12 = this.SELECTED.thingDef.costList != null;
				if (flag12)
				{
					try
					{
						foreach (ThingDefCountClass thingDefCountClass in this.SELECTED.thingDef.costList)
						{
							bool flag13 = listing_X.ListviewTDC(400f, thingDefCountClass, thingDefCountClass.thingDef.defName == this.SELECTED.tdcDefName, this.bAllowRemove, new Action<string>(this.ARemoveCosts), this.SELECTED.thingDef);
							if (flag13)
							{
								this.SELECTED.tdcDefName = ((this.SELECTED.tdcDefName != null) ? null : thingDefCountClass.thingDef.defName);
							}
						}
					}
					catch
					{
					}
				}
				listing_X.Gap(25f);
				Text.Font = GameFont.Medium;
				listing_X.Label(Label.EXTRAS, 300f, 0f, -1f, null);
				Text.Font = GameFont.Small;
				listing_X.FloatMenuButtonWithLabel<TechLevel>(Label.TECHLEVEL, 100f, 200f, Enum.GetName(typeof(TechLevel), this.SELECTED.thingDef.techLevel), CEditor.API.ListOf<TechLevel>(EType.TechLevel), this.FGetTechLevelLabel, new Action<TechLevel>(this.ASetTechLevel));
				listing_X.FloatMenuButtonWithLabel<Tradeability>(Label.TRADEABILITY, 100f, 200f, Enum.GetName(typeof(Tradeability), this.SELECTED.thingDef.tradeability), CEditor.API.ListOf<Tradeability>(EType.Tradeability), this.FGetTradeabilityLabel, new Action<Tradeability>(this.ASetTradeability));
				listing_X.CheckboxLabeled(Label.STEALABLE, 2f, 200f, ref this.SELECTED.thingDef.stealable, null);
				Text.Font = GameFont.Medium;
				listing_X.GapLine(12f);
				listing_X.Label(Label.GRAPHICS, 300f, 0f, -1f, null);
				Text.Font = GameFont.Tiny;
				Text.WordWrap = false;
				listing_X.Label(Label.TEXPATH + this.SELECTED.thingDef.graphicData.texPath, 1000f, 0f, -1f, null);
				listing_X.Label(Label.DRAWSIZE + this.SELECTED.thingDef.graphicData.drawSize.ToString(), 1000f, 0f, -1f, null);
				Text.WordWrap = true;
				Text.Font = GameFont.Small;
				listing_X.End();
				Widgets.EndScrollView();
			}
		}

		// Token: 0x06000418 RID: 1048 RVA: 0x000231F8 File Offset: 0x000213F8
		private void AToggleStats()
		{
			CEditor.IsExtendedUI = !CEditor.IsExtendedUI;
		}

		// Token: 0x06000419 RID: 1049 RVA: 0x0002320C File Offset: 0x0002140C
		private void ASelectItemFromSelector(Thing toSelect = null)
		{
			bool flag = !this.lOfItems.EnumerableNullOrEmpty<ThingDef>();
			if (flag)
			{
				bool flag2 = toSelect == null;
				if (flag2)
				{
					this.SELECTED.Init(this.lOfItems.FirstOrFallback(null), true, null, null);
				}
				else
				{
					this.SELECTED.Init(toSelect.def, false, toSelect, null);
				}
			}
		}

		// Token: 0x0600041A RID: 1050 RVA: 0x00023268 File Offset: 0x00021468
		private void AItemSelected(ThingDef thingDef)
		{
			bool flag = this.firstPreselection;
			if (flag)
			{
				this.firstPreselection = false;
			}
			else
			{
				bool flag2 = thingDef != null;
				if (flag2)
				{
					bool flag3 = !CEditor.InStartingScreen && thingDef.soundInteract != null;
					if (flag3)
					{
						thingDef.soundInteract.PlayOneShot(new TargetInfo(UI.MouseMapPosition().ToIntVec3(), Find.CurrentMap, false));
					}
					this.SELECTED.Init(thingDef, true, null, null);
				}
			}
		}

		// Token: 0x0600041B RID: 1051 RVA: 0x000232E4 File Offset: 0x000214E4
		private void AResetAll()
		{
			foreach (PresetItem presetItem in CEditor.API.Get<Dictionary<string, PresetItem>>(EType.ItemPreset).Values)
			{
				presetItem.UpdateItem();
			}
			CEditor.API.SetCustom(OptionS.CUSTOMITEM, "", "");
			this.SELECTED.Init(this.lOfItems.FirstOrFallback(null), true, null, null);
			MessageTool.Show("reset all items done", null);
		}

		// Token: 0x0600041C RID: 1052 RVA: 0x00023384 File Offset: 0x00021584
		private void AResetOne()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				PresetItem.ResetToDefault(this.SELECTED.thingDef.defName);
				this.SELECTED.Init(this.lOfItems.FirstOrFallback(null), true, null, null);
				MessageTool.Show("reset item " + this.SELECTED.thingDef.defName + " done", null);
			}
		}

		// Token: 0x0600041D RID: 1053 RVA: 0x00023400 File Offset: 0x00021600
		private void ASave()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				PresetItem presetItem = new PresetItem(this.SELECTED.thingDef);
				presetItem.UpdateItem();
				presetItem.SaveCustom();
				MessageTool.Show(presetItem.label + " " + Label.SETTINGSSAVED, null);
			}
		}

		// Token: 0x0600041E RID: 1054 RVA: 0x00023460 File Offset: 0x00021660
		private void CalcHSCROLL()
		{
			int num = this.SELECTED.thingDef.stuffCategories.CountAllowNull<StuffCategoryDef>();
			int num2 = this.SELECTED.thingDef.statBases.CountAllowNull<StatModifier>();
			int num3 = this.SELECTED.thingDef.equippedStatOffsets.CountAllowNull<StatModifier>();
			int num4 = num + num2 + num3;
			this.HSCROLLPARAM = (float)(900 + num4 * 28);
		}

		// Token: 0x0600041F RID: 1055 RVA: 0x000234C8 File Offset: 0x000216C8
		private void AAddStat(StatDef stat)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddStat(stat, 0f);
				bool isApparel = this.SELECTED.thingDef.IsApparel;
				if (isApparel)
				{
					ApparelTool.FixForBelts(this.SELECTED.thingDef, stat);
				}
			}
		}

		// Token: 0x06000420 RID: 1056 RVA: 0x00023528 File Offset: 0x00021728
		private void ADialogAddStat()
		{
			List<StatDef> list = CEditor.API.ListOfStatDef(null, false, false);
			for (int i = 0; i < list.Count; i++)
			{
				bool flag = this.SELECTED.thingDef.HasStat(list[i]);
				if (flag)
				{
					list.Remove(list[i]);
					i--;
				}
			}
			SZWidgets.FloatMenuOnRect<StatDef>(list, this.FGetStatLabel, new Action<StatDef>(this.AAddStat));
		}

		// Token: 0x06000421 RID: 1057 RVA: 0x000235A4 File Offset: 0x000217A4
		private void AAllowRemoveStat()
		{
			bool flag = this.iTick_AllowRemoveStat > 0;
			if (flag)
			{
				this.iTick_AllowRemoveStat = 0;
			}
			else
			{
				this.iTick_AllowRemoveStat = 1000;
			}
		}

		// Token: 0x06000422 RID: 1058 RVA: 0x000235D4 File Offset: 0x000217D4
		private void ACopyStat()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.statBases.NullOrEmpty<StatModifier>();
			if (flag)
			{
				this.lCopyStat = new List<StatModifier>();
				foreach (StatModifier item in this.SELECTED.thingDef.statBases)
				{
					this.lCopyStat.Add(item);
				}
			}
		}

		// Token: 0x06000423 RID: 1059 RVA: 0x00023674 File Offset: 0x00021874
		private void APasteStat()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteStats(this.lCopyStat);
			}
		}

		// Token: 0x06000424 RID: 1060 RVA: 0x000236AC File Offset: 0x000218AC
		private void ARemoveStat(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveStat(DefTool.StatDef(defName));
			}
		}

		// Token: 0x06000425 RID: 1061 RVA: 0x000236E4 File Offset: 0x000218E4
		private void AAddEquipStat(StatDef stat)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddEquipStat(stat, 0f);
				bool isApparel = this.SELECTED.thingDef.IsApparel;
				if (isApparel)
				{
					ApparelTool.FixForBelts(this.SELECTED.thingDef, stat);
				}
			}
		}

		// Token: 0x06000426 RID: 1062 RVA: 0x00023744 File Offset: 0x00021944
		private void ADialogAddEquipStat()
		{
			List<StatDef> list = CEditor.API.ListOfStatDef(null, false, true);
			for (int i = 0; i < list.Count; i++)
			{
				bool flag = this.SELECTED.thingDef.HasStat(list[i]);
				if (flag)
				{
					list.Remove(list[i]);
					i--;
				}
			}
			SZWidgets.FloatMenuOnRect<StatDef>(list, this.FGetStatLabel, new Action<StatDef>(this.AAddEquipStat));
		}

		// Token: 0x06000427 RID: 1063 RVA: 0x000237C0 File Offset: 0x000219C0
		private void ACopyEquipStat()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.equippedStatOffsets.NullOrEmpty<StatModifier>();
			if (flag)
			{
				this.lCopyEquipStat = new List<StatModifier>();
				foreach (StatModifier item in this.SELECTED.thingDef.equippedStatOffsets)
				{
					this.lCopyEquipStat.Add(item);
				}
			}
		}

		// Token: 0x06000428 RID: 1064 RVA: 0x00023860 File Offset: 0x00021A60
		private void APasteEquipStat()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteEquipStats(this.lCopyEquipStat);
			}
		}

		// Token: 0x06000429 RID: 1065 RVA: 0x00023898 File Offset: 0x00021A98
		private void ARemoveEquipStat(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveEquipStat(DefTool.StatDef(defName));
			}
		}

		// Token: 0x0600042A RID: 1066 RVA: 0x000238D0 File Offset: 0x00021AD0
		private void AAddCosts(ThingDef thingDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddCost(thingDef, 0);
			}
		}

		// Token: 0x0600042B RID: 1067 RVA: 0x00023904 File Offset: 0x00021B04
		private void ACopyCosts()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.costList.NullOrEmpty<ThingDefCountClass>();
			if (flag)
			{
				this.lCopyCost = new List<ThingDefCountClass>();
				foreach (ThingDefCountClass item in this.SELECTED.thingDef.costList)
				{
					this.lCopyCost.Add(item);
				}
			}
		}

		// Token: 0x0600042C RID: 1068 RVA: 0x000239A4 File Offset: 0x00021BA4
		private void APasteCosts()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteCost(this.lCopyCost);
			}
		}

		// Token: 0x0600042D RID: 1069 RVA: 0x000239DC File Offset: 0x00021BDC
		private void ARemoveCosts(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveCost(DefTool.ThingDef(defName));
			}
		}

		// Token: 0x0600042E RID: 1070 RVA: 0x00023A14 File Offset: 0x00021C14
		private void AAddStuffCategory(StuffCategoryDef stuffCategoryDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddStuff(stuffCategoryDef);
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x0600042F RID: 1071 RVA: 0x00023A54 File Offset: 0x00021C54
		private void ACopyStuff()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.stuffCategories.NullOrEmpty<StuffCategoryDef>();
			if (flag)
			{
				this.lCopyStuff = new List<StuffCategoryDef>();
				foreach (StuffCategoryDef item in this.SELECTED.thingDef.stuffCategories)
				{
					this.lCopyStuff.Add(item);
				}
			}
		}

		// Token: 0x06000430 RID: 1072 RVA: 0x00023AF4 File Offset: 0x00021CF4
		private void APasteStuff()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteStuff(this.lCopyStuff);
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x06000431 RID: 1073 RVA: 0x00023B3C File Offset: 0x00021D3C
		private void ARemoveStuffCategory(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveStuff(DefTool.StuffCategoryDef(defName));
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x06000432 RID: 1074 RVA: 0x00023B84 File Offset: 0x00021D84
		private void ASetTechLevel(TechLevel techLevel)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.techLevel = techLevel;
				this.SELECTED.thingDef.ResolveReferences();
			}
		}

		// Token: 0x06000433 RID: 1075 RVA: 0x00023BC8 File Offset: 0x00021DC8
		private void ASetTradeability(Tradeability tradeability)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.tradeability = tradeability;
				this.SELECTED.thingDef.ResolveReferences();
			}
		}

		// Token: 0x06000434 RID: 1076 RVA: 0x00023C0C File Offset: 0x00021E0C
		private void ABuyItem()
		{
			ThingTool.BeginBuyItem(this.SELECTED);
			CEditor.API.CloseEditor();
			this.Close(true);
		}

		// Token: 0x04000206 RID: 518
		private DialogCapsuleUI CapsuleUI;

		// Token: 0x04000207 RID: 519
		private Vector2 scrollPos;

		// Token: 0x04000208 RID: 520
		private Vector2 scrollPosParam;

		// Token: 0x04000209 RID: 521
		private Selected SELECTED;

		// Token: 0x0400020A RID: 522
		private Color previousColor;

		// Token: 0x0400020B RID: 523
		private List<string> lOfModnames;

		// Token: 0x0400020C RID: 524
		private List<ThingCategoryDef> lOfThingCategories;

		// Token: 0x0400020D RID: 525
		private List<string> lOfFilter;

		// Token: 0x0400020E RID: 526
		private List<ThingDef> lOfItems;

		// Token: 0x0400020F RID: 527
		private List<int> lOfQuality;

		// Token: 0x04000210 RID: 528
		private List<StatModifier> lCopyStat;

		// Token: 0x04000211 RID: 529
		private List<StatModifier> lCopyEquipStat;

		// Token: 0x04000212 RID: 530
		private List<StuffCategoryDef> lCopyStuff;

		// Token: 0x04000213 RID: 531
		private List<ThingDefCountClass> lCopyCost;

		// Token: 0x04000214 RID: 532
		private int iTick_AllowRemoveStat;

		// Token: 0x04000215 RID: 533
		private int iChangeTick0;

		// Token: 0x04000216 RID: 534
		private int idxList;

		// Token: 0x04000217 RID: 535
		private int maxSliderVal;

		// Token: 0x04000218 RID: 536
		private bool bAllowRemove;

		// Token: 0x04000219 RID: 537
		private bool bDoOnce;

		// Token: 0x0400021A RID: 538
		private bool firstPreselection;

		// Token: 0x0400021B RID: 539
		private string toggleText;

		// Token: 0x0400021C RID: 540
		private string tempName;

		// Token: 0x0400021D RID: 541
		private const float WTITLE = 300f;

		// Token: 0x0400021E RID: 542
		private float MAXW = 850f;

		// Token: 0x0400021F RID: 543
		private float W_APPA = 270f;

		// Token: 0x04000220 RID: 544
		private float W_PARAMS = 530f;

		// Token: 0x04000221 RID: 545
		private float HSCROLLPARAM;

		// Token: 0x04000222 RID: 546
		private SearchTool search;

		// Token: 0x04000223 RID: 547
		private const int TEXTURESIZE = 192;

		// Token: 0x04000224 RID: 548
		private Func<StuffCategoryDef, string> FGetStuffCategoryLabel;

		// Token: 0x04000225 RID: 549
		private Func<ThingDef, string> FItemLabel;

		// Token: 0x04000226 RID: 550
		private Func<ThingDef, string> FItemTooltip;

		// Token: 0x04000227 RID: 551
		private Func<ThingDef, ThingDef, bool> FItemComparator;

		// Token: 0x04000228 RID: 552
		private Func<TechLevel, string> FGetTechLevelLabel;

		// Token: 0x04000229 RID: 553
		private Func<Tradeability, string> FGetTradeabilityLabel;

		// Token: 0x0400022A RID: 554
		private Func<StatDef, string> FGetStatLabel;

		// Token: 0x0400022B RID: 555
		private Func<ThingStyleDef, string> FGetThingStyleDefLabel;

		// Token: 0x0400022C RID: 556
		private bool bInPlacingMode = false;
	}
}
