﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000051 RID: 81
	public class DialogRelations : Window
	{
		// Token: 0x17000060 RID: 96
		// (get) Token: 0x06000504 RID: 1284 RVA: 0x000339E8 File Offset: 0x00031BE8
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(700f, 650f);
			}
		}

		// Token: 0x06000505 RID: 1285 RVA: 0x00033A0C File Offset: 0x00031C0C
		public DialogRelations()
		{
			this.scrollPos = default(Vector2);
			this.scrollPos2 = default(Vector2);
			this.dicFaction = FactionTool.GetDicOfFactions(false, false, true);
			this.selectedKey = CEditor.ListName;
			this.lOfPawns = PawnxTool.GetPawnList(this.selectedKey, false, this.dicFaction.GetValue(this.selectedKey));
			this.lOfRelations = new List<PawnRelationDef>();
			foreach (PawnRelationDef pawnRelationDef in DefTool.ListAll<PawnRelationDef>())
			{
				bool flag = !pawnRelationDef.implied;
				if (flag)
				{
					this.lOfRelations.Add(pawnRelationDef);
				}
			}
			this.selectedRelation = this.lOfRelations.FirstOrFallback(null);
			this.selectedPawn = this.lOfPawns.FirstOrFallback(null);
			this.dpr = null;
			this.bImplicite = true;
			this.ASelectFaction(Label.COLONISTS);
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x06000506 RID: 1286 RVA: 0x00033B3C File Offset: 0x00031D3C
		private string GetRelationLabel(Pawn p, PawnRelationDef relationDef = null)
		{
			bool flag = relationDef == null;
			string result;
			if (flag)
			{
				string text = (p.gender == Gender.Female) ? this.selectedRelation.labelFemale : this.selectedRelation.label;
				bool flag2 = string.IsNullOrEmpty(text) || text == "(missing label)";
				if (flag2)
				{
					text = this.selectedRelation.label;
				}
				result = text;
			}
			else
			{
				string text2 = (p.gender == Gender.Female) ? relationDef.labelFemale : relationDef.label;
				bool flag3 = string.IsNullOrEmpty(text2) || text2 == "(missing label)";
				if (flag3)
				{
					text2 = relationDef.label;
				}
				result = text2;
			}
			return result;
		}

		// Token: 0x06000507 RID: 1287 RVA: 0x00033BE8 File Offset: 0x00031DE8
		public override void DoWindowContents(Rect inRect)
		{
			float num = this.InitialSize.x - 40f;
			float h = this.InitialSize.y - 200f;
			float num2 = 0f;
			float num3 = 0f;
			Text.Font = GameFont.Medium;
			Widgets.Label(new Rect(0f, 0f, num, 50f), Label.RELATIONSOF + CEditor.API.Pawn.GetPawnName(false));
			num2 += 30f;
			Text.Font = GameFont.Small;
			Widgets.Label(new Rect(0f, num2, num, 40f), Label.RELATION_INFO);
			num2 += 40f;
			Widgets.CheckboxLabeled(new Rect(0f, num2, 200f, 30f), Label.SHOW_IMPLICITE, ref this.bImplicite, false, null, null, false);
			Widgets.Label(new Rect(220f, num2 + 4f, 65f, 30f), "Faction".Translate());
			SZWidgets.FloatMenuOnButtonText<string>(new Rect(285f, num2, 200f, 30f), this.selectedKey, this.dicFaction.Keys.ToList<string>(), (string s) => s, new Action<string>(this.ASelectFaction));
			num2 += 30f;
			bool flag = this.selectedPawn == null || this.lOfPawns.NullOrEmpty<Pawn>();
			if (!flag)
			{
				Widgets.Label(new Rect(0f, num2 + 3f, 100f, 30f), Label.RELATIONTO);
				num3 += 100f;
				SZWidgets.FloatMenuOnButtonText<Pawn>(new Rect(num3, num2, 145f, 30f), this.selectedPawn.LabelShort, this.lOfPawns, (Pawn p) => p.LabelShort, new Action<Pawn>(this.ASelectPawn));
				num3 += 145f;
				SZWidgets.FloatMenuOnButtonText<PawnRelationDef>(new Rect(num3, num2, 120f, 30f), this.GetRelationLabel(this.selectedPawn, this.selectedRelation), this.lOfRelations, (PawnRelationDef r) => this.GetRelationLabel(this.selectedPawn, r), new Action<PawnRelationDef>(this.ASelectRelation));
				num3 += 120f;
				GUI.color = Color.green;
				SZWidgets.ButtonText(num3, num2, 120f, 30f, Label.CREATE, new Action(this.AAddRelation), "");
				GUI.color = Color.white;
				SZWidgets.ButtonTextVar<Pawn>(inRect.x + inRect.width - 150f, num2, 150f, 30f, Label.SWAP_TO_PAWN, new Action<Pawn>(this.ASwapToPawn), this.selectedPawn);
				num2 += 30f;
				this.CreateDirect(0f, num2, num / 2f, h);
				bool flag2 = this.bImplicite;
				if (flag2)
				{
					this.CreateIndirect(num / 2f, num2, num / 2f, h);
				}
			}
		}

		// Token: 0x06000508 RID: 1288 RVA: 0x00033F04 File Offset: 0x00032104
		private void AAddRelation()
		{
			bool flag = CEditor.API.Pawn.relations == null;
			if (flag)
			{
				CEditor.API.Pawn.relations = new Pawn_RelationsTracker(CEditor.API.Pawn);
			}
			CEditor.API.Pawn.relations.AddDirectRelation(this.selectedRelation, this.selectedPawn);
		}

		// Token: 0x06000509 RID: 1289 RVA: 0x00033F67 File Offset: 0x00032167
		private void ASelectRelation(PawnRelationDef relationDef)
		{
			this.selectedRelation = relationDef;
		}

		// Token: 0x0600050A RID: 1290 RVA: 0x00033F71 File Offset: 0x00032171
		private void ASelectPawn(Pawn pawn)
		{
			this.selectedPawn = pawn;
		}

		// Token: 0x0600050B RID: 1291 RVA: 0x00033F7B File Offset: 0x0003217B
		private void ASelectFaction(string key)
		{
			this.selectedKey = key;
			this.lOfPawns = PawnxTool.GetPawnList(key, false, this.dicFaction.GetValue(key));
			this.selectedPawn = this.lOfPawns.FirstOrFallback(null);
		}

		// Token: 0x0600050C RID: 1292 RVA: 0x00033FB0 File Offset: 0x000321B0
		private void ASwapToPawn(Pawn p)
		{
			CEditor.API.Pawn = this.selectedPawn;
			this.Close(true);
		}

		// Token: 0x0600050D RID: 1293 RVA: 0x00033FCC File Offset: 0x000321CC
		public void CreateDirect(float x, float y, float w, float h)
		{
			bool flag = CEditor.API.Pawn.relations == null || CEditor.API.Pawn.relations.DirectRelations.NullOrEmpty<DirectPawnRelation>();
			if (!flag)
			{
				Rect outRect = new Rect(x, y, w, h);
				Rect rect = new Rect(0f, y, outRect.width - 16f, (float)CEditor.API.Pawn.relations.DirectRelations.Count * 205f);
				Widgets.BeginScrollView(outRect, ref this.scrollPos, rect, true);
				Rect rect2 = rect.ContractedBy(4f);
				rect2.height = (float)CEditor.API.Pawn.relations.DirectRelations.Count * 205f;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(rect2);
				try
				{
					foreach (DirectPawnRelation directPawnRelation in CEditor.API.Pawn.relations.DirectRelations)
					{
						Pawn otherPawn = directPawnRelation.otherPawn;
						RenderTexture image = PortraitsCache.Get(otherPawn, new Vector2(128f, 180f), Rot4.South, default(Vector3), 1f, true, true, true, true, null, null, false);
						//List<DirectPawnRelation>.Enumerator enumerator;
						string relationLabel = this.GetRelationLabel(otherPawn, directPawnRelation.def);
						int num = listing_X.Selectable(relationLabel + " - " + otherPawn.GetPawnName(false), this.dpr == directPawnRelation, "", image, null, null, new Vector2(128f, 180f), true, 22f, default(Color), default(Color), true);
						bool flag2 = this.dpr == directPawnRelation;
						if (flag2)
						{
							this.selectedPawn = directPawnRelation.otherPawn;
							MessageTool.Show("selected pawn = " + this.selectedPawn.Name.ToStringShort, null);
						}
						bool flag3 = num == 1;
						if (flag3)
						{
							this.dpr = directPawnRelation;
						}
						else
						{
							bool flag4 = num == 2;
							if (flag4)
							{
								CEditor.API.Pawn.relations.RemoveDirectRelation(directPawnRelation);
								this.dpr = null;
							}
						}
					}
				}
				catch
				{
				}
				listing_X.End();
				Widgets.EndScrollView();
			}
		}

		// Token: 0x0600050E RID: 1294 RVA: 0x0003428C File Offset: 0x0003248C
		public void CreateIndirect(float x, float y, float w, float h)
		{
			bool flag = CEditor.API.Pawn.relations == null;
			if (!flag)
			{
				List<Pawn> list = new List<Pawn>();
				int num = 0;
				foreach (Pawn pawn in CEditor.API.Pawn.relations.RelatedPawns)
				{
					list.Add(pawn);
					foreach (PawnRelationDef pawnRelationDef in pawn.GetRelations(CEditor.API.Pawn).ToList<PawnRelationDef>())
					{
						num++;
					}
				}
				Rect outRect = new Rect(x, y, w, h);
				Rect rect = new Rect(0f, y, outRect.width - 16f, (float)(num * 205));
				Widgets.BeginScrollView(outRect, ref this.scrollPos2, rect, true);
				Rect rect2 = rect.ContractedBy(4f);
				rect2.height = (float)(num * 205);
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(rect2);
				try
				{
					foreach (Pawn pawn2 in list)
					{
						List<PawnRelationDef> list2 = pawn2.GetRelations(CEditor.API.Pawn).ToList<PawnRelationDef>();
						foreach (PawnRelationDef relationDef in list2)
						{
							string text = this.GetRelationLabel(CEditor.API.Pawn, relationDef);
							text = text + Label.VON + pawn2.GetPawnName(false);
							RenderTexture image = PortraitsCache.Get(pawn2, new Vector2(128f, 180f), Rot4.South, default(Vector3), 1f, true, true, true, true, null, null, false);
							listing_X.Selectable(text, false, "", image, null, null, new Vector2(128f, 180f), false, 22f, default(Color), default(Color), true);
						}
					}
				}
				catch
				{
				}
				listing_X.End();
				Widgets.EndScrollView();
			}
		}

		// Token: 0x04000495 RID: 1173
		private Vector2 scrollPos;

		// Token: 0x04000496 RID: 1174
		private Vector2 scrollPos2;

		// Token: 0x04000497 RID: 1175
		private List<Pawn> lOfPawns;

		// Token: 0x04000498 RID: 1176
		private List<PawnRelationDef> lOfRelations;

		// Token: 0x04000499 RID: 1177
		private Pawn selectedPawn;

		// Token: 0x0400049A RID: 1178
		private string selectedKey;

		// Token: 0x0400049B RID: 1179
		private Dictionary<string, Faction> dicFaction;

		// Token: 0x0400049C RID: 1180
		private PawnRelationDef selectedRelation;

		// Token: 0x0400049D RID: 1181
		private DirectPawnRelation dpr;

		// Token: 0x0400049E RID: 1182
		private bool bImplicite;
	}
}
