﻿using System;
using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace CharacterEditor
{
	// Token: 0x02000046 RID: 70
	public class DialogWeaponry : Window
	{
		// Token: 0x17000059 RID: 89
		// (get) Token: 0x06000437 RID: 1079 RVA: 0x00023C44 File Offset: 0x00021E44
		public override Vector2 InitialSize
		{
			get
			{
				return new Vector2(this.MAXW, 800f);
			}
		}

		// Token: 0x06000438 RID: 1080 RVA: 0x00023C68 File Offset: 0x00021E68
		private string GetThingStyleDefLabel(ThingStyleDef t)
		{
			return (t == null) ? Label.NONE : (t.label.NullOrEmpty() ? t.defName : t.label.CapitalizeFirst());
		}

		// Token: 0x06000439 RID: 1081 RVA: 0x00023CA4 File Offset: 0x00021EA4
		private string GetStatLabel(StatDef s)
		{
			return (s == null) ? Label.NONE : s.label;
		}

		// Token: 0x0600043A RID: 1082 RVA: 0x00023CC8 File Offset: 0x00021EC8
		private string GetTradeabilityLabel(Tradeability t)
		{
			return Enum.GetName(typeof(Tradeability), t);
		}

		// Token: 0x0600043B RID: 1083 RVA: 0x00023CF0 File Offset: 0x00021EF0
		private string GetTechLevelLabel(TechLevel t)
		{
			return Enum.GetName(typeof(TechLevel), t);
		}

		// Token: 0x0600043C RID: 1084 RVA: 0x00023D18 File Offset: 0x00021F18
		private string GetTagLabel(string s)
		{
			return s ?? Label.NONE;
		}

		// Token: 0x0600043D RID: 1085 RVA: 0x00023D34 File Offset: 0x00021F34
		private string GetStuffCategoryLabel(StuffCategoryDef stuffCategoryDef)
		{
			return stuffCategoryDef.label;
		}

		// Token: 0x0600043E RID: 1086 RVA: 0x00023D4C File Offset: 0x00021F4C
		private string GetWeaponLabel(ThingDef thingDef)
		{
			string label = thingDef.label;
			return (label != null) ? label.CapitalizeFirst() : null;
		}

		// Token: 0x0600043F RID: 1087 RVA: 0x00023D70 File Offset: 0x00021F70
		private string GetWeaponTooltip(ThingDef thingDef)
		{
			return thingDef.description;
		}

		// Token: 0x06000440 RID: 1088 RVA: 0x00023D88 File Offset: 0x00021F88
		private bool IsSameWeapon(ThingDef a, ThingDef b)
		{
			return a == b;
		}

		// Token: 0x06000441 RID: 1089 RVA: 0x00023DA0 File Offset: 0x00021FA0
		private string GetBulletLabel(ThingDef thingDef)
		{
			return thingDef.label;
		}

		// Token: 0x06000442 RID: 1090 RVA: 0x00023DB8 File Offset: 0x00021FB8
		private string GetBulletTooltip(ThingDef thingDef)
		{
			return thingDef.description;
		}

		// Token: 0x06000443 RID: 1091 RVA: 0x00023DD0 File Offset: 0x00021FD0
		private bool IsSameBullet(ThingDef a, ThingDef b)
		{
			return a == b;
		}

		// Token: 0x06000444 RID: 1092 RVA: 0x00023DE8 File Offset: 0x00021FE8
		private string GetSoundLabel(SoundDef soundDef)
		{
			return soundDef.defName;
		}

		// Token: 0x06000445 RID: 1093 RVA: 0x00023E00 File Offset: 0x00022000
		private string GetSoundTooltip(SoundDef soundDef)
		{
			return soundDef.description;
		}

		// Token: 0x06000446 RID: 1094 RVA: 0x00023E18 File Offset: 0x00022018
		private bool IsSameSound(SoundDef a, SoundDef b)
		{
			return a == b;
		}

		// Token: 0x06000447 RID: 1095 RVA: 0x00023E30 File Offset: 0x00022030
		private bool IsPrimary(ThingWithComps t)
		{
			return t != null && CEditor.API.Pawn.HasEquipmentTracker() && CEditor.API.Pawn.equipment.Primary != null && CEditor.API.Pawn.equipment.Primary.ThingID == t.ThingID;
		}

		// Token: 0x06000448 RID: 1096 RVA: 0x00023E90 File Offset: 0x00022090
		public DialogWeaponry(ThingWithComps toSelect = null)
		{
			this.idxList = 7;
			bool flag = toSelect != null;
			if (flag)
			{
				SearchTool.ClearSearch(this.idxList);
			}
			this.search = SearchTool.Update(this.idxList);
			this.changeFirstWeapon = this.IsPrimary(toSelect);
			this.scrollPos = default(Vector2);
			this.scrollPosBullet = default(Vector2);
			this.scrollPosSound = default(Vector2);
			this.scrollPosParam = default(Vector2);
			this.lOfWeapon = new List<ThingDef>();
			this.lOfBullet = WeaponTool.ListOfBullets(null, null);
			this.lOfSound = new List<SoundDef>();
			this.lOfQuality = new List<int>();
			foreach (QualityCategory item in QualityUtility.AllQualityCategories)
			{
				this.lOfQuality.Add((int)item);
			}
			this.lCopyStat = new List<StatModifier>();
			this.lCopyEquipStat = new List<StatModifier>();
			this.lCopyStuff = new List<StuffCategoryDef>();
			this.lCopyCost = new List<ThingDefCountClass>();
			this.lCopyWeaponTag = new List<string>();
			this.iChangeTick0 = 0;
			this.iTick_AllowRemoveStat = 0;
			this.previousColor = Color.white;
			this.allowRemove = false;
			this.bAllSounds = false;
			this.doOnce = true;
			this.toggleText = ">>";
			this.tempName = "";
			this.lOfWeaponType = new List<WeaponType>();
			this.lOfWeaponType.Add(WeaponType.Melee);
			this.lOfWeaponType.Add(WeaponType.Ranged);
			this.lOfWeaponType.Add(WeaponType.Turret);
			this.FWeaponComparator = new Func<ThingDef, ThingDef, bool>(this.IsSameWeapon);
			this.FWeaponLabel = new Func<ThingDef, string>(this.GetWeaponLabel);
			this.FWeaponTooltip = new Func<ThingDef, string>(this.GetWeaponTooltip);
			this.FBulletComparator = new Func<ThingDef, ThingDef, bool>(this.IsSameBullet);
			this.FBulletLabel = new Func<ThingDef, string>(this.GetBulletLabel);
			this.FBulletTooltip = new Func<ThingDef, string>(this.GetBulletTooltip);
			this.FSoundComparator = new Func<SoundDef, SoundDef, bool>(this.IsSameSound);
			this.FSoundLabel = new Func<SoundDef, string>(this.GetSoundLabel);
			this.FSoundTooltip = new Func<SoundDef, string>(this.GetSoundTooltip);
			this.FGetStuffCategoryLabel = new Func<StuffCategoryDef, string>(this.GetStuffCategoryLabel);
			this.FGetTagLabel = new Func<string, string>(this.GetTagLabel);
			this.FGetTechLevelLabel = new Func<TechLevel, string>(this.GetTechLevelLabel);
			this.FGetTradeabilityLabel = new Func<Tradeability, string>(this.GetTradeabilityLabel);
			this.FGetStatLabel = new Func<StatDef, string>(this.GetStatLabel);
			this.FGetThingStyleDefLabel = new Func<ThingStyleDef, string>(this.GetThingStyleDefLabel);
			this.SELECTED = Selected.ByThingDef(null);
			this.search.ofilter1 = ((toSelect != null) ? (toSelect.def.IsRangedWeapon ? WeaponType.Ranged : WeaponType.Melee) : WeaponType.Ranged);
			this.lOfWeapon = WeaponTool.ListOfWeapons(null, this.search.modName, (WeaponType)this.search.ofilter1);
			this.lOfSound = WeaponTool.ListOfSounds(this.bAllSounds);
			this.ASelectWeaponFromSelector(toSelect);
			this.firstPreselection = (toSelect != null);
			this.doCloseX = true;
			this.absorbInputAroundWindow = true;
			this.draggable = true;
			this.layer = CEditor.Layer;
		}

		// Token: 0x06000449 RID: 1097 RVA: 0x00024218 File Offset: 0x00022418
		public override void DoWindowContents(Rect inRect)
		{
			float num = this.InitialSize.y - 104f;
			bool flag = (WeaponType)this.search.ofilter1 == WeaponType.Ranged || (WeaponType)this.search.ofilter1 == WeaponType.Turret;
			if (flag)
			{
				this.MAXW = this.W_GUNS + this.W_BULLETS + this.W_SOUND + this.W_PARAMS + 10f;
			}
			else
			{
				this.MAXW = this.W_GUNS + this.W_PARAMS + 10f;
			}
			this.windowRect.width = (CEditor.IsExtendedUI ? this.MAXW : (this.W_GUNS + 36f));
			this.toggleText = (CEditor.IsExtendedUI ? "<<" : ">>");
			bool flag2 = this.doOnce;
			if (flag2)
			{
				this.doOnce = false;
				this.windowRect.position = new Vector2((float)(CEditor.API.EditorPosX + 70), this.windowRect.position.y);
			}
			float num2 = 0f;
			float num3 = 0f;
			this.SELECTED.tooltip = Label.PRICE + this.SELECTED.buyPrice.ToString();
			this.DrawEditLabel(num3, num2, this.W_GUNS, 30f);
			num2 += 30f;
			this.DrawBulletTexture(num3 + this.W_GUNS + 25f, num2, this.W_GUNS, this.W_GUNS);
			this.DrawTexture(num3 + 50f, ref num2, this.W_GUNS, this.W_GUNS);
			this.DrawDropdown(num3, num2, this.W_GUNS, 30f);
			num2 += 62f;
			this.DrawList(num3, num2, this.W_GUNS, num - num2 + 20f);
			num2 += num - num2 + 30f;
			this.DrawParameter((int)(this.MAXW - this.W_PARAMS), 0, (int)(this.W_PARAMS - 40f), (int)(num + 20f));
			this.DrawLowerButtons(num3, num2, this.MAXW - this.W_GUNS - 40f, 40f);
			SZWidgets.ButtonText(240f, this.InitialSize.y - 75f, 30f, 40f, this.toggleText, new Action(this.AToggleStats), "");
			bool flag3 = this.SELECTED != null && this.SELECTED.thingDef != null && Widgets.ButtonText(new Rect(140f, this.InitialSize.y - 75f, 100f, 40f), "OK".Translate(), true, true, true);
			if (flag3)
			{
				this.DoAndClose();
			}
		}

		// Token: 0x0600044A RID: 1098 RVA: 0x000244E2 File Offset: 0x000226E2
		public override void OnAcceptKeyPressed()
		{
			base.OnAcceptKeyPressed();
			this.DoAndClose();
		}

		// Token: 0x0600044B RID: 1099 RVA: 0x000244F4 File Offset: 0x000226F4
		public void DoAndClose()
		{
			bool flag = this.SELECTED.Stuff != null && !this.SELECTED.thingDef.MadeFromStuff;
			if (flag)
			{
				this.SELECTED.Stuff = null;
			}
			bool flag2 = this.SELECTED.thingDef != null;
			if (flag2)
			{
				SearchTool.Save(this.idxList);
				CEditor.API.Pawn.Reequip(this.SELECTED, this.changeFirstWeapon ? 0 : 1, false);
				CEditor.API.UpdateGraphics();
			}
			this.Close(true);
		}

		// Token: 0x0600044C RID: 1100 RVA: 0x0002458C File Offset: 0x0002278C
		private void DrawDropdown(float x, float y, float w, float h)
		{
			Rect rect = new Rect(x, y, w, h);
			SZWidgets.FloatMenuOnButtonText<string>(rect, this.search.SelectedModName, CEditor.API.ListOf<string>(EType.ModsWeapons), (string s) => s ?? Label.ALL, new Action<string>(this.ASelectedModName), "");
			Rect rect2 = new Rect(rect);
			rect2.y += 30f;
			SZWidgets.FloatMenuOnButtonText<WeaponType>(rect2, this.GetNameForWeaponType((WeaponType)this.search.ofilter1), this.lOfWeaponType, (WeaponType tcd) => this.GetNameForWeaponType(tcd), new Action<WeaponType>(this.ASelectWeaponType));
		}

		// Token: 0x0600044D RID: 1101 RVA: 0x0002464C File Offset: 0x0002284C
		private string GetNameForWeaponType(WeaponType type)
		{
			bool flag = type == WeaponType.Melee;
			string result;
			if (flag)
			{
				result = Label.MELEE;
			}
			else
			{
				bool flag2 = type == WeaponType.Ranged;
				if (flag2)
				{
					result = Label.RANGED;
				}
				else
				{
					bool flag3 = type == WeaponType.Turret;
					if (flag3)
					{
						result = Label.TURRET;
					}
					else
					{
						result = "";
					}
				}
			}
			return result;
		}

		// Token: 0x0600044E RID: 1102 RVA: 0x00024693 File Offset: 0x00022893
		private void ASelectedModName(string val)
		{
			this.search.modName = val;
			this.ASelectWeaponType(WeaponType.Ranged);
		}

		// Token: 0x0600044F RID: 1103 RVA: 0x000246AC File Offset: 0x000228AC
		private void ASelectWeaponType(WeaponType val)
		{
			this.search.ofilter1 = val;
			this.lOfWeapon = WeaponTool.ListOfWeapons(null, this.search.modName, val);
			this.SELECTED.Init(this.lOfWeapon.FirstOrFallback(null), true, null, new Action(this.PostSelect));
		}

		// Token: 0x06000450 RID: 1104 RVA: 0x0002470C File Offset: 0x0002290C
		private void DrawEditLabel(float x, float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				Text.Font = GameFont.Medium;
				Rect rect = new Rect(x, y, w, h);
				bool flag2 = this.iChangeTick0 <= 0;
				if (flag2)
				{
					SZWidgets.Label(rect, this.SELECTED.thingDef.label, delegate()
					{
						this.iChangeTick0 = 300;
					}, "");
				}
				else
				{
					this.tempName = Widgets.TextField(rect, this.SELECTED.currentGun.label);
					bool flag3 = !this.SELECTED.thingDef.label.Equals(this.tempName);
					if (flag3)
					{
						this.SELECTED.currentGun.label = this.tempName;
						this.iChangeTick0 = 300;
					}
					this.iChangeTick0--;
				}
			}
		}

		// Token: 0x06000451 RID: 1105 RVA: 0x000247F4 File Offset: 0x000229F4
		private void DrawList(float x, float y, float w, float h)
		{
			SZWidgets.ListView<ThingDef>(x, y, this.W_GUNS, h, this.lOfWeapon, this.FWeaponLabel, this.FWeaponTooltip, this.FWeaponComparator, ref this.SELECTED.thingDef, ref this.scrollPos, false, new Action<ThingDef>(this.AWeaponSelected), true, true, true, false);
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				bool flag2 = !this.SELECTED.thingDef.IsMeleeWeapon && this.SELECTED.bulletDef != null;
				if (flag2)
				{
					SZWidgets.ListView<ThingDef>(x + this.W_GUNS, y + 4f, this.W_BULLETS, h, this.lOfBullet, this.FBulletLabel, this.FBulletTooltip, this.FBulletComparator, ref this.SELECTED.bulletDef, ref this.scrollPosBullet, false, new Action<ThingDef>(this.ABulletSelected), false, true, true, false);
				}
				bool flag3 = !this.SELECTED.thingDef.IsMeleeWeapon && this.SELECTED.soundDef != null;
				if (flag3)
				{
					SZWidgets.CheckBoxOnChange(new Rect(this.W_GUNS + this.W_BULLETS, y, 110f, 24f), Label.ALLSOUNDS, this.bAllSounds, new Action<bool>(this.ASoundListChange));
					SZWidgets.ListView<SoundDef>(x + this.W_GUNS + this.W_BULLETS, y + 4f, this.W_SOUND, h, this.lOfSound, this.FSoundLabel, this.FSoundTooltip, this.FSoundComparator, ref this.SELECTED.soundDef, ref this.scrollPosSound, false, new Action<SoundDef>(this.ASoundSelected), false, true, false, false);
				}
			}
		}

		// Token: 0x06000452 RID: 1106 RVA: 0x000249A5 File Offset: 0x00022BA5
		private void ASoundListChange(bool val)
		{
			this.bAllSounds = val;
			this.lOfSound = WeaponTool.ListOfSounds(this.bAllSounds);
		}

		// Token: 0x06000453 RID: 1107 RVA: 0x000249C0 File Offset: 0x00022BC0
		private void DrawLowerButtons(float x, float y, float w, float h)
		{
			int num = (int)(w / 3f);
			x += this.W_GUNS;
			SZWidgets.ButtonText(x, y, (float)num, h, Label.RESET, new Action(this.AResetOne), "");
			x += (float)num;
			SZWidgets.ButtonText(x, y, (float)num, h, Label.RESETALL, new Action(this.AResetAll), "");
			x += (float)num;
			SZWidgets.ButtonText(x, y, (float)num, h, Label.SAVE, new Action(this.ASave), "");
		}

		// Token: 0x06000454 RID: 1108 RVA: 0x00024A54 File Offset: 0x00022C54
		private Texture2D GetTexture()
		{
			bool flag = this.SELECTED.thingDef.IsTurret();
			Texture2D result;
			if (flag)
			{
				result = this.SELECTED.thingDef.uiIcon;
			}
			else
			{
				result = this.SELECTED.thingDef.GetTexture(1, this.SELECTED.Style, default(Rot4));
			}
			return result;
		}

		// Token: 0x06000455 RID: 1109 RVA: 0x00024AB4 File Offset: 0x00022CB4
		private void DrawTexture(float x, ref float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				Texture2D texture = this.GetTexture();
				GUI.color = this.SELECTED.DrawColor;
				Rect rect = new Rect(39f, y, 192f, 192f);
				GUI.DrawTexture(rect, texture, ScaleMode.ScaleAndCrop, true, 1f);
				y += 192f;
				y = SZWidgets.NavSelectorQuality(new Rect(0f, y, w, 25f), this.SELECTED, this.lOfQuality);
				y = SZWidgets.NavSelectorStuff(new Rect(0f, y, w, 25f), this.SELECTED);
				y = SZWidgets.NavSelectorStyle(new Rect(0f, y, w, 25f), this.SELECTED);
				TooltipHandler.TipRegion(rect, this.SELECTED.tip);
				bool flag2 = !CEditor.InStartingScreen;
				if (flag2)
				{
					SZWidgets.ButtonInvisible(rect, new Action(this.ABuyWeapon), "");
				}
			}
		}

		// Token: 0x06000456 RID: 1110 RVA: 0x00024BBC File Offset: 0x00022DBC
		private void DrawBulletTexture(float x, float y, float w, float h)
		{
			bool flag = this.SELECTED.thingDef == null || this.SELECTED.bulletDef == null;
			if (!flag)
			{
				Texture2D texture = this.SELECTED.bulletDef.GetTexture(1, null, default(Rot4));
				GUI.color = this.SELECTED.bulletDef.GetColor(null);
				Rect rect = new Rect(x, y, w - 100f, h - 100f);
				bool flag2 = Prefs.UIScale != 1f;
				if (flag2)
				{
					GUI.DrawTexture(rect, texture, ScaleMode.ScaleToFit, true, 1f);
				}
				else
				{
					Widgets.DrawTextureRotated(rect, texture, 90f);
				}
				GUI.color = Color.white;
				SZWidgets.ButtonInvisible(rect, new Action(this.ABuyAmmo), "");
			}
		}

		// Token: 0x06000457 RID: 1111 RVA: 0x00024C94 File Offset: 0x00022E94
		private void DrawParameter(int x, int y, int w, int h)
		{
			bool flag = this.SELECTED.thingDef == null;
			if (!flag)
			{
				this.CalcHSCROLL();
				bool flag2 = this.iTick_AllowRemoveStat > 0;
				if (flag2)
				{
					this.iTick_AllowRemoveStat--;
				}
				this.allowRemove = (this.iTick_AllowRemoveStat > 0);
				Rect outRect = new Rect((float)x, (float)y, (float)w, (float)h);
				Rect rect = new Rect(0f, 0f, outRect.width - 16f, this.HSCROLLPARAM);
				Widgets.BeginScrollView(outRect, ref this.scrollPosParam, rect, true);
				Rect rect2 = rect.ContractedBy(4f);
				rect2.y -= 4f;
				rect2.height = this.HSCROLLPARAM;
				Listing_X listing_X = new Listing_X();
				listing_X.Begin(rect2);
				listing_X.verticalSpacing = 30f;
				this.DrawDamage(listing_X);
				this.DrawRangedStats(listing_X);
				this.DrawStatProperties(listing_X, w);
				this.DrawEquipStatProperties(listing_X, w);
				this.DrawStuffProperties(listing_X, w);
				this.DrawCosts(listing_X, w);
				this.DrawTags(listing_X, w);
				this.DrawExtras(listing_X, w);
				listing_X.End();
				Widgets.EndScrollView();
			}
		}

		// Token: 0x06000458 RID: 1112 RVA: 0x00024DCC File Offset: 0x00022FCC
		private void DrawDamage(Listing_X view)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, "Damage".Translate() + " [" + this.SELECTED.thingDef.GetDmg().ToString() + "]");
			view.Gap(30f);
		}

		// Token: 0x06000459 RID: 1113 RVA: 0x00024E48 File Offset: 0x00023048
		private void DrawRangedStats(Listing_X view)
		{
			bool flag = !this.SELECTED.thingDef.IsMeleeWeapon;
			if (flag)
			{
				Text.Font = GameFont.Medium;
				view.GapLine(12f);
				view.Label(Label.GUNSTATS, 300f, 0f, -1f, null);
				view.Gap(12f);
				bool flag2 = !string.IsNullOrEmpty(this.SELECTED.currentGun.bulletName);
				if (flag2)
				{
					view.AddSection(Label.BULLET_SPEED, "cps", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.bulletSpeed, 0f, 150f, false, "");
				}
				view.AddIntSection("BurstShotFireRate".Translate(), "rpm", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.bulletInterval, 0, 100, false, "");
				view.AddIntSection("BurstShotCount".Translate(), "", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.bulletCount, 1, 150, false, "");
				bool flag3 = CEditor.IsCombatExtendedActive || (this.SELECTED.thingDef.IsTurret() && !string.IsNullOrEmpty(this.SELECTED.currentGun.bulletName));
				if (flag3)
				{
					view.AddIntSection(Label.MAGAZIN, "", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponFuelCapacity, 0, 500, false, "");
				}
				view.AddSection("Range".Translate(), "", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponRange, 0f, 100f, false, "");
				view.AddSection(Label.MIN + "Range".Translate(), "", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponMinRange, 0f, 50f, false, "");
				bool isCombatExtendedActive = CEditor.IsCombatExtendedActive;
				if (isCombatExtendedActive)
				{
					view.AddSection(Label.RELOADTIME, "s", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponCEReload, 0f, 30f, false, "");
				}
				bool flag4 = this.SELECTED.thingDef.HasStat(StatDefOf.RangedWeapon_Cooldown);
				bool flag5 = this.SELECTED.thingDef.IsTurret() || flag4;
				if (flag5)
				{
					view.AddSection(StatDefOf.RangedWeapon_Cooldown.LabelCap, "s", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponCooldown, 0f, 30f, false, "");
					bool flag6 = flag4;
					if (flag6)
					{
						this.SELECTED.thingDef.SetStatBaseValue(StatDefOf.RangedWeapon_Cooldown, this.SELECTED.currentGun.weaponCooldown);
					}
				}
				view.AddSection(Label.WARMUP, "s", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponWarmup, 0f, 30f, false, "");
				bool flag7 = this.SELECTED.thingDef.HasStat(StatDefOf.AccuracyTouch);
				if (flag7)
				{
					view.AddSection(StatDefOf.AccuracyTouch.label, "%", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.accuracyTouch, 0f, 1f, false, "");
					this.SELECTED.thingDef.SetStatBaseValue(StatDefOf.AccuracyTouch, this.SELECTED.currentGun.accuracyTouch);
				}
				bool flag8 = this.SELECTED.thingDef.HasStat(StatDefOf.AccuracyShort);
				if (flag8)
				{
					view.AddSection(StatDefOf.AccuracyShort.label, "%", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.accuracyShort, 0f, 1f, false, "");
					this.SELECTED.thingDef.SetStatBaseValue(StatDefOf.AccuracyShort, this.SELECTED.currentGun.accuracyShort);
				}
				bool flag9 = this.SELECTED.thingDef.HasStat(StatDefOf.AccuracyMedium);
				if (flag9)
				{
					view.AddSection(StatDefOf.AccuracyMedium.label, "%", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.accuracyMedium, 0f, 1f, false, "");
					this.SELECTED.thingDef.SetStatBaseValue(StatDefOf.AccuracyMedium, this.SELECTED.currentGun.accuracyMedium);
				}
				bool flag10 = this.SELECTED.thingDef.HasStat(StatDefOf.AccuracyLong);
				if (flag10)
				{
					view.AddSection(StatDefOf.AccuracyLong.label, "%", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.accuracyLong, 0f, 1f, false, "");
					this.SELECTED.thingDef.SetStatBaseValue(StatDefOf.AccuracyLong, this.SELECTED.currentGun.accuracyLong);
				}
				bool flag11 = !string.IsNullOrEmpty(this.SELECTED.currentGun.bulletName);
				if (flag11)
				{
					view.AddSection("StoppingPower".Translate(), "s", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.bulletStoppingPower, 0f, 10f, false, "");
				}
				view.AddSection(Label.SPRAYING, "", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.weaponScatter, 0f, 10f, false, "");
				bool flag12 = this.SELECTED.thingDef.DoesExplosion();
				if (flag12)
				{
					view.AddSection(Label.EXPL_RADIUS, "cells", ref this.SELECTED.paramName, ref this.SELECTED.currentGun.bulletExplosionRadius, 0f, 50f, false, "");
				}
				view.CheckboxLabeled(Label.TARGET_LOCATION, 4f, 250f, ref this.SELECTED.currentGun.bTargetGround, null);
				this.SELECTED.currentGun.techLevel = Enum.GetName(typeof(TechLevel), this.SELECTED.thingDef.techLevel);
				this.SELECTED.currentGun.tradeability = Enum.GetName(typeof(Tradeability), this.SELECTED.thingDef.tradeability);
				this.SELECTED.currentGun.stealable = this.SELECTED.thingDef.stealable.ToString();
				this.SELECTED.currentGun.costStuffCount = this.SELECTED.thingDef.costStuffCount.ToString();
				this.SELECTED.currentGun.stuffCategories = this.SELECTED.thingDef.stuffCategories.ListToString<StuffCategoryDef>();
				this.SELECTED.currentGun.statBases = this.SELECTED.thingDef.statBases.ListToString<StatModifier>();
				this.SELECTED.currentGun.equippedStats = this.SELECTED.thingDef.equippedStatOffsets.ListToString<StatModifier>();
				this.SELECTED.currentGun.tags = this.SELECTED.thingDef.weaponTags.ListToString<string>();
				this.SELECTED.currentGun.UpdateWeapon();
			}
		}

		// Token: 0x0600045A RID: 1114 RVA: 0x00025638 File Offset: 0x00023838
		private void DrawStatProperties(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.GapLine(12f);
			view.Label(0f, 0f, 300f, 30f, Label.STATS);
			Text.Font = GameFont.Small;
			view.ButtonImage((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.ADialogAddStat));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyStat));
			bool flag = !this.lCopyStat.NullOrEmpty<StatModifier>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteStat));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.statBases != null;
			if (flag2)
			{
				try
				{
					foreach (StatModifier statModifier in this.SELECTED.thingDef.statBases)
					{
						bool flag3 = view.ListviewSM(400f, statModifier, statModifier.stat.defName == this.SELECTED.statDefName, this.allowRemove, new Action<string>(this.ARemoveStat));
						if (flag3)
						{
							this.SELECTED.statDefName = ((this.SELECTED.statDefName != null) ? null : statModifier.stat.defName);
						}
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x0600045B RID: 1115 RVA: 0x00025844 File Offset: 0x00023A44
		private void DrawEquipStatProperties(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.GapLine(12f);
			view.Label(0f, 0f, 300f, 30f, Label.EQUIPSTATS);
			Text.Font = GameFont.Small;
			view.ButtonImage((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", new Action(this.ADialogAddEquipStat));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyEquipStat));
			bool flag = !this.lCopyEquipStat.NullOrEmpty<StatModifier>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteEquipStat));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.equippedStatOffsets != null;
			if (flag2)
			{
				try
				{
					foreach (StatModifier statModifier in this.SELECTED.thingDef.equippedStatOffsets)
					{
						bool flag3 = view.ListviewSM(400f, statModifier, statModifier.stat.defName == this.SELECTED.equipStatDefName, this.allowRemove, new Action<string>(this.ARemoveEquipStat));
						if (flag3)
						{
							this.SELECTED.equipStatDefName = ((this.SELECTED.equipStatDefName != null) ? null : statModifier.stat.defName);
						}
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x0600045C RID: 1116 RVA: 0x00025A50 File Offset: 0x00023C50
		private void DrawStuffProperties(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.GapLine(12f);
			view.Label(0f, 0f, 300f, 30f, Label.STUFFPROPS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<StuffCategoryDef>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<StuffCategoryDef>(EType.StuffCategory), this.FGetStuffCategoryLabel, new Action<StuffCategoryDef>(this.AAddStuffCategory));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyStuff));
			bool flag = !this.lCopyStuff.NullOrEmpty<StuffCategoryDef>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteStuff));
			}
			view.Gap(30f);
			bool madeFromStuff = this.SELECTED.thingDef.MadeFromStuff;
			if (madeFromStuff)
			{
				view.LabelSimple(Label.COSTSTUFFCOUNT + this.SELECTED.thingDef.costStuffCount.ToString(), 0f, view.CurY, 300f, 25f, null);
				view.Gap(20f);
				this.SELECTED.thingDef.costStuffCount = (int)view.Slider((float)this.SELECTED.thingDef.costStuffCount, 1f, 200f, 300f);
				bool flag2 = this.SELECTED.OldStuffCount != this.SELECTED.thingDef.costStuffCount;
				if (flag2)
				{
					this.SELECTED.OldStuffCount = this.SELECTED.thingDef.costStuffCount;
					this.SELECTED.thingDef.UpdateCostStuffCount();
				}
				try
				{
					foreach (StuffCategoryDef stuffCategoryDef in this.SELECTED.thingDef.stuffCategories)
					{
						view.Listview(400f, stuffCategoryDef.defName, stuffCategoryDef.label, stuffCategoryDef.description, this.allowRemove, new Action<string>(this.ARemoveStuffCategory));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x0600045D RID: 1117 RVA: 0x00025D08 File Offset: 0x00023F08
		private void DrawCosts(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.Label(0f, 0f, 300f, 30f, Label.COSTS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<ThingDef>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<ThingDef>(EType.CostItems), this.FWeaponLabel, new Action<ThingDef>(this.AAddCosts));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyCosts));
			bool flag = !this.lCopyCost.NullOrEmpty<ThingDefCountClass>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteCosts));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.costList != null;
			if (flag2)
			{
				try
				{
					foreach (ThingDefCountClass thingDefCountClass in this.SELECTED.thingDef.costList)
					{
						bool flag3 = view.ListviewTDC(400f, thingDefCountClass, thingDefCountClass.thingDef.defName == this.SELECTED.tdcDefName, this.allowRemove, new Action<string>(this.ARemoveCosts), this.SELECTED.thingDef);
						if (flag3)
						{
							this.SELECTED.tdcDefName = ((this.SELECTED.tdcDefName != null) ? null : thingDefCountClass.thingDef.defName);
						}
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x0600045E RID: 1118 RVA: 0x00025F28 File Offset: 0x00024128
		private void DrawTags(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.GapLine(12f);
			view.Label(0f, 0f, 300f, 30f, Label.TAGS);
			Text.Font = GameFont.Small;
			view.FloatMenuOnButtonImage<string>((float)(w - 60), 5f, 24f, 24f, "UI/Buttons/Dev/Add", CEditor.API.ListOf<string>(EType.WeaponTags), this.FGetTagLabel, new Action<string>(this.AAddWeaponTag));
			view.ButtonImage((float)(w - 85), 5f, 24f, 24f, "bminus", new Action(this.AAllowRemoveStat));
			view.ButtonImage((float)(w - 110), 5f, 18f, 24f, "UI/Buttons/Copy", new Action(this.ACopyWeaponTag));
			bool flag = !this.lCopyWeaponTag.NullOrEmpty<string>();
			if (flag)
			{
				view.ButtonImage((float)(w - 130), 5f, 18f, 24f, "UI/Buttons/Paste", new Action(this.APasteWeaponTag));
			}
			view.Gap(30f);
			bool flag2 = this.SELECTED.thingDef.weaponTags != null;
			if (flag2)
			{
				try
				{
					foreach (string text in this.SELECTED.thingDef.weaponTags)
					{
						view.Listview(400f, text, text, null, this.allowRemove, new Action<string>(this.ARemoveWeaponTag));
					}
				}
				catch
				{
				}
			}
			view.Gap(25f);
		}

		// Token: 0x0600045F RID: 1119 RVA: 0x000260FC File Offset: 0x000242FC
		private void DrawExtras(Listing_X view, int w)
		{
			Text.Font = GameFont.Medium;
			view.GapLine(12f);
			view.Label(Label.EXTRAS, 300f, 0f, -1f, null);
			Text.Font = GameFont.Small;
			view.FloatMenuButtonWithLabel<TechLevel>(Label.TECHLEVEL, 100f, 200f, Enum.GetName(typeof(TechLevel), this.SELECTED.thingDef.techLevel), CEditor.API.ListOf<TechLevel>(EType.TechLevel), this.FGetTechLevelLabel, new Action<TechLevel>(this.ASetTechLevel));
			view.FloatMenuButtonWithLabel<Tradeability>(Label.TRADEABILITY, 100f, 200f, Enum.GetName(typeof(Tradeability), this.SELECTED.thingDef.tradeability), CEditor.API.ListOf<Tradeability>(EType.Tradeability), this.FGetTradeabilityLabel, new Action<Tradeability>(this.ASetTradeability));
			view.CheckboxLabeled(Label.STEALABLE, 2f, 120f, ref this.SELECTED.thingDef.stealable, null);
			Text.Font = GameFont.Medium;
			view.GapLine(12f);
			view.Label(Label.GRAPHICS, 300f, 0f, -1f, null);
			Text.Font = GameFont.Tiny;
			Text.WordWrap = false;
			view.Label(Label.TEXPATH + this.SELECTED.thingDef.graphicData.texPath, 1000f, 0f, -1f, null);
			view.Label(Label.DRAWSIZE + this.SELECTED.thingDef.graphicData.drawSize.ToString(), 1000f, 0f, -1f, null);
			Text.WordWrap = true;
			Text.Font = GameFont.Small;
		}

		// Token: 0x06000460 RID: 1120 RVA: 0x000262D9 File Offset: 0x000244D9
		private void AToggleStats()
		{
			CEditor.IsExtendedUI = !CEditor.IsExtendedUI;
		}

		// Token: 0x06000461 RID: 1121 RVA: 0x000262EC File Offset: 0x000244EC
		private void ASelectWeaponFromSelector(ThingWithComps toSelect = null)
		{
			bool flag = !this.lOfWeapon.NullOrEmpty<ThingDef>();
			if (flag)
			{
				bool flag2 = toSelect == null;
				if (flag2)
				{
					this.SELECTED.Init(this.lOfWeapon.FirstOrFallback(null), true, null, null);
				}
				else
				{
					this.SELECTED.Init(toSelect.def, false, toSelect, null);
				}
			}
		}

		// Token: 0x06000462 RID: 1122 RVA: 0x00026348 File Offset: 0x00024548
		private void ASoundSelected(SoundDef soundDef)
		{
			this.SELECTED.currentGun.UpdateFromSound(soundDef);
			bool flag = !CEditor.InStartingScreen;
			if (flag)
			{
				soundDef.PlayOneShot(new TargetInfo(UI.MouseMapPosition().ToIntVec3(), Find.CurrentMap, false));
			}
		}

		// Token: 0x06000463 RID: 1123 RVA: 0x00026398 File Offset: 0x00024598
		private void ABulletSelected(ThingDef bulletDef)
		{
			this.SELECTED.currentGun.UpdateFromBullet(bulletDef);
			bool flag = bulletDef != null;
			if (flag)
			{
				bool flag2 = !CEditor.InStartingScreen && bulletDef.projectile != null && bulletDef.projectile.soundExplode != null;
				if (flag2)
				{
					bulletDef.projectile.soundExplode.PlayOneShot(new TargetInfo(UI.MouseMapPosition().ToIntVec3(), Find.CurrentMap, false));
				}
			}
			this.ChangeAmmo();
		}

		// Token: 0x06000464 RID: 1124 RVA: 0x00026418 File Offset: 0x00024618
		private void ChangeAmmo()
		{
		}

		// Token: 0x06000465 RID: 1125 RVA: 0x0002641C File Offset: 0x0002461C
		private void AWeaponSelected(ThingDef thingDef)
		{
			bool flag = this.firstPreselection;
			if (flag)
			{
				this.firstPreselection = false;
			}
			else
			{
				bool flag2 = thingDef != null;
				if (flag2)
				{
					bool flag3 = !CEditor.InStartingScreen && thingDef.soundInteract != null;
					if (flag3)
					{
						thingDef.soundInteract.PlayOneShot(new TargetInfo(UI.MouseMapPosition().ToIntVec3(), Find.CurrentMap, false));
					}
					this.SELECTED.Init(thingDef, true, null, new Action(this.PostSelect));
				}
			}
		}

		// Token: 0x06000466 RID: 1126 RVA: 0x000264A0 File Offset: 0x000246A0
		private void PostSelect()
		{
			this.scrollPos.CheckAndSetScrollPos(this.lOfWeapon, this.SELECTED.thingDef, 28f, 300f);
			this.scrollPosBullet.CheckAndSetScrollPos(this.lOfBullet, this.SELECTED.bulletDef, 28f, 300f);
			this.scrollPosSound.CheckAndSetScrollPos(this.lOfSound, this.SELECTED.soundDef, 28f, 300f);
		}

		// Token: 0x06000467 RID: 1127 RVA: 0x00026524 File Offset: 0x00024724
		private void AResetAll()
		{
			foreach (PresetWeapon presetWeapon in CEditor.API.Get<Dictionary<string, PresetWeapon>>(EType.WeaponPreset).Values)
			{
				presetWeapon.UpdateWeapon();
			}
			CEditor.API.SetCustom(OptionS.CUSTOMWEAPON, "", "");
			this.SELECTED.Init(this.lOfWeapon.FirstOrFallback(null), true, null, new Action(this.PostSelect));
			MessageTool.Show("weapon reset done", null);
		}

		// Token: 0x06000468 RID: 1128 RVA: 0x000265D0 File Offset: 0x000247D0
		private void AResetOne()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				PresetWeapon.ResetToDefault(this.SELECTED.thingDef.defName);
				bool flag2 = this.SELECTED.thingDef.IsTurret();
				if (flag2)
				{
					PresetWeapon.ResetToDefault(this.SELECTED.thingDef.building.turretGunDef.defName);
				}
				this.SELECTED.Init(this.lOfWeapon.FirstOrFallback(null), true, null, new Action(this.PostSelect));
				MessageTool.Show("reset weapon " + this.SELECTED.thingDef.defName + " done", null);
			}
		}

		// Token: 0x06000469 RID: 1129 RVA: 0x0002668C File Offset: 0x0002488C
		private void ASave()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				PresetWeapon presetWeapon = new PresetWeapon(this.SELECTED.thingDef);
				bool flag2 = presetWeapon.UpdateWeapon();
				if (flag2)
				{
					presetWeapon.SaveCustom();
					MessageTool.Show(presetWeapon.label + " " + Label.SETTINGSSAVED, null);
				}
			}
		}

		// Token: 0x0600046A RID: 1130 RVA: 0x000266F0 File Offset: 0x000248F0
		private void CalcHSCROLL()
		{
			int num = this.SELECTED.thingDef.stuffCategories.CountAllowNull<StuffCategoryDef>();
			int num2 = this.SELECTED.thingDef.statBases.CountAllowNull<StatModifier>();
			int num3 = this.SELECTED.thingDef.equippedStatOffsets.CountAllowNull<StatModifier>();
			int num4 = this.SELECTED.thingDef.weaponTags.CountAllowNull<string>();
			int num5 = (!this.SELECTED.thingDef.IsMeleeWeapon) ? 700 : 0;
			int num6 = num + num2 + num3 + num4;
			this.HSCROLLPARAM = (float)(num5 + 900 + num6 * 28);
		}

		// Token: 0x0600046B RID: 1131 RVA: 0x00026790 File Offset: 0x00024990
		private void AAddStat(StatDef stat)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddStat(stat, 0f);
			}
		}

		// Token: 0x0600046C RID: 1132 RVA: 0x000267CC File Offset: 0x000249CC
		private void ADialogAddStat()
		{
      List<StatDef> hashSet = DefTool.ListBy<StatDef>(DefTool.CONDITION_STATDEFS_WEAPON(ThingTool.GetAllStatCategoriesOnEquip()));
			for (int i = 0; i < hashSet.Count; i++)
			{
				bool flag = this.SELECTED.thingDef.HasStat(hashSet.At(i));
				if (flag)
				{
					hashSet.Remove(hashSet.At(i));
					i--;
				}
			}
			SZWidgets.FloatMenuOnRect<StatDef>(hashSet, this.FGetStatLabel, new Action<StatDef>(this.AAddStat));
		}

		// Token: 0x0600046D RID: 1133 RVA: 0x0002684C File Offset: 0x00024A4C
		private void AAllowRemoveStat()
		{
			bool flag = this.iTick_AllowRemoveStat > 0;
			if (flag)
			{
				this.iTick_AllowRemoveStat = 0;
			}
			else
			{
				this.iTick_AllowRemoveStat = 1000;
			}
		}

		// Token: 0x0600046E RID: 1134 RVA: 0x0002687C File Offset: 0x00024A7C
		private void ACopyStat()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.statBases.NullOrEmpty<StatModifier>();
			if (flag)
			{
				this.lCopyStat = new List<StatModifier>();
				foreach (StatModifier item in this.SELECTED.thingDef.statBases)
				{
					this.lCopyStat.Add(item);
				}
			}
		}

		// Token: 0x0600046F RID: 1135 RVA: 0x0002691C File Offset: 0x00024B1C
		private void APasteStat()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteStats(this.lCopyStat);
			}
		}

		// Token: 0x06000470 RID: 1136 RVA: 0x00026954 File Offset: 0x00024B54
		private void ARemoveStat(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveStat(DefTool.StatDef(defName));
			}
		}

		// Token: 0x06000471 RID: 1137 RVA: 0x0002698C File Offset: 0x00024B8C
		private void AAddEquipStat(StatDef stat)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddEquipStat(stat, 0f);
			}
		}

		// Token: 0x06000472 RID: 1138 RVA: 0x000269C8 File Offset: 0x00024BC8
		private void ADialogAddEquipStat()
		{
			List<StatDef> list = CEditor.API.ListOfStatDef(null, true, true);
			for (int i = 0; i < list.Count; i++)
			{
				bool flag = this.SELECTED.thingDef.HasStat(list[i]);
				if (flag)
				{
					list.Remove(list[i]);
					i--;
				}
			}
			SZWidgets.FloatMenuOnRect<StatDef>(list, this.FGetStatLabel, new Action<StatDef>(this.AAddEquipStat));
		}

		// Token: 0x06000473 RID: 1139 RVA: 0x00026A44 File Offset: 0x00024C44
		private void ACopyEquipStat()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.equippedStatOffsets.NullOrEmpty<StatModifier>();
			if (flag)
			{
				this.lCopyEquipStat = new List<StatModifier>();
				foreach (StatModifier item in this.SELECTED.thingDef.equippedStatOffsets)
				{
					this.lCopyEquipStat.Add(item);
				}
			}
		}

		// Token: 0x06000474 RID: 1140 RVA: 0x00026AE4 File Offset: 0x00024CE4
		private void APasteEquipStat()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteEquipStats(this.lCopyEquipStat);
			}
		}

		// Token: 0x06000475 RID: 1141 RVA: 0x00026B1C File Offset: 0x00024D1C
		private void ARemoveEquipStat(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveEquipStat(DefTool.StatDef(defName));
			}
		}

		// Token: 0x06000476 RID: 1142 RVA: 0x00026B54 File Offset: 0x00024D54
		private void AAddCosts(ThingDef thingDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddCost(thingDef, 0);
			}
		}

		// Token: 0x06000477 RID: 1143 RVA: 0x00026B88 File Offset: 0x00024D88
		private void ACopyCosts()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.costList.NullOrEmpty<ThingDefCountClass>();
			if (flag)
			{
				this.lCopyCost = new List<ThingDefCountClass>();
				foreach (ThingDefCountClass item in this.SELECTED.thingDef.costList)
				{
					this.lCopyCost.Add(item);
				}
			}
		}

		// Token: 0x06000478 RID: 1144 RVA: 0x00026C28 File Offset: 0x00024E28
		private void APasteCosts()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteCost(this.lCopyCost);
			}
		}

		// Token: 0x06000479 RID: 1145 RVA: 0x00026C60 File Offset: 0x00024E60
		private void ARemoveCosts(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveCost(DefTool.ThingDef(defName));
			}
		}

		// Token: 0x0600047A RID: 1146 RVA: 0x00026C98 File Offset: 0x00024E98
		private void AAddStuffCategory(StuffCategoryDef stuffCategoryDef)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddStuff(stuffCategoryDef);
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x0600047B RID: 1147 RVA: 0x00026CD8 File Offset: 0x00024ED8
		private void ACopyStuff()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.stuffCategories.NullOrEmpty<StuffCategoryDef>();
			if (flag)
			{
				this.lCopyStuff = new List<StuffCategoryDef>();
				foreach (StuffCategoryDef item in this.SELECTED.thingDef.stuffCategories)
				{
					this.lCopyStuff.Add(item);
				}
			}
		}

		// Token: 0x0600047C RID: 1148 RVA: 0x00026D78 File Offset: 0x00024F78
		private void APasteStuff()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteStuff(this.lCopyStuff);
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x0600047D RID: 1149 RVA: 0x00026DC0 File Offset: 0x00024FC0
		private void ARemoveStuffCategory(string defName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveStuff(DefTool.StuffCategoryDef(defName));
				this.SELECTED.UpdateStuffList();
			}
		}

		// Token: 0x0600047E RID: 1150 RVA: 0x00026E08 File Offset: 0x00025008
		private void AAddWeaponTag(string tag)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.AddWeaponTag(tag);
			}
		}

		// Token: 0x0600047F RID: 1151 RVA: 0x00026E3C File Offset: 0x0002503C
		private void ACopyWeaponTag()
		{
			bool flag = this.SELECTED.thingDef != null && !this.SELECTED.thingDef.weaponTags.NullOrEmpty<string>();
			if (flag)
			{
				this.lCopyWeaponTag = new List<string>();
				foreach (string item in this.SELECTED.thingDef.weaponTags)
				{
					this.lCopyWeaponTag.Add(item);
				}
			}
		}

		// Token: 0x06000480 RID: 1152 RVA: 0x00026EDC File Offset: 0x000250DC
		private void APasteWeaponTag()
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.PasteWeaponTag(this.lCopyWeaponTag);
			}
		}

		// Token: 0x06000481 RID: 1153 RVA: 0x00026F14 File Offset: 0x00025114
		private void ARemoveWeaponTag(string tagName)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.RemoveWeaponTag(tagName);
			}
		}

		// Token: 0x06000482 RID: 1154 RVA: 0x00026F48 File Offset: 0x00025148
		private void ASetTechLevel(TechLevel techLevel)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.techLevel = techLevel;
				this.SELECTED.thingDef.ResolveReferences();
			}
		}

		// Token: 0x06000483 RID: 1155 RVA: 0x00026F8C File Offset: 0x0002518C
		private void ASetTradeability(Tradeability tradeability)
		{
			bool flag = this.SELECTED.thingDef != null;
			if (flag)
			{
				this.SELECTED.thingDef.tradeability = tradeability;
				this.SELECTED.thingDef.ResolveReferences();
			}
		}

		// Token: 0x06000484 RID: 1156 RVA: 0x00026FD0 File Offset: 0x000251D0
		private void ABuyWeapon()
		{
			ThingTool.BeginBuyItem(this.SELECTED);
			CEditor.API.CloseEditor();
			this.Close(true);
		}

		// Token: 0x06000485 RID: 1157 RVA: 0x00026FF2 File Offset: 0x000251F2
		private void ABuyAmmo()
		{
		}

		// Token: 0x0400022D RID: 557
		private Vector2 scrollPos;

		// Token: 0x0400022E RID: 558
		private Vector2 scrollPosBullet;

		// Token: 0x0400022F RID: 559
		private Vector2 scrollPosSound;

		// Token: 0x04000230 RID: 560
		private Vector2 scrollPosParam;

		// Token: 0x04000231 RID: 561
		private Selected SELECTED;

		// Token: 0x04000232 RID: 562
		private Color previousColor;

		// Token: 0x04000233 RID: 563
		private List<ThingDef> lOfWeapon;

		// Token: 0x04000234 RID: 564
		private List<ThingDef> lOfBullet;

		// Token: 0x04000235 RID: 565
		private List<SoundDef> lOfSound;

		// Token: 0x04000236 RID: 566
		private List<int> lOfQuality;

		// Token: 0x04000237 RID: 567
		private List<StatModifier> lCopyStat;

		// Token: 0x04000238 RID: 568
		private List<StatModifier> lCopyEquipStat;

		// Token: 0x04000239 RID: 569
		private List<StuffCategoryDef> lCopyStuff;

		// Token: 0x0400023A RID: 570
		private List<ThingDefCountClass> lCopyCost;

		// Token: 0x0400023B RID: 571
		private List<string> lCopyWeaponTag;

		// Token: 0x0400023C RID: 572
		private int iTick_AllowRemoveStat;

		// Token: 0x0400023D RID: 573
		private int iChangeTick0;

		// Token: 0x0400023E RID: 574
		private int idxList;

		// Token: 0x0400023F RID: 575
		private bool allowRemove;

		// Token: 0x04000240 RID: 576
		private bool bAllSounds;

		// Token: 0x04000241 RID: 577
		private bool doOnce;

		// Token: 0x04000242 RID: 578
		private bool firstPreselection;

		// Token: 0x04000243 RID: 579
		private string toggleText;

		// Token: 0x04000244 RID: 580
		private string tempName;

		// Token: 0x04000245 RID: 581
		private const float WTITLE = 300f;

		// Token: 0x04000246 RID: 582
		private float MAXW = 1250f;

		// Token: 0x04000247 RID: 583
		private float W_GUNS = 270f;

		// Token: 0x04000248 RID: 584
		private float W_BULLETS = 240f;

		// Token: 0x04000249 RID: 585
		private float W_SOUND = 200f;

		// Token: 0x0400024A RID: 586
		private float W_PARAMS = 530f;

		// Token: 0x0400024B RID: 587
		private float HSCROLLPARAM;

		// Token: 0x0400024C RID: 588
		private const int TEXTURESIZE = 192;

		// Token: 0x0400024D RID: 589
		private List<WeaponType> lOfWeaponType;

		// Token: 0x0400024E RID: 590
		private bool changeFirstWeapon;

		// Token: 0x0400024F RID: 591
		private SearchTool search;

		// Token: 0x04000250 RID: 592
		private Func<string, string> FGetTagLabel;

		// Token: 0x04000251 RID: 593
		private Func<StuffCategoryDef, string> FGetStuffCategoryLabel;

		// Token: 0x04000252 RID: 594
		private Func<ThingDef, string> FWeaponLabel;

		// Token: 0x04000253 RID: 595
		private Func<ThingDef, string> FWeaponTooltip;

		// Token: 0x04000254 RID: 596
		private Func<ThingDef, ThingDef, bool> FWeaponComparator;

		// Token: 0x04000255 RID: 597
		private Func<ThingDef, string> FBulletLabel;

		// Token: 0x04000256 RID: 598
		private Func<ThingDef, string> FBulletTooltip;

		// Token: 0x04000257 RID: 599
		private Func<ThingDef, ThingDef, bool> FBulletComparator;

		// Token: 0x04000258 RID: 600
		private Func<SoundDef, string> FSoundLabel;

		// Token: 0x04000259 RID: 601
		private Func<SoundDef, string> FSoundTooltip;

		// Token: 0x0400025A RID: 602
		private Func<SoundDef, SoundDef, bool> FSoundComparator;

		// Token: 0x0400025B RID: 603
		private Func<TechLevel, string> FGetTechLevelLabel;

		// Token: 0x0400025C RID: 604
		private Func<Tradeability, string> FGetTradeabilityLabel;

		// Token: 0x0400025D RID: 605
		private Func<StatDef, string> FGetStatLabel;

		// Token: 0x0400025E RID: 606
		private Func<ThingStyleDef, string> FGetThingStyleDefLabel;
	}
}
