﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  // Token: 0x02000047 RID: 71
  public static class Extension
  {

    

    

    public static string ConstructLabelFromDef(this Def def)
    {
      if (def == null)
      {
        return "null";
      }
      //if (!def.LabelCap.NullOrEmpty())
      //{
      //  return def.LabelCap;
      //}
      string label = def.defName.ConstructLabelFromDefname();
      if (def is HeadTypeDef def2)
      {
        label = label.Replace("Female", "♀");
        label = label.Replace("Male", "♂");
      }
      return label;
    }

    public static string ConstructLabelFromDefname(this string defName)
    {
      if (defName.NullOrEmpty())
      {
        return "null";
      }
      string label = defName.Replace('_', ' ');
      List<char> charList = label.ToCharArray().ToList();
      label = "";
      for (int i = 0; i < charList.Count(); i++)
      {
        bool isCapitalized = charList[i].ToString() == charList[i].ToString().CapitalizeFirst();
        label += isCapitalized ? " " + charList[i] : charList[i];
      }
      label = label.Trim();
      string[] array = label.Split(' ');
      label = "";
      foreach (string item in array)
      {
        if (!item.NullOrEmpty())
        {
          label += item + " ";
        }
      }
      return label.Trim();
    }

    // Token: 0x06000488 RID: 1160 RVA: 0x0002700C File Offset: 0x0002520C
    public static void ChangeXPosition(this Rect rect, ref bool bdoFlag, int xoffset)
    {
      bool flag = !bdoFlag;
      if (!flag)
      {
        bdoFlag = false;
        rect.position = new Vector2(rect.position.x + (float)xoffset, rect.position.y);
      }
    }

    // Token: 0x06000489 RID: 1161 RVA: 0x00027050 File Offset: 0x00025250
    public static void AddFromList<T1, T2>(this SortedDictionary<T1, T2> dic, List<T1> otherList, T2 defaultVal = default(T2))
    {
      bool flag = dic != null && !otherList.NullOrEmpty<T1>();
      if (flag)
      {
        foreach (T1 key in otherList)
        {
          bool flag2 = !dic.ContainsKey(key);
          if (flag2)
          {
            dic.Add(key, defaultVal);
          }
        }
      }
    }

    // Token: 0x0600048A RID: 1162 RVA: 0x000270C8 File Offset: 0x000252C8
    public static void AddFromList<T1, T2>(this Dictionary<T1, T2> dic, List<T1> otherList, T2 defaultVal = default(T2))
    {
      bool flag = dic != null && !otherList.NullOrEmpty<T1>();
      if (flag)
      {
        foreach (T1 key in otherList)
        {
          bool flag2 = !dic.ContainsKey(key);
          if (flag2)
          {
            dic.Add(key, defaultVal);
          }
        }
      }
    }

    // Token: 0x0600048B RID: 1163 RVA: 0x00027140 File Offset: 0x00025340
    public static void AddLabeled<T>(this Dictionary<string, T> dic, string key, T value)
    {
      bool flag = dic == null;
      if (flag)
      {
        dic = new Dictionary<string, T>();
      }
      bool flag2 = !dic.ContainsKey(key);
      if (flag2)
      {
        dic.Add(key, value);
      }
      else
      {
        dic.Add(key + dic.Count.ToString(), value);
      }
    }

    // Token: 0x0600048C RID: 1164 RVA: 0x00027194 File Offset: 0x00025394
    public static void AddLabeled<T>(this SortedDictionary<string, T> dic, string key, T value)
    {
      bool flag = dic == null;
      if (flag)
      {
        dic = new SortedDictionary<string, T>();
      }
      bool flag2 = !dic.ContainsKey(key);
      if (flag2)
      {
        dic.Add(key, value);
      }
      else
      {
        dic.Add(key + dic.Count.ToString(), value);
      }
    }

    // Token: 0x0600048D RID: 1165 RVA: 0x000271E8 File Offset: 0x000253E8
    public static void AddSkipDuplicate<T1, T2>(this Dictionary<T1, T2> dic, T1 key, T2 value)
    {
      bool flag = dic == null;
      if (flag)
      {
        dic = new Dictionary<T1, T2>();
      }
      bool flag2 = !dic.ContainsKey(key);
      if (flag2)
      {
        dic.Add(key, value);
      }
    }

    // Token: 0x0600048E RID: 1166 RVA: 0x0002721C File Offset: 0x0002541C
    public static void AddSkipDuplicate<T1, T2>(this SortedDictionary<T1, T2> dic, T1 key, T2 value)
    {
      bool flag = dic == null;
      if (flag)
      {
        dic = new SortedDictionary<T1, T2>();
      }
      bool flag2 = !dic.ContainsKey(key);
      if (flag2)
      {
        dic.Add(key, value);
      }
    }

    // Token: 0x0600048F RID: 1167 RVA: 0x00027250 File Offset: 0x00025450
    public static string AsString<T1, T2>(this Dictionary<T1, T2> dic, string pairator = "", string separator = "")
    {
      StringBuilder stringBuilder = new StringBuilder();
      bool flag = dic != null && dic.Count > 0;
      if (flag)
      {
        int num = dic.Count;
        using (Dictionary<T1, T2>.Enumerator enumerator = dic.GetEnumerator())
        {
          while (enumerator.MoveNext())
          {
            num--;
            StringBuilder stringBuilder2 = stringBuilder;
            KeyValuePair<T1, T2> keyValuePair = enumerator.Current;
            T1 key = keyValuePair.Key;
            stringBuilder2.Append(key.ToString());
            stringBuilder.Append(pairator);
            StringBuilder stringBuilder3 = stringBuilder;
            keyValuePair = enumerator.Current;
            T2 value = keyValuePair.Value;
            stringBuilder3.Append(value.ToString());
            bool flag2 = num > 0;
            if (flag2)
            {
              stringBuilder.Append(separator);
            }
          }
        }
      }
      return stringBuilder.ToString();
    }

    // Token: 0x06000490 RID: 1168 RVA: 0x00027334 File Offset: 0x00025534
    public static string AsString<T1, T2>(this SortedDictionary<T1, T2> dic, string pairator = "", string separator = "")
    {
      StringBuilder stringBuilder = new StringBuilder();
      bool flag = dic != null && dic.Count > 0;
      if (flag)
      {
        int num = dic.Count;
        using (SortedDictionary<T1, T2>.Enumerator enumerator = dic.GetEnumerator())
        {
          while (enumerator.MoveNext())
          {
            num--;
            StringBuilder stringBuilder2 = stringBuilder;
            KeyValuePair<T1, T2> keyValuePair = enumerator.Current;
            T1 key = keyValuePair.Key;
            stringBuilder2.Append(key.ToString());
            stringBuilder.Append(pairator);
            StringBuilder stringBuilder3 = stringBuilder;
            keyValuePair = enumerator.Current;
            T2 value = keyValuePair.Value;
            stringBuilder3.Append(value.ToString());
            bool flag2 = num > 0;
            if (flag2)
            {
              stringBuilder.Append(separator);
            }
          }
        }
      }
      return stringBuilder.ToString();
    }

    // Token: 0x06000491 RID: 1169 RVA: 0x00027418 File Offset: 0x00025618
    public static T2 GetValue<T1, T2>(this Dictionary<T1, T2> dic, T1 key)
    {
      bool flag = dic != null && dic.ContainsKey(key);
      T2 result;
      if (flag)
      {
        result = dic[key];
      }
      else
      {
        result = default(T2);
      }
      return result;
    }

    // Token: 0x06000492 RID: 1170 RVA: 0x00027450 File Offset: 0x00025650
    public static T2 GetValue<T1, T2>(this SortedDictionary<T1, T2> dic, T1 key)
    {
      bool flag = dic != null && dic.ContainsKey(key);
      T2 result;
      if (flag)
      {
        result = dic[key];
      }
      else
      {
        result = default(T2);
      }
      return result;
    }

    // Token: 0x06000493 RID: 1171 RVA: 0x00027488 File Offset: 0x00025688
    public static T1 KeyByValue<T1, T2>(this Dictionary<T1, T2> dic, T2 value)
    {
      bool flag = dic != null;
      if (flag)
      {
        foreach (T1 t in dic.Keys)
        {
          bool flag2;
          if (dic[t] != null)
          {
            T2 t2 = dic[t];
            flag2 = (t2.ToString() == value.ToString());
          }
          else
          {
            flag2 = false;
          }
          bool flag3 = flag2;
          if (flag3)
          {
            return t;
          }
        }
      }
      return default(T1);
    }

    // Token: 0x06000494 RID: 1172 RVA: 0x00027538 File Offset: 0x00025738
    public static string KeyByValue<T>(this SortedDictionary<string, T> dic, T value)
    {
      bool flag = dic != null;
      if (flag)
      {
        foreach (string text in dic.Keys)
        {
          T t = dic[text];
          bool flag2 = t.Equals(value);
          if (flag2)
          {
            return text;
          }
        }
      }
      return "";
    }

    // Token: 0x06000495 RID: 1173 RVA: 0x000275C4 File Offset: 0x000257C4
    public static void Merge<T1, T2>(this Dictionary<T1, T2> dic1, Dictionary<T1, T2> dic2)
    {
      bool flag = dic1 == null;
      if (flag)
      {
        dic1 = new Dictionary<T1, T2>();
      }
      foreach (T1 key in dic2.Keys)
      {
        bool flag2 = !dic1.ContainsKey(key);
        if (flag2)
        {
          dic1.Add(key, dic2[key]);
        }
      }
    }

    // Token: 0x06000496 RID: 1174 RVA: 0x00027644 File Offset: 0x00025844
    public static void AddFromList<T>(this List<T> l, List<T> otherList)
    {
      bool flag = l != null && !otherList.NullOrEmpty<T>();
      if (flag)
      {
        foreach (T item in otherList)
        {
          bool flag2 = !l.Contains(item);
          if (flag2)
          {
            l.Add(item);
          }
        }
      }
    }

    // Token: 0x06000497 RID: 1175 RVA: 0x000276BC File Offset: 0x000258BC
    public static bool IsSame<T>(this List<T> l, List<T> otherList)
    {
      foreach (T item in otherList)
      {
        bool flag = !l.Contains(item);
        if (flag)
        {
          return false;
        }
      }
      foreach (T item2 in l)
      {
        bool flag2 = !l.Contains(item2);
        if (flag2)
        {
          return false;
        }
      }
      return true;
    }

    // Token: 0x06000498 RID: 1176 RVA: 0x00027770 File Offset: 0x00025970
    public static string ListToString<T>(this List<T> list)
    {
      StringBuilder stringBuilder = new StringBuilder();
      bool flag = list != null;
      string result;
      if (flag)
      {
        bool flag2 = typeof(T) == typeof(string);
        if (flag2)
        {
          foreach (string str in list.OfType<string>())
          {
            stringBuilder.Append(str + "|");
          }
        }
        bool flag3 = typeof(T) == typeof(int);
        if (flag3)
        {
          foreach (int num in list.OfType<int>())
          {
            stringBuilder.Append(num.ToString() + "|");
          }
        }
        else
        {
          bool flag4 = typeof(T) == typeof(StuffCategoryDef);
          if (flag4)
          {
            foreach (StuffCategoryDef stuffCategoryDef in list.OfType<StuffCategoryDef>())
            {
              bool flag5 = stuffCategoryDef != null && stuffCategoryDef.defName != null;
              if (flag5)
              {
                stringBuilder.Append(stuffCategoryDef.defName + "|");
              }
            }
          }
          else
          {
            bool flag6 = typeof(T) == typeof(ThingCategoryDef);
            if (flag6)
            {
              foreach (ThingCategoryDef thingCategoryDef in list.OfType<ThingCategoryDef>())
              {
                bool flag7 = thingCategoryDef != null && thingCategoryDef.label != null;
                if (flag7)
                {
                  stringBuilder.Append(thingCategoryDef.label + "|");
                }
              }
            }
            else
            {
              bool flag8 = typeof(T) == typeof(StatModifier);
              if (flag8)
              {
                foreach (StatModifier statModifier in list.OfType<StatModifier>())
                {
                  bool flag9 = statModifier != null && statModifier.stat != null && statModifier.stat.defName != null;
                  if (flag9)
                  {
                    stringBuilder.Append(statModifier.stat.defName + "!" + statModifier.value.ToString() + "|");
                  }
                }
              }
              else
              {
                bool flag10 = typeof(T) == typeof(BodyPartRecord);
                if (flag10)
                {
                  foreach (BodyPartRecord bodyPartRecord in list.OfType<BodyPartRecord>())
                  {
                    bool flag11 = bodyPartRecord != null && bodyPartRecord.def.defName != null;
                    if (flag11)
                    {
                      stringBuilder.Append(bodyPartRecord.def.defName + "|");
                    }
                  }
                }
                else
                {
                  bool flag12 = typeof(T) == typeof(BodyPartGroupDef);
                  if (flag12)
                  {
                    foreach (BodyPartGroupDef bodyPartGroupDef in list.OfType<BodyPartGroupDef>())
                    {
                      bool flag13 = bodyPartGroupDef != null && bodyPartGroupDef.defName != null;
                      if (flag13)
                      {
                        stringBuilder.Append(bodyPartGroupDef.defName + "|");
                      }
                    }
                  }
                  else
                  {
                    bool flag14 = typeof(T) == typeof(BodyTypeDef);
                    if (flag14)
                    {
                      foreach (BodyTypeDef bodyTypeDef in list.OfType<BodyTypeDef>())
                      {
                        bool flag15 = bodyTypeDef != null && bodyTypeDef.defName != null;
                        if (flag15)
                        {
                          stringBuilder.Append(bodyTypeDef.defName + " " + bodyTypeDef.bodyNakedGraphicPath + "\n");
                        }
                      }
                    }
                    else
                    {
                      bool flag16 = typeof(T) == typeof(ApparelLayerDef);
                      if (flag16)
                      {
                        foreach (ApparelLayerDef apparelLayerDef in list.OfType<ApparelLayerDef>())
                        {
                          bool flag17 = apparelLayerDef != null && apparelLayerDef.defName != null;
                          if (flag17)
                          {
                            stringBuilder.Append(apparelLayerDef.defName + "|");
                          }
                        }
                      }
                      else
                      {
                        bool flag18 = typeof(T) == typeof(ThingDef);
                        if (flag18)
                        {
                          foreach (ThingDef thingDef in list.OfType<ThingDef>())
                          {
                            bool flag19 = thingDef != null && thingDef.defName != null;
                            if (flag19)
                            {
                              stringBuilder.Append(thingDef.defName + "|");
                            }
                          }
                        }
                        else
                        {
                          bool flag20 = typeof(T) == typeof(ThingStyleDef);
                          if (flag20)
                          {
                            foreach (ThingStyleDef thingStyleDef in list.OfType<ThingStyleDef>())
                            {
                              bool flag21 = thingStyleDef != null && thingStyleDef.defName != null;
                              if (flag21)
                              {
                                stringBuilder.Append(thingStyleDef.defName + "|");
                              }
                            }
                          }
                          else
                          {
                            bool flag22 = typeof(T) == typeof(TraitDef);
                            if (flag22)
                            {
                              foreach (TraitDef traitDef in list.OfType<TraitDef>())
                              {
                                bool flag23 = traitDef != null && traitDef.defName != null;
                                if (flag23)
                                {
                                  stringBuilder.Append(traitDef.defName + " (" + traitDef.degreeDatas.ListToString<TraitDegreeData>() + ")|");
                                }
                              }
                            }
                            else
                            {
                              bool flag24 = typeof(T) == typeof(SkillDef);
                              if (flag24)
                              {
                                foreach (SkillDef skillDef in list.OfType<SkillDef>())
                                {
                                  bool flag25 = skillDef != null && skillDef.defName != null;
                                  if (flag25)
                                  {
                                    stringBuilder.Append(skillDef.defName + "|");
                                  }
                                }
                              }
                              else
                              {
                                bool flag26 = typeof(T) == typeof(TraitDegreeData);
                                if (flag26)
                                {
                                  foreach (TraitDegreeData traitDegreeData in list.OfType<TraitDegreeData>())
                                  {
                                    bool flag27 = traitDegreeData != null && traitDegreeData.label != null;
                                    if (flag27)
                                    {
                                      stringBuilder.Append(traitDegreeData.label + "|");
                                    }
                                  }
                                }
                                else
                                {
                                  bool flag28 = typeof(T) == typeof(Apparel);
                                  if (flag28)
                                  {
                                    foreach (Apparel apparel in list.OfType<Apparel>())
                                    {
                                      bool flag29 = apparel != null && apparel.Label != null;
                                      if (flag29)
                                      {
                                        stringBuilder.Append(apparel.Label + ", ");
                                      }
                                    }
                                  }
                                  else
                                  {
                                    bool flag30 = typeof(T) == typeof(Pawn);
                                    if (flag30)
                                    {
                                      foreach (Pawn pawn in list.OfType<Pawn>())
                                      {
                                        bool flag31 = pawn != null && pawn.Label != null;
                                        if (flag31)
                                        {
                                          stringBuilder.Append(pawn.Label + "|");
                                        }
                                      }
                                    }
                                    else
                                    {
                                      bool flag32 = typeof(T) == typeof(Texture2D);
                                      if (flag32)
                                      {
                                        foreach (Texture2D texture2D in list.OfType<Texture2D>())
                                        {
                                          bool flag33 = texture2D != null && texture2D.name != null;
                                          if (flag33)
                                          {
                                            stringBuilder.Append(texture2D.name + "\n");
                                          }
                                        }
                                      }
                                      else
                                      {
                                        bool flag34 = typeof(T) == typeof(Def);
                                        if (flag34)
                                        {
                                          foreach (Def def in list.OfType<Def>())
                                          {
                                            bool flag35 = def != null && def.defName != null;
                                            if (flag35)
                                            {
                                              stringBuilder.Append(def.defName + "|");
                                            }
                                          }
                                        }
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
        string text = stringBuilder.ToString();
        bool flag36 = !string.IsNullOrEmpty(text);
        if (flag36)
        {
          text = text.Remove(text.Length - 1, 1);
        }
        result = text;
      }
      else
      {
        result = "";
      }
      return result;
    }

    public static T GetNextPrevRandom<T>(this List<T> list, T oldItem, bool next, bool random)
    {
      return list.NullOrEmpty<T>() ? default(T) : list[list.NextOrPrevIndex(list.IndexOf(oldItem), next, random)];
    }

    public static int NextOrPrevIndex<T>(this List<T> l, int index, bool next, bool random)
    {
      return l.NullOrEmpty<T>() ? -1 : l.Count().GetNewIndexFromCount(index, next, random);
    }

    public static int GetNewIndexFromCount(this int count, int index, bool next, bool random)
    {
      if (count <= 1)
      {
        return 0;
      }
      if (random)
      {
        List<int> ints = new List<int>();
        for (int i = 0; i < count; i++)
        {
          if (i != index)
          {
            ints.Add(i);
          }
        }
        return ints.RandomElement();
      }
      index += next ? 1 : -1;
      if (index > count - 1)
      {
        return 0;
      }
      if (index < 0)
      {
        return count - 1;
      }
      return index;
    }

    //public static int NextOrPrevIndex<T>(this HashSet<T> l, int index, bool next, bool random)
    //{
    //  return l.ToList().NextOrPrevIndex(index, next, random);
    //}

    //public static int IndexOf<T>(this HashSet<T> l, T val)
    //{
    //  return (l.NullOrEmpty<T>() || val == null) ? 0 : l.FirstIndexOf((T y) => val.Equals(y));
    //}

    //public static T At<T>(this HashSet<T> l, int index)
    //{
    //  return l.NullOrEmpty<T>() ? default(T) : l.ElementAt(index);
    //}

    public static T At<T>(this List<T> l, int index)
    {
      return l.NullOrEmpty<T>() ? default(T) : l.ElementAt(index);
    }

    public static bool NullOrEmpty<T>(this HashSet<T> l)
    {
      return l == null || l.Count == 0;
    }

    //public static void RemoveDuplicates<T>(this HashSet<T> l) where T : Def
    //{
    //  HashSet<T> hashSet = new HashSet<T>();
    //  foreach (T item in l)
    //  {
    //    bool flag = !hashSet.Contains(item);
    //    if (flag)
    //    {
    //      hashSet.Add(item);
    //    }
    //  }
    //  l = hashSet;
    //}

    // Token: 0x060004A0 RID: 1184 RVA: 0x00028470 File Offset: 0x00026670
    public static string[] SplitNo(this string s, string x)
    {
      return s.Split(new string[]
      {
        x
      }, StringSplitOptions.None);
    }

    // Token: 0x060004A1 RID: 1185 RVA: 0x00028484 File Offset: 0x00026684
    public static bool AsBool(this string s)
    {
      return s == "1";
    }

    // Token: 0x060004A2 RID: 1186 RVA: 0x000284A4 File Offset: 0x000266A4
    public static bool AsBoolWithDefault(this string s, bool defVal)
    {
      return s.NullOrEmpty() ? defVal : (s == "1");
    }

    // Token: 0x060004A3 RID: 1187 RVA: 0x000284CC File Offset: 0x000266CC
    public static float AsFloat(this string input)
    {
      float result = 0f;
      float.TryParse(input, out result);
      return result;
    }

    // Token: 0x060004A4 RID: 1188 RVA: 0x000284F0 File Offset: 0x000266F0
    public static int AsInt32(this string input)
    {
      int result = 0;
      int.TryParse(input, out result);
      return result;
    }

    // Token: 0x060004A5 RID: 1189 RVA: 0x00028510 File Offset: 0x00026710
    public static long AsLong(this string input)
    {
      long result = 0L;
      long.TryParse(input, out result);
      return result;
    }

    // Token: 0x060004A6 RID: 1190 RVA: 0x00028530 File Offset: 0x00026730
    public static KeyCode AsKeyCode(this string key)
    {
      KeyCode result;
      try
      {
        result = (KeyCode)Enum.Parse(typeof(KeyCode), key);
      }
      catch
      {
        result = KeyCode.None;
      }
      return result;
    }

    // Token: 0x060004A7 RID: 1191 RVA: 0x00028574 File Offset: 0x00026774
    public static List<string> StringToList(this string slist)
    {
      List<string> list = new List<string>();
      bool flag = !string.IsNullOrEmpty(slist);
      if (flag)
      {
        string[] array = slist.Split(new char[]
        {
          '|'
        });
        foreach (string item in array)
        {
          list.Add(item);
        }
      }
      return list.NullOrEmpty<string>() ? new List<string>() : list;
    }

    // Token: 0x060004A8 RID: 1192 RVA: 0x000285E4 File Offset: 0x000267E4
    public static List<ApparelLayerDef> StringToListApparelLayer(this string slist, ref bool errorOccured)
    {
      List<ApparelLayerDef> list = new List<ApparelLayerDef>();
      bool flag = !string.IsNullOrEmpty(slist);
      if (flag)
      {
        string[] array = slist.Split(new char[]
        {
          '|'
        });
        foreach (string defName in array)
        {
          ApparelLayerDef apparelLayerDef = DefTool.ApparelLayerDef(defName);
          bool flag2 = apparelLayerDef != null;
          if (flag2)
          {
            list.Add(apparelLayerDef);
          }
          else
          {
            errorOccured = true;
          }
        }
      }
      return list.NullOrEmpty<ApparelLayerDef>() ? new List<ApparelLayerDef>() : list;
    }

    // Token: 0x060004A9 RID: 1193 RVA: 0x00028670 File Offset: 0x00026870
    public static List<BodyPartGroupDef> StringToListBodyPartGroup(this string slist, ref bool errorOccured)
    {
      List<BodyPartGroupDef> list = new List<BodyPartGroupDef>();
      bool flag = !string.IsNullOrEmpty(slist);
      if (flag)
      {
        string[] array = slist.Split(new char[]
        {
          '|'
        });
        foreach (string defName in array)
        {
          BodyPartGroupDef bodyPartGroupDef = DefTool.BodyPartGroupDef(defName);
          bool flag2 = bodyPartGroupDef != null;
          if (flag2)
          {
            list.Add(bodyPartGroupDef);
          }
          else
          {
            errorOccured = true;
          }
        }
      }
      return list.NullOrEmpty<BodyPartGroupDef>() ? new List<BodyPartGroupDef>() : list;
    }

    // Token: 0x060004AA RID: 1194 RVA: 0x000286FC File Offset: 0x000268FC
    public static List<StatModifier> StringToListStatModifier(this string slist, ref bool errorOccured)
    {
      List<StatModifier> list = new List<StatModifier>();
      bool flag = !string.IsNullOrEmpty(slist);
      if (flag)
      {
        string[] array = slist.Split(new char[]
        {
          '|'
        });
        foreach (string text in array)
        {
          string[] array3 = text.Split(new char[]
          {
            '!'
          });
          string defName = array3[0];
          string input = array3[1];
          StatModifier statModifier = new StatModifier();
          statModifier.stat = DefTool.StatDef(defName);
          bool flag2 = statModifier.stat != null;
          if (flag2)
          {
            statModifier.value = input.AsFloat();
            list.Add(statModifier);
          }
          else
          {
            errorOccured = true;
          }
        }
      }
      return list.NullOrEmpty<StatModifier>() ? new List<StatModifier>() : list;
    }

    // Token: 0x060004AB RID: 1195 RVA: 0x000287CC File Offset: 0x000269CC
    public static List<StuffCategoryDef> StringToListStuffCategory(this string slist, ref bool errorOccured)
    {
      List<StuffCategoryDef> list = new List<StuffCategoryDef>();
      bool flag = !string.IsNullOrEmpty(slist);
      if (flag)
      {
        string[] array = slist.Split(new char[]
        {
          '|'
        });
        foreach (string defName in array)
        {
          StuffCategoryDef stuffCategoryDef = DefTool.StuffCategoryDef(defName);
          bool flag2 = stuffCategoryDef != null;
          if (flag2)
          {
            list.Add(stuffCategoryDef);
          }
          else
          {
            errorOccured = true;
          }
        }
      }
      return list.NullOrEmpty<StuffCategoryDef>() ? new List<StuffCategoryDef>() : list;
    }

    // Token: 0x060004AC RID: 1196 RVA: 0x00028858 File Offset: 0x00026A58
    public static string SubstringBackwardFrom(this string text, string startFrom, bool withoutIt = true)
    {
      bool flag = text != null;
      if (flag)
      {
        int num = text.LastIndexOf(startFrom);
        bool flag2 = num >= 0;
        if (flag2)
        {
          if (withoutIt)
          {
            return text.Substring(num + startFrom.Length);
          }
          return text.Substring(num);
        }
      }
      return text;
    }

    // Token: 0x060004AD RID: 1197 RVA: 0x000288B0 File Offset: 0x00026AB0
    public static string SubstringBackwardTo(this string text, string endOn, bool withoutIt = true)
    {
      bool flag = text != null;
      if (flag)
      {
        int num = text.LastIndexOf(endOn);
        bool flag2 = num >= 0;
        if (flag2)
        {
          if (withoutIt)
          {
            return text.Substring(0, num);
          }
          return text.Substring(num);
        }
      }
      return text;
    }

    // Token: 0x060004AE RID: 1198 RVA: 0x00028900 File Offset: 0x00026B00
    public static string SubstringFrom(this string text, string from, int occuranceCount)
    {
      string text2 = text;
      for (int i = 0; i < occuranceCount; i++)
      {
        text2 = text2.SubstringFrom(from, true);
      }
      return text2;
    }

    // Token: 0x060004AF RID: 1199 RVA: 0x00028930 File Offset: 0x00026B30
    public static string SubstringFrom(this string text, string startFrom, bool withoutIt = true)
    {
      bool flag = text != null;
      if (flag)
      {
        int num = text.IndexOf(startFrom);
        bool flag2 = num >= 0;
        if (flag2)
        {
          if (withoutIt)
          {
            return text.Substring(num + startFrom.Length);
          }
          return text.Substring(num);
        }
      }
      return text;
    }

    // Token: 0x060004B0 RID: 1200 RVA: 0x00028988 File Offset: 0x00026B88
    public static string SubstringTo(this string text, string to, int occuranceCount)
    {
      string text2 = text;
      string text3 = "";
      for (int i = 0; i < occuranceCount; i++)
      {
        text3 += text2.SubstringTo(to, false);
        text2 = text2.SubstringFrom(to, true);
      }
      bool flag = text3.Length > 0;
      string result;
      if (flag)
      {
        result = text3.Substring(0, text3.Length - 1);
      }
      else
      {
        result = text3;
      }
      return result;
    }

    // Token: 0x060004B1 RID: 1201 RVA: 0x000289F4 File Offset: 0x00026BF4
    public static string SubstringTo(this string text, string endOn, bool withoutIt = true)
    {
      bool flag = text != null;
      if (flag)
      {
        int num = text.IndexOf(endOn);
        bool flag2 = num >= 0;
        if (flag2)
        {
          if (withoutIt)
          {
            return text.Substring(0, num);
          }
          return text.Substring(0, num + endOn.Length);
        }
      }
      return text;
    }

    // Token: 0x060004B2 RID: 1202 RVA: 0x00028A4B File Offset: 0x00026C4B
    public static string SubstringRemoveLast(this string text)
    {
      return text.NullOrEmpty() ? text : text.Substring(0, text.Length - 1);
    }

    // Token: 0x060004B3 RID: 1203 RVA: 0x00028A68 File Offset: 0x00026C68
    public static string AsStringUNICODE(this byte[] input)
    {
      return input.AsString(Encoding.UTF8);
    }

    // Token: 0x060004B4 RID: 1204 RVA: 0x00028A88 File Offset: 0x00026C88
    public static string AsString(this byte[] input, Encoding enc)
    {
      bool flag = input.NullOrEmpty<byte>();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        result = enc.GetString(input);
      }
      return result;
    }

    // Token: 0x060004B5 RID: 1205 RVA: 0x00028AB4 File Offset: 0x00026CB4
    public static byte[] AsBytes(this string text, Encoding enc)
    {
      bool flag = text == null;
      byte[] result;
      if (flag)
      {
        result = null;
      }
      else
      {
        bool flag2 = enc == null;
        if (flag2)
        {
          result = Encoding.Default.GetBytes(text);
        }
        else
        {
          result = enc.GetBytes(text);
        }
      }
      return result;
    }

    // Token: 0x060004B6 RID: 1206 RVA: 0x00028AF0 File Offset: 0x00026CF0
    public static string AsBase64(this string text, Encoding enc)
    {
      byte[] inArray = text.AsBytes(enc);
      return Convert.ToBase64String(inArray);
    }

    // Token: 0x060004B7 RID: 1207 RVA: 0x00028B10 File Offset: 0x00026D10
    public static byte[] Base64ToBytes(this string base64)
    {
      byte[] result = new byte[0];
      bool flag = !string.IsNullOrEmpty(base64);
      if (flag)
      {
        try
        {
          result = Convert.FromBase64String(base64);
        }
        catch
        {
        }
      }
      return result;
    }

    // Token: 0x060004B8 RID: 1208 RVA: 0x00028B5C File Offset: 0x00026D5C
    public static string Base64ToString(this string base64, Encoding enc)
    {
      bool flag = base64 == null;
      string result;
      if (flag)
      {
        result = null;
      }
      else
      {
        bool flag2 = enc == null;
        if (flag2)
        {
          result = Encoding.Default.GetString(base64.Base64ToBytes());
        }
        else
        {
          result = enc.GetString(base64.Base64ToBytes());
        }
      }
      return result;
    }

    // Token: 0x060004B9 RID: 1209 RVA: 0x00028BA4 File Offset: 0x00026DA4
    public static bool HasMID(string packageId)
    {
      ModMetaData modWithIdentifier = ModLister.GetModWithIdentifier(packageId, false);
      return modWithIdentifier != null && modWithIdentifier.Active;
    }
  }
}
