﻿using System;
using System.Collections.Generic;
using System.Linq;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000019 RID: 25
	public static class FacialTool
	{
		// Token: 0x060000E7 RID: 231 RVA: 0x00007280 File Offset: 0x00005480
		public static string GetFacialAnimationParams(this Pawn p)
		{
			bool flag = p == null || !CEditor.IsFacialAnimationActive;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				text = text + p.FA_GetCurrentDefName(FacialTool.FACE) + "|";
				text = text + p.FA_GetCurrentDefName(FacialTool.EYE) + "|";
				text = text + p.FA_GetCurrentDefName(FacialTool.LID) + "|";
				text = text + p.FA_GetCurrentDefName(FacialTool.BROW) + "|";
				text = text + p.FA_GetCurrentDefName(FacialTool.MOUTH) + "|";
				text += p.FA_GetCurrentDefName(FacialTool.SKIN);
				result = text;
			}
			return result;
		}

		// Token: 0x060000E8 RID: 232 RVA: 0x0000733C File Offset: 0x0000553C
		public static void SetFacialAnimationParams(this Pawn p, string s)
		{
			bool flag = p == null || !CEditor.IsFacialAnimationActive || s.NullOrEmpty();
			if (!flag)
			{
				string[] array = s.SplitNo("|");
				bool flag2 = array.Length >= 6;
				if (flag2)
				{
					p.FA_SetDefByName(FacialTool.FACE, array[0].Trim());
					p.FA_SetDefByName(FacialTool.EYE, array[1].Trim());
					p.FA_SetDefByName(FacialTool.LID, array[2].Trim());
					p.FA_SetDefByName(FacialTool.BROW, array[3].Trim());
					p.FA_SetDefByName(FacialTool.MOUTH, array[4].Trim());
					p.FA_SetDefByName(FacialTool.SKIN, array[5].Trim());
				}
			}
		}

		// Token: 0x060000E9 RID: 233 RVA: 0x000073FC File Offset: 0x000055FC
		public static ThingComp GetRJWComp(this Pawn p)
		{
			bool flag = p == null || p.AllComps.NullOrEmpty<ThingComp>() || !CEditor.IsRJWActive;
			ThingComp result;
			if (flag)
			{
				result = null;
			}
			else
			{
				foreach (ThingComp thingComp in p.AllComps)
				{
					bool flag2 = thingComp.GetType().ToString().StartsWith("rjw");
					if (flag2)
					{
						bool flag3 = thingComp.GetType().ToString().EndsWith("CompRJW");
						if (flag3)
						{
							return thingComp;
						}
					}
				}
				result = null;
			}
			return result;
		}

		// Token: 0x060000EA RID: 234 RVA: 0x000074B4 File Offset: 0x000056B4
		public static ThingComp GetPersoComp(this Pawn p)
		{
			bool flag = p == null || p.AllComps.NullOrEmpty<ThingComp>() || !CEditor.IsPersonalitiesActive;
			ThingComp result;
			if (flag)
			{
				result = null;
			}
			else
			{
				foreach (ThingComp thingComp in p.AllComps)
				{
					bool flag2 = thingComp.GetType().ToString().StartsWith("SPM1");
					if (flag2)
					{
						bool flag3 = thingComp.GetType().ToString().EndsWith("CompEnneagram");
						if (flag3)
						{
							return thingComp;
						}
					}
				}
				result = null;
			}
			return result;
		}

		// Token: 0x060000EB RID: 235 RVA: 0x0000756C File Offset: 0x0000576C
		public static ThingComp FA_GetControllerComp(this Pawn p, string controller)
		{
			bool flag = p == null || p.AllComps.NullOrEmpty<ThingComp>() || !CEditor.IsFacialAnimationActive;
			ThingComp result;
			if (flag)
			{
				result = null;
			}
			else
			{
				foreach (ThingComp thingComp in p.AllComps)
				{
					bool flag2 = thingComp.GetType().ToString().StartsWith("FacialAnimation");
					if (flag2)
					{
						bool flag3 = thingComp.GetType().ToString().EndsWith(controller);
						if (flag3)
						{
							return thingComp;
						}
					}
				}
				result = null;
			}
			return result;
		}

		// Token: 0x060000EC RID: 236 RVA: 0x00007620 File Offset: 0x00005820
		public static string FA_GetCurrentDefName(this Pawn p, string controller)
		{
			object obj = p.FA_GetCurrentDef(controller);
			return (obj != null) ? ((Def)obj).defName : "";
		}

		// Token: 0x060000ED RID: 237 RVA: 0x00007650 File Offset: 0x00005850
		public static object FA_GetCurrentDef(this Pawn p, string controller)
		{
			ThingComp thingComp = p.FA_GetControllerComp(controller);
			return (thingComp != null) ? thingComp.GetMemberValue(FacialTool.FACETYPE) : null;
		}

		// Token: 0x060000EE RID: 238 RVA: 0x0000767C File Offset: 0x0000587C
		public static List<string> FA_GetDefStringList(this Pawn p, string controller)
		{
			List<string> list = new List<string>();
			object obj = p.FA_GetCurrentDef(controller);
			bool flag = obj != null;
			if (flag)
			{
				List<Def> list2 = FacialTool.FA_FilterOutIncompatible(p, controller, FacialTool.FA_GetDefList(obj));
				bool flag2 = list2 != null;
				if (flag2)
				{
					foreach (Def def in list2)
					{
						list.Add(def.defName);
					}
				}
			}
			return list;
		}

		// Token: 0x060000EF RID: 239 RVA: 0x00007710 File Offset: 0x00005910
		public static List<Def> FA_GetDefList(object curDef)
		{
			return GenDefDatabase.GetAllDefsInDatabaseForDef(curDef.GetType()).ToList<Def>();
		}

		// Token: 0x060000F0 RID: 240 RVA: 0x00007724 File Offset: 0x00005924
		public static List<Def> FA_FilterOutIncompatible(Pawn p, string controller, List<Def> l)
		{
			bool flag = p == null || l == null || l.Count == 1;
			List<Def> result;
			if (flag)
			{
				result = l;
			}
			else
			{
				List<Def> list = new List<Def>();
				bool flag2 = controller == FacialTool.FACE;
				if (flag2)
				{
					bool flag3 = p.gender == Gender.Male;
					if (flag3)
					{
						foreach (Def def in l)
						{
							bool flag4 = def.defName != "HeadPointy";
							if (flag4)
							{
								list.Add(def);
							}
						}
					}
					else
					{
						foreach (Def def2 in l)
						{
							bool flag5 = def2.defName != "HeadSquare";
							if (flag5)
							{
								list.Add(def2);
							}
						}
					}
					result = list;
				}
				else
				{
					bool flag6 = controller == FacialTool.EYE;
					if (flag6)
					{
						bool flag7 = p.gender == Gender.Male;
						if (flag7)
						{
							foreach (Def def3 in l)
							{
								bool flag8 = def3.defName != "EyeDull";
								if (flag8)
								{
									list.Add(def3);
								}
							}
							result = list;
						}
						else
						{
							result = l;
						}
					}
					else
					{
						bool flag9 = controller == FacialTool.LID;
						if (flag9)
						{
							bool flag10 = p.gender == Gender.Male;
							if (flag10)
							{
								foreach (Def def4 in l)
								{
									bool flag11 = def4.defName != "LidPointy" && def4.defName != "LidFlashy" && def4.defName != "LidQuite" && def4.defName != "LidSleepy";
									if (flag11)
									{
										list.Add(def4);
									}
								}
								result = list;
							}
							else
							{
								result = l;
							}
						}
						else
						{
							result = l;
						}
					}
				}
			}
			return result;
		}

		// Token: 0x060000F1 RID: 241 RVA: 0x0000799C File Offset: 0x00005B9C
		public static Def FA_GetDefByName(Pawn p, string controller, string defName)
		{
			object obj = p.FA_GetCurrentDef(controller);
			bool flag = obj == null;
			Def result;
			if (flag)
			{
				result = null;
			}
			else
			{
				List<Def> source = FacialTool.FA_FilterOutIncompatible(p, controller, FacialTool.FA_GetDefList(obj));
				result = (from td in source
				where td.defName == defName
				select td).FirstOrDefault<Def>();
			}
			return result;
		}

		// Token: 0x060000F2 RID: 242 RVA: 0x000079F8 File Offset: 0x00005BF8
		public static void FA_SetDefByName(this Pawn p, string controller, string defName)
		{
			ThingComp thingComp = p.FA_GetControllerComp(controller);
			bool flag = thingComp != null;
			if (flag)
			{
				thingComp.SetMemberValue(FacialTool.FACETYPE, FacialTool.FA_GetDefByName(p, controller, defName));
				thingComp.PostExposeData();
			}
		}

		// Token: 0x060000F3 RID: 243 RVA: 0x00007A34 File Offset: 0x00005C34
		public static bool FA_SetDef(this Pawn p, string controller, bool next, bool random)
		{
			ThingComp thingComp = p.FA_GetControllerComp(controller);
			bool flag = thingComp != null;
			if (flag)
			{
				object memberValue = thingComp.GetMemberValue(FacialTool.FACETYPE);
				bool flag2 = memberValue != null;
				if (flag2)
				{
					string defName = ((Def)memberValue).defName;
					List<Def> list = FacialTool.FA_FilterOutIncompatible(p, controller, FacialTool.FA_GetDefList(memberValue));
					bool flag3 = list != null;
					if (flag3)
					{
						int index = 0;
						for (int i = 0; i < list.Count; i++)
						{
							bool flag4 = list[i].defName == defName;
							if (flag4)
							{
								index = i;
								break;
							}
						}
						index = list.NextOrPrevIndex(index, next, random);
						thingComp.SetMemberValue(FacialTool.FACETYPE, list[index]);
						return true;
					}
				}
				thingComp.PostExposeData();
			}
			return false;
		}

		// Token: 0x040000D2 RID: 210
		internal static string FACE = "HeadControllerComp";

		// Token: 0x040000D3 RID: 211
		internal static string EYE = "EyeballControllerComp";

		// Token: 0x040000D4 RID: 212
		internal static string LID = "LidControllerComp";

		// Token: 0x040000D5 RID: 213
		internal static string BROW = "BrowControllerComp";

		// Token: 0x040000D6 RID: 214
		internal static string MOUTH = "MouthControllerComp";

		// Token: 0x040000D7 RID: 215
		internal static string SKIN = "SkinControllerComp";

		// Token: 0x040000D8 RID: 216
		internal static string FACETYPE = "faceType";
	}
}
