﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200002B RID: 43
	public static class FactionTool
	{
		// Token: 0x17000040 RID: 64
		// (get) Token: 0x060001E3 RID: 483 RVA: 0x0000E5DF File Offset: 0x0000C7DF
		public static Dictionary<string, Faction> DicFactions
		{
			get
			{
				return CEditor.API.Get<Dictionary<string, Faction>>(EType.Factions);
			}
		}

		// Token: 0x060001E4 RID: 484 RVA: 0x0000E5EC File Offset: 0x0000C7EC
		public static List<Faction> FactionEnemies(this Pawn pawn)
		{
			bool flag = pawn == null || Find.FactionManager == null;
			List<Faction> result;
			if (flag)
			{
				result = new List<Faction>();
			}
			else
			{
				List<Faction> list = (from f in Find.FactionManager.AllFactionsInViewOrder
				where f != pawn.Faction && pawn.Faction.RelationWith(f, false) != null && !pawn.IsFriendlyFaction(f)
				select f).ToList<Faction>();
				result = list;
			}
			return result;
		}

		// Token: 0x060001E5 RID: 485 RVA: 0x0000E64C File Offset: 0x0000C84C
		public static Dictionary<string, Faction> GetDicOfFactions(bool addPseudoFaction = true, bool addCreatures = true, bool addHumanoids = true)
		{
			Dictionary<string, Faction> dictionary = new Dictionary<string, Faction>();
			List<Faction> list = Current.Game.World.factionManager.AllFactions.ToList<Faction>();
			list.SortBy((Faction s) => s.Name);
			if (addPseudoFaction)
			{
				dictionary.Add(Label.HUMANOID, null);
				dictionary.Add(Label.COLONISTS, Faction.OfPlayer);
				dictionary.Add(Label.COLONYANIMALS, Faction.OfPlayer);
				bool flag = !CEditor.InStartingScreen;
				if (flag)
				{
					dictionary.Add(Label.WILDANIMALS, null);
				}
			}
			foreach (Faction faction in list)
			{
				bool flag2 = !addCreatures && faction.IsCreature();
				if (!flag2)
				{
					bool flag3 = !addHumanoids && faction.IsHumanlike();
					if (!flag3)
					{
						dictionary.AddLabeled((faction.GetCallLabel() ?? faction.Name).CapitalizeFirst(), faction);
					}
				}
			}
			return dictionary;
		}

		// Token: 0x060001E6 RID: 486 RVA: 0x0000E788 File Offset: 0x0000C988
		public static string GetFactionFullDesc(this Pawn pawn, Faction f = null)
		{
			bool flag = pawn == null || pawn.Faction.IsNullOrEmpty();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				text = ((f == null) ? pawn.Faction.Name.CapitalizeFirst() : f.Name.CapitalizeFirst());
				text += "\n";
				bool flag2 = f == null && pawn.Faction != Faction.OfPlayer;
				if (flag2)
				{
					f = Faction.OfPlayer;
				}
				bool flag3 = f != null;
				if (flag3)
				{
					FactionRelation factionRelation = pawn.Faction.RelationWith(f, true);
					bool flag4 = factionRelation != null;
					if (flag4)
					{
						int num = 0;
						try
						{
							num = pawn.Faction.GoodwillWith(f);
						}
						catch
						{
						}
						string str = num.ToString() + " ";
						FactionRelationKind kind = factionRelation.kind;
						text += (str + kind.GetLabel() + "\n").Colorize(kind.GetColor());
					}
				}
				text += "\n";
				bool flag5 = f == null;
				if (flag5)
				{
					string str2 = text;
					FactionDef def = pawn.Faction.def;
					text = str2 + ((def != null) ? def.description : null);
				}
				else
				{
					string str3 = text;
					FactionDef def2 = f.def;
					text = str3 + ((def2 != null) ? def2.description : null);
				}
				result = text;
			}
			return result;
		}

		// Token: 0x060001E7 RID: 487 RVA: 0x0000E8FC File Offset: 0x0000CAFC
		public static Color GetFacionColor(this Faction f)
		{
			return f.HasFactionColor() ? ColorsFromSpectrum.Get(f.def.colorSpectrum, f.colorFromSpectrum) : Color.white;
		}

		// Token: 0x060001E8 RID: 488 RVA: 0x0000E923 File Offset: 0x0000CB23
		public static Color GetFacionColor(this Pawn pawn)
		{
			return (pawn == null) ? Color.white : pawn.Faction.GetFacionColor();
		}

		// Token: 0x060001E9 RID: 489 RVA: 0x0000E93A File Offset: 0x0000CB3A
		public static bool HasFactionColor(this Faction f)
		{
			return !f.IsNullOrEmpty() && !f.def.colorSpectrum.NullOrEmpty<Color>();
		}

		// Token: 0x060001EA RID: 490 RVA: 0x0000E95C File Offset: 0x0000CB5C
		public static void ChangeFaction(this Pawn p, Faction f)
		{
			bool flag = p == null || f == null || p.Faction == f;
			if (!flag)
			{
				p.SetFactionDirect(f);
			}
		}

		// Token: 0x060001EB RID: 491 RVA: 0x0000E98A File Offset: 0x0000CB8A
		public static bool CanCreateRelations(this Faction f, PawnKindDef pkd)
		{
			return f.IsNotInsectXenoMechZombie() && pkd.modContentPack != null && (pkd.modContentPack.IsCoreMod || pkd.modContentPack.Name == "Royalty");
		}

		// Token: 0x060001EC RID: 492 RVA: 0x0000E9C7 File Offset: 0x0000CBC7
		public static bool IsAbomination(this Faction f)
		{
			return !f.IsNullOrEmpty() && f.def.defName == "Abomination";
		}

		// Token: 0x060001ED RID: 493 RVA: 0x0000E9E9 File Offset: 0x0000CBE9
		public static bool IsAnimal(this Faction f, string key)
		{
			return f.IsNotInsectXenoMechZombie() && key != Label.COLONISTS && (key == Label.COLONYANIMALS || key == Label.WILDANIMALS);
		}

		// Token: 0x060001EE RID: 494 RVA: 0x0000EA1E File Offset: 0x0000CC1E
		public static bool IsCreature(this Faction f)
		{
			return f.IsInsektoid() || f.IsXeno() || f.IsMechanoid() || f.IsAbomination() || f.IsZombie();
		}

		// Token: 0x060001EF RID: 495 RVA: 0x0000EA49 File Offset: 0x0000CC49
		public static bool IsFriendlyFaction(this Pawn pawn, Faction f)
		{
			return pawn != null && (f == pawn.Faction || !f.HostileTo(pawn.Faction));
		}

		// Token: 0x060001F0 RID: 496 RVA: 0x0000EA6C File Offset: 0x0000CC6C
		public static bool IsHumanlike(this Faction f)
		{
			return !f.IsNullOrEmpty() && f.def.humanlikeFaction;
		}

		// Token: 0x060001F1 RID: 497 RVA: 0x0000EA84 File Offset: 0x0000CC84
		public static bool IsHumanoid(this Faction f, string key)
		{
			return f.IsNotInsectXenoMechZombie() && key != Label.COLONYANIMALS && key != Label.WILDANIMALS;
		}

		// Token: 0x060001F2 RID: 498 RVA: 0x0000EAA9 File Offset: 0x0000CCA9
		public static bool IsInsektoid(this Faction f)
		{
			return f == Faction.OfInsects;
		}

		// Token: 0x060001F3 RID: 499 RVA: 0x0000EAB3 File Offset: 0x0000CCB3
		public static bool IsMechanoid(this Faction f)
		{
			return f == Faction.OfMechanoids;
		}

		// Token: 0x060001F4 RID: 500 RVA: 0x0000EABD File Offset: 0x0000CCBD
		public static bool IsNotInsectXenoMechZombie(this Faction f)
		{
			return !f.IsInsektoid() && !f.IsXeno() && !f.IsMechanoid() && !f.IsAbomination() && !f.IsZombie();
		}

		// Token: 0x060001F5 RID: 501 RVA: 0x0000EAEB File Offset: 0x0000CCEB
		public static bool IsNullOrEmpty(this Faction f)
		{
			return f == null || f.def == null;
		}

		// Token: 0x060001F6 RID: 502 RVA: 0x0000EAFC File Offset: 0x0000CCFC
		public static bool IsOther(this Faction f, string key)
		{
			return !f.IsNullOrEmpty() && key != Label.COLONISTS && key != Label.COLONYANIMALS;
		}

		// Token: 0x060001F7 RID: 503 RVA: 0x0000EB21 File Offset: 0x0000CD21
		public static bool IsXeno(this Faction f)
		{
			return !f.IsNullOrEmpty() && f.Name.ToLower().Contains("xenomorph");
		}

		// Token: 0x060001F8 RID: 504 RVA: 0x0000EB43 File Offset: 0x0000CD43
		public static bool IsZombie(this Faction f)
		{
			return !f.IsNullOrEmpty() && f.def.defName == "Zombies";
		}

		// Token: 0x060001F9 RID: 505 RVA: 0x0000EB65 File Offset: 0x0000CD65
		public static Faction ThisOrDefault(this Faction f)
		{
			return f ?? FactionTool.GetDicOfFactions(false, false, true).Values.RandomElement<Faction>();
		}

		// Token: 0x060001FA RID: 506 RVA: 0x0000EB80 File Offset: 0x0000CD80
		public static string GetPawnFactionAsSeparatedString(this Pawn p)
		{
			return (p != null && p.Faction != null) ? (p.Faction.Name + "|" + p.Faction.def.defName) : (Faction.OfPlayer.Name + "|" + Faction.OfPlayer.def.defName);
		}

		// Token: 0x060001FB RID: 507 RVA: 0x0000EBE4 File Offset: 0x0000CDE4
		public static Faction GetBySeparatedString(string s, Faction factionFallback)
		{
			bool flag = s.NullOrEmpty();
			Faction result;
			if (flag)
			{
				result = factionFallback;
			}
			else
			{
				Faction faction = null;
				string[] array = s.SplitNo("|");
				bool flag2 = array.Length == 2;
				if (flag2)
				{
					string key = array[0];
					string b = array[1];
					faction = FactionTool.DicFactions.GetValue(key);
					bool flag3 = faction == null;
					if (flag3)
					{
						foreach (Faction faction2 in FactionTool.DicFactions.Values)
						{
							bool flag4 = faction2 != null && faction2.def.defName == b;
							if (flag4)
							{
								faction = faction2;
								break;
							}
						}
					}
				}
				result = (faction ?? factionFallback);
			}
			return result;
		}
	}
}
