﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200002D RID: 45
	public static class HairTool
	{
		// Token: 0x06000205 RID: 517 RVA: 0x0000EED8 File Offset: 0x0000D0D8
		public static Color GetHairColor(this Pawn p, bool primary)
		{
			return p.HasStoryTracker() ? (primary ? p.story.HairColor : p.GetGradientColor()) : Color.white;
		}

		// Token: 0x06000206 RID: 518 RVA: 0x0000EF00 File Offset: 0x0000D100
		public static void SetGradientMask(this Pawn p, string mask)
		{
			bool flag = p == null || mask.NullOrEmpty();
			if (!flag)
			{
				ThingComp gradientComp = p.GetGradientComp();
				bool flag2 = gradientComp != null;
				if (flag2)
				{
					object memberValue = gradientComp.GetMemberValue("settings");
					bool flag3 = memberValue != null;
					if (flag3)
					{
						memberValue.SetMemberValue("mask", mask);
						memberValue.SetMemberValue("enabled", true);
					}
					CEditor.API.UpdateGraphics();
				}
			}
		}

		// Token: 0x06000207 RID: 519 RVA: 0x0000EF74 File Offset: 0x0000D174
		public static ThingComp GetGradientComp(this Pawn p)
		{
			bool flag = p == null || p.AllComps.NullOrEmpty<ThingComp>() || !CEditor.IsGradientHairActive;
			ThingComp result;
			if (flag)
			{
				result = null;
			}
			else
			{
				foreach (ThingComp thingComp in p.AllComps)
				{
					bool flag2 = thingComp.GetType().ToString().EndsWith("CompGradientHair");
					if (flag2)
					{
						return thingComp;
					}
				}
				result = null;
			}
			return result;
		}

		// Token: 0x06000208 RID: 520 RVA: 0x0000F00C File Offset: 0x0000D20C
		public static List<string> GetAllGradientHairs()
		{
			List<Texture2D> list = ContentFinder<Texture2D>.GetAllInFolder("GradientHair").ToList<Texture2D>();
			HashSet<Texture2D> hashSet = new HashSet<Texture2D>();
			HashSet<string> hashSet2 = new HashSet<string>();
			foreach (Texture2D texture2D in list)
			{
				bool flag = !hashSet2.Contains(texture2D.name);
				if (flag)
				{
					hashSet.Add(texture2D);
					hashSet2.Add(texture2D.name);
				}
			}
			return (from t in hashSet
			orderby t.name
			select "GradientHair/" + t.name).ToList<string>();
		}

		// Token: 0x06000209 RID: 521 RVA: 0x0000F0F8 File Offset: 0x0000D2F8
		public static void RandomizeGradientMask(this Pawn p)
		{
			bool flag = p == null;
			if (!flag)
			{
				List<string> allGradientHairs = HairTool.GetAllGradientHairs();
				string mask = allGradientHairs.RandomElement<string>();
				p.SetGradientMask(mask);
			}
		}

		// Token: 0x0600020A RID: 522 RVA: 0x0000F128 File Offset: 0x0000D328
		public static string GetGradientMask(this Pawn p)
		{
			bool flag = p == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				ThingComp gradientComp = p.GetGradientComp();
				bool flag2 = gradientComp != null;
				if (flag2)
				{
					object memberValue = gradientComp.GetMemberValue("settings");
					bool flag3 = memberValue != null;
					if (flag3)
					{
						result = (string)memberValue.GetMemberValue("mask");
					}
					else
					{
						result = "";
					}
				}
				else
				{
					result = "";
				}
			}
			return result;
		}

		// Token: 0x0600020B RID: 523 RVA: 0x0000F194 File Offset: 0x0000D394
		private static Color GetGradientColor(this Pawn p)
		{
			bool isGradientHairActive = CEditor.IsGradientHairActive;
			Color result;
			if (isGradientHairActive)
			{
				object[] array = new object[3];
				array[0] = p;
				object[] array2 = array;
				Reflect.GetAType("GradientHair", "PublicApi").CallMethod("GetGradientHair", array2);
				result = (Color)array2[2];
			}
			else
			{
				result = Color.white;
			}
			return result;
		}

		// Token: 0x0600020C RID: 524 RVA: 0x0000F1E6 File Offset: 0x0000D3E6
		public static string GetHairDefName(this Pawn p)
		{
			return (p.HasStoryTracker() && p.story.hairDef != null) ? p.story.hairDef.defName : "";
		}

		// Token: 0x0600020D RID: 525 RVA: 0x0000F214 File Offset: 0x0000D414
		public static string GetHairName(this Pawn p)
		{
			return (p.HasStoryTracker() && p.story.hairDef != null) ? p.story.hairDef.LabelCap.ToString() : "";
		}

		// Token: 0x0600020E RID: 526 RVA: 0x0000F25B File Offset: 0x0000D45B
		public static List<HairDef> GetHairList(string modname)
		{
			return DefTool.ListByMod<HairDef>(modname);
		}

		// Token: 0x0600020F RID: 527 RVA: 0x0000F264 File Offset: 0x0000D464
		public static int GetHairDefCount(string modname)
		{
			return DefTool.ListByMod<HairDef>(modname).ToList<HairDef>().CountAllowNull<HairDef>();
		}

		// Token: 0x06000210 RID: 528 RVA: 0x0000F288 File Offset: 0x0000D488
		public static HairDef GetRandomHairDef(string modname)
		{
			List<HairDef> source = DefTool.ListByMod<HairDef>(modname).ToList<HairDef>();
			return source.RandomElement<HairDef>();
		}

		private static void SetGradientHair(this Pawn pawn, Color colorB)
		{
			bool isGradientHairActive = CEditor.IsGradientHairActive;
			if (isGradientHairActive)
			{
				object[] param = new object[]
				{
					pawn,
					true,
					colorB
				};
				Reflect.GetAType("GradientHair", "PublicApi").CallMethod("SetGradientHair", param);
			}
		}

		public static void SetHair(this Pawn p, HairDef h)
		{
			bool flag = !p.HasStoryTracker() || h == null;
			if (!flag)
			{
				p.story.hairDef = h;
				CEditor.API.UpdateGraphics();
			}
		}

		public static void SetHair(this Pawn pawn, bool next, bool random, string modname = null)
		{
			bool flag = pawn == null || pawn.story == null;
			if (!flag)
			{
				List<HairDef> list = DefTool.ListByMod<HairDef>(modname).ToList<HairDef>();
				int index = list.IndexOf(pawn.story.hairDef);
				index = list.NextOrPrevIndex(index, next, random);
				pawn.story.hairDef = list[index];
			}
		}

		public static void SetHairColor(this Pawn p, bool primary, Color col)
		{
			bool flag = p == null || p.story == null;
			if (!flag)
			{
				if (primary)
				{
					p.story.HairColor = col;
				}
				else
				{
					p.SetGradientHair(col);
				}
			}
		}
	}
}
