﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  // Token: 0x0200002E RID: 46
  public static class HeadTool
  {
    public static Pawn cachedPawn;
    //public static Dictionary<Pawn, Dictionary<HeadTypeDef, string>> cachedHeadNamesDict = new Dictionary<Pawn, Dictionary<HeadTypeDef, string>>();
    //public static Dictionary<Pawn, List<HeadTypeDef>> cachedSortedHeadList = new Dictionary<Pawn, List<HeadTypeDef>>();
    //public static Dictionary<Pawn, HashSet<HeadTypeDef>> cachedHeadListDict = new Dictionary<Pawn, List<Pair<HeadTypeDef, string>>>();


    public static string GetAllHeadAddonsAsSeparatedString(this Pawn p)
    {
      bool flag = !p.IsAlienRace();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        List<int> listHeadAddons = p.GetListHeadAddons();
        string text = "";
        bool flag2 = !listHeadAddons.NullOrEmpty<int>();
        if (flag2)
        {
          foreach (int num in listHeadAddons)
          {
            text += num.ToString();
            text += ":";
          }
          text = text.SubstringRemoveLast();
        }
        result = text;
      }
      return result;
    }

    public static void SetHeadAddonsFromSeparatedString(this Pawn p, string s)
    {
      bool flag = s.NullOrEmpty() || !p.IsAlienRace();
      if (!flag)
      {
        string[] array = s.SplitNo(":");
        List<int> list = new List<int>();
        bool flag2 = array.Length != 0;
        if (flag2)
        {
          foreach (string input in array)
          {
            list.Add(input.AsInt32());
          }
        }
        p.SetListHeadAddons(list);
      }
    }

    public static void RemoveAlienBodyAddons(this Pawn pawn)
    {
      bool flag = pawn.IsAlienRace();
      if (flag)
      {
        object memberValue = pawn.def.GetMemberValue("alienRace");
        bool flag2 = memberValue != null;
        if (flag2)
        {
          object memberValue2 = memberValue.GetMemberValue("generalSettings");
          bool flag3 = memberValue2 != null;
          if (flag3)
          {
            object memberValue3 = memberValue2.GetMemberValue("alienPartGenerator");
            bool flag4 = memberValue3 != null;
            if (flag4)
            {
              object memberValue4 = memberValue3.GetMemberValue("bodyAddons");
              memberValue4.CallMethod("Clear", null);
              memberValue3.SetMemberValue("bodyAddons", memberValue4);
            }
          }
        }
      }
    }

    public static List<Graphic> GetListHeadAddonGraphics(this Pawn pawn)
    {
      bool flag = pawn != null && pawn.IsAlienRace();
      List<Graphic> result;
      if (flag)
      {
        List<Graphic> list;
        try
        {
          ThingComp alienRaceComp = pawn.GetAlienRaceComp();
          object memberValue = alienRaceComp.GetMemberValue("addonGraphics");
          list = (memberValue as List<Graphic>);
        }
        catch
        {
          list = new List<Graphic>();
        }
        result = list;
      }
      else
      {
        result = new List<Graphic>();
      }
      return result;
    }

    public static List<int> GetListHeadAddons(this Pawn pawn)
    {
      bool flag = pawn != null && pawn.IsAlienRace();
      List<int> result;
      if (flag)
      {
        List<int> list;
        try
        {
          ThingComp alienRaceComp = pawn.GetAlienRaceComp();
          object memberValue = alienRaceComp.GetMemberValue("addonVariants");
          list = (memberValue as List<int>);
        }
        catch
        {
          list = new List<int>();
        }
        result = list;
      }
      else
      {
        result = null;
      }
      return result;
    }

    public static void SetListHeadAddonGraphics(this Pawn pawn, List<Graphic> l)
    {
      bool flag = pawn.IsAlienRace();
      if (flag)
      {
        ThingComp alienRaceComp = pawn.GetAlienRaceComp();
        alienRaceComp.SetMemberValue("addonGraphics", l);
      }
    }

    public static void SetListHeadAddons(this Pawn pawn, List<int> l)
    {
      bool flag = pawn.IsAlienRace();
      if (flag)
      {
        ThingComp alienRaceComp = pawn.GetAlienRaceComp();
        alienRaceComp.SetMemberValue("addonVariants", l);
      }
    }

    public static void DeleteHeadAddons(this Pawn pawn)
    {
      bool flag = pawn.IsAlienRace();
      if (flag)
      {
        ThingComp alienRaceComp = pawn.GetAlienRaceComp();
        alienRaceComp.SetMemberValue("addonVariants", new List<int>());
        alienRaceComp.SetMemberValue("addonGraphics", new List<Graphic>());
      }
    }

    public static List<string> GetHeadPathsFor(string racePath, Gender gender)
    {
      bool flag = racePath.NullOrEmpty();
      List<string> result;
      if (flag)
      {
        result = new List<string>();
      }
      else
      {
        if (racePath.StartsWith("Things/Pawn/Humanlike/Heads/"))
        {
          if (gender == Gender.Female)
          {
            racePath = racePath.Replace("Male", "Female");
          }
          else if (gender == Gender.Male)
          {
            racePath = racePath.Replace("Female", "Male");
          }
        }
        List<Texture2D> list = ContentFinder<Texture2D>.GetAllInFolder(racePath).ToList<Texture2D>();
        List<string> list2 = new List<string>();
        foreach (Texture2D texture2D in list)
        {
          bool flag5 = texture2D == null || texture2D.name.NullOrEmpty();
          if (!flag5)
          {
            string text = racePath + "/" + texture2D.name.SubstringBackwardTo("_", true);
            text = text.Replace("\\", "/");
            bool flag6 = !list2.Contains(text);
            if (flag6)
            {
              list2.Add(text);
            }
          }
        }
        list2.Sort();
        result = list2;
      }
      return result;
    }

    public static string GetCrown(this Pawn p)
    {
      return (string)(p.IsAlienRace() ? p.GetAlienRaceComp().GetMemberValue("crownType") : "");
    }

    //public static Dictionary<HeadTypeDef, string> GetHeadDict(this Pawn pawn)
    //{
    //  if (pawn == null)
    //  {
    //    return null;
    //  }
    //  HashSet<HeadTypeDef> head = new HashSet<HeadTypeDef>();
    //  if (cachedHeadNamesDict.TryGetValue(pawn, out Dictionary<HeadTypeDef, string> dict))
    //  {
    //    return dict;
    //  }
    //  pawn.InitializeHeadNameDict();
    //  return cachedHeadNamesDict[pawn];
    //}

    public static List<HeadTypeDef> GetSortedHeadList(this Pawn pawn)
    {
      //if (cachedSortedHeadList.TryGetValue(pawn, out List<HeadTypeDef> list))
      //{
      //  return list;
      //}
      List<HeadTypeDef> list = new List<HeadTypeDef>();
      foreach (HeadTypeDef headTypeDef in DefDatabase<HeadTypeDef>.AllDefs)
      {
        // ToDo: Make a setting or checkbox to allow different gender headtypes
        //if (headTypeDef.gender != Gender.None && headTypeDef.gender != pawn.gender)
        //{
        //  continue;
        //}
        // ToDo: Make this a setting or checkbox to allow bypass of required genes
        bool hasRequiredGenes = true;
        if (!headTypeDef.requiredGenes.NullOrEmpty())
        {
          foreach (GeneDef geneDef in headTypeDef.requiredGenes)
          {
            if (!pawn.genes.HasGene(geneDef))
            {
              hasRequiredGenes = false;
              break;
            }
          }
        }
        if (!hasRequiredGenes)
        {
          continue;
        }
        list.Add(headTypeDef);
      }
      list.SortBy(headType => pawn.GetHeadName(headType));
      return list;
    }

    //public static void InitializeHeadNameDict(this Pawn pawn)
    //{
    //  cachedHeadNamesDict[pawn] = new Dictionary<HeadTypeDef, string>();
    //  foreach (HeadTypeDef headTypeDef in DefDatabase<HeadTypeDef>.AllDefsListForReading)
    //  {
    //    // ToDo: Make this a setting to allow different gender headtypes
    //    //if (headTypeDef.gender != Gender.None && headTypeDef.gender != pawn.gender)
    //    //{
    //    //  continue;
    //    //}
    //    // ToDo: Make this a setting to allow bypass of required genes
    //    bool hasRequiredGenes = true;
    //    if (!headTypeDef.requiredGenes.NullOrEmpty())
    //    {
    //      foreach (GeneDef geneDef in headTypeDef.requiredGenes)
    //      {
    //        if (!pawn.genes.HasGene(geneDef))
    //        {
    //          hasRequiredGenes = false;
    //          break;
    //        }
    //      }
    //    }
    //    if (!hasRequiredGenes)
    //    {
    //      continue;
    //    }
    //    cachedHeadNamesDict[pawn][headTypeDef] = headTypeDef.ConstructHeadName();
    //  }
    //  cachedSortedHeadList[pawn] = new List<HeadTypeDef>();
    //  List<KeyValuePair<HeadTypeDef, string>> list = cachedHeadNamesDict[pawn].ToList();
    //  list.SortBy(kvp => kvp.Value);
    //  foreach (KeyValuePair<HeadTypeDef, string> kvp in list)
    //  {
    //    cachedSortedHeadList[pawn].Add(kvp.Key);
    //  }
    //}


    //public static List<string> GetHeadList(this Pawn pawn)
    //{
    //  bool flag = pawn == null;
    //  List<string> result;
    //  if (flag)
    //  {
    //    result = null;
    //  }
    //  else
    //  {
    //    string headPath = pawn.story.headType.graphicPath;
    //    List<string> headPathsFor = HeadTool.GetHeadPathsFor(headPath.SubstringBackwardTo("/", true), pawn.gender);
    //    bool flag2 = headPathsFor.NullOrEmpty<string>();
    //    if (flag2)
    //    {
    //      result = new List<string> { headPath };
    //    }
    //    else
    //    {
    //      bool flag3 = pawn.IsAlienRace();
    //      if (flag3)
    //      {
    //        result = pawn.GetAlienCrownTypes();
    //      }
    //      else
    //      {
    //        result = headPathsFor;
    //      }
    //    }
    //  }
    //  return result;
    //}

    // Token: 0x06000221 RID: 545 RVA: 0x0000F8F8 File Offset: 0x0000DAF8
    //public static string GetHeadName(this Pawn pawn, string path = null)
    //{
    //	string text = "";
    //	bool flag = pawn == null;
    //	string result;
    //	if (flag)
    //	{
    //		result = "";
    //	}
    //	else
    //	{
    //		string text2;
    //		if ((text2 = path) == null)
    //		{
    //			if (pawn == null)
    //			{
    //				text2 = null;
    //			}
    //			else
    //			{
    //				Pawn_StoryTracker story = pawn.story;
    //				text2 = ((story != null) ? story.HeadGraphicPath : null);
    //			}
    //		}
    //		path = text2;
    //		path = (path ?? "");
    //		bool flag2 = path.Contains("Female");
    //		if (flag2)
    //		{
    //			text += "♀ ";
    //		}
    //		else
    //		{
    //			bool flag3 = path.Contains("Male");
    //			if (flag3)
    //			{
    //				text += "♂ ";
    //			}
    //		}
    //		bool flag4 = path.Contains("Average");
    //		if (flag4)
    //		{
    //			text += "Average";
    //		}
    //		else
    //		{
    //			bool flag5 = path.Contains("Narrow");
    //			if (flag5)
    //			{
    //				text += "Narrow";
    //			}
    //		}
    //		bool flag6 = path.Contains("Wide");
    //		if (flag6)
    //		{
    //			text += " Wide";
    //		}
    //		else
    //		{
    //			bool flag7 = path.Contains("Pointy");
    //			if (flag7)
    //			{
    //				text += " Pointy";
    //			}
    //			else
    //			{
    //				bool flag8 = path.Contains("Normal");
    //				if (flag8)
    //				{
    //					text += " Normal";
    //				}
    //				else
    //				{
    //					text += ((path.EndsWith("Average") || path.EndsWith("Narrow")) ? "" : path.SubstringBackwardFrom("_", false));
    //				}
    //			}
    //		}
    //		bool flag9 = text.Contains("/");
    //		if (flag9)
    //		{
    //			text = text.SubstringBackwardFrom("/", true);
    //		}
    //		result = text;
    //	}
    //	return result;
    //}

    //public static string ConstructHeadName(this HeadTypeDef headTypeDef)
    //{
    //  string headName = headTypeDef.ConstructLabelFromDef();
    //  headName = headName.Replace("Female", "♀");
    //  headName = headName.Replace("Male", "♂");
    //  return headName;
    //}

    public static string GetHeadName(this Pawn pawn, HeadTypeDef headType = null)
    {
      if (!pawn.HasStoryTracker())
      {
        return "";
      }
      if (headType == null)
      {
        headType = pawn.story.headType;
      }
      return headType.ConstructLabelFromDef(); ;

      //if (headTypeDef == null)
      //{
      //  if (!pawn.HasStoryTracker())
      //  {
      //    return "";
      //  }
      //  headTypeDef = pawn.story.headType;
      //}
      //if (cachedHeadNamesDict.TryGetValue(pawn, out Dictionary<HeadTypeDef, string> nameDict))
      //{
      //  if (nameDict.TryGetValue(headTypeDef, out string name))
      //  {
      //    return name;
      //  }
      //}
      //pawn.InitializeHeadNameDict();
      //if (cachedHeadNamesDict.TryGetValue(pawn, out Dictionary<HeadTypeDef, string> nameDict2))
      //{
      //  if (nameDict2.TryGetValue(headTypeDef, out string name))
      //  {
      //    return name;
      //  }
      //}
      //return "";
    }

    // Token: 0x06000222 RID: 546 RVA: 0x0000FA74 File Offset: 0x0000DC74
    //public static string GetHeadPath(this Pawn p)
    //{
    //	return (p != null && p.story != null) ? ((string)p.story.GetMemberValue("headGraphicPath")) : "";
    //}

    public static string GetHeadTypeDefname(this Pawn p)
    {
      return p?.story?.headType?.defName == null ? "" : p.story.headType.defName;
    }

    // Token: 0x06000223 RID: 547 RVA: 0x0000FAA0 File Offset: 0x0000DCA0
    //public static void SetCrownType(this Pawn p, HeadTypeDef c)
    //{
    //	bool flag = p == null || p.story == null;
    //	if (!flag)
    //	{
    //		p.story.crownType = c;
    //	}
    //}

    // Token: 0x06000224 RID: 548 RVA: 0x0000FAD0 File Offset: 0x0000DCD0
    //public static void SetCrown(this Pawn pawn)
    //{
    //	bool flag = pawn == null;
    //	if (!flag)
    //	{
    //		bool flag2 = pawn.IsAlienRace();
    //		if (flag2)
    //		{
    //			string text = pawn.ExtractCrown();
    //			bool flag3 = !text.NullOrEmpty();
    //			if (flag3)
    //			{
    //				ThingComp alienRaceComp = pawn.GetAlienRaceComp();
    //				alienRaceComp.SetMemberValue("crownType", text);
    //			}
    //		}
    //		else
    //		{
    //			bool flag4 = pawn.story != null;
    //			if (flag4)
    //			{
    //				string headPath = pawn.GetHeadPath();
    //				bool flag5 = headPath == null;
    //				if (flag5)
    //				{
    //            //pawn.story.crownType = 1;
    //            pawn.story.crownType = CrownType.Average;
    //          }
    //				else
    //				{
    //					bool flag6 = headPath.Contains(Enum.GetName(typeof(CrownType), 1));
    //					if (flag6)
    //					{
    //              //pawn.story.crownType = 1;
    //              pawn.story.crownType = CrownType.Average;
    //            }
    //					else
    //					{
    //						bool flag7 = headPath.Contains(Enum.GetName(typeof(CrownType), 2));
    //						if (flag7)
    //						{
    //                //pawn.story.crownType = 2;
    //                //pawn.story.crownType = CrownType.Narrow;
    //                pawn.story.crownType = CrownType.Narrow;
    //              }
    //						else
    //						{
    //                //pawn.story.crownType = 0;
    //                pawn.story.crownType = CrownType.Undefined;
    //              }
    //					}
    //				}
    //			}
    //		}
    //	}
    //}

    // Token: 0x06000225 RID: 549 RVA: 0x0000FBCC File Offset: 0x0000DDCC
    //private static string ExtractCrown(this Pawn pawn)
    //{
    //  string headPath = pawn.GetHeadPath();
    //  string text = headPath.SubstringBackwardFrom("/", true);
    //  bool flag = text == null;
    //  string result;
    //  if (flag)
    //  {
    //    result = "";
    //  }
    //  else
    //  {
    //    List<string> headList = pawn.GetHeadList();
    //    bool flag2 = !headList.Contains(text);
    //    if (flag2)
    //    {
    //      text = (text.StartsWith("Male_") ? text.Replace("Male_", "") : text);
    //      text = (text.StartsWith("Female_") ? text.Replace("Female_", "") : text);
    //    }
    //    result = text;
    //  }
    //  return result;
    //}

    // Token: 0x06000226 RID: 550 RVA: 0x0000FC64 File Offset: 0x0000DE64
    //public static bool SetHead(this Pawn pawn, bool next, bool random)
    //{
    //  bool flag = pawn == null;
    //  bool result;
    //  if (flag)
    //  {
    //    result = false;
    //  }
    //  else
    //  {
    //    List<string> headList = pawn.GetHeadList();
    //    bool flag2 = headList.NullOrEmpty<string>();
    //    if (flag2)
    //    {
    //      result = false;
    //    }
    //    else
    //    {
    //      string headPath = pawn.GetHeadPath();
    //      string text = headPath.SubstringBackwardFrom("/", true);
    //      int num = headList.IndexOf(headPath);
    //      bool flag3 = num < 0;
    //      if (flag3)
    //      {
    //        num = headList.IndexOf(text);
    //      }
    //      bool flag4 = num < 0 && text.StartsWith("Female_");
    //      if (flag4)
    //      {
    //        num = headList.IndexOf(text.SubstringFrom("Female_", true));
    //      }
    //      bool flag5 = num < 0 && text.StartsWith("Male_");
    //      if (flag5)
    //      {
    //        num = headList.IndexOf(text.SubstringFrom("Male_", true));
    //      }
    //      num = headList.NextOrPrevIndex(num, next, random);
    //      string val = headList[num];
    //      pawn.SetHeadPath(val, true);
    //      pawn.SetCrown();
    //      result = true;
    //    }
    //  }
    //  return result;
    //}

    public static bool SetHead(this Pawn pawn, bool next, bool random)
    {
      if (pawn == null)
      {
        return false;
      }
      List<HeadTypeDef> headList = pawn.GetSortedHeadList();
      if (headList.NullOrEmpty())
      {
        return false;
      }
      int index = headList.IndexOf(pawn.story.headType);
      if (index == -1)
      {
        return false;
      }
      if (random)
      {
        if (headList.Count() > 1)
        {
          headList.RemoveAt(index);
          pawn.SetHeadTypeDef(headList.RandomElement());
        }
        return true;
      }
      if (next)
      {
        index++;
        if (index >= headList.Count())
        {
          index = 0;
        }
      }
      else
      {
        index--;
        if (index < 0)
        {
          index = headList.Count() - 1;
        }
      }
      pawn.SetHeadTypeDef(headList[index]);
      return true;





      //string headPath = pawn.story.headType.graphicPath;
      //string text = headPath.SubstringBackwardFrom("/", true);
      //int num = headList.IndexOf(headPath);
      //if (num < 0)
      //{
      //  num = headList.IndexOf(text);
      //}
      //if (num < 0 && text.StartsWith("Female_"))
      //{
      //  num = headList.IndexOf(text.SubstringFrom("Female_", true));
      //}
      //if (num < 0 && text.StartsWith("Male_"))
      //{
      //  num = headList.IndexOf(text.SubstringFrom("Male_", true));
      //}
      //num = headList.NextOrPrevIndex(num, next, random);
      //string val = headList[num];

      //pawn.SetHeadPath(val, true);
      //pawn.SetCrown();
      //pawn.SetHeadTypeDef(val, true);
      //return true;

    }

    public static void SetHeadTypeDef(this Pawn p, HeadTypeDef headTypeDef)
    {
      if (!p.HasStoryTracker() || headTypeDef == null)
      {
        return;
      }
      p.story.headType = headTypeDef;
    }

    public static void SetHeadTypeDef(this Pawn p, string headTypeDefnameOrHeadGraphicPath, bool noTest = true)
    {
      ////Log.Message("SetHeadTypeDef, start, headTypeDefnameOrHeadGraphicPath = " + headTypeDefnameOrHeadGraphicPath);
      if (!p.HasStoryTracker() || headTypeDefnameOrHeadGraphicPath.NullOrEmpty())
      {
        return;
      }
      HeadTypeDef headTypeDef = DefDatabase<HeadTypeDef>.GetNamedSilentFail(headTypeDefnameOrHeadGraphicPath);
      if (headTypeDef != null)
      {
        ////Log.Message("SetHeadTypeDef, found TypeTypeDef " + headTypeDef);
        p.story.headType = headTypeDef;
        return;
      }
      ////Log.Message("SetHeadTypeDef, looking for graphicPath = " + headTypeDefnameOrHeadGraphicPath);
      HeadTypeDef oldHeadTypeDef = p.story.headType;
      IEnumerable<HeadTypeDef> possibleHeadTypes = DefDatabase<HeadTypeDef>.AllDefs.Where((HeadTypeDef x) => x.graphicPath == headTypeDefnameOrHeadGraphicPath);
      if (p.story.TryGetRandomHeadFromSet(possibleHeadTypes))
      {
        if (p.story.headType != null)
        {
          return;
        }
      }
      p.story.headType = oldHeadTypeDef;
    }

    // Token: 0x06000227 RID: 551 RVA: 0x0000FD50 File Offset: 0x0000DF50
    //public static void SetHeadPath(this Pawn p, string val, bool noTest = true)
    //{
    //  bool flag = !p.HasStoryTracker() || val.NullOrEmpty();
    //  if (!flag)
    //  {
    //    bool flag2 = noTest || p.TestHead(val);
    //    if (flag2)
    //    {
    //      p.story.SetMemberValue("headGraphicPath", val);
    //    }
    //    else
    //    {
    //      MessageTool.Show("head not found:" + val, null);
    //    }
    //  }
    //}

    // Token: 0x06000228 RID: 552 RVA: 0x0000FDAC File Offset: 0x0000DFAC
    //public static bool TestHead(this Pawn pawn, string headPath)
    //{
    //  bool flag = pawn == null || headPath.NullOrEmpty();
    //  bool result;
    //  if (flag)
    //  {
    //    result = false;
    //  }
    //  else
    //  {
    //    string text = headPath.Replace("\\", "/");
    //    bool flag2 = TextureTool.TestTexturePath(text + "_south", false);
    //    bool flag3 = !flag2;
    //    if (flag3)
    //    {
    //      result = false;
    //    }
    //    else
    //    {
    //      bool flag4 = pawn.IsAlienRace();
    //      if (flag4)
    //      {
    //        bool flag5 = text.ToLower().Contains("/female");
    //        bool flag6 = text.ToLower().Contains("/male");
    //        bool flag7 = flag5 || flag6;
    //        if (flag7)
    //        {
    //          flag2 = ((pawn.gender == Gender.Female && flag5) || (pawn.gender == Gender.Male && flag6));
    //        }
    //        bool flag8 = !flag5 && !flag6;
    //        if (flag8)
    //        {
    //          flag2 = true;
    //        }
    //      }
    //      result = flag2;
    //    }
    //  }
    //  return result;
    //}

    // Token: 0x06000229 RID: 553 RVA: 0x0000FE7C File Offset: 0x0000E07C
    public static Color GetEyeColor(this Pawn p)
    {
      ThingComp thingComp = p.FA_GetControllerComp(FacialTool.EYE);
      return (thingComp != null) ? ((Color)thingComp.GetMemberValue("color")) : Color.white;
    }

    // Token: 0x0600022A RID: 554 RVA: 0x0000FEB4 File Offset: 0x0000E0B4
    public static void SetEyeColor(this Pawn p, Color col)
    {
      ThingComp thingComp = p.FA_GetControllerComp(FacialTool.EYE);
      bool flag = thingComp != null;
      if (flag)
      {
        thingComp.SetMemberValue("color", col);
        CEditor.API.UpdateGraphics();
      }
    }
  }
}
