﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
  // Token: 0x0200003C RID: 60
  public static class HealthTool
  {
    // Token: 0x060002E3 RID: 739 RVA: 0x000172DC File Offset: 0x000154DC
    public static string GetHediffAsSeparatedString(this Hediff h)
    {
      bool flag = h == null || h.def.IsNullOrEmpty();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        string text = "";
        text = text + h.def.defName + "|";
        text = text + h.Severity.ToString() + "|";
        text = text + ((h.Part != null) ? h.Part.def.defName : "") + "|";
        text = text + ((h.Part != null) ? h.Part.Index.ToString() : "") + "|";
        text = text + (h.IsPermanent() ? "1" : "0") + "|";
        text = text + h.GetLevel().ToString() + "|";
        text = text + h.GetPainValue().ToString() + "|";
        text = text + h.GetDuration().ToString() + "|";
        text += h.GetOtherPawn().GetPawnNameAsSeparatedString();
        result = text;
      }
      return result;
    }

    // Token: 0x060002E4 RID: 740 RVA: 0x00017424 File Offset: 0x00015624
    public static string GetAllHediffsAsSeparatedString(this Pawn p)
    {
      bool flag = !p.HasHealthTracker() || p.health.hediffSet.hediffs.NullOrEmpty<Hediff>();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        string text = "";
        foreach (Hediff h in p.health.hediffSet.hediffs)
        {
          text += h.GetHediffAsSeparatedString();
          text += ":";
        }
        text = text.SubstringRemoveLast();
        result = text;
      }
      return result;
    }

    // Token: 0x060002E5 RID: 741 RVA: 0x000174D8 File Offset: 0x000156D8
    public static void SetHediffsFromSeparatedString(this Pawn p, string s)
    {
      bool flag = !p.HasHealthTracker();
      if (!flag)
      {
        p.health.hediffSet.hediffs.Clear();
        bool flag2 = s.NullOrEmpty();
        if (!flag2)
        {
          string[] array = s.Split(new string[]
          {
            ":"
          }, StringSplitOptions.None);
          foreach (string text in array)
          {
            string[] array3 = text.Split(new string[]
            {
              "|"
            }, StringSplitOptions.None);
            bool flag3 = array3.Length >= 5;
            if (flag3)
            {
              int level = (array3.Length > 5) ? array3[5].AsInt32() : -1;
              int pain = (array3.Length > 6) ? array3[6].AsInt32() : -1;
              int duration = (array3.Length > 7) ? array3[7].AsInt32() : -1;
              string otherPawnName = (array3.Length > 8) ? array3[8] : "";
              p.AddHediffByName(array3[0], array3[1].AsFloat(), array3[2], array3[3].AsInt32(), array3[4].AsBool(), level, pain, duration, otherPawnName);
            }
          }
        }
      }
    }

    // Token: 0x1700004A RID: 74
    // (get) Token: 0x060002E6 RID: 742 RVA: 0x00017608 File Offset: 0x00015808
    public static Dictionary<string, Func<HediffDef, bool>> AllBodyPartChecks
    {
      get
      {
        return new Dictionary<string, Func<HediffDef, bool>>
        {
          {
            "All",
            new Func<HediffDef, bool>(HealthTool.IsForAllParts)
          },
          {
            "AllImplants",
            new Func<HediffDef, bool>(HealthTool.IsForAllParts)
          },
          {
            "AllAddictions",
            new Func<HediffDef, bool>(HealthTool.IsForAllParts)
          },
          {
            "AllDiseases",
            new Func<HediffDef, bool>(HealthTool.IsForAllParts)
          },
          {
            "AllInjuries",
            new Func<HediffDef, bool>(HealthTool.IsForAllParts)
          },
          {
            "AllTime",
            new Func<HediffDef, bool>(HealthTool.IsForAllParts)
          },
          {
            "Arm",
            new Func<HediffDef, bool>(HealthTool.IsForArm)
          },
          {
            "Brain",
            new Func<HediffDef, bool>(HealthTool.IsForBrain)
          },
          {
            "Clavicle",
            new Func<HediffDef, bool>(HealthTool.IsForClavicle)
          },
          {
            "Ear",
            new Func<HediffDef, bool>(HealthTool.IsForEar)
          },
          {
            "Eye",
            new Func<HediffDef, bool>(HealthTool.IsForEye)
          },
          {
            "Femur",
            new Func<HediffDef, bool>(HealthTool.IsForFemur)
          },
          {
            "Finger",
            new Func<HediffDef, bool>(HealthTool.IsForFinger)
          },
          {
            "Foot",
            new Func<HediffDef, bool>(HealthTool.IsForFoot)
          },
          {
            "Hand",
            new Func<HediffDef, bool>(HealthTool.IsForHand)
          },
          {
            "Head",
            new Func<HediffDef, bool>(HealthTool.IsForHead)
          },
          {
            "Heart",
            new Func<HediffDef, bool>(HealthTool.IsForHeart)
          },
          {
            "Humerus",
            new Func<HediffDef, bool>(HealthTool.IsForHumerus)
          },
          {
            "Jaw",
            new Func<HediffDef, bool>(HealthTool.IsForJaw)
          },
          {
            "Kidney",
            new Func<HediffDef, bool>(HealthTool.IsForKidney)
          },
          {
            "Leg",
            new Func<HediffDef, bool>(HealthTool.IsForLeg)
          },
          {
            "Liver",
            new Func<HediffDef, bool>(HealthTool.IsForLiver)
          },
          {
            "Lung",
            new Func<HediffDef, bool>(HealthTool.IsForLung)
          },
          {
            "Nose",
            new Func<HediffDef, bool>(HealthTool.IsForNose)
          },
          {
            "Pelvis",
            new Func<HediffDef, bool>(HealthTool.IsForPelvis)
          },
          {
            "Radius",
            new Func<HediffDef, bool>(HealthTool.IsForRadius)
          },
          {
            "Shoulder",
            new Func<HediffDef, bool>(HealthTool.IsForShoulder)
          },
          {
            "Skull",
            new Func<HediffDef, bool>(HealthTool.IsForSkull)
          },
          {
            "Spine",
            new Func<HediffDef, bool>(HealthTool.IsForSpine)
          },
          {
            "Sternum",
            new Func<HediffDef, bool>(HealthTool.IsForSternum)
          },
          {
            "Stomach",
            new Func<HediffDef, bool>(HealthTool.IsForStomach)
          },
          {
            "Tibia",
            new Func<HediffDef, bool>(HealthTool.IsForTibia)
          },
          {
            "Toe",
            new Func<HediffDef, bool>(HealthTool.IsForToe)
          },
          {
            "Torso",
            new Func<HediffDef, bool>(HealthTool.IsForTorso)
          },
          {
            "UtilitySlot",
            new Func<HediffDef, bool>(HealthTool.IsForUtilitySlot)
          },
          {
            "WholeBody",
            new Func<HediffDef, bool>(HealthTool.IsForWholeBody)
          }
        };
      }
    }

    // Token: 0x060002E7 RID: 743 RVA: 0x0001797C File Offset: 0x00015B7C
    public static string GetFullLabel(this Hediff h)
    {
      bool flag = h == null;
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        try
        {
          string text = h.Label;
          text = (text ?? "");
          string text2 = (h.Part == null) ? "" : ((h.Part.Label == null) ? "" : (h.Part.Label + " "));
          text2 = (text2 ?? "");
          result = text2 + text;
        }
        catch
        {
          result = h.def.label;
        }
      }
      return result;
    }

    // Token: 0x060002E8 RID: 744 RVA: 0x00017A20 File Offset: 0x00015C20
    public static bool IsAdjustableSeverity(HediffDef hediff)
    {
      bool result = false;
      bool flag = hediff != null;
      if (flag)
      {
        bool flag2 = hediff.injuryProps != null && hediff.hediffClass != typeof(Hediff_MissingPart);
        if (flag2)
        {
          result = true;
        }
        else
        {
          bool isAddiction = hediff.IsAddiction;
          if (isAddiction)
          {
            result = true;
          }
          else
          {
            bool flag3 = hediff.maxSeverity > 1f;
            if (flag3)
            {
              result = true;
            }
            else
            {
              bool flag4 = hediff.addedPartProps != null || hediff.hediffClass == typeof(Hediff_MissingPart) || hediff.countsAsAddedPartOrImplant;
              result = !flag4;
            }
          }
        }
        bool flag5 = hediff.hediffClass == typeof(Hediff_Psylink);
        if (flag5)
        {
          result = false;
        }
      }
      return result;
    }

    // Token: 0x060002E9 RID: 745 RVA: 0x00017AE4 File Offset: 0x00015CE4
    public static float GetMaxSeverity(HediffDef hediff)
    {
      float num = 0f;
      bool flag = hediff == null;
      float result;
      if (flag)
      {
        result = 0f;
      }
      else
      {
        try
        {
          num = ((hediff.lethalSeverity >= 0f) ? hediff.lethalSeverity : hediff.maxSeverity);
          bool flag2 = num > 50f;
          if (flag2)
          {
            num = ((hediff.stages.CountAllowNull<HediffStage>() > 0) ? 1f : 50f);
          }
        }
        catch
        {
        }
        result = num;
      }
      return result;
    }

    // Token: 0x060002EA RID: 746 RVA: 0x00017B6C File Offset: 0x00015D6C
    public static bool NeedBodyPart(HediffDef hediff)
    {
      bool flag = hediff == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        bool flag2 = hediff.defName.Contains("_Force") && hediff.hediffClass == typeof(HediffWithComps);
        if (flag2)
        {
          result = false;
        }
        else
        {
          bool flag3 = hediff.hediffClass == typeof(Hediff_AddedPart) || hediff.hediffClass == typeof(Hediff_Injury) || hediff.hediffClass == typeof(HediffWithComps) || hediff.hediffClass == typeof(Hediff_MissingPart) || hediff.hediffClass == typeof(Hediff_Implant);
          result = flag3;
        }
      }
      return result;
    }

    // Token: 0x060002EB RID: 747 RVA: 0x00017C40 File Offset: 0x00015E40
    public static BodyPartRecord GetBodyPart(Pawn pawn, Hediff h)
    {
      bool flag = pawn != null && h != null && h.Part != null && h.Part.def != null && pawn.RaceProps.body.AllParts != null;
      if (flag)
      {
        try
        {
          foreach (BodyPartRecord bodyPartRecord in pawn.RaceProps.body.AllParts)
          {
            bool flag2 = bodyPartRecord.def.defName == h.Part.def.defName && bodyPartRecord.Index == h.Part.Index;
            if (flag2)
            {
              return bodyPartRecord;
            }
          }
        }
        catch (Exception ex)
        {
          Log.Error("GetBodyPart: " + ex.Message);
        }
      }
      return null;
    }

    // Token: 0x060002EC RID: 748 RVA: 0x00017D4C File Offset: 0x00015F4C
    public static BodyPartRecord GetBodyPartByDefName(Pawn pawn, string defName, int index)
    {
      bool flag = pawn != null && !string.IsNullOrEmpty(defName);
      if (flag)
      {
        foreach (BodyPartRecord bodyPartRecord in pawn.RaceProps.body.AllParts)
        {
          bool flag2 = bodyPartRecord.def.defName == defName && bodyPartRecord.Index == index;
          if (flag2)
          {
            return bodyPartRecord;
          }
        }
      }
      return null;
    }

    // Token: 0x060002ED RID: 749 RVA: 0x00017DEC File Offset: 0x00015FEC
    public static bool IsFor(HediffDef h, string bodyparttype)
    {
      return HealthTool.AllBodyPartChecks[bodyparttype](h);
    }

    // Token: 0x060002EE RID: 750 RVA: 0x00017E00 File Offset: 0x00016000
    public static bool IsForAllParts(this HediffDef h)
    {
      return h.hediffClass == typeof(Hediff_MissingPart) || h.defName == "ChemicalDamageModerate" || h.defName == "ChemicalDamageSevere" || h.defName == "MuscleParasites" || h.defName == "FibrousMechanites" || h.defName == "SensoryMechanites" || h.defName == "Carcinoma" || h.defName == "WoundInfection";
    }

    // Token: 0x060002EF RID: 751 RVA: 0x00017EA8 File Offset: 0x000160A8
    public static bool IsForArm(this HediffDef h)
    {
      return h.defName.Contains("Arm") || h.defName == "PowerClaw" || h.defName == "ElbowBlade";
    }

    // Token: 0x060002F0 RID: 752 RVA: 0x00017EE4 File Offset: 0x000160E4
    public static bool IsForBrain(this HediffDef h)
    {
      return h.defName.Contains("Brain") || (h.defName.StartsWith("Psychic") && h.defName != "PsychicEntropy") || (h.HasComp(typeof(HediffComp_Disappears)) && h.defName.Contains("_Psychic")) || h.defName.StartsWith("Circadian") || h.defName == "Dementia" || h.defName == "Alzheimers" || h.defName == "ResurrectionPsychosis" || h.defName == "TraumaSavant" || h.defName == "Joywire" || h.defName == "Painstopper" || h.defName == "Neurocalculator" || h.defName == "LearningAssistant" || h.defName == "Mindscrew" || h.defName == "Joyfuzz" || h.defName == "Abasia" || h.defName == "NoPain" || h.defName == "HungerMaker" || h.defName == "SpeedBoost" || h.defName.EndsWith("Command") || h.HasComp(typeof(HediffComp_EntropyLink)) || h.HasComp(typeof(HediffComp_Link));
    }

    // Token: 0x060002F1 RID: 753 RVA: 0x000180B7 File Offset: 0x000162B7
    public static bool IsForClavicle(this HediffDef h)
    {
      return h.defName.Contains("Clavicle");
    }

    // Token: 0x060002F2 RID: 754 RVA: 0x000180CC File Offset: 0x000162CC
    public static bool IsForEar(this HediffDef h)
    {
      return h.defName.Contains("Ear") || h.defName.Contains("Hearing") || h.defName.Contains("Cochlear") || h.defName == "HearingLoss";
    }

    // Token: 0x060002F3 RID: 755 RVA: 0x00018122 File Offset: 0x00016322
    public static bool IsForEye(this HediffDef h)
    {
      return h.defName.Contains("Eye") || h.defName == "Cataract" || h.defName == "Blindness";
    }

    // Token: 0x060002F4 RID: 756 RVA: 0x0001815B File Offset: 0x0001635B
    public static bool IsForFemur(this HediffDef h)
    {
      return h.defName.Contains("Femur");
    }

    // Token: 0x060002F5 RID: 757 RVA: 0x0001816D File Offset: 0x0001636D
    public static bool IsForFinger(this HediffDef h)
    {
      return h.defName.Contains("Finger") || h.defName == "VenomTalon";
    }

    // Token: 0x060002F6 RID: 758 RVA: 0x00018194 File Offset: 0x00016394
    public static bool IsForFoot(this HediffDef h)
    {
      return h.defName.Contains("Foot");
    }

    // Token: 0x060002F7 RID: 759 RVA: 0x000181A6 File Offset: 0x000163A6
    public static bool IsForHand(this HediffDef h)
    {
      return h.defName.Contains("Hand");
    }

    // Token: 0x060002F8 RID: 760 RVA: 0x000181B8 File Offset: 0x000163B8
    public static bool IsForHead(this HediffDef h)
    {
      return h.defName.Contains("Head") || h.defName == "Hangover" || h.defName == "TortureCrown" || h.defName == "Blindfold";
    }

    // Token: 0x060002F9 RID: 761 RVA: 0x0001820E File Offset: 0x0001640E
    public static bool IsForHeart(this HediffDef h)
    {
      return h.defName.Contains("Heart");
    }

    // Token: 0x060002FA RID: 762 RVA: 0x00018220 File Offset: 0x00016420
    public static bool IsForHumerus(this HediffDef h)
    {
      return h.defName.Contains("Humerus");
    }

    // Token: 0x060002FB RID: 763 RVA: 0x00018232 File Offset: 0x00016432
    public static bool IsForJaw(this HediffDef h)
    {
      return h.defName.Contains("Jaw") || h.defName.StartsWith("Denture") || h.defName.Contains("Fangs");
    }

    // Token: 0x060002FC RID: 764 RVA: 0x0001826B File Offset: 0x0001646B
    public static bool IsForKidney(this HediffDef h)
    {
      return h.defName.Contains("Kidney") || h.defName == "Immunoenhancer";
    }

    // Token: 0x060002FD RID: 765 RVA: 0x00018292 File Offset: 0x00016492
    public static bool IsForLeg(this HediffDef h)
    {
      return h.defName.Contains("Leg") || h.defName == "KneeSpike";
    }

    // Token: 0x060002FE RID: 766 RVA: 0x000182B9 File Offset: 0x000164B9
    public static bool IsForLiver(this HediffDef h)
    {
      return h.defName.Contains("Liver") || h.defName == "Cirrhosis" || h.defName == "AlcoholTolerance";
    }

    // Token: 0x060002FF RID: 767 RVA: 0x000182F2 File Offset: 0x000164F2
    public static bool IsForLung(this HediffDef h)
    {
      return h.defName.Contains("Lung") || h.defName == "Asthma";
    }

    // Token: 0x06000300 RID: 768 RVA: 0x00018319 File Offset: 0x00016519
    public static bool IsForNose(this HediffDef h)
    {
      return h.defName.Contains("Nose") || h.defName.Contains("Smelling") || h.defName == "GastroAnalyzer";
    }

    // Token: 0x06000301 RID: 769 RVA: 0x00018352 File Offset: 0x00016552
    public static bool IsForPelvis(this HediffDef h)
    {
      return h.defName.Contains("Pelvis");
    }

    // Token: 0x06000302 RID: 770 RVA: 0x00018364 File Offset: 0x00016564
    public static bool IsForRadius(this HediffDef h)
    {
      return h.defName.Contains("Radius");
    }

    // Token: 0x06000303 RID: 771 RVA: 0x00018376 File Offset: 0x00016576
    public static bool IsForShoulder(this HediffDef h)
    {
      return h.defName.Contains("Shoulder");
    }

    // Token: 0x06000304 RID: 772 RVA: 0x00018388 File Offset: 0x00016588
    public static bool IsForSkull(this HediffDef h)
    {
      return h.defName.Contains("Skull");
    }

    // Token: 0x06000305 RID: 773 RVA: 0x0001839A File Offset: 0x0001659A
    public static bool IsForSpine(this HediffDef h)
    {
      return h.defName.Contains("Spine") || h.defName == "BadBack";
    }

    // Token: 0x06000306 RID: 774 RVA: 0x000183C1 File Offset: 0x000165C1
    public static bool IsForSternum(this HediffDef h)
    {
      return h.defName.Contains("Sternum");
    }

    // Token: 0x06000307 RID: 775 RVA: 0x000183D3 File Offset: 0x000165D3
    public static bool IsForStomach(this HediffDef h)
    {
      return h.defName.Contains("Stomach") || h.defName == "GutWorms";
    }

    // Token: 0x06000308 RID: 776 RVA: 0x000183FA File Offset: 0x000165FA
    public static bool IsForTibia(this HediffDef h)
    {
      return h.defName.Contains("Tibia");
    }

    // Token: 0x06000309 RID: 777 RVA: 0x0001840C File Offset: 0x0001660C
    public static bool IsForToe(this HediffDef h)
    {
      return h.defName.Contains("Toe");
    }

    // Token: 0x0600030A RID: 778 RVA: 0x00018420 File Offset: 0x00016620
    public static bool IsForTorso(this HediffDef h)
    {
      return h.defName.Contains("Torso") || h.defName.EndsWith("skinGland") || h.defName == "Coagulator" || h.defName == "HealingEnhancer" || h.defName == "AestheticShaper" || h.defName == "LoveEnhancer";
    }

    // Token: 0x0600030B RID: 779 RVA: 0x0001849A File Offset: 0x0001669A
    public static bool IsForUtilitySlot(this HediffDef h)
    {
      return h.defName.Contains("Utility");
    }

    // Token: 0x0600030C RID: 780 RVA: 0x000184AC File Offset: 0x000166AC
    public static bool IsForWholeBody(this HediffDef h)
    {
      return h.IsAddiction || h.defName.ToLower().Contains("tolerance") || h.defName.EndsWith("High") || h.defName == "PsychicEntropy" || h.defName == "PsychicShock" || h.defName == "NeuralHealRecoveryGain" || h.defName == "NeuralSupercharge" || h.defName == "WorkDrive" || h.defName == "ImmunityDrive" || h.defName == "WorkFocus" || h.defName == "PreachHealth" || h.defName == "BerserkTrance" || h.defName == "CatatonicBreakdown" || h.defName == "BloodLoss" || h.defName.EndsWith("Flu") || h.defName.EndsWith("Plague") || h.defName == "Malaria" || h.defName == "SleepingSickness" || h.defName == "Anesthetic" || h.defName == "Frail" || h.defName == "CryptosleepSickness" || h.defName == "FoodPoisoning" || h.defName == "ToxicBuildup" || h.defName == "Pregnant" || h.defName == "DrugOverdose" || h.defName == "ResurrectionSickness" || h.defName == "Malnutrition";
    }

    // Token: 0x0600030D RID: 781 RVA: 0x000186DC File Offset: 0x000168DC
    public static void AddHediffByName(this Pawn p, string defName, float severity, string bodyPartDefName, int bodyPartIndex, bool permanent, int level, int pain, int duration, string otherPawnName)
    {
      HediffDef hediff = DefTool.HediffDef(defName);
      Pawn otherPawnFromSeparatedString = PawnxTool.GetOtherPawnFromSeparatedString(otherPawnName);
      BodyPartRecord bodyPartByDefName = HealthTool.GetBodyPartByDefName(p, bodyPartDefName, bodyPartIndex);
      p.AddHediff2(false, hediff, severity, bodyPartByDefName, permanent, level, pain, duration, otherPawnFromSeparatedString);
    }

    // Token: 0x0600030E RID: 782 RVA: 0x00018718 File Offset: 0x00016918
    public static void AddHediff2(this Pawn pawn, bool random, HediffDef hediff = null, float severity = 1f, BodyPartRecord bpr = null, bool isPermanent = false, int level = -1, int pain = -1, int duration = -1, Pawn otherPawn = null)
    {
      bool flag = pawn == null;
      if (!flag)
      {
        if (random)
        {
          bool flag2 = hediff == null;
          if (flag2)
          {
            hediff = (from td in DefDatabase<HediffDef>.AllDefs
                      where td.defName != null
                      select td).ToList<HediffDef>().RandomElement<HediffDef>();
          }
          bool flag3 = HealthTool.IsAdjustableSeverity(hediff);
          bool flag4 = flag3;
          if (flag4)
          {
            float maxSeverity = HealthTool.GetMaxSeverity(hediff);
            int maxValue = (int)(maxSeverity * 100f);
            int minValue = (int)(hediff.minSeverity * 100f);
            severity = (float)CEditor.zufallswert.Next(minValue, maxValue) / 100f;
            bool flag5 = severity > maxSeverity;
            if (flag5)
            {
              severity = maxSeverity;
            }
          }
          bool flag6 = hediff.injuryProps != null;
          if (flag6)
          {
            isPermanent = (CEditor.zufallswert.Next(0, 5) == 0);
          }
          bool flag7 = hediff.IsHediffWithLevel();
          if (flag7)
          {
            level = CEditor.zufallswert.Next((int)hediff.minSeverity, (int)hediff.maxSeverity);
          }
          bool flag8 = hediff.IsHediffWithComps();
          if (flag8)
          {
            pain = (int)HealthTool.ConvertSliderToPainCategory(CEditor.zufallswert.Next(0, 3));
            duration = CEditor.zufallswert.Next(0, 220000);
          }
          bool flag9 = hediff.IsHediffWithOtherPawn();
          if (flag9)
          {
            otherPawn = Find.WorldPawns.AllPawnsAlive.RandomElement<Pawn>();
          }
          List<BodyPartRecord> listOfAllowedBodyPartRecords = pawn.GetListOfAllowedBodyPartRecords(hediff);
          bool flag10 = !listOfAllowedBodyPartRecords.NullOrEmpty<BodyPartRecord>();
          if (flag10)
          {
            bpr = listOfAllowedBodyPartRecords.RandomElement<BodyPartRecord>();
          }
        }
        bool flag11 = hediff != null;
        if (flag11)
        {
          try
          {
            bool flag12 = bpr != null;
            if (flag12)
            {
              bool flag13 = !pawn.health.hediffSet.PartIsMissing(bpr);
              if (flag13)
              {
                Hediff hediff2 = HediffMaker.MakeHediff(hediff, pawn, bpr);
                hediff2.Severity = severity;
                hediff2.SetLevel(level);
                hediff2.SetPermanent(isPermanent);
                hediff2.SetPainValue(pain);
                hediff2.SetDuration(duration);
                hediff2.SetOtherPawn(otherPawn);
                pawn.health.AddHediff(hediff2, bpr, null, null);
                pawn.health.Notify_HediffChanged(hediff2);
                bool flag14 = hediff2.GetType() == typeof(Hediff_Psylink);
                if (flag14)
                {
                  pawn.CheckAddPsylink((int)Math.Round((double)severity));
                }
              }
            }
            else
            {
              List<BodyPartRecord> listOfAllowedBodyPartRecords2 = pawn.GetListOfAllowedBodyPartRecords(hediff);
              bool flag15 = hediff.IsForWholeBody() || listOfAllowedBodyPartRecords2.CountAllowNull<BodyPartRecord>() > 1;
              if (flag15)
              {
                HealthUtility.AdjustSeverity(pawn, hediff, severity);
              }
              Hediff hediff3 = pawn.TryGetHediffByDefName(hediff.defName);
              bool flag16 = hediff3 != null;
              if (flag16)
              {
                hediff3.Severity = severity;
                hediff3.SetLevel(level);
                hediff3.SetPermanent(isPermanent);
                hediff3.SetPainValue(pain);
                hediff3.SetDuration(duration);
                hediff3.SetOtherPawn(otherPawn);
              }
            }
            pawn.health.summaryHealth.Notify_HealthChanged();
          }
          catch
          {
          }
          Pawn_NeedsTracker needs = pawn.needs;
          if (needs != null)
          {
            needs.AddOrRemoveNeedsAsAppropriate();
          }
          CEditor.API.UpdateGraphics();
        }
      }
    }

    // Token: 0x0600030F RID: 783 RVA: 0x00018A48 File Offset: 0x00016C48
    public static Hediff TryGetHediffByDefName(this Pawn p, string defName)
    {
      Hediff result = null;
      bool flag = p.HasHealthTracker();
      if (flag)
      {
        foreach (Hediff hediff in p.health.hediffSet.hediffs)
        {
          bool flag2 = hediff.def.defName == defName;
          if (flag2)
          {
            result = hediff;
            break;
          }
        }
      }
      return result;
    }

    // Token: 0x06000310 RID: 784 RVA: 0x00018AD4 File Offset: 0x00016CD4
    public static int GetLevel(this Hediff h)
    {
      int result = -1;
      bool flag = h != null && h.def.IsHediffWithLevel();
      if (flag)
      {
        result = ((Hediff_Level)h).level;
      }
      return result;
    }

    // Token: 0x06000311 RID: 785 RVA: 0x00018B0C File Offset: 0x00016D0C
    public static void SetLevel(this Hediff h, int val)
    {
      bool flag = h != null && val >= 0 && h.def.IsHediffWithLevel();
      if (flag)
      {
        ((Hediff_Level)h).level = val;
        h.Severity = (float)val;
      }
    }

    // Token: 0x06000312 RID: 786 RVA: 0x00018B4C File Offset: 0x00016D4C
    public static void SetPermanent(this Hediff h, bool permanent)
    {
      bool flag = h != null && h.def.injuryProps != null;
      if (flag)
      {
        HediffComp_GetsPermanent hediffComp_GetsPermanent = h.TryGetComp<HediffComp_GetsPermanent>();
        bool flag2 = hediffComp_GetsPermanent != null;
        if (flag2)
        {
          hediffComp_GetsPermanent.IsPermanent = permanent;
        }
      }
    }

    // Token: 0x06000313 RID: 787 RVA: 0x00018B90 File Offset: 0x00016D90
    public static PainCategory ConvertSliderToPainCategory(int val)
    {
      bool flag = val <= 0;
      PainCategory result;
      if (flag)
      {
        result = PainCategory.Painless;
      }
      else
      {
        bool flag2 = val == 1;
        if (flag2)
        {
          result = PainCategory.LowPain;
        }
        else
        {
          bool flag3 = val == 2;
          if (flag3)
          {
            result = PainCategory.MediumPain;
          }
          else
          {
            bool flag4 = val == 3;
            if (flag4)
            {
              result = PainCategory.HighPain;
            }
            else
            {
              result = PainCategory.HighPain;
            }
          }
        }
      }
      return result;
    }

    // Token: 0x06000314 RID: 788 RVA: 0x00018BD8 File Offset: 0x00016DD8
    public static int ConvertPainCategoryToSliderVal(PainCategory val)
    {
      bool flag = val == PainCategory.Painless;
      int result;
      if (flag)
      {
        result = 0;
      }
      else
      {
        bool flag2 = val == PainCategory.LowPain;
        if (flag2)
        {
          result = 1;
        }
        else
        {
          bool flag3 = val == PainCategory.MediumPain;
          if (flag3)
          {
            result = 2;
          }
          else
          {
            bool flag4 = val == PainCategory.HighPain;
            if (flag4)
            {
              result = 3;
            }
            else
            {
              result = 3;
            }
          }
        }
      }
      return result;
    }

    // Token: 0x06000315 RID: 789 RVA: 0x00018C20 File Offset: 0x00016E20
    public static int GetPainValue(this Hediff h)
    {
      int result = -1;
      bool flag = h != null && h.def.injuryProps != null;
      if (flag)
      {
        HediffComp_GetsPermanent hediffComp_GetsPermanent = h.TryGetComp<HediffComp_GetsPermanent>();
        bool flag2 = hediffComp_GetsPermanent != null;
        if (flag2)
        {
          result = (int)hediffComp_GetsPermanent.PainCategory;
        }
      }
      return result;
    }

    // Token: 0x06000316 RID: 790 RVA: 0x00018C6C File Offset: 0x00016E6C
    public static void SetPainValue(this Hediff h, int val)
    {
      bool flag = h != null && val >= 0 && h.def.injuryProps != null;
      if (flag)
      {
        HediffComp_GetsPermanent hediffComp_GetsPermanent = h.TryGetComp<HediffComp_GetsPermanent>();
        bool flag2 = hediffComp_GetsPermanent != null;
        if (flag2)
        {
          hediffComp_GetsPermanent.SetPainCategory((PainCategory)val);
        }
      }
    }

    // Token: 0x06000317 RID: 791 RVA: 0x00018CB4 File Offset: 0x00016EB4
    public static int GetDuration(this Hediff h)
    {
      int result = -1;
      bool flag = h != null;
      if (flag)
      {
        HediffComp_Disappears hediffComp_Disappears = h.TryGetComp<HediffComp_Disappears>();
        bool flag2 = hediffComp_Disappears != null;
        if (flag2)
        {
          result = hediffComp_Disappears.ticksToDisappear;
        }
      }
      return result;
    }

    // Token: 0x06000318 RID: 792 RVA: 0x00018CF0 File Offset: 0x00016EF0
    public static void SetDuration(this Hediff h, int val)
    {
      bool flag = h != null && val >= 0;
      if (flag)
      {
        HediffComp_Disappears hediffComp_Disappears = h.TryGetComp<HediffComp_Disappears>();
        bool flag2 = hediffComp_Disappears != null;
        if (flag2)
        {
          hediffComp_Disappears.ticksToDisappear = val;
          hediffComp_Disappears.Props.showRemainingTime = true;
        }
      }
    }

    // Token: 0x06000319 RID: 793 RVA: 0x00018D38 File Offset: 0x00016F38
    public static Pawn GetOtherPawn(this Hediff h)
    {
      Pawn result = null;
      bool flag = h != null;
      if (flag)
      {
        bool flag2 = h.def.IsHediffWithTarget();
        if (flag2)
        {
          HediffWithTarget hediffWithTarget = h as HediffWithTarget;
          bool flag3 = hediffWithTarget.target is Pawn;
          if (flag3)
          {
            result = (Pawn)hediffWithTarget.target;
          }
        }
        else
        {
          bool flag4 = h.def.IsHediffLink();
          if (flag4)
          {
            HediffComp_Link hediffComp_Link = h.TryGetComp<HediffComp_Link>();
            bool flag5 = hediffComp_Link != null;
            if (flag5)
            {
              result = (Pawn)hediffComp_Link.other;
            }
          }
        }
      }
      return result;
    }

    // Token: 0x0600031A RID: 794 RVA: 0x00018DC4 File Offset: 0x00016FC4
    public static void SetOtherPawn(this Hediff h, Pawn p)
    {
      bool flag = h != null && p != null;
      if (flag)
      {
        bool flag2 = h.def.IsHediffWithTarget();
        if (flag2)
        {
          HediffWithTarget hediffWithTarget = h as HediffWithTarget;
          hediffWithTarget.target = p;
        }
        else
        {
          bool flag3 = h.def.IsHediffLink();
          if (flag3)
          {
            HediffComp_Link hediffComp_Link = h.TryGetComp<HediffComp_Link>();
            bool flag4 = hediffComp_Link != null;
            if (flag4)
            {
              hediffComp_Link.other = p;
            }
          }
        }
      }
    }

    // Token: 0x0600031B RID: 795 RVA: 0x00018E2F File Offset: 0x0001702F
    public static bool IsHediffWithOtherPawn(this HediffDef h)
    {
      return h != null && (h.IsHediffWithTarget() || h.IsHediffLink());
    }

    // Token: 0x0600031C RID: 796 RVA: 0x00018E48 File Offset: 0x00017048
    public static bool IsHediffLink(this HediffDef h)
    {
      return h != null && (h.HasComp(typeof(HediffComp_Link)) || h.HasComp(typeof(HediffComp_EntropyLink)));
    }

    // Token: 0x0600031D RID: 797 RVA: 0x00018E78 File Offset: 0x00017078
    public static bool IsHediffWithTarget(this HediffDef h)
    {
      return h != null && h.hediffClass != null && (h.hediffClass == typeof(HediffWithTarget) || h.hediffClass.BaseType == typeof(HediffWithTarget));
    }

    // Token: 0x0600031E RID: 798 RVA: 0x00018ED0 File Offset: 0x000170D0
    public static bool IsHediffWithComps(this HediffDef h)
    {
      return h != null && h.hediffClass != null && (h.hediffClass == typeof(HediffWithComps) || h.hediffClass.BaseType == typeof(HediffWithComps));
    }

    // Token: 0x0600031F RID: 799 RVA: 0x00018F28 File Offset: 0x00017128
    public static bool IsHediffWithLevel(this HediffDef h)
    {
      return h != null && h.hediffClass != null && (h.hediffClass == typeof(Hediff_Level) || h.hediffClass.BaseType == typeof(Hediff_Level));
    }

    // Token: 0x06000320 RID: 800 RVA: 0x00018F80 File Offset: 0x00017180
    public static bool IsImplant(this HediffDef h)
    {
      return h != null && h.hediffClass != null && (h.hediffClass == typeof(Hediff_Implant) || h.hediffClass.BaseType == typeof(Hediff_Implant));
    }

    // Token: 0x06000321 RID: 801 RVA: 0x00018FD8 File Offset: 0x000171D8
    public static bool IsAddedPart(this HediffDef h)
    {
      return h != null && h.hediffClass != null && (h.hediffClass == typeof(Hediff_AddedPart) || h.hediffClass.BaseType == typeof(Hediff_AddedPart));
    }

    // Token: 0x06000322 RID: 802 RVA: 0x00019030 File Offset: 0x00017230
    public static bool IsHigh(this HediffDef h)
    {
      return h != null && h.hediffClass != null && (h.hediffClass == typeof(Hediff_High) || h.hediffClass.BaseType == typeof(Hediff_High));
    }

    // Token: 0x06000323 RID: 803 RVA: 0x00019088 File Offset: 0x00017288
    public static void RemoveHediff(this Pawn pawn, Hediff hediff)
    {
      bool flag = pawn == null || hediff == null;
      if (!flag)
      {
        pawn.health.hediffSet.hediffs.Remove(hediff);
        pawn.health.summaryHealth.Notify_HealthChanged();
        pawn.health.Notify_HediffChanged(hediff);
        pawn.needs.AddOrRemoveNeedsAsAppropriate();
        StatsReportUtility.Reset();
      }
    }

    // Token: 0x06000324 RID: 804 RVA: 0x000190F0 File Offset: 0x000172F0
    public static void ResurrectAndHeal(this Pawn pawn)
    {
      bool dead = pawn.Dead;
      if (dead)
      {
        ResurrectionUtility.Resurrect(pawn);
      }
      try
      {
        List<Hediff> hediffs = pawn.health.hediffSet.hediffs;
        Dictionary<Hediff, bool> dictionary = new Dictionary<Hediff, bool>();
        using (List<Hediff>.Enumerator enumerator = hediffs.GetEnumerator())
        {
          while (enumerator.MoveNext())
          {
            bool flag = enumerator.Current.Part != null && enumerator.Current.def.hediffClass == typeof(Hediff_AddedPart);
            if (flag)
            {
              dictionary.Add(enumerator.Current, false);
            }
            else
            {
              dictionary.Add(enumerator.Current, true);
            }
          }
        }
        foreach (Hediff hediff in dictionary.Keys)
        {
          try
          {
            bool flag2 = dictionary[hediff];
            if (flag2)
            {
              pawn.RemoveHediff(hediff);
            }
          }
          catch
          {
            bool dead2 = pawn.Dead;
            if (dead2)
            {
              ResurrectionUtility.Resurrect(pawn);
            }
          }
        }
      }
      catch
      {
        MessageTool.Show("failed to heal", null);
      }
    }

    // Token: 0x06000325 RID: 805 RVA: 0x0001928C File Offset: 0x0001748C
    public static void Medicate(this Pawn pawn)
    {
      bool flag = !pawn.HasHealthTracker();
      if (!flag)
      {
        Medicine medicine = null;
        MedicalCareCategory medicalCareCategory = (pawn.playerSettings != null) ? pawn.playerSettings.medCare : MedicalCareCategory.Best;
        medicalCareCategory = ((medicalCareCategory == MedicalCareCategory.NoCare || medicalCareCategory == MedicalCareCategory.NoMeds) ? MedicalCareCategory.HerbalOrWorse : medicalCareCategory);
        bool flag2 = medicalCareCategory == MedicalCareCategory.HerbalOrWorse;
        if (flag2)
        {
          medicine = (Medicine)ThingMaker.MakeThing(ThingDefOf.MedicineHerbal, null);
        }
        else
        {
          bool flag3 = medicalCareCategory == MedicalCareCategory.NormalOrWorse;
          if (flag3)
          {
            medicine = (Medicine)ThingMaker.MakeThing(ThingDefOf.MedicineIndustrial, null);
          }
          else
          {
            bool flag4 = medicalCareCategory == MedicalCareCategory.Best;
            if (flag4)
            {
              medicine = (Medicine)ThingMaker.MakeThing(ThingDefOf.MedicineUltratech, null);
            }
          }
        }
        TendUtility.DoTend(null, pawn, medicine);
      }
    }

    // Token: 0x06000326 RID: 806 RVA: 0x00019330 File Offset: 0x00017530
    public static void Hurt(this Pawn pawn)
    {
      int num = CEditor.zufallswert.Next(1, 3);
      for (int i = 0; i < num; i++)
      {
        HediffDef hediff = (from td in DefDatabase<HediffDef>.AllDefs
                            where td.defName != null && td.isBad && td.injuryProps != null
                            select td).ToList<HediffDef>().RandomElement<HediffDef>();
        pawn.AddHediff2(true, hediff, 1f, null, false, -1, -1, -1, null);
      }
    }

    // Token: 0x06000327 RID: 807 RVA: 0x000193A8 File Offset: 0x000175A8
    public static void Addictize(this Pawn pawn)
    {
      HediffDef hediffDef = null;
      for (int i = 0; i < 10; i++)
      {
        hediffDef = (from td in DefDatabase<HediffDef>.AllDefs
                     where td.defName != null && td.IsAddiction
                     select td).ToList<HediffDef>().RandomElement<HediffDef>();
        bool flag = !pawn.health.hediffSet.HasHediff(hediffDef, false);
        if (flag)
        {
          break;
        }
      }
      bool flag2 = hediffDef != null;
      if (flag2)
      {
        pawn.AddHediff2(true, hediffDef, 1f, null, false, -1, -1, -1, null);
      }
    }

    // Token: 0x06000328 RID: 808 RVA: 0x00019438 File Offset: 0x00017638
    public static void Deaddictize(this Pawn pawn)
    {
      foreach (Hediff hediff in pawn.health.hediffSet.hediffs)
      {
        bool flag = hediff.def != null && hediff.def.IsAddiction;
        if (flag)
        {
          pawn.RemoveHediff(hediff);
          break;
        }
      }
    }

    // Token: 0x06000329 RID: 809 RVA: 0x000194B8 File Offset: 0x000176B8
    public static void DamageUntilDeath(this Pawn pawn)
    {
      bool flag = !pawn.Dead;
      if (flag)
      {
        bool downed = pawn.Downed;
        if (downed)
        {
          HealthUtility.DamageUntilDead(pawn);
        }
        else
        {
          HealthUtility.DamageUntilDowned(pawn, true);
        }
      }
    }

    // Token: 0x0600032A RID: 810 RVA: 0x000194F0 File Offset: 0x000176F0
    public static void Anaesthetize(this Pawn pawn)
    {
      HealthUtility.TryAnesthetize(pawn);
    }

    // Token: 0x0600032B RID: 811 RVA: 0x000194FC File Offset: 0x000176FC
    public static void ShowDebugInfo(this Hediff h)
    {
      bool flag = h == null || !Prefs.DevMode;
      if (!flag)
      {
        try
        {
          bool flag2 = h is HediffWithComps;
          if (flag2)
          {
            foreach (HediffComp hediffComp in (h as HediffWithComps).comps)
            {
              bool flag3 = hediffComp != null;
              if (flag3)
              {
                string str = "compType=";
                Type type = hediffComp.GetType();
                MessageTool.Show(str + ((type != null) ? type.ToString() : null), null);
              }
            }
          }
          MessageTool.Show("tags=" + h.def.tags.ListToString<string>(), null);
          string[] array = new string[5];
          array[0] = h.def.defName;
          array[1] = "  Class=";
          int num = 2;
          Type hediffClass = h.def.hediffClass;
          array[num] = ((hediffClass != null) ? hediffClass.ToString() : null);
          array[3] = "  Bclass=";
          int num2 = 4;
          Type baseType = h.def.hediffClass.BaseType;
          array[num2] = ((baseType != null) ? baseType.ToString() : null);
          MessageTool.Show(string.Concat(array), null);
        }
        catch
        {
        }
      }
    }

    // Token: 0x0600032C RID: 812 RVA: 0x00019660 File Offset: 0x00017860
    public static List<HediffDef> ListOfHediffDef(string modname, Pawn p, BodyPartRecord bpr, string filter, bool wholeBody)
    {
      bool bAll1 = modname.NullOrEmpty();
      List<HediffDef> list = (from td in DefDatabase<HediffDef>.AllDefs
                              where !td.label.NullOrEmpty() && (bAll1 || td.IsFromMod(modname))
                              orderby td.label
                              select td).ToList<HediffDef>();
      bool flag = filter == Label.HB_ALLIMPLANTS;
      if (flag)
      {
        list = (from td in list
                where td.IsImplant() || td.IsHediffWithLevel()
                select td).ToList<HediffDef>();
      }
      else
      {
        bool flag2 = filter == Label.HB_ALLADDICTIONS;
        if (flag2)
        {
          list = (from td in list
                  where td.IsAddiction || td.IsHigh() || td.HasComp(typeof(HediffComp_DrugEffectFactor)) || td.HasComp(typeof(HediffComp_Effecter))
                  select td).ToList<HediffDef>();
        }
        else
        {
          bool flag3 = filter == Label.HB_ALLDISEASES;
          if (flag3)
          {
            list = (from td in list
                    where td.isBad && td.defName != "NoPain" && td.defName != "SpeedBoost"
                    select td).ToList<HediffDef>();
          }
          else
          {
            bool flag4 = filter == Label.HB_ALLINJURIES;
            if (flag4)
            {
              list = (from td in list
                      where td.injuryProps != null
                      select td).ToList<HediffDef>();
            }
            else
            {
              bool flag5 = filter == Label.HB_ALLTIME;
              if (flag5)
              {
                list = (from td in list
                        where td.HasComp(typeof(HediffComp_Disappears))
                        select td).ToList<HediffDef>();
              }
              else
              {
                bool flag6 = filter == Label.HB_WITHLEVEL;
                if (flag6)
                {
                  list = (from td in list
                          where td.IsHediffWithLevel()
                          select td).ToList<HediffDef>();
                }
              }
            }
          }
        }
      }
      bool flag7 = bpr != null;
      if (flag7)
      {
        list = (from td in list
                where p.GetListOfAllowedBodyPartRecords(td).Contains(bpr)
                select td).ToList<HediffDef>();
      }
      else if (wholeBody)
      {
        list = (from td in list
                where td.IsForWholeBody()
                select td).ToList<HediffDef>();
      }
      bool devMode = Prefs.DevMode;
      if (devMode)
      {
        MessageTool.Show("list count=" + list.CountAllowNull<HediffDef>().ToString(), null);
      }
      return list;
    }

    // Token: 0x0600032D RID: 813 RVA: 0x000198E0 File Offset: 0x00017AE0
    public static List<BodyPartRecord> GetListOfAllowedBodyPartRecords(this Pawn p, HediffDef h)
    {
      bool flag = p == null || h == null;
      List<BodyPartRecord> result;
      if (flag)
      {
        result = null;
      }
      else
      {
        List<BodyPartRecord> list = new List<BodyPartRecord>();
        bool flag2 = !h.descriptionHyperlinks.NullOrEmpty<DefHyperlink>();
        if (flag2)
        {
          foreach (DefHyperlink defHyper in h.descriptionHyperlinks)
          {
            HealthTool.ResolveHyperlink(defHyper, p, ref list, true);
          }
        }
        bool flag3 = list.NullOrEmpty<BodyPartRecord>() && !h.tags.NullOrEmpty<string>();
        if (flag3)
        {
          List<BodyPartRecord> list2 = new List<BodyPartRecord>();
          foreach (string text in h.tags)
          {
            list2 = p.GetListOfBodyPartRecordsByName(text, h, text == "All");
            bool flag4 = !list2.NullOrEmpty<BodyPartRecord>();
            if (flag4)
            {
              list.AddRange(list2);
            }
          }
        }
        bool flag5 = list.NullOrEmpty<BodyPartRecord>();
        if (flag5)
        {
          foreach (string text2 in HealthTool.AllBodyPartChecks.Keys)
          {
            bool flag6 = HealthTool.AllBodyPartChecks[text2](h);
            if (flag6)
            {
              list = p.GetListOfBodyPartRecordsByName(text2, h, text2 == "All");
              break;
            }
          }
        }
        bool flag7 = list.NullOrEmpty<BodyPartRecord>();
        if (flag7)
        {
          list = p.GetListOfBodyPartRecordsByName(null, h, true);
          bool flag8 = h.modContentPack == null || !h.modContentPack.IsCoreMod;
          bool flag9 = h.injuryProps == null && !h.HasComp(typeof(HediffComp_TendDuration)) && (h.IsHediffWithComps() || flag8);
          if (flag9)
          {
            list.Insert(0, null);
          }
        }
        result = list;
      }
      return result;
    }

    // Token: 0x0600032E RID: 814 RVA: 0x00019AF8 File Offset: 0x00017CF8
    public static void ResolveHyperlink(DefHyperlink defHyper, Pawn p, ref List<BodyPartRecord> l, bool firstPass)
    {
      bool flag = defHyper.def is RecipeDef;
      if (flag)
      {
        RecipeDef recipeDef = (RecipeDef)defHyper.def;
        bool flag2 = !recipeDef.appliedOnFixedBodyParts.NullOrEmpty<BodyPartDef>();
        if (flag2)
        {
          foreach (BodyPartRecord bodyPartRecord in p.RaceProps.body.AllParts)
          {
            foreach (BodyPartDef bodyPartDef in recipeDef.appliedOnFixedBodyParts)
            {
              bool flag3 = bodyPartRecord.def.defName == bodyPartDef.defName;
              if (flag3)
              {
                bool flag4 = !l.Contains(bodyPartRecord);
                if (flag4)
                {
                  l.Add(bodyPartRecord);
                }
              }
            }
          }
        }
        else
        {
          bool flag5 = !recipeDef.appliedOnFixedBodyPartGroups.NullOrEmpty<BodyPartGroupDef>();
          if (flag5)
          {
            foreach (BodyPartRecord bodyPartRecord2 in p.RaceProps.body.AllParts)
            {
              foreach (BodyPartGroupDef bodyPartGroupDef in recipeDef.appliedOnFixedBodyPartGroups)
              {
                bool flag6 = bodyPartRecord2.def.defName == bodyPartGroupDef.defName;
                if (flag6)
                {
                  bool flag7 = !l.Contains(bodyPartRecord2);
                  if (flag7)
                  {
                    l.Add(bodyPartRecord2);
                  }
                }
              }
            }
          }
          else if (firstPass)
          {
            bool flag8 = !recipeDef.descriptionHyperlinks.NullOrEmpty<DefHyperlink>();
            if (flag8)
            {
              foreach (DefHyperlink defHyper2 in recipeDef.descriptionHyperlinks)
              {
                HealthTool.ResolveHyperlink(defHyper2, p, ref l, false);
              }
            }
          }
        }
      }
      else
      {
        bool flag9 = defHyper.def is ThingDef;
        if (flag9)
        {
          ThingDef thingDef = (ThingDef)defHyper.def;
          CompProperties compByType = thingDef.GetCompByType(typeof(CompProperties_UseEffectInstallImplant));
          bool flag10 = compByType != null;
          if (flag10)
          {
            CompProperties_UseEffectInstallImplant compProperties_UseEffectInstallImplant = compByType as CompProperties_UseEffectInstallImplant;
            bool flag11 = compProperties_UseEffectInstallImplant.bodyPart != null;
            if (flag11)
            {
              foreach (BodyPartRecord bodyPartRecord3 in p.RaceProps.body.AllParts)
              {
                bool flag12 = bodyPartRecord3.def.defName == compProperties_UseEffectInstallImplant.bodyPart.defName;
                if (flag12)
                {
                  bool flag13 = !l.Contains(bodyPartRecord3);
                  if (flag13)
                  {
                    l.Add(bodyPartRecord3);
                  }
                }
              }
            }
          }
          else if (firstPass)
          {
            bool flag14 = !thingDef.descriptionHyperlinks.NullOrEmpty<DefHyperlink>();
            if (flag14)
            {
              foreach (DefHyperlink defHyper3 in thingDef.descriptionHyperlinks)
              {
                HealthTool.ResolveHyperlink(defHyper3, p, ref l, false);
              }
            }
          }
        }
      }
    }

    // Token: 0x0600032F RID: 815 RVA: 0x00019ECC File Offset: 0x000180CC
    public static List<BodyPartRecord> GetListOfBodyPartRecordsByName(this Pawn p, string defName, HediffDef h, bool all = false)
    {
      List<BodyPartRecord> list = new List<BodyPartRecord>();
      bool flag = p != null && h != null;
      if (flag)
      {
        bool flag2 = defName == "WholeBody";
        if (flag2)
        {
          list.Add(null);
        }
        else
        {
          foreach (BodyPartRecord bodyPartRecord in p.RaceProps.body.AllParts)
          {
            if (all)
            {
              bool flag3 = !list.Contains(bodyPartRecord);
              if (flag3)
              {
                list.Add(bodyPartRecord);
              }
            }
            else
            {
              bool flag4 = !h.hediffGivers.NullOrEmpty<HediffGiver>();
              if (flag4)
              {
                foreach (HediffGiver hediffGiver in h.hediffGivers)
                {
                  bool flag5 = hediffGiver.partsToAffect.NullOrEmpty<BodyPartDef>();
                  if (flag5)
                  {
                    bool flag6 = bodyPartRecord.def.defName == defName;
                    if (flag6)
                    {
                      bool flag7 = !list.Contains(bodyPartRecord);
                      if (flag7)
                      {
                        list.Add(bodyPartRecord);
                      }
                      break;
                    }
                  }
                  else
                  {
                    bool flag8 = hediffGiver.partsToAffect.Contains(bodyPartRecord.def);
                    if (flag8)
                    {
                      bool flag9 = !list.Contains(bodyPartRecord);
                      if (flag9)
                      {
                        list.Add(bodyPartRecord);
                      }
                      break;
                    }
                  }
                }
              }
              else
              {
                bool flag10 = bodyPartRecord.def.defName == defName;
                if (flag10)
                {
                  bool flag11 = !list.Contains(bodyPartRecord);
                  if (flag11)
                  {
                    list.Add(bodyPartRecord);
                  }
                }
              }
            }
          }
        }
      }
      return list;
    }

    // Token: 0x04000134 RID: 308
    public const string HB_All = "All";

    // Token: 0x04000135 RID: 309
    public const string HB_AllImplants = "AllImplants";

    // Token: 0x04000136 RID: 310
    public const string HB_AllAddictions = "AllAddictions";

    // Token: 0x04000137 RID: 311
    public const string HB_AllDiseases = "AllDiseases";

    // Token: 0x04000138 RID: 312
    public const string HB_AllInjuries = "AllInjuries";

    // Token: 0x04000139 RID: 313
    public const string HB_AllTime = "AllTime";

    // Token: 0x0400013A RID: 314
    public const string HB_Arm = "Arm";

    // Token: 0x0400013B RID: 315
    public const string HB_Brain = "Brain";

    // Token: 0x0400013C RID: 316
    public const string HB_Clavicle = "Clavicle";

    // Token: 0x0400013D RID: 317
    public const string HB_Ear = "Ear";

    // Token: 0x0400013E RID: 318
    public const string HB_Eye = "Eye";

    // Token: 0x0400013F RID: 319
    public const string HB_Femur = "Femur";

    // Token: 0x04000140 RID: 320
    public const string HB_Finger = "Finger";

    // Token: 0x04000141 RID: 321
    public const string HB_Foot = "Foot";

    // Token: 0x04000142 RID: 322
    public const string HB_Hand = "Hand";

    // Token: 0x04000143 RID: 323
    public const string HB_Head = "Head";

    // Token: 0x04000144 RID: 324
    public const string HB_Heart = "Heart";

    // Token: 0x04000145 RID: 325
    public const string HB_Humerus = "Humerus";

    // Token: 0x04000146 RID: 326
    public const string HB_Jaw = "Jaw";

    // Token: 0x04000147 RID: 327
    public const string HB_Kidney = "Kidney";

    // Token: 0x04000148 RID: 328
    public const string HB_Leg = "Leg";

    // Token: 0x04000149 RID: 329
    public const string HB_Liver = "Liver";

    // Token: 0x0400014A RID: 330
    public const string HB_Lung = "Lung";

    // Token: 0x0400014B RID: 331
    public const string HB_Neck = "Neck";

    // Token: 0x0400014C RID: 332
    public const string HB_Nose = "Nose";

    // Token: 0x0400014D RID: 333
    public const string HB_Pelvis = "Pelvis";

    // Token: 0x0400014E RID: 334
    public const string HB_Radius = "Radius";

    // Token: 0x0400014F RID: 335
    public const string HB_Shoulder = "Shoulder";

    // Token: 0x04000150 RID: 336
    public const string HB_Skull = "Skull";

    // Token: 0x04000151 RID: 337
    public const string HB_Spine = "Spine";

    // Token: 0x04000152 RID: 338
    public const string HB_Sternum = "Sternum";

    // Token: 0x04000153 RID: 339
    public const string HB_Stomach = "Stomach";

    // Token: 0x04000154 RID: 340
    public const string HB_Tibia = "Tibia";

    // Token: 0x04000155 RID: 341
    public const string HB_Toe = "Toe";

    // Token: 0x04000156 RID: 342
    public const string HB_Torso = "Torso";

    // Token: 0x04000157 RID: 343
    public const string HB_UtilitySlot = "UtilitySlot";

    // Token: 0x04000158 RID: 344
    public const string HB_Utility = "Utility";

    // Token: 0x04000159 RID: 345
    public const string HB_WholeBody = "WholeBody";

    // Token: 0x0400015A RID: 346
    public const string H_Abasia = "Abasia";

    // Token: 0x0400015B RID: 347
    public const string H_AestheticShaper = "AestheticShaper";

    // Token: 0x0400015C RID: 348
    public const string H_AlcoholTolerance = "AlcoholTolerance";

    // Token: 0x0400015D RID: 349
    public const string H_Alzheimers = "Alzheimers";

    // Token: 0x0400015E RID: 350
    public const string H_Anesthetic = "Anesthetic";

    // Token: 0x0400015F RID: 351
    public const string H_Asthma = "Asthma";

    // Token: 0x04000160 RID: 352
    public const string H_BadBack = "BadBack";

    // Token: 0x04000161 RID: 353
    public const string H_Blindness = "Blindness";

    // Token: 0x04000162 RID: 354
    public const string H_BloodLoss = "BloodLoss";

    // Token: 0x04000163 RID: 355
    public const string H_Carcinoma = "Carcinoma";

    // Token: 0x04000164 RID: 356
    public const string H_Cataract = "Cataract";

    // Token: 0x04000165 RID: 357
    public const string H_CatatonicBreakdown = "CatatonicBreakdown";

    // Token: 0x04000166 RID: 358
    public const string H_ChemicalDamageModerate = "ChemicalDamageModerate";

    // Token: 0x04000167 RID: 359
    public const string H_ChemicalDamageSevere = "ChemicalDamageSevere";

    // Token: 0x04000168 RID: 360
    public const string H_Circadian = "Circadian";

    // Token: 0x04000169 RID: 361
    public const string H_Cirrhosis = "Cirrhosis";

    // Token: 0x0400016A RID: 362
    public const string H_Coagulator = "Coagulator";

    // Token: 0x0400016B RID: 363
    public const string H_Cochlear = "Cochlear";

    // Token: 0x0400016C RID: 364
    public const string H_CryptosleepSickness = "CryptosleepSickness";

    // Token: 0x0400016D RID: 365
    public const string H_Dementia = "Dementia";

    // Token: 0x0400016E RID: 366
    public const string H_Denture = "Denture";

    // Token: 0x0400016F RID: 367
    public const string H_DrugOverdose = "DrugOverdose";

    // Token: 0x04000170 RID: 368
    public const string H_ElbowBlade = "ElbowBlade";

    // Token: 0x04000171 RID: 369
    public const string H_Fangs = "Fangs";

    // Token: 0x04000172 RID: 370
    public const string H_FibrousMechanites = "FibrousMechanites";

    // Token: 0x04000173 RID: 371
    public const string H_FieldHand = "FieldHand";

    // Token: 0x04000174 RID: 372
    public const string H_Flu = "Flu";

    // Token: 0x04000175 RID: 373
    public const string H_FoodPoisoning = "FoodPoisoning";

    // Token: 0x04000176 RID: 374
    public const string H_Force = "_Force";

    // Token: 0x04000177 RID: 375
    public const string H_Frail = "Frail";

    // Token: 0x04000178 RID: 376
    public const string H_GastroAnalyzer = "GastroAnalyzer";

    // Token: 0x04000179 RID: 377
    public const string H_GutWorms = "GutWorms";

    // Token: 0x0400017A RID: 378
    public const string H_HandTalon = "HandTalon";

    // Token: 0x0400017B RID: 379
    public const string H_Hangover = "Hangover";

    // Token: 0x0400017C RID: 380
    public const string H_TortureCrown = "TortureCrown";

    // Token: 0x0400017D RID: 381
    public const string H_BlindFold = "Blindfold";

    // Token: 0x0400017E RID: 382
    public const string H_HealingEnhancer = "HealingEnhancer";

    // Token: 0x0400017F RID: 383
    public const string H_Hear = "Hearing";

    // Token: 0x04000180 RID: 384
    public const string H_HearingLoss = "HearingLoss";

    // Token: 0x04000181 RID: 385
    public const string H_High = "High";

    // Token: 0x04000182 RID: 386
    public const string H_HungerMaker = "HungerMaker";

    // Token: 0x04000183 RID: 387
    public const string H_Immunoenhancer = "Immunoenhancer";

    // Token: 0x04000184 RID: 388
    public const string H_Joyfuzz = "Joyfuzz";

    // Token: 0x04000185 RID: 389
    public const string H_Joywire = "Joywire";

    // Token: 0x04000186 RID: 390
    public const string H_KneeSpike = "KneeSpike";

    // Token: 0x04000187 RID: 391
    public const string H_LearningAssistant = "LearningAssistant";

    // Token: 0x04000188 RID: 392
    public const string H_LoveEnhancer = "LoveEnhancer";

    // Token: 0x04000189 RID: 393
    public const string H_Malaria = "Malaria";

    // Token: 0x0400018A RID: 394
    public const string H_Malnutrition = "Malnutrition";

    // Token: 0x0400018B RID: 395
    public const string H_Mindscrew = "Mindscrew";

    // Token: 0x0400018C RID: 396
    public const string H_Command = "Command";

    // Token: 0x0400018D RID: 397
    public const string H_MuscleParasites = "MuscleParasites";

    // Token: 0x0400018E RID: 398
    public const string H_Neurocalcularor = "Neurocalculator";

    // Token: 0x0400018F RID: 399
    public const string H_NoPain = "NoPain";

    // Token: 0x04000190 RID: 400
    public const string H_Painstopper = "Painstopper";

    // Token: 0x04000191 RID: 401
    public const string H_Plague = "Plague";

    // Token: 0x04000192 RID: 402
    public const string H_PowerClaw = "PowerClaw";

    // Token: 0x04000193 RID: 403
    public const string H_Pregnant = "Pregnant";

    // Token: 0x04000194 RID: 404
    public const string H_Psychic = "Psychic";

    // Token: 0x04000195 RID: 405
    public const string H_PsychicShock = "PsychicShock";

    // Token: 0x04000196 RID: 406
    public const string H_PsychicEntropy = "PsychicEntropy";

    // Token: 0x04000197 RID: 407
    public const string H_NeuralHealRecoveryGain = "NeuralHealRecoveryGain";

    // Token: 0x04000198 RID: 408
    public const string H_WorkDrive = "WorkDrive";

    // Token: 0x04000199 RID: 409
    public const string H_PreachHealth = "PreachHealth";

    // Token: 0x0400019A RID: 410
    public const string H_BerserkTrance = "BerserkTrance";

    // Token: 0x0400019B RID: 411
    public const string H_ImmunityDrive = "ImmunityDrive";

    // Token: 0x0400019C RID: 412
    public const string H_WorkFocus = "WorkFocus";

    // Token: 0x0400019D RID: 413
    public const string H_NeuralSupercharge = "NeuralSupercharge";

    // Token: 0x0400019E RID: 414
    public const string H_ResurrectionPsychosis = "ResurrectionPsychosis";

    // Token: 0x0400019F RID: 415
    public const string H_ResurrectionSickness = "ResurrectionSickness";

    // Token: 0x040001A0 RID: 416
    public const string H_SensoryMechanites = "SensoryMechanites";

    // Token: 0x040001A1 RID: 417
    public const string H_Skingland = "skinGland";

    // Token: 0x040001A2 RID: 418
    public const string H_SleepingSickness = "SleepingSickness";

    // Token: 0x040001A3 RID: 419
    public const string H_Smelling = "Smelling";

    // Token: 0x040001A4 RID: 420
    public const string H_SpeedBoost = "SpeedBoost";

    // Token: 0x040001A5 RID: 421
    public const string H_tolerance = "tolerance";

    // Token: 0x040001A6 RID: 422
    public const string H_ToxicBuildup = "ToxicBuildup";

    // Token: 0x040001A7 RID: 423
    public const string H_TraumaSavant = "TraumaSavant";

    // Token: 0x040001A8 RID: 424
    public const string H_VenomTalon = "VenomTalon";

    // Token: 0x040001A9 RID: 425
    public const string H_WoodenFoot = "WoodenFoot";

    // Token: 0x040001AA RID: 426
    public const string H_WoundInfection = "WoundInfection";
  }
}
