﻿using System;
using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
  public class Listing_X : Listing
  {
    public float DefSelectionLineHeight = 22f;
    private GameFont font;
    private Color MoodColor = new Color(0.1f, 1f, 0.1f);
    private Color MoodColorNegative = new Color(0.8f, 0.4f, 0.4f);
    private Color NoEffectColor = new Color(0.5f, 0.5f, 0.5f, 0.75f);
    private List<Pair<Vector2, Vector2>> labelScrollbarPositions;
    private List<Vector2> labelScrollbarPositionsSetThisFrame;
    private Texture2D texRemove;
    private bool alternate = true;
    //public string buffer;
    //public int ticker = 0;
    //public bool open = false;

    public Listing_X(GameFont font)
    {
      this.font = font;
    }

    public Listing_X()
    {
      this.font = GameFont.Small;
    }

    public float CurY
    {
      get
      {
        return this.curY;
      }
      set
      {
        this.curY = value;
      }
    }

    public float CurX
    {
      get
      {
        return this.curX;
      }
      set
      {
        this.curX = value;
      }
    }

    // Token: 0x0600056D RID: 1389 RVA: 0x00037D8A File Offset: 0x00035F8A
    public override void Begin(Rect rect)
    {
      base.Begin(rect);
      Text.Font = this.font;
      this.texRemove = ContentFinder<Texture2D>.Get("UI/Buttons/Delete", true);
    }

    // Token: 0x0600056E RID: 1390 RVA: 0x00037DB4 File Offset: 0x00035FB4
    public Listing_Standard BeginSection(float height)
    {
      Rect rect = base.GetRect(height + 8f);
      Widgets.DrawMenuSection(rect);
      Listing_Standard listing_Standard = new Listing_Standard();
      listing_Standard.Begin(rect.ContractedBy(4f));
      return listing_Standard;
    }

    // Token: 0x0600056F RID: 1391 RVA: 0x00037DF4 File Offset: 0x00035FF4
    public void FloatMenuOnButtonImage<T>(float xOff, float yOff, float w, float h, string texPath, List<T> l, Func<T, string> labelGetter, Action<T> action)
    {
      Rect butRect = new Rect(this.curX + xOff, this.curY + yOff, w, h);
      bool flag = Widgets.ButtonImage(butRect, ContentFinder<Texture2D>.Get(texPath, true), true);
      if (flag)
      {
        SZWidgets.FloatMenuOnRect<T>(l, labelGetter, action);
      }
    }

    // Token: 0x06000570 RID: 1392 RVA: 0x00037E40 File Offset: 0x00036040
    public bool ButtonImage(float xOff, float yOff, float w, float h, string texPath, Action action)
    {
      Rect butRect = new Rect(this.curX + xOff, this.curY + yOff, w, h);
      bool flag = Widgets.ButtonImage(butRect, ContentFinder<Texture2D>.Get(texPath, true), true);
      bool flag2 = flag && action != null;
      if (flag2)
      {
        action();
      }
      return flag;
    }

    // Token: 0x06000571 RID: 1393 RVA: 0x00037E94 File Offset: 0x00036094
    public bool ButtonImage(Texture2D tex, float xOffset, float width, float height, Action action)
    {
      base.NewColumnIfNeeded(height);
      bool flag = Widgets.ButtonImage(new Rect(this.curX + xOffset, this.curY, width, height), tex, true);
      base.Gap(height + this.verticalSpacing);
      bool flag2 = flag && action != null;
      if (flag2)
      {
        action();
      }
      return flag;
    }

    // Token: 0x06000572 RID: 1394 RVA: 0x00037EF8 File Offset: 0x000360F8
    public bool ButtonText(string label, float x, float y, float w, float h, Action action, string highlightTag = null)
    {
      Rect rect = new Rect(x, y, w, h);
      bool flag = Widgets.ButtonText(rect, label, true, false, true);
      bool flag2 = highlightTag != null;
      if (flag2)
      {
        UIHighlighter.HighlightOpportunity(rect, highlightTag);
      }
      bool flag3 = flag && action != null;
      if (flag3)
      {
        action();
      }
      return flag;
    }

    // Token: 0x06000573 RID: 1395 RVA: 0x00037F50 File Offset: 0x00036150
    public bool ButtonText(string label, string highlightTag = null)
    {
      Rect rect = base.GetRect(30f);
      bool result = Widgets.ButtonText(rect, label, true, false, true);
      bool flag = highlightTag != null;
      if (flag)
      {
        UIHighlighter.HighlightOpportunity(rect, highlightTag);
      }
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x06000574 RID: 1396 RVA: 0x00037F9C File Offset: 0x0003619C
    public bool ButtonTextLabeled(string label, string buttonLabel)
    {
      Rect rect = base.GetRect(30f);
      Widgets.Label(rect.LeftHalf(), label);
      bool result = Widgets.ButtonText(rect.RightHalf(), buttonLabel, true, false, true);
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x06000575 RID: 1397 RVA: 0x00037FE8 File Offset: 0x000361E8
    public void CheckboxLabeledWithDefault(string label, float xOff, float width, ref bool checkOn, bool defaultVal, string tooltip = null)
    {
      Rect rect = this.BaseCheckboxLabeled(label, xOff, width - 24f, ref checkOn, tooltip);
      Rect rect2 = new Rect(rect.x + rect.width, rect.y, 24f, 24f);
      bool flag = Widgets.ButtonImage(rect2, ContentFinder<Texture2D>.Get("bdefault", true), true);
      if (flag)
      {
        checkOn = defaultVal;
      }
      TooltipHandler.TipRegion(rect2, CharacterEditor.Label.O_SETTODEFAULT);
      base.Gap(this.verticalSpacing);
    }

    // Token: 0x06000576 RID: 1398 RVA: 0x0003806C File Offset: 0x0003626C
    public void CheckboxLabeled(string label, float xOff, float width, ref bool checkOn, string tooltip = null)
    {
      this.BaseCheckboxLabeled(label, xOff, width, ref checkOn, tooltip);
      base.Gap(this.verticalSpacing);
    }

    // Token: 0x06000577 RID: 1399 RVA: 0x0003808C File Offset: 0x0003628C
    private Rect BaseCheckboxLabeled(string label, float xOff, float width, ref bool checkOn, string tooltip = null)
    {
      float lineHeight = Text.LineHeight;
      Rect rect = base.GetRect(lineHeight);
      rect.width = width;
      rect.x += xOff;
      bool flag = !tooltip.NullOrEmpty();
      if (flag)
      {
        bool flag2 = Mouse.IsOver(rect);
        if (flag2)
        {
          Widgets.DrawHighlight(rect);
        }
        TooltipHandler.TipRegion(rect, tooltip);
      }
      Widgets.CheckboxLabeled(rect, label, ref checkOn, false, null, null, false);
      return rect;
    }

    // Token: 0x06000578 RID: 1400 RVA: 0x00038108 File Offset: 0x00036308
    public bool CheckboxLabeledSelectable(string label, ref bool selected, ref bool checkOn)
    {
      float lineHeight = Text.LineHeight;
      Rect rect = base.GetRect(lineHeight);
      bool result = Widgets.CheckboxLabeledSelectable(rect, label, ref selected, ref checkOn, null, 1f);
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x06000579 RID: 1401 RVA: 0x00038148 File Offset: 0x00036348
    public override void End()
    {
      base.End();
      bool flag = this.labelScrollbarPositions != null;
      if (flag)
      {
        for (int i = this.labelScrollbarPositions.Count - 1; i >= 0; i--)
        {
          bool flag2 = !this.labelScrollbarPositionsSetThisFrame.Contains(this.labelScrollbarPositions[i].First);
          if (flag2)
          {
            this.labelScrollbarPositions.RemoveAt(i);
          }
        }
        this.labelScrollbarPositionsSetThisFrame.Clear();
      }
    }

    // Token: 0x0600057A RID: 1402 RVA: 0x000381D0 File Offset: 0x000363D0
    public void EndSection(Listing_Standard listing)
    {
      listing.End();
    }

    // Token: 0x0600057B RID: 1403 RVA: 0x000381DC File Offset: 0x000363DC
    public void FloatMenuButtonWithLabel<T>(string label, float wLabel, float wDropbox, string currentVal, List<T> l, Func<T, string> labelGetter, Action<T> action)
    {
      Rect rect = new Rect(this.curX, this.curY + 4f, wLabel, 30f);
      Rect rect2 = new Rect(this.curX + wLabel, this.curY, wDropbox, 30f);
      Widgets.Label(rect, label);
      bool flag = !l.NullOrEmpty<T>();
      if (flag)
      {
        SZWidgets.FloatMenuOnButtonText<T>(rect2, currentVal, l, labelGetter, action);
      }
      base.Gap(this.verticalSpacing);
    }

    public void IntAdjuster(ref int val, int countChange, int min = 0)
    {
      Rect rect = base.GetRect(24f);
      rect.width = 42f;
      bool flag = Widgets.ButtonText(rect, "-" + countChange.ToString(), true, false, true);
      if (flag)
      {
        val -= countChange * GenUI.CurrentAdjustmentMultiplier();
        bool flag2 = val < min;
        if (flag2)
        {
          val = min;
        }
      }
      rect.x += rect.width + 2f;
      bool flag3 = Widgets.ButtonText(rect, "+" + countChange.ToString(), true, false, true);
      if (flag3)
      {
        val += countChange * GenUI.CurrentAdjustmentMultiplier();
        bool flag4 = val < min;
        if (flag4)
        {
          val = min;
        }
      }
      base.Gap(this.verticalSpacing);
    }

    // Token: 0x0600057D RID: 1405 RVA: 0x00038320 File Offset: 0x00036520
    public void IntEntry(ref int val, ref string editBuffer, int multiplier = 1)
    {
      Rect rect = base.GetRect(24f);
      Widgets.IntEntry(rect, ref val, ref editBuffer, multiplier);
      base.Gap(this.verticalSpacing);
    }

    // Token: 0x0600057E RID: 1406 RVA: 0x00038354 File Offset: 0x00036554
    public void IntRange(ref IntRange range, int min, int max)
    {
      Rect rect = base.GetRect(28f);
      Widgets.IntRange(rect, (int)base.CurHeight, ref range, min, max, null, 0);
      base.Gap(this.verticalSpacing);
    }

    public void IntSetter(ref int val, int target, string label)
    {
      Rect rect = base.GetRect(24f);
      bool flag = Widgets.ButtonText(rect, label, true, false, true);
      if (flag)
      {
        val = target;
      }
      base.Gap(this.verticalSpacing);
    }

    public void Label(float xOff, float yOff, float w, float h, string text)
    {
      Rect rect = new Rect(this.curX + xOff, this.curY + yOff, w, h);
      Widgets.Label(rect, text);
    }

    public void LabelSimple(string label, float x, float y, float w, float h, string tooltip = null)
    {
      Rect rect = new Rect(x, y, w, h);
      Widgets.Label(rect, label);
      bool flag = tooltip != null;
      if (flag)
      {
        TooltipHandler.TipRegion(rect, tooltip);
      }
    }

    // Token: 0x06000582 RID: 1410 RVA: 0x0003843C File Offset: 0x0003663C
    public void Label(string text, float width = -1f, float yGap = -1f, float maxHeight = -1f, string tooltip = null)
    {
      float num = Text.CalcHeight(text, base.ColumnWidth);
      bool flag = false;
      bool flag2 = maxHeight >= 0f && num > maxHeight;
      if (flag2)
      {
        num = maxHeight;
        flag = true;
      }
      Rect rect = base.GetRect(num);
      bool flag3 = width >= 0f;
      if (flag3)
      {
        rect.width = width;
      }
      bool flag4 = flag;
      if (flag4)
      {
        Vector2 labelScrollbarPosition = this.GetLabelScrollbarPosition(this.curX, this.curY);
        Widgets.LabelScrollable(rect, text, ref labelScrollbarPosition, false, true, false);
        this.SetLabelScrollbarPosition(this.curX, this.curY, labelScrollbarPosition);
      }
      else
      {
        Widgets.Label(rect, text);
      }
      bool flag5 = tooltip != null;
      if (flag5)
      {
        TooltipHandler.TipRegion(rect, tooltip);
      }
      bool flag6 = yGap == -1f;
      if (flag6)
      {
        base.Gap(this.verticalSpacing);
      }
      else
      {
        base.Gap(yGap);
      }
    }

    // Token: 0x06000583 RID: 1411 RVA: 0x00038524 File Offset: 0x00036724
    public bool Listview(float width, string defName, string name, string tooltip, bool withRemove, Action<string> action)
    {
      bool flag = name == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        Text.Font = GameFont.Small;
        Text.Anchor = TextAnchor.MiddleLeft;
        Rect rect = new Rect(this.curX, this.curY, width - this.DefSelectionLineHeight, this.DefSelectionLineHeight);
        Text.WordWrap = false;
        Widgets.Label(rect, name);
        Text.WordWrap = true;
        bool flag2 = !string.IsNullOrEmpty(tooltip);
        if (flag2)
        {
          TipSignal tip = new TipSignal(() => tooltip, 275);
          TooltipHandler.TipRegion(rect, tip);
        }
        Text.Anchor = TextAnchor.UpperLeft;
        this.curY += this.DefSelectionLineHeight;
        if (withRemove)
        {
          Rect butRect = new Rect(rect.x + rect.width, rect.y, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          bool flag3 = Widgets.ButtonImage(butRect, this.texRemove, Color.grey, true);
          bool flag4 = flag3 && action != null;
          if (flag4)
          {
            action(defName);
          }
          GUI.color = Color.white;
          result = flag3;
        }
        else
        {
          result = Widgets.ButtonInvisible(rect, true);
        }
      }
      return result;
    }

    // Token: 0x06000584 RID: 1412 RVA: 0x00038668 File Offset: 0x00036868
    public bool ListviewTDC(float width, ThingDefCountClass tdc, bool selected, bool withRemove, Action<string> action, ThingDef t)
    {
      bool flag = tdc == null || tdc.thingDef == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        Text.Font = GameFont.Small;
        Text.Anchor = TextAnchor.MiddleLeft;
        Rect rect = new Rect(this.curX, this.curY, width - this.DefSelectionLineHeight, this.DefSelectionLineHeight);
        Text.WordWrap = false;
        Widgets.Label(rect, tdc.thingDef.label + ": " + tdc.count.ToString());
        Text.WordWrap = true;
        if (selected)
        {
          this.curY += this.DefSelectionLineHeight;
          int value = tdc.count;
          value = SZWidgets.NumericIntBox(this.curX, this.curY, 70f, 30f, tdc.count, 0, 10000);
          t.UpdateCost(tdc.thingDef, value);
          this.curY += this.DefSelectionLineHeight;
        }
        if (!string.IsNullOrEmpty(tdc.thingDef.description))
        {
          TipSignal tip = new TipSignal(() => tdc.thingDef.description, 21275);
          TooltipHandler.TipRegion(rect, tip);
        }
        Text.Anchor = TextAnchor.UpperLeft;
        this.curY += this.DefSelectionLineHeight;
        if (withRemove)
        {
          Rect butRect = new Rect(rect.x + rect.width, rect.y, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          bool flag3 = Widgets.ButtonImage(butRect, this.texRemove, Color.grey, true);
          if (flag3)
          {
            bool flag4 = action != null;
            if (flag4)
            {
              action(tdc.thingDef.defName);
            }
          }
          GUI.color = Color.white;
        }
        result = Widgets.ButtonInvisible(rect, true);
      }
      return result;
    }

    public bool ListviewPart(float width, float itemH, Pair<ScenPart, EntryInt> entryPair, bool withRemove, Action<Pair<ScenPart, EntryInt>> action)
    {
      ScenPart part = entryPair.First;
      EntryInt entry = entryPair.Second;

      if (!part.IsSupportedScenarioPart())
      {
        return false;
      }

      Text.Font = GameFont.Small;
      Text.Anchor = TextAnchor.MiddleLeft;
      Rect rect = new Rect(this.curX, this.curY, width - itemH + 7f, itemH);
      Widgets.DrawBoxSolid(rect, this.alternate ? new Color(0.3f, 0.3f, 0.3f, 0.5f) : new Color(0.2f, 0.2f, 0.2f, 0.5f));
      this.alternate = !this.alternate;
      Text.WordWrap = false;
      int num = 0;
      ThingDef td = null;
      if (part.IsScenarioAnimal())
      {
        PawnKindDef scenarioPartAnimal = part.GetScenarioPartAnimal(out num);
        num = entry.valInt;
        if (scenarioPartAnimal != null)
        {
          Rect rect2 = new Rect(this.curX, this.curY, itemH, itemH);
          Widgets.ThingIcon(rect2, scenarioPartAnimal.race, null, null, 1f, null);
          Rect rect3 = new Rect(this.curX + 5f + itemH, this.curY, width, itemH);
          string str = (scenarioPartAnimal.race != null && scenarioPartAnimal.race.label != null) ? scenarioPartAnimal.race.label : "";
          Widgets.Label(rect3, str + ": " + num.ToString());
        }
        else
        {
          Widgets.Label(rect, part.Label + ": " + num.ToString());
        }
      }
      else
      {
        ThingDef thingDef = null;
        ThingStyleDef tsd = null;
        td = ScenarioTool.GetScenarioPartThingDef<ScenPart>(part, out thingDef, out tsd, out num);
        num = entry.valInt;
        if (td != null)
        {
          Rect position = new Rect(this.curX, this.curY, itemH, itemH);
          GUI.color = td.GetColor(thingDef);
          if (td.mineable && thingDef == null)
          {
            GUI.color = td.uiIconColor;
          }
          Texture2D texture = td.GetTexture(num >= td.stackLimit ? td.stackLimit : num, tsd, default(Rot4));
          GUI.DrawTexture(position, texture, ScaleMode.ScaleAndCrop, true);
          GUI.color = Color.white;
          Rect rect4 = new Rect(this.curX + 5f + itemH, this.curY, width, itemH);
          string str2 = (thingDef != null) ? (" (" + thingDef.label + ")") : "";
          Widgets.Label(rect4, td.label + str2 + ": " + num.ToString());
        }
      }
      Text.WordWrap = true;
      //if (selected)
      //{

      entry.NumericTextField(this.curX + width - 140f - (withRemove ? 25f : 0f), this.curY + 2f, 70f, 26f, 0, 20000);
      //if (num != entry.valInt)
      //{
      //  ScenarioTool.SetScenarioPartCount<ScenPart>(part, entry.valInt);
      //}
      //int num2 = num;
      //num = SZWidgets.NumericIntBox(this.curX + width - 140f - (withRemove ? 25f : 0f), this.curY + 2f, 70f, 26f, num, 1, 20000);
      //if (num != num2)
      //{
      //  ScenarioTool.SetScenarioPartCount<ScenPart>(part, num);
      //}
      //}
      if (td != null && !string.IsNullOrEmpty(td.description))
      {
        TipSignal tip = new TipSignal(() => td.description, 4539982);
        TooltipHandler.TipRegion(rect, tip);
      }
      Text.Anchor = TextAnchor.UpperLeft;
      this.curY += itemH;
      if (withRemove)
      {
        Rect butRect = new Rect(rect.x + rect.width - itemH, rect.y, itemH, itemH);
        if (Widgets.ButtonImage(butRect, this.texRemove, Color.grey, true))
        {
          if (action != null)
          {
            action(entryPair);
          }
        }
        GUI.color = Color.white;
      }
      if (entry.valInt == 0 && !entry.Open)
      {
        action(entryPair);
      }
      return Widgets.ButtonInvisible(rect, true);
    }

    // Token: 0x06000586 RID: 1414 RVA: 0x00038C78 File Offset: 0x00036E78
    public bool ListviewSM(float width, StatModifier sm, bool selected, bool withRemove, Action<string> action)
    {
      bool flag = sm == null || sm.stat == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        Text.Font = GameFont.Small;
        Text.Anchor = TextAnchor.MiddleLeft;
        Rect rect = new Rect(this.curX, this.curY, width - this.DefSelectionLineHeight, this.DefSelectionLineHeight);
        Text.WordWrap = false;
        Widgets.Label(rect, sm.stat.label + ": " + sm.value.ToString());
        Text.WordWrap = true;
        if (selected)
        {
          this.curY += this.DefSelectionLineHeight;
          float min = (sm.value < sm.stat.minValue) ? (sm.value - 10f) : sm.stat.minValue;
          sm.value = SZWidgets.NumericFloatBox(this.curX, this.curY, 30f, sm.value, min, sm.stat.maxValue);
          this.curY += this.DefSelectionLineHeight;
        }
        bool flag2 = !string.IsNullOrEmpty(sm.stat.description);
        if (flag2)
        {
          TipSignal tip = new TipSignal(() => sm.stat.description, 21275);
          TooltipHandler.TipRegion(rect, tip);
        }
        Text.Anchor = TextAnchor.UpperLeft;
        this.curY += this.DefSelectionLineHeight;
        if (withRemove)
        {
          Rect butRect = new Rect(rect.x + rect.width, rect.y, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          bool flag3 = Widgets.ButtonImage(butRect, this.texRemove, Color.grey, true);
          if (flag3)
          {
            bool flag4 = action != null;
            if (flag4)
            {
              action(sm.stat.defName);
            }
          }
          GUI.color = Color.white;
        }
        result = Widgets.ButtonInvisible(rect, true);
      }
      return result;
    }

    private string GetFormattedValue(string format, float value)
    {
      //string result;
      if (format == "%")
      {
        return " [" + Math.Round((double)(100f * value), 0).ToString() + " %]";
      }
      if (format == "s")
      {
        return " [" + value.ToString() + " s]";
      }

      if (format == "ticks")
      {
        return " [" + value.ToString() + " ticks]";
      }

      if (format == "rpm")
      {
        return " [" + ((value == 0f) ? CharacterEditor.Label.INFINITE : Math.Round((double)(60f / value * 60f), 0).ToString()) + " rpm]";
      }
      if (format == "cps")
      {
        return " [" + ((value == 0f) ? CharacterEditor.Label.INFINITE : value.ToString()) + " cps]";
      }
      if (format == "cells")
      {
        return " [" + value.ToString() + " cells]";
      }
      if (format.StartsWith("max"))
      {
        return " [" + value.ToString() + "/" + format.SubstringFrom("max", true) + "]";
        //result = string.Concat(new string[]
        //{
        //            " [",
        //            value.ToString(),
        //            "/",
        //            format.SubstringFrom("max", true),
        //            "]"
        //});
      }

      if (format == "int")
      {
        return " [" + ((int)Math.Round((double)value)).ToString() + "]";
      }

      if (format == "quadrum")
      {
        return " [" + Enum.GetName(typeof(Quadrum), (int)value) + "]";
      }

      if (format == "addict")
      {
        return " " + (100.0 - Math.Round((double)(100f * value), 0)).ToString() + " %";
      }
      if (format == "high")
      {
        return " " + value.ToStringPercent("F0");
      }
      if (format.StartsWith("DEF"))
      {
        return " [" + format.SubstringFrom("DEF", true) + "]";
      }
      if (format.StartsWith("dauer"))
      {
        return " " + format.SubstringFrom("dauer", true);
      }
      if (format == "pain")
      {
        int num = (int)value;
        return num == 0 ? CharacterEditor.Label.PAINLESS : ("PainCategory_" + HealthTool.ConvertSliderToPainCategory(num).ToString()).Translate().ToString();
      }
      if (format.StartsWith("comp"))
      {
        if (!format.Contains("%"))
        {
          return " " + value.ToStringPercent("F0") + " " + format.SubstringFrom("comp", true);
        }
        return " " + format.SubstringFrom("comp", true);
      }
      return " [" + value.ToString() + "]";
    }

    // Token: 0x06000588 RID: 1416 RVA: 0x00039290 File Offset: 0x00037490
    public void AddMultiplierSection(string paramName, string format, ref string selectedName, float baseValue, ref float value, float min = -3.4028235E+38f, float max = 3.4028235E+38f, bool small = false)
    {
      Text.Font = (small ? GameFont.Small : GameFont.Medium);
      Rect butRect = new Rect(this.curX, this.curY, this.listingRect.width - 130f, 30f);
      Rect rect = new Rect(this.curX, this.curY, this.listingRect.width, 30f);
      float num = baseValue * value;
      string s = paramName + this.GetFormattedValue(format, num);
      Widgets.Label(rect, s.Colorize((num > 0f) ? Color.green : ((num == 0f) ? Color.grey : Color.red)));
      bool flag = paramName == selectedName;
      if (flag)
      {
        Text.Font = GameFont.Small;
        value = SZWidgets.NumericFloatBox(this.curX + this.listingRect.width - 105f, this.curY + 4f, 25f, value, min, float.MaxValue);
      }
      this.curY += 30f;
      Rect rect2 = new Rect(this.curX, this.curY, this.listingRect.width, 20f);
      Widgets.DrawBoxSolid(new Rect(rect2.x, rect2.y, rect2.width, rect2.height / 2f), new Color(0.195f, 0.195f, 0.193f));
      value = Widgets.HorizontalSlider(rect2, value, min, max, false, null, null, null, -1f);
      string text = value.ToString().SubstringFrom(".", true);
      bool flag2 = text.Length > 2;
      if (flag2)
      {
        value = (float)Math.Round((double)value, 2);
      }
      this.curY += 20f;
      bool flag3 = Widgets.ButtonInvisible(butRect, true);
      if (flag3)
      {
        selectedName = ((selectedName == paramName) ? null : paramName);
      }
    }

    // Token: 0x06000589 RID: 1417 RVA: 0x00039484 File Offset: 0x00037684
    public void AddSection(string paramName, string format, ref string selectedName, ref float value, float min = -3.4028235E+38f, float max = 3.4028235E+38f, bool small = false, string toolTip = "")
    {
      Text.Font = (small ? GameFont.Small : GameFont.Medium);
      Rect butRect = new Rect(this.curX, this.curY, this.listingRect.width - 130f, 30f);
      Rect rect = new Rect(this.curX, this.curY, this.listingRect.width, 30f);
      Widgets.Label(rect, paramName + this.GetFormattedValue(format, value));
      bool flag = !toolTip.NullOrEmpty();
      if (flag)
      {
        TooltipHandler.TipRegion(rect, toolTip);
      }
      bool flag2 = paramName == selectedName;
      if (flag2)
      {
        Text.Font = GameFont.Small;
        value = SZWidgets.NumericFloatBox(this.curX + this.listingRect.width - 105f, this.curY + 4f, 25f, value, min, float.MaxValue);
      }
      this.curY += 30f;
      Rect rect2 = new Rect(this.curX, this.curY, this.listingRect.width, 20f);
      Widgets.DrawBoxSolid(new Rect(rect2.x, rect2.y, rect2.width, rect2.height / 2f), new Color(0.195f, 0.195f, 0.193f));
      value = Widgets.HorizontalSlider(rect2, value, min, max, false, null, null, null, -1f);
      string text = value.ToString().SubstringFrom(".", true);
      bool flag3 = text.Length > 2;
      if (flag3)
      {
        value = (float)Math.Round((double)value, 2);
      }
      this.curY += 20f;
      bool flag4 = Widgets.ButtonInvisible(butRect, true);
      if (flag4)
      {
        selectedName = ((selectedName == paramName) ? null : paramName);
      }
    }

    public void AddIntSection(string paramName, string format, ref string selectedName, ref int value, int min = -2147483648, int max = 2147483647, bool small = false, string toolTip = "")
    {
      Text.Font = (small ? GameFont.Small : GameFont.Medium);
      Rect rect = new Rect(this.curX, this.curY, this.listingRect.width - 130f, 30f);
      Rect rect2 = new Rect(this.curX, this.curY, this.listingRect.width, 30f);
      Widgets.Label(rect2, paramName + this.GetFormattedValue(format, (float)value));
      bool flag = paramName == selectedName;
      if (flag)
      {
        Text.Font = GameFont.Small;
        value = SZWidgets.NumericIntBox(this.curX + this.listingRect.width - 105f, this.curY + 4f, 70f, 25f, value, min, int.MaxValue);
      }
      this.curY += 30f;
      Rect rect3 = new Rect(this.curX, this.curY, this.listingRect.width, 20f);
      Widgets.DrawBoxSolid(new Rect(rect3.x, rect3.y, rect3.width, rect3.height / 2f), new Color(0.195f, 0.195f, 0.193f));
      value = (int)Widgets.HorizontalSlider(rect3, (float)value, (float)min, (float)max, false, null, null, null, -1f);
      this.curY += 20f;
      bool flag2 = Widgets.ButtonInvisible(rect, true);
      if (flag2)
      {
        selectedName = ((selectedName == paramName) ? null : paramName);
      }
      bool flag3 = !toolTip.NullOrEmpty();
      if (flag3)
      {
        TooltipHandler.TipRegion(rect, toolTip);
      }
    }

    // Token: 0x0600058A RID: 1418 RVA: 0x00039664 File Offset: 0x00037864
    //public void AddIntEntrySection(string paramName, string format, ref string selectedName, EntryInt entry, int min, int max, int sliderMin, int sliderMax, bool small = false, string toolTip = "")
    public void AddIntEntrySection(string paramName, string format, EntryInt entry, int min, int max, int sliderMin, int sliderMax, bool small = false, string toolTip = "")
    {
      Text.Font = (small ? GameFont.Small : GameFont.Medium);
      //Rect rect = new Rect(this.curX, this.curY, this.listingRect.width - 130f, 30f);
      Rect rect = new Rect(this.curX, this.curY, this.listingRect.width - 130f + 105f + 70f, 30f);
      Rect rect2 = new Rect(this.curX, this.curY, this.listingRect.width, 30f);
      Widgets.Label(rect2, paramName + this.GetFormattedValue(format, (float)entry.valInt));
      Text.Font = GameFont.Small;
      //if (paramName != selectedName)
      //{
      //  entry.Open = false;
      //}
      entry.Open = true; // Keep this open at all times?
      entry.NumericTextField(this.curX + this.listingRect.width - 105f, this.curY - 3f, 70f, 25f, min, max);
      //if (entry.Open)
      //{
      //  selectedName = paramName;
      //}
      this.curY += 30f;
      Rect rect3 = new Rect(this.curX, this.curY, this.listingRect.width, 20f);
      Widgets.DrawBoxSolid(new Rect(rect3.x, rect3.y, rect3.width, rect3.height / 2f), new Color(0.195f, 0.195f, 0.193f));
      entry.HorizontalSlider(rect3, sliderMin, sliderMax);
      //int sliderVal = entry.valInt;
      //entry.valInt = (int)Widgets.HorizontalSlider(rect3, (float)sliderVal, (float)min, (float)max, false, null, null, null, -1f);
      //if (entry.valInt != sliderVal)
      //{
      //  entry.buffer = entry.valInt.ToString();
      //}
      this.curY += 20f;
      //if (Widgets.ButtonInvisible(rect, true))
      //{
      //  selectedName = paramName;
      //  if (entry.Open)
      //  {
      //    entry.Open = false;
      //  }
      //  //selectedName = selectedName == paramName ? null : paramName;
      //}
      if (!toolTip.NullOrEmpty())
      {
        TooltipHandler.TipRegion(rect, toolTip);
      }
    }

    // Token: 0x0600058B RID: 1419 RVA: 0x0003981C File Offset: 0x00037A1C
    public bool RadioButton(string label, bool active, float tabIn = 0f, string tooltip = null)
    {
      float lineHeight = Text.LineHeight;
      Rect rect = base.GetRect(lineHeight);
      rect.xMin += tabIn;
      bool flag = !tooltip.NullOrEmpty();
      if (flag)
      {
        bool flag2 = Mouse.IsOver(rect);
        if (flag2)
        {
          Widgets.DrawHighlight(rect);
        }
        TooltipHandler.TipRegion(rect, tooltip);
      }
      bool result = Widgets.RadioButtonLabeled(rect, label, active);
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x0600058C RID: 1420 RVA: 0x00039898 File Offset: 0x00037A98
    public bool SelectableFast(string text, bool selected, string tooltip)
    {
      Rect rect = new Rect(this.curX, this.curY, this.listingRect.width, this.DefSelectionLineHeight);
      if (selected)
      {
        GUI.DrawTexture(rect, TexUI.TextBGBlack);
        GUI.color = Color.green;
        Widgets.DrawHighlight(rect);
      }
      else
      {
        GUI.color = Color.white;
      }
      Widgets.Label(rect, text);
      TooltipHandler.TipRegion(rect, tooltip);
      this.curY += this.DefSelectionLineHeight;
      return Widgets.ButtonInvisible(rect, true);
    }

    // Token: 0x0600058D RID: 1421 RVA: 0x00039930 File Offset: 0x00037B30
    public bool SelectableAbility(string name, bool selected, string tooltip, AbilityDef def, Color selColor)
    {
      GUI.color = Color.white;
      Text.Anchor = TextAnchor.UpperLeft;
      float width = this.listingRect.width;
      Rect position = new Rect(this.curX, this.curY, 64f, 64f);
      Rect rect = new Rect(position.x, position.y, width, 64f);
      GUI.DrawTexture(position, def.uiIcon);
      TooltipHandler.TipRegion(rect, tooltip);
      if (selected)
      {
        GUI.DrawTexture(rect, TexUI.TextBGBlack);
        GUI.color = selColor;
        Widgets.DrawHighlight(rect);
      }
      GUI.color = Color.white;
      Widgets.Label(new Rect(position.x + 68f, position.y + 23f, width - 64f, 28f), name);
      return Widgets.ButtonInvisible(rect, false);
    }

    // Token: 0x0600058E RID: 1422 RVA: 0x00039A1C File Offset: 0x00037C1C
    public bool SelectableText<T>(string name, bool isThing, bool isAbility, bool isHair, bool isBeard, bool selected, string tooltip, bool withRemove, bool isWhite, T def, Color selColor, bool hasIcon = false, bool selectOnMouseOver = false, float buttonHeight = -1f)
    {
      //selectOnMouseOver = false; // ToDo: remember to undo this after testing
      bool result;
      if (name == null)
      {
        Log.Warning("SelectableText, name is null");
        return false;
      }

      float width = this.listingRect.width;
      Rect rect = new Rect(this.curX, this.curY, width, this.DefSelectionLineHeight);
      try
      {
        if (isAbility)
        {
          result = this.SelectableAbility(name, selected, tooltip, def as AbilityDef, selColor);
        }
        else
        {
          if (selected)
          {
            GUI.DrawTexture(rect, TexUI.TextBGBlack);
            GUI.color = selColor;
            Widgets.DrawHighlight(rect);
          }
          if (hasIcon || isThing)
          {
            Rect rect2 = new Rect(this.curX, this.curY, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
            ThingDef thingDef = def as ThingDef;
            if (thingDef.uiIcon != null)
            {
              GUI.color = thingDef.uiIconColor;
              Widgets.DrawTextureFitted(rect2, thingDef.uiIcon, 1f);
            }
            else
            {
              Widgets.ThingIcon(rect2, thingDef, null, null, 1f, null);
            }
          }
          Text.WordWrap = false;
          GUI.color = (isWhite ? (selected ? Color.green : Color.white) : Color.gray);
          if (hasIcon || isThing)
          {
            Widgets.Label(new Rect(rect.x + this.DefSelectionLineHeight, rect.y, rect.width, rect.height), name);
          }
          else
          {
            Widgets.Label(rect, name);
          }
          GUI.color = Color.white;
          SZWidgets.TooltipSafe(rect, tooltip);
          //if (!string.IsNullOrEmpty(tooltip))
          //{
          //  TooltipHandler.TipRegion(rect, tooltip);
          //}
          Text.WordWrap = true;
          bool mouseIsOverRects = false;
          if (isHair && def is HairDef hairDef)
          {
            DrawSouthEastNorthGraphics(hairDef.texPath, selectOnMouseOver, ref mouseIsOverRects);
          }
          else if (isBeard && def is BeardDef beardDef)
          {
            DrawSouthEastNorthGraphics(beardDef.texPath, selectOnMouseOver, ref mouseIsOverRects);
            //Graphic g2 = GraphicDatabase.Get<Graphic_Multi>((def as BeardDef).texPath);
            //Rect position2 = new Rect(this.curX, this.curY + this.DefSelectionLineHeight, 64f, 64f);
            //Texture2D textureFromMulti4 = g2.GetTextureFromMulti("_south");
            //GUI.color = Color.white;
            //GUI.DrawTexture(position2, textureFromMulti4);
            //Rect rect5 = new Rect(this.curX + 64f, this.curY + this.DefSelectionLineHeight, 64f, 64f);
            //Texture2D textureFromMulti5 = g2.GetTextureFromMulti("_east");
            //bool flag9 = textureFromMulti5 == null;
            //if (flag9)
            //{
            //  textureFromMulti5 = g2.GetTextureFromMulti("_west");
            //}
            //GUI.color = Color.white;
            //GUI.DrawTexture(rect5, textureFromMulti5);
            //Rect rect6 = new Rect(this.curX + 128f, this.curY + this.DefSelectionLineHeight, 64f, 64f);
            //Texture2D textureFromMulti6 = g2.GetTextureFromMulti("_north");
            //GUI.color = Color.white;
            //GUI.DrawTexture(rect6, textureFromMulti6);
            //if (selectOnMouseOver)
            //{
            //  if (Mouse.IsOver(rect5) || Mouse.IsOver(rect6))
            //  {
            //    mouseIsOverRects = true;
            //  }
            //}
          }
          if (withRemove)
          {
            Rect butRect = new Rect(rect.x + rect.width - this.DefSelectionLineHeight - 12f, rect.y, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
            result = Widgets.ButtonImage(butRect, this.texRemove, true);
          }
          else
          {
            if (selectOnMouseOver && mouseIsOverRects)
            {
              result = true;
            }
            else
            {
              //result = Widgets.ButtonInvisible(rect, false);
              Rect buttonRect = buttonHeight > 0f ? new Rect(rect.x, rect.y, rect.width, buttonHeight) : rect;
              result = Widgets.ButtonInvisible(buttonRect, false);
            }
          }
        }
      }
      catch (Exception ex)
      {
        Log.ErrorOnce("SelectableText, failure for name: " + name + ", excepton: " + ex, 2341897);
        result = false;
      }
      return result;
    }

    public void DrawSouthEastNorthGraphics(string texPath, bool selectOnMouseOver, ref bool mouseIsOverRects)
    {
      try
      {
        if (texPath == null)
        {
          //Log.Message("texPath was null");
          return;
        }

        Graphic g = GraphicDatabase.Get<Graphic_Multi>(texPath);
        if (g == null)
        {
          //Log.Message("Graphic was null for texPath = " + texPath);
          return;
        }

        Rect rectSouth = new Rect(this.curX, this.curY + this.DefSelectionLineHeight, 64f, 64f);
        Texture2D texSouth = g.GetTextureFromMulti("_south");
        if (texSouth != null)
        {
          GUI.color = Color.white;
          GUI.DrawTexture(rectSouth, texSouth);
        }

        Rect rectEast = new Rect(this.curX + 64f, this.curY + this.DefSelectionLineHeight, 64f, 64f);
        Texture2D texEast = g.GetTextureFromMulti("_east");
        if (texEast == null)
        {
          texEast = g.GetTextureFromMulti("_west");
        }
        if (texEast != null)
        {
          GUI.color = Color.white;
          GUI.DrawTexture(rectEast, texEast);
        }

        Rect rectNorth = new Rect(this.curX + 128f, this.curY + this.DefSelectionLineHeight, 64f, 64f);
        Texture2D texNorth = g.GetTextureFromMulti("_north");
        if (texNorth != null)
        {
          GUI.color = Color.white;
          GUI.DrawTexture(rectNorth, texNorth);
        }
        if (selectOnMouseOver && (Mouse.IsOver(rectSouth) || Mouse.IsOver(rectEast) || Mouse.IsOver(rectNorth)))
        {
          mouseIsOverRects = true;
        }
      }
      catch (Exception ex)
      {
        Log.Warning("DrawSouthEastNorthGraphics, exception: " + ex);
      }

    }

    public int SelectableHorizontal(string name, bool selected, string tooltip = "", RenderTexture image = null, ThingDef thingDef = null, HairDef hairDef = null, Vector2 imageSize = default(Vector2), bool withRemove = false, float selectHeight = 22f, Color backColor = default(Color))
    {
      bool flag = name == null;
      int result;
      if (flag)
      {
        result = 0;
      }
      else
      {
        Text.Font = GameFont.Small;
        float num = this.listingRect.width - this.DefSelectionLineHeight;
        Text.Anchor = TextAnchor.MiddleLeft;
        Rect rect = new Rect(this.curX, this.curY, num + this.DefSelectionLineHeight, selectHeight);
        if (selected)
        {
          GUI.DrawTexture(rect, TexUI.TextBGBlack);
          GUI.color = Color.green;
          Widgets.DrawHighlight(rect);
        }
        bool flag2 = thingDef != null;
        Rect rect2;
        if (flag2)
        {
          rect2 = new Rect(this.curX + 25f, this.curY, num, this.DefSelectionLineHeight);
        }
        else
        {
          if (selectHeight == 22f)
          {
            rect2 = new Rect(this.curX, this.curY, num, this.DefSelectionLineHeight);
          }
          else
          {
            rect2 = new Rect(this.curX, this.curY, num + this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          }
        }
        Text.WordWrap = false;
        bool flag4 = backColor != default(Color);
        if (flag4)
        {
          GUI.color = backColor;
        }
        Widgets.Label(rect2, name);
        GUI.color = Color.white;
        bool flag5 = !string.IsNullOrEmpty(tooltip);
        if (flag5)
        {
          TooltipHandler.TipRegion(rect2, tooltip);
        }
        Text.WordWrap = true;
        bool flag6 = image != null;
        if (flag6)
        {
          bool flag7 = imageSize == default(Vector2);
          Rect position;
          if (flag7)
          {
            position = new Rect(this.curX, this.curY + this.DefSelectionLineHeight, 64f, 90f);
          }
          else
          {
            position = new Rect(this.curX, this.curY + this.DefSelectionLineHeight, imageSize.x, imageSize.y);
          }
          GUI.color = Color.white;
          GUI.DrawTexture(position, image);
        }
        Text.Anchor = TextAnchor.UpperLeft;
        bool flag8 = image != null;
        if (flag8)
        {
          bool flag9 = imageSize == default(Vector2);
          if (flag9)
          {
            this.curX += 100f;
          }
          else
          {
            this.curX += imageSize.x;
          }
        }
        if (withRemove)
        {
          Rect butRect = new Rect(rect.x + rect.width - this.DefSelectionLineHeight, rect.y, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          bool flag10 = Widgets.ButtonImage(butRect, this.texRemove, true);
          if (flag10)
          {
            result = 2;
          }
          else
          {
            bool flag11 = Widgets.ButtonInvisible(rect, false);
            if (flag11)
            {
              result = 1;
            }
            else
            {
              result = 0;
            }
          }
        }
        else
        {
          bool flag12 = Widgets.ButtonInvisible(rect, false);
          if (flag12)
          {
            result = 1;
          }
          else
          {
            result = 0;
          }
        }
      }
      return result;
    }

    // Token: 0x06000590 RID: 1424 RVA: 0x0003A1B4 File Offset: 0x000383B4
    public int Selectable(string name, bool selected, string tooltip = "", RenderTexture image = null, ThingDef thingDef = null, HairDef hairDef = null, Vector2 imageSize = default(Vector2), bool withRemove = false, float selectHeight = 22f, Color backColor = default(Color), Color selectedColor = default(Color), bool autoincrement = true)
    {
      bool flag = name == null;
      int result;
      if (flag)
      {
        result = 0;
      }
      else
      {
        Text.Font = GameFont.Small;
        float num = this.listingRect.width - this.DefSelectionLineHeight;
        Text.Anchor = TextAnchor.MiddleLeft;
        Rect rect = new Rect(this.curX, this.curY, this.listingRect.width, selectHeight);
        if (selected)
        {
          GUI.DrawTexture(rect, TexUI.TextBGBlack);
          GUI.color = selectedColor;
          Widgets.DrawHighlight(rect);
        }
        GUI.color = Color.white;
        bool flag2 = thingDef != null;
        if (flag2)
        {
          Rect rect2 = new Rect(this.curX, this.curY, this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          Widgets.ThingIcon(rect2, thingDef, null, null, 1f, null);
        }
        Rect rect3;
        if (thingDef != null)
        {
          rect3 = new Rect(this.curX + 25f, this.curY, num, this.DefSelectionLineHeight);
        }
        else
        {
          if (selectHeight == 22f)
          {
            rect3 = new Rect(this.curX, this.curY, num, this.DefSelectionLineHeight);
          }
          else
          {
            rect3 = new Rect(this.curX, this.curY, num + this.DefSelectionLineHeight, (name.Length > 10) ? (this.DefSelectionLineHeight + 12f) : this.DefSelectionLineHeight);
          }
        }
        if (backColor != default(Color))
        {
          GUI.color = backColor;
        }
        Widgets.Label(rect3, name);
        GUI.color = Color.white;
        if (!string.IsNullOrEmpty(tooltip))
        {
          TooltipHandler.TipRegion(rect3, tooltip);
        }
        if (image != null)
        {
          float num2 = (this.listingRect.width - imageSize.x) / 2f;
          bool flag8 = imageSize == default(Vector2);
          Rect position;
          if (flag8)
          {
            position = new Rect(this.curX, this.curY + this.DefSelectionLineHeight, 64f, 90f);
          }
          else
          {
            position = new Rect(this.curX + num2, this.curY + 11f, imageSize.x, imageSize.y);
          }
          GUI.color = Color.white;
          GUI.DrawTexture(position, image);
        }
        if (hairDef != null)
        {
          Rect position2 = new Rect(this.curX, this.curY + this.DefSelectionLineHeight, 64f, 64f);
          Graphic g = GraphicDatabase.Get<Graphic_Multi>(hairDef.texPath);
          Texture2D textureFromMulti = g.GetTextureFromMulti("_south");
          GUI.color = Color.white;
          GUI.DrawTexture(position2, textureFromMulti);
          Rect position3 = new Rect(this.curX + 64f, this.curY + this.DefSelectionLineHeight, 64f, 64f);
          Texture2D textureFromMulti2 = g.GetTextureFromMulti("_east");
          bool flag10 = textureFromMulti2 == null;
          if (flag10)
          {
            textureFromMulti2 = g.GetTextureFromMulti("_west");
          }
          GUI.color = Color.white;
          GUI.DrawTexture(position3, textureFromMulti2);
          Rect position4 = new Rect(this.curX + 128f, this.curY + this.DefSelectionLineHeight, 64f, 64f);
          Texture2D textureFromMulti3 = g.GetTextureFromMulti("_north");
          GUI.color = Color.white;
          GUI.DrawTexture(position4, textureFromMulti3);
        }
        Text.Anchor = TextAnchor.UpperLeft;
        if (autoincrement)
        {
          this.curY += this.DefSelectionLineHeight;
          if (image != null)
          {
            if (imageSize == default(Vector2))
            {
              this.curY += 90f;
            }
            else
            {
              this.curY += imageSize.y - this.DefSelectionLineHeight;
            }
          }
          else
          {
            if (hairDef != null)
            {
              this.curY += 70f;
            }
          }
        }
        if (withRemove)
        {
          Rect butRect = new Rect(rect.x + rect.width - this.DefSelectionLineHeight, rect.y + (float)((name.Length > 20) ? 6 : 0), this.DefSelectionLineHeight, this.DefSelectionLineHeight);
          bool flag14 = Widgets.ButtonImage(butRect, this.texRemove, true);
          if (flag14)
          {
            result = 2;
          }
          else
          {
            bool flag15 = Widgets.ButtonInvisible(rect, false);
            if (flag15)
            {
              result = 1;
            }
            else
            {
              result = 0;
            }
          }
        }
        else
        {
          bool flag16 = Widgets.ButtonInvisible(rect, false);
          if (flag16)
          {
            result = 1;
          }
          else
          {
            result = 0;
          }
        }
      }
      return result;
    }

    // Token: 0x06000591 RID: 1425 RVA: 0x0003A65C File Offset: 0x0003885C
    public bool TableLine(Texture2D col0Icon, string col1Val, string col2Val, string col3Val, Texture2D col4Tex, string tooltip)
    {
      Text.Font = GameFont.Small;
      float num = this.listingRect.width - this.DefSelectionLineHeight;
      float num2 = (float)((int)(num / 3f));
      float defSelectionLineHeight = this.DefSelectionLineHeight;
      Rect position = new Rect(this.curX, this.curY, defSelectionLineHeight, defSelectionLineHeight);
      Rect rect = new Rect(this.curX + defSelectionLineHeight, this.curY, num2, defSelectionLineHeight);
      Rect rect2 = new Rect(this.curX + num2 + defSelectionLineHeight, this.curY, num2, defSelectionLineHeight);
      Rect rect3 = new Rect(this.curX + num2 * 2f + defSelectionLineHeight, this.curY, num2, defSelectionLineHeight);
      Rect butRect = new Rect(this.curX + num2 * 3f + defSelectionLineHeight, this.curY, defSelectionLineHeight, defSelectionLineHeight);
      GUI.DrawTexture(position, col0Icon);
      Widgets.Label(rect, col1Val);
      Widgets.Label(rect2, col2Val);
      Widgets.Label(rect3, col3Val);
      bool result = Widgets.ButtonImage(butRect, col4Tex, true);
      bool flag = !string.IsNullOrEmpty(tooltip);
      if (flag)
      {
        TipSignal tip = new TipSignal(() => tooltip, 2347778);
        TooltipHandler.TipRegion(rect2, tip);
      }
      this.curY += this.DefSelectionLineHeight;
      return result;
    }

    // Token: 0x06000592 RID: 1426 RVA: 0x0003A7AC File Offset: 0x000389AC
    public bool SelectableThought(string name, Texture2D icon, Color iconColor, float valopin = -3.4028235E+38f, float valmood = -3.4028235E+38f, string tooltip = "", string pName = "", bool withRemove = true)
    {
      Text.Font = GameFont.Small;
      float num = this.listingRect.width - this.DefSelectionLineHeight;
      Text.Anchor = TextAnchor.MiddleLeft;
      Rect rect = new Rect(this.curX, this.curY, num + this.DefSelectionLineHeight, this.DefSelectionLineHeight);
      GUI.color = iconColor;
      bool flag = icon != null;
      if (flag)
      {
        GUI.DrawTexture(new Rect(rect.x, rect.y, 24f, 24f), icon);
      }
      Text.WordWrap = false;
      Rect rect2 = new Rect(this.curX + 30f, this.curY, num, this.DefSelectionLineHeight);
      Widgets.DrawBoxSolid(rect2, this.alternate ? new Color(0.15f, 0.15f, 0.15f) : new Color(0.1f, 0.1f, 0.1f));
      this.alternate = !this.alternate;
      Widgets.Label(rect2, name);
      bool flag2 = tooltip != null;
      if (flag2)
      {
        TipSignal tip = new TipSignal(() => tooltip, 275);
        TooltipHandler.TipRegion(rect2, tip);
      }
      bool flag3 = !string.IsNullOrEmpty(pName);
      if (flag3)
      {
        Rect rect3 = new Rect(rect2.x + rect2.width - 190f, rect2.y - 1f, 120f, 24f);
        Widgets.Label(rect3, pName);
      }
      bool flag4 = valopin != float.MinValue;
      float num2 = flag4 ? valopin : valmood;
      int num3 = (num2 < 0f) ? 4 : 0;
      Rect rect4 = new Rect(rect2.x + rect2.width - 80f - (float)num3, rect2.y, 48f, 24f);
      bool flag5 = num2 == 0f;
      if (flag5)
      {
        GUI.color = (flag4 ? ColorTool.colBeige : this.NoEffectColor);
      }
      else
      {
        bool flag6 = num2 > 0f;
        if (flag6)
        {
          GUI.color = (flag4 ? ColorTool.colLightBlue : this.MoodColor);
        }
        else
        {
          GUI.color = (flag4 ? ColorTool.colPink : this.MoodColorNegative);
        }
      }
      Widgets.Label(rect4, num2.ToString("##0"));
      bool result = false;
      if (withRemove)
      {
        Rect butRect = new Rect(rect2.x + rect2.width - 42f, rect2.y, 24f, 24f);
        result = Widgets.ButtonImage(butRect, ContentFinder<Texture2D>.Get("UI/Buttons/Delete", true), true);
      }
      Text.WordWrap = true;
      Text.Anchor = TextAnchor.UpperLeft;
      this.curY += this.DefSelectionLineHeight;
      return result;
    }

    // Token: 0x06000593 RID: 1427 RVA: 0x0003AA88 File Offset: 0x00038C88
    public float Slider(float val, float min, float max, Color color = default(Color))
    {
      if (color != default(Color))
      {
        GUI.color = color;
      }
      float result = Widgets.HorizontalSlider(base.GetRect(22f), val, min, max, false, null, null, null, -1f);
      base.Gap(this.verticalSpacing);
      GUI.color = Color.white; // Added this
      return result;
    }

    // Token: 0x06000594 RID: 1428 RVA: 0x0003AAD0 File Offset: 0x00038CD0
    //public float Slider(float val, float min, float max)
    //{
    //  Rect rect = base.GetRect(22f);
    //  float result = Widgets.HorizontalSlider(rect, val, min, max, false, null, null, null, -1f);
    //  base.Gap(this.verticalSpacing);
    //  return result;
    //}

    // Token: 0x06000595 RID: 1429 RVA: 0x0003AB10 File Offset: 0x00038D10
    public float Slider(float val, float min, float max, float width)
    {
      Rect rect = base.GetRect(22f);
      rect.width = width;
      return Widgets.HorizontalSlider(rect, val, min, max, false, null, null, null, -1f);
    }

    // Token: 0x06000596 RID: 1430 RVA: 0x0003AB4C File Offset: 0x00038D4C
    public string TextEntry(string text, int lineCount = 1)
    {
      Rect rect = base.GetRect(Text.LineHeight * (float)lineCount);
      bool flag = lineCount == 1;
      string result;
      if (flag)
      {
        result = Widgets.TextField(rect, text);
      }
      else
      {
        result = Widgets.TextArea(rect, text, false);
      }
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x06000597 RID: 1431 RVA: 0x0003AB9C File Offset: 0x00038D9C
    public string TextEntryLabeledWithDefaultAndCopy(string label, string text, string defaultVal)
    {
      Rect rect = base.GetRect(Text.LineHeight * 1f);
      rect.width -= 50f;
      Rect rect2 = rect.LeftHalf().Rounded();
      Rect rect3 = rect.RightHalf().Rounded();
      Widgets.Label(rect2, label);
      string result = Widgets.TextField(rect3, text);
      TooltipHandler.TipRegion(rect2, text);
      Rect rect4 = new Rect(rect.x + rect.width, rect.y, 24f, 24f);
      bool flag = Widgets.ButtonImage(rect4, ContentFinder<Texture2D>.Get("bdefault", true), true);
      if (flag)
      {
        result = defaultVal;
      }
      TooltipHandler.TipRegion(rect4, CharacterEditor.Label.O_SETTODEFAULT);
      Rect rect5 = new Rect(rect.x + rect.width + 25f, rect.y, 24f, 24f);
      bool flag2 = Widgets.ButtonImage(rect5, ContentFinder<Texture2D>.Get("UI/Buttons/Copy", true), true);
      if (flag2)
      {
        Clipboard.CopyToClip(text);
      }
      TooltipHandler.TipRegion(rect5, CharacterEditor.Label.O_COPYTOCLIPBOARD);
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x06000598 RID: 1432 RVA: 0x0003ACD4 File Offset: 0x00038ED4
    public string TextEntryLabeled(string label, string text, int lineCount = 1)
    {
      Rect rect = base.GetRect(Text.LineHeight * (float)lineCount);
      string result = Widgets.TextEntryLabeled(rect, label, text);
      base.Gap(this.verticalSpacing);
      return result;
    }

    // Token: 0x06000599 RID: 1433 RVA: 0x0003AD0C File Offset: 0x00038F0C
    public void TextFieldNumeric<T>(float xStart, float width, ref T val, ref string buffer, float min = 0f, float max = 1E+09f) where T : struct
    {
      Rect rect = base.GetRect(Text.LineHeight);
      rect.x = xStart;
      rect.width = width;
      Widgets.TextFieldNumeric<T>(rect, ref val, ref buffer, min, max);
      base.Gap(this.verticalSpacing);
    }

    // Token: 0x0600059A RID: 1434 RVA: 0x0003AD54 File Offset: 0x00038F54
    public void TextFieldNumericLabeled<T>(string label, float xStart, float width, ref T val, ref string buffer, float min = 0f, float max = 1E+09f) where T : struct
    {
      Rect rect = base.GetRect(Text.LineHeight);
      rect.x = xStart;
      rect.width = width;
      Widgets.TextFieldNumericLabeled<T>(rect, label, ref val, ref buffer, min, max);
      base.Gap(this.verticalSpacing);
    }

    // Token: 0x0600059B RID: 1435 RVA: 0x0003ADA0 File Offset: 0x00038FA0
    private Vector2 GetLabelScrollbarPosition(float x, float y)
    {
      bool flag = this.labelScrollbarPositions == null;
      Vector2 zero;
      if (flag)
      {
        zero = Vector2.zero;
      }
      else
      {
        for (int i = 0; i < this.labelScrollbarPositions.Count; i++)
        {
          Vector2 first = this.labelScrollbarPositions[i].First;
          bool flag2 = first.x == x && first.y == y;
          if (flag2)
          {
            return this.labelScrollbarPositions[i].Second;
          }
        }
        zero = Vector2.zero;
      }
      return zero;
    }

    // Token: 0x0600059C RID: 1436 RVA: 0x0003AE38 File Offset: 0x00039038
    private void SetLabelScrollbarPosition(float x, float y, Vector2 scrollbarPosition)
    {
      bool flag = this.labelScrollbarPositions == null;
      if (flag)
      {
        this.labelScrollbarPositions = new List<Pair<Vector2, Vector2>>();
        this.labelScrollbarPositionsSetThisFrame = new List<Vector2>();
      }
      this.labelScrollbarPositionsSetThisFrame.Add(new Vector2(x, y));
      for (int i = 0; i < this.labelScrollbarPositions.Count; i++)
      {
        Vector2 first = this.labelScrollbarPositions[i].First;
        bool flag2 = first.x == x && first.y == y;
        if (flag2)
        {
          this.labelScrollbarPositions[i] = new Pair<Vector2, Vector2>(new Vector2(x, y), scrollbarPosition);
          return;
        }
      }
      this.labelScrollbarPositions.Add(new Pair<Vector2, Vector2>(new Vector2(x, y), scrollbarPosition));
    }


  }
}
