﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000033 RID: 51
	public static class MindTool
	{
		// Token: 0x06000268 RID: 616 RVA: 0x00011A70 File Offset: 0x0000FC70
		public static string GetInspirationTooltip(this Pawn p)
		{
			return (p == null || p.Inspiration == null || p.Inspiration.def.beginLetter.NullOrEmpty()) ? "" : p.Inspiration.def.beginLetter.Formatted(p.NameShortColored, p.Named("PAWN")).AdjustedFor(p, "PAWN", true).ToString();
		}

		// Token: 0x06000269 RID: 617 RVA: 0x00011AF0 File Offset: 0x0000FCF0
		public static string GetMentalStateTooltip(this Pawn p)
		{
			return (p == null || p.MentalState == null) ? "" : p.MentalState.GetBeginLetterText().ToString();
		}

		// Token: 0x0600026A RID: 618 RVA: 0x00011B28 File Offset: 0x0000FD28
		public static Dictionary<EType, HashSet<ThoughtDef>> GetAllThoughtLists()
		{
			HashSet<ThoughtDef> hashSet = new HashSet<ThoughtDef>();
			HashSet<ThoughtDef> hashSet2 = new HashSet<ThoughtDef>();
			HashSet<ThoughtDef> hashSet3 = new HashSet<ThoughtDef>();
			HashSet<ThoughtDef> hashSet4 = new HashSet<ThoughtDef>();
			HashSet<ThoughtDef> hashSet5 = new HashSet<ThoughtDef>();
			HashSet<ThoughtDef> hashSet6 = new HashSet<ThoughtDef>();
			HashSet<ThoughtDef> hashSet7 = new HashSet<ThoughtDef>();
			foreach (ThoughtDef thoughtDef in DefTool.ListAll<ThoughtDef>())
			{
				try
				{
					bool flag = !thoughtDef.IsNullOrEmpty() && !hashSet7.Contains(thoughtDef);
					if (flag)
					{
						bool flag2 = !thoughtDef.GetThoughtLabel(0, null).NullOrEmpty();
						if (flag2)
						{
							hashSet7.Add(thoughtDef);
							bool flag3 = thoughtDef.IsTypeOf<Thought_Memory>();
							if (flag3)
							{
								bool flag4 = thoughtDef.IsTrulySocial();
								if (flag4)
								{
									hashSet4.Add(thoughtDef);
								}
								else
								{
									hashSet.Add(thoughtDef);
								}
							}
							else
							{
								bool flag5 = thoughtDef.IsTypeOf<Thought_Situational>();
								if (flag5)
								{
									bool flag6 = thoughtDef.IsTrulySocial();
									if (flag6)
									{
										hashSet5.Add(thoughtDef);
									}
									else
									{
										hashSet2.Add(thoughtDef);
									}
								}
								else
								{
									hashSet6.Add(thoughtDef);
								}
							}
						}
					}
				}
				catch
				{
				}
			}
			bool devMode = Prefs.DevMode;
			if (devMode)
			{
				MessageTool.Show(string.Concat(new string[]
				{
					"total=",
					hashSet7.Count.ToString(),
					", mem=",
					hashSet.Count.ToString(),
					", memsoz=",
					hashSet4.Count.ToString(),
					", situ=",
					hashSet2.Count.ToString(),
					", situsoz=",
					hashSet5.Count.ToString(),
					", not=",
					hashSet6.Count.ToString()
				}), null);
			}
			hashSet = (from t in hashSet
			orderby t.GetThoughtLabel(0, null)
			select t).ToHashSet<ThoughtDef>();
			hashSet4 = (from t in hashSet4
			orderby t.GetThoughtLabel(0, null)
			select t).ToHashSet<ThoughtDef>();
			hashSet2 = (from t in hashSet2
			orderby t.GetThoughtLabel(0, null)
			select t).ToHashSet<ThoughtDef>();
			hashSet5 = (from t in hashSet5
			orderby t.GetThoughtLabel(0, null)
			select t).ToHashSet<ThoughtDef>();
			hashSet6 = (from t in hashSet6
			orderby t.GetThoughtLabel(0, null)
			select t).ToHashSet<ThoughtDef>();
			hashSet7 = (from t in hashSet7
			orderby t.GetThoughtLabel(0, null)
			select t).ToHashSet<ThoughtDef>();
			return new Dictionary<EType, HashSet<ThoughtDef>>
			{
				{
					EType.ThoughtMemory,
					hashSet
				},
				{
					EType.ThoughtMemorySocial,
					hashSet4
				},
				{
					EType.ThoughtSituational,
					hashSet2
				},
				{
					EType.ThoughtSituationalSocial,
					hashSet5
				},
				{
					EType.ThoughtUnsupported,
					hashSet6
				},
				{
					EType.ThoughtsAll,
					hashSet7
				}
			};
		}

		// Token: 0x0600026B RID: 619 RVA: 0x00011E98 File Offset: 0x00010098
		public static List<MentalStateDef> GetAllMentalStates()
		{
			List<MentalStateDef> list = (from td in DefDatabase<MentalStateDef>.AllDefs
			where td != null && !string.IsNullOrEmpty(td.label)
			orderby td.label
			select td).ToList<MentalStateDef>();
			list.Insert(0, null);
			return list;
		}

		// Token: 0x0600026C RID: 620 RVA: 0x00011F08 File Offset: 0x00010108
		public static List<InspirationDef> GetAllInspirations()
		{
			List<InspirationDef> list = (from td in DefDatabase<InspirationDef>.AllDefs
			where td != null && !string.IsNullOrEmpty(td.label)
			orderby td.label
			select td).ToList<InspirationDef>();
			list.Insert(0, null);
			return list;
		}

		// Token: 0x0600026D RID: 621 RVA: 0x00011F78 File Offset: 0x00010178
		public static string GetAllNeedsAsSeparatedString(this Pawn p)
		{
			bool flag = !p.HasNeedsTracker() && !p.needs.AllNeeds.NullOrEmpty<Need>();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				foreach (Need need in p.needs.AllNeeds)
				{
					text = text + need.def.defName + "|";
					text += need.CurLevelPercentage.ToString();
					text += ":";
				}
				text = text.SubstringRemoveLast();
				result = text;
			}
			return result;
		}

		// Token: 0x0600026E RID: 622 RVA: 0x0001204C File Offset: 0x0001024C
		public static Need GetNeedForThis(this Pawn p, NeedDef n)
		{
			return p.HasNeedsTracker() ? p.needs.TryGetNeed(n) : null;
		}

		// Token: 0x0600026F RID: 623 RVA: 0x00012068 File Offset: 0x00010268
		public static void SetNeeds(this Pawn p, string s)
		{
			bool flag = s.NullOrEmpty() || !p.HasNeedsTracker();
			if (!flag)
			{
				string[] array = s.SplitNo(":");
				foreach (string s2 in array)
				{
					string[] array3 = s2.SplitNo("|");
					bool flag2 = array3.Length == 2;
					if (flag2)
					{
						NeedDef needDef = DefTool.NeedDef(array3[0]);
						bool flag3 = needDef != null;
						if (flag3)
						{
							Need needForThis = p.GetNeedForThis(needDef);
							bool flag4 = needForThis != null;
							if (flag4)
							{
								needForThis.CurLevelPercentage = array3[1].AsFloat();
							}
						}
					}
				}
			}
		}
	}
}
