﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
  // Token: 0x02000030 RID: 48
  public static class PawnKindTool
  {
    // Token: 0x0600023E RID: 574 RVA: 0x00010AB0 File Offset: 0x0000ECB0
    public static string GetPawnKindDefname(this Pawn p)
    {
      return (p != null && p.kindDef != null) ? p.kindDef.defName : "";
    }

    // Token: 0x0600023F RID: 575 RVA: 0x00010ACF File Offset: 0x0000ECCF
    public static bool IsFromMod(this PawnKindDef pkd, string modname)
    {
      return modname.NullOrEmpty() || (pkd.modContentPack != null && pkd.modContentPack.Name == modname);
    }

    // Token: 0x06000240 RID: 576 RVA: 0x00010AF8 File Offset: 0x0000ECF8
    public static bool IsTribalOrBoss(this PawnKindDef pkd)
    {
      return pkd != null && pkd.defName != null && (pkd.defName.Contains("Tribal_") || pkd.defName.Contains("Empire_Fighter_Champion") || pkd.defName.Contains("ZhthyhlMid"));
    }

    // Token: 0x06000241 RID: 577 RVA: 0x00010B4A File Offset: 0x0000ED4A
    public static bool IsDroidColonist(this PawnKindDef pkd)
    {
      return pkd != null && pkd.defName != null && pkd.defName.EndsWith("DroidColonist");
    }

    // Token: 0x06000242 RID: 578 RVA: 0x00010B6A File Offset: 0x0000ED6A
    public static bool IsDorfbewohner(this PawnKindDef pkd)
    {
      return pkd != null && pkd.defName != null && pkd.defName.StartsWith("Villager");
    }

    // Token: 0x06000243 RID: 579 RVA: 0x00010B8C File Offset: 0x0000ED8C
    public static bool IsXeno(this PawnKindDef pkd)
    {
      return pkd.race != null && pkd.modContentPack != null && pkd.modContentPack.Name == "Alien Vs Predator" && !pkd.RaceProps.Humanlike && pkd.race.defName != null && !pkd.race.defName.ToLower().Contains("yautja");
    }

    // Token: 0x06000244 RID: 580 RVA: 0x00010BFA File Offset: 0x0000EDFA
    public static bool IsAbomination(this PawnKindDef pkd)
    {
      return pkd != null && pkd.defName != null && pkd.defName.StartsWith("Abomination");
    }

    // Token: 0x06000245 RID: 581 RVA: 0x00010C1A File Offset: 0x0000EE1A
    public static bool IsInsektoid(this PawnKindDef pkd)
    {
      return pkd.RaceProps.FleshType == FleshTypeDefOf.Insectoid;
    }

    // Token: 0x06000246 RID: 582 RVA: 0x00010C2E File Offset: 0x0000EE2E
    public static bool IsMechanoid(this PawnKindDef pkd)
    {
      return pkd.RaceProps.FleshType == FleshTypeDefOf.Mechanoid;
    }

    // Token: 0x06000247 RID: 583 RVA: 0x00010C42 File Offset: 0x0000EE42
    public static bool IsZombie(this PawnKindDef pkd)
    {
      return pkd.defName == "Zombie";
    }

    // Token: 0x06000248 RID: 584 RVA: 0x00010C54 File Offset: 0x0000EE54
    public static bool IsAnimal(this PawnKindDef pkd)
    {
      return pkd != null && pkd.RaceProps != null && pkd.RaceProps.Animal;
    }

    // Token: 0x06000249 RID: 585 RVA: 0x00010C70 File Offset: 0x0000EE70
    public static PawnKindDef ThisOrFromList(this PawnKindDef pkd, List<PawnKindDef> lpkd)
    {
      bool flag = pkd == null;
      PawnKindDef result;
      if (flag)
      {
        List<PawnKindDef> list = new List<PawnKindDef>();
        list.AddRange(lpkd);
        list.Remove(null);
        result = list.RandomElementWithFallback(null);
      }
      else
      {
        result = pkd;
      }
      return result;
    }

    // Token: 0x0600024A RID: 586 RVA: 0x00010CAC File Offset: 0x0000EEAC
    public static PawnKindDef ThisOrRandom(this PawnKindDef pkd, string modname)
    {
      return pkd ?? PawnKindTool.GetRandomPawnKindDef(modname);
    }

    // Token: 0x0600024B RID: 587 RVA: 0x00010CCC File Offset: 0x0000EECC
    public static PawnKindDef GetRandomPawnKindDef(string modname)
    {
      List<PawnKindDef> source = DefTool.ListByMod<PawnKindDef>(modname);
      return source.RandomElement<PawnKindDef>();
    }

    // Token: 0x0600024C RID: 588 RVA: 0x00010CEC File Offset: 0x0000EEEC
    public static List<PawnKindDef> ThisOrDefault(this List<PawnKindDef> l)
    {
      List<PawnKindDef> result;
      if (!l.NullOrEmpty<PawnKindDef>() && (l.Count != 1 || l.First<PawnKindDef>() != null))
      {
        result = l;
      }
      else
      {
        result = (from pkd in CEditor.API.ListOf<PawnKindDef>(EType.PawnKindListed)
                  where pkd != null
                  select pkd).ToList<PawnKindDef>();
      }
      return result;
    }

    // Token: 0x0600024D RID: 589 RVA: 0x00010D4C File Offset: 0x0000EF4C
    public static List<PawnKindDef> GetHumanlikes()
    {
      return (from td in DefDatabase<PawnKindDef>.AllDefs
              where td.race != null && td.race.label != null && td.RaceProps.Humanlike
              orderby td.race.label
              select td).ToList<PawnKindDef>();
    }

    // Token: 0x0600024E RID: 590 RVA: 0x00010DB0 File Offset: 0x0000EFB0
    public static List<PawnKindDef> GetAnimals()
    {
      return (from td in DefDatabase<PawnKindDef>.AllDefs
              where td.race != null && td.race.label != null && td.RaceProps.Animal
              orderby td.race.label
              select td).ToList<PawnKindDef>();
    }

    // Token: 0x0600024F RID: 591 RVA: 0x00010E14 File Offset: 0x0000F014
    public static List<PawnKindDef> GetOther()
    {
      return (from td in DefDatabase<PawnKindDef>.AllDefs
              where td.race != null && td.race.label != null && !td.RaceProps.Animal && !td.RaceProps.Humanlike
              orderby td.race.label
              select td).ToList<PawnKindDef>();
    }

    // Token: 0x06000250 RID: 592 RVA: 0x00010E78 File Offset: 0x0000F078
    public static List<PawnKindDef> GetPawnKindListxx(Faction f, string key = null)
    {
      key = (key ?? CEditor.ListName);
      bool flag;
      bool flag2;
      bool flag3;
      bool flag4;
      bool flag5;
      bool flag6;
      PawnxTool.SetPawnKindFlags(key, f, out flag, out flag2, out flag3, out flag4, out flag5, out flag6);
      List<PawnKindDef> list = flag2 ? CEditor.API.ListOf<PawnKindDef>(EType.PawnKindHuman) : null;
      List<PawnKindDef> list2 = (flag3 || f == Faction.OfInsects) ? CEditor.API.ListOf<PawnKindDef>(EType.PawnKindAnimal) : null;
      List<PawnKindDef> list3 = flag4 ? CEditor.API.ListOf<PawnKindDef>(EType.PawnKindOther) : null;
      List<PawnKindDef> list4 = new List<PawnKindDef>();
      bool flag7 = !list.NullOrEmpty<PawnKindDef>();
      if (flag7)
      {
        list4.AddRange(list);
      }
      bool flag8 = !list2.NullOrEmpty<PawnKindDef>();
      if (flag8)
      {
        list4.AddRange(list2);
      }
      bool flag9 = !list3.NullOrEmpty<PawnKindDef>();
      if (flag9)
      {
        list4.AddRange(list3);
      }
      bool flag10 = f.IsInsektoid();
      if (flag10)
      {
        list4 = (from td in list4
                 where td.RaceProps.FleshType == FleshTypeDefOf.Insectoid
                 select td).ToList<PawnKindDef>();
      }
      else
      {
        bool flag11 = f.IsMechanoid();
        if (flag11)
        {
          list4 = (from td in list4
                   where td.RaceProps.FleshType == FleshTypeDefOf.Mechanoid
                   select td).ToList<PawnKindDef>();
        }
        else
        {
          list4 = (from td in list4
                   where td.defName != "Zombie"
                   orderby td.label
                   select td).ToList<PawnKindDef>();
        }
      }
      bool flag12 = flag5;
      if (flag12)
      {
        list4 = (from td in list4
                 where td.race != null && td.IsFromMod("Alien Vs Predator") && !td.RaceProps.Humanlike && td.race.defName != null && !td.race.defName.ToLower().Contains("yautja")
                 select td).ToList<PawnKindDef>();
      }
      return list4;
    }

    // Token: 0x06000251 RID: 593 RVA: 0x00011048 File Offset: 0x0000F248
    public static HashSet<ThingDef> ListOfRaces(bool humanlike, bool nonhumanlike)
    {
      HashSet<ThingDef> hashSet = new HashSet<ThingDef>();
      HashSet<ThingDef> hashSet2 = new HashSet<ThingDef>();
      IEnumerable<PawnKindDef> allDefs = DefDatabase<PawnKindDef>.AllDefs;
      foreach (PawnKindDef pawnKindDef in allDefs)
      {
        bool flag = pawnKindDef == null || pawnKindDef.race == null || pawnKindDef.race.label == null;
        if (!flag)
        {
          bool humanlike2 = pawnKindDef.RaceProps.Humanlike;
          if (humanlike2)
          {
            bool flag2 = !hashSet.Contains(pawnKindDef.race);
            if (flag2)
            {
              hashSet.Add(pawnKindDef.race);
            }
          }
          else
          {
            bool flag3 = !hashSet2.Contains(pawnKindDef.race);
            if (flag3)
            {
              hashSet2.Add(pawnKindDef.race);
            }
          }
        }
      }
      bool flag4 = (humanlike && nonhumanlike) || (!humanlike && !nonhumanlike);
      HashSet<ThingDef> hashSet3 = new HashSet<ThingDef>();
      bool flag5 = flag4 || humanlike;
      if (flag5)
      {
        hashSet3.AddRange(hashSet);
      }
      bool flag6 = flag4 || nonhumanlike;
      if (flag6)
      {
        hashSet3.AddRange(hashSet2);
      }
      return hashSet3;
    }

    // Token: 0x06000252 RID: 594 RVA: 0x0001117C File Offset: 0x0000F37C
    public static List<PawnKindDef> ListOfPawnKindDefByRace(ThingDef raceDef, bool humanlike, bool nonhumanlike)
    {
      IEnumerable<PawnKindDef> allDefs = DefDatabase<PawnKindDef>.AllDefs;
      bool all = (humanlike && nonhumanlike) || (!humanlike && !nonhumanlike);
      bool flag = raceDef == null;
      List<PawnKindDef> result;
      if (flag)
      {
        result = (from td in allDefs
                  where td != null && td.race != null && (all || td.RaceProps.Humanlike == humanlike)
                  orderby td.label
                  select td).ToList<PawnKindDef>();
      }
      else
      {
        result = (from td in allDefs
                  where td != null && td.race != null && td.race == raceDef && (all || td.RaceProps.Humanlike == humanlike)
                  orderby td.label
                  select td).ToList<PawnKindDef>();
      }
      return result;
    }

    // Token: 0x06000253 RID: 595 RVA: 0x00011250 File Offset: 0x0000F450
    public static List<PawnKindDef> ListOfPawnKindDef(Faction f, string key, string modname)
    {
      key = (key ?? CEditor.ListName);
      bool flag;
      bool flag2;
      bool flag3;
      bool flag4;
      bool flag5;
      bool flag6;
      PawnxTool.SetPawnKindFlags(key, f, out flag, out flag2, out flag3, out flag4, out flag5, out flag6);
      List<PawnKindDef> list = flag2 ? CEditor.API.ListOf<PawnKindDef>(EType.PawnKindHuman) : null;
      List<PawnKindDef> list2 = (flag3 || f == Faction.OfInsects) ? CEditor.API.ListOf<PawnKindDef>(EType.PawnKindAnimal) : null;
      List<PawnKindDef> list3 = flag4 ? CEditor.API.ListOf<PawnKindDef>(EType.PawnKindOther) : null;
      List<PawnKindDef> list4 = new List<PawnKindDef>();
      bool flag7 = !list.NullOrEmpty<PawnKindDef>();
      if (flag7)
      {
        list4.AddRange(list);
      }
      bool flag8 = !list2.NullOrEmpty<PawnKindDef>();
      if (flag8)
      {
        list4.AddRange(list2);
      }
      bool flag9 = !list3.NullOrEmpty<PawnKindDef>();
      if (flag9)
      {
        list4.AddRange(list3);
      }
      bool flag10 = f.IsInsektoid();
      if (flag10)
      {
        list4 = (from td in list4
                 where td.IsFromMod(modname) && td.IsInsektoid()
                 select td).ToList<PawnKindDef>();
      }
      else
      {
        bool flag11 = f.IsMechanoid();
        if (flag11)
        {
          list4 = (from td in list4
                   where td.IsFromMod(modname) && td.IsMechanoid()
                   select td).ToList<PawnKindDef>();
        }
        else
        {
          bool flag12 = flag5;
          if (flag12)
          {
            list4 = (from td in list4
                     where td.IsFromMod(modname) && td.IsXeno()
                     select td).ToList<PawnKindDef>();
          }
          else
          {
            list4 = (from td in list4
                     where td.IsFromMod(modname) && !td.IsZombie()
                     orderby td.label
                     select td).ToList<PawnKindDef>();
          }
        }
      }
      return list4;
    }
  }
}
