﻿using System;
using System.Collections.Generic;
using System.Linq;
using HarmonyLib;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI.Group;

namespace CharacterEditor
{
  // Token: 0x0200003D RID: 61
  internal static class PawnxTool
  {
    // Token: 0x06000330 RID: 816 RVA: 0x0001A0C8 File Offset: 0x000182C8
    internal static bool IsAnimal(this Pawn p)
    {
      return p != null && p.kindDef.IsAnimal();
    }

    // Token: 0x06000331 RID: 817 RVA: 0x0001A0DC File Offset: 0x000182DC
    internal static Pawn GetOtherPawnFromSeparatedString(string s)
    {
      Pawn result = null;
      bool flag = !s.NullOrEmpty();
      if (flag)
      {
        bool flag2 = s.Contains("?");
        if (flag2)
        {
          result = NameTool.GetPawnByNameTriple(NameTool.GetPawnNameFromSeparatedString(s));
        }
        else
        {
          result = NameTool.GetPawnByNameSingle(NameTool.GetPawnNameFromSeparatedString(s, null));
        }
      }
      return result;
    }

    // Token: 0x06000332 RID: 818 RVA: 0x0001A12C File Offset: 0x0001832C
    internal static void CreatePawnLord(this Pawn newPawn, IntVec3 forcePosition = default(IntVec3))
    {
      bool flag = newPawn == null;
      if (!flag)
      {
        Faction faction = newPawn.Faction;
        bool flag2 = faction != null && faction != Faction.OfPlayer;
        if (flag2)
        {
          Lord lord = null;
          try
          {
            bool flag3 = newPawn.Map.mapPawns.SpawnedPawnsInFaction(faction).Any((Pawn p) => p != newPawn);
            if (flag3)
            {
              Pawn p2 = (Pawn)GenClosest.ClosestThing_Global(newPawn.Position, newPawn.Map.mapPawns.SpawnedPawnsInFaction(faction), 99999f, (Thing p) => p != newPawn && ((Pawn)p).GetLord() != null, null);
              lord = p2.GetLord();
            }
          }
          catch
          {
          }
          bool flag4 = lord == null;
          if (flag4)
          {
            LordJob_DefendPoint lordJob = new LordJob_DefendPoint((forcePosition == default(IntVec3)) ? newPawn.Position : forcePosition, null, false, true);
            lord = LordMaker.MakeNewLord(faction, lordJob, Find.CurrentMap, null);
          }
          lord.AddPawn(newPawn);
        }
      }
    }

    // Token: 0x06000333 RID: 819 RVA: 0x0001A270 File Offset: 0x00018470
    internal static void SpawnOrPassPawn(this Pawn p, Faction f, PresetPawn ppn, IntVec3 pos = default(IntVec3))
    {
      bool inStartingScreen = CEditor.InStartingScreen;
      if (inStartingScreen)
      {
        ////Log.Message("LoadPawn, call GeneratePossessions");
        AccessTools.Method(typeof(StartingPawnUtility), "GeneratePossessions").CallMethod(new object[] { p });
        if (CEditor.ListName == Label.COLONISTS)
        {
          p.PostCompsAfterCreate(ppn);
        }
        else
        {
          p.PassToWorld(ppn, pos);
        }
      }
      else
      {
        bool flag2 = !CEditor.OnMap;
        if (flag2)
        {
          p.PassToWorld(ppn, pos);
        }
        else
        {
          bool shift = Event.current.shift;
          if (shift)
          {
            CEditor.API.EditorMoveRight();
            PlacingTool.DropWithPod(p, ppn);
          }
          else
          {
            bool flag3 = (ppn != null && ppn.bPlace) || (ppn == null && Event.current.control);
            if (flag3)
            {
              CEditor.API.EditorMoveRight();
              PlacingTool.PlaceInCustomPosition(p, ppn);
            }
            else
            {
              PlacingTool.PlaceInPosition(p, ppn, pos);
            }
          }
        }
      }
    }

    // Token: 0x06000334 RID: 820 RVA: 0x0001A33C File Offset: 0x0001853C
    internal static void PostCompsAfterCreate(this Pawn p, PresetPawn ppn)
    {
      List<ThingComp> list = (List<ThingComp>)p.GetMemberValue("comps");
      bool flag = list != null;
      if (flag)
      {
        for (int i = 0; i < list.Count; i++)
        {
          list[i].PostSpawnSetup(true);
        }
      }
      PawnxTool.PostProcess(p, ppn);
    }

    // Token: 0x06000335 RID: 821 RVA: 0x0001A390 File Offset: 0x00018590
    internal static void PassToWorld(this Pawn p, PresetPawn ppn, IntVec3 pos = default(IntVec3))
    {
      bool flag = p == null;
      if (!flag)
      {
        bool flag2 = p.Faction.IsZombie();
        bool flag3 = flag2;
        if (flag3)
        {
          bool flag4 = !CEditor.InStartingScreen;
          if (flag4)
          {
            p.ZombieWorldCreationFixInSpace();
          }
          else
          {
            p.SpawnAsZombie(pos);
          }
        }
        else
        {
          bool flag5 = !Find.World.worldPawns.Contains(p);
          if (flag5)
          {
            Find.World.worldPawns.PassToWorld(p, PawnDiscardDecideMode.Decide);
          }
        }
        Dictionary<string, Faction> dictionary = CEditor.API.Get<Dictionary<string, Faction>>(EType.Factions);
        bool flag6 = dictionary != null;
        if (flag6)
        {
          Faction value = dictionary.GetValue(CEditor.ListName);
          bool flag7 = value != null;
          if (flag7)
          {
            p.SetFaction(value, null);
          }
        }
        PawnxTool.PostProcess(p, ppn);
      }
    }

    // Token: 0x06000336 RID: 822 RVA: 0x0001A44C File Offset: 0x0001864C
    internal static void SpawnPawn(this Pawn newPawn, PresetPawn ppn, IntVec3 pos = default(IntVec3))
    {
      bool flag = newPawn == null || CEditor.InStartingScreen;
      if (!flag)
      {
        try
        {
          bool flag2 = newPawn.Dead && newPawn.Corpse != null && newPawn.Corpse.Spawned;
          if (flag2)
          {
            newPawn.Corpse.DeSpawn(DestroyMode.Vanish);
          }
          bool spawned = newPawn.Spawned;
          if (spawned)
          {
            newPawn.DeSpawn(DestroyMode.Vanish);
          }
          bool flag3 = pos == default(IntVec3);
          if (flag3)
          {
            pos = UI.MouseCell();
            pos.x -= 5;
          }
          bool flag4 = !pos.InBounds(Find.CurrentMap);
          if (flag4)
          {
            pos = Find.CurrentMap.AllCells.RandomElement<IntVec3>();
          }
          bool flag5 = newPawn.Faction.IsZombie();
          bool flag6 = flag5;
          if (flag6)
          {
            newPawn.SpawnAsZombie(pos);
          }
          else
          {
            GenSpawn.Spawn(newPawn, pos, Find.CurrentMap, Rot4.South, WipeMode.Vanish, false);
            newPawn.CreatePawnLord(default(IntVec3));
          }
          bool flag7 = newPawn.Faction == Faction.OfPlayer;
          if (flag7)
          {
            newPawn.playerSettings.hostilityResponse = HostilityResponseMode.Attack;
          }
          PawnxTool.PostProcess(newPawn, ppn);
        }
        catch (Exception ex)
        {
          Log.Error(ex.Message + "\n" + ex.StackTrace);
        }
      }
    }

    // Token: 0x06000337 RID: 823 RVA: 0x0001A5B4 File Offset: 0x000187B4
    internal static void PostProcess(Pawn p, PresetPawn ppn)
    {
      bool flag = p == null;
      if (!flag)
      {
        MeditationFocusTypeAvailabilityCache.ClearFor(p);
        StatsReportUtility.Reset();
        List<Pawn> list = CEditor.API.ListOf<Pawn>(EType.Pawns);
        bool flag2 = !list.Contains(p);
        if (flag2)
        {
          list.Insert(0, p);
        }
        CEditor.API.Pawn = p;
        bool inStartingScreen = CEditor.InStartingScreen;
        if (inStartingScreen)
        {
          PawnxTool.Notify_CheckStartPawnsListChanged();
        }
        bool flag3 = ppn != null;
        if (flag3)
        {
          ppn.PostProcess();
        }
        else
        {
          Capturer capturer = CEditor.API.Get<Capturer>(EType.Capturer);
          capturer.UpdatePawnGraphic(p);
        }
      }
    }

    // Token: 0x06000338 RID: 824 RVA: 0x0001A644 File Offset: 0x00018844
    internal static void SpawnAsZombie(this Pawn pawn, IntVec3 pos)
    {
      try
      {
        bool flag = pawn.story != null;
        if (flag)
        {
          pawn.story.traits.allTraits.Clear();
        }
        pawn.inventory.DestroyAll(DestroyMode.Vanish);
        pawn.equipment.DestroyAllEquipment(DestroyMode.Vanish);
        GenSpawn.Spawn(pawn, pos, Find.CurrentMap, Rot4.South, WipeMode.Vanish, false);
        object[] param = new object[]
        {
          pawn,
          false
        };
        Reflect.GetAType("ZombieLand", "Tools").CallMethod("ConvertToZombie", param);
        bool dead = pawn.Dead;
        if (dead)
        {
          pawn.Delete(false);
        }
      }
      catch
      {
      }
    }

    // Token: 0x06000339 RID: 825 RVA: 0x0001A700 File Offset: 0x00018900
    internal static void DiscardGeneratedPawn(this Pawn p)
    {
      bool flag = p == null;
      if (!flag)
      {
        try
        {
          typeof(PawnGenerator).CallMethod("DiscardGeneratedPawn", new object[]
          {
            p
          });
        }
        catch
        {
        }
      }
    }

    // Token: 0x0600033A RID: 826 RVA: 0x0001A754 File Offset: 0x00018954
    internal static void Delete(this Pawn p, bool force = false)
    {
      bool flag = p == null;
      if (!flag)
      {
        try
        {
          List<Pawn> list = CEditor.API.ListOf<Pawn>(EType.Pawns);
          bool flag2 = !list.NullOrEmpty<Pawn>() && list.Contains(p);
          if (flag2)
          {
            bool flag3 = !force && CEditor.InStartingScreen && list.Count == 1;
            if (flag3)
            {
              MessageTool.Show("can't delete below the minimum number", MessageTypeDefOf.RejectInput);
              return;
            }
            p.ideo = null;
            list.Remove(p);
            CEditor.API.Pawn = list.FirstOrFallback(null);
            bool flag4 = CEditor.InStartingScreen && CEditor.ListName == Label.COLONISTS;
            if (flag4)
            {
              PawnxTool.Notify_CheckStartPawnsListChanged();
            }
          }
          bool flag5 = p.Dead && p.Corpse != null && p.Corpse.Spawned;
          if (flag5)
          {
            p.Corpse.DeSpawn(DestroyMode.Vanish);
          }
          bool spawned = p.Spawned;
          if (spawned)
          {
            p.DeSpawn(DestroyMode.Vanish);
          }
          bool flag6 = !p.Destroyed;
          if (flag6)
          {
            p.Destroy(DestroyMode.Vanish);
          }
          Find.World.worldPawns.RemoveAndDiscardPawnViaGC(p);
        }
        catch (Exception ex)
        {
          Log.Error(ex.Message + "\n" + ex.StackTrace);
        }
      }
    }

    // Token: 0x0600033B RID: 827 RVA: 0x0001A8BC File Offset: 0x00018ABC
    internal static void DeleteAllPawns(string listname, bool onMap, Faction faction)
    {
      List<Pawn> pawnList = PawnxTool.GetPawnList(listname, onMap, faction);
      for (int i = pawnList.Count - 1; i >= 0; i--)
      {
        pawnList[i].Delete(true);
      }
      PawnxTool.Notify_CheckStartPawnsListChanged();
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x0600033C RID: 828 RVA: 0x0001A90C File Offset: 0x00018B0C
    internal static void Notify_CheckStartPawnsListChanged()
    {
      bool flag = CEditor.InStartingScreen && CEditor.ListName == Label.COLONISTS;
      if (flag)
      {
        Find.GameInitData.startingAndOptionalPawns.Clear();
        Find.GameInitData.startingAndOptionalPawns.AddRange(CEditor.API.ListOf<Pawn>(EType.Pawns));
        bool flag2 = Find.GameInitData.startingPawnCount > Find.GameInitData.startingAndOptionalPawns.Count;
        if (flag2)
        {
          Find.GameInitData.startingPawnCount = Find.GameInitData.startingAndOptionalPawns.Count;
        }
      }
    }

    // Token: 0x0600033D RID: 829 RVA: 0x0001A9A0 File Offset: 0x00018BA0
    internal static bool FactionIsNotInsectNotMecha(Faction f)
    {
      return f != Faction.OfInsects && f != Faction.OfMechanoids;
    }

    // Token: 0x0600033E RID: 830 RVA: 0x0001A9C8 File Offset: 0x00018BC8
    internal static bool KeyIsHumanoid(bool notInsectNotMecha, string key)
    {
      return notInsectNotMecha && key != Label.COLONYANIMALS && key != Label.WILDANIMALS;
    }

    // Token: 0x0600033F RID: 831 RVA: 0x0001A9F8 File Offset: 0x00018BF8
    internal static bool KeyIsAnimal(bool notInsectNotMecha, string key, Faction f)
    {
      return notInsectNotMecha && key != Label.COLONISTS && (key == Label.COLONYANIMALS || key == Label.WILDANIMALS || (f != null && f.def.defName != "Zombies"));
    }

    // Token: 0x06000340 RID: 832 RVA: 0x0001AA58 File Offset: 0x00018C58
    internal static bool KeyIsOther(string key, Faction f)
    {
      return f != null && key != Label.COLONISTS && key != Label.COLONYANIMALS;
    }

    // Token: 0x06000341 RID: 833 RVA: 0x0001AA88 File Offset: 0x00018C88
    internal static bool FactionIsXeno(Faction f)
    {
      return f != null && f.Name.ToLower().Contains("xenomorph");
    }

    // Token: 0x06000342 RID: 834 RVA: 0x0001AAB5 File Offset: 0x00018CB5
    internal static bool IsOnMap(this Pawn p)
    {
      return p != null && (p.Spawned || (p.Corpse != null && p.Corpse.Spawned));
    }

    // Token: 0x06000343 RID: 835 RVA: 0x0001AAE0 File Offset: 0x00018CE0
    internal static List<Pawn> GetPawnList(string key, bool onMap, Faction f = null)
    {
      List<Pawn> result;
      try
      {
        bool flag = CEditor.InStartingScreen && key == Label.COLONISTS;
        if (flag)
        {
          List<Pawn> list = Find.GameInitData.startingAndOptionalPawns.ListFullCopy<Pawn>();
          result = list;
        }
        else
        {
          bool flag2;
          bool flag3;
          bool allFac;
          bool humanoid;
          bool animal;
          bool other;
          PawnxTool.SetPawnKindFlags(key, f, out flag2, out humanoid, out animal, out other, out flag3, out allFac);
          Map currentMap = Find.CurrentMap;
          List<Pawn> list2;
          if (currentMap == null)
          {
            list2 = null;
          }
          else
          {
            MapPawns mapPawns = currentMap.mapPawns;
            list2 = ((mapPawns != null) ? mapPawns.AllPawns : null);
          }
          List<Pawn> list3 = list2;
          bool flag4 = list3 == null;
          if (flag4)
          {
            list3 = new List<Pawn>();
          }
          World world = Find.World;
          List<Pawn> list4;
          if (world == null)
          {
            list4 = null;
          }
          else
          {
            WorldPawns worldPawns = world.worldPawns;
            list4 = ((worldPawns != null) ? worldPawns.AllPawnsAliveOrDead : null);
          }
          List<Pawn> list5 = list4;
          bool flag5 = list5 == null;
          if (flag5)
          {
            list5 = new List<Pawn>();
          }
          List<Pawn> list6 = list3.Concat(list5).ToList<Pawn>();
          bool flag6 = list6 == null;
          if (flag6)
          {
            list6 = new List<Pawn>();
          }
          bool o = CEditor.API.GetO(OptionB.USESORTEDPAWNLIST);
          List<Pawn> list7;
          if (o)
          {
            list7 = (from td in list6
                     where (allFac || td.Faction == f) && ((humanoid && td.RaceProps.Humanlike) || (animal && td.RaceProps.Animal) || (other && !td.RaceProps.Animal && !td.RaceProps.Humanlike)) && (!onMap || td.Spawned || (td.Corpse != null && td.Corpse.Spawned))
                     select td).OrderBy(delegate (Pawn td)
                     {
                       Faction faction = td.Faction;
                       return (faction != null) ? faction.Name : null;
                     }).ThenBy((Pawn td) => td.LabelShort).ToList<Pawn>();
          }
          else
          {
            list7 = (from td in list6
                     where (allFac || td.Faction == f) && ((humanoid && td.RaceProps.Humanlike) || (animal && td.RaceProps.Animal) || (other && !td.RaceProps.Animal && !td.RaceProps.Humanlike)) && (!onMap || td.Spawned || (td.Corpse != null && td.Corpse.Spawned))
                     select td).ToList<Pawn>();
          }
          result = list7;
        }
      }
      catch
      {
        result = null;
      }
      return result;
    }

    // Token: 0x06000344 RID: 836 RVA: 0x0001ACB4 File Offset: 0x00018EB4
    internal static void SetPawnKindFlags(string key, Faction f, out bool notInsectNotMecha, out bool humanoid, out bool animal, out bool other, out bool xeno, out bool allFac)
    {
      notInsectNotMecha = PawnxTool.FactionIsNotInsectNotMecha(f);
      humanoid = PawnxTool.KeyIsHumanoid(notInsectNotMecha, key);
      animal = PawnxTool.KeyIsAnimal(notInsectNotMecha, key, f);
      other = (PawnxTool.KeyIsOther(key, f) && f.def.defName != "Zombies");
      xeno = PawnxTool.FactionIsXeno(f);
      allFac = (f == null);
    }

    // Token: 0x06000345 RID: 837 RVA: 0x0001AD14 File Offset: 0x00018F14
    internal static void TeleportPawn(this Pawn pawn, IntVec3 pos = default(IntVec3))
    {
      bool flag = pawn == null;
      if (!flag)
      {
        pos = ((pos == default(IntVec3)) ? UI.MouseCell() : pos);
        bool flag2 = !pos.InBounds(Find.CurrentMap);
        if (!flag2)
        {
          bool flag3 = !pawn.Spawned || !pawn.Position.InBounds(Find.CurrentMap) || (pawn.Dead && (pawn.Corpse == null || pawn.Corpse.Position.InBounds(Find.CurrentMap)));
          bool flag4 = flag3;
          if (flag4)
          {
            pawn.SpawnPawn(null, pos);
          }
          bool dead = pawn.Dead;
          if (dead)
          {
            pawn.Corpse.Position = pos;
          }
          else
          {
            pawn.Position = pos;
            pawn.Notify_Teleported(true, true);
            pawn.Position = pos;
            pawn.Notify_Teleported(true, true);
          }
        }
      }
    }

    // Token: 0x06000346 RID: 838 RVA: 0x0001AE00 File Offset: 0x00019000
    internal static string GetTrainabilityLabel(this Pawn p)
    {
      return "CreatureTrainability".Translate().Formatted(p.def.label) + ": " + p.RaceProps.trainability.LabelCap;
    }

    // Token: 0x06000347 RID: 839 RVA: 0x0001AE58 File Offset: 0x00019058
    internal static string GetWildnessLabel(this Pawn p)
    {
      return "CreatureWildness".Translate().Formatted(p.def.label) + ": " + p.RaceProps.wildness.ToStringPercent();
    }

    // Token: 0x06000348 RID: 840 RVA: 0x0001AEB0 File Offset: 0x000190B0
    internal static void RecruitPawn(this Pawn p)
    {
      bool flag = p == null || p.Faction == Faction.OfPlayer || !p.RaceProps.Humanlike;
      if (!flag)
      {
        bool inStartingScreen = CEditor.InStartingScreen;
        Pawn recruiter;
        if (inStartingScreen)
        {
          recruiter = Find.GameInitData.startingAndOptionalPawns.RandomElement<Pawn>();
        }
        else
        {
          recruiter = PawnxTool.GetPawnList(Label.COLONISTS, CEditor.OnMap, Faction.OfPlayer).RandomElement<Pawn>();
        }
        bool inStartingScreen2 = CEditor.InStartingScreen;
        if (inStartingScreen2)
        {
          p.SetFaction(Faction.OfPlayer, recruiter);
        }
        else
        {
          InteractionWorker_RecruitAttempt.DoRecruit(recruiter, p, true);
        }
        bool inStartingScreen3 = CEditor.InStartingScreen;
        if (inStartingScreen3)
        {
          Find.GameInitData.startingAndOptionalPawns.Add(p);
          CEditor.API.ListOf<Pawn>(EType.Pawns).Add(p);
        }
        else
        {
          DebugActionsUtility.DustPuffFrom(p);
        }
        p.AllowAllApparel(null);
      }
    }

    // Token: 0x06000349 RID: 841 RVA: 0x0001AF84 File Offset: 0x00019184
    internal static void EnslavePawn(this Pawn p)
    {
      bool flag = p == null || p.Faction == Faction.OfPlayer || !p.RaceProps.Humanlike;
      if (!flag)
      {
        bool inStartingScreen = CEditor.InStartingScreen;
        Pawn warden;
        if (inStartingScreen)
        {
          warden = Find.GameInitData.startingAndOptionalPawns.RandomElement<Pawn>();
        }
        else
        {
          warden = PawnxTool.GetPawnList(Label.COLONISTS, CEditor.OnMap, Faction.OfPlayer).RandomElement<Pawn>();
        }
        GenGuest.EnslavePrisoner(warden, p);
        bool inStartingScreen2 = CEditor.InStartingScreen;
        if (inStartingScreen2)
        {
          Find.GameInitData.startingAndOptionalPawns.Add(p);
          CEditor.API.ListOf<Pawn>(EType.Pawns).Add(p);
        }
        else
        {
          DebugActionsUtility.DustPuffFrom(p);
        }
        p.AllowAllApparel(null);
      }
    }

    // Token: 0x0600034A RID: 842 RVA: 0x0001B038 File Offset: 0x00019238
    internal static string GetNamesOfUsedMods(string modids)
    {
      HashSet<string> l = PawnxTool.ModStringToHashSet(modids);
      return PawnxTool.ModListIdsToString(l);
    }

    // Token: 0x0600034B RID: 843 RVA: 0x0001B058 File Offset: 0x00019258
    internal static string GetNamesOfInactiveMods(string modids)
    {
      string text = "";
      HashSet<string> hashSet = PawnxTool.ModStringToHashSet(modids);
      try
      {
        foreach (string text2 in hashSet)
        {
          string text3 = text2.ToLower();
          bool flag = false;
          string str = text3;
          foreach (ModMetaData modMetaData in ModLister.AllInstalledMods)
          {
            bool flag2 = modMetaData.PackageId.ToLower().Equals(text3);
            if (flag2)
            {
              bool active = modMetaData.Active;
              if (active)
              {
                flag = true;
              }
              else
              {
                str = modMetaData.Name;
              }
              break;
            }
          }
          bool flag3 = !flag;
          if (flag3)
          {
            text = text + str + "|";
          }
        }
      }
      catch
      {
      }
      return text.SubstringRemoveLast();
    }

    // Token: 0x0600034C RID: 844 RVA: 0x0001B178 File Offset: 0x00019378
    internal static HashSet<string> ModStringToHashSet(string s)
    {
      bool flag = s.NullOrEmpty();
      HashSet<string> result;
      if (flag)
      {
        result = new HashSet<string>();
      }
      else
      {
        HashSet<string> hashSet = new HashSet<string>();
        try
        {
          s = s.Replace("[", "").Replace("]", "");
          string[] array = s.SplitNo("|");
          foreach (string item in array)
          {
            hashSet.Add(item);
          }
        }
        catch
        {
        }
        result = hashSet;
      }
      return result;
    }

    // Token: 0x0600034D RID: 845 RVA: 0x0001B210 File Offset: 0x00019410
    internal static string ModListIdsToString(HashSet<string> l)
    {
      string text = "";
      foreach (string text2 in l)
      {
        string value = text2.ToLower();
        foreach (ModMetaData modMetaData in ModLister.AllInstalledMods)
        {
          bool flag = modMetaData.PackageId.ToLower().Equals(value);
          if (flag)
          {
            text = text + modMetaData.Name + "|";
            break;
          }
        }
      }
      return text.SubstringRemoveLast();
    }

    // Token: 0x0600034E RID: 846 RVA: 0x0001B2E4 File Offset: 0x000194E4
    internal static string GetUsedModIds(this Pawn p)
    {
      HashSet<string> hashSet = new HashSet<string>();
      bool flag = p != null;
      if (flag)
      {
        try
        {
          hashSet.Add(p.kindDef.modContentPack.PackageId);
          bool flag2 = p.HasStoryTracker() && p.story.bodyType != null;
          if (flag2)
          {
            hashSet.Add(p.story.bodyType.modContentPack.PackageId);
          }
          bool flag3 = p.HasStoryTracker() && p.story.hairDef != null;
          if (flag3)
          {
            hashSet.Add(p.story.hairDef.modContentPack.PackageId);
          }
          bool flag4 = p.HasStoryTracker();
          if (flag4)
          {
            BackstoryDef backstory = p.story.GetBackstory(BackstorySlot.Childhood);
            BackstoryDef backstory2 = p.story.GetBackstory(BackstorySlot.Adulthood);
            bool flag5 = backstory.nameMaker != null;
            if (flag5)
            {
              hashSet.Add(backstory.nameMaker.modContentPack.PackageId);
            }
            bool flag6 = backstory2.nameMaker != null;
            if (flag6)
            {
              hashSet.Add(backstory2.nameMaker.modContentPack.PackageId);
            }
          }
          bool flag7 = p.HasEquipmentTracker();
          if (flag7)
          {
            foreach (ThingWithComps thingWithComps in p.equipment.AllEquipmentListForReading)
            {
              hashSet.Add(thingWithComps.def.modContentPack.PackageId);
            }
          }
          bool flag8 = p.HasApparelTracker();
          if (flag8)
          {
            foreach (Apparel apparel in p.apparel.WornApparel)
            {
              hashSet.Add(apparel.def.modContentPack.PackageId);
            }
          }
          bool flag9 = p.HasInventoryTracker();
          if (flag9)
          {
            foreach (Thing thing in p.inventory.innerContainer)
            {
              hashSet.Add(thing.def.modContentPack.PackageId);
            }
          }
          bool flag10 = p.HasStyleTracker();
          if (flag10)
          {
            bool flag11 = p.style.beardDef != null;
            if (flag11)
            {
              hashSet.Add(p.style.beardDef.modContentPack.PackageId);
            }
            bool flag12 = p.style.BodyTattoo != null;
            if (flag12)
            {
              hashSet.Add(p.style.BodyTattoo.modContentPack.PackageId);
            }
            bool flag13 = p.style.FaceTattoo != null;
            if (flag13)
            {
              hashSet.Add(p.style.FaceTattoo.modContentPack.PackageId);
            }
          }
          bool flag14 = p.HasAbilityTracker();
          if (flag14)
          {
            foreach (Ability ability in p.abilities.abilities)
            {
              hashSet.Add(ability.def.modContentPack.PackageId);
            }
          }
          bool flag15 = p.HasTraitTracker();
          if (flag15)
          {
            foreach (Trait trait in p.story.traits.allTraits)
            {
              hashSet.Add(trait.def.modContentPack.PackageId);
            }
          }
          bool flag16 = p.HasHealthTracker();
          if (flag16)
          {
            foreach (Hediff hediff in p.health.hediffSet.hediffs)
            {
              hashSet.Add(hediff.def.modContentPack.PackageId);
            }
          }
          bool flag17 = p.HasMemoryTracker();
          if (flag17)
          {
            foreach (Thought_Memory thought_Memory in p.needs.mood.thoughts.memories.Memories)
            {
              hashSet.Add(thought_Memory.def.modContentPack.PackageId);
            }
          }
          bool isGradientHairActive = CEditor.IsGradientHairActive;
          if (isGradientHairActive)
          {
            hashSet.Add("automatic.gradienthair");
          }
          bool isDualWieldActive = CEditor.IsDualWieldActive;
          if (isDualWieldActive)
          {
            hashSet.Add("Roolo.DualWield");
          }
          bool isFacialAnimationActive = CEditor.IsFacialAnimationActive;
          if (isFacialAnimationActive)
          {
            hashSet.Add("Nals.FacialAnimation");
          }
          bool isAlienRaceActive = CEditor.IsAlienRaceActive;
          if (isAlienRaceActive)
          {
            hashSet.Add("erdelf.HumanoidAlienRaces");
          }
          bool isCombatExtendedActive = CEditor.IsCombatExtendedActive;
          if (isCombatExtendedActive)
          {
            hashSet.Add("CETeam.CombatExtended");
          }
          bool flag18 = !p.GetRoyalTitleAsSeparatedString().NullOrEmpty();
          if (flag18)
          {
            hashSet.Add(RoyalTitleDefOf.Knight.modContentPack.PackageId);
          }
        }
        catch
        {
        }
      }
      return "[" + hashSet.ToList<string>().ListToString<string>() + "]";
    }

    // Token: 0x0600034F RID: 847 RVA: 0x0001B8E0 File Offset: 0x00019AE0
    internal static bool IsZombie(this Pawn p)
    {
      return p.Faction.IsZombie();
    }

    // Token: 0x06000350 RID: 848 RVA: 0x0001B8F0 File Offset: 0x00019AF0
    internal static void ReplacePawnWithPawnOfSameRace(Gender gender, Faction faction)
    {
      PawnKindDef kindDef = CEditor.API.Pawn.kindDef;
      ThingDef def = CEditor.API.Pawn.def;
      IntVec3 position = CEditor.API.Pawn.Position;
      int num = 10;
      BodyTypeDef bodyTypeDef = (gender == Gender.Female) ? BodyTypeDefOf.Female : BodyTypeDefOf.Male;
      do
      {
        num--;
        CEditor.API.Pawn.Delete(false);
        PawnxTool.AddOrCreateNewPawn(kindDef, faction, def, null, position);
        bool flag = gender == Gender.Female && CEditor.API.Pawn.story.bodyType == BodyTypeDefOf.Male;
        if (!flag)
        {
          bool flag2 = gender == Gender.Male && CEditor.API.Pawn.story.bodyType == BodyTypeDefOf.Female;
          if (!flag2)
          {
            bool flag3 = gender == Gender.None;
            if (flag3)
            {
              break;
            }
            CEditor.API.Pawn.gender = gender;
          }
        }
      }
      while (CEditor.API.Pawn.gender != gender && num > 0);
      bool flag4 = CEditor.API.Pawn.HasStyleTracker();
      if (flag4)
      {
        bool flag5 = gender == Gender.Female;
        if (flag5)
        {
          CEditor.API.Pawn.SetBeard(BeardDefOf.NoBeard);
        }
      }
    }

    // Token: 0x06000351 RID: 849 RVA: 0x0001BA30 File Offset: 0x00019C30
    internal static void ForceGenderizedBodyTypeDef(Gender gender)
    {
      CEditor.API.Pawn.gender = gender;
      bool flag = gender == Gender.Female && CEditor.API.Pawn.story.bodyType == BodyTypeDefOf.Male && PawnxTool.IsBodyInList(BodyTypeDefOf.Female);
      if (flag)
      {
        CEditor.API.Pawn.SetBody(BodyTypeDefOf.Female);
      }
      else
      {
        bool flag2 = gender == Gender.Male && CEditor.API.Pawn.story.bodyType == BodyTypeDefOf.Female && PawnxTool.IsBodyInList(BodyTypeDefOf.Male);
        if (flag2)
        {
          CEditor.API.Pawn.SetBody(BodyTypeDefOf.Male);
        }
      }
    }

    // Token: 0x06000352 RID: 850 RVA: 0x0001BADC File Offset: 0x00019CDC
    internal static bool IsBodyInList(BodyTypeDef b)
    {
      bool flag = CEditor.API.Pawn == null;
      bool result;
      if (flag)
      {
        result = false;
      }
      else
      {
        List<BodyTypeDef> bodyList = CEditor.API.Pawn.GetBodyList();
        bool flag2 = !bodyList.NullOrEmpty<BodyTypeDef>();
        result = (flag2 && bodyList.Contains(b));
      }
      return result;
    }

    // Token: 0x06000353 RID: 851 RVA: 0x0001BB2C File Offset: 0x00019D2C
    internal static Pawn CreateNewPawn(PawnKindDef pkd, Faction f, ThingDef raceDef)
    {
      ////Log.Message("PawnxTool.CreateNewPawn, start");
      if (pkd == null)
      {
        ////Log.Message("PawnxTool.CreateNewPawn, pawnkinddef = null");
        return null;
      }
      //////Log.Message("PawnxTool.CreateNewPawn, step 3");
      Pawn pawn = null;
      //////Log.Message("PawnxTool.CreateNewPawn, step 4");
      f = f.ThisOrDefault();
      bool flag2 = pkd.IsDroidColonist();
      bool flag3 = f.IsAbomination();
      bool flag4 = flag2 || (flag3 && pkd.IsDorfbewohner());
      //////Log.Message("PawnxTool.CreateNewPawn, step 5");
      if (flag4)
      {
        f = Faction.OfPlayer;
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 6");
      if (CEditor.ListName == Label.WILDANIMALS)
      {
        f = null;
      }
      bool flag7 = f.CanCreateRelations(pkd);
      PawnGenerationContext pawnGenerationContext = (f == Faction.OfPlayer) ? PawnGenerationContext.PlayerStarter : PawnGenerationContext.NonPlayer;
      // ToDo: grab arguments from 1.3
      //PawnGenerationRequest request = new PawnGenerationRequest(pkd, f, pawnGenerationContext, -1, true, false, false, false, flag7, false, 1f, false, true, true, true, false, false, false, false, 0f, 0f, null, 1f, null, null, null, null, null, null, null, null, null, null, null, null, null, false, false, false);
      //   PawnGenerationRequest requestOld = new PawnGenerationRequest(
      //pkd,
      //faction: f,
      //context: pawnGenerationContext,
      //tile: -1,
      //     forceGenerateNewPawn: true,
      //     newborn: false,
      //     allowDead: false,
      //     allowDowned: false,
      //     canGeneratePawnRelations: flag7,
      //     mustBeCapableOfViolence: false,
      //     colonistRelationChanceFactor: 1f,
      //     forceAddFreeWarmLayerIfNeeded: false,
      //     allowGay: true,
      //     allowFood: true,
      //     allowAddictions: true,
      //     inhabitant: false,
      //     certainlyBeenInCryptosleep: false,
      //     forceRedressWorldPawnIfFormerColonist: false,
      //     worldPawnFactionDoesntMatter: false,
      //     biocodeWeaponChance: 0f,
      //     biocodeApparelChance: 0f,
      //     extraPawnForExtraRelationChance: null,
      //     relationWithExtraPawnChanceFactor: 1f,
      //     validatorPreGear: null,
      //     validatorPostGear: null,
      //     forcedTraits: null,
      //     prohibitedTraits: null,
      //     minChanceToRedressWorldPawn: null,
      //     fixedBiologicalAge: null,
      //     fixedChronologicalAge: null,
      //     fixedGender: null,
      //     fixedMelanin: null,
      //     fixedLastName: null,
      //     fixedBirthName: null,
      //     fixedTitle: null,
      //     fixedIdeo: null,
      //     forceNoIdeo: false,
      //     forceNoBackstory: false,
      //     forbidAnyTitle: false
      //);
      //PawnGenerationRequest request = new PawnGenerationRequest(
      //pkd,
      //faction: f,
      //context: pawnGenerationContext,
      //tile: -1,
      //forceGenerateNewPawn: true,
      //allowDead: false,
      //allowDowned: false,
      //canGeneratePawnRelations: flag7,
      //mustBeCapableOfViolence: false,
      //colonistRelationChanceFactor: 1f,
      //forceAddFreeWarmLayerIfNeeded: false,
      //allowGay: true,
      //allowPregnant: false,
      //allowFood: true,
      //allowAddictions: true,
      //inhabitant: false,
      //certainlyBeenInCryptosleep: false,
      //forceRedressWorldPawnIfFormerColonist: false,
      //worldPawnFactionDoesntMatter: false,
      //biocodeWeaponChance: 0f,
      //biocodeApparelChance: 0f,
      //extraPawnForExtraRelationChance: null,
      //relationWithExtraPawnChanceFactor: 1f,
      //validatorPreGear: null,
      //validatorPostGear: null,
      //forcedTraits: null,
      //prohibitedTraits: null,
      //minChanceToRedressWorldPawn: null,
      //fixedBiologicalAge: null,
      //fixedChronologicalAge: null,
      //fixedGender: null,
      //fixedLastName: null,
      //fixedBirthName: null,
      //fixedTitle: null,
      //fixedIdeo: null,
      //forceNoIdeo: false,
      //forceNoBackstory: false,
      //forbidAnyTitle: false
      //);

      //Removed options in 1.4:
      //float? fixedMelanin: null,

      //New options in 1.4:
      //bool allowPregnant = false,
      //bool forceDead = false,
      //List< GeneDef > forcedXenogenes = null, 
      //List<GeneDef> forcedEndogenes = null,
      //XenotypeDef forcedXenotype = null, 
      //CustomXenotype forcedCustomXenotype = null,
      //List< XenotypeDef > allowedXenotypes = null, 
      //float forceBaselinerChance = 0f,
      //DevelopmentalStage developmentalStages = DevelopmentalStage.Adult, 
      //Func<XenotypeDef,
      //PawnKindDef> pawnKindDefGetter = null,
      //FloatRange? excludeBiologicalAgeRange = null, 
      //FloatRange? biologicalAgeRange = null,
      //bool forceRecruitable = false

      PawnGenerationRequest request = new PawnGenerationRequest(
      pkd,
      faction: f,
      context: pawnGenerationContext,
      forceGenerateNewPawn: true,
      canGeneratePawnRelations: flag7,
      allowPregnant: true
      );

      pawn = PawnGenerator.GeneratePawn(request);
      int num = 5;
      ////Log.Message("PawnxTool.CreateNewPawn, step 7");
      if (raceDef != null)
      {
        while (pawn.def != raceDef && num > 0)
        {
          pawn = PawnGenerator.GeneratePawn(request);
          num--;
        }
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 8");
      try
      {
        if (pawn == null && (pkd.IsTribalOrBoss() || (flag3 && !flag4)))
        {
          pawn = PawnGenerator.GeneratePawn(pkd, null);
        }
      }
      catch
      {
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 9");
      try
      {
        if (pawn == null)
        {
          pawn = PawnxTool.GeneratePawn(new PawnGenerationRequest(pkd, faction: f, context: pawnGenerationContext, forceGenerateNewPawn: true, canGeneratePawnRelations: flag7, allowPregnant: true));
          //pawn = PawnxTool.GeneratePawn(new PawnGenerationRequest(pkd, f, pawnGenerationContext, -1, true, false, false, false, flag7, false, 1f, false, true, true, true, false, false, false, false, 0f, 0f, null, 1f, null, null, null, null, null, null, null, null, null, null, null, null, null, false, false, false));
        }
      }
      catch
      {
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 10");
      try
      {
        if (pawn == null)
        {
          pawn = PawnxTool.GeneratePawn(new PawnGenerationRequest(pkd, faction: f, context: PawnGenerationContext.NonPlayer, forceGenerateNewPawn: true, canGeneratePawnRelations: flag7, allowPregnant: true));
          // ToDo: grab arguments from 1.3
          //pawn = PawnxTool.GeneratePawn(new PawnGenerationRequest(pkd, null, PawnGenerationContext.NonPlayer, -1, true, false, false, false, flag7, false, 1f, false, true, true, true, false, false, false, false, 0f, 0f, null, 1f, null, null, null, null, null, null, null, null, null, null, null, null, null, false, false, false));
        }
      }
      catch
      {
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 11");
      try
      {
        if (pawn == null)
        {
          pawn = PawnxTool.GeneratePawn(new PawnGenerationRequest(pkd, faction: f, context: PawnGenerationContext.All, forceGenerateNewPawn: true, canGeneratePawnRelations: flag7, allowPregnant: true));
          //pawn = PawnxTool.GeneratePawn(new PawnGenerationRequest(pkd, null, PawnGenerationContext.All, -1, true, false, false, false, flag7, false, 1f, false, true, true, true, false, false, false, false, 0f, 0f, null, 1f, null, null, null, null, null, null, null, null, null, null, null, null, null, false, false, false));
        }
      }
      catch
      {
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 12");
      try
      {
        if (pawn == null)
        {
          pawn = PawnGenerator.GeneratePawn(pkd, f);
        }
      }
      catch
      {
        Log.Error("failed to create pawn!");
      }
      if (!CEditor.API.GetO(OptionB.CREATERACESPECIFIC))
      {
        try
        {
          bool flag15 = pawn.HasStoryTracker();
          if (flag15)
          {
            pawn.Reequip(null, -1, true);
          }
          pawn.Redress(null, true, -1, true);
        }
        catch
        {
        }
      }
      else
      {
        try
        {
          string modName = pawn.kindDef.GetModName();
          List<HairDef> hashSet = DefTool.ListByMod<HairDef>(modName);
          if (!hashSet.EnumerableNullOrEmpty<HairDef>())
          {
            pawn.SetHair(hashSet.RandomElement<HairDef>());
          }
        }
        catch
        {
        }
      }
      ////Log.Message("PawnxTool.CreateNewPawn, step 14");
      if (CEditor.API.GetO(OptionB.CREATENUDE))
      {
        pawn.DestroyAllApparel();
      }
      if (CEditor.API.GetO(OptionB.CREATENOWEAPON))
      {
        pawn.DestroyAllEquipment();
      }
      if (CEditor.API.GetO(OptionB.CREATENOINV))
      {
        pawn.DestroyAllItems();
      }
      pawn.FixInvalidTracker();
      pawn.FixInvalidNames();
      pawn.ChangeFaction(f);
      ////Log.Message("PawnxTool.CreateNewPawn, end");
      return pawn;
    }

    // Token: 0x06000354 RID: 852 RVA: 0x0001BF90 File Offset: 0x0001A190
    internal static Pawn GeneratePawn(PawnKindDef pkd, Faction f)
    {
      int num = 10;
      Pawn pawn = null;
      do
      {
        num--;
        try
        {
          pawn = PawnGenerator.GeneratePawn(pkd, f);
        }
        catch
        {
        }
      }
      while (pawn == null && num > 0);
      return pawn;
    }

    // Token: 0x06000355 RID: 853 RVA: 0x0001BFE0 File Offset: 0x0001A1E0
    internal static Pawn GeneratePawn(PawnGenerationRequest pgr)
    {
      int num = 10;
      Pawn pawn = null;
      do
      {
        num--;
        try
        {
          pawn = PawnGenerator.GeneratePawn(pgr);
        }
        catch
        {
        }
      }
      while (pawn == null && num > 0);
      return pawn;
    }

    // Token: 0x06000356 RID: 854 RVA: 0x0001C02C File Offset: 0x0001A22C
    internal static PresetPawn ClonePawn(this Pawn p)
    {
      PresetPawn presetPawn = new PresetPawn();
      presetPawn.SavePawn(p, -1);
      presetPawn.GeneratePawn(true, true);
      return presetPawn;
    }

    // Token: 0x06000357 RID: 855 RVA: 0x0001C058 File Offset: 0x0001A258
    internal static void AddOrCreateExistingPawn(PresetPawn ppn)
    {
      if (ppn != null)
      {
        PawnxTool.AddOrCreateNewPawn(ppn.pawn.kindDef, ppn.pawn.Faction, ppn.pawn.def, ppn, default(IntVec3));
      }
    }

    // Token: 0x06000358 RID: 856 RVA: 0x0001C0A4 File Offset: 0x0001A2A4
    internal static bool CheckIfSpaceZombie(Faction f)
    {
      bool flag = f.IsZombie();
      bool flag2 = flag && CEditor.InStartingScreen && !PawnxTool.CanCreateZombieInSpace;
      bool result;
      if (flag2)
      {
        MessageTool.Show("can't create zombies in space!", null);
        result = true;
      }
      else
      {
        result = false;
      }
      return result;
    }

    // Token: 0x06000359 RID: 857 RVA: 0x0001C0E8 File Offset: 0x0001A2E8
    internal static void AddOrCreateNewPawn(PawnKindDef pkd, Faction f, ThingDef raceDef, PresetPawn ppn, IntVec3 pos = default(IntVec3))
    {
      bool flag = pkd == null;
      if (!flag)
      {
        bool flag2 = PawnxTool.CheckIfSpaceZombie(f);
        if (!flag2)
        {
          Pawn pawn = (ppn == null) ? PawnxTool.CreateNewPawn(pkd, f, raceDef) : ppn.pawn;
          bool flag3 = pawn != null;
          if (flag3)
          {
            pawn.SpawnOrPassPawn(f, ppn, pos);
          }
        }
      }
    }

    // Token: 0x0600035A RID: 858 RVA: 0x0001C134 File Offset: 0x0001A334
    internal static void FixInvalidNames(this Pawn p)
    {
      bool flag = p == null;
      if (!flag)
      {
        bool flag2 = p.Name != null && !p.Name.IsValid && p.Name.GetType() == typeof(NameTriple);
        if (flag2)
        {
          NameTriple nameTriple = p.Name as NameTriple;
          bool flag3 = nameTriple.Last.NullOrEmpty();
          if (flag3)
          {
            p.Name = new NameTriple(nameTriple.First, nameTriple.Nick, CEditor.zufallswert.Next(0, 99).ToString("2"));
          }
        }
      }
    }

    // Token: 0x0600035B RID: 859 RVA: 0x0001C1D5 File Offset: 0x0001A3D5
    internal static void ZombieWorldCreationFixInSpace(this Pawn p)
    {
    }

    // Token: 0x040001AB RID: 427
    internal static bool CanCreateZombieInSpace = false;
  }
}
