﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace CharacterEditor
{
	// Token: 0x0200001B RID: 27
	public static class PlacingTool
	{
		// Token: 0x060000F9 RID: 249 RVA: 0x00007D14 File Offset: 0x00005F14
		public static void DropWithPod(Pawn p, PresetPawn ppn)
		{
			DebugMenuOption debugMenuOption = new DebugMenuOption(Label.PLACE_PAWN + " droppod", DebugMenuOptionMode.Tool, delegate()
			{
				List<List<Thing>> list = new List<List<Thing>>
				{
					new List<Thing>
					{
						p
					}
				};
				bool flag = !p.IsColonist;
				if (flag)
				{
					p.CreatePawnLord(default(IntVec3));
				}
				DropPodUtility.DropThingGroupsNear(UI.MouseCell(), Find.CurrentMap, list, 110, false, false, true, true, true, false);
				PawnxTool.PostProcess(p, ppn);
				PlacingTool.TryCloneOrAbort(p);
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x060000FA RID: 250 RVA: 0x00007D70 File Offset: 0x00005F70
		public static void DoEffect(Pawn p, IntVec3 pos)
		{
			try
			{
				Ability ability = CEditor.API.GetO(OptionB.USECHAOSABILITY) ? AbilityUtility.MakeAbility(DefTool.AbilityDef("ChaosSkip"), p) : null;
				bool flag = ability != null;
				if (flag)
				{
					float warmupTime = ability.VerbProperties.First<VerbProperties>().warmupTime;
					FloatRange randomRange = new FloatRange(0f, 0f);
					foreach (AbilityComp abilityComp in ability.comps)
					{
						bool flag2 = abilityComp.props.GetType() == typeof(CompProperties_AbilityTeleport);
						if (flag2)
						{
							randomRange = (abilityComp.props as CompProperties_AbilityTeleport).randomRange;
							(abilityComp.props as CompProperties_AbilityTeleport).randomRange = new FloatRange(0f, 0f);
						}
					}
					ability.VerbProperties.First<VerbProperties>().warmupTime = 0f;
					ability.VerbTracker.PrimaryVerb.DrawHighlight(UI.MouseCell());
					ability.VerbTracker.PrimaryVerb.TryStartCastOn(p, false, true, false);
					ability.VerbTracker.PrimaryVerb.DrawHighlight(UI.MouseCell());
					DefTool.SoundDef("PsychicSoothePulserCast").PlayOneShot(new TargetInfo(UI.MouseCell(), Find.CurrentMap, false));
					p.jobs.ClearQueuedJobs(true);
					p.jobs.SetMemberValue("jobsGivenThisTick", 0);
					ability.VerbProperties.First<VerbProperties>().warmupTime = warmupTime;
					foreach (AbilityComp abilityComp2 in ability.comps)
					{
						bool flag3 = abilityComp2.props.GetType() == typeof(CompProperties_AbilityTeleport);
						if (flag3)
						{
							(abilityComp2.props as CompProperties_AbilityTeleport).randomRange = randomRange;
						}
					}
				}
			}
			catch
			{
			}
		}

		// Token: 0x060000FB RID: 251 RVA: 0x00007FD8 File Offset: 0x000061D8
		public static void PlaceInCustomPosition(Pawn p, PresetPawn ppn)
		{
			string label = Event.current.capsLock ? Label.CLONE_PAWN : Label.PLACE_PAWN;
			DebugMenuOption debugMenuOption = new DebugMenuOption(label, DebugMenuOptionMode.Tool, delegate()
			{
				p.SpawnPawn(ppn, UI.MouseCell());
				PlacingTool.DoEffect(p, UI.MouseCell());
				PlacingTool.TryCloneOrAbort(p);
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x060000FC RID: 252 RVA: 0x00008040 File Offset: 0x00006240
		public static void PlaceInCustomPosition(Selected s)
		{
			DebugMenuOption debugMenuOption = new DebugMenuOption("place object", DebugMenuOptionMode.Tool, delegate()
			{
				bool isApparel = s.thingDef.IsApparel;
				if (isApparel)
				{
					Apparel t = ApparelTool.GenerateApparel(s);
					t.Spawn(PlacingTool.rotation, default(IntVec3));
				}
				else
				{
					bool isWeapon = s.thingDef.IsWeapon;
					if (isWeapon)
					{
						ThingWithComps t2 = WeaponTool.GenerateWeapon(s);
						t2.Spawn(PlacingTool.rotation, default(IntVec3));
					}
					else
					{
						Thing t3 = ThingTool.GenerateItem(s);
						t3.Spawn(PlacingTool.rotation, default(IntVec3));
					}
				}
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x060000FD RID: 253 RVA: 0x0000808B File Offset: 0x0000628B
		public static void PlaceInPosition(Pawn p, PresetPawn ppn, IntVec3 pos)
		{
			p.SpawnPawn(ppn, pos);
		}

		// Token: 0x060000FE RID: 254 RVA: 0x00008098 File Offset: 0x00006298
		public static void TryCloneOrAbort(Pawn p)
		{
			bool capsLock = Event.current.capsLock;
			if (capsLock)
			{
				PresetPawn ppn = p.ClonePawn();
				PawnxTool.AddOrCreateExistingPawn(ppn);
			}
			else
			{
				DebugTools.curTool = null;
			}
		}

		// Token: 0x060000FF RID: 255 RVA: 0x000080CC File Offset: 0x000062CC
		public static void DeletePawnFromCustomPosition()
		{
			DebugMenuOption debugMenuOption = new DebugMenuOption(Label.DELETE_PAWN, DebugMenuOptionMode.Tool, delegate()
			{
				Find.CurrentMap.weatherManager.eventHandler.AddEvent(new WeatherEvent_LightningStrike(Find.CurrentMap, UI.MouseCell()));
				IntVec3 cell = UI.MouseCell();
				cell.DeletePawnsInCell();
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x06000100 RID: 256 RVA: 0x00008120 File Offset: 0x00006320
		public static void BeginTeleportCustomPawn()
		{
			Selector selector = Find.Selector;
			if (selector != null)
			{
				selector.ClearSelection();
			}
			DebugMenuOption debugMenuOption = new DebugMenuOption(Label.SELECT_PAWN, DebugMenuOptionMode.Tool, delegate()
			{
				Pawn pawn = UI.MouseCell().FirstPawnInCellArea();
				bool flag = pawn != null;
				if (flag)
				{
					PlacingTool.TeleportPawnAndReselect(pawn);
				}
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x06000101 RID: 257 RVA: 0x00008184 File Offset: 0x00006384
		public static void BeginTeleportPawn(Pawn p)
		{
			DebugMenuOption debugMenuOption = new DebugMenuOption(Label.TELEPORT, DebugMenuOptionMode.Tool, delegate()
			{
				p.TeleportPawn(default(IntVec3));
				PlacingTool.DoEffect(p, UI.MouseCell());
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x06000102 RID: 258 RVA: 0x000081D0 File Offset: 0x000063D0
		private static void TeleportPawnAndReselect(Pawn p)
		{
			DebugMenuOption debugMenuOption = new DebugMenuOption(Label.TELEPORT, DebugMenuOptionMode.Tool, delegate()
			{
				p.TeleportPawn(default(IntVec3));
				PlacingTool.DoEffect(p, UI.MouseCell());
				PlacingTool.BeginTeleportCustomPawn();
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x06000103 RID: 259 RVA: 0x0000821C File Offset: 0x0000641C
		public static void Destroy()
		{
			DebugMenuOption debugMenuOption = new DebugMenuOption("destroy", DebugMenuOptionMode.Tool, delegate()
			{
				IntVec3 intVec = UI.MouseCell();
				Find.CurrentMap.roofGrid.SetRoof(intVec, null);
				PlacingTool.ClearCell(intVec, Find.CurrentMap);
			});
			DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
		}

		// Token: 0x06000104 RID: 260 RVA: 0x00008270 File Offset: 0x00006470
		public static void ClearCell(IntVec3 pos, Map map)
		{
			try
			{
				Dictionary<int, Thing> dictionary = new Dictionary<int, Thing>();
				foreach (Thing value in pos.GetThingList(map))
				{
					dictionary.Add(dictionary.Count, value);
				}
				foreach (int key in dictionary.Keys)
				{
					Thing thing = dictionary[key];
					bool flag = thing != null && thing.def != null && thing.def.category != ThingCategory.Pawn;
					if (flag)
					{
						bool destroyable = thing.def.destroyable;
						if (destroyable)
						{
							thing.Destroy(DestroyMode.Vanish);
						}
						else
						{
							bool flag2 = thing.def.category == ThingCategory.Building;
							if (flag2)
							{
								thing.DeSpawn(DestroyMode.Vanish);
							}
						}
					}
				}
			}
			catch
			{
			}
		}

		// Token: 0x040000D9 RID: 217
		public static Rot4 rotation;
	}
}
