﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200003E RID: 62
	public static class PosTool
	{
		// Token: 0x0600035C RID: 860 RVA: 0x0001C1D8 File Offset: 0x0001A3D8
		public static void DeletePawnsInCell(this IntVec3 cell)
		{
			Pawn pawn = cell.FirstPawnFromCell();
			if (pawn != null)
			{
				pawn.Delete(false);
			}
		}

		// Token: 0x0600035D RID: 861 RVA: 0x0001C200 File Offset: 0x0001A400
		public static Pawn FirstPawnFromCell(this IntVec3 cell)
		{
			Pawn pawn = cell.FirstFromCell<Pawn>();
			bool flag = pawn != null;
			Pawn result;
			if (flag)
			{
				result = pawn;
			}
			else
			{
				Corpse corpse = cell.FirstFromCell<Corpse>();
				bool flag2 = corpse != null;
				if (flag2)
				{
					result = corpse.InnerPawn;
				}
				else
				{
					result = null;
				}
			}
			return result;
		}

		// Token: 0x0600035E RID: 862 RVA: 0x0001C248 File Offset: 0x0001A448
		public static Pawn FirstPawnInCellArea(this IntVec3 pos)
		{
			Pawn pawn = pos.FirstPawnFromCell();
			bool flag = pawn != null;
			Pawn result;
			if (flag)
			{
				result = pawn;
			}
			else
			{
				foreach (IntVec3 cell in new List<IntVec3>
				{
					pos,
					new IntVec3(pos.x, pos.y, pos.z),
					new IntVec3(pos.x, pos.y, pos.z + 1),
					new IntVec3(pos.x, pos.y, pos.z - 1),
					new IntVec3(pos.x + 1, pos.y, pos.z),
					new IntVec3(pos.x + 1, pos.y, pos.z + 1),
					new IntVec3(pos.x + 1, pos.y, pos.z - 1),
					new IntVec3(pos.x - 1, pos.y, pos.z),
					new IntVec3(pos.x - 1, pos.y, pos.z + 1),
					new IntVec3(pos.x - 1, pos.y, pos.z - 1)
				})
				{
					pawn = cell.FirstPawnFromCell();
					bool flag2 = pawn != null;
					if (flag2)
					{
						break;
					}
				}
				result = pawn;
			}
			return result;
		}

		// Token: 0x0600035F RID: 863 RVA: 0x0001C400 File Offset: 0x0001A600
		public static T FirstFromCell<T>(this IntVec3 cell)
		{
			bool flag = Find.CurrentMap == null || !cell.InBounds(Find.CurrentMap);
			T result;
			if (flag)
			{
				result = default(T);
			}
			else
			{
				result = Find.CurrentMap.thingGrid.ThingsAt(cell).ToList<Thing>().OfType<T>().FirstOrFallback(default(T));
			}
			return result;
		}

		// Token: 0x06000360 RID: 864 RVA: 0x0001C464 File Offset: 0x0001A664
		public static Pawn FirstPawnFromSelector(this Selector selector)
		{
			bool flag = selector != null && selector.FirstSelectedObject != null;
			if (flag)
			{
				object firstSelectedObject = selector.FirstSelectedObject;
				bool flag2 = firstSelectedObject.GetType() == typeof(Pawn);
				if (flag2)
				{
					return firstSelectedObject as Pawn;
				}
				bool flag3 = firstSelectedObject.GetType() == typeof(Corpse);
				if (flag3)
				{
					return (firstSelectedObject as Corpse).InnerPawn;
				}
			}
			return null;
		}

		// Token: 0x06000361 RID: 865 RVA: 0x0001C4E0 File Offset: 0x0001A6E0
		public static ThingDef FirstThingFromSelector(this Selector selector)
		{
			bool flag = selector != null && selector.FirstSelectedObject != null;
			if (flag)
			{
				object firstSelectedObject = selector.FirstSelectedObject;
				bool flag2 = firstSelectedObject.GetType() == typeof(ThingDef);
				if (flag2)
				{
					return firstSelectedObject as ThingDef;
				}
			}
			return null;
		}

		// Token: 0x06000362 RID: 866 RVA: 0x0001C534 File Offset: 0x0001A734
		public static void CheckAndSetScrollPos<T>(this Vector2 scrollPos, List<T> l, T tSelected, float elemenH, float maxH)
		{
			bool flag = tSelected != null && !l.NullOrEmpty<T>();
			if (flag)
			{
				int num = l.IndexOf(tSelected);
				bool flag2 = num >= 0;
				if (flag2)
				{
					Vector2 vector = new Vector2(scrollPos.x, (float)num * elemenH);
					float num2 = vector.y - scrollPos.y;
					bool flag3 = num2 < 0f || num2 > maxH;
					if (flag3)
					{
						scrollPos.x = vector.x;
						scrollPos.y = vector.y;
					}
				}
			}
		}

		// Token: 0x06000363 RID: 867 RVA: 0x0001C5C8 File Offset: 0x0001A7C8
		public static void CheckAndSetScrollPos<T>(this Vector2 scrollPos, HashSet<T> l, T tSelected, float elemenH, float maxH)
		{
			bool flag = tSelected != null && !l.NullOrEmpty<T>();
			if (flag)
			{
				int num = l.FirstIndexOf((T y) => tSelected.Equals(y));
				bool flag2 = num >= 0;
				if (flag2)
				{
					Vector2 vector = new Vector2(scrollPos.x, (float)num * elemenH);
					float num2 = vector.y - scrollPos.y;
					bool flag3 = num2 < 0f || num2 > maxH;
					if (flag3)
					{
						scrollPos.x = vector.x;
						scrollPos.y = vector.y;
					}
				}
			}
		}
	}
}
