﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200004C RID: 76
	public class PresetApparel
	{
		// Token: 0x060004CB RID: 1227 RVA: 0x0002FE2C File Offset: 0x0002E02C
		public PresetApparel()
		{
			this.label = "";
			this.defName = "";
			this.techLevel = "";
			this.tradeability = "";
			this.stealable = "";
			this.costStuffCount = "";
			this.stuffCategories = "";
			this.statBases = "";
			this.eqippedStats = "";
			this.bodyPartGroups = "";
			this.layers = "";
			this.tags = "";
			this.defaultOutfitTags = "";
		}

		// Token: 0x060004CC RID: 1228 RVA: 0x0002FF60 File Offset: 0x0002E160
		public PresetApparel(ThingDef armor)
		{
			bool flag = armor == null || !armor.IsApparel;
			if (!flag)
			{
				this.defName = armor.defName;
				this.label = armor.label;
				this.techLevel = Enum.GetName(typeof(TechLevel), armor.techLevel);
				this.tradeability = Enum.GetName(typeof(Tradeability), armor.tradeability);
				this.stealable = armor.stealable.ToString();
				this.costStuffCount = armor.costStuffCount.ToString();
				this.stuffCategories = armor.stuffCategories.ListToString<StuffCategoryDef>();
				this.statBases = armor.statBases.ListToString<StatModifier>();
				this.eqippedStats = armor.equippedStatOffsets.ListToString<StatModifier>();
				this.bodyPartGroups = armor.apparel.bodyPartGroups.ListToString<BodyPartGroupDef>();
				this.layers = armor.apparel.layers.ListToString<ApparelLayerDef>();
				this.tags = armor.apparel.tags.ListToString<string>();
				this.defaultOutfitTags = armor.apparel.defaultOutfitTags.ListToString<string>();
			}
		}

		// Token: 0x1700005C RID: 92
		// (get) Token: 0x060004CD RID: 1229 RVA: 0x00030124 File Offset: 0x0002E324
		public string AsString
		{
			get
			{
				string str = this.defName + ",";
				str = str + this.label + ",";
				str = str + this.techLevel + ",";
				str = str + this.tradeability + ",";
				str = str + this.stealable + ",";
				str = str + this.costStuffCount + ",";
				str = str + this.stuffCategories + ",";
				str = str + this.statBases + ",";
				str = str + this.eqippedStats + ",";
				str = str + this.bodyPartGroups + ",";
				str = str + this.layers + ",";
				str = str + this.tags + ",";
				return str + this.defaultOutfitTags + ";";
			}
		}

		// Token: 0x060004CE RID: 1230 RVA: 0x00030220 File Offset: 0x0002E420
		public void UpdateArmor()
		{
			bool flag = this.defName.NullOrEmpty();
			if (!flag)
			{
				ThingDef thingDef = DefTool.ThingDef(this.defName);
				bool flag2 = thingDef != null;
				if (flag2)
				{
					bool flag3 = false;
					List<StuffCategoryDef> list = this.stuffCategories.StringToListStuffCategory(ref flag3);
					List<StatModifier> list2 = this.statBases.StringToListStatModifier(ref flag3);
					List<StatModifier> equippedStatOffsets = this.eqippedStats.StringToListStatModifier(ref flag3);
					List<BodyPartGroupDef> list3 = this.bodyPartGroups.StringToListBodyPartGroup(ref flag3);
					List<ApparelLayerDef> list4 = this.layers.StringToListApparelLayer(ref flag3);
					bool flag4 = flag3;
					if (!flag4)
					{
						thingDef.label = this.label;
						thingDef.techLevel = (TechLevel)Enum.Parse(typeof(TechLevel), this.techLevel);
						thingDef.tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), this.tradeability);
						thingDef.stealable = (this.stealable.ToLower() == "true");
						thingDef.costStuffCount = this.costStuffCount.AsInt32();
						thingDef.stuffCategories = list;
						thingDef.statBases = list2;
						thingDef.equippedStatOffsets = equippedStatOffsets;
						thingDef.apparel.bodyPartGroups = list3;
						thingDef.apparel.layers = list4;
						thingDef.apparel.tags = this.tags.StringToList();
						thingDef.apparel.defaultOutfitTags = this.defaultOutfitTags.StringToList();
						thingDef.ResolveReferences();
						thingDef.PostLoad();
						bool devMode = Prefs.DevMode;
						if (devMode)
						{
							////Log.Message(this.label + " " + Label.MODIFICATIONLOADED);
						}
					}
				}
			}
		}

		// Token: 0x060004CF RID: 1231 RVA: 0x000303C4 File Offset: 0x0002E5C4
		public void SaveCustom()
		{
			CEditor.API.SetCustom(OptionS.CUSTOMARMOR, this.AsString, this.defName);
		}

		// Token: 0x060004D0 RID: 1232 RVA: 0x000303DF File Offset: 0x0002E5DF
		public void Reset()
		{
			CEditor.API.SetCustom(OptionS.CUSTOMARMOR, "", this.defName);
		}

		// Token: 0x060004D1 RID: 1233 RVA: 0x000303FC File Offset: 0x0002E5FC
		public void FromPreset(PresetApparel p)
		{
			bool flag = p != null;
			if (flag)
			{
				this.defName = p.defName;
				this.label = p.label;
				this.techLevel = p.techLevel;
				this.tradeability = p.tradeability;
				this.stealable = p.stealable;
				this.costStuffCount = p.costStuffCount;
				this.stuffCategories = p.stuffCategories;
				this.statBases = p.statBases;
				this.eqippedStats = p.eqippedStats;
				this.bodyPartGroups = p.bodyPartGroups;
				this.layers = p.layers;
				this.tags = p.tags;
				this.defaultOutfitTags = p.defaultOutfitTags;
			}
		}

		// Token: 0x060004D2 RID: 1234 RVA: 0x000304B4 File Offset: 0x0002E6B4
		public static void LoadCustomString(string custom)
		{
			bool flag = !string.IsNullOrEmpty(custom);
			if (flag)
			{
				try
				{
					string text = custom.Trim();
					string[] array = text.Split(new string[]
					{
						";"
					}, StringSplitOptions.RemoveEmptyEntries);
					foreach (string text2 in array)
					{
						string[] array3 = text2.Split(new string[]
						{
							","
						}, StringSplitOptions.None);
						bool flag2 = array3.Length >= 12;
						if (flag2)
						{
							int num = -1;
							new PresetApparel
							{
								defName = array3[++num],
								label = array3[++num],
								techLevel = array3[++num],
								tradeability = array3[++num],
								stealable = array3[++num],
								costStuffCount = array3[++num],
								stuffCategories = array3[++num],
								statBases = array3[++num],
								eqippedStats = array3[++num],
								bodyPartGroups = array3[++num],
								layers = array3[++num],
								tags = array3[++num],
								defaultOutfitTags = array3[num + 1]
							}.UpdateArmor();
						}
					}
				}
				catch (Exception ex)
				{
					bool devMode = Prefs.DevMode;
					if (devMode)
					{
						Log.Error(ex.Message + "\n" + ex.StackTrace);
					}
				}
			}
		}

		// Token: 0x060004D3 RID: 1235 RVA: 0x00030680 File Offset: 0x0002E880
		public static void ResetToDefault(string defName)
		{
			PresetApparel presetApparel;
			bool flag = CEditor.API.Get<Dictionary<string, PresetApparel>>(EType.ApparelPreset).TryGetValue(defName, out presetApparel);
			if (flag)
			{
				presetApparel.UpdateArmor();
				presetApparel.Reset();
			}
		}

		// Token: 0x060004D4 RID: 1236 RVA: 0x000306B8 File Offset: 0x0002E8B8
		public static Dictionary<string, PresetApparel> GetAllPresets()
		{
			Dictionary<string, PresetApparel> dictionary = new Dictionary<string, PresetApparel>();
			List<ThingDef> list = (from td in DefDatabase<ThingDef>.AllDefs
			where td.IsApparel && !td.label.NullOrEmpty()
			select td).ToList<ThingDef>();
			foreach (ThingDef thingDef in list)
			{
				dictionary.Add(thingDef.defName, new PresetApparel(thingDef));
			}
			return dictionary;
		}

		// Token: 0x0400043B RID: 1083
		public string label = "";

		// Token: 0x0400043C RID: 1084
		public string defName = "";

		// Token: 0x0400043D RID: 1085
		public string techLevel = "";

		// Token: 0x0400043E RID: 1086
		public string tradeability = "";

		// Token: 0x0400043F RID: 1087
		public string stealable = "";

		// Token: 0x04000440 RID: 1088
		public string costStuffCount = "";

		// Token: 0x04000441 RID: 1089
		public string stuffCategories = "";

		// Token: 0x04000442 RID: 1090
		public string statBases = "";

		// Token: 0x04000443 RID: 1091
		public string eqippedStats = "";

		// Token: 0x04000444 RID: 1092
		public string bodyPartGroups = "";

		// Token: 0x04000445 RID: 1093
		public string layers = "";

		// Token: 0x04000446 RID: 1094
		public string tags = "";

		// Token: 0x04000447 RID: 1095
		public string defaultOutfitTags = "";
	}
}
