﻿using System;
using System.Collections.Generic;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200004D RID: 77
	public class PresetItem
	{
		// Token: 0x060004D5 RID: 1237 RVA: 0x00030754 File Offset: 0x0002E954
		public PresetItem()
		{
			this.label = "";
			this.defName = "";
			this.techLevel = "";
			this.tradeability = "";
			this.stealable = "";
			this.costStuffCount = "";
			this.stuffCategories = "";
			this.statBases = "";
			this.eqippedStats = "";
		}

		// Token: 0x060004D6 RID: 1238 RVA: 0x00030830 File Offset: 0x0002EA30
		public PresetItem(ThingDef item)
		{
			bool flag = item == null;
			if (!flag)
			{
				this.defName = item.defName;
				this.label = item.label;
				this.techLevel = Enum.GetName(typeof(TechLevel), item.techLevel);
				this.tradeability = Enum.GetName(typeof(Tradeability), item.tradeability);
				this.stealable = item.stealable.ToString();
				this.costStuffCount = item.costStuffCount.ToString();
				this.stuffCategories = item.stuffCategories.ListToString<StuffCategoryDef>();
				this.statBases = item.statBases.ListToString<StatModifier>();
				this.eqippedStats = item.equippedStatOffsets.ListToString<StatModifier>();
			}
		}

		// Token: 0x1700005D RID: 93
		// (get) Token: 0x060004D7 RID: 1239 RVA: 0x00030964 File Offset: 0x0002EB64
		public string AsString
		{
			get
			{
				string str = this.defName + ",";
				str = str + this.label + ",";
				str = str + this.techLevel + ",";
				str = str + this.tradeability + ",";
				str = str + this.stealable + ",";
				str = str + this.costStuffCount + ",";
				str = str + this.stuffCategories + ",";
				str = str + this.statBases + ",";
				return str + this.eqippedStats + ";";
			}
		}

		// Token: 0x060004D8 RID: 1240 RVA: 0x00030A18 File Offset: 0x0002EC18
		public void UpdateItem()
		{
			bool flag = this.defName.NullOrEmpty();
			if (!flag)
			{
				ThingDef thingDef = DefTool.ThingDef(this.defName);
				bool flag2 = thingDef != null;
				if (flag2)
				{
					bool flag3 = false;
					List<StuffCategoryDef> list = this.stuffCategories.StringToListStuffCategory(ref flag3);
					List<StatModifier> list2 = this.statBases.StringToListStatModifier(ref flag3);
					List<StatModifier> equippedStatOffsets = this.eqippedStats.StringToListStatModifier(ref flag3);
					bool flag4 = flag3;
					if (!flag4)
					{
						thingDef.label = this.label;
						thingDef.techLevel = (TechLevel)Enum.Parse(typeof(TechLevel), this.techLevel);
						thingDef.tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), this.tradeability);
						thingDef.stealable = (this.stealable.ToLower() == "true");
						thingDef.costStuffCount = this.costStuffCount.AsInt32();
						thingDef.stuffCategories = list;
						thingDef.statBases = list2;
						thingDef.equippedStatOffsets = equippedStatOffsets;
						thingDef.ResolveReferences();
						thingDef.PostLoad();
						bool devMode = Prefs.DevMode;
						if (devMode)
						{
							////Log.Message(this.label + " " + Label.MODIFICATIONLOADED);
						}
					}
				}
			}
		}

		// Token: 0x060004D9 RID: 1241 RVA: 0x00030B58 File Offset: 0x0002ED58
		public void SaveCustom()
		{
			CEditor.API.SetCustom(OptionS.CUSTOMITEM, this.AsString, this.defName);
		}

		// Token: 0x060004DA RID: 1242 RVA: 0x00030B73 File Offset: 0x0002ED73
		public void Reset()
		{
			CEditor.API.SetCustom(OptionS.CUSTOMITEM, "", this.defName);
		}

		// Token: 0x060004DB RID: 1243 RVA: 0x00030B90 File Offset: 0x0002ED90
		public void FromPreset(PresetItem p)
		{
			bool flag = p != null;
			if (flag)
			{
				this.defName = p.defName;
				this.label = p.label;
				this.techLevel = p.techLevel;
				this.tradeability = p.tradeability;
				this.stealable = p.stealable;
				this.costStuffCount = p.costStuffCount;
				this.stuffCategories = p.stuffCategories;
				this.statBases = p.statBases;
				this.eqippedStats = p.eqippedStats;
			}
		}

		// Token: 0x060004DC RID: 1244 RVA: 0x00030C14 File Offset: 0x0002EE14
		public static void LoadCustomString(string custom)
		{
			bool flag = !string.IsNullOrEmpty(custom);
			if (flag)
			{
				try
				{
					string text = custom.Trim();
					string[] array = text.Split(new string[]
					{
						";"
					}, StringSplitOptions.RemoveEmptyEntries);
					foreach (string text2 in array)
					{
						string[] array3 = text2.Split(new string[]
						{
							","
						}, StringSplitOptions.None);
						bool flag2 = array3.Length >= 9;
						if (flag2)
						{
							int num = -1;
							new PresetItem
							{
								defName = array3[++num],
								label = array3[++num],
								techLevel = array3[++num],
								tradeability = array3[++num],
								stealable = array3[++num],
								costStuffCount = array3[++num],
								stuffCategories = array3[++num],
								statBases = array3[++num],
								eqippedStats = array3[num + 1]
							}.UpdateItem();
						}
					}
				}
				catch (Exception ex)
				{
					bool devMode = Prefs.DevMode;
					if (devMode)
					{
						Log.Error(ex.Message + "\n" + ex.StackTrace);
					}
				}
			}
		}

		// Token: 0x060004DD RID: 1245 RVA: 0x00030D9C File Offset: 0x0002EF9C
		public static void ResetToDefault(string defName)
		{
			PresetItem presetItem;
			bool flag = CEditor.API.Get<Dictionary<string, PresetItem>>(EType.ItemPreset).TryGetValue(defName, out presetItem);
			if (flag)
			{
				presetItem.UpdateItem();
				presetItem.Reset();
			}
		}

		// Token: 0x060004DE RID: 1246 RVA: 0x00030DD4 File Offset: 0x0002EFD4
		public static Dictionary<string, PresetItem> GetAllPresets()
		{
			Dictionary<string, PresetItem> dictionary = new Dictionary<string, PresetItem>();
      List<ThingDef> hashSet = ThingTool.ListOfItems(null, null);
			foreach (ThingDef thingDef in hashSet)
			{
				dictionary.Add(thingDef.defName, new PresetItem(thingDef));
			}
			return dictionary;
		}

		// Token: 0x04000448 RID: 1096
		public string label = "";

		// Token: 0x04000449 RID: 1097
		public string defName = "";

		// Token: 0x0400044A RID: 1098
		public string techLevel = "";

		// Token: 0x0400044B RID: 1099
		public string tradeability = "";

		// Token: 0x0400044C RID: 1100
		public string stealable = "";

		// Token: 0x0400044D RID: 1101
		public string costStuffCount = "";

		// Token: 0x0400044E RID: 1102
		public string stuffCategories = "";

		// Token: 0x0400044F RID: 1103
		public string statBases = "";

		// Token: 0x04000450 RID: 1104
		public string eqippedStats = "";
	}
}
