﻿using System;
using System.Collections.Generic;
using RimWorld;
using Verse;
using Verse.Sound;
using HarmonyLib;
using System.Reflection;

namespace CharacterEditor
{
  // Token: 0x02000014 RID: 20
  public class PresetPawn
  {
    public SortedDictionary<PresetPawn.Param, string> dicParams;
    private bool setBodyParts;
    internal Pawn pawn;
    internal bool bPlace;
    private int ParamCount => Enum.GetNames(typeof(PresetPawn.Param)).EnumerableCount();
    private int MinParams => 34;

    private string AsString
    {
      get
      {
        string text = "";
        foreach (PresetPawn.Param key in this.dicParams.Keys)
        {
          text = text + this.dicParams[key] + "*";
        }
        text = text.SubstringRemoveLast();
        return text;
      }
    }

    private string Save(int slot)
    {
      string asString = this.AsString;
      CEditor.API.SetSlot(slot, asString, true);
      return asString;
    }

    private bool Load(int slot, string custom)
    {
      ////Log.Message("PresetPawn.Load, start");
      bool result;
      if (!custom.NullOrEmpty())
      {
        result = this.LoadString(custom);
      }
      else
      {
        custom = CEditor.API.GetSlot(slot);
        result = this.LoadString(custom);
      }
      ////Log.Message("PresetPawn.Load, end");
      return result;
    }

    // Token: 0x060000CA RID: 202 RVA: 0x00005BE8 File Offset: 0x00003DE8
    private bool LoadString(string custom)
    {
      ////Log.Message("LoadString, custom = " + custom);
      this.dicParams = new SortedDictionary<PresetPawn.Param, string>();
      if (!string.IsNullOrEmpty(custom))
      {
        string[] array = custom.Trim().SplitNo("*");
        if (array.Length >= this.MinParams)
        {
          foreach (string value in array)
          {
            if (this.dicParams.Count < this.ParamCount)
            {
              this.dicParams.Add((PresetPawn.Param)this.dicParams.Count, value);
            }
          }
        }
      }
      if (this.dicParams.Count == 0)
      {
        MessageTool.Show("no data or not readable", null);
      }
      return this.dicParams.Count != 0;
    }

    // Token: 0x060000CB RID: 203 RVA: 0x00005CB8 File Offset: 0x00003EB8
    public PresetPawn()
    {
      this.pawn = null;
      this.bPlace = false;
      this.setBodyParts = true;
    }

    // Token: 0x060000CC RID: 204 RVA: 0x00005CD8 File Offset: 0x00003ED8
    public Pawn LoadPawn(int slot, bool choosePlace, string custom = "")
    {
      ////Log.Message("LoadPawn, start");
      this.pawn = null;
      this.bPlace = choosePlace;
      if (this.Load(slot, custom))
      {
        this.GeneratePawn(true, true);
        PawnxTool.AddOrCreateExistingPawn(this);
      }
      ////Log.Message("LoadPawn, end");
      return this.pawn;
    }

    public string SavePawn(Pawn p, int slot)
    {
      ////Log.Message("SavePawn, start");
      this.dicParams = new SortedDictionary<PresetPawn.Param, string>();
      //bool flag = p == null;
      //string result;
      if (p == null)
      {
        return "";
      }
      ////Log.Message("SavePawn, step 1");
      this.dicParams.Add(PresetPawn.Param.P00_id, p.GetPawnName(false));
      this.dicParams.Add(PresetPawn.Param.P01_pawnkinddef, p.GetPawnKindDefname());
      this.dicParams.Add(PresetPawn.Param.P02_faction, p.GetPawnFactionAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P03_name, p.GetPawnNameAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P04_ageticks, p.GetAgeTicks().ToString());
      this.dicParams.Add(PresetPawn.Param.P05_chronoticks, p.GetChronoAgeTicks().ToString());
      this.dicParams.Add(PresetPawn.Param.P06_gender, p.GetGenderInt());
      this.dicParams.Add(PresetPawn.Param.P07_bodydefname, p.GetBodyTypeDefName());

      //this.dicParams.Add(PresetPawn.Param.P08_headpath, p.GetHeadPath());
      this.dicParams.Add(PresetPawn.Param.P08_headtypedef, p.GetHeadTypeDefname());

      //this.dicParams.Add(PresetPawn.Param.P09_crowntype, p.GetCrownType().ToString());
      this.dicParams.Add(PresetPawn.Param.P09_crowntype, "crowntypeDefunct");

      this.dicParams.Add(PresetPawn.Param.P10_crown, p.GetCrown());
      //this.dicParams.Add(PresetPawn.Param.P10_crown, "crownDefunct");

      this.dicParams.Add(PresetPawn.Param.P11_headaddons, p.GetAllHeadAddonsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P12_hairdefname, p.GetHairDefName());
      this.dicParams.Add(PresetPawn.Param.P13_gradientmask, p.GetGradientMask());
      this.dicParams.Add(PresetPawn.Param.P14_haircolor1, p.GetHairColor(true).ColorHexString());
      this.dicParams.Add(PresetPawn.Param.P15_haircolor2, p.GetHairColor(false).ColorHexString());
      ////Log.Message("SavePawn, step 3");
      //this.dicParams.Add(PresetPawn.Param.P16_melanin, p.GetMelanin().ToString());
      this.dicParams.Add(PresetPawn.Param.P16_genetracker, p.GetGeneTrackerAsString());
      this.dicParams.Add(PresetPawn.Param.P17_skincolor1, p.GetSkinColor(true).ColorHexString());
      this.dicParams.Add(PresetPawn.Param.P18_skincolor2, p.GetSkinColor(false).ColorHexString());
      this.dicParams.Add(PresetPawn.Param.P19_eyecolor, p.GetEyeColor().ColorHexString());
      this.dicParams.Add(PresetPawn.Param.P20_skills, p.GetAllSkillsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P21_traits, p.GetAllTraitsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P22_hediffs, p.GetAllHediffsAsSeparatedString());
      ////Log.Message("SavePawn, step 4");
      this.dicParams.Add(PresetPawn.Param.P23_backstorychild, p.GetChildhoodDefname());
      this.dicParams.Add(PresetPawn.Param.P24_backstoryadult, p.GetAdulthoodDefname());
      this.dicParams.Add(PresetPawn.Param.P25_workpriorities, p.GetWorkPrioritiesAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P26_weaponlist, p.GetAllWeaponsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P27_apparellist, p.GetAllApparelAsSeparatedString());
      ////Log.Message("SavePawn, step 5");
      this.dicParams.Add(PresetPawn.Param.P28_itemlist, p.GetAllItemsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P29_records, p.GetAllRecordsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.p30_needs, p.GetAllNeedsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P31_memories, p.GetAllMemoriesAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P32_situationalmemories, p.GetAllSituationalMemoriesAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P33_relations, p.GetRelationsAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P34_royaltitles, p.GetRoyalTitleAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P35_beard, p.GetBeardDefName());
      ////Log.Message("SavePawn, step 6");
      this.dicParams.Add(PresetPawn.Param.P36_facetattoo, p.GetFaceTattooDefName());
      this.dicParams.Add(PresetPawn.Param.P37_bodytattoo, p.GetBodyTattooDefName());
      this.dicParams.Add(PresetPawn.Param.P38_entropy, p.GetEntropy().ToString());
      this.dicParams.Add(PresetPawn.Param.P39_psyfocus, p.GetPsyfocus().ToString());
      this.dicParams.Add(PresetPawn.Param.P40_psyabilities, p.GetPsyAbilitiesAsSeparatedString());
      this.dicParams.Add(PresetPawn.Param.P41_favcolor, p.GetFavColor().ColorHexString());
      ////Log.Message("SavePawn, step 7");
      this.dicParams.Add(PresetPawn.Param.P42_ideoculture, p.GetPawnCultureDefName());
      this.dicParams.Add(PresetPawn.Param.P43_ideoname, p.GetPawnIdeoName());
      this.dicParams.Add(PresetPawn.Param.P44_modids, p.GetUsedModIds());
      this.dicParams.Add(PresetPawn.Param.P45_facialanimation, p.GetFacialAnimationParams());
      this.dicParams.Add(PresetPawn.Param.P46_personality, p.GetPersonality());
      ////Log.Message("SavePawn, step 8");
      if (Prefs.DevMode)
      {
        MessageTool.Show(PawnxTool.GetNamesOfUsedMods(this.dicParams.GetValue(PresetPawn.Param.P44_modids)), null);
      }
      ////Log.Message("SavePawn, step 9");
      string result = this.Save(slot);
      ////Log.Message("SavePawn, end, result = " + result);
      return result;
    }

    // Token: 0x060000CE RID: 206 RVA: 0x00006174 File Offset: 0x00004374
    public void GeneratePawn(bool _setBodyParts = true, bool setApparel = true)
    {
      ////Log.Message("PresetPawn.GeneratePawn start");
      this.setBodyParts = _setBodyParts;
      ////Log.Message("PresetPawn.GeneratePawn step 0");
      PawnKindDef pawnKindDef = DefTool.PawnKindDef(this.dicParams.GetValue(PresetPawn.Param.P01_pawnkinddef), PawnKindDefOf.Colonist);
      ////Log.Message("PresetPawn.GeneratePawn step 1");
      ThingDef raceDef = (pawnKindDef == null) ? null : pawnKindDef.race;
      ////Log.Message("PresetPawn.GeneratePawn step 2");
      Faction bySeparatedString = FactionTool.GetBySeparatedString(this.dicParams.GetValue(PresetPawn.Param.P02_faction), Faction.OfPlayer);
      ////Log.Message("PresetPawn.GeneratePawn step 3");
      this.pawn = PawnxTool.CreateNewPawn(pawnKindDef, bySeparatedString, raceDef);
      bool flag = this.pawn == null;
      if (!flag)
      {
        ////Log.Message("PresetPawn.GeneratePawn step 4");
        this.pawn.ResurrectAndHeal();
        this.pawn.SetNameFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P03_name));
        this.pawn.SetAgeTicks(this.dicParams.GetValue(PresetPawn.Param.P04_ageticks).AsLong());
        this.pawn.SetChronoAgeTicks(this.dicParams.GetValue(PresetPawn.Param.P05_chronoticks).AsLong());
        this.pawn.SetGenderInt(this.dicParams.GetValue(PresetPawn.Param.P06_gender).AsInt32());
        bool flag2 = this.setBodyParts;
        if (flag2)
        {
          BodyTypeDef bodyTypeDef = DefTool.BodyTypeDef(this.dicParams.GetValue(PresetPawn.Param.P07_bodydefname));
          bool flag3 = bodyTypeDef != null;
          if (flag3)
          {
            this.pawn.SetBody(bodyTypeDef);
          }
          else
          {
            bool flag4 = !pawnKindDef.IsAnimal() && !pawnKindDef.RaceProps.Humanlike;
            if (flag4)
            {
              MessageTool.Show("Body not found:" + this.dicParams.GetValue(PresetPawn.Param.P07_bodydefname), null);
            }
          }
          //this.pawn.SetHeadPath(this.dicParams.GetValue(PresetPawn.Param.P08_headpath), false);
          ////this.pawn.SetCrownType((byte)this.dicParams.GetValue(PresetPawn.Param.P09_crowntype).AsInt32());
          //this.pawn.SetCrownType((CrownType)Enum.Parse(typeof(CrownType), this.dicParams.GetValue(PresetPawn.Param.P09_crowntype)));
          //this.pawn.SetCrown();
          this.pawn.SetHeadTypeDef(this.dicParams.GetValue(PresetPawn.Param.P08_headtypedef), false);
          this.pawn.SetHeadAddonsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P11_headaddons));
          HairDef hairDef = DefTool.HairDef(this.dicParams.GetValue(PresetPawn.Param.P12_hairdefname));
          bool flag5 = hairDef != null;
          if (flag5)
          {
            this.pawn.SetHair(hairDef);
          }
          else
          {
            bool flag6 = !pawnKindDef.IsAnimal() && !pawnKindDef.RaceProps.Humanlike;
            if (flag6)
            {
              MessageTool.Show("Hair not found:" + this.dicParams.GetValue(PresetPawn.Param.P12_hairdefname), null);
            }
          }
        }
        this.pawn.SetGradientMask(this.dicParams.GetValue(PresetPawn.Param.P13_gradientmask));
        this.pawn.SetHairColor(true, this.dicParams.GetValue(PresetPawn.Param.P14_haircolor1).HexStringToColor());
        this.pawn.SetHairColor(false, this.dicParams.GetValue(PresetPawn.Param.P15_haircolor2).HexStringToColor());
        this.pawn.SetGeneTracker(this.dicParams.GetValue(PresetPawn.Param.P16_genetracker));
        this.pawn.SetSkinColor(true, this.dicParams.GetValue(PresetPawn.Param.P17_skincolor1).HexStringToColor());
        this.pawn.SetSkinColor(false, this.dicParams.GetValue(PresetPawn.Param.P18_skincolor2).HexStringToColor());
        this.pawn.SetEyeColor(this.dicParams.GetValue(PresetPawn.Param.P19_eyecolor).HexStringToColor());
        this.pawn.SetSkillsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P20_skills));
        this.pawn.SetTraitsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P21_traits));
        this.pawn.SetHediffsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P22_hediffs));
        this.pawn.SetBackstory(
          BackstoryTool.GetBackstoryDef(this.dicParams.GetValue(PresetPawn.Param.P23_backstorychild)),
          BackstoryTool.GetBackstoryDef(this.dicParams.GetValue(PresetPawn.Param.P24_backstoryadult))
          );
        this.pawn.SetWorkPrioritiesFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P25_workpriorities));
        this.pawn.SetWeaponsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P26_weaponlist));
        if (setApparel)
        {
          this.pawn.SetApparelFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P27_apparellist));
        }
        this.pawn.SetItemsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P28_itemlist));
        this.pawn.SetRecords(this.dicParams.GetValue(PresetPawn.Param.P29_records));
        this.pawn.SetNeeds(this.dicParams.GetValue(PresetPawn.Param.p30_needs));
        this.pawn.SetMemories(this.dicParams.GetValue(PresetPawn.Param.P31_memories));
        this.pawn.SetSituationalMemories(this.dicParams.GetValue(PresetPawn.Param.P32_situationalmemories));
        this.pawn.SetRelationsFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P33_relations));
        this.pawn.SetGradientMask(this.dicParams.GetValue(PresetPawn.Param.P13_gradientmask));
        this.pawn.SetHairColor(true, this.dicParams.GetValue(PresetPawn.Param.P14_haircolor1).HexStringToColor());
        this.pawn.SetHairColor(false, this.dicParams.GetValue(PresetPawn.Param.P15_haircolor2).HexStringToColor());
        this.pawn.SetEyeColor(this.dicParams.GetValue(PresetPawn.Param.P19_eyecolor).HexStringToColor());
        if (ModsConfig.RoyaltyActive && this.setBodyParts)
        {
          this.pawn.SetRoyalTitleFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P34_royaltitles));
        }
        if (this.dicParams.Count > 35)
        {
          this.pawn.SetBeard(DefTool.BeardDef(this.dicParams.GetValue(PresetPawn.Param.P35_beard)));
        }
        if (this.dicParams.Count > 36)
        {
          this.pawn.SetFaceTattoo(DefTool.TattooDef(this.dicParams.GetValue(PresetPawn.Param.P36_facetattoo)));
        }
        if (this.dicParams.Count > 37)
        {
          this.pawn.SetBodyTattoo(DefTool.TattooDef(this.dicParams.GetValue(PresetPawn.Param.P37_bodytattoo)));
        }
        if (this.pawn.psychicEntropy != null && this.pawn.psychicEntropy.Psylink != null)
        {
          if (this.dicParams.Count > 38)
          {
            this.pawn.SetEntropy(this.dicParams.GetValue(PresetPawn.Param.P38_entropy).AsFloat());
          }
          if (this.dicParams.Count > 39)
          {
            this.pawn.SetPsyfocus(this.dicParams.GetValue(PresetPawn.Param.P39_psyfocus).AsFloat());
          }
          if (this.dicParams.Count > 40)
          {
            this.pawn.SetPsyAbilitiesFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P40_psyabilities));
          }
        }
        if (this.dicParams.Count > 41)
        {
          this.pawn.SetFavColor(this.dicParams.GetValue(PresetPawn.Param.P41_favcolor).HexStringToColor());
        }
        if (ModsConfig.IdeologyActive && this.dicParams.Count > 43)
        {
          this.pawn.SetPawnIdeo(this.dicParams.GetValue(PresetPawn.Param.P42_ideoculture), this.dicParams.GetValue(PresetPawn.Param.P43_ideoname));
        }
        if (this.dicParams.Count > 44)
        {
          string value = this.dicParams.GetValue(PresetPawn.Param.P44_modids);
          string namesOfInactiveMods = PawnxTool.GetNamesOfInactiveMods(value);
          bool flag18 = !namesOfInactiveMods.NullOrEmpty();
          if (flag18)
          {
            MessageTool.Show("Missing Mods: " + namesOfInactiveMods, null);
            bool flag19 = !CEditor.InStartingScreen;
            if (flag19)
            {
              SoundDefOf.Quest_Failed.PlayOneShot(new TargetInfo(UI.MouseCell(), Find.CurrentMap, false));
            }
          }
        }
        if (this.dicParams.Count > 45)
        {
          this.pawn.SetFacialAnimationParams(this.dicParams.GetValue(PresetPawn.Param.P45_facialanimation));
        }
        if (this.dicParams.Count > 46)
        {
          this.pawn.SetPersonality(this.dicParams.GetValue(PresetPawn.Param.P46_personality));
        }
        if (this.dicParams.Count > 47)
        {
          CompatibilityTool.SetPsychology(this.pawn, this.dicParams.GetValue(PresetPawn.Param.P47_psychology));
        }
        try
        {
          float bleedRateTotal = this.pawn.health.hediffSet.BleedRateTotal;
        }
        catch
        {
          this.pawn.health.hediffSet.Clear();
        }
      }
    }

    // Token: 0x060000CF RID: 207 RVA: 0x0000697C File Offset: 0x00004B7C
    public void PostProcess()
    {
      Capturer capturer = CEditor.API.Get<Capturer>(EType.Capturer);
      capturer.UpdatePawnGraphic(this.pawn);
      this.pawn.SetNameFromSeparatedString(this.dicParams.GetValue(PresetPawn.Param.P03_name));
      bool flag = this.setBodyParts;
      if (flag)
      {
        //this.pawn.SetHeadPath(this.dicParams.GetValue(PresetPawn.Param.P08_headpath), true);
        //this.pawn.SetCrownType((CrownType)Enum.Parse(typeof(CrownType), this.dicParams.GetValue(PresetPawn.Param.P09_crowntype)));
        //this.pawn.SetCrown();
        this.pawn.SetHeadTypeDef(this.dicParams.GetValue(PresetPawn.Param.P08_headtypedef), true);
      }
      this.pawn.SetGeneTracker(this.dicParams.GetValue(PresetPawn.Param.P16_genetracker));
      this.pawn.SetSkinColor(true, this.dicParams.GetValue(PresetPawn.Param.P17_skincolor1).HexStringToColor());
      this.pawn.SetSkinColor(false, this.dicParams.GetValue(PresetPawn.Param.P18_skincolor2).HexStringToColor());
      if (this.dicParams.Count > 45)
      {
        this.pawn.SetFacialAnimationParams(this.dicParams.GetValue(PresetPawn.Param.P45_facialanimation));
      }
    }

    // Token: 0x060000D0 RID: 208 RVA: 0x00006A94 File Offset: 0x00004C94
    public static void ClearAllSlots()
    {
      int num = CEditor.API.NumSlots - 1;
      for (int i = num; i > 0; i--)
      {
        CEditor.API.SetSlot(i, "", i == 0);
      }
    }

    // Token: 0x060000D1 RID: 209 RVA: 0x00006AD8 File Offset: 0x00004CD8
    public static void ClearAllCustoms()
    {
      CEditor.API.SetCustom(OptionS.CUSTOMARMOR, "", "");
      CEditor.API.SetCustom(OptionS.CUSTOMWEAPON, "", "");
      CEditor.API.SetCustom(OptionS.CUSTOMITEM, "", "");
    }

    // Token: 0x060000D2 RID: 210 RVA: 0x00006B28 File Offset: 0x00004D28
    public static void RescueOldPawns()
    {
    }




    // Token: 0x0200006C RID: 108
    public enum Param
    {
      P00_id,
      P01_pawnkinddef,
      P02_faction,
      P03_name,
      P04_ageticks,
      P05_chronoticks,
      P06_gender,
      P07_bodydefname,
      P08_headtypedef, //P08_headpath
      P09_crowntype, // Defunct
      P10_crown, // Might be defunct
      P11_headaddons,
      P12_hairdefname,
      P13_gradientmask,
      P14_haircolor1,
      P15_haircolor2,
      P16_genetracker, // P16_melanin
      P17_skincolor1,
      P18_skincolor2,
      P19_eyecolor,
      P20_skills,
      P21_traits,
      P22_hediffs,
      P23_backstorychild,
      P24_backstoryadult,
      P25_workpriorities,
      P26_weaponlist,
      P27_apparellist,
      P28_itemlist,
      P29_records,
      p30_needs,
      P31_memories,
      P32_situationalmemories,
      P33_relations,
      P34_royaltitles,
      P35_beard,
      P36_facetattoo,
      P37_bodytattoo,
      P38_entropy,
      P39_psyfocus,
      P40_psyabilities,
      P41_favcolor,
      P42_ideoculture,
      P43_ideoname,
      P44_modids,
      P45_facialanimation,
      P46_personality,
      P47_psychology
    }
  }
}
