﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200004E RID: 78
	public class PresetWeapon
	{
		// Token: 0x060004DF RID: 1247 RVA: 0x00030E48 File Offset: 0x0002F048
		public PresetWeapon()
		{
			this.label = "";
			this.defName = "";
			this.bulletName = "";
			this.soundName = "";
			this.techLevel = "";
			this.tradeability = "";
			this.stealable = "";
			this.costStuffCount = "";
			this.stuffCategories = "";
			this.statBases = "";
			this.equippedStats = "";
			this.tags = "";
			this.bTargetGround = false;
			this.bulletExplosionRadius = 0f;
			this.bulletStoppingPower = 0f;
			this.bulletSpeed = 0f;
			this.bulletCount = 0;
			this.bulletInterval = 0;
			this.weaponCooldown = 0f;
			this.accuracyTouch = 0f;
			this.accuracyShort = 0f;
			this.accuracyMedium = 0f;
			this.accuracyLong = 0f;
			this.weaponWarmup = 0f;
			this.weaponMinRange = 0f;
			this.weaponRange = 0f;
			this.weaponScatter = 0f;
			this.weaponFuelCapacity = 0;
			this.weaponCEReload = 0f;
		}

		// Token: 0x060004E0 RID: 1248 RVA: 0x000310BC File Offset: 0x0002F2BC
		public PresetWeapon(ThingDef gun)
		{
			bool flag = gun == null || gun.IsApparel;
			if (!flag)
			{
				ThingDef turretDef = gun.GetTurretDef();
				ThingDef realWeaponDef = gun.GetRealWeaponDef();
				bool flag2 = realWeaponDef == null || realWeaponDef.Verbs.Count == 0;
				if (!flag2)
				{
					this.defName = realWeaponDef.defName;
					this.label = realWeaponDef.label;
					this.bulletName = realWeaponDef.GetBulletDefName();
					this.soundName = realWeaponDef.GetSoundCastDefName();
					this.techLevel = Enum.GetName(typeof(TechLevel), realWeaponDef.techLevel);
					this.tradeability = Enum.GetName(typeof(Tradeability), realWeaponDef.tradeability);
					this.stealable = realWeaponDef.stealable.ToString();
					this.costStuffCount = realWeaponDef.costStuffCount.ToString();
					this.stuffCategories = realWeaponDef.stuffCategories.ListToString<StuffCategoryDef>();
					this.statBases = realWeaponDef.statBases.ListToString<StatModifier>();
					this.equippedStats = realWeaponDef.equippedStatOffsets.ListToString<StatModifier>();
					this.tags = realWeaponDef.weaponTags.ListToString<string>();
					bool flag3 = turretDef != null;
					if (flag3)
					{
						this.bTargetGround = turretDef.building.turretGunDef.GetWeaponTargetGround();
					}
					else
					{
						this.bTargetGround = realWeaponDef.GetWeaponTargetGround();
					}
					this.bulletExplosionRadius = realWeaponDef.GetBulletExplosionRadius();
					this.bulletStoppingPower = realWeaponDef.GetBulletStoppingPower();
					bool flag4 = turretDef != null;
					if (flag4)
					{
						this.bulletSpeed = turretDef.building.turretGunDef.GetBulletSpeed();
					}
					else
					{
						this.bulletSpeed = realWeaponDef.GetBulletSpeed();
					}
					this.bulletCount = realWeaponDef.Verbs[0].burstShotCount;
					this.bulletInterval = realWeaponDef.Verbs[0].ticksBetweenBurstShots;
					this.weaponCooldown = realWeaponDef.GetStatValue(StatDefOf.RangedWeapon_Cooldown);
					this.accuracyTouch = realWeaponDef.GetStatValue(StatDefOf.AccuracyTouch);
					this.accuracyShort = realWeaponDef.GetStatValue(StatDefOf.AccuracyShort);
					this.accuracyMedium = realWeaponDef.GetStatValue(StatDefOf.AccuracyMedium);
					this.accuracyLong = realWeaponDef.GetStatValue(StatDefOf.AccuracyLong);
					this.weaponWarmup = realWeaponDef.Verbs[0].warmupTime;
					this.weaponMinRange = realWeaponDef.Verbs[0].minRange;
					this.weaponRange = realWeaponDef.Verbs[0].range;
					this.weaponScatter = realWeaponDef.Verbs[0].ForcedMissRadius;
					StatDef statDef = DefTool.StatDef("ShotSpread");
					bool flag5 = statDef != null;
					if (flag5)
					{
						this.weaponScatter = realWeaponDef.GetStatValue(statDef);
					}
					this.weaponFuelCapacity = WeaponTool.GetCompRefuelable(null, realWeaponDef);
					this.weaponCEReload = WeaponTool.GetCompReloadTime(null, realWeaponDef);
					bool flag6 = turretDef != null;
					if (flag6)
					{
						this.weaponCooldown = turretDef.building.turretBurstCooldownTime;
						//this.weaponWarmup = turretDef.building.turretBurstWarmupTime;
						this.weaponCEReload = WeaponTool.GetCompReloadTime(turretDef, realWeaponDef);
						this.weaponFuelCapacity = WeaponTool.GetCompRefuelable(turretDef, realWeaponDef);
					}
				}
			}
		}

		// Token: 0x1700005E RID: 94
		// (get) Token: 0x060004E1 RID: 1249 RVA: 0x000314F0 File Offset: 0x0002F6F0
		public string AsString
		{
			get
			{
				string str = this.defName + ",";
				str = str + this.label + ",";
				str = str + this.bulletName + ",";
				str = str + this.soundName + ",";
				str = str + this.techLevel + ",";
				str = str + this.tradeability + ",";
				str = str + this.stealable + ",";
				str = str + this.costStuffCount + ",";
				str = str + this.stuffCategories + ",";
				str = str + this.statBases + ",";
				str = str + this.equippedStats + ",";
				str = str + this.tags + ",";
				str = str + this.bTargetGround.ToString() + ",";
				str = str + this.bulletExplosionRadius.ToString() + ",";
				str = str + this.bulletStoppingPower.ToString() + ",";
				str = str + this.bulletSpeed.ToString() + ",";
				str = str + this.bulletCount.ToString() + ",";
				str = str + this.bulletInterval.ToString() + ",";
				str = str + this.weaponCooldown.ToString() + ",";
				str = str + this.weaponWarmup.ToString() + ",";
				str = str + this.weaponMinRange.ToString() + ",";
				str = str + this.weaponRange.ToString() + ",";
				str = str + this.weaponScatter.ToString() + ",";
				str = str + this.weaponFuelCapacity.ToString() + ",";
				return str + this.weaponCEReload.ToString() + ";";
			}
		}

		// Token: 0x060004E2 RID: 1250 RVA: 0x00031708 File Offset: 0x0002F908
		public bool UpdateWeapon()
		{
			bool flag = this.defName.NullOrEmpty();
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				ThingDef thingDef = DefTool.ThingDef(this.defName);
				bool flag2 = thingDef != null;
				if (flag2)
				{
					bool flag3 = false;
					List<StuffCategoryDef> list = this.stuffCategories.StringToListStuffCategory(ref flag3);
					List<StatModifier> list2 = this.statBases.StringToListStatModifier(ref flag3);
					List<StatModifier> equippedStatOffsets = this.equippedStats.StringToListStatModifier(ref flag3);
					bool flag4 = flag3;
					if (flag4)
					{
						result = false;
					}
					else
					{
						SoundDef soundCast = DefTool.SoundDef(this.soundName);
						ThingDef thingDef2 = DefTool.ThingDef(this.bulletName);
						ThingDef turretDef = thingDef.GetTurretDef();
						ThingDef realWeaponDef = thingDef.GetRealWeaponDef();
						bool flag5 = thingDef2 != null;
						if (flag5)
						{
							bool flag6 = thingDef2.projectile != null;
							if (flag6)
							{
								thingDef2.projectile.speed = this.bulletSpeed;
								thingDef2.projectile.stoppingPower = this.bulletStoppingPower;
								thingDef2.projectile.explosionRadius = this.bulletExplosionRadius;
							}
							thingDef2.ResolveReferences();
						}
						bool flag7 = realWeaponDef != null;
						if (flag7)
						{
							realWeaponDef.label = this.label;
							bool flag8 = realWeaponDef.Verbs.Count > 0;
							if (flag8)
							{
								realWeaponDef.SetBullet(thingDef2);
								realWeaponDef.SetBulletSpeed(this.bulletSpeed);
								realWeaponDef.SetStoppingPower(this.bulletStoppingPower);
								realWeaponDef.SetExplosionRadius(this.bulletExplosionRadius);
								realWeaponDef.Verbs[0].warmupTime = this.weaponWarmup;
								realWeaponDef.Verbs[0].range = this.weaponRange;
								realWeaponDef.Verbs[0].minRange = this.weaponMinRange;
								realWeaponDef.Verbs[0].burstShotCount = this.bulletCount;
								realWeaponDef.Verbs[0].ticksBetweenBurstShots = this.bulletInterval;
								realWeaponDef.Verbs[0].SetMemberValue("forcedMissRadius", this.weaponScatter);
								realWeaponDef.SetTargetParams(this.bTargetGround);
								realWeaponDef.Verbs[0].soundCast = soundCast;
								realWeaponDef.techLevel = (TechLevel)Enum.Parse(typeof(TechLevel), this.techLevel);
								realWeaponDef.tradeability = (Tradeability)Enum.Parse(typeof(Tradeability), this.tradeability);
								realWeaponDef.stealable = (this.stealable.ToLower() == "true");
								realWeaponDef.costStuffCount = this.costStuffCount.AsInt32();
								realWeaponDef.stuffCategories = list;
								realWeaponDef.statBases = list2;
								realWeaponDef.equippedStatOffsets = equippedStatOffsets;
								realWeaponDef.weaponTags = this.tags.StringToList();
								StatDef statDef = DefTool.StatDef("ShotSpread");
								bool flag9 = statDef != null;
								if (flag9)
								{
									realWeaponDef.UpdateStat(statDef, this.weaponScatter);
								}
							}
							WeaponTool.SetCompRefuelable(null, realWeaponDef, this.weaponFuelCapacity);
							WeaponTool.SetCompReloadTime(null, realWeaponDef, this.weaponCEReload);
							realWeaponDef.ResolveReferences();
						}
						bool flag10 = turretDef != null;
						if (flag10)
						{
							bool flag11 = turretDef.building != null;
							if (flag11)
							{
								//turretDef.building.turretBurstWarmupTime = this.weaponWarmup;
								turretDef.building.turretBurstCooldownTime = this.weaponCooldown;
							}
							WeaponTool.SetCompRefuelable(turretDef, realWeaponDef, this.weaponFuelCapacity);
							WeaponTool.SetCompReloadTime(turretDef, realWeaponDef, this.weaponCEReload);
							turretDef.ResolveReferences();
						}
						result = true;
					}
				}
				else
				{
					result = false;
				}
			}
			return result;
		}

		// Token: 0x060004E3 RID: 1251 RVA: 0x00031A96 File Offset: 0x0002FC96
		public void SaveCustom()
		{
			CEditor.API.SetCustom(OptionS.CUSTOMWEAPON, this.AsString, this.defName);
		}

		// Token: 0x060004E4 RID: 1252 RVA: 0x00031AB1 File Offset: 0x0002FCB1
		public void Reset()
		{
			CEditor.API.SetCustom(OptionS.CUSTOMWEAPON, "", this.defName);
		}

		// Token: 0x060004E5 RID: 1253 RVA: 0x00031ACC File Offset: 0x0002FCCC
		public void UpdateFromBullet(ThingDef bullet)
		{
			bool flag = bullet != null;
			if (flag)
			{
				this.bulletExplosionRadius = bullet.projectile.explosionRadius;
				this.bulletStoppingPower = bullet.projectile.stoppingPower;
				this.bulletSpeed = bullet.projectile.speed;
				this.bulletName = bullet.defName;
			}
		}

		// Token: 0x060004E6 RID: 1254 RVA: 0x00031B24 File Offset: 0x0002FD24
		public void UpdateFromSound(SoundDef sound)
		{
			bool flag = sound != null;
			if (flag)
			{
				this.soundName = sound.defName;
			}
		}

		// Token: 0x060004E7 RID: 1255 RVA: 0x00031B48 File Offset: 0x0002FD48
		public void FromPreset(PresetWeapon p)
		{
			bool flag = p != null;
			if (flag)
			{
				this.defName = p.defName;
				this.label = p.label;
				this.bulletName = p.bulletName;
				this.soundName = p.soundName;
				this.techLevel = p.techLevel;
				this.tradeability = p.tradeability;
				this.stealable = p.stealable;
				this.costStuffCount = p.costStuffCount;
				this.stuffCategories = p.stuffCategories;
				this.statBases = p.statBases;
				this.equippedStats = p.equippedStats;
				this.tags = p.tags;
				this.bTargetGround = p.bTargetGround;
				this.bulletExplosionRadius = p.bulletExplosionRadius;
				this.bulletStoppingPower = p.bulletStoppingPower;
				this.bulletSpeed = p.bulletSpeed;
				this.bulletCount = p.bulletCount;
				this.bulletInterval = p.bulletInterval;
				this.weaponCooldown = p.weaponCooldown;
				this.accuracyTouch = p.accuracyTouch;
				this.accuracyShort = p.accuracyShort;
				this.accuracyMedium = p.accuracyMedium;
				this.accuracyLong = p.accuracyLong;
				this.weaponWarmup = p.weaponWarmup;
				this.weaponMinRange = p.weaponMinRange;
				this.weaponRange = p.weaponRange;
				this.weaponScatter = p.weaponScatter;
				this.weaponFuelCapacity = p.weaponFuelCapacity;
				this.weaponCEReload = p.weaponCEReload;
			}
		}

		// Token: 0x060004E8 RID: 1256 RVA: 0x00031CC0 File Offset: 0x0002FEC0
		public static void LoadCustomString(string custom)
		{
			bool flag = !string.IsNullOrEmpty(custom);
			if (flag)
			{
				try
				{
					string text = custom.Trim();
					string[] array = text.Split(new string[]
					{
						";"
					}, StringSplitOptions.RemoveEmptyEntries);
					foreach (string text2 in array)
					{
						string[] array3 = text2.Split(new string[]
						{
							","
						}, StringSplitOptions.None);
						bool flag2 = array3.Length >= 25;
						if (flag2)
						{
							int num = -1;
							PresetWeapon presetWeapon = new PresetWeapon();
							presetWeapon.defName = array3[++num];
							presetWeapon.label = array3[++num];
							presetWeapon.bulletName = array3[++num];
							presetWeapon.soundName = array3[++num];
							presetWeapon.techLevel = array3[++num];
							presetWeapon.tradeability = array3[++num];
							presetWeapon.stealable = array3[++num];
							presetWeapon.costStuffCount = array3[++num];
							presetWeapon.stuffCategories = array3[++num];
							presetWeapon.statBases = array3[++num];
							presetWeapon.equippedStats = array3[++num];
							presetWeapon.tags = array3[++num];
							presetWeapon.bTargetGround = array3[++num].AsBool();
							presetWeapon.bulletExplosionRadius = array3[++num].AsFloat();
							presetWeapon.bulletStoppingPower = array3[++num].AsFloat();
							presetWeapon.bulletSpeed = array3[++num].AsFloat();
							presetWeapon.bulletCount = array3[++num].AsInt32();
							presetWeapon.bulletInterval = array3[++num].AsInt32();
							presetWeapon.weaponCooldown = array3[++num].AsFloat();
							presetWeapon.weaponWarmup = array3[++num].AsFloat();
							presetWeapon.weaponMinRange = array3[++num].AsFloat();
							presetWeapon.weaponRange = array3[++num].AsFloat();
							presetWeapon.weaponScatter = array3[++num].AsFloat();
							presetWeapon.weaponFuelCapacity = array3[++num].AsInt32();
							presetWeapon.weaponCEReload = array3[num + 1].AsFloat();
							presetWeapon.UpdateWeapon();
							bool devMode = Prefs.DevMode;
							if (devMode)
							{
								////Log.Message(presetWeapon.label + " " + Label.MODIFICATIONLOADED);
							}
						}
					}
				}
				catch (Exception ex)
				{
					bool devMode2 = Prefs.DevMode;
					if (devMode2)
					{
						Log.Error(ex.Message + "\n" + ex.StackTrace);
					}
				}
			}
		}

		// Token: 0x060004E9 RID: 1257 RVA: 0x00031FC0 File Offset: 0x000301C0
		public static void ResetToDefault(string defName)
		{
			PresetWeapon presetWeapon;
			bool flag = CEditor.API.Get<Dictionary<string, PresetWeapon>>(EType.WeaponPreset).TryGetValue(defName, out presetWeapon);
			if (flag)
			{
				presetWeapon.UpdateWeapon();
				presetWeapon.Reset();
			}
		}

		// Token: 0x060004EA RID: 1258 RVA: 0x00031FF8 File Offset: 0x000301F8
		public static Dictionary<string, PresetWeapon> GetAllPresets()
		{
			Dictionary<string, PresetWeapon> dictionary = new Dictionary<string, PresetWeapon>();
			List<ThingDef> list = (from td in DefDatabase<ThingDef>.AllDefs
			where td.IsWeapon && !td.label.NullOrEmpty()
			select td).ToList<ThingDef>();
			bool flag = list.NullOrEmpty<ThingDef>();
			Dictionary<string, PresetWeapon> result;
			if (flag)
			{
				result = dictionary;
			}
			else
			{
				foreach (ThingDef thingDef in list)
				{
					bool flag2 = !thingDef.defName.NullOrEmpty();
					if (flag2)
					{
						thingDef.AddCompColorable();
						dictionary.Add(thingDef.defName, new PresetWeapon(thingDef));
					}
				}
				result = dictionary;
			}
			return result;
		}

		// Token: 0x060004EB RID: 1259 RVA: 0x000320C4 File Offset: 0x000302C4
		public static Dictionary<string, ThingDef> GetAllTurrets()
		{
			Dictionary<string, ThingDef> dictionary = new Dictionary<string, ThingDef>();
			List<ThingDef> list = WeaponTool.ListOfTurrets();
			bool flag = list.NullOrEmpty<ThingDef>();
			Dictionary<string, ThingDef> result;
			if (flag)
			{
				result = dictionary;
			}
			else
			{
				foreach (ThingDef thingDef in list)
				{
					bool flag2 = thingDef != null && !thingDef.defName.NullOrEmpty() && thingDef.building != null && thingDef.building.turretGunDef != null;
					if (flag2)
					{
						dictionary.Add(thingDef.defName, thingDef.building.turretGunDef);
					}
				}
				result = dictionary;
			}
			return result;
		}

		// Token: 0x04000451 RID: 1105
		public string label = "";

		// Token: 0x04000452 RID: 1106
		public string defName = "";

		// Token: 0x04000453 RID: 1107
		public string bulletName = "";

		// Token: 0x04000454 RID: 1108
		public string soundName = "";

		// Token: 0x04000455 RID: 1109
		public string techLevel = "";

		// Token: 0x04000456 RID: 1110
		public string tradeability = "";

		// Token: 0x04000457 RID: 1111
		public string stealable = "";

		// Token: 0x04000458 RID: 1112
		public string costStuffCount = "";

		// Token: 0x04000459 RID: 1113
		public string stuffCategories = "";

		// Token: 0x0400045A RID: 1114
		public string statBases = "";

		// Token: 0x0400045B RID: 1115
		public string equippedStats = "";

		// Token: 0x0400045C RID: 1116
		public string tags = "";

		// Token: 0x0400045D RID: 1117
		public bool bTargetGround = false;

		// Token: 0x0400045E RID: 1118
		public float bulletExplosionRadius = 0f;

		// Token: 0x0400045F RID: 1119
		public float bulletStoppingPower = 0f;

		// Token: 0x04000460 RID: 1120
		public float bulletSpeed = 0f;

		// Token: 0x04000461 RID: 1121
		public int bulletCount = 0;

		// Token: 0x04000462 RID: 1122
		public int bulletInterval = 0;

		// Token: 0x04000463 RID: 1123
		public float weaponCooldown = 0f;

		// Token: 0x04000464 RID: 1124
		public float accuracyTouch = 0f;

		// Token: 0x04000465 RID: 1125
		public float accuracyShort = 0f;

		// Token: 0x04000466 RID: 1126
		public float accuracyMedium = 0f;

		// Token: 0x04000467 RID: 1127
		public float accuracyLong = 0f;

		// Token: 0x04000468 RID: 1128
		public float weaponWarmup = 0f;

		// Token: 0x04000469 RID: 1129
		public float weaponMinRange = 0f;

		// Token: 0x0400046A RID: 1130
		public float weaponRange = 0f;

		// Token: 0x0400046B RID: 1131
		public float weaponScatter = 0f;

		// Token: 0x0400046C RID: 1132
		public int weaponFuelCapacity = 0;

		// Token: 0x0400046D RID: 1133
		public float weaponCEReload = 0f;
	}
}
