﻿using System;
using System.Linq;
using System.Reflection;
using System.Text;
using Verse;
using HarmonyLib;
using Unity.Jobs.LowLevel.Unsafe;
using UnityEngine;

namespace CharacterEditor
{
  // Token: 0x02000032 RID: 50
  public static class Reflect
  {
    // Token: 0x17000043 RID: 67
    // (get) Token: 0x06000258 RID: 600 RVA: 0x00011664 File Offset: 0x0000F864
    public static string APP_NAME_AND_VERISON
    {
      get
      {
        StringBuilder stringBuilder = new StringBuilder();
        Version version = Assembly.GetExecutingAssembly().GetName().Version;
        stringBuilder.Append(Reflect.APP_ATTRIBUTE_TITLE);
        stringBuilder.Append(" v");
        stringBuilder.Append(version.Major.ToString());
        stringBuilder.Append(".");
        stringBuilder.Append(version.Minor.ToString());
        stringBuilder.Append(".");
        stringBuilder.Append(version.Build.ToString());
        return stringBuilder.ToString();
      }
    }

    // Token: 0x17000044 RID: 68
    // (get) Token: 0x06000259 RID: 601 RVA: 0x00011704 File Offset: 0x0000F904
    public static string APP_ATTRIBUTE_TITLE
    {
      get
      {
        return ((AssemblyTitleAttribute)Attribute.GetCustomAttribute(Assembly.GetExecutingAssembly(), typeof(AssemblyTitleAttribute), false)).Title;
      }
    }

    // Token: 0x0600025A RID: 602 RVA: 0x00011738 File Offset: 0x0000F938
    public static Type ByName(string name)
    {
      foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies().Reverse<Assembly>())
      {
        Type type = assembly.GetType(name);
        bool flag = type != null;
        if (flag)
        {
          return type;
        }
      }
      return null;
    }

    public static object GetMemberConstValue(this object obj, string name)
    {
      FieldInfo fieldInfo = obj != null ? obj.GetType().GetField(name) : null;
      return fieldInfo != null ? fieldInfo.GetValue(null) : null;
    }

    public static object GetMemberValue(this object obj, string name)
    {
      if (obj == null)
      {
        Log.Error("GetMemberValue, obj was null, field name " + name);
        return null;
      }
      return obj.GetType().GetMemberValue(name, obj);
    }

    public static object GetMemberValue(this Type type, string name, object instance = null)
    {
      BindingFlags bindingAttr = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
      FieldInfo fieldInfo = type.GetField(name, bindingAttr);
      if (fieldInfo == null)
      {
        fieldInfo = AccessTools.Field(type, name);
      }
      if (fieldInfo == null)
      {
        Log.Error("GetMemberValue, fieldInfo is null, field name " + name);
        return null;
      }
      return fieldInfo.GetValue(instance);
      //BindingFlags bindingAttr = BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic;
      //FieldInfo fieldInfo = (type != null) ? type.GetField(name, bindingAttr) : null;
      //return (fieldInfo != null) ? fieldInfo.GetValue(null) : null;
    }

    public static void SetMemberValue(this object obj, string name, object value)
    {
      if (obj == null)
      {
        Log.Error("SetMemberValue, object is null for field name " + name);
        return;
      }
      obj.GetType().SetMemberValue(name, value, obj);
      //BindingFlags bindingAttr = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
      //Type objType = obj.GetType();
      //FieldInfo fieldInfo = (obj != null) ? objType.GetField(name, bindingAttr) : null;
      //if (fieldInfo == null)
      //{
      //  fieldInfo = AccessTools.Field(objType, name);
      //}
      //if (fieldInfo != null)
      //{
      //  fieldInfo.SetValue(obj, value);
      //}
    }

    public static void SetMemberValue(this Type type, string name, object value, object obj = null)
    {
      if (type == null)
      {
        Log.Error("SetMemberValue, type is null for field name " + name);
        return;
      }
      BindingFlags bindingAttr = BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic;
      FieldInfo fieldInfo = type.GetField(name, bindingAttr);
      if (fieldInfo == null)
      {
        //Log.Warning("SetMemberValue, using AccessTools to find field name " + name);
        fieldInfo = AccessTools.Field(type, name);
      }
      if (fieldInfo == null)
      {
        Log.Error("SetMemberValue, failed to find fieldInfo for field name " + name);
        return;
      }
      fieldInfo.SetValue(obj, value);
    }

    public static object CallMethod(this object obj, string name, object[] param, bool silent = false)
    {
      if (obj == null)
      {
        Log.Error("CallMethod, object is null for method name " + name);
        return null;
      }
      return obj.GetType().CallMethod(name, param, obj, null, silent);
    }

    public static object CallMethod(this Type type, string name, object[] param, object instance = null, Type[] paramTypes = null, bool silent = false)
    {
      bool foundMethodAccessTools = false;
      if (type == null)
      {
        Log.ErrorOnce("CallMethod, type is null for method name " + name, name.GetHashCode() + 123);
        return null;
      }
      object result = null;
      try
      {
        //Log.Message("CallMethod, AccessTools start");
        MethodInfo methodAccessTools = null;
        if (param.NullOrEmpty())
        {
          methodAccessTools = AccessTools.Method(type, name);
        }
        else if (paramTypes.NullOrEmpty())
        {
          //Log.Message("CallMethod, AccessTools, construct Type array");
          paramTypes = new Type[param.Count()];
          for (int i = 0; i < param.Count(); i++)
          {
            paramTypes[i] = param[i] != null ? param[i].GetType() : typeof(object);
          }
          methodAccessTools = AccessTools.Method(type, name, paramTypes);
        }
        else if (param.Count() == paramTypes.Count())
        {
          methodAccessTools = AccessTools.Method(type, name, paramTypes);
        }
        else
        {
          Log.Warning("CallMethod, AccessTools, param array and paramTypes array not the same length");
          methodAccessTools = AccessTools.Method(type, name);
        }
        if (methodAccessTools != null)
        {
          //Log.Message("CallMethod, AccessTools, call on non-null MethodInfo");
          result = methodAccessTools.Invoke(instance, param);
          foundMethodAccessTools = true;
        }
        else
        {
          //Log.Message("CallMethod, AccessTools, MethodInfo was null");
        }
      }
      catch (Exception ex)
      {
        Log.ErrorOnce("CallMethod, AccessTools exception for method name " + name + ": " + ex, name.GetHashCode() + ex.GetHashCode());
      }
      if (result != null)
      {
        //Log.Message("CallMethod, AccessTools were successful");
        return result;
      }
      ////Log.Message("CallMethod, AccessTools gave null result");
      //string foundMessage = "CallMethod, AccessTools found method successfully but returned null object for name " + name;
      //string notFoundMessage = "CallMethod, AccessTools could not find method successfully for name " + name;
      if (!foundMethodAccessTools)
      {
        //Log.Warning("CallMethod, AccessTools could not find method successfully for name " + name);
      }
      else if (!silent)
      {
        //Log.Warning("CallMethod, AccessTools found method successfully but returned null object for name " + name);
      }
      bool succesSystemMethods = false;
      try
      {
        //Log.Message("CallMethod, System tools, start");
        BindingFlags bindingAttr = BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic;
        //Log.Message("CallMethod, System tools, attempt to find method info");
        MethodInfo method = type.GetMethod(name, bindingAttr);
        if (method != null)
        {
          //Log.Message("CallMethod, System tools, found non-null method info, invoking");
          result = method.Invoke(instance, param);
          succesSystemMethods = true;
          //Log.Message("CallMethod, System tools, success");
        }
      }
      catch (Exception ex2)
      {
        Log.ErrorOnce("CallMethod, System methods excpetion for method name " + name + ": " + ex2, name.GetHashCode() + ex2.GetHashCode());
      }
      if (result != null)
      {
        return result;
      }
      if (succesSystemMethods)
      {
        //Log.Warning("CallMethod, System tools found method successfully but returned null object for name " + name);
        return result;
      }
      Log.ErrorOnce("CallMethod, System tools could not find method successfully for name " + name + ", returning null", name.GetHashCode());
      return result;
    }

    public static object CallMethod(this MethodInfo mi, object[] param, object instance = null)
    {
      if (mi == null)
      {
        Log.Error("CallMethod, methodInfo was null");
        return null;
      }
      return mi.Invoke(instance, param);
    }

    // Token: 0x06000261 RID: 609 RVA: 0x000118E8 File Offset: 0x0000FAE8
    //public static object CallMethodAmbiguous(this object obj, string name, object[] param)
    //{
    //  if (obj == null)
    //  {
    //    return null;
    //  }
    //  Type objType = obj.GetType();
    //  try
    //  {
    //    MethodInfo methodAccessTools = null;
    //    if (!param.NullOrEmpty())
    //    {
    //      Type[] types = new Type[param.Count()];
    //      for (int i = 0; i < param.Count(); i++)
    //      {
    //        types[i] = param[i].GetType();
    //      }
    //      methodAccessTools = AccessTools.Method(objType, name, types);
    //    }
    //    else
    //    {
    //      methodAccessTools = AccessTools.Method(objType, name);
    //    }
    //    if (methodAccessTools != null)
    //    {
    //      return methodAccessTools.Invoke(obj, param);
    //    }
    //  }
    //  catch
    //  {
    //    Log.Warning("Ambigious");
    //  }
    //  BindingFlags bindingAttr = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
    //  MethodInfo[] array = (obj != null) ? obj.GetType().GetMethods(bindingAttr) : null;
    //  foreach (MethodInfo methodInfo in array)
    //  {
    //    if (methodInfo.Name == name && methodInfo.GetParameters().Length == param.Length)
    //    {
    //      return (methodInfo != null) ? methodInfo.Invoke(obj, param) : null;
    //    }
    //  }
    //  return null;
    //}

    public static object GetProperty(this object obj, string name)
    {
      if (obj == null)
      {
        Log.Warning("GetProperty, instance object was null, returning null");
        return null;
      }
      return obj.GetType().GetProperty(name, obj);
    }

    public static object GetProperty(this Type type, string name, object instance = null)
    {
      MethodInfo propertyGetter = AccessTools.PropertyGetter(type, name);
      if (propertyGetter != null)
      {
        return propertyGetter.CallMethod(null, instance);
      }
      else
      {
        Log.Warning("GetProperty, method info was null, returning null");
        return null;
      }
    }

    public static void SetProperty(this object obj, string name, object value)
    {
      if (obj == null)
      {
        Log.Warning("SetProperty, instance object was null");
        return;
      }
      obj.GetType().SetProperty(name, value, obj);
    }

    public static void SetProperty(this Type type, string name, object value, object instance = null)
    {
      MethodInfo propertyGetter = AccessTools.PropertySetter(type, name);
      if (propertyGetter != null)
      {
        propertyGetter.CallMethod(new object[] { value }, instance);
      }
      else
      {
        Log.Warning("SetProperty, method info was null");
      }
    }


    public static MethodInfo GetMethodInfo(this Type type, string name)
    {
      if (type == null)
      {
        return null;
      }
      BindingFlags bindingAttr = BindingFlags.Static | BindingFlags.Public | BindingFlags.NonPublic;
      MethodInfo method = type.GetMethod(name, bindingAttr);
      if (method != null)
      {
        return method;
      }
      return AccessTools.Method(type, name);
    }

    public static MethodInfo GetMethodInfo(this object obj, string name)
    {
      if (obj == null)
      {
        Log.Warning("GetMethodInfo, object was null");
        return null;
      }
      return obj.GetType().GetMethodInfo(name);
      //BindingFlags bindingAttr = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic;
      //return (obj != null) ? obj.GetType().GetMethod(name, bindingAttr) : null;
    }

    public static Type GetAssemblyType(string name, string type)
    {
      Assembly assembly2 = AppDomain.CurrentDomain.GetAssemblies().SingleOrDefault((Assembly assembly) => assembly.GetName().Name == name);
      return assembly2.GetType(type);
    }

    public static Type GetAType(string nameSpace, string className)
    {
      return GenTypes.GetTypeInAnyAssembly(nameSpace + "." + className, nameSpace);
    }
  }
}
