﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000022 RID: 34
	public static class RelationTool
	{
		// Token: 0x0600017B RID: 379 RVA: 0x0000BBB8 File Offset: 0x00009DB8
		public static bool AreThosePawnSisBro(Pawn a, Pawn b)
		{
			Pawn commonParent = RelationTool.GetCommonParent(a, b, Gender.Male);
			Pawn commonParent2 = RelationTool.GetCommonParent(a, b, Gender.Female);
			return commonParent != null && commonParent2 != null;
		}

		// Token: 0x0600017C RID: 380 RVA: 0x0000BBE8 File Offset: 0x00009DE8
		public static Pawn GetCommonParent(Pawn a, Pawn b, Gender gender)
		{
			Pawn pawn = null;
			bool flag = a == null || b == null || a.relations == null || b.relations == null;
			Pawn result;
			if (flag)
			{
				result = pawn;
			}
			else
			{
				foreach (DirectPawnRelation directPawnRelation in a.relations.DirectRelations)
				{
					bool flag2 = directPawnRelation.def == PawnRelationDefOf.Parent;
					if (flag2)
					{
						foreach (DirectPawnRelation directPawnRelation2 in b.relations.DirectRelations)
						{
							bool flag3 = directPawnRelation2.def == PawnRelationDefOf.Parent;
							if (flag3)
							{
								bool flag4 = directPawnRelation.otherPawn == directPawnRelation2.otherPawn;
								if (flag4)
								{
									bool flag5 = directPawnRelation.otherPawn.gender == gender;
									if (flag5)
									{
										return pawn;
									}
								}
							}
						}
					}
				}
				result = null;
			}
			return result;
		}

		// Token: 0x0600017D RID: 381 RVA: 0x0000BD18 File Offset: 0x00009F18
		public static Pawn GetFirstParentForPawn(this Pawn p, Gender g)
		{
			bool flag = p == null;
			Pawn result;
			if (flag)
			{
				result = null;
			}
			else
			{
				foreach (Pawn pawn in p.relations.RelatedPawns)
				{
					bool flag2 = pawn.gender == g;
					if (flag2)
					{
						DirectPawnRelation directRelation = p.relations.GetDirectRelation(PawnRelationDefOf.Parent, pawn);
						bool flag3 = directRelation != null;
						if (flag3)
						{
							return pawn;
						}
					}
				}
				result = null;
			}
			return result;
		}

		// Token: 0x0600017E RID: 382 RVA: 0x0000BDB0 File Offset: 0x00009FB0
		public static List<Pawn> GetRelatedPawns(this Pawn p, out int countImpliedByOtherPawn)
		{
			countImpliedByOtherPawn = 0;
			List<Pawn> list = new List<Pawn>();
			bool flag = !p.HasRelationTracker() || p.relations.RelatedPawns.EnumerableNullOrEmpty<Pawn>();
			List<Pawn> result;
			if (flag)
			{
				result = list;
			}
			else
			{
				foreach (Pawn pawn in p.relations.RelatedPawns)
				{
					list.Add(pawn);
					foreach (PawnRelationDef pawnRelationDef in p.GetRelations(pawn).ToList<PawnRelationDef>())
					{
						bool implied = pawnRelationDef.implied;
						if (implied)
						{
							countImpliedByOtherPawn++;
						}
					}
				}
				result = list;
			}
			return result;
		}

		// Token: 0x0600017F RID: 383 RVA: 0x0000BE9C File Offset: 0x0000A09C
		public static string GetRelationAsSeparatedString(this DirectPawnRelation d)
		{
			bool flag = d == null || d.def.IsNullOrEmpty();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				text = text + d.def.defName + "|";
				text = text + d.otherPawn.GetPawnNameAsSeparatedString() + "|";
				text += d.startTicks.ToString();
				result = text;
			}
			return result;
		}

		// Token: 0x06000180 RID: 384 RVA: 0x0000BF14 File Offset: 0x0000A114
		public static string GetRelationsAsSeparatedString(this Pawn p)
		{
			bool flag = !p.HasRelationTracker() || p.relations.DirectRelations.NullOrEmpty<DirectPawnRelation>();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				foreach (DirectPawnRelation d in p.relations.DirectRelations)
				{
					text += d.GetRelationAsSeparatedString();
					text += ":";
				}
				text = text.SubstringRemoveLast();
				result = text;
			}
			return result;
		}

		// Token: 0x06000181 RID: 385 RVA: 0x0000BFC0 File Offset: 0x0000A1C0
		public static string RelationLabelDirect(this DirectPawnRelation dr)
		{
			bool flag = dr == null || dr.otherPawn == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = ((dr.otherPawn.gender == Gender.Female) ? dr.def.labelFemale : dr.def.label) + " " + dr.otherPawn.GetPawnName(true);
				result = text;
			}
			return result;
		}

		// Token: 0x06000182 RID: 386 RVA: 0x0000C02C File Offset: 0x0000A22C
		public static string RelationLabelIndirect(this PawnRelationDef prd, Pawn otherPawn)
		{
			bool flag = prd == null || otherPawn == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = ((otherPawn.gender == Gender.Female) ? prd.labelFemale : prd.label) + " " + otherPawn.GetPawnName(true);
				result = text;
			}
			return result;
		}

		// Token: 0x06000183 RID: 387 RVA: 0x0000C080 File Offset: 0x0000A280
		public static string RelationTooltip(this Pawn pawn, Pawn otherPawn)
		{
			return otherPawn.GetPawnDescription(pawn) + "\n" + RelationTool.GetPawnRowTooltip(otherPawn, pawn);
		}

		// Token: 0x06000184 RID: 388 RVA: 0x0000C0AC File Offset: 0x0000A2AC
		public static void SetRelationsFromSeparatedString(this Pawn p, string s)
		{
			bool flag = s.NullOrEmpty() || !p.HasRelationTracker();
			if (!flag)
			{
				string[] array = s.SplitNo(":");
				p.relations.ClearAllRelations();
				foreach (string s2 in array)
				{
					string[] array3 = s2.SplitNo("|");
					bool flag2 = array3.Length == 3;
					if (flag2)
					{
						PawnRelationDef pawnRelationDef = DefTool.PawnRelationDef(array3[0]);
						bool flag3 = pawnRelationDef != null;
						if (flag3)
						{
							Pawn otherPawnFromSeparatedString = PawnxTool.GetOtherPawnFromSeparatedString(array3[1]);
							bool flag4 = otherPawnFromSeparatedString != null;
							if (flag4)
							{
								p.relations.AddDirectRelation(pawnRelationDef, otherPawnFromSeparatedString);
								bool flag5 = p.relations.DirectRelationExists(pawnRelationDef, otherPawnFromSeparatedString);
								if (flag5)
								{
									DirectPawnRelation directPawnRelation = p.relations.DirectRelations.Last<DirectPawnRelation>();
									bool flag6 = directPawnRelation.def.defName == pawnRelationDef.defName && directPawnRelation.otherPawn == otherPawnFromSeparatedString;
									if (flag6)
									{
										directPawnRelation.startTicks = array3[2].AsInt32();
									}
								}
							}
						}
					}
				}
			}
		}

		// Token: 0x06000185 RID: 389 RVA: 0x0000C1D8 File Offset: 0x0000A3D8
		private static string GetPawnRowTooltip(Pawn otherPawn, Pawn pawn)
		{
			StringBuilder stringBuilder = new StringBuilder();
			bool flag = otherPawn.RaceProps.Humanlike && pawn.RaceProps.Humanlike;
			if (flag)
			{
				stringBuilder.AppendLine(pawn.relations.OpinionExplanation(otherPawn));
				stringBuilder.AppendLine();
				stringBuilder.Append("SomeonesOpinionOfMe".Translate(otherPawn.LabelShort, otherPawn));
				stringBuilder.Append(": ");
				stringBuilder.Append(otherPawn.relations.OpinionOf(pawn).ToStringWithSign());
			}
			else
			{
				stringBuilder.AppendLine(otherPawn.LabelCapNoCount);
				string pawnSituationLabel = SocialCardUtility.GetPawnSituationLabel(otherPawn, pawn);
				bool flag2 = !pawnSituationLabel.NullOrEmpty();
				if (flag2)
				{
					stringBuilder.AppendLine(pawnSituationLabel);
				}
				stringBuilder.AppendLine("--------------");
				string text = "";
				bool flag3 = otherPawn.relations.DirectRelations.Count == 0;
				if (flag3)
				{
					bool flag4 = otherPawn.relations.OpinionOf(pawn) < -20;
					if (flag4)
					{
						return "Rival".Translate();
					}
					return (otherPawn.relations.OpinionOf(pawn) > 20) ? "Friend".Translate() : "Acquaintance".Translate();
				}
				else
				{
					for (int i = 0; i < otherPawn.relations.DirectRelations.Count; i++)
					{
						PawnRelationDef def = otherPawn.relations.DirectRelations[i].def;
						text = (text.NullOrEmpty() ? def.GetGenderSpecificLabelCap(otherPawn) : (text + ", " + def.GetGenderSpecificLabel(otherPawn)));
					}
					stringBuilder.Append(text);
				}
			}
			stringBuilder.AppendLine();
			stringBuilder.AppendLine("Compatibility: " + pawn.relations.CompatibilityWith(otherPawn).ToString("F2"));
			stringBuilder.Append("RomanceChanceFactor: " + pawn.relations.SecondaryRomanceChanceFactor(otherPawn).ToString("F2"));
			return stringBuilder.ToString();
		}
	}
}
