﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Noise;
using Verse.Sound;
using System.Linq;
using UnityEngine.UIElements;

namespace CharacterEditor
{
  // Token: 0x0200005B RID: 91
  public static class SZWidgets
  {
    public static bool bCountOpen = false;
    public static bool bQualityOpen = false;
    public static bool bStuffOpen = false;
    public static bool bStyleOpen = false;
    public static List<string> lSimilar = new List<string>();
    public static string sFind = "";
    public static string sFindOld = "";
    public static string sFindTemp = "";
    public static string tDefName = null;
    public static int waitTimer = 0;
    public static bool bToggleSearch = false;
    public static Rect rectLabelBackground;
    public static QuickSearchWidget quickSearchWidget = new QuickSearchWidget();
    public static string filterTextCached = "";
    public static int tickerSelected = 0;
    //public static readonly Color toggleColor = GenUI.SubtleMouseoverColor;
    public static readonly Color toggleColor = Color.gray;

    public static bool LogFocus = false;
    public static bool focusStateCached = false;

    public static Rect cachedSearchRect = new Rect();

    public static void GeneralButton(Rect rect, Action action, bool buttonPressed, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      if (buttonPressed)
      {
        action();
      }
      TooltipSafe(rect, toolTip);
    }

    public static void GeneralButton<T>(Rect rect, Action<T> action, T value, bool buttonPressed, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      if (buttonPressed)
      {
        action(value);
      }
      TooltipSafe(rect, toolTip);
    }

    public static void TooltipSafe(Rect rect, string toolTip = "")
    {
      if (!toolTip.NullOrEmpty())
      {
        TooltipHandler.TipRegion(rect, toolTip);
      }
    }



    public static void ButtonImage(float x, float y, float w, float h, string texPath, Action action, string toolTip = "", bool toggle = false)
    {
      Rect rect = new Rect(x, y, w, h);
      //ButtonImage(rect, texPath, action, toolTip, toggle);
      ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), action, toolTip, toggle);
    }

    public static void ButtonImage(Rect rect, string texPath, Action action, string toolTip = "", bool toggle = false)
    {
      if (texPath.NullOrEmpty())
      {
        return;
      }
      ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), action, toolTip, toggle);
    }

    public static void ButtonImage(Rect rect, Texture2D tex, Action action, string toolTip = "", bool toggle = false)
    {
      bool buttonPressed = Widgets.ButtonImage(rect, tex, BaseColor(toggle), true);
      GeneralButton(rect, action, buttonPressed, toolTip);
    }

    public static void ButtonImage(Rect rect, string texPath, ref bool toggle, string toolTip = "")
    {
      ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), ref toggle, toolTip);
    }

    public static void ButtonImage(Rect rect, Texture2D tex, ref bool toggle, string toolTip = "")
    {
      if (SZWidgets.ButtonImage(rect, tex, BaseColor(toggle), true))
      {
        toggle = !toggle;
      }
      TooltipSafe(rect, toolTip);
    }

    public static bool ButtonImage(Rect butRect, Texture2D tex, Color baseColor, bool doMouseoverSound = true)
    {
      if (tex != null)
      {
        GUI.color = Mouse.IsOver(butRect) ? GenUI.MouseoverColor : baseColor;
        GUI.DrawTexture(butRect, tex);
      }
      GUI.color = Color.white;
      return Widgets.ButtonInvisible(butRect, doMouseoverSound);
    }

    public static Color BaseColor(bool toggle)
    {
      return toggle ? ColorTool.colGray : Color.white;
    }

    public static void ButtonImageCol(Rect rect, string texPath, Action action, Color color, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      bool buttonPressed = Widgets.ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), color, true);
      GeneralButton(rect, action, buttonPressed, toolTip);
      //if (Widgets.ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), color, true))
      //{
      //  action();
      //}
      //if (!toolTip.NullOrEmpty())
      //{
      //  TooltipHandler.TipRegion(rect, toolTip);
      //}
    }

    public static void ButtonImageCol(float x, float y, float w, float h, string texPath, Action<Color> action, Color color, string toolTip = "")
    {
      Rect rect = new Rect(x, y, w, h);
      ButtonImageCol(rect, texPath, action, color, toolTip);
    }

    public static void ButtonImageCol(Rect rect, string texPath, Action<Color> action, Color color, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      bool buttonPressed = Widgets.ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), color, true);
      GeneralButton(rect, action, color, buttonPressed, toolTip);
    }

    public static void ButtonImageCol2<T>(Rect rect, string texPath, Action<T> action, T value, Color color, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      bool buttonPressed = Widgets.ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), color, true);
      GeneralButton(rect, action, value, buttonPressed, toolTip);
    }

    public static void ButtonImageVar<T>(Rect rect, string texPath, Action<T> action, T value, string toolTip = "", bool toggle = false)
    {
      if (action == null)
      {
        return;
      }
      bool buttonPressed = Widgets.ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), BaseColor(toggle), true);
      GeneralButton(rect, action, value, buttonPressed, toolTip);
      //if (action != null)
      //{
      //  //if (Widgets.ButtonImage(rect, ContentFinder<Texture2D>.Get(texPath, true), BaseColor(toggle), true) && action != null)
      //  //{
      //  //  action(value);
      //  //}
      //  //if (!toolTip.NullOrEmpty())
      //  //{
      //  //  TooltipHandler.TipRegion(rect, toolTip);
      //  //}
      //}
    }

    public static void ButtonImageVar<T>(float x, float y, float w, float h, string texPath, Action<T> action, T value, string toolTip = "", bool toggle = false)
    {
      Rect rect = new Rect(x, y, w, h);
      ButtonImageVar<T>(rect, texPath, action, value, toolTip, toggle);
    }

    public static void ButtonInvisible(Rect rect, Action action, string toolTip = "")
    {
      if (action != null)
      {
        bool buttonPressed = Widgets.ButtonInvisible(rect, true);
        GeneralButton(rect, action, buttonPressed, toolTip);
      }
      //bool flag = Widgets.ButtonInvisible(rect, true);
      //if (flag)
      //{
      //  bool flag2 = action != null;
      //  if (flag2)
      //  {
      //    action();
      //  }
      //}
      //bool flag3 = !toolTip.NullOrEmpty();
      //if (flag3)
      //{
      //  TooltipHandler.TipRegion(rect, toolTip);
      //}
    }

    public static void ButtonHighlightMouseOver(Rect rect, Action action, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      if (Mouse.IsOver(rect))
      {
        Widgets.DrawHighlight(rect);
      }
      bool buttonPressed = Widgets.ButtonInvisible(rect, true);
      GeneralButton(rect, action, buttonPressed, toolTip);
      //  if (Mouse.IsOver(rect))
      //  {
      //    Widgets.DrawHighlight(rect);
      //  }
      //  if (Widgets.ButtonInvisible(rect, true))
      //  {
      //    if (action != null)
      //    {
      //      action();
      //    }
      //  }
      //  bool flag4 = !toolTip.NullOrEmpty();
      //  if (flag4)
      //  {
      //    TooltipHandler.TipRegion(rect, toolTip);
      //  }
    }

    public static void ButtonInvisibleMouseOverVar<T>(Rect rect, Action<T> action, T val, string toolTip = "")
    {
      if (Mouse.IsOver(rect) && action != null)
      {
        Widgets.DrawHighlight(rect);
      }
      ButtonInvisibleVar<T>(rect, action, val, toolTip);
    }

    public static void ButtonInvisibleVar<T>(Rect rect, Action<T> action, T value, string toolTip = "")
    {
      if (action == null)
      {
        return;
      }
      bool buttonPressed = Widgets.ButtonInvisible(rect, true);
      GeneralButton(rect, action, value, buttonPressed, toolTip);
      //if (action != null)
      //{

      //  //if (Widgets.ButtonInvisible(rect, true))
      //  //{
      //  //  action(value);
      //  //}
      //  //if (!toolTip.NullOrEmpty())
      //  //{
      //  //  TooltipHandler.TipRegion(rect, toolTip);
      //  //}
      //}
    }

    //public static void ButtonSolid(Rect rect, Color color, Action action, string tooltip = "")
    //{
    //  Widgets.DrawRectFast(rect, color, null);
    //  SZWidgets.ButtonInvisible(rect, action, tooltip);
    //  GUI.color = Color.white;
    //}

    public static void ButtonText(Rect rect, string label, Action action, string toolTip = "")
    {
      if (action == null)
      {
        //Log.Message("ButtonText, action is null for label " + label);
        return;
      }
      bool buttonPressed = Widgets.ButtonText(rect, label, true, true, true);
      GeneralButton(rect, action, buttonPressed, toolTip);
    }

    public static void ButtonText(float x, float y, float w, float h, string label, Action action, string toolTip = "")
    {
      Rect rect = new Rect(x, y, w, h);
      ButtonText(rect, label, action, toolTip);
    }

    public static void ButtonTextureTextHighlight(Rect rect, string text, Texture2D icon, Color color, Action action, string toolTip = "")
    {
      if (Mouse.IsOver(rect))
      {
        Widgets.DrawHighlight(rect);
      }
      GUI.color = color;
      GUI.DrawTexture(new Rect(rect.x, rect.y, rect.height, rect.height), icon);
      GUI.color = Color.white;
      Text.Font = GameFont.Small;
      float num = rect.height - 10f - rect.height / 2f;
      Widgets.Label(new Rect(rect.x + rect.height + 5f, rect.y + num, rect.width - rect.height - 5f, rect.height), text);
      SZWidgets.ButtonInvisible(rect, action, toolTip);
    }

    public static void ButtonTextureTextHighlight2(Rect rect, string text, string texPath, Color color, Action action, string toolTip = "", bool withButton = true)
    {
      if (Mouse.IsOver(rect))
      {
        Widgets.DrawHighlight(rect);
      }
      if (texPath != null)
      {
        GUI.color = color;
        GUI.DrawTexture(new Rect(rect.x, rect.y, rect.height, rect.height), ContentFinder<Texture2D>.Get(texPath, true));
        GUI.color = Color.white;
      }
      Text.Font = GameFont.Small;
      float num = rect.height - 10f - rect.height / 2f;
      if (texPath == null)
      {
        Widgets.Label(rect, text);
      }
      else
      {
        Widgets.Label(new Rect(rect.x + rect.height + 5f, rect.y + num, rect.width - rect.height - 5f, rect.height), text);
      }
      if (withButton)
      {
        SZWidgets.ButtonInvisible(rect, action, toolTip);
      }
    }

    public static void ButtonTextVar<T>(float x, float y, float w, float h, string label, Action<T> action, T value)
    {
      if (action != null)
      {
        Rect rect = new Rect(x, y, w, h);
        bool buttonPressed = Widgets.ButtonText(rect, label, true, true, true);
        GeneralButton(rect, action, value, buttonPressed, "");
      }
      //Rect rect = new Rect(x, y, w, h);
      //bool flag = Widgets.ButtonText(rect, label, true, true, true);
      //if (flag)
      //{
      //  bool flag2 = action != null;
      //  if (flag2)
      //  {
      //    action(value);
      //  }
      //}
    }

    public static void ButtonThingVar<T>(Rect rect, T val, Action<T> action, string tooltip)
    {
      if (Mouse.IsOver(rect))
      {
        Widgets.DrawHighlight(rect);
      }
      SZWidgets.ThingDrawer(rect, val as Thing);
      SZWidgets.ButtonInvisibleVar<T>(rect, action, val, tooltip);
    }

    public static void CheckBoxOnChange(Rect rect, string label, bool checkState, Action<bool> action)
    {
      if (action != null)
      {
        bool checkStateOld = checkState;
        Widgets.CheckboxLabeled(rect, label, ref checkState, false, null, null, false);
        if (checkStateOld != checkState)
        {
          action(checkState);
        }
      }
      //bool checkStateOld = checkState;
      //Widgets.CheckboxLabeled(rect, label, ref checkState, false, null, null, false);
      //if (checkStateOld != checkState)
      //{
      //  bool flag3 = action != null;
      //  if (flag3)
      //  {
      //    action(checkState);
      //  }
      //}
    }

    public static void ColorBox(Rect rect, Color col, Action<Color> action, bool halfAlfa = false)
    {
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect);
      GUI.color = Color.red;
      float num = listing_X.Slider(col.r, 0f, (float)ColorTool.IMAX);
      GUI.color = Color.green;
      float num2 = listing_X.Slider(col.g, 0f, (float)ColorTool.IMAX);
      GUI.color = Color.blue;
      float num3 = listing_X.Slider(col.b, 0f, (float)ColorTool.IMAX);
      GUI.color = Color.white;
      float num4 = listing_X.Slider(col.a, halfAlfa ? 0.49f : 0f, (float)ColorTool.IMAX);
      bool flag = col.r != num || col.g != num2 || col.b != num3 || col.a != num4;
      listing_X.End();
      if (flag && action != null)
      {
        action(new Color(num, num2, num3, num4));
      }
    }

    public static void FloatMenuOnButtonImage<T>(Rect rect, Texture2D tex, List<T> l, Func<T, string> labelGetter, Action<T> action, string toolTip = "")
    {
      GUI.DrawTexture(rect, tex);
      GUI.color = Color.white;
      if (Widgets.ButtonInvisible(rect, true))
      {
        SZWidgets.FloatMenuOnRect<T>(l, labelGetter, action);
      }
      TooltipSafe(rect, toolTip);
      //bool flag2 = !toolTip.NullOrEmpty();
      //if (flag2)
      //{
      //  TooltipHandler.TipRegion(rect, toolTip);
      //}
    }

    //public static void FloatMenuOnButtonInvisible<T>(Rect rect, HashSet<T> l, Func<T, string> labelGetter, Action<T> action, string toolTip = "")
    //{
    //  if (Widgets.ButtonInvisible(rect, true))
    //  {
    //    SZWidgets.FloatMenuOnRect<T>(l, labelGetter, action);
    //  }
    //  TooltipSafe(rect, toolTip);
    //  //bool flag2 = !toolTip.NullOrEmpty();
    //  //if (flag2)
    //  //{
    //  //  TooltipHandler.TipRegion(rect, toolTip);
    //  //}
    //}

    public static void FloatMenuOnButtonInvisible<T>(Rect rect, List<T> l, Func<T, string> labelGetter, Action<T> action, string toolTip = "")
    {
      if (Widgets.ButtonInvisible(rect, true))
      {
        SZWidgets.FloatMenuOnRect<T>(l, labelGetter, action);
      }
      TooltipSafe(rect, toolTip);
      //bool flag2 = !toolTip.NullOrEmpty();
      //if (flag2)
      //{
      //  TooltipHandler.TipRegion(rect, toolTip);
      //}
    }

    public static void FloatMenuOnButtonText<T>(Rect rect, string curVal, List<T> l, Func<T, string> labelGetter, Action<T> action, string toolTip = "")
    {
      //bool flag = Widgets.ButtonText(rect, curVal, true, true, true);
      if (Widgets.ButtonText(rect, curVal, true, true, true))
      {
        Log.Message("FloatMenuOnButtonText, button pressed");
        SZWidgets.FloatMenuOnRect<T>(l, labelGetter, action);
      }
      TooltipSafe(rect, toolTip);
      //bool flag2 = !toolTip.NullOrEmpty();
      //if (flag2)
      //{
      //  TooltipHandler.TipRegion(rect, toolTip);
      //}
    }

    public static void FloatMenuOnButtonText<T>(Rect rect, string curVal, List<T> l, Func<T, string> labelGetter, Action<T> action)
    {
      if (Widgets.ButtonText(rect, curVal, true, true, true))
      {
        SZWidgets.FloatMenuOnRect<T>(l, labelGetter, action);
      }
    }

    public static void FloatMenuOnButtonThingStuffIcon(Rect rectThing, Rect rectClickable, Selected s, Action<ThingDef> action, string toolTip = "")
    {
      if (Mouse.IsOver(rectClickable))
      {
        Widgets.DrawHighlight(rectClickable);
      }
      Widgets.ThingIcon(rectThing, s.Stuff, s.Stuff, null, 1f, null);
      GUI.color = Color.white;
      if (Widgets.ButtonInvisible(rectClickable, true))
      {
        SZWidgets.FloatMenuOnRect<ThingDef>(s.lOfStuff, s.StuffLabelGetter, action);
      }
      TooltipSafe(rectClickable, toolTip);
      //bool flag3 = !toolTip.NullOrEmpty();
      //if (flag3)
      //{
      //  TooltipHandler.TipRegion(rectClickable, toolTip);
      //}
    }

    public static void FloatMenuOnButtonThingStyleIcon(Rect rectThing, Rect rectClickable, Selected s, Action<ThingStyleDef> action, string toolTip = "")
    {
      if (Mouse.IsOver(rectClickable))
      {
        Widgets.DrawHighlight(rectClickable);
      }
      Widgets.ThingIcon(rectThing, s.thingDef, s.Stuff, s.Style, 1f, null);
      GUI.color = Color.white;
      if (Widgets.ButtonInvisible(rectClickable, true))
      {
        SZWidgets.FloatMenuOnRect<ThingStyleDef>(s.lOfStyle, s.StyleLabelGetter, action);
      }
      TooltipSafe(rectClickable, toolTip);
      //bool flag3 = !toolTip.NullOrEmpty();
      //if (flag3)
      //{
      //  TooltipHandler.TipRegion(rectClickable, toolTip);
      //}
    }

    public static void FloatMenuOnLabel<T>(Rect rect, Color color, List<T> l, Func<T, string> labelGetter, T selected, Action<T> action, string toolTip = "")
    {
      SZWidgets.LabelBackground(rect, labelGetter(selected), color, 0, "");
      if (Mouse.IsOver(rect))
      {
        Widgets.DrawLightHighlight(rect);
      }
      SZWidgets.FloatMenuOnButtonInvisible<T>(new Rect(rect.x, rect.y, rect.width - 20f, rect.height), l, labelGetter, action, toolTip);
    }

    public static void FloatMenuOnLabelAndImage<T>(Rect rect, Color imgColor, string texPath, Pawn pawnForImage, Color lblColor, List<T> l, Func<T, string> labelGetter, T selected, Action<T> action, Action imgAction, bool showFloatMenu = true)
    {
      Widgets.DrawBoxSolid(rect, imgColor);
      if (pawnForImage != null)
      {
        RenderTexture image = PortraitsCache.Get(pawnForImage, new Vector2(rect.width, rect.height), Rot4.South, default(Vector3), 1f, true, true, true, true, null, null, false);
        GUI.DrawTexture(rect, image);
      }
      else
      {
        SZWidgets.Image(rect, texPath);
      }
      if (showFloatMenu)
      {
        SZWidgets.FloatMenuOnLabel<T>(new Rect(rect.x, rect.y - 20f, rect.width, 20f), lblColor, l, labelGetter, selected, action, "");
      }
      if (Widgets.ButtonInvisible(rect, true))
      {
        if (imgAction != null)
        {
          imgAction();
        }
      }

      //bool flag2 = Widgets.ButtonInvisible(rect, true);
      //if (flag2)
      //{
      //  bool flag3 = imgAction != null;
      //  if (flag3)
      //  {
      //    imgAction();
      //  }
      //}
    }

    //public static void FloatMenuOnRect<T>(HashSet<T> l, Func<T, string> labelGetter, Action<T> action)
    //{
    //  FloatMenuOnRect<T>(l.ToList(), labelGetter, action);
    //  //if (l != null && l.Count > 0)
    //  //{
    //  //  List<FloatMenuOption> list = new List<FloatMenuOption>();
    //  //  bool flag2 = typeof(T) == typeof(HairDef);
    //  //  bool flag3 = typeof(T) == typeof(BeardDef);
    //  //  bool flag4 = typeof(T) == typeof(ThingDef);
    //  //  using (HashSet<T>.Enumerator enumerator = l.GetEnumerator())
    //  //  {
    //  //    while (enumerator.MoveNext())
    //  //    {
    //  //      T element = enumerator.Current;
    //  //      string label = labelGetter(element);
    //  //      FloatMenuOption floatMenuOption = new FloatMenuOption(label, delegate ()
    //  //      {
    //  //        bool flag10 = action != null;
    //  //        if (flag10)
    //  //        {
    //  //          action(element);
    //  //        }
    //  //      }, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0);
    //  //      bool flag5 = element != null;
    //  //      if (flag5)
    //  //      {
    //  //        bool flag6 = flag2;
    //  //        if (flag6)
    //  //        {
    //  //          floatMenuOption.SetMemberValue("itemIcon", (element as HairDef).Icon);
    //  //        }
    //  //        else
    //  //        {
    //  //          bool flag7 = flag3;
    //  //          if (flag7)
    //  //          {
    //  //            floatMenuOption.SetMemberValue("itemIcon", (element as BeardDef).Icon);
    //  //          }
    //  //          else
    //  //          {
    //  //            bool flag8 = flag4;
    //  //            if (flag8)
    //  //            {
    //  //              bool flag9 = !(element as ThingDef).uiIconPath.NullOrEmpty();
    //  //              if (flag9)
    //  //              {
    //  //                floatMenuOption.SetMemberValue("itemIcon", (element as ThingDef).uiIcon);
    //  //                floatMenuOption.iconColor = (element as ThingDef).uiIconColor;
    //  //              }
    //  //            }
    //  //          }
    //  //        }
    //  //      }
    //  //      list.Add(floatMenuOption);
    //  //    }
    //  //  }
    //  //  WindowTool.Open(new FloatMenu(list));
    //  //}
    //}

    public static void FloatMenuOnRect<T>(List<T> l, Func<T, string> labelGetter, Action<T> action)
    {
      Log.Message("FloatMenuOnRect, start, l.Count(): " + (l == null ? "null" : l.Count()));
      if (!l.NullOrEmpty<T>())
      {
        List<FloatMenuOption> list = new List<FloatMenuOption>();
        using (List<T>.Enumerator enumerator = l.GetEnumerator())
        {
          while (enumerator.MoveNext())
          {
            T element = enumerator.Current;
            string label = labelGetter(element);
            FloatMenuOption floatMenuOption = new FloatMenuOption(label, delegate ()
            {
              if (action != null)
              {
                action(element);
              }
            }, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0);
            if (element != null)
            {
              if (typeof(T) == typeof(HairDef))
              {
                floatMenuOption.SetMemberValue("itemIcon", (element as HairDef).Icon);
              }
              else if (typeof(T) == typeof(BeardDef))
              {
                floatMenuOption.SetMemberValue("itemIcon", (element as BeardDef).Icon);
              }
              else if (typeof(T) == typeof(ThingDef))
              {
                if (!(element as ThingDef).uiIconPath.NullOrEmpty())
                {
                  floatMenuOption.SetMemberValue("itemIcon", (element as ThingDef).uiIcon);
                  floatMenuOption.iconColor = (element as ThingDef).uiIconColor;
                }
              }
            }
            list.Add(floatMenuOption);
          }
        }
        Log.Message("FloatMenuOnRect, WindowTool.Open, before");
        WindowTool.Open(new FloatMenu(list));
        Log.Message("FloatMenuOnRect, WindowTool.Open, after");
      }
      Log.Message("FloatMenuOnRect, end");
    }

    public static void Image(Rect rect, string texPath)
    {
      GUI.DrawTexture(rect, ContentFinder<Texture2D>.Get(texPath, true));
    }

    // Token: 0x060005BD RID: 1469 RVA: 0x0003BECC File Offset: 0x0003A0CC
    public static void Label(Rect rect, string text, Action action = null, string tooltip = "")
    {
      Widgets.Label(rect, text);
      bool flag = action != null;
      if (flag)
      {
        SZWidgets.ButtonInvisible(rect, action, "");
      }
      TooltipSafe(rect, tooltip);
      //bool flag2 = !tooltip.NullOrEmpty();
      //if (flag2)
      //{
      //  TooltipHandler.TipRegion(rect, tooltip);
      //}
    }

    // Token: 0x060005BE RID: 1470 RVA: 0x0003BF14 File Offset: 0x0003A114
    public static void Label(float x, float y, float w, float h, string text, Action action = null)
    {
      Rect rect = new Rect(x, y, w, h);
      Widgets.Label(rect, text);
      bool flag = action != null;
      if (flag)
      {
        SZWidgets.ButtonInvisible(rect, action, "");
      }
    }

    // Token: 0x060005BF RID: 1471 RVA: 0x0003BF50 File Offset: 0x0003A150
    public static void LabelBackground(Rect rect, string text, Color col, int offset = 0, string tooltip = "")
    {
      Widgets.DrawBoxSolid(rect, col);
      if (text == null)
      {
        text = "";
      }
      rectLabelBackground = new Rect(rect.x + 3f + (float)offset, rect.y, rect.width - 3f, rect.height);
      bool flag2 = rect.height > 20f && text.Length <= 22;
      if (flag2)
      {
        rectLabelBackground.y += (rect.height - 20f) / 2f;
      }
      Widgets.Label(rectLabelBackground, text);
      TooltipSafe(rectLabelBackground, tooltip);
      //if (!tooltip.NullOrEmpty())
      //{
      //  TooltipHandler.TipRegion(rectLabelBackground, tooltip);
      //}
    }

    // Token: 0x060005C0 RID: 1472 RVA: 0x0003C010 File Offset: 0x0003A210
    public static void LabelCol<T>(Rect rect, string text, Color col, Action<T> action, T value, string tooltip = "")
    {
      Widgets.Label(rect, text);
      if (action != null)
      {
        SZWidgets.ButtonInvisibleVar<T>(rect, action, value, tooltip);
      }
    }

    public static void TraitListView(float x, float y, float w, float h, List<Trait> l, ref Vector2 scrollPos, int elemH, Action<Trait> onClick, Action<Trait> onRandom, Action<Trait> onPrev, Action<Trait> onNext, Func<Trait, string> Flabel, Func<Trait, string> Ftooltip)
    {
      bool flag = l.NullOrEmpty<Trait>();
      if (!flag)
      {
        Rect outRect = new Rect(x, y, w, h);
        float height = (float)(l.Count * elemH + 20);
        Rect rect = new Rect(0f, 0f, outRect.width - 16f, height);
        Widgets.BeginScrollView(outRect, ref scrollPos, rect, true);
        Rect rect2 = rect.ContractedBy(6f);
        rect2.height = height;
        Color green = Color.green;
        Listing_X listing_X = new Listing_X();
        rect2.width += 18f;
        listing_X.Begin(rect2);
        listing_X.DefSelectionLineHeight = (float)elemH;
        for (int i = 0; i < l.Count; i++)
        {
          bool flag2 = listing_X.CurY + (float)elemH > scrollPos.y && listing_X.CurY - h < scrollPos.y;
          if (flag2)
          {
            Trait trait = l[i];
            Rect rect3 = new Rect(listing_X.CurX, listing_X.CurY, outRect.width - 16f, (float)elemH);
            string tooltip = "";
            bool flag3 = Mouse.IsOver(rect3);
            if (flag3)
            {
              tooltip = Ftooltip(trait);
            }
            SZWidgets.NavSelectorVar<Trait>(rect3, trait, onClick, onRandom, onPrev, onNext, null, Flabel(trait), tooltip, null);
          }
          listing_X.CurY += 25f;
        }
        listing_X.End();
        Widgets.EndScrollView();
      }
    }

    //public static void AToggleSearch()
    //{
    //  SZWidgets.bToggleSearch = !SZWidgets.bToggleSearch;
    //}

    public static void ListView<T>(float x, float y, float w, float h, ICollection<T> l, Func<T, string> labelGetter, Func<T, string> tooltipGetter, Func<T, T, bool> comparator, ref T selectedThing, ref Vector2 scrollPos, bool withRemove = false, Action<T> action = null, bool withSearch = true, bool drawSection = false, bool hasIcon = false, bool selectOnMouseOver = false)
    {
      //FocusTextLogMessage("ListViewNew, start");
      selectOnMouseOver = false; // ToDo: remember to undo this after testing
      if (l == null)
      {
        return;
      }
      bool isAbility = typeof(T) == typeof(AbilityDef);
      bool isHair = typeof(T) == typeof(HairDef);
      bool isBeard = typeof(T) == typeof(BeardDef);
      bool isThing = typeof(T) == typeof(ThingDef);
      float num = isAbility ? 0f : 32f;
      float num2 = isAbility || isHair || isBeard ? 64f : 0f;
      float num3 = 25f;
      float buttonHeight = num + num2;
      //float scrollHeight = 0f;
      List<T> l2;
      if (withSearch)
      {
        Rect rect = new Rect(x, y, w - 30f, num3);
        l2 = ListViewQuickSearch<T>(rect, l, labelGetter, tooltipGetter, comparator, ref selectedThing, action);
        y += 4f;
      }
      else
      {
        l2 = l.ToList().ListFullCopy();
      }
      float scrollHeight = l2.Count() * buttonHeight + 3f; // Added +3 leeway

      Rect rect2 = new Rect(x, y + num3, w, h - num3);
      //Rect rect3 = new Rect(0f, 0f, rect2.width - 16f, scrollHeight);
      Rect scrollRect = new Rect(0f, 0f, rect2.width - GenUI.ScrollBarWidth, scrollHeight);
      if (drawSection)
      {
        Widgets.DrawMenuSection(rect2);
      }
      Widgets.BeginScrollView(rect2, ref scrollPos, scrollRect, true);
      Rect rect4 = scrollRect.ContractedBy(6f);
      rect4.height = scrollHeight;
      Color selColor = drawSection ? Color.blue : Color.green;
      Listing_X listing_X = new Listing_X();
      rect4.width += 18f;
      listing_X.Begin(rect4);
      listing_X.DefSelectionLineHeight = num;
      try
      {
        Text.Font = GameFont.Small;
        Text.Anchor = TextAnchor.MiddleLeft;
        //FocusTextLogMessage("ListViewNew, before SelectableText");
        foreach (T t in l2)
        {
          //if (listing_X.CurY + num + num2 > scrollPos.y && listing_X.CurY - 700f < scrollPos.y)
          if (listing_X.CurY + buttonHeight > scrollPos.y && listing_X.CurY - 700f < scrollPos.y)
          {
            string label = labelGetter(t);
            string tooltip = tooltipGetter(t);
            bool selected = comparator(t, selectedThing);
            bool isWhite = !withSearch || SZWidgets.lSimilar.NullOrEmpty<string>();

            if (listing_X.SelectableText(label, isThing, isAbility, isHair, isBeard, selected, tooltip, withRemove, isWhite, t, selColor, hasIcon, selectOnMouseOver, buttonHeight))
            {
              selectedThing = t;
              if (action != null)
              {
                action(t);
              }
              else
              {
                SoundDefOf.Mouseover_Category.PlayOneShotOnCamera(null);
              }
            }
          }
          //listing_X.CurY += num;
          //listing_X.CurY += num2;
          listing_X.CurY += buttonHeight;
        }
        //FocusTextLogMessage("ListViewNew, after SelectableText");
        Text.Anchor = TextAnchor.UpperLeft;
      }
      catch (Exception ex)
      {
        Log.ErrorOnce("ListViewNew, BeginScrollView, exception: " + ex, 23409711);
      }
      listing_X.End();
      Widgets.EndScrollView();

      //FocusTextLogMessage("ListViewNew, end");
    }

    private static List<T> ListViewQuickSearch<T>(Rect rect, ICollection<T> l, Func<T, string> labelGetter, Func<T, string> tooltipGetter, Func<T, T, bool> comparator, ref T selectedThing, Action<T> action = null)
    {
      bool isAbility = typeof(T) == typeof(AbilityDef);
      bool isHair = typeof(T) == typeof(HairDef);
      bool isBeard = typeof(T) == typeof(BeardDef);
      bool isThing = typeof(T) == typeof(ThingDef);

      try
      {
        List<T> l2 = new List<T>();
        ////Log.Message("ListView, quickSearchWidget start");

        quickSearchWidget.OnGUI(rect);
        ////Log.Message("ListView, after quickSearchWiget");
        foreach (T t in l)
        {
          string listText = SZWidgets.bToggleSearch ? tooltipGetter(t) : labelGetter(t);
          if (!quickSearchWidget.filter.Matches(listText))
          {
            continue;
          }
          l2.Add(t);
        }
        ////Log.Message("ListView, set noResultsMatched");
        quickSearchWidget.noResultsMatched = l2.NullOrEmpty();
        if (quickSearchWidget.noResultsMatched)
        {
          return l2;
        }

        if (l2.Count() == 1 && l2[0] != null && !comparator(selectedThing, l2[0]))
        {
          //Log.Message("ListView, set selectedThing");
          selectedThing = l2[0];
          if (!isAbility && !isHair && !isBeard && action != null)
          {
            //Log.Message("ListView, start action");
            action(selectedThing);
          }
          //if (tickerSelected <= 0)
          //{
          //  tickerSelected = 5;
          //}
          //else
          //{
          //  tickerSelected--;
          //  if (tickerSelected <= 0)
          //  {
          //    //Log.Message("ListView, set selectedThing");
          //    selectedThing = l2[0];
          //    if (!isAbility && !isHair && !isBeard && action != null)
          //    {
          //      //Log.Message("ListView, start action");
          //      action(selectedThing);
          //    }
          //  }
          //}
        }

        
        if (!isAbility && !isHair && !isBeard && isThing && !SZWidgets.tDefName.NullOrEmpty())
        {
          //float num4 = 0f;
          bool actionFired = false;
          foreach (T t2 in l2)
          {
            if ((t2 as ThingDef).defName == SZWidgets.tDefName && action != null && selectedThing != null)
            {
              //Log.Message("ListView, fire action");
              action(selectedThing);
              actionFired = true;
              break;
            }
          }
          if (actionFired)
          {
            SZWidgets.tDefName = null;
          }
          ////float num4 = 0f;
          //bool actionFired = false;
          //foreach (T t2 in l2)
          //{
          //  if ((t2 as ThingDef).defName == SZWidgets.tDefName)
          //  {
          //    //scrollPos = new Vector2(scrollPos.x, num4);
          //    if (action != null && selectedThing != null)
          //    {
          //      action(selectedThing);
          //      actionFired = true;
          //    }
          //  }
          //  //num4 += num + num2;
          //}
          //if (actionFired)
          //{
          //  SZWidgets.tDefName = null;
          //}
        }
        return l2;
      }
      catch (Exception ex)
      {
        Log.ErrorOnce("ListViewNew, quickSearchWidget, exception: " + ex, 230974);
        return l.ToList().ListFullCopy();
      }
    }

    //public static void FocusTextLogMessage(string message)
    //{
    //  if (LogFocus)
    //  {
    //    if (quickSearchWidget.CurrentlyFocused())
    //    {
    //      //Log.Message("LogFocus: true, CurrentlyFocused: true, text: " + quickSearchWidget.filter.Text + ", cached text: " + filterTextCached + ", message: " + message);
    //    }
    //    else
    //    {
    //      Log.Warning("LogFocus: true, CurrentlyFocused: false, text: " + quickSearchWidget.filter.Text + ", cached text: " + filterTextCached + ", message: " + message + " FocusedControl: " + GUI.GetNameOfFocusedControl());
    //    }
    //  }
    //}

    //public static void ListView<T>(float x, float y, float w, float h, ICollection<T> l, Func<T, string> labelGetter, Func<T, string> tooltipGetter, Func<T, T, bool> comparator, ref T selectedThing, ref Vector2 scrollPos, bool withRemove = false, Action<T> action = null, bool withSearch = true, bool drawSection = false, bool hasIcon = false, bool selectOnMouseOver = false)
    //{
    //  SZWidgets.ListViewNew<T>(x, y, w, h, l, labelGetter, tooltipGetter, comparator, ref selectedThing, ref scrollPos, withRemove, action, withSearch, drawSection, hasIcon, selectOnMouseOver);
    //}

    //public static void ListView<T>(Rect rect, ICollection<T> l, Func<T, string> labelGetter, Func<T, string> tooltipGetter, Func<T, T, bool> comparator, ref T selectedThing, ref Vector2 scrollPos, bool withRemove = false, Action<T> action = null, bool withSearch = true, bool drawSection = false, bool isHead = false, bool selectOnMouse = false)
    //{
    //  SZWidgets.ListView<T>(rect.x, rect.y, rect.width, rect.height, l, labelGetter, tooltipGetter, comparator, ref selectedThing, ref scrollPos, withRemove, action, withSearch, drawSection, isHead, selectOnMouse);
    //}

    public static float NavSelectorCount(Rect rect, Selected s, int max)
    {
      Text.Font = GameFont.Small;
      Rect rectSolid = SZWidgets.RectSolid(rect, true);
      Rect rectToggle = SZWidgets.RectToggle(rect);
      if (Widgets.ButtonImage(SZWidgets.RectPrevious(rect), ContentFinder<Texture2D>.Get("bbackward", true), true))
      {
        s.stackValEntryInt.valInt--;
        //s.oldStackVal = s.stackVal;
        //s.UpdateBuyPrice();
      }
      if (Widgets.ButtonImage(SZWidgets.RectNext(rect), ContentFinder<Texture2D>.Get("bforward", true), true))
      {
        s.stackValEntryInt.valInt++;
        //s.oldStackVal = s.stackVal;
        //s.UpdateBuyPrice();
      }
      string amount = s.stackValEntryInt.Open ? "" : s.stackVal.ToString();
      SZWidgets.LabelBackground(rectSolid, CharacterEditor.Label.COUNT + amount, ColorTool.colAsche);
      SZWidgets.ButtonImage(rectToggle, "UI/Buttons/DragHash", ref SZWidgets.bCountOpen, "");

      Rect rectTextField = rectLabelBackground;
      rectTextField.xMin += Text.CalcSize(CharacterEditor.Label.COUNT).x;
      rectTextField.xMax = rectToggle.x - 5f;
      rectTextField.y -= 1.5f;
      s.stackValEntryInt.NumericTextField(rectTextField, 1, max, false);

      int num = 0;

      if (SZWidgets.bCountOpen)
      {
        num++;
        s.stackValEntryInt.HorizontalSlider(SZWidgets.RectSlider(rect, num), 1, max);
        //s.stackVal = (int)Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), (float)s.stackVal, 1f, (float)max, false, null, null, null, -1f);
      }
      if (s.stackVal != s.oldStackVal)
      {
        s.oldStackVal = s.stackVal;
        s.UpdateBuyPrice();
      }
      return rect.y + 27f + (float)(num * 26);
    }

    //public static void AToggleCountOpen()
    //{
    //  SZWidgets.bCountOpen = !SZWidgets.bCountOpen;
    //}

    public static void NavSelectorImageBox(Rect rect, Action onClicked, Action onRandom, Action onPrev, Action onNext, Action onTextureClick, Action onToggle, string label, string tipLabel = null, string tipRandom = null, string tipTexture = null, string texturePath = null, Color colTex = default(Color), bool toggle = false)
    {
      Text.Font = GameFont.Small;
      SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), label, ColorTool.colAsche, texturePath != null ? 25 : 0, "");
      if (onPrev != null)
      {
        SZWidgets.ButtonImage(SZWidgets.RectPrevious(rect), "bbackward", onPrev, "");
      }
      if (onNext != null)
      {
        SZWidgets.ButtonImage(SZWidgets.RectNext(rect), "bforward", onNext, "");
      }
      NavSelectorImageBoxEnd(rect, onClicked, onRandom, onTextureClick, onToggle, tipLabel, tipRandom, tipTexture, texturePath, colTex, toggle);
    }

    public static void NavSelectorImageBoxSectionHeader(Rect rect, Action onClicked, Action onRandom, Action onToggle, string label, string tipLabel = null, string tipRandom = null, bool toggle = false)
    {
      Text.Font = GameFont.Small;
      SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), label, ColorTool.colDarkDimGray, 0, "");
      NavSelectorImageBoxEnd(rect, onClicked, onRandom, null, onToggle, tipLabel: tipLabel, tipRandom: tipRandom, toggle: toggle);
    }

    public static void NavSelectorImageBoxEnd(Rect rect, Action onClicked, Action onRandom, Action onTextureClick, Action onToggle, string tipLabel = null, string tipRandom = null, string tipTexture = null, string texturePath = null, Color colTex = default(Color), bool toggle = false)
    {
      bool flagRandom = CEditor.IsRandom && onRandom != null;
      if (flagRandom)
      {
        SZWidgets.ButtonImage(SZWidgets.RectRandom(rect), "brandom", onRandom, tipRandom, toggle);
      }

      int num = (flagRandom ? 25 : 0) + (onToggle != null ? 25 : 0);
      Rect rectOnClicked = SZWidgets.RectOnClick(rect, !texturePath.NullOrEmpty(), num, true);
      Rect rectOnToggle = flagRandom ? SZWidgets.RectToggleLeft(rect) : SZWidgets.RectToggle(rect);
      if (onClicked == onToggle && onToggle != null)
      {
        SZWidgets.CombinedHighightDragHashButton(rectOnClicked, rectOnToggle, onToggle, flagRandom, tipLabel, toggle);
        return;
      }
      if (onClicked != null)
      {
        if (onToggle == null)
        {
          rectOnClicked.xMax = rectOnToggle.xMax;
        }
        SZWidgets.ButtonHighlightMouseOver(rectOnClicked, onClicked, tipLabel);
      }
      if (onToggle != null)
      {
        SZWidgets.ButtonImage(rectOnToggle, "UI/Buttons/DragHash", onToggle, tipTexture, toggle);
      }

      SZWidgets.ButtonImage(SZWidgets.RectTexture(rect), texturePath, onTextureClick, tipTexture);
    }


    public static void CombinedHighightDragHashButton(Rect rectOnClicked, Rect rectOnToggle, Action onToggle, bool flagRandom, string tipLabel = null, bool toggle = false)
    {
      Rect rectButton = rectOnClicked;
      rectButton.xMax = rectOnToggle.xMax;
      SZWidgets.ButtonHighlightMouseOver(rectButton, onToggle, tipLabel);
      GUI.color = Mouse.IsOver(rectButton) ? GenUI.MouseoverColor : BaseColor(toggle);
      GUI.DrawTexture(rectOnToggle, ContentFinder<Texture2D>.Get("UI/Buttons/DragHash", true));
      GUI.color = Color.white;
    }

    // Token: 0x060005C8 RID: 1480 RVA: 0x0003D0BC File Offset: 0x0003B2BC
    public static void NavSelectorVar<T>(Rect rect, T val, Action<T> onClick, Action<T> onRandom, Action<T> onPrev, Action<T> onNext, Action<T> onToggle, string label, string tooltip, string tipRandom, bool toggle = false)
    {
      bool isRandom = CEditor.IsRandom;
      if (isRandom && onPrev != null)
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectPrevious(rect), "bbackward", onPrev, val, "");
      }
      if (isRandom && onNext != null)
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectNext(rect), "bforward", onNext, val, "");
      }
      Text.Font = GameFont.Small;
      bool flag3 = onToggle != null;
      int offset = CEditor.IsRandom ? 25 : 0;
      SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, isRandom), label, ColorTool.colAsche, 0, "");
      if (onClick != null)
      {
        SZWidgets.ButtonInvisibleMouseOverVar<T>(SZWidgets.RectOnClick(rect, false, offset, isRandom), onClick, val, tooltip);
      }
      if (CEditor.IsRandom)
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectRandom(rect), "brandom", onRandom, val, tipRandom);
        if (flag3)
        {
          SZWidgets.ButtonImageVar<T>(SZWidgets.RectToggleLeft(rect), flag3 ? "UI/Buttons/DragHash" : null, onToggle, val, "", toggle);
        }
      }
      else
      {
        if (flag3)
        {
          SZWidgets.ButtonImageVar<T>(SZWidgets.RectToggle(rect), flag3 ? "UI/Buttons/DragHash" : null, onToggle, val, "", toggle);
        }
      }
    }

    // Token: 0x060005C9 RID: 1481 RVA: 0x0003D1E4 File Offset: 0x0003B3E4
    public static void NavSelectorImageBox2<T>(Rect rect, T val, Action<T> onClicked, Action<T> onRandom, Action<T> onPrev, Action<T> onNext, Action<T> onTextureClick, Action<T> onToggle, string label, string tipLabel = null, string tipRandom = null, string tipTexture = null, string texturePath = null, Color colTex = default(Color), bool toggle = false)
    {
      if (onPrev != null)
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectPrevious(rect), "bbackward", onPrev, val, "");
      }
      if (onNext != null)
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectNext(rect), "bforward", onNext, val, "");
      }
      Text.Font = GameFont.Small;
      bool flag3 = val is Thing;
      bool flag4 = flag3 || texturePath != null;
      bool flag5 = onToggle != null;
      SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), label, ColorTool.colAsche, flag4 ? 25 : 0, "");
      int num = CEditor.IsRandom ? 25 : 0;
      num += (flag5 ? 25 : 0);
      bool flag6 = flag3;
      if (flag6)
      {
        SZWidgets.ButtonThingVar<T>(SZWidgets.RectTexture(rect), val, onTextureClick, tipTexture);
      }
      else
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectTexture(rect), texturePath, onTextureClick, val, tipTexture);
      }
      SZWidgets.ButtonInvisibleMouseOverVar<T>(SZWidgets.RectOnClick(rect, flag4, num, true), onClicked, val, tipLabel);
      bool isRandom = CEditor.IsRandom;
      if (isRandom)
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectRandom(rect), "brandom", onRandom, val, tipRandom);
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectToggleLeft(rect), flag5 ? "UI/Buttons/DragHash" : null, onToggle, val, "", toggle);
      }
      else
      {
        SZWidgets.ButtonImageVar<T>(SZWidgets.RectToggle(rect), flag5 ? "UI/Buttons/DragHash" : null, onToggle, val, "", toggle);
      }
    }

    // Token: 0x060005CA RID: 1482 RVA: 0x0003D340 File Offset: 0x0003B540
    public static float NavSelectorQuality(Rect rect, Selected s, List<int> lOfQuality)
    {
      float result;
      if (s.thingDef.HasComp(typeof(CompQuality)))
      {
        if (Widgets.ButtonImage(SZWidgets.RectPrevious(rect), ContentFinder<Texture2D>.Get("bbackward", true), true))
        {
          s.quality = lOfQuality.NextOrPrevIndex(s.quality, false, false);
          s.oldQuality = s.quality;
          s.UpdateBuyPrice();
        }
        bool flag3 = Widgets.ButtonImage(SZWidgets.RectNext(rect), ContentFinder<Texture2D>.Get("bforward", true), true);
        if (flag3)
        {
          s.quality = lOfQuality.NextOrPrevIndex(s.quality, true, false);
          s.oldQuality = s.quality;
          s.UpdateBuyPrice();
        }
        Text.Font = GameFont.Small;
        SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), CharacterEditor.Label.QUALITY + ((QualityCategory)s.quality).GetLabel().CapitalizeFirst(), ColorTool.colAsche, 0, "");
        SZWidgets.ButtonImage(SZWidgets.RectToggle(rect), "UI/Buttons/DragHash", ref SZWidgets.bQualityOpen, "");
        //if (Widgets.ButtonImage(SZWidgets.RectToggle(rect), ContentFinder<Texture2D>.Get("UI/Buttons/DragHash", true), true))
        //{
        //  SZWidgets.bQualityOpen = !SZWidgets.bQualityOpen;
        //}

        Rect rect2 = SZWidgets.RectClickableT(rect);
        if (Mouse.IsOver(rect2))
        {
          Widgets.DrawHighlight(rect2);
        }
        SZWidgets.FloatMenuOnButtonInvisible<int>(rect2, lOfQuality, (int x) => ((QualityCategory)x).GetLabel(), delegate (int x)
        {
          s.quality = lOfQuality[x];
          s.oldQuality = s.quality;
          s.UpdateBuyPrice();
        }, "");
        int num = 0;
        if (SZWidgets.bQualityOpen)
        {
          num++;
          s.quality = (int)Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), (float)s.quality, 0f, (float)(lOfQuality.Count - 1), false, null, null, null, -1f);
        }
        result = rect.y + 27f + (float)(num * 26);
      }
      else
      {
        result = rect.y + 27f;
        Text.Font = GameFont.Small;
        SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), CharacterEditor.Label.QUALITY + CharacterEditor.Label.NONE, ColorTool.colAsche, 0, "");
      }
      return result;
    }

    // Token: 0x060005CB RID: 1483 RVA: 0x0003D588 File Offset: 0x0003B788
    public static float NavSelectorStuff(Rect rect, Selected s)
    {
      Text.Font = GameFont.Small;
      bool madeFromStuff = s.thingDef.MadeFromStuff;
      bool flag = Widgets.ButtonImage(SZWidgets.RectPrevious(rect), ContentFinder<Texture2D>.Get("bbackward", true), true);
      if (flag)
      {
        bool flag2 = madeFromStuff;
        if (flag2)
        {
          s.SetStuff(false, false);
        }
      }
      bool flag3 = Widgets.ButtonImage(SZWidgets.RectNext(rect), ContentFinder<Texture2D>.Get("bforward", true), true);
      if (flag3)
      {
        bool flag4 = madeFromStuff;
        if (flag4)
        {
          s.SetStuff(true, false);
        }
      }
      SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), CharacterEditor.Label.STUFF + s.StuffLabelGetter(s.Stuff), ColorTool.colAsche, madeFromStuff ? 25 : 0, "");
      SZWidgets.ButtonImage(SZWidgets.RectToggle(rect), "UI/Buttons/DragHash", ref SZWidgets.bStuffOpen, "");
      //bool flag5 = Widgets.ButtonImage(SZWidgets.RectToggle(rect), ContentFinder<Texture2D>.Get("UI/Buttons/DragHash", true), true);
      //if (flag5)
      //{
      //  SZWidgets.bStuffOpen = !SZWidgets.bStuffOpen;
      //}


      if (madeFromStuff)
      {
        SZWidgets.FloatMenuOnButtonThingStuffIcon(SZWidgets.RectTexture(rect), SZWidgets.RectClickableT(rect), s, delegate (ThingDef x)
        {
          s.SetStuff(x);
        }, s.StuffDescrGetter(s.Stuff));
      }
      int num = 0;
      if (SZWidgets.bStuffOpen)
      {
        if (madeFromStuff)
        {
          num++;
          s.stuffIndex = (int)Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), (float)s.stuffIndex, 0f, (float)(s.lOfStuff.Count - 1), false, null, null, null, -1f);
          s.CheckSetStuff();
        }
        GUI.color = Color.red;
        num++;
        s.DrawColor.r = Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), s.DrawColor.r, 0f, 1f, false, null, null, null, -1f);
        GUI.color = Color.green;
        num++;
        s.DrawColor.g = Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), s.DrawColor.g, 0f, 1f, false, null, null, null, -1f);
        GUI.color = Color.blue;
        num++;
        s.DrawColor.b = Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), s.DrawColor.b, 0f, 1f, false, null, null, null, -1f);
        GUI.color = Color.white;
        num++;
        s.DrawColor.a = Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), s.DrawColor.a, 0f, 1f, false, null, null, null, -1f);
      }
      return rect.y + 27f + (float)(num * 26);
    }

    // Token: 0x060005CC RID: 1484 RVA: 0x0003D8A0 File Offset: 0x0003BAA0
    public static float NavSelectorStyle(Rect rect, Selected s)
    {
      Text.Font = GameFont.Small;
      bool flag = s.thingDef.CanBeStyled() && s.lOfStyle.Count > 1;
      bool flag2 = Widgets.ButtonImage(SZWidgets.RectPrevious(rect), ContentFinder<Texture2D>.Get("bbackward", true), true);
      if (flag2)
      {
        bool flag3 = flag;
        if (flag3)
        {
          s.SetStyle(false, false);
        }
      }
      bool flag4 = Widgets.ButtonImage(SZWidgets.RectNext(rect), ContentFinder<Texture2D>.Get("bforward", true), true);
      if (flag4)
      {
        bool flag5 = flag;
        if (flag5)
        {
          s.SetStyle(true, false);
        }
      }
      string str = flag ? s.StyleLabelGetter(s.Style) : "";
      SZWidgets.LabelBackground(SZWidgets.RectSolid(rect, true), CharacterEditor.Label.STYLE + str, ColorTool.colAsche, flag ? 25 : 0, "");
      if (flag)
      {
        SZWidgets.ButtonImage(SZWidgets.RectToggle(rect), "UI/Buttons/DragHash", ref SZWidgets.bStyleOpen, "");
      }
      //bool flag6 = Widgets.ButtonImage(SZWidgets.RectToggle(rect), ContentFinder<Texture2D>.Get("UI/Buttons/DragHash", true), true);
      //if (flag6)
      //{
      //  SZWidgets.bStyleOpen = !SZWidgets.bStyleOpen;
      //}
      if (flag)
      {
        SZWidgets.FloatMenuOnButtonThingStyleIcon(SZWidgets.RectTexture(rect), SZWidgets.RectClickableT(rect), s, delegate (ThingStyleDef x)
        {
          s.SetStyle(x);
        }, s.StyleDescrGetter(s.Style));
      }
      int num = 0;
      if (SZWidgets.bStyleOpen)
      {
        if (flag)
        {
          num++;
          s.styleIndex = (int)Widgets.HorizontalSlider(SZWidgets.RectSlider(rect, num), (float)s.styleIndex, 0f, (float)(s.lOfStyle.Count - 1), false, null, null, null, -1f);
          s.CheckSetStyle();
        }
      }
      return rect.y + 27f + (float)(num * 26);
    }

    // Token: 0x060005CD RID: 1485 RVA: 0x0003DAA0 File Offset: 0x0003BCA0
    public static float NumericFloatBox(float x, float y, float h, float value, float min, float max)
    {
      Rect butRect = new Rect(x, y, 25f, h);
      Rect rect = new Rect(x + 20f, y, 70f, h);
      Rect butRect2 = new Rect(x + 85f, y, 25f, h);
      bool flag = Widgets.ButtonImage(butRect, ContentFinder<Texture2D>.Get("bbackward", true), true);
      if (flag)
      {
        value -= 1f;
        value = (float)Math.Round((double)value, 2);
      }
      bool flag2 = Widgets.ButtonImage(butRect2, ContentFinder<Texture2D>.Get("bforward", true), true);
      if (flag2)
      {
        value += 1f;
        value = (float)Math.Round((double)value, 2);
      }
      string text = value.ToString();
      bool flag3 = text.EndsWith(".");
      if (flag3)
      {
        text += "0";
      }
      else
      {
        bool flag4 = !text.Contains(".");
        if (flag4)
        {
          text += ".0";
        }
      }
      text = Widgets.TextField(rect, text);
      bool flag5 = text.EndsWith(".");
      if (flag5)
      {
        text += "0";
      }
      else
      {
        bool flag6 = !text.Contains(".");
        if (flag6)
        {
          text += ".0";
        }
      }
      float num = 0f;
      bool flag7 = float.TryParse(text, out num);
      if (flag7)
      {
        bool flag8 = num < min;
        if (flag8)
        {
          value = min;
        }
        else
        {
          bool flag9 = num > max;
          if (flag9)
          {
            value = max;
          }
          else
          {
            value = num;
          }
        }
      }
      return value;
    }

    public static int NumericIntBox(float x, float y, float w, float h, int value, int min, int max)
    {
      Rect butRect = new Rect(x, y, 25f, h);
      Rect rect = new Rect(x + 20f, y, w, h);
      Rect butRect2 = new Rect(x + w + 15f, y, 25f, h);
      bool flag = Widgets.ButtonImage(butRect, ContentFinder<Texture2D>.Get("bbackward", true), true);
      if (flag)
      {
        value--;
      }
      bool flag2 = Widgets.ButtonImage(butRect2, ContentFinder<Texture2D>.Get("bforward", true), true);
      if (flag2)
      {
        value++;
      }
      string text = value.ToString();
      text = Widgets.TextField(rect, text);
      int num = 0;
      bool flag3 = int.TryParse(text, out num);
      if (flag3)
      {
        bool flag4 = num < min;
        if (flag4)
        {
          value = min;
        }
        else
        {
          bool flag5 = num > max;
          if (flag5)
          {
            value = max;
          }
          else
          {
            value = num;
          }
        }
      }
      return value;
    }

    public static int NumericTextField(float x, float y, float w, float h, int value, int min, int max)
    {
      Rect rect = new Rect(x + 20f, y, w, h);
      string text = value.ToString();
      text = Widgets.TextField(rect, text);
      if (int.TryParse(text, out int num))
      {
        return Mathf.Clamp(num, min, max);
      }
      return value;
    }

    // Token: 0x060005D0 RID: 1488 RVA: 0x0003DD58 File Offset: 0x0003BF58
    public static void ScrollView(int x, int y, int w, int h, int objCount, int objH, ref Vector2 scrollPos, Action<Listing_X> drawFunction)
    {
      Rect outRect = new Rect((float)x, (float)y, (float)w, (float)h);
      Rect rect = new Rect(0f, (float)y, outRect.width - 16f, (float)(objCount * objH));
      Widgets.BeginScrollView(outRect, ref scrollPos, rect, true);
      Rect rect2 = rect.ContractedBy(4f);
      rect2.height = (float)(objCount * objH);
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect2);
      drawFunction(listing_X);
      listing_X.End();
      Widgets.EndScrollView();
    }

    // Token: 0x060005D1 RID: 1489 RVA: 0x0003DDE0 File Offset: 0x0003BFE0
    public static void SimpleMultiplierSlider(Rect rect, string label, string format, bool showNumeric, float baseValue, ref float currentVal, float min, float max)
    {
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect);
      string text = showNumeric ? label : "";
      listing_X.AddMultiplierSection(label, format, ref text, baseValue, ref currentVal, min, max, true);
      listing_X.End();
    }

    // Token: 0x060005D2 RID: 1490 RVA: 0x0003DE24 File Offset: 0x0003C024
    public static void SimpleSlider(Rect rect, string label, ref float currentVal, float min, float max)
    {
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect);
      string text = label;
      listing_X.AddSection(label, "", ref text, ref currentVal, min, max, true, "");
      listing_X.End();
    }

    // Token: 0x060005D3 RID: 1491 RVA: 0x0003DE64 File Offset: 0x0003C064
    public static void SingleSlinder(Rect rect, float currentVal, float min, float max, Action<float> action)
    {
      Listing_X listing_X = new Listing_X();
      listing_X.Begin(rect);
      float num = listing_X.Slider(currentVal, min, max);
      bool flag = num != currentVal;
      listing_X.End();
      bool flag2 = flag && action != null;
      if (flag2)
      {
        action(num);
      }
    }

    // Token: 0x060005D4 RID: 1492 RVA: 0x0003DEB4 File Offset: 0x0003C0B4
    public static string TextArea(Rect rect, string text, int max, Regex regex)
    {
      bool flag = text == null;
      if (flag)
      {
        text = "";
      }
      string text2 = GUI.TextArea(rect, text, max, Text.CurTextAreaStyle);
      bool flag2 = text2.Length <= max && regex != null && regex.IsMatch(text2);
      string result;
      if (flag2)
      {
        result = text2;
      }
      else
      {
        result = text;
      }
      return result;
    }

    // Token: 0x060005D5 RID: 1493 RVA: 0x0003DF04 File Offset: 0x0003C104
    public static void ThingDrawer(Rect rect, Thing t)
    {
      Widgets.ThingIcon(rect, t, 1f, null);
      GUI.color = Color.white;
    }

    // Token: 0x060005D6 RID: 1494 RVA: 0x0003DF33 File Offset: 0x0003C133
    private static Rect RectClickableT(Rect rect)
    {
      return new Rect(rect.x + 21f, rect.y, rect.width - 64f, 24f);
    }

    // Token: 0x060005D7 RID: 1495 RVA: 0x0003DF60 File Offset: 0x0003C160
    private static Rect RectNext(Rect rect)
    {
      return new Rect(rect.x + rect.width - 22f, rect.y + 2f, 22f, 22f);
    }

    // Token: 0x060005D8 RID: 1496 RVA: 0x0003DF94 File Offset: 0x0003C194
    private static Rect RectOnClick(Rect rect, bool hasTexture, int offset = 0, bool showEdit = true)
    {
      return new Rect(rect.x + (float)(showEdit ? 21 : 0) + (float)(hasTexture ? 25 : 0), rect.y, rect.width - (float)(showEdit ? 40 : 19) - (float)(hasTexture ? 25 : 0) - (float)offset, 24f);
    }

    // Token: 0x060005D9 RID: 1497 RVA: 0x0003DFEC File Offset: 0x0003C1EC
    private static Rect RectPrevious(Rect rect)
    {
      return new Rect(rect.x, rect.y + 2f, 22f, 22f);
    }

    // Token: 0x060005DA RID: 1498 RVA: 0x0003E011 File Offset: 0x0003C211
    private static Rect RectRandom(Rect rect)
    {
      return new Rect(rect.x + rect.width - 42f, rect.y, 22f, 22f);
    }

    // Token: 0x060005DB RID: 1499 RVA: 0x0003E03E File Offset: 0x0003C23E
    private static Rect RectSlider(Rect rect, int i)
    {
      return new Rect(rect.x, rect.y + 10f + (float)i * rect.height, rect.width, rect.height);
    }

    // Token: 0x060005DC RID: 1500 RVA: 0x0003E072 File Offset: 0x0003C272
    private static Rect RectSolid(Rect rect, bool showEdit = true)
    {
      return new Rect(rect.x + (float)(showEdit ? 21 : 0), rect.y, rect.width - (float)(showEdit ? 40 : 19), 24f);
    }

    // Token: 0x060005DD RID: 1501 RVA: 0x0003E0A8 File Offset: 0x0003C2A8
    private static Rect RectTexture(Rect rect)
    {
      return new Rect(rect.x + 25f, rect.y, 24f, 24f);
    }

    // Token: 0x060005DE RID: 1502 RVA: 0x0003E0CD File Offset: 0x0003C2CD
    private static Rect RectToggle(Rect rect)
    {
      return new Rect(rect.x + rect.width - 42f, rect.y, 22f, 22f);
    }

    // Token: 0x060005DF RID: 1503 RVA: 0x0003E0FA File Offset: 0x0003C2FA
    private static Rect RectToggleLeft(Rect rect)
    {
      return new Rect(rect.x + rect.width - 67f, rect.y, 22f, 22f);
    }



  }
}
