﻿using System;
using System.Collections.Generic;
using System.Text;
using RimWorld;
using Verse;

namespace CharacterEditor
{
  // Token: 0x0200003F RID: 63
  public static class ScenarioTool
  {
    public static List<Pair<ScenPart, EntryInt>> takenPartEntryList = new List<Pair<ScenPart, EntryInt>>();
    public static List<Pair<ScenPart, EntryInt>> onMapPartEntryList = new List<Pair<ScenPart, EntryInt>>();
    public static List<Pair<ScenPart, EntryInt>> animalPartEntryList = new List<Pair<ScenPart, EntryInt>>();
    public static List<ScenPart> otherPartList = new List<ScenPart>();
    public static Pair<ScenPart, EntryInt> tmpEntryPair;

    public static int CurrentTakenPawnCount => CEditor.InStartingScreen ? Find.GameInitData.startingPawnCount : ScenarioTool.CurrentPawnList.Count;
    public static List<Pawn> CurrentPawnList => CEditor.InStartingScreen ? Find.GameInitData.startingAndOptionalPawns : PawnxTool.GetPawnList(Label.COLONISTS, true, Faction.OfPlayer);
    public static List<ScenPart> ScenarioParts
    {
      get
      {
        return ((List<ScenPart>)Find.Scenario.GetMemberValue("parts")).ListFullCopy();
      }
      set
      {
        Find.Scenario.SetMemberValue("parts", value);
      }
    }
    public static List<ScenPart> CustomScenarioParts
    {
      get
      {
        List<ScenPart> parts = new List<ScenPart>();
        parts.AddRange(SaveEntriesToParts(takenPartEntryList));
        parts.AddRange(SaveEntriesToParts(onMapPartEntryList));
        parts.AddRange(SaveEntriesToParts(animalPartEntryList));
        parts.AddRange(otherPartList);
        return parts;
      }
      set
      {
        takenPartEntryList = new List<Pair<ScenPart, EntryInt>>();
        onMapPartEntryList = new List<Pair<ScenPart, EntryInt>>();
        animalPartEntryList = new List<Pair<ScenPart, EntryInt>>();
        otherPartList = new List<ScenPart>();
        foreach (ScenPart scenPart in value)
        {
          AddScenePart(scenPart);
        }
      }

    }

    public static List<ScenPart> SaveEntriesToParts(List<Pair<ScenPart, EntryInt>> entryPairList)
    {
      List<ScenPart> scenPartList = new List<ScenPart>();
      foreach (Pair<ScenPart, EntryInt> pair in entryPairList)
      {
        SetScenarioPartCount(pair.First, pair.Second.valInt);
        scenPartList.Add(pair.First);
      }
      return scenPartList;
    }

    public static void SetScenarioPartCount<T>(T part, int count) where T : ScenPart
    {
      if (part.IsSupportedScenarioPart())
      {
        part.SetMemberValue("count", count);
      }
    }

    public static bool IsScenarioAnimal(this ScenPart part)
    {
      return part != null && part.GetType() == typeof(ScenPart_StartingAnimal);
    }

    public static bool IsScatterAnywherePart(this ScenPart part)
    {
      return part != null && part.GetType() == typeof(ScenPart_ScatterThingsAnywhere);
    }

    public static bool IsSupportedScenarioPart(this ScenPart part)
    {
      bool flag = part == null;
      return part != null && (part.GetType() == typeof(ScenPart_StartingAnimal) || part.GetType() == typeof(ScenPart_StartingThing_Defined) || part.GetType() == typeof(ScenPart_ScatterThingsAnywhere) || part.GetType() == typeof(ScenPart_ScatterThingsNearPlayerStart));
    }

    public static void LoadCapsuleSetup(string filepath)
    {
      if (FileIO.Exists(filepath))
      {
        byte[] input = FileIO.ReadFile(filepath);
        string text = input.AsString(Encoding.UTF8);
        if (!text.NullOrEmpty())
        {
          string[] array = text.SplitNo("\n");
          if (array.NullOrEmpty<string>())
          {
            MessageTool.Show("can't load scenario file. reason-> no data in file=" + filepath, null);
          }
          else
          {
            try
            {
              PawnxTool.DeleteAllPawns(Label.COLONISTS, true, Faction.OfPlayer);
            }
            catch
            {
            }
            Dictionary<Pawn, string> dictionary = new Dictionary<Pawn, string>();
            int startingPawnCount = 0;
            for (int i = 0; i < array.Length; i++)
            {
              try
              {
                if (i == 0)
                {
                  ScenarioTool.SetScenarioParameterFromSeparatedString(array[i], out startingPawnCount);
                }
                else
                {
                  if (i == 1)
                  {
                    ScenarioTool.SetScenaioPartsFromSeparatedString(array[i]);
                  }
                  else
                  {
                    PresetPawn presetPawn = new PresetPawn();
                    Pawn key = presetPawn.LoadPawn(-1, false, array[i]);
                    dictionary.Add(key, presetPawn.dicParams.GetValue(PresetPawn.Param.P33_relations));
                  }
                }
              }
              catch
              {
                MessageTool.Show("error while parsing scenario file. reason -> wrong or invalid data format", null);
              }
            }
            bool inStartingScreen = CEditor.InStartingScreen;
            if (inStartingScreen)
            {
              Find.GameInitData.startingPawnCount = startingPawnCount;
            }
            foreach (Pawn pawn in dictionary.Keys)
            {
              pawn.SetRelationsFromSeparatedString(dictionary[pawn]);
            }
          }
        }
      }
    }

    public static void SaveCapsuleSetup(string filepath)
    {
      bool flag = filepath.NullOrEmpty();
      if (!flag)
      {
        string str = ScenarioTool.GetAllScenarioParameter();
        string allScenarioPartsAsSeparatedString = ScenarioTool.GetAllScenarioPartsAsSeparatedString();
        string text = "";
        string text2 = "";
        List<Pawn> currentPawnList = ScenarioTool.CurrentPawnList;
        foreach (Pawn pawn in currentPawnList)
        {
          PresetPawn presetPawn = new PresetPawn();
          string str2 = presetPawn.SavePawn(pawn, -1);
          text2 = text2 + pawn.GetPawnName(false) + ",";
          text = text + "\n" + str2;
        }
        text2 = text2.SubstringRemoveLast();
        str += text2;
        string text3 = str + allScenarioPartsAsSeparatedString + text;
        FileIO.WriteFile(filepath, text3.AsBytes(Encoding.UTF8));
      }
    }

    public static int GetTypeReplacer<T>(T sd) where T : ScenPart
    {
      if (sd.GetType() == typeof(ScenPart_StartingThing_Defined))
      {
        return 1;
      }
      if (sd.GetType() == typeof(ScenPart_ScatterThingsAnywhere))
      {
        return 2;
      }
      if (sd.GetType() == typeof(ScenPart_ScatterThingsNearPlayerStart))
      {
        return 3;
      }
      if (sd.GetType() == typeof(ScenPart_StartingAnimal))
      {
        return 4;
      }
      return 0;
    }

    public static string GetScenarioPartString<T>(T sd) where T : ScenPart
    {
      string text = "";
      if (sd.IsScenarioAnimal())
      {
        int num;
        PawnKindDef scenarioPartAnimal = sd.GetScenarioPartAnimal(out num);
        text = text + ScenarioTool.GetTypeReplacer<T>(sd).ToString() + "|";
        text += ((scenarioPartAnimal != null) ? (scenarioPartAnimal.defName + "|") : "|");
        text += num.ToString();
      }
      else
      {
        ThingDef thingDef;
        ThingStyleDef thingStyleDef;
        int num2;
        ThingDef scenarioPartThingDef = ScenarioTool.GetScenarioPartThingDef<T>(sd, out thingDef, out thingStyleDef, out num2);
        text = text + ScenarioTool.GetTypeReplacer<T>(sd).ToString() + "|";
        text += scenarioPartThingDef != null ? scenarioPartThingDef.defName + "|" : "|";
        text += thingDef != null ? thingDef.defName + "|" : "|";
        text += thingStyleDef != null ? thingStyleDef.defName + "|" : "|";
        text += num2.ToString();
      }
      return text;
    }

    public static string GetAllScenarioParameter()
    {
      string str = "";
      str = str + ScenarioTool.CurrentPawnList.FirstOrFallback(null).Faction.Name + "|";
      return str + (CEditor.InStartingScreen ? Find.GameInitData.startingPawnCount : ScenarioTool.CurrentPawnList.Count).ToString() + "|";
    }

    public static string GetAllScenarioPartsAsSeparatedString()
    {
      List<ScenPart> scenarioParts = ScenarioTool.CustomScenarioParts;
      if (scenarioParts.NullOrEmpty())
      {
        return "";
      }
      string text = "\n";
      foreach (ScenPart sd in scenarioParts)
      {
        text += ScenarioTool.GetScenarioPartString<ScenPart>(sd);
        text += ":";
      }
      text = text.SubstringRemoveLast();
      return text;
    }

    public static void SetScenarioParameterFromSeparatedString(string s, out int pawnCount)
    {
      pawnCount = 0;
      if (!s.NullOrEmpty())
      {
        string[] array = s.SplitNo("|");
        if (array.Length >= 1)
        {
          Faction.OfPlayer.Name = array[0];
        }
        if (array.Length >= 2)
        {
          pawnCount = array[1].AsInt32();
        }
      }
    }

    public static void SetScenaioPartsFromSeparatedString(string s)
    {
      if (s.NullOrEmpty())
      {
        return;
      }
      List<ScenPart> scenarioParts = ScenarioTool.AllNotSupportedScenarioParts();
      string[] array = s.SplitNo(":");
      foreach (string text in array)
      {
        if (text.NullOrEmpty())
        {
          continue;
        }
        string[] array3 = text.SplitNo("|");
        if (array3.Length < 3)
        {
          continue;
        }
        if (array3.Length >= 5)
        {
          if (array3[0] == "1" || array3[0] == "3")
          {
            //ScenarioTool.AddScenePart(ScenarioTool.CreateScenePart_Defined(DefTool.ThingDef(array3[1]), DefTool.ThingDef(array3[2]), array3[4].AsInt32()));
            scenarioParts.Add(ScenarioTool.CreateScenePart_Defined(DefTool.ThingDef(array3[1]), DefTool.ThingDef(array3[2]), array3[4].AsInt32()));
            continue;
          }
          if (array3[0] == "2")
          {
            //ScenarioTool.AddScenePart(ScenarioTool.CreateScenePart_Anywhere(DefTool.ThingDef(array3[1]), DefTool.ThingDef(array3[2]), array3[4].AsInt32()));
            scenarioParts.Add(ScenarioTool.CreateScenePart_Anywhere(DefTool.ThingDef(array3[1]), DefTool.ThingDef(array3[2]), array3[4].AsInt32()));
            continue;
          }
        }
        else if (array3[0] == "4")
        {
          //ScenarioTool.AddScenePart(ScenarioTool.CreateScenePart_Animal(DefTool.PawnKindDef(array3[1]), array3[2].AsInt32()));
          scenarioParts.Add(ScenarioTool.CreateScenePart_Animal(DefTool.PawnKindDef(array3[1]), array3[2].AsInt32()));
        }
      }
      ScenarioTool.ScenarioParts = scenarioParts;
      ScenarioTool.CustomScenarioParts = ScenarioTool.ScenarioParts;
    }

    public static List<ScenPart> AllNotSupportedScenarioParts()
    {
      List<ScenPart> scenarioParts = ScenarioTool.ScenarioParts;
      if (!scenarioParts.NullOrEmpty<ScenPart>())
      {
        for (int i = scenarioParts.Count - 1; i >= 0; i--)
        {
          bool flag2 = scenarioParts[i].IsSupportedScenarioPart();
          if (flag2)
          {
            scenarioParts.Remove(scenarioParts[i]);
          }
        }
      }
      return scenarioParts;
    }

    public static ThingDef GetScenarioPartThingDef<T>(T part, out ThingDef stuff, out ThingStyleDef styleDef, out int count) where T : ScenPart
    {
      count = 0;
      stuff = null;
      styleDef = null;
      ThingDef result = null;
      if (part.IsSupportedScenarioPart() && !part.IsScenarioAnimal())
      {
        count = (int)part.GetMemberValue("count");
        stuff = (ThingDef)part.GetMemberValue("stuff");
        result = (ThingDef)part.GetMemberValue("thingDef");
        styleDef = (ThingStyleDef)part.GetMemberValue("styleDef");
      }
      return result;
    }

    public static PawnKindDef GetScenarioPartAnimal(this ScenPart part, out int count)
    {
      count = 0;
      PawnKindDef result = null;
      if (part.IsScenarioAnimal())
      {
        ScenPart_StartingAnimal obj = part as ScenPart_StartingAnimal;
        count = (int)obj.GetMemberValue("count");
        result = (PawnKindDef)obj.GetMemberValue("animalKind");
      }
      return result;
    }

    //public static void SaveEntriesToScenarioParts()
    //{
    //  //ScenarioParts = CustomScenarioParts;
    //  //List<ScenPart> parts = new List<ScenPart>();
    //  //SaveEntriesToParts(parts, takenPartEntryList);
    //  //SaveEntriesToParts(parts, onMapPartEntryList);
    //  //SaveEntriesToParts(parts, animalPartEntryList);
    //  //foreach (ScenPart otherPart in otherPartList)
    //  //{
    //  //  parts.Add(otherPart);
    //  //}
    //  //Find.Scenario.SetMemberValue("parts", parts);
    //}

    public static int GetScenarioPartCount<T>(T part) where T : ScenPart
    {
      ////Log.Message("GetScenarioPartCount, start");
      if (part.IsSupportedScenarioPart())
      {
        ////Log.Message("GetScenarioPartCount, end");
        return (int)part.GetMemberValue("count");
      }
      ////Log.Message("GetScenarioPartCount, end");
      return 0;
    }

    public static ScenPart_StartingThing_Defined CreateScenePart_Defined(ThingDef thingDef, ThingDef stuffDef, int count)
    {
      ScenPart_StartingThing_Defined scenPart_StartingThing_Defined = new ScenPart_StartingThing_Defined();
      scenPart_StartingThing_Defined.def = ScenPartDefOf.StartingThing_Defined;
      scenPart_StartingThing_Defined.SetMemberValue("thingDef", thingDef);
      scenPart_StartingThing_Defined.SetMemberValue("stuff", stuffDef);
      scenPart_StartingThing_Defined.SetMemberValue("count", count);
      return scenPart_StartingThing_Defined;
    }

    public static ScenPart_ScatterThingsAnywhere CreateScenePart_Anywhere(ThingDef thingDef, ThingDef stuffDef, int count)
    {
      ScenPart_ScatterThingsAnywhere scenPart_ScatterThingsAnywhere = new ScenPart_ScatterThingsAnywhere();
      scenPart_ScatterThingsAnywhere.def = ScenPartDefOf.ScatterThingsAnywhere;
      scenPart_ScatterThingsAnywhere.SetMemberValue("thingDef", thingDef);
      scenPart_ScatterThingsAnywhere.SetMemberValue("stuff", stuffDef);
      scenPart_ScatterThingsAnywhere.SetMemberValue("count", count);
      return scenPart_ScatterThingsAnywhere;
    }

    public static ScenPart_StartingAnimal CreateScenePart_Animal(PawnKindDef pkd, int count)
    {
      ScenPart_StartingAnimal scenPart_StartingAnimal = new ScenPart_StartingAnimal();
      scenPart_StartingAnimal.def = ScenPartDefOf.StartingAnimal;
      scenPart_StartingAnimal.SetMemberValue("animalKind", pkd);
      scenPart_StartingAnimal.SetMemberValue("count", count);
      return scenPart_StartingAnimal;
    }

    // Token: 0x06000377 RID: 887 RVA: 0x0001D094 File Offset: 0x0001B294
    //public static void AddScenePart<T>(T scenPart) where T : ScenPart
    //{
    //	bool flag = !scenPart.IsSupportedScenarioPart();
    //	if (!flag)
    //	{
    //		List<ScenPart> list = ScenarioTool.ScenarioParts;
    //		bool flag2 = list == null;
    //		if (flag2)
    //		{
    //			list = new List<ScenPart>();
    //		}
    //		try
    //		{
    //			list.Add(scenPart);
    //			ScenarioTool.SetScenarioParts(list);
    //		}
    //		catch
    //		{
    //			MessageTool.Show("couldn't add scenario parts", MessageTypeDefOf.RejectInput);
    //		}
    //	}
    //}

    public static void AddScenePart<T>(T scenPart) where T : ScenPart
    {
      try
      {
        if (!scenPart.IsSupportedScenarioPart())
        {
          otherPartList.Add(scenPart);
          return;
        }
        EntryInt entryInt = new EntryInt(ScenarioTool.GetScenarioPartCount(scenPart));
        Pair<ScenPart, EntryInt> entryPair = new Pair<ScenPart, EntryInt>(scenPart, entryInt);
        if (scenPart.IsScatterAnywherePart())
        {
          onMapPartEntryList.Add(entryPair);
        }
        else if (scenPart.IsScenarioAnimal())
        {
          animalPartEntryList.Add(entryPair);
        }
        else
        {
          takenPartEntryList.Add(entryPair);
        }
      }
      catch
      {
        MessageTool.Show("couldn't add scenario parts", MessageTypeDefOf.RejectInput);
      }
    }

    public static void AddScenarioPartTaken(ThingDef thingDef, ThingDef stuffDef, ThingStyleDef styleDef, int count)
    {
      ScenarioTool.AddScenePart(ScenarioTool.CreateScenePart_Defined(thingDef, stuffDef, count));
    }

    public static void AddScenarioPartMap(ThingDef thingDef, ThingDef stuffDef, ThingStyleDef styleDef, int count)
    {
      ScenarioTool.AddScenePart(ScenarioTool.CreateScenePart_Anywhere(thingDef, stuffDef, count));
    }

    public static void AddScenarioPartAnimal(PawnKindDef pkd, int count)
    {
      ScenarioTool.AddScenePart(ScenarioTool.CreateScenePart_Animal(pkd, count));
    }

    // Token: 0x0600037B RID: 891 RVA: 0x0001D13C File Offset: 0x0001B33C
    

    //public static void ResetListsFromScenarioParts()
    //{
    //  otherPartList = new List<ScenPart>();
    //  takenPartEntryList = new List<Pair<ScenPart, EntryInt>>();
    //  onMapPartEntryList = new List<Pair<ScenPart, EntryInt>>();
    //  animalPartEntryList = new List<Pair<ScenPart, EntryInt>>();
    //  foreach (ScenPart scenPart in ScenarioTool.ScenarioParts)
    //  {
    //    if (!scenPart.IsSupportedScenarioPart())
    //    {
    //      otherPartList.Add(scenPart);
    //      continue;
    //    }
    //    EntryInt entryInt = new EntryInt(ScenarioTool.GetScenarioPartCount(scenPart));
    //    Pair<ScenPart, EntryInt> entryPair = new Pair<ScenPart, EntryInt>(scenPart, entryInt);
    //    if (scenPart.IsScatterAnywherePart())
    //    {
    //      onMapPartEntryList.Add(entryPair);
    //    }
    //    else if (scenPart.IsScenarioAnimal())
    //    {
    //      animalPartEntryList.Add(entryPair);
    //    }
    //    else
    //    {
    //      takenPartEntryList.Add(entryPair);
    //    }

    //  }
    //}


  }


}
