﻿using System;
using System.Collections.Generic;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000041 RID: 65
	public class Selected
	{
    public ThingStyleDef Style;
    public ThingDef thingDef;
    public ThingDef bulletDef;
    public ThingDef Stuff;
    public SoundDef soundDef;
    public string paramName;
    public string statDefName;
    public string equipStatDefName;
    public string tdcDefName;
    public int OldStuffCount;
    public int quality;
    public int oldQuality;
    public int styleIndex;
    public int oldstyleIndex;
    public int stuffIndex;
    public int oldStuffIndex;
    public int buyPrice;
    public TipSignal tip;
    public string tooltip;
    public PresetWeapon currentGun;
    public PresetApparel currentApparel;
    public Color DrawColor;
    public List<ThingDef> lOfStuff;
    public List<ThingStyleDef> lOfStyle;

    public EntryInt stackValEntryInt = new EntryInt();
    public int oldStackVal;
    public int stackVal
    {
      get
      {
        return stackValEntryInt.valInt;
      }
      set
      {
        stackValEntryInt.UpdateValueAndBuffer(value);
      }
    }
    // Token: 0x17000051 RID: 81
    // (get) Token: 0x06000389 RID: 905 RVA: 0x0001D470 File Offset: 0x0001B670
    public Func<ThingStyleDef, string> StyleLabelGetter
		{
			get
			{
				return (ThingStyleDef x) => (x != null) ? (x.label.NullOrEmpty() ? x.defName : x.label.CapitalizeFirst()) : Label.NONE;
			}
		}

		// Token: 0x17000052 RID: 82
		// (get) Token: 0x0600038A RID: 906 RVA: 0x0001D491 File Offset: 0x0001B691
		public Func<ThingStyleDef, string> StyleDescrGetter
		{
			get
			{
				return (ThingStyleDef x) => (x != null) ? x.description : null;
			}
		}

		// Token: 0x17000053 RID: 83
		// (get) Token: 0x0600038B RID: 907 RVA: 0x0001D4B2 File Offset: 0x0001B6B2
		public Func<ThingDef, string> StuffLabelGetter
		{
			get
			{
				return (ThingDef x) => (x != null) ? (x.label.NullOrEmpty() ? x.defName : x.label.CapitalizeFirst()) : Label.NONE;
			}
		}

		// Token: 0x17000054 RID: 84
		// (get) Token: 0x0600038C RID: 908 RVA: 0x0001D4D3 File Offset: 0x0001B6D3
		public Func<ThingDef, string> StuffDescrGetter
		{
			get
			{
				return (ThingDef x) => (x != null) ? x.description : null;
			}
		}

		// Token: 0x0600038D RID: 909 RVA: 0x0001D4F4 File Offset: 0x0001B6F4
		private Selected(ThingDef t)
		{
			this.Init(t, true, null, null);
		}

		// Token: 0x0600038E RID: 910 RVA: 0x0001D509 File Offset: 0x0001B709
		private Selected(Thing t)
		{
			this.Init(t.def, false, t, null);
		}

		// Token: 0x0600038F RID: 911 RVA: 0x0001D524 File Offset: 0x0001B724
		public static Selected Random(IEnumerable<ThingDef> l, bool originalColors, bool randomStakcount = false)
		{
			bool flag = l.EnumerableNullOrEmpty<ThingDef>();
			Selected result;
			if (flag)
			{
				result = null;
			}
			else
			{
				ThingDef thingDef = l.RandomElement<ThingDef>();
				bool flag2 = thingDef == null;
				if (flag2)
				{
					result = null;
				}
				else
				{
					Selected selected = new Selected(thingDef);
					selected.DrawColor = (originalColors ? selected.DrawColor : ColorTool.RandomColor);
					if (randomStakcount)
					{
						selected.stackVal = CEditor.zufallswert.Next(1, selected.thingDef.stackLimit);
					}
					result = selected;
				}
			}
			return result;
		}

		// Token: 0x06000390 RID: 912 RVA: 0x0001D59C File Offset: 0x0001B79C
		public static Selected ByName(string defname, string stuffdefname, string styledefname, Color col, int quali, int stack)
		{
			ThingDef t = defname.NullOrEmpty() ? null : DefTool.ThingDef(defname);
			ThingDef stuff = stuffdefname.NullOrEmpty() ? null : DefTool.ThingDef(stuffdefname);
			ThingStyleDef style = styledefname.NullOrEmpty() ? null : DefTool.ThingStyleDef(styledefname);
			Selected selected = new Selected(t);
			selected.Set(stuff, style, col, quali, stack);
			return selected;
		}

		// Token: 0x06000391 RID: 913 RVA: 0x0001D5FC File Offset: 0x0001B7FC
		public static Selected ByThing(Thing _thing)
		{
			return new Selected(_thing);
		}

		// Token: 0x06000392 RID: 914 RVA: 0x0001D604 File Offset: 0x0001B804
		public static Selected ByThingDef(ThingDef _thingDef)
		{
			return new Selected(_thingDef);
		}

		// Token: 0x06000393 RID: 915 RVA: 0x0001D60C File Offset: 0x0001B80C
		private void Set(ThingDef stuff, ThingStyleDef style, Color col, int quali, int stack)
		{
			this.Stuff = stuff;
			this.stuffIndex = ((this.Stuff != null) ? this.lOfStuff.IndexOf(this.Stuff) : 0);
			this.Style = style;
			this.styleIndex = ((this.Style != null) ? this.lOfStyle.IndexOf(this.Style) : 0);
			this.quality = ((quali < 0) ? ((int)QualityUtility.AllQualityCategories.RandomElement<QualityCategory>()) : quali);
			this.DrawColor = col;
			this.stackVal = stack;
		}

		// Token: 0x06000394 RID: 916 RVA: 0x0001D694 File Offset: 0x0001B894
		public void SetStyle(bool next, bool random)
		{
			this.styleIndex = this.lOfStyle.NextOrPrevIndex(this.styleIndex, next, random);
			this.oldstyleIndex = this.styleIndex;
			this.Style = this.lOfStyle.At(this.styleIndex);
			this.UpdateBuyPrice();
		}

		// Token: 0x06000395 RID: 917 RVA: 0x0001D6E5 File Offset: 0x0001B8E5
		public void SetStyle(ThingStyleDef _style)
		{
			this.styleIndex = this.lOfStyle.IndexOf(_style);
			this.oldstyleIndex = this.styleIndex;
			this.Style = _style;
			this.UpdateBuyPrice();
		}

		// Token: 0x06000396 RID: 918 RVA: 0x0001D714 File Offset: 0x0001B914
		public void CheckSetStyle()
		{
			bool flag = this.styleIndex != this.oldstyleIndex;
			if (flag)
			{
				this.SetStyle(this.lOfStyle.At(this.styleIndex));
			}
		}

		// Token: 0x06000397 RID: 919 RVA: 0x0001D750 File Offset: 0x0001B950
		public void SetStuff(bool next, bool random)
		{
			this.stuffIndex = this.lOfStuff.NextOrPrevIndex(this.stuffIndex, next, random);
			this.oldStuffIndex = this.stuffIndex;
			this.Stuff = this.lOfStuff.At(this.stuffIndex);
			this.DrawColor = this.thingDef.GetColor(this.Stuff);
			this.UpdateBuyPrice();
		}

		// Token: 0x06000398 RID: 920 RVA: 0x0001D7B8 File Offset: 0x0001B9B8
		public void SetStuff(ThingDef _stuff)
		{
			this.stuffIndex = this.lOfStuff.IndexOf(_stuff);
			this.oldStuffIndex = this.stuffIndex;
			this.Stuff = _stuff;
			this.DrawColor = this.thingDef.GetColor(this.Stuff);
			this.UpdateBuyPrice();
		}

		// Token: 0x06000399 RID: 921 RVA: 0x0001D80C File Offset: 0x0001BA0C
		public void CheckSetStuff()
		{
			bool flag = this.stuffIndex != this.oldStuffIndex;
			if (flag)
			{
				this.SetStuff(this.lOfStuff.At(this.stuffIndex));
			}
		}

		// Token: 0x0600039A RID: 922 RVA: 0x0001D848 File Offset: 0x0001BA48
		public void UpdateStuffList()
		{
			this.Stuff = ((this.thingDef != null) ? this.thingDef.GetStuff(ref this.lOfStuff, ref this.stuffIndex, false) : null);
			this.UpdateBuyPrice();
			ThingDef thingDef = this.thingDef;
			if (thingDef != null)
			{
				thingDef.UpdateRecipes();
			}
		}

		// Token: 0x0600039B RID: 923 RVA: 0x0001D898 File Offset: 0x0001BA98
		public void Init(ThingDef _thingDef, bool random = true, Thing thing = null, Action postProcess = null)
		{
			this.thingDef = _thingDef;
			this.currentGun = new PresetWeapon(_thingDef);
			this.currentApparel = new PresetApparel(_thingDef);
			this.bulletDef = DefTool.ThingDef(this.currentGun.bulletName);
			this.soundDef = DefTool.SoundDef(this.currentGun.soundName);
			this.paramName = null;
			this.statDefName = null;
			this.equipStatDefName = null;
			this.tdcDefName = null;
			this.Stuff = ((this.thingDef != null) ? this.thingDef.GetStuff(ref this.lOfStuff, ref this.stuffIndex, random) : null);
			this.DrawColor = ((this.Stuff != null) ? this.thingDef.GetColor(this.Stuff) : ((this.thingDef != null) ? this.thingDef.uiIconColor : Color.white));
			this.Style = ((this.thingDef != null) ? this.thingDef.GetStyle(ref this.lOfStyle, ref this.styleIndex, random) : null);
			this.RandomQuality();
			bool flag = thing != null;
			if (flag)
			{
				SZWidgets.tDefName = thing.def.defName;
				this.Set(thing.Stuff, thing.StyleDef, thing.DrawColor, thing.GetQuality(), thing.stackCount);
			}
			this.oldQuality = this.quality;
			this.stuffIndex = ((this.Stuff == null) ? 0 : this.stuffIndex);
			this.oldStuffIndex = ((this.Stuff == null) ? 0 : this.stuffIndex);
			this.OldStuffCount = 0;
			this.styleIndex = ((this.Style == null) ? 0 : this.styleIndex);
			this.oldstyleIndex = ((this.Style == null) ? 0 : this.styleIndex);
			this.buyPrice = 0;
			this.stackVal = 1;
			this.oldStackVal = 0;
			this.tooltip = "";
			this.tip = new TipSignal(() => this.tooltip, 943275);
			this.UpdateBuyPrice();
			if (Prefs.DevMode && this.thingDef != null)
			{
				//Log.Message("selected=" + this.thingDef.defName);
      }
			if (postProcess != null)
			{
				postProcess();
			}
		}

		// Token: 0x0600039C RID: 924 RVA: 0x0001DAD8 File Offset: 0x0001BCD8
		public void UpdateBuyPrice()
		{
			bool flag = this.thingDef == null;
			if (!flag)
			{
				bool flag2 = this.thingDef.HasComp(typeof(CompQuality));
				double num;
				if (flag2)
				{
					num = 0.333333333 + (double)this.quality * 0.333333333;
					bool flag3 = this.quality == 6;
					if (flag3)
					{
						num += 1.0;
					}
				}
				else
				{
					num = 1.0;
				}
				double num2 = (double)this.thingDef.GetStatValueAbstract(StatDefOf.MarketValue, this.Stuff) * num;
				double num3 = num2 * (double)this.stackVal;
				bool flag4 = num3 < 1.0;
				if (flag4)
				{
					this.buyPrice = 1;
				}
				else
				{
					this.buyPrice = (int)Math.Round(num3);
				}
			}
		}

		public void RandomQuality()
		{
			this.quality = (int)QualityUtility.AllQualityCategories.RandomElement<QualityCategory>();
		}
		
	}
}
