﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200001D RID: 29
	public static class StyleTool
	{
		// Token: 0x06000106 RID: 262 RVA: 0x000083A7 File Offset: 0x000065A7
		public static string GetBeardDefName(this Pawn p)
		{
			return (p.HasStyleTracker() && p.style.beardDef != null) ? p.style.beardDef.defName : "";
		}

		// Token: 0x06000107 RID: 263 RVA: 0x000083D8 File Offset: 0x000065D8
		public static string GetBeardName(this Pawn p)
		{
			return (p.HasStyleTracker() && p.style.beardDef != null) ? p.style.beardDef.LabelCap.ToString() : "";
		}

		// Token: 0x06000108 RID: 264 RVA: 0x0000841F File Offset: 0x0000661F
		public static List<BeardDef> GetBeardList(string modname)
		{
			return DefTool.ListByMod<BeardDef>(modname);
		}

		// Token: 0x06000109 RID: 265 RVA: 0x00008428 File Offset: 0x00006628
		public static bool SetBeard(this Pawn pawn, bool next, bool random)
		{
			bool flag = pawn == null || pawn.style == null;
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				List<BeardDef> list = StyleTool.GetBeardList(null).ToList<BeardDef>();
				bool flag2 = list.EnumerableNullOrEmpty<BeardDef>();
				if (flag2)
				{
					result = false;
				}
				else
				{
					BeardDef beardDef = pawn.style.beardDef;
					int index = list.IndexOf(beardDef);
					index = list.NextOrPrevIndex(index, next, random);
					BeardDef beardDef2 = list[index];
					pawn.SetBeard(beardDef2);
					result = true;
				}
			}
			return result;
		}

		// Token: 0x0600010A RID: 266 RVA: 0x000084A4 File Offset: 0x000066A4
		public static void SetBeard(this Pawn pawn, BeardDef beardDef)
		{
			bool flag = pawn == null || pawn.style == null;
			if (!flag)
			{
				pawn.style.beardDef = beardDef;
			}
		}

		// Token: 0x0600010B RID: 267 RVA: 0x000084D3 File Offset: 0x000066D3
		public static string GetFaceTattooDefName(this Pawn p)
		{
			return (p.HasStyleTracker() && p.style.FaceTattoo != null) ? p.style.FaceTattoo.defName : "";
		}

		// Token: 0x0600010C RID: 268 RVA: 0x00008504 File Offset: 0x00006704
		public static string GetFaceTattooName(this Pawn p)
		{
			return (p.HasStyleTracker() && p.style.FaceTattoo != null) ? p.style.FaceTattoo.LabelCap.ToString() : "";
		}

		// Token: 0x0600010D RID: 269 RVA: 0x0000854B File Offset: 0x0000674B
		public static List<TattooDef> GetFaceTattooList(string modname)
		{
			return DefTool.ListByMod<TattooDef>(modname, (TattooDef x) => x.tattooType == TattooType.Face);
		}

		// Token: 0x0600010E RID: 270 RVA: 0x00008574 File Offset: 0x00006774
		public static bool SetFaceTattoo(this Pawn pawn, bool next, bool random)
		{
			bool flag = pawn == null || pawn.style == null;
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				List<TattooDef> list = StyleTool.GetFaceTattooList(null).ToList<TattooDef>();
				bool flag2 = list.EnumerableNullOrEmpty<TattooDef>();
				if (flag2)
				{
					result = false;
				}
				else
				{
					TattooDef faceTattoo = pawn.style.FaceTattoo;
					int index = list.IndexOf(faceTattoo);
					index = list.NextOrPrevIndex(index, next, random);
					TattooDef tattooDef = list[index];
					pawn.SetFaceTattoo(tattooDef);
					result = true;
				}
			}
			return result;
		}

		// Token: 0x0600010F RID: 271 RVA: 0x000085F0 File Offset: 0x000067F0
		public static void SetFaceTattoo(this Pawn pawn, TattooDef tattooDef)
		{
			bool flag = pawn == null || pawn.style == null;
			if (!flag)
			{
				pawn.style.SetMemberValue("faceTattoo", tattooDef);
			}
		}

		// Token: 0x06000110 RID: 272 RVA: 0x00008625 File Offset: 0x00006825
		public static string GetBodyTattooDefName(this Pawn p)
		{
			return (p.HasStyleTracker() && p.style.BodyTattoo != null) ? p.style.BodyTattoo.defName : "";
		}

		// Token: 0x06000111 RID: 273 RVA: 0x00008654 File Offset: 0x00006854
		public static string GetBodyTattooName(this Pawn p)
		{
			return (p.HasStyleTracker() && p.style.BodyTattoo != null) ? p.style.BodyTattoo.LabelCap.ToString() : "";
		}

		// Token: 0x06000112 RID: 274 RVA: 0x0000869B File Offset: 0x0000689B
		public static List<TattooDef> GetBodyTattooList(string modname)
		{
      return DefTool.ListByMod<TattooDef>(modname, (TattooDef x) => x.tattooType == TattooType.Body);
		}

		// Token: 0x06000113 RID: 275 RVA: 0x000086C4 File Offset: 0x000068C4
		public static bool SetBodyTattoo(this Pawn pawn, bool next, bool random)
		{
			bool flag = pawn == null || pawn.style == null;
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				List<TattooDef> list = StyleTool.GetBodyTattooList(null).ToList<TattooDef>();
				bool flag2 = list.EnumerableNullOrEmpty<TattooDef>();
				if (flag2)
				{
					result = false;
				}
				else
				{
					TattooDef bodyTattoo = pawn.style.BodyTattoo;
					int index = list.IndexOf(bodyTattoo);
					index = list.NextOrPrevIndex(index, next, random);
					TattooDef tattooDef = list[index];
					pawn.SetBodyTattoo(tattooDef);
					result = true;
				}
			}
			return result;
		}

		// Token: 0x06000114 RID: 276 RVA: 0x00008740 File Offset: 0x00006940
		public static void SetBodyTattoo(this Pawn pawn, TattooDef tattooDef)
		{
			bool flag = pawn == null || pawn.style == null;
			if (!flag)
			{
				pawn.style.SetMemberValue("bodyTattoo", tattooDef);
			}
		}
	}
}
