﻿using System;
using RimWorld;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x0200005D RID: 93
	public static class TextureTool
	{
		// Token: 0x060005E1 RID: 1505 RVA: 0x0003E188 File Offset: 0x0003C388
		public static Texture2D GetTextureFromStackCount(this ThingDef t)
		{
			Texture2D result = null;
			bool flag = t != null && t.graphic != null && t.graphic.GetType() == typeof(Graphic_StackCount);
			if (flag)
			{
				Graphic_StackCount graphic_StackCount = t.graphic as Graphic_StackCount;
				Graphic graphic = graphic_StackCount.SubGraphicForStackCount(100, t);
				result = ContentFinder<Texture2D>.Get(graphic.path, false);
			}
			return result;
		}

		// Token: 0x060005E2 RID: 1506 RVA: 0x0003E1F0 File Offset: 0x0003C3F0
		public static Texture2D GetTextureFromMulti(this Graphic g, string rotate = "_south")
		{
			Graphic_Multi graphic_Multi = g as Graphic_Multi;
			return ContentFinder<Texture2D>.Get(graphic_Multi.path + rotate, false);
		}

		// Token: 0x060005E3 RID: 1507 RVA: 0x0003E220 File Offset: 0x0003C420
		public static string Rot4ToString(Rot4 rot4)
		{
			bool flag = rot4 == Rot4.North;
			string result;
			if (flag)
			{
				result = "_north";
			}
			else
			{
				bool flag2 = rot4 == Rot4.South;
				if (flag2)
				{
					result = "_south";
				}
				else
				{
					bool flag3 = rot4 == Rot4.East;
					if (flag3)
					{
						result = "_east";
					}
					else
					{
						bool flag4 = rot4 == Rot4.West;
						if (flag4)
						{
							result = "_west";
						}
						else
						{
							result = "_north";
						}
					}
				}
			}
			return result;
		}

		// Token: 0x060005E4 RID: 1508 RVA: 0x0003E298 File Offset: 0x0003C498
		public static Texture2D GetTexture(this ThingDef t, int stackCount = 1, ThingStyleDef tsd = null, Rot4 rotation = default(Rot4))
		{
			Texture2D texture2D = null;
			bool flag = texture2D == null && t != null;
			if (flag)
			{
				GraphicData graphicData = (tsd != null) ? tsd.graphicData : t.graphicData;
				graphicData = (graphicData ?? t.graphicData);
				bool flag2 = graphicData != null;
				if (flag2)
				{
					bool flag3 = graphicData.graphicClass == typeof(Graphic_Multi);
					if (flag3)
					{
						texture2D = TextureTool.GetGraphicForMulti(graphicData, rotation);
					}
					else
					{
						bool flag4 = graphicData.graphicClass == typeof(Graphic_Random);
						if (flag4)
						{
							texture2D = TextureTool.GetGraphicForRandom(graphicData);
						}
						else
						{
							bool flag5 = graphicData.graphicClass == typeof(Graphic_StackCount);
							if (flag5)
							{
								texture2D = TextureTool.GetGraphicForStackCount(graphicData, t, stackCount);
							}
							else
							{
								bool flag6 = graphicData.graphicClass == typeof(Graphic_Single);
								if (flag6)
								{
									texture2D = TextureTool.GetGraphicForSingle(graphicData);
								}
								else
								{
									texture2D = TextureTool.GetGraphicForUndefined(graphicData);
								}
							}
						}
					}
				}
			}
			bool flag7 = texture2D == null && t.uiIcon != null;
			Texture2D result;
			if (flag7)
			{
				result = t.uiIcon;
			}
			else
			{
				result = texture2D;
			}
			return result;
		}

		// Token: 0x060005E5 RID: 1509 RVA: 0x0003E3BF File Offset: 0x0003C5BF
		public static Texture2D GetGraphicForUndefined(GraphicData g)
		{
			return (g != null) ? ContentFinder<Texture2D>.Get(g.texPath, false) : null;
		}

		// Token: 0x060005E6 RID: 1510 RVA: 0x0003E3D4 File Offset: 0x0003C5D4
		public static Texture2D GetGraphicForMulti(GraphicData g, Rot4 rotation)
		{
			string str = TextureTool.Rot4ToString(rotation);
			Texture2D texture2D = ContentFinder<Texture2D>.Get((g.Graphic as Graphic_Multi).path + str, false);
			bool flag = texture2D == null;
			if (flag)
			{
				bool flag2 = rotation == Rot4.West;
				if (flag2)
				{
					texture2D = ContentFinder<Texture2D>.Get((g.Graphic as Graphic_Multi).path + "_east", false);
				}
				bool flag3 = texture2D == null;
				if (flag3)
				{
					texture2D = ContentFinder<Texture2D>.Get((g.Graphic as Graphic_Multi).path + "_north", false);
				}
			}
			return texture2D;
		}

		// Token: 0x060005E7 RID: 1511 RVA: 0x0003E478 File Offset: 0x0003C678
		public static Texture2D GetGraphicForRandom(GraphicData g)
		{
			return ContentFinder<Texture2D>.Get((g.Graphic as Graphic_Random).FirstSubgraphic().path, false);
		}

		// Token: 0x060005E8 RID: 1512 RVA: 0x0003E498 File Offset: 0x0003C698
		public static Texture2D GetGraphicForSingle(GraphicData g)
		{
			bool flag = g.Graphic == null;
			Texture2D result;
			if (flag)
			{
				result = ContentFinder<Texture2D>.Get(g.texPath, false);
			}
			else
			{
				try
				{
					result = ContentFinder<Texture2D>.Get((g.Graphic as Graphic_Single).path, false);
				}
				catch
				{
					result = ContentFinder<Texture2D>.Get(g.texPath, false);
				}
			}
			return result;
		}

		public static Texture2D GetGraphicForStackCount(GraphicData g, ThingDef t, int stackVal)
		{
			Texture2D result;
			try
			{
				result = ContentFinder<Texture2D>.Get((g.Graphic as Graphic_StackCount).SubGraphicForStackCount(stackVal, t).path, false);
			}
			catch
			{
				result = ContentFinder<Texture2D>.Get(g.Graphic.path, true);
			}
			return result;
		}

		public static void SetGraphicDataSingle(this ThingDef td, string texPath, string uiTexPath)
		{
			bool flag = td != null;
			if (flag)
			{
				td.graphicData = new GraphicData();
				td.graphicData.texPath = texPath;
				td.graphicData.shaderType = ShaderTypeDefOf.MetaOverlay;
				td.graphicData.graphicClass = typeof(Graphic_Single);
				td.uiIconPath = uiTexPath;
			}
		}

		// Token: 0x060005EB RID: 1515 RVA: 0x0003E5B4 File Offset: 0x0003C7B4
		public static bool TestTexturePath(string path, bool showError = true)
		{
			Texture2D x = ContentFinder<Texture2D>.Get(path, false);
			bool flag = x == null && showError;
			if (flag)
			{
				MessageTool.Show("Missing Texture=" + path, MessageTypeDefOf.RejectInput);
			}
			return x != null;
		}
	}
}
