﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using HarmonyLib;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace CharacterEditor
{
  // Token: 0x0200005E RID: 94
  public static class ThingTool
  {
    // Token: 0x060005EC RID: 1516 RVA: 0x0003E5F8 File Offset: 0x0003C7F8
    public static bool HasAnyItem(this Pawn pawn)
    {
      return pawn.HasInventoryTracker() && !pawn.inventory.innerContainer.NullOrEmpty<Thing>();
    }

    // Token: 0x060005ED RID: 1517 RVA: 0x0003E618 File Offset: 0x0003C818
    public static Thing RandomInventoryItem(this Pawn pawn)
    {
      return pawn.inventory.innerContainer.RandomElement<Thing>();
    }

    // Token: 0x060005EE RID: 1518 RVA: 0x0003E62C File Offset: 0x0003C82C
    public static void Spawn(this Thing t, Rot4 rot4 = default(Rot4), IntVec3 pos = default(IntVec3))
    {
      bool flag = t == null;
      if (!flag)
      {
        bool flag2 = pos == default(IntVec3);
        if (flag2)
        {
          pos = UI.MouseCell();
        }
        bool flag3 = !pos.InBounds(Find.CurrentMap);
        if (flag3)
        {
          pos = Find.CurrentMap.AllCells.RandomElement<IntVec3>();
        }
        GenSpawn.Spawn(t, UI.MouseCell(), Find.CurrentMap, rot4, WipeMode.Vanish, false);
      }
    }

    // Token: 0x060005EF RID: 1519 RVA: 0x0003E698 File Offset: 0x0003C898
    public static string GetAsSeparatedString(this Thing t)
    {
      bool flag = t == null || t.def.IsNullOrEmpty();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        string text = "";
        text = text + t.def.defName + "|";
        text = text + t.GetStuffDefName() + "|";
        text = text + t.DrawColor.ColorHexString() + "|";
        text = text + t.GetQuality().ToString() + "|";
        text = text + t.stackCount.ToString() + "|";
        text += t.HitPoints.ToString();
        bool flag2 = t.StyleDef != null;
        if (flag2)
        {
          text += "|";
          text += t.StyleDef.defName;
        }
        result = text;
      }
      return result;
    }

    // Token: 0x060005F0 RID: 1520 RVA: 0x0003E788 File Offset: 0x0003C988
    public static string GetAllItemsAsSeparatedString(this Pawn p)
    {
      bool flag = !p.HasInventoryTracker() || p.inventory.innerContainer.NullOrEmpty<Thing>();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        string text = "";
        foreach (Thing t in p.inventory.innerContainer)
        {
          text += t.GetAsSeparatedString();
          text += ":";
        }
        text = text.SubstringRemoveLast();
        result = text;
      }
      return result;
    }

    // Token: 0x060005F1 RID: 1521 RVA: 0x0003E834 File Offset: 0x0003CA34
    public static void SetItemsFromSeparatedString(this Pawn p, string s)
    {
      bool flag = !p.HasInventoryTracker();
      if (!flag)
      {
        p.inventory.DestroyAll(DestroyMode.Vanish);
        bool flag2 = s.NullOrEmpty();
        if (!flag2)
        {
          string[] array = s.SplitNo(":");
          foreach (string s2 in array)
          {
            string[] array3 = s2.SplitNo("|");
            bool flag3 = array3.Length >= 6;
            if (flag3)
            {
              string styledefname = (array3.Length >= 7) ? array3[6] : "";
              Thing thing = ThingTool.GenerateItem(Selected.ByName(array3[0], array3[1], styledefname, array3[2].HexStringToColor(), array3[3].AsInt32(), array3[4].AsInt32()));
              bool flag4 = thing != null;
              if (flag4)
              {
                thing.HitPoints = array3[5].AsInt32();
                p.inventory.innerContainer.TryAdd(thing, true);
              }
            }
          }
        }
      }
    }

    // Token: 0x060005F2 RID: 1522 RVA: 0x0003E938 File Offset: 0x0003CB38
    public static void DestroyAllItems(this Pawn pawn)
    {
      bool flag = !pawn.HasInventoryTracker();
      if (!flag)
      {
        pawn.inventory.innerContainer.ClearAndDestroyContents(DestroyMode.Vanish);
      }
    }

    // Token: 0x060005F3 RID: 1523 RVA: 0x0003E968 File Offset: 0x0003CB68
    public static void DestroyItem(this Pawn pawn, Thing t)
    {
      bool flag = pawn.HasInventoryTracker() && t != null;
      if (flag)
      {
        pawn.inventory.innerContainer.Remove(t);
        bool flag2 = t.def.IsApparel && pawn.HasApparelTracker();
        if (flag2)
        {
          pawn.outfits.forcedHandler.ForcedApparel.Remove(t as Apparel);
        }
        t.Destroy(DestroyMode.Vanish);
      }
    }

    // Token: 0x060005F4 RID: 1524 RVA: 0x0003E9DB File Offset: 0x0003CBDB
    public static List<Thing> ListOfCopyItems(this Pawn pawn)
    {
      return pawn.HasAnyItem() ? pawn.inventory.innerContainer.InnerListForReading.ListFullCopy<Thing>() : null;
    }

    // Token: 0x060005F5 RID: 1525 RVA: 0x0003EA00 File Offset: 0x0003CC00
    public static void PasteCopyItems(this Pawn pawn, List<Thing> l)
    {
      bool flag = pawn.HasInventoryTracker();
      if (flag)
      {
        pawn.DestroyAllItems();
        bool flag2 = !l.NullOrEmpty<Thing>();
        if (flag2)
        {
          foreach (Thing thing in l)
          {
            Thing t = ThingTool.GenerateItem(Selected.ByThing(thing));
            pawn.AddItemToInventory(t);
          }
        }
      }
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x060005F6 RID: 1526 RVA: 0x0003EA90 File Offset: 0x0003CC90
    public static void AddItemToInventory(this Pawn pawn, Thing t)
    {
      bool flag = pawn.HasInventoryTracker() && t != null;
      if (flag)
      {
        pawn.inventory.innerContainer.TryAdd(t, true);
      }
    }

    // Token: 0x060005F7 RID: 1527 RVA: 0x0003EAC8 File Offset: 0x0003CCC8
    public static void TransferToInventory(this Pawn pawn, Thing t)
    {
      bool flag = pawn.HasInventoryTracker() && t != null;
      if (flag)
      {
        bool flag2 = pawn.HasApparelTracker() && pawn.HasForcedApparel() && t.def.IsApparel;
        if (flag2)
        {
          pawn.outfits.forcedHandler.ForcedApparel.Remove(t as Apparel);
        }
        bool flag3 = pawn.inventory.innerContainer == null;
        if (flag3)
        {
          pawn.inventory.innerContainer = new ThingOwner<Thing>();
        }
        pawn.inventory.innerContainer.TryAddOrTransfer(t, true);
      }
    }

    // Token: 0x060005F8 RID: 1528 RVA: 0x0003EB60 File Offset: 0x0003CD60
    public static void TransferFromInventory(this Pawn pawn, Thing t)
    {
      bool flag = pawn.HasInventoryTracker() && t != null;
      if (flag)
      {
        bool flag2 = pawn.inventory.innerContainer == null;
        if (flag2)
        {
          pawn.inventory.innerContainer = new ThingOwner<Thing>();
        }
        bool flag3 = pawn.HasApparelTracker() && t.def.IsApparel;
        if (flag3)
        {
          pawn.inventory.innerContainer.Remove(t);
          pawn.WearThatApparel(t as Apparel);
        }
        else
        {
          bool flag4 = t.def.IsWeapon && pawn.HasEquipmentTracker();
          if (flag4)
          {
            pawn.inventory.innerContainer.Remove(t);
            pawn.AddWeaponToEquipment(t as ThingWithComps, true, false);
          }
        }
      }
    }

    // Token: 0x060005F9 RID: 1529 RVA: 0x0003EC24 File Offset: 0x0003CE24
    public static void CreateAndAddItem(this Pawn pawn, Selected s)
    {
      bool flag = pawn.HasInventoryTracker();
      if (flag)
      {
        Thing t = ThingTool.GenerateItem(s);
        pawn.AddItemToInventory(t);
      }
    }

    // Token: 0x060005FA RID: 1530 RVA: 0x0003EC50 File Offset: 0x0003CE50
    public static Thing GenerateItem(Selected s)
    {
      bool flag = s == null || s.thingDef == null;
      Thing result;
      if (flag)
      {
        result = null;
      }
      else
      {
        ThingDef thingDef = DefTool.ThingDef(s.thingDef.defName);
        bool flag2 = thingDef == null;
        if (flag2)
        {
          result = null;
        }
        else
        {
          s.Stuff = s.thingDef.ThisOrDefaultStuff(s.Stuff);
          bool flag3 = !s.thingDef.MadeFromStuff;
          if (flag3)
          {
            s.Stuff = null;
          }
          Thing thing = ThingMaker.MakeThing(s.thingDef, s.Stuff);
          thing.HitPoints = thing.MaxHitPoints;
          thing.SetQuality(s.quality);
          thing.SetDrawColor(s.DrawColor);
          thing.stackCount = s.stackVal;
          bool flag4 = s.Style != null;
          if (flag4)
          {
            thing.StyleDef = s.Style;
            thing.StyleDef.color = s.Style.color;
          }
          result = thing;
        }
      }
      return result;
    }

    // Token: 0x060005FB RID: 1531 RVA: 0x0003ED4C File Offset: 0x0003CF4C
    public static Thing GenerateRandomItem(ThingCategoryDef tc, bool originalColors = true, bool randomStack = true)
    {
      List<ThingDef> l = ThingTool.ListOfItems(tc, null);
      return ThingTool.GenerateItem(Selected.Random(l, originalColors, randomStack));
    }

    // Token: 0x060005FC RID: 1532 RVA: 0x0003ED74 File Offset: 0x0003CF74
    public static bool IsMineableMineral(this ThingDef td)
    {
      return td != null && td.defName != null && td.building != null && td.building.mineableThing != null && td.building.isResourceRock;
    }

    // Token: 0x060005FD RID: 1533 RVA: 0x0003EDB4 File Offset: 0x0003CFB4
    public static bool IsMineableRock(this ThingDef td)
    {
      return td != null && td.defName != null && td.building != null && td.building.mineableThing != null && !td.building.isResourceRock;
    }

    // Token: 0x060005FE RID: 1534 RVA: 0x0003EDF8 File Offset: 0x0003CFF8
    public static ThingDef GetBaseThingDefFromMinified(this ThingDef miniThingDef)
    {
      ThingDef result = null;
      bool flag = miniThingDef != null && miniThingDef.defName != null && miniThingDef.defName.Contains("Minified");
      if (flag)
      {
        string defName = miniThingDef.defName.Replace("Minified", "");
        result = DefTool.ThingDef(defName);
      }
      return result;
    }

    // Token: 0x060005FF RID: 1535 RVA: 0x0003EE50 File Offset: 0x0003D050
    public static bool IsMinified(this ThingDef t)
    {
      return t != null && t.defName != null && t.defName.Contains("Minified");
    }

    // Token: 0x06000600 RID: 1536 RVA: 0x0003EE80 File Offset: 0x0003D080
    public static List<StatCategoryDef> GetAllStatCategoriesApparel()
    {
      List<StatCategoryDef> lequip = ThingTool.GetAllStatCategoriesOnEquip();
      List<StatCategoryDef> list = (from td in DefDatabase<StatCategoryDef>.AllDefs
                                    where td != StatCategoryDefOf.Building && !lequip.Contains(td)
                                    orderby td.label
                                    select td).ToList<StatCategoryDef>();
      list.Remove(StatCategoryDefOf.Weapon);
      list.RemoveCategoriesWithoutStats();
      list.Insert(0, null);
      return list;
    }

    // Token: 0x06000601 RID: 1537 RVA: 0x0003EF00 File Offset: 0x0003D100
    public static List<StatCategoryDef> GetAllStatCategoriesOnEquip()
    {
      List<StatCategoryDef> list = new List<StatCategoryDef>();
      list = new List<StatCategoryDef>();
      list.Add(StatCategoryDefOf.BasicsPawn);
      list.Add(StatCategoryDefOf.EquippedStatOffsets);
      list.Add(StatCategoryDefOf.PawnCombat);
      list.Add(StatCategoryDefOf.PawnMisc);
      list.Add(StatCategoryDefOf.PawnSocial);
      list.Add(StatCategoryDefOf.PawnWork);
      list.Add(StatCategoryDefOf.StuffOfEquipmentStatFactors);
      list.RemoveCategoriesWithoutStats();
      list.Insert(0, null);
      return list;
    }

    // Token: 0x06000602 RID: 1538 RVA: 0x0003EF84 File Offset: 0x0003D184
    public static List<StatCategoryDef> GetAllStatCategoriesWeapon()
    {
      List<StatCategoryDef> lequip = ThingTool.GetAllStatCategoriesOnEquip();
      List<StatCategoryDef> list = (from td in DefDatabase<StatCategoryDef>.AllDefs
                                    where td != StatCategoryDefOf.Building && !lequip.Contains(td)
                                    orderby td.label
                                    select td).ToList<StatCategoryDef>();
      list.Remove(StatCategoryDefOf.Apparel);
      list.RemoveCategoriesWithoutStats();
      list.Insert(0, null);
      return list;
    }

    // Token: 0x06000603 RID: 1539 RVA: 0x0003F004 File Offset: 0x0003D204
    public static bool HasStatsForCategory(StatCategoryDef s)
    {
      foreach (StatDef statDef in DefDatabase<StatDef>.AllDefs)
      {
        bool flag = statDef.category == s;
        if (flag)
        {
          return true;
        }
      }
      return false;
    }

    // Token: 0x06000604 RID: 1540 RVA: 0x0003F064 File Offset: 0x0003D264
    public static void RemoveCategoriesWithoutStats(this List<StatCategoryDef> l)
    {
      bool flag = l != null;
      if (flag)
      {
        for (int i = 0; i < l.Count; i++)
        {
          bool flag2 = !ThingTool.HasStatsForCategory(l[i]);
          if (flag2)
          {
            l.RemoveAt(i);
            i--;
          }
        }
      }
    }

    // Token: 0x06000605 RID: 1541 RVA: 0x0003F0B4 File Offset: 0x0003D2B4
    public static void RemoveCategoriesWithoutThings(this List<ThingCategoryDef> l)
    {
      bool flag = l != null;
      if (flag)
      {
        Dictionary<ThingCategoryDef, int> dictionary = new Dictionary<ThingCategoryDef, int>();
        foreach (ThingCategoryDef key in l)
        {
          dictionary.Add(key, 0);
        }
        foreach (ThingDef thingDef in DefDatabase<ThingDef>.AllDefs)
        {
          bool flag2 = !thingDef.thingCategories.NullOrEmpty<ThingCategoryDef>();
          if (flag2)
          {
            foreach (ThingCategoryDef thingCategoryDef in thingDef.thingCategories)
            {
              bool flag3 = dictionary.ContainsKey(thingCategoryDef);
              if (flag3)
              {
                Dictionary<ThingCategoryDef, int> dictionary2 = dictionary;
                ThingCategoryDef key2 = thingCategoryDef;
                int num = dictionary2[key2];
                dictionary2[key2] = num + 1;
              }
            }
          }
        }
        foreach (ThingCategoryDef thingCategoryDef2 in dictionary.Keys)
        {
          bool flag4 = dictionary[thingCategoryDef2] == 0;
          if (flag4)
          {
            l.Remove(thingCategoryDef2);
          }
        }
      }
    }

    // Token: 0x06000606 RID: 1542 RVA: 0x0003F23C File Offset: 0x0003D43C
    public static void AddEquipStat(this ThingDef t, StatDef stat, float value)
    {
      bool flag = t != null && stat != null;
      if (flag)
      {
        bool flag2 = t.HasEquipStat(stat);
        if (flag2)
        {
          t.UpdateEquipStat(stat, value);
        }
        else
        {
          StatModifier statModifier = new StatModifier();
          statModifier.stat = stat;
          statModifier.value = value;
          bool flag3 = t.equippedStatOffsets == null;
          if (flag3)
          {
            t.equippedStatOffsets = new List<StatModifier>();
          }
          t.equippedStatOffsets.Add(statModifier);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000607 RID: 1543 RVA: 0x0003F2B4 File Offset: 0x0003D4B4
    public static void AddStat(this ThingDef t, StatDef stat, float value)
    {
      bool flag = t != null && stat != null;
      if (flag)
      {
        bool flag2 = t.HasStat(stat);
        if (flag2)
        {
          t.UpdateStat(stat, value);
        }
        else
        {
          StatModifier statModifier = new StatModifier();
          statModifier.stat = stat;
          statModifier.value = value;
          bool flag3 = t.statBases == null;
          if (flag3)
          {
            t.statBases = new List<StatModifier>();
          }
          t.statBases.Add(statModifier);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000608 RID: 1544 RVA: 0x0003F32C File Offset: 0x0003D52C
    public static List<StatDef> GetAllApparelStatDefs()
    {
      List<StatCategoryDef> lequip = ThingTool.GetAllStatCategoriesOnEquip();
      return (from td in DefDatabase<StatDef>.AllDefs
              where !lequip.Contains(td.category) || td.category == StatCategoryDefOf.Apparel
              orderby td.label
              select td).ToList<StatDef>();
    }

    // Token: 0x06000609 RID: 1545 RVA: 0x0003F390 File Offset: 0x0003D590
    public static List<StatDef> GetAllOnEquipStatDefs()
    {
      List<StatCategoryDef> lequip = ThingTool.GetAllStatCategoriesOnEquip();
      return (from td in DefDatabase<StatDef>.AllDefs
              where lequip.Contains(td.category)
              orderby td.label
              select td).ToList<StatDef>();
    }

    // Token: 0x0600060A RID: 1546 RVA: 0x0003F3F4 File Offset: 0x0003D5F4
    public static List<StatDef> GetAllWeaponStatDefs()
    {
      List<StatCategoryDef> lequip = ThingTool.GetAllStatCategoriesOnEquip();
      return (from td in DefDatabase<StatDef>.AllDefs
              where !lequip.Contains(td.category) || td.category == StatCategoryDefOf.Weapon
              orderby td.label
              select td).ToList<StatDef>();
    }

    // Token: 0x0600060B RID: 1547 RVA: 0x0003F458 File Offset: 0x0003D658
    public static float GetEquipStatValue(this ThingDef t, StatDef stat)
    {
      bool flag = t != null && t.equippedStatOffsets != null;
      if (flag)
      {
        foreach (StatModifier statModifier in t.equippedStatOffsets)
        {
          bool flag2 = statModifier.stat == stat;
          if (flag2)
          {
            return statModifier.value;
          }
        }
      }
      return 0f;
    }

    // Token: 0x0600060C RID: 1548 RVA: 0x0003F4E0 File Offset: 0x0003D6E0
    public static float GetStatValue(this ThingDef t, StatDef stat)
    {
      bool flag = t != null && t.statBases != null;
      if (flag)
      {
        foreach (StatModifier statModifier in t.statBases)
        {
          bool flag2 = statModifier.stat == stat;
          if (flag2)
          {
            return statModifier.value;
          }
        }
      }
      return 0f;
    }

    // Token: 0x0600060D RID: 1549 RVA: 0x0003F568 File Offset: 0x0003D768
    public static bool HasEquipStat(this ThingDef t, StatDef stat)
    {
      bool flag = t != null && t.equippedStatOffsets != null;
      if (flag)
      {
        foreach (StatModifier statModifier in t.equippedStatOffsets)
        {
          bool flag2 = statModifier.stat == stat;
          if (flag2)
          {
            return true;
          }
        }
      }
      return false;
    }

    // Token: 0x0600060E RID: 1550 RVA: 0x0003F5E8 File Offset: 0x0003D7E8
    public static bool HasStat(this ThingDef t, StatDef stat)
    {
      bool flag = t != null && t.statBases != null;
      if (flag)
      {
        foreach (StatModifier statModifier in t.statBases)
        {
          bool flag2 = statModifier.stat == stat;
          if (flag2)
          {
            return true;
          }
        }
      }
      return false;
    }

    // Token: 0x0600060F RID: 1551 RVA: 0x0003F668 File Offset: 0x0003D868
    public static void PasteEquipStats(this ThingDef t, List<StatModifier> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.equippedStatOffsets == null;
        if (flag2)
        {
          t.equippedStatOffsets = new List<StatModifier>();
        }
        foreach (StatModifier statModifier in l)
        {
          t.AddEquipStat(statModifier.stat, statModifier.value);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000610 RID: 1552 RVA: 0x0003F6F4 File Offset: 0x0003D8F4
    public static void PasteStats(this ThingDef t, List<StatModifier> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.statBases == null;
        if (flag2)
        {
          t.statBases = new List<StatModifier>();
        }
        foreach (StatModifier statModifier in l)
        {
          t.AddStat(statModifier.stat, statModifier.value);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000611 RID: 1553 RVA: 0x0003F780 File Offset: 0x0003D980
    public static void RemoveEquipStat(this ThingDef t, StatDef stat)
    {
      bool flag = t != null && t.equippedStatOffsets != null;
      if (flag)
      {
        foreach (StatModifier statModifier in t.equippedStatOffsets)
        {
          bool flag2 = statModifier.stat == stat;
          if (flag2)
          {
            t.equippedStatOffsets.Remove(statModifier);
            break;
          }
        }
        bool flag3 = t.equippedStatOffsets.Count == 0;
        if (flag3)
        {
          t.equippedStatOffsets = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000612 RID: 1554 RVA: 0x0003F828 File Offset: 0x0003DA28
    public static void RemoveStat(this ThingDef t, StatDef stat)
    {
      bool flag = t != null && t.statBases != null;
      if (flag)
      {
        foreach (StatModifier statModifier in t.statBases)
        {
          bool flag2 = statModifier.stat == stat;
          if (flag2)
          {
            t.statBases.Remove(statModifier);
            break;
          }
        }
        bool flag3 = t.statBases.Count == 0;
        if (flag3)
        {
          t.statBases = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000613 RID: 1555 RVA: 0x0003F8D0 File Offset: 0x0003DAD0
    public static void UpdateEquipStat(this ThingDef t, StatDef stat, float value)
    {
      bool flag = t != null && t.equippedStatOffsets != null;
      if (flag)
      {
        bool flag2 = t.HasEquipStat(stat);
        if (flag2)
        {
          foreach (StatModifier statModifier in t.equippedStatOffsets)
          {
            bool flag3 = statModifier.stat == stat;
            if (flag3)
            {
              statModifier.value = value;
              break;
            }
          }
        }
        else
        {
          t.AddEquipStat(stat, value);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000614 RID: 1556 RVA: 0x0003F970 File Offset: 0x0003DB70
    public static void UpdateStat(this ThingDef t, StatDef stat, float value)
    {
      bool flag = t != null && t.statBases != null;
      if (flag)
      {
        bool flag2 = t.HasStat(stat);
        if (flag2)
        {
          foreach (StatModifier statModifier in t.statBases)
          {
            bool flag3 = statModifier.stat == stat;
            if (flag3)
            {
              statModifier.value = value;
              break;
            }
          }
        }
        else
        {
          t.AddStat(stat, value);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000615 RID: 1557 RVA: 0x0003FA10 File Offset: 0x0003DC10
    public static ThingDef RandomAllowedStuff(this ThingDef t)
    {
      return (t != null && t.MadeFromStuff && t.stuffCategories != null) ? GenStuff.AllowedStuffsFor(t, TechLevel.Undefined).ToList<ThingDef>().RandomElement<ThingDef>() : null;
    }

    // Token: 0x06000616 RID: 1558 RVA: 0x0003FA39 File Offset: 0x0003DC39
    public static ThingDef ThisOrDefaultStuff(this ThingDef t, ThingDef stuff)
    {
      return (t != null && t.MadeFromStuff && stuff == null) ? GenStuff.DefaultStuffFor(t) : stuff;
    }

    // Token: 0x06000617 RID: 1559 RVA: 0x0003FA52 File Offset: 0x0003DC52
    public static ThingStyleDef RandomAllowedStyle(this ThingDef thingDef)
    {
      return ThingTool.ListOfThingStyleDefs(thingDef, null, true).RandomElement<ThingStyleDef>();
    }

    // Token: 0x06000618 RID: 1560 RVA: 0x0003FA61 File Offset: 0x0003DC61
    public static string GetStuffDefName(this Thing t)
    {
      return (t.Stuff != null) ? t.Stuff.defName : "";
    }

    // Token: 0x06000619 RID: 1561 RVA: 0x0003FA80 File Offset: 0x0003DC80
    public static void AddStuff(this ThingDef t, StuffCategoryDef stuffcat)
    {
      bool flag = t != null && stuffcat != null;
      if (flag)
      {
        bool flag2 = t.stuffCategories == null;
        if (flag2)
        {
          t.stuffCategories = new List<StuffCategoryDef>();
        }
        bool flag3 = !t.HasStuff(stuffcat);
        if (flag3)
        {
          t.stuffCategories.Add(stuffcat);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x0600061A RID: 1562 RVA: 0x0003FAD8 File Offset: 0x0003DCD8
    public static List<StuffCategoryDef> GetAllStuffCategories()
    {
      return (from td in DefDatabase<StuffCategoryDef>.AllDefs
              orderby td.label
              select td).ToList<StuffCategoryDef>();
    }

    // Token: 0x0600061B RID: 1563 RVA: 0x0003FB18 File Offset: 0x0003DD18
    public static bool HasStuff(this ThingDef t, StuffCategoryDef stuffcat)
    {
      bool flag = t != null && t.stuffCategories != null;
      return flag && t.stuffCategories.Contains(stuffcat);
    }

    // Token: 0x0600061C RID: 1564 RVA: 0x0003FB50 File Offset: 0x0003DD50
    public static void PasteStuff(this ThingDef t, List<StuffCategoryDef> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.stuffCategories == null;
        if (flag2)
        {
          t.stuffCategories = new List<StuffCategoryDef>();
        }
        foreach (StuffCategoryDef stuffcat in l)
        {
          t.AddStuff(stuffcat);
        }
      }
    }

    // Token: 0x0600061D RID: 1565 RVA: 0x0003FBCC File Offset: 0x0003DDCC
    public static void RemoveStuff(this ThingDef t, StuffCategoryDef stuffCat)
    {
      bool flag = t != null && t.stuffCategories != null;
      if (flag)
      {
        foreach (StuffCategoryDef stuffCategoryDef in t.stuffCategories)
        {
          bool flag2 = stuffCategoryDef == stuffCat;
          if (flag2)
          {
            t.stuffCategories.Remove(stuffCategoryDef);
            break;
          }
        }
        bool flag3 = t.stuffCategories.Count == 0;
        if (flag3)
        {
          t.stuffCategories = null;
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x0600061E RID: 1566 RVA: 0x0003FC6C File Offset: 0x0003DE6C
    public static ThingDef GetStuff(this ThingDef thingDef, ref List<ThingDef> lOfStuff, ref int stuffIndex, bool random = false)
    {
      lOfStuff = new List<ThingDef>();
      bool flag = thingDef == null || !thingDef.MadeFromStuff;
      ThingDef result;
      if (flag)
      {
        result = null;
      }
      else
      {
        lOfStuff = GenStuff.AllowedStuffsFor(thingDef, TechLevel.Undefined).ToList<ThingDef>();
        if (random)
        {
          stuffIndex = CEditor.zufallswert.Next(lOfStuff.Count);
        }
        bool flag2 = !lOfStuff.NullOrEmpty<ThingDef>();
        if (flag2)
        {
          bool flag3 = lOfStuff.Count > stuffIndex;
          if (flag3)
          {
            result = lOfStuff.At(stuffIndex);
          }
          else
          {
            stuffIndex = 0;
            result = lOfStuff.At(stuffIndex);
          }
        }
        else
        {
          result = null;
        }
      }
      return result;
    }

    // Token: 0x0600061F RID: 1567 RVA: 0x0003FD00 File Offset: 0x0003DF00
    public static void UpdateCostStuffCount(this ThingDef product)
    {
      bool flag = product == null;
      if (!flag)
      {
        foreach (RecipeDef recipeDef in DefDatabase<RecipeDef>.AllDefs)
        {
          bool flag2 = recipeDef.ProducedThingDef == product;
          if (flag2)
          {
            bool flag3 = !recipeDef.ingredients.NullOrEmpty<IngredientCount>();
            if (flag3)
            {
              recipeDef.ingredients[0].SetBaseCount((float)product.costStuffCount);
            }
          }
        }
      }
    }

    // Token: 0x06000620 RID: 1568 RVA: 0x0003FD90 File Offset: 0x0003DF90
    public static void UpdateRecipes(this ThingDef product)
    {
      bool flag = product == null;
      if (!flag)
      {
        foreach (RecipeDef recipeDef in DefDatabase<RecipeDef>.AllDefs)
        {
          bool flag2 = recipeDef.ProducedThingDef == product;
          if (flag2)
          {
            recipeDef.ingredients.Clear();
            recipeDef.ClearCachedData();
            recipeDef.fixedIngredientFilter.SetDisallowAll(null, null);
            bool flag3 = !product.stuffCategories.NullOrEmpty<StuffCategoryDef>();
            if (flag3)
            {
              IngredientCount ingredientCount = new IngredientCount();
              ingredientCount.filter.allowedHitPointsConfigurable = false;
              ingredientCount.filter.allowedQualitiesConfigurable = false;
              List<ThingDef> list = GenStuff.AllowedStuffsFor(product, TechLevel.Undefined).ToList<ThingDef>();
              foreach (ThingDef thingDef in list)
              {
                CollectionExtensions.AddItem<ThingDef>(ingredientCount.filter.AllowedThingDefs, thingDef);
                ingredientCount.filter.SetAllow(thingDef, true);
                ingredientCount.filter.SetAllowAllWhoCanMake(thingDef);
                ingredientCount.filter.Allows(thingDef);
                CollectionExtensions.AddItem<ThingDef>(recipeDef.fixedIngredientFilter.AllowedThingDefs, thingDef);
                recipeDef.fixedIngredientFilter.SetAllow(thingDef, true);
                recipeDef.fixedIngredientFilter.SetAllowAllWhoCanMake(thingDef);
                recipeDef.fixedIngredientFilter.Allows(thingDef);
              }
              ingredientCount.SetBaseCount((float)product.costStuffCount);
              recipeDef.ingredients.Add(ingredientCount);
              ingredientCount.filter.ResolveReferences();
              ingredientCount.ResolveReferences();
            }
            else
            {
              foreach (ThingDefCountClass thingDefCountClass in product.costList)
              {
                IngredientCount ingredientCount2 = new IngredientCount();
                CollectionExtensions.AddItem<ThingDef>(ingredientCount2.filter.AllowedThingDefs, thingDefCountClass.thingDef);
                ingredientCount2.filter.SetAllow(thingDefCountClass.thingDef, true);
                ingredientCount2.filter.SetAllowAllWhoCanMake(thingDefCountClass.thingDef);
                ingredientCount2.filter.Allows(thingDefCountClass.thingDef);
                ingredientCount2.SetBaseCount((float)thingDefCountClass.count);
                recipeDef.ingredients.Add(ingredientCount2);
                ingredientCount2.filter.ResolveReferences();
                ingredientCount2.ResolveReferences();
              }
            }
            recipeDef.fixedIngredientFilter.allowedHitPointsConfigurable = false;
            recipeDef.fixedIngredientFilter.allowedQualitiesConfigurable = false;
            recipeDef.fixedIngredientFilter.ResolveReferences();
            recipeDef.ResolveReferences();
          }
        }
        bool flag4 = product.recipeMaker != null;
        if (flag4)
        {
          foreach (ThingDef thingDef2 in product.recipeMaker.recipeUsers)
          {
            thingDef2.ResolveReferences();
          }
        }
      }
    }

    // Token: 0x06000621 RID: 1569 RVA: 0x000400F8 File Offset: 0x0003E2F8
    public static void AddCost(this ThingDef t, ThingDef tcost, int value)
    {
      bool flag = t != null;
      if (flag)
      {
        bool flag2 = t.HasCost(tcost);
        if (flag2)
        {
          t.UpdateCost(tcost, value);
        }
        else
        {
          ThingDefCountClass item = new ThingDefCountClass(tcost, value);
          bool flag3 = t.costList == null;
          if (flag3)
          {
            t.costList = new List<ThingDefCountClass>();
          }
          t.costList.Add(item);
        }
        t.UpdateRecipes();
        t.ResolveReferences();
      }
    }

    // Token: 0x06000622 RID: 1570 RVA: 0x00040168 File Offset: 0x0003E368
    public static List<ThingDef> GetAllCostThingDefs()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where !td.label.NullOrEmpty() && !td.IsWeapon && !td.IsApparel && td.CountAsResource && !td.mineable && td.plant == null && !td.IsSmoothed && !td.IsDrug && !td.IsIngestible
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000623 RID: 1571 RVA: 0x000401CC File Offset: 0x0003E3CC
    public static bool HasCost(this ThingDef t, ThingDef tcost)
    {
      bool flag = t != null && t.costList != null;
      if (flag)
      {
        foreach (ThingDefCountClass thingDefCountClass in t.costList)
        {
          bool flag2 = thingDefCountClass.thingDef == tcost;
          if (flag2)
          {
            return true;
          }
        }
      }
      return false;
    }

    // Token: 0x06000624 RID: 1572 RVA: 0x0004024C File Offset: 0x0003E44C
    public static void PasteCost(this ThingDef t, List<ThingDefCountClass> l)
    {
      bool flag = t != null && l != null;
      if (flag)
      {
        bool flag2 = t.costList == null;
        if (flag2)
        {
          t.costList = new List<ThingDefCountClass>();
        }
        foreach (ThingDefCountClass thingDefCountClass in l)
        {
          t.costList.Add(new ThingDefCountClass(thingDefCountClass.thingDef, thingDefCountClass.count));
        }
      }
    }

    // Token: 0x06000625 RID: 1573 RVA: 0x000402DC File Offset: 0x0003E4DC
    public static void RemoveCost(this ThingDef t, ThingDef tcost)
    {
      bool flag = t != null && t.costList != null;
      if (flag)
      {
        foreach (ThingDefCountClass thingDefCountClass in t.costList)
        {
          bool flag2 = thingDefCountClass.thingDef == tcost;
          if (flag2)
          {
            t.costList.Remove(thingDefCountClass);
            break;
          }
        }
        bool flag3 = t.costList.Count == 0;
        if (flag3)
        {
          t.costList = null;
        }
        t.UpdateRecipes();
        t.ResolveReferences();
      }
    }

    // Token: 0x06000626 RID: 1574 RVA: 0x00040388 File Offset: 0x0003E588
    public static void UpdateCost(this ThingDef t, ThingDef tcost, int value)
    {
      bool flag = t != null && t.costList != null;
      if (flag)
      {
        bool flag2 = t.HasCost(tcost);
        if (flag2)
        {
          foreach (ThingDefCountClass thingDefCountClass in t.costList)
          {
            bool flag3 = thingDefCountClass.thingDef == tcost;
            if (flag3)
            {
              thingDefCountClass.count = value;
            }
          }
        }
        else
        {
          t.AddCost(tcost, value);
        }
        t.UpdateRecipes();
        t.ResolveReferences();
      }
    }

    // Token: 0x06000627 RID: 1575 RVA: 0x00040430 File Offset: 0x0003E630
    public static void AddCompColorable(this ThingDef t)
    {
      bool flag = t == null || t.HasComp(typeof(CompColorable));
      if (!flag)
      {
        CompProperties compProperties = new CompProperties();
        compProperties.compClass = typeof(CompColorable);
        bool flag2 = t.comps == null;
        if (flag2)
        {
          t.comps = new List<CompProperties>();
        }
        t.comps.Add(compProperties);
        compProperties.ResolveReferences(t);
        t.ResolveReferences();
        t.PostLoad();
      }
    }

    // Token: 0x06000628 RID: 1576 RVA: 0x000404AC File Offset: 0x0003E6AC
    public static Color GetColor(this ThingDef t, ThingDef stuff)
    {
      bool flag = t == null;
      Color result;
      if (flag)
      {
        result = Color.white;
      }
      else
      {
        bool flag2 = stuff != null && t.MadeFromStuff;
        if (flag2)
        {
          result = stuff.stuffProps.color;
        }
        else
        {
          bool flag3 = t.mineable && t.graphicData != null;
          if (flag3)
          {
            result = t.graphicData.colorTwo;
          }
          else
          {
            bool flag4 = t.colorGenerator != null;
            if (flag4)
            {
              result = t.colorGenerator.ExemplaryColor;
            }
            else
            {
              bool flag5 = t.graphicData != null;
              if (flag5)
              {
                result = t.graphicData.color;
              }
              else
              {
                result = Color.white;
              }
            }
          }
        }
      }
      return result;
    }

    // Token: 0x06000629 RID: 1577 RVA: 0x00040554 File Offset: 0x0003E754
    public static void SetDrawColor(this Thing t, Color col)
    {
      bool flag = t == null;
      if (!flag)
      {
        CompColorable compColorable = t.TryGetComp<CompColorable>();
        bool flag2 = compColorable != null;
        if (flag2)
        {
          compColorable.SetColor(col);
        }
      }
    }

    // Token: 0x0600062A RID: 1578 RVA: 0x00040584 File Offset: 0x0003E784
    public static void AddCompQuality(this ThingDef t)
    {
      bool flag = t == null || t.HasComp(typeof(CompQuality));
      if (!flag)
      {
        CompProperties compProperties = new CompProperties();
        compProperties.compClass = typeof(CompQuality);
        bool flag2 = t.comps == null;
        if (flag2)
        {
          t.comps = new List<CompProperties>();
        }
        t.comps.Add(compProperties);
        compProperties.ResolveReferences(t);
        t.ResolveReferences();
        t.PostLoad();
      }
    }

    // Token: 0x0600062B RID: 1579 RVA: 0x00040600 File Offset: 0x0003E800
    public static void CopyQuality(this Thing t1, Thing fromT2)
    {
      bool flag = t1 == null || fromT2 == null;
      if (!flag)
      {
        QualityCategory quali;
        bool flag2 = fromT2.TryGetQuality(out quali);
        if (flag2)
        {
          t1.SetQuality((int)quali);
        }
      }
    }

    // Token: 0x0600062C RID: 1580 RVA: 0x00040634 File Offset: 0x0003E834
    public static void SetQuality(this Thing t, int quali)
    {
      bool flag = t == null;
      if (!flag)
      {
        CompQuality compQuality = t.TryGetComp<CompQuality>();
        bool flag2 = compQuality != null;
        if (flag2)
        {
          compQuality.SetQuality((QualityCategory)quali, ArtGenerationContext.Colony);
        }
      }
    }

    // Token: 0x0600062D RID: 1581 RVA: 0x00040668 File Offset: 0x0003E868
    public static int GetQuality(this Thing t)
    {
      QualityCategory qualityCategory;
      bool flag = t.TryGetQuality(out qualityCategory);
      int result;
      if (flag)
      {
        result = (int)qualityCategory;
      }
      else
      {
        result = 0;
      }
      return result;
    }

    // Token: 0x0600062E RID: 1582 RVA: 0x0004068C File Offset: 0x0003E88C
    public static CompProperties GetCompByType(this ThingDef thingDef, string typeString)
    {
      bool flag = thingDef != null && thingDef.comps != null;
      if (flag)
      {
        foreach (CompProperties compProperties in thingDef.comps)
        {
          bool flag2 = compProperties.GetType().ToString() == typeString;
          if (flag2)
          {
            return compProperties;
          }
        }
      }
      return null;
    }

    // Token: 0x0600062F RID: 1583 RVA: 0x00040714 File Offset: 0x0003E914
    public static CompProperties GetCompByType(this ThingDef thingDef, Type type)
    {
      bool flag = thingDef != null && thingDef.comps != null;
      if (flag)
      {
        foreach (CompProperties compProperties in thingDef.comps)
        {
          bool flag2 = compProperties.GetType() == type;
          if (flag2)
          {
            return compProperties;
          }
        }
      }
      return null;
    }

    // Token: 0x06000630 RID: 1584 RVA: 0x00040798 File Offset: 0x0003E998
    public static void Reinvent(this Pawn pawn, Selected selected, int createRandomAmount = 1)
    {
      bool flag = !pawn.HasInventoryTracker();
      if (!flag)
      {
        bool flag2 = selected == null;
        if (flag2)
        {
          try
          {
            pawn.inventory.DestroyAll(DestroyMode.Vanish);
          }
          catch
          {
          }
          for (int i = 0; i < createRandomAmount; i++)
          {
            Thing t = ThingTool.GenerateRandomItem(null, true, true);
            pawn.AddItemToInventory(t);
          }
        }
        else
        {
          Thing t2 = ThingTool.GenerateItem(selected);
          pawn.AddItemToInventory(t2);
        }
      }
    }

    // Token: 0x06000631 RID: 1585 RVA: 0x00040820 File Offset: 0x0003EA20
    public static void ReplaceItem(this Pawn pawn, Thing t)
    {
      bool flag = !pawn.HasInventoryTracker();
      if (!flag)
      {
        Thing t2 = ThingTool.GenerateRandomItem(t.def.thingCategories.FirstOrDefault<ThingCategoryDef>(), true, true);
        pawn.inventory.innerContainer.Remove(t);
        pawn.AddItemToInventory(t2);
      }
    }

    // Token: 0x06000632 RID: 1586 RVA: 0x00040870 File Offset: 0x0003EA70
    public static int GetSilverAmountNear(IntVec3 pos, Map map)
    {
      int num = 0;
      bool flag = map == null;
      int result;
      if (flag)
      {
        result = num;
      }
      else
      {
        int x = map.Size.x;
        int z = map.Size.z;
        for (int i = 0; i < x; i++)
        {
          for (int j = 0; j < z; j++)
          {
            IntVec3 c = new IntVec3(i, pos.y, j);
            List<Thing> thingList = c.GetThingList(map);
            foreach (Thing thing in thingList)
            {
              bool flag2 = ((thing != null) ? thing.def : null) == ThingDefOf.Silver;
              if (flag2)
              {
                num += thing.stackCount;
                break;
              }
            }
          }
        }
        result = num;
      }
      return result;
    }

    // Token: 0x06000633 RID: 1587 RVA: 0x00040970 File Offset: 0x0003EB70
    public static void ReduceSilverAmount(IntVec3 pos, Map map, int buyPrice)
    {
      bool flag = map == null;
      if (!flag)
      {
        int x = map.Size.x;
        int z = map.Size.z;
        for (int i = 0; i < x; i++)
        {
          for (int j = 0; j < z; j++)
          {
            IntVec3 c = new IntVec3(i, pos.y, j);
            List<Thing> thingList = c.GetThingList(map);
            foreach (Thing thing in thingList)
            {
              bool flag2 = ((thing != null) ? thing.def : null) == ThingDefOf.Silver;
              if (flag2)
              {
                bool flag3 = thing.stackCount > buyPrice;
                if (flag3)
                {
                  thing.stackCount -= buyPrice;
                  buyPrice = 0;
                  break;
                }
                buyPrice -= thing.stackCount;
                thing.Destroy(DestroyMode.Vanish);
                break;
              }
            }
            bool flag4 = buyPrice <= 0;
            if (flag4)
            {
              break;
            }
          }
          bool flag5 = buyPrice <= 0;
          if (flag5)
          {
            break;
          }
        }
      }
    }

    // Token: 0x06000634 RID: 1588 RVA: 0x00040AB4 File Offset: 0x0003ECB4
    public static void BeginBuyItem(Selected selected)
    {
      bool flag = selected != null && selected.thingDef != null;
      if (flag)
      {
        IntVec3 pos = UI.MouseCell();
        int silverAmountNear = ThingTool.GetSilverAmountNear(pos, Find.CurrentMap);
        selected.UpdateBuyPrice();
        MessageTool.Show(Label.SILVER_NEAR + silverAmountNear.ToString() + Label.SILVER_NEEDED + selected.buyPrice.ToString(), (silverAmountNear < selected.buyPrice) ? MessageTypeDefOf.RejectInput : MessageTypeDefOf.PositiveEvent);
        bool flag2 = silverAmountNear >= selected.buyPrice;
        if (flag2)
        {
          string buy_ = Label.BUY_1;
          object[] args = new string[]
          {
            selected.thingDef.label,
            selected.buyPrice.ToString()
          };
          WindowTool.Open(Dialog_MessageBox.CreateConfirmation(string.Format(buy_, args), delegate
          {
            selected.thingDef.DmoBuyAndPlaceItem(selected.Stuff, selected.quality, selected.stackVal, selected.buyPrice);
          }, false, null, WindowLayer.Dialog));
        }
      }
    }

    // Token: 0x06000635 RID: 1589 RVA: 0x00040BC8 File Offset: 0x0003EDC8
    public static void DmoBuyAndPlaceItem(this ThingDef thingDef, ThingDef stuff, int quality, int stackCount, int buyPrice)
    {
      bool flag = thingDef == null;
      if (!flag)
      {
        DebugMenuOption debugMenuOption = new DebugMenuOption(Label.PLACE_ITEM, DebugMenuOptionMode.Tool, delegate ()
        {
          IntVec3 pos = UI.MouseCell();
          int silverAmountNear = ThingTool.GetSilverAmountNear(pos, Find.CurrentMap);
          bool flag2 = silverAmountNear >= buyPrice;
          if (flag2)
          {
            SoundDefOf.ExecuteTrade.PlayOneShotOnCamera(null);
            ThingTool.ReduceSilverAmount(pos, Find.CurrentMap, buyPrice);
            bool flag3 = !thingDef.MadeFromStuff && !thingDef.HasComp(typeof(CompQuality));
            if (flag3)
            {
              DebugThingPlaceHelper.DebugSpawn(thingDef, UI.MouseCell(), stackCount, false, null);
            }
            else
            {
              for (int i = 0; i < stackCount; i++)
              {
                Thing thing = ThingMaker.MakeThing(thingDef, stuff);
                thing.SetQuality(quality);
                bool flag4 = thing.CanStackWith(thing);
                if (flag4)
                {
                  GenPlace.TryPlaceThing(thing, UI.MouseCell(), Find.CurrentMap, ThingPlaceMode.Near, null, null, default(Rot4));
                }
                else
                {
                  GenPlace.TryPlaceThing(thing.MakeMinified(), UI.MouseCell(), Find.CurrentMap, ThingPlaceMode.Near, null, null, default(Rot4));
                }
              }
            }
            EditWindow_Log.wantsToOpen = false;
          }
          else
          {
            DebugTools.curTool = null;
          }
        });
        DebugTools.curTool = new DebugTool(debugMenuOption.label, debugMenuOption.method, null);
      }
    }

    // Token: 0x06000636 RID: 1590 RVA: 0x00040C40 File Offset: 0x0003EE40
    public static void CreateBuilding(string defName, string label, string dsc, Type thingClass, string textur)
    {
      //////Log.Message("CreateBuilding, start");
      ThingDef thingDef = DefTool.ThingDef(defName);
      bool flag = thingDef == null;
      if (flag)
      {
        try
        {
          //////Log.Message("CreateBuilding, step 1");
          thingDef = new ThingDef();
          thingDef.defName = defName;
          thingDef.label = label;
          thingDef.description = dsc;
          thingDef.designatorDropdown = DefTool.DesignatorDropdownGroupDef("Zombrella");
          thingDef.designationCategory = DefTool.DesignationCategoryDef("Misc");
          thingDef.thingClass = thingClass;
          thingDef.category = ThingCategory.Building;
          thingDef.altitudeLayer = AltitudeLayer.Building;
          thingDef.passability = Traversability.Impassable;
          thingDef.thingCategories = new List<ThingCategoryDef>();
          thingDef.thingCategories.Add(ThingCategoryDef.Named("BuildingsMisc"));
          //////Log.Message("CreateBuilding, step 2");
          ThingCategoryDef.Named("BuildingsMisc").childThingDefs.Add(thingDef);
          thingDef.placeWorkers = new List<Type>();
          thingDef.placeWorkers.Add(typeof(PlaceWorker_Heater));
          thingDef.drawPlaceWorkersWhileSelected = true;
          ThingDef thingDef2 = DefTool.ThingDef(textur, ThingDefOf.Grave);
          thingDef.graphicData = new GraphicData();
          thingDef.graphicData.texPath = thingDef2.graphicData.texPath;
          thingDef.graphicData.graphicClass = typeof(Graphic_Multi);
          bool flag2 = thingClass == typeof(CharacterEditorGrave);
          //////Log.Message("CreateBuilding, step 3");
          if (flag2)
          {
            thingDef.graphicData.drawSize = new Vector2(3f, 4f);
            thingDef.graphicData.color = new Color(0.4f, 0.4f, 0.5f);
          }
          else
          {
            thingDef.graphicData.drawSize = new Vector2(1f, 2f);
            thingDef.graphicData.color = new Color(0.3f, 0.3f, 0.4f);
          }
          thingDef.graphicData.shadowData = new ShadowData();
          thingDef.graphicData.shadowData.volume = new Vector3(0.83f, 0.3f, 1.7f);
          thingDef.size = new IntVec2(1, 2);
          thingDef.placingDraggableDimensions = 1;
          thingDef.rotatable = true;
          thingDef.selectable = true;
          thingDef.useHitPoints = true;
          thingDef.leaveResourcesWhenKilled = false;
          thingDef.destroyable = true;
          thingDef.blockWind = true;
          thingDef.blockLight = true;
          thingDef.building = new BuildingProperties();
          thingDef.building.claimable = true;
          thingDef.building.alwaysDeconstructible = false;
          //////Log.Message("CreateBuilding, step 4");
          bool flag3 = thingClass == typeof(CharacterEditorGrave);
          if (flag3)
          {
            thingDef.building.fullGraveGraphicData = new GraphicData();
            thingDef.building.fullGraveGraphicData.texPath = thingDef2.building.fullGraveGraphicData.texPath;
            thingDef.building.fullGraveGraphicData.graphicClass = typeof(Graphic_Multi);
            thingDef.building.fullGraveGraphicData.drawSize = new Vector2(3f, 4f);
            thingDef.building.fullGraveGraphicData.color = new Color(0.4f, 0.4f, 0.5f);
          }
          thingDef.defaultPlacingRot = Rot4.South;
          thingDef.terrainAffordanceNeeded = TerrainAffordanceDefOf.Light;
          bool flag4 = thingClass != typeof(CharacterEditorGrave);
          if (flag4)
          {
            thingDef.constructionSkillPrerequisite = 5;
          }
          bool flag5 = thingClass == typeof(CharacterEditorGrave);
          if (flag5)
          {
            thingDef.constructEffect = EffecterDefOf.ConstructDirt;
          }
          else
          {
            thingDef.constructEffect = EffecterDefOf.ConstructMetal;
          }
          //////Log.Message("CreateBuilding, step 5");
          thingDef.soundImpactDefault = DefTool.SoundDef("BulletImpact_Metal");
          thingDef.repairEffect = EffecterDefOf.ConstructMetal;
          thingDef.hasInteractionCell = true;
          thingDef.interactionCellOffset = new IntVec3(1, 0, 0);
          thingDef.tickerType = TickerType.Normal;
          thingDef.AddStat(StatDefOf.MaxHitPoints, 500f);
          bool flag6 = thingClass != typeof(CharacterEditorGrave);
          if (flag6)
          {
            thingDef.AddStat(StatDefOf.MarketValue, 2000f);
          }
          thingDef.AddStat(StatDefOf.WorkToBuild, 5000f);
          thingDef.AddStat(StatDefOf.Flammability, 0.2f);
          thingDef.costList = new List<ThingDefCountClass>();
          thingDef.costList.Add(new ThingDefCountClass(ThingDefOf.ComponentIndustrial, 4));
          bool flag7 = thingClass == typeof(CharacterEditorGrave);
          if (flag7)
          {
            thingDef.costList.Add(new ThingDefCountClass(ThingDefOf.WoodLog, 200));
          }
          else
          {
            thingDef.costList.Add(new ThingDefCountClass(ThingDefOf.Steel, 350));
            thingDef.costList.Add(new ThingDefCountClass(ThingDefOf.Plasteel, 50));
          }
          thingDef.modContentPack = thingDef.designatorDropdown.modContentPack;
          bool flag8 = thingClass == typeof(CharacterEditorGrave);
          if (flag8)
          {
            thingDef.minifiedDef = null;
          }
          else
          {
            thingDef.minifiedDef = ThingDefOf.MinifiedThing;
          }
          thingDef.RegisterBuildingDef();
          thingDef.designationCategory.ResolveReferences();
          thingDef.designationCategory.PostLoad();
        }
        catch (Exception ex)
        {
          bool devMode = Prefs.DevMode;
          if (devMode)
          {
            Log.Error(ex.Message + "\n" + ex.StackTrace);
          }
        }
      }
    }

    // Token: 0x06000637 RID: 1591 RVA: 0x00041154 File Offset: 0x0003F354
    public static void RegisterBuildingDef(this ThingDef td)
    {
      //////Log.Message("RegisterBuildingDef, step 0");
      bool flag = td == null || DefTool.ThingDef(td.defName) != null;
      if (!flag)
      {
        //////Log.Message("RegisterBuildingDef, step 1");
        Type typeFromHandle = typeof(ThingDefGenerator_Buildings);
        object[] param = new object[] { td };
        td.frameDef = (ThingDef)typeFromHandle.CallMethod("NewFrameDef_Thing", param);
        //td.frameDef = (ThingDef)AccessTools.Method(typeFromHandle, "NewFrameDef_Thing").CallMethod(param);
        td.frameDef.ResolveReferences();
        td.frameDef.PostLoad();
        //object[] array = new object[3];
        //array[0] = td;
        //array[1] = false;
        //////Log.Message("RegisterBuildingDef, step 2");
        object[] param2 = new object[] { td, false, null };
        //Log.Message("RegisterBuildingDef, (ThingDef)typeFromHandle.CallMethod(\"NewBlueprintDef_Thing\", param2)");
        td.blueprintDef = (ThingDef)typeFromHandle.CallMethod("NewBlueprintDef_Thing", param2);
        td.blueprintDef.entityDefToBuild = td;
        td.blueprintDef.ResolveReferences();
        td.blueprintDef.PostLoad();
        ThingDef thingDef = null;
        bool flag2 = td.minifiedDef != null;
        //////Log.Message("RegisterBuildingDef, step 3");
        if (flag2)
        {
          object[] param3 = new object[] { td, true, td.blueprintDef };
          //Log.Message("RegisterBuildingDef, (ThingDef)typeFromHandle.CallMethod(\"NewBlueprintDef_Thing\", param3)");
          thingDef = (ThingDef)typeFromHandle.CallMethod("NewBlueprintDef_Thing", param3, new Type[] { typeof(ThingDef), typeof(bool), typeof(ThingDef) });
          thingDef.ResolveReferences();
          thingDef.PostLoad();
        }
        td.ResolveReferences();
        td.PostLoad();
        //////Log.Message("RegisterBuildingDef, step 4");
        MethodInfo methodInfo = typeof(ShortHashGiver).GetMethodInfo("GiveShortHash");
        FieldInfo takenHashesFieldInfo = AccessTools.Field(typeof(ShortHashGiver), "takenHashesPerDeftype");
        Dictionary<Type, HashSet<ushort>> takenHashesDict = (Dictionary<Type, HashSet<ushort>>)takenHashesFieldInfo.GetValue(null);
        Type typeFromHandle2 = typeof(ThingDef);
        methodInfo.CallMethod(new object[] { td, typeFromHandle2, takenHashesDict[typeFromHandle2] });
        bool flag3 = td.minifiedDef != null;
        if (flag3)
        {
          methodInfo.CallMethod(new object[] { thingDef, typeFromHandle2, takenHashesDict[typeFromHandle2] });
        }
        methodInfo.CallMethod(new object[] { td.blueprintDef, typeFromHandle2, takenHashesDict[typeFromHandle2] });
        methodInfo.CallMethod(new object[] { td.frameDef, typeFromHandle2, takenHashesDict[typeFromHandle2] });
        //////Log.Message("RegisterBuildingDef, step 5");
        DefDatabase<ThingDef>.Add(td);
        bool flag4 = td.minifiedDef != null;
        if (flag4)
        {
          DefDatabase<ThingDef>.Add(thingDef);
        }
        DefDatabase<ThingDef>.Add(td.blueprintDef);
        DefDatabase<ThingDef>.Add(td.frameDef);
      }
      //////Log.Message("RegisterBuildingDef, end");
    }

    // Token: 0x06000638 RID: 1592 RVA: 0x00041348 File Offset: 0x0003F548
    public static List<string> GetAllApparelTags()
    {
      SortedDictionary<string, int> sortedDictionary = new SortedDictionary<string, int>();
      foreach (ThingDef thingDef in DefDatabase<ThingDef>.AllDefs)
      {
        bool isApparel = thingDef.IsApparel;
        if (isApparel)
        {
          sortedDictionary.AddFromList(thingDef.apparel.tags, 0);
        }
      }
      foreach (PawnKindDef pawnKindDef in DefDatabase<PawnKindDef>.AllDefs)
      {
        sortedDictionary.AddFromList(pawnKindDef.apparelTags, 0);
      }
      return sortedDictionary.Keys.ToList<string>();
    }

    // Token: 0x06000639 RID: 1593 RVA: 0x00041414 File Offset: 0x0003F614
    public static List<string> GetAllOutfitTags()
    {
      SortedDictionary<string, int> sortedDictionary = new SortedDictionary<string, int>();
      foreach (ThingDef thingDef in DefDatabase<ThingDef>.AllDefs)
      {
        bool isApparel = thingDef.IsApparel;
        if (isApparel)
        {
          sortedDictionary.AddFromList(thingDef.apparel.defaultOutfitTags, 0);
        }
      }
      return sortedDictionary.Keys.ToList<string>();
    }

    // Token: 0x0600063A RID: 1594 RVA: 0x00041494 File Offset: 0x0003F694
    public static List<string> GetAllWeaponTags()
    {
      SortedDictionary<string, int> sortedDictionary = new SortedDictionary<string, int>();
      foreach (ThingDef thingDef in DefDatabase<ThingDef>.AllDefs)
      {
        bool isWeapon = thingDef.IsWeapon;
        if (isWeapon)
        {
          sortedDictionary.AddFromList(thingDef.weaponTags, 0);
        }
      }
      foreach (PawnKindDef pawnKindDef in DefDatabase<PawnKindDef>.AllDefs)
      {
        sortedDictionary.AddFromList(pawnKindDef.weaponTags, 0);
      }
      return sortedDictionary.Keys.ToList<string>();
    }

    // Token: 0x0600063B RID: 1595 RVA: 0x00041558 File Offset: 0x0003F758
    public static List<ThingDef> ListOfItems(ThingCategoryDef tc, string modname)
    {
      return DefTool.ListBy<ThingDef>(DefTool.CONDITION_IS_ITEM(tc, modname));
    }

    // Token: 0x0600063C RID: 1596 RVA: 0x00041568 File Offset: 0x0003F768
    public static ThingStyleDef GetStyle(this ThingDef thingDef, ref List<ThingStyleDef> l, ref int styleIndex, bool random)
    {
      l = new List<ThingStyleDef>();
      bool flag = thingDef == null || !thingDef.CanBeStyled();
      ThingStyleDef result;
      if (flag)
      {
        result = null;
      }
      else
      {
        l = ThingTool.ListOfThingStyleDefs(thingDef, null, true);
        if (random)
        {
          styleIndex = CEditor.zufallswert.Next(l.Count);
        }
        bool flag2 = !l.NullOrEmpty<ThingStyleDef>();
        if (flag2)
        {
          bool flag3 = l.Count <= styleIndex;
          if (flag3)
          {
            styleIndex = 0;
          }
          result = l.At(styleIndex);
        }
        else
        {
          result = null;
        }
      }
      return result;
    }

    // Token: 0x0600063D RID: 1597 RVA: 0x000415F0 File Offset: 0x0003F7F0
    public static List<ThingStyleDef> ListOfThingStyleDefs(ThingDef thingDef, string modname, bool withNull)
    {
      List<ThingStyleDef> hashSet = new List<ThingStyleDef>();
      if (withNull)
      {
        hashSet.Add(null);
      }
      bool flag = thingDef == null;
      List<ThingStyleDef> result;
      if (flag)
      {
        result = hashSet;
      }
      else
      {
        bool flag2 = modname.NullOrEmpty();
        List<StyleCategoryDef> hashSet2 = DefTool.ListByMod<StyleCategoryDef>(modname);
        foreach (StyleCategoryDef styleCategoryDef in hashSet2)
        {
          foreach (ThingDefStyle thingDefStyle in styleCategoryDef.thingDefStyles)
          {
            bool flag3 = thingDef == thingDefStyle.ThingDef;
            if (flag3)
            {
              hashSet.Add(thingDefStyle.StyleDef);
            }
          }
        }
        result = hashSet;
      }
      return result;
    }
  }
}
