﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000036 RID: 54
	public static class ThoughtTool
	{
		// Token: 0x06000286 RID: 646 RVA: 0x00012F58 File Offset: 0x00011158
		public static bool IsTrulySocial(this ThoughtDef t)
		{
			return t != null && (t.IsSocial || (t.IsTypeOf<Thought_Memory>() && t.HasLabelPlaceholder("{0}")));
		}

		// Token: 0x06000287 RID: 647 RVA: 0x00012F81 File Offset: 0x00011181
		public static bool IsClassOf<T>(this ThoughtDef t)
		{
			return t.thoughtClass == typeof(T);
		}

		// Token: 0x06000288 RID: 648 RVA: 0x00012F98 File Offset: 0x00011198
		public static bool IsTypeOf<T>(this ThoughtDef t)
		{
			return t != null && (t.thoughtClass == typeof(T) || t.ThoughtClass == typeof(T) || (t.thoughtClass != null && (t.thoughtClass.BaseType == typeof(T) || (t.thoughtClass.BaseType != null && (t.thoughtClass.BaseType.BaseType == typeof(T) || (t.thoughtClass.BaseType.BaseType != null && t.thoughtClass.BaseType.BaseType.BaseType == typeof(T)))))));
		}

		// Token: 0x06000289 RID: 649 RVA: 0x00013091 File Offset: 0x00011291
		public static bool IsForWeapon(this ThoughtDef t)
		{
			return t.HasLabelPlaceholder("{WEAPON}");
		}

		// Token: 0x0600028A RID: 650 RVA: 0x0001309E File Offset: 0x0001129E
		public static bool IsForTitle(this ThoughtDef t)
		{
			return t.IsTypeOf<Thought_MemoryRoyalTitle>();
		}

		// Token: 0x0600028B RID: 651 RVA: 0x000130A6 File Offset: 0x000112A6
		public static bool HasLabelPlaceholder(this ThoughtDef t, string ph = "{0}")
		{
			return t != null && !t.stages.NullOrEmpty<ThoughtStage>() && t.stages[0].SLabel().Contains(ph);
		}

		// Token: 0x0600028C RID: 652 RVA: 0x000130D8 File Offset: 0x000112D8
		public static bool HasMemories(this Pawn pawn)
		{
			return pawn != null && pawn.needs != null && pawn.needs.mood != null && pawn.needs.mood.thoughts != null && pawn.needs.mood.thoughts.memories != null && pawn.needs.mood.thoughts.memories.Memories != null;
		}

		// Token: 0x0600028D RID: 653 RVA: 0x00013146 File Offset: 0x00011346
		public static Gender GetOppositeGender(this Pawn p)
		{
			return (p != null) ? ((p.gender == Gender.Male) ? Gender.Female : Gender.Male) : Gender.Female;
		}

		// Token: 0x0600028E RID: 654 RVA: 0x0001315B File Offset: 0x0001135B
		public static List<Thought_Situational> GetAllThoughtSituationals(this Pawn p)
		{
			return (List<Thought_Situational>)p.needs.mood.thoughts.situational.GetMemberValue("cachedThoughts");
		}

		// Token: 0x0600028F RID: 655 RVA: 0x00013181 File Offset: 0x00011381
		public static int FixStageValue(ThoughtDef t, int stage)
		{
			return (stage < 0 || t.stages.NullOrEmpty<ThoughtStage>()) ? 0 : ((t.stages.CountAllowNull<ThoughtStage>() <= stage) ? t.stages.IndexOf(t.stages.Last<ThoughtStage>()) : stage);
		}

		// Token: 0x06000290 RID: 656 RVA: 0x000131C0 File Offset: 0x000113C0
		public static string GetAsSeparatedString(this Thought t)
		{
			bool flag = t == null || t.def.IsNullOrEmpty();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				text = text + t.def.defName + "|";
				string text2;
				Pawn otherPawn = t.GetOtherPawn(out text2);
				bool flag2 = otherPawn == null && !text2.NullOrEmpty();
				if (flag2)
				{
					text = text + text2 + "|";
				}
				else
				{
					text = text + otherPawn.GetPawnNameAsSeparatedString() + "|";
				}
				text = text + t.CurStageIndex.ToString() + "|";
				bool flag3 = t.def.IsTypeOf<Thought_MemoryRoyalTitle>();
				if (flag3)
				{
					text = text + ((Thought_MemoryRoyalTitle)t).titleDef.defName + "|";
				}
				else
				{
					text += "|";
				}
				bool flag4 = t.def.IsTypeOf<Thought_Memory>();
				if (flag4)
				{
					text = text + ((Thought_Memory)t).moodPowerFactor.ToString() + "|";
				}
				else
				{
					text += "1|";
				}
				float opinionOffset = t.GetOpinionOffset();
				text = text + ((opinionOffset == float.MinValue) ? 0f : opinionOffset).ToString() + "|";
				bool flag5 = t.def.IsTypeOf<Thought_Memory>();
				if (flag5)
				{
					text += ((Thought_Memory)t).age.ToString();
				}
				else
				{
					text += "0";
				}
				result = text;
			}
			return result;
		}

		// Token: 0x06000291 RID: 657 RVA: 0x00013348 File Offset: 0x00011548
		public static string GetAllMemoriesAsSeparatedString(this Pawn p)
		{
			bool flag = !p.HasMemoryTracker() || p.needs.mood.thoughts.memories.Memories.NullOrEmpty<Thought_Memory>();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				foreach (Thought_Memory t in p.needs.mood.thoughts.memories.Memories)
				{
					text += t.GetAsSeparatedString();
					text += ":";
				}
				text = text.SubstringRemoveLast();
				result = text;
			}
			return result;
		}

		// Token: 0x06000292 RID: 658 RVA: 0x00013410 File Offset: 0x00011610
		public static string GetAllSituationalMemoriesAsSeparatedString(this Pawn p)
		{
			bool flag = !p.HasSituationalTracker() || p.GetAllThoughtSituationals().NullOrEmpty<Thought_Situational>();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				foreach (Thought_Situational t in p.GetAllThoughtSituationals())
				{
					text += t.GetAsSeparatedString();
					text += ":";
				}
				text = text.SubstringRemoveLast();
				result = text;
			}
			return result;
		}

		// Token: 0x06000293 RID: 659 RVA: 0x000134B0 File Offset: 0x000116B0
		public static void SetMemories(this Pawn p, string s)
		{
			bool flag = !p.HasMemoryTracker();
			if (!flag)
			{
				p.needs.mood.thoughts.memories.Memories.Clear();
				bool flag2 = s.NullOrEmpty();
				if (!flag2)
				{
					string[] array = s.SplitNo(":");
					foreach (string s2 in array)
					{
						string[] array3 = s2.SplitNo("|");
						bool flag3 = array3.Length == 7;
						if (flag3)
						{
							ThoughtDef thoughtDef = DefTool.ThoughtDef(array3[0]);
							bool flag4 = thoughtDef != null;
							if (flag4)
							{
								Pawn otherPawnFromSeparatedString = PawnxTool.GetOtherPawnFromSeparatedString(array3[1]);
								p.AddThought(thoughtDef, otherPawnFromSeparatedString, array3[2].AsInt32(), array3[3], array3[4].AsFloat(), array3[5].AsFloat(), array3[6].AsInt32());
							}
						}
					}
				}
			}
		}

		// Token: 0x06000294 RID: 660 RVA: 0x000135A4 File Offset: 0x000117A4
		public static void SetSituationalMemories(this Pawn p, string s)
		{
			bool flag = !p.HasSituationalTracker();
			if (!flag)
			{
				p.needs.mood.thoughts.situational.Notify_SituationalThoughtsDirty();
				p.needs.mood.thoughts.situational.SetMemberValue("lastMoodThoughtsRecalculationTick", Find.TickManager.TicksGame);
				bool flag2 = s.NullOrEmpty();
				if (!flag2)
				{
					string[] array = s.SplitNo(":");
					foreach (string s2 in array)
					{
						string[] array3 = s2.SplitNo("|");
						bool flag3 = array3.Length == 7;
						if (flag3)
						{
							ThoughtDef thoughtDef = DefTool.ThoughtDef(array3[0]);
							bool flag4 = thoughtDef != null;
							if (flag4)
							{
								Pawn otherPawnFromSeparatedString = PawnxTool.GetOtherPawnFromSeparatedString(array3[1]);
								p.AddThought(thoughtDef, otherPawnFromSeparatedString, array3[2].AsInt32(), array3[3], array3[4].AsFloat(), array3[5].AsFloat(), array3[6].AsInt32());
							}
						}
					}
				}
			}
		}

		// Token: 0x06000295 RID: 661 RVA: 0x000136C0 File Offset: 0x000118C0
		public static bool HasOtherPawnMember(this ThoughtDef t)
		{
			return t != null && (t.IsSocial || t.stackLimitForSameOtherPawn > 0 || t.IsTypeOf<Thought_MemorySocialCumulative>() || t.IsTypeOf<Thought_MemorySocial>() || t.IsTypeOf<Thought_SituationalSocial>() || t.IsTypeOf<Thought_PsychicHarmonizer>() || (t.IsTypeOf<Thought_Memory>() && t.GetThoughtLabel(0, null).Contains("{0}")) || t.IsTypeOf<Thought_BondedAnimalMaster>() || t.IsTypeOf<Thought_NotBondedAnimalMaster>() || t.SDefname().Equals("Jealous"));
		}

		// Token: 0x06000296 RID: 662 RVA: 0x00013748 File Offset: 0x00011948
		public static Pawn GetOtherPawn(this Thought t, out string reason)
		{
			Pawn pawn = null;
			reason = "";
			bool flag = t == null;
			Pawn result;
			if (flag)
			{
				result = pawn;
			}
			else
			{
				bool flag2 = t.def.IsTypeOf<Thought_Memory>();
				if (flag2)
				{
					result = ((Thought_Memory)t).otherPawn;
				}
				else
				{
					bool flag3 = t.def.IsTypeOf<Thought_SituationalSocial>();
					if (flag3)
					{
						result = ((Thought_SituationalSocial)t).otherPawn;
					}
					else
					{
						bool flag4 = t.def.IsTypeOf<Thought_BondedAnimalMaster>() || t.def.IsTypeOf<Thought_NotBondedAnimalMaster>();
						if (flag4)
						{
							reason = (string)((Thought_Situational)t).GetMemberValue("reason");
							List<DirectPawnRelation> directRelations = t.pawn.relations.DirectRelations;
							for (int i = 0; i < directRelations.Count; i++)
							{
								DirectPawnRelation directPawnRelation = directRelations[i];
								pawn = directPawnRelation.otherPawn;
								bool flag5 = directPawnRelation.def == PawnRelationDefOf.Bond && !pawn.Dead && pawn.Spawned && pawn.Faction == Faction.OfPlayer;
								if (flag5)
								{
									bool flag6 = pawn.GetPawnName(false) == reason;
									if (flag6)
									{
										return pawn;
									}
								}
							}
							result = null;
						}
						else
						{
							bool flag7 = t.def.SDefname().Equals("Jealous");
							if (flag7)
							{
								reason = (string)((Thought_Situational)t).GetMemberValue("reason");
								result = null;
							}
							else
							{
								bool flag8 = t.def.IsTypeOf<ThoughtWorker_WantToSleepWithSpouseOrLover>() || t.def.IsTypeOf<Thought_OpinionOfMyLover>();
								if (flag8)
								{
									DirectPawnRelation directPawnRelation2 = LovePartnerRelationUtility.ExistingMostLikedLovePartnerRel(t.pawn, false);
									bool flag9 = directPawnRelation2 != null;
									if (flag9)
									{
										reason = directPawnRelation2.def.GetGenderSpecificLabel(directPawnRelation2.otherPawn);
										return directPawnRelation2.otherPawn;
									}
								}
								result = pawn;
							}
						}
					}
				}
			}
			return result;
		}

		// Token: 0x06000297 RID: 663 RVA: 0x00013924 File Offset: 0x00011B24
		public static float GetOpinionOffset(this Thought t)
		{
			float num = float.MinValue;
			bool flag = t == null;
			float result;
			if (flag)
			{
				result = num;
			}
			else
			{
				bool flag2 = t.def.IsTypeOf<Thought_MemorySocial>();
				if (flag2)
				{
					num = ((Thought_MemorySocial)t).opinionOffset;
				}
				else
				{
					bool flag3 = t.def.IsTypeOf<Thought_SituationalSocial>();
					if (flag3)
					{
						num = ((t.CurStageIndex < 0) ? 0f : ((Thought_SituationalSocial)t).OpinionOffset());
					}
					else
					{
						num = float.MinValue;
					}
				}
				result = num;
			}
			return result;
		}

		// Token: 0x06000298 RID: 664 RVA: 0x0001399C File Offset: 0x00011B9C
		public static float TryGetMoodOffset(this Thought t)
		{
			float result = 0f;
			bool flag = t != null;
			if (flag)
			{
				try
				{
					result = t.MoodOffset();
				}
				catch
				{
				}
			}
			return result;
		}

		// Token: 0x06000299 RID: 665 RVA: 0x000139E0 File Offset: 0x00011BE0
		public static string GetThoughtLabel(this ThoughtDef t, int stage = 0, Pawn p = null)
		{
			bool flag = t == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = null;
				bool flag2 = !t.stages.NullOrEmpty<ThoughtStage>();
				if (flag2)
				{
					try
					{
						text = (text ?? t.stages[stage].labelSocial);
					}
					catch
					{
					}
					try
					{
						text = (text ?? t.stages[stage].label);
					}
					catch
					{
					}
					try
					{
						text = (text ?? t.stages[stage].untranslatedLabelSocial);
					}
					catch
					{
					}
					try
					{
						text = (text ?? t.stages[stage].untranslatedLabel);
					}
					catch
					{
					}
				}
				text = (text ?? t.label);
				text = (text ?? t.defName);
				text = text.CapitalizeFirst();
				bool flag3 = text.StartsWith("{");
				if (flag3)
				{
					bool flag4 = t.defName.StartsWith("HumanLeatherApparel");
					if (flag4)
					{
						text = string.Format(text, "Apparel");
					}
					else
					{
						bool flag5 = t.defName == "JealousRage" || t.defName == "OnKill_GoodThought" || t.defName == "RelicAtRitual" || t.defName == "IdeoBuildingMissing" || t.defName == "IdeoBuildingDisrespected" || t.defName == "RitualDelayed" || t.defName == "OnKill_BadThought" || t.defName == "ObservedTerror" || t.defName == "KillThirst" || t.defName.StartsWith("BondedThought");
						if (flag5)
						{
							text = t.defName;
						}
						else
						{
							bool flag6 = p != null && t.Worker != null;
							if (flag6)
							{
								text = t.Worker.PostProcessLabel(p, text);
							}
						}
					}
				}
				result = text;
			}
			return result;
		}

		// Token: 0x0600029A RID: 666 RVA: 0x00013C20 File Offset: 0x00011E20
		public static string GetThoughtDescription(this ThoughtDef t, int stage = 0, Thought thought = null)
		{
			bool flag = t == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = null;
				bool flag2 = !t.stages.NullOrEmpty<ThoughtStage>();
				if (flag2)
				{
					try
					{
						text = t.stages[stage].description;
					}
					catch
					{
					}
				}
				text = (text ?? t.description);
				text = (text ?? "");
				try
				{
					bool flag3 = thought != null;
					if (flag3)
					{
						bool flag4 = t.IsClassOf<Thought_IdeoRoleEmpty>();
						if (flag4)
						{
							Thought_IdeoRoleEmpty thought_IdeoRoleEmpty = (Thought_IdeoRoleEmpty)thought;
							text = text.Replace("{0}", thought_IdeoRoleEmpty.Role.ideo.memberName).Replace("{ROLE_labelIndef}", thought_IdeoRoleEmpty.Role.LabelCap);
						}
						else
						{
							bool flag5 = t.IsClassOf<Thought_MemoryRoyalTitle>();
							if (flag5)
							{
								Thought_MemoryRoyalTitle thought_MemoryRoyalTitle = (Thought_MemoryRoyalTitle)thought;
								string newValue = (CEditor.API.Pawn.gender == Gender.Female) ? thought_MemoryRoyalTitle.titleDef.labelFemale : thought_MemoryRoyalTitle.titleDef.label;
								text = text.Replace("{TITLE_label}", newValue);
							}
							else
							{
								bool flag6 = t.IsClassOf<Thought_IdeoRoleLost>();
								if (flag6)
								{
									Thought_IdeoRoleLost thought_IdeoRoleLost = (Thought_IdeoRoleLost)thought;
									text = text.Replace("{0}", thought_IdeoRoleLost.Role.ideo.memberName).Replace("{ROLE_labelIndef}", thought_IdeoRoleLost.Role.LabelCap);
								}
								else
								{
									bool flag7 = t.IsClassOf<Thought_IdeoRoleApparelRequirementNotMet>();
									if (flag7)
									{
										Thought_IdeoRoleApparelRequirementNotMet thought_IdeoRoleApparelRequirementNotMet = (Thought_IdeoRoleApparelRequirementNotMet)thought;
										text = text.Replace("{0}", thought_IdeoRoleApparelRequirementNotMet.Role.ideo.memberName).Replace("{ROLE_labelIndef}", thought_IdeoRoleApparelRequirementNotMet.Role.LabelCap);
									}
									else
									{
										bool flag8 = t.IsClassOf<Thought_RelicAtRitual>();
										if (flag8)
										{
											Thought_RelicAtRitual thought_RelicAtRitual = (Thought_RelicAtRitual)thought;
											text = text.Replace("{RELICNAME_labelIndef}", thought_RelicAtRitual.relicName);
										}
										else
										{
											bool flag9 = t.IsClassOf<Thought_Situational_WearingDesiredApparel>();
											if (flag9)
											{
												Thought_Situational_WearingDesiredApparel thought_Situational_WearingDesiredApparel = (Thought_Situational_WearingDesiredApparel)thought;
												Precept_Apparel precept_Apparel = (Precept_Apparel)thought_Situational_WearingDesiredApparel.sourcePrecept;
												text = text.Replace("{APPAREL_indefinite}", precept_Apparel.apparelDef.LabelCap);
											}
										}
									}
								}
							}
						}
					}
				}
				catch
				{
				}
				text = (text ?? "");
				result = text.CapitalizeFirst();
			}
			return result;
		}

		// Token: 0x0600029B RID: 667 RVA: 0x00013E94 File Offset: 0x00012094
		public static int CountOfDefs<T>(List<Thought> l, out Thought example, string startsWdefname = null)
		{
			int num = 0;
			example = null;
			foreach (Thought thought in l)
			{
				bool flag = thought.def.IsClassOf<T>();
				if (flag)
				{
					bool flag2 = startsWdefname == null || thought.def.defName.StartsWith(startsWdefname);
					if (flag2)
					{
						example = thought;
						num++;
					}
				}
			}
			return num;
		}

		// Token: 0x0600029C RID: 668 RVA: 0x00013F24 File Offset: 0x00012124
		public static string GetThoughtLabel(this Thought t)
		{
			bool flag = t == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				int stage = ThoughtTool.FixStageValue(t.def, t.CurStageIndex);
				result = t.def.GetThoughtLabel(stage, null);
			}
			return result;
		}

		// Token: 0x0600029D RID: 669 RVA: 0x00013F68 File Offset: 0x00012168
		public static string GetThoughtDescription(this Thought t)
		{
			bool flag = t == null;
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				int stage = ThoughtTool.FixStageValue(t.def, t.CurStageIndex);
				result = t.def.GetThoughtDescription(stage, t);
			}
			return result;
		}

		// Token: 0x0600029E RID: 670 RVA: 0x00013FAC File Offset: 0x000121AC
		public static List<Thought> GetThoughtsSorted(this Pawn p)
		{
			List<Thought> list = new List<Thought>();
			bool flag = p.HasMemoryTracker() && p.HasSituationalTracker();
			if (flag)
			{
				List<Thought_Memory> memories = p.needs.mood.thoughts.memories.Memories;
				List<Thought_Situational> allThoughtSituationals = p.GetAllThoughtSituationals();
				List<Thought> list2 = new List<Thought>();
				list2.AddRange(memories);
				list2.AddRange(allThoughtSituationals);
				List<Thought> collection = (from td in list2
				where td.GetOpinionOffset() == float.MinValue
				orderby td.TryGetMoodOffset()
				select td).ToList<Thought>();
				List<Thought> collection2 = (from td in list2
				where td.GetOpinionOffset() != float.MinValue
				orderby td.GetOpinionOffset()
				select td).ToList<Thought>();
				list.AddRange(collection);
				list.Reverse();
				list.AddRange(collection2);
			}
			return list;
		}

		// Token: 0x0600029F RID: 671 RVA: 0x000140D8 File Offset: 0x000122D8
		public static void AddThought(this Pawn pawn, ThoughtDef t, Pawn otherPawn = null, int stage = 0, string optDefName = null, float moodPowerFactor = 1f, float opinionOffset = 1f, int age = 0)
		{
			bool flag = pawn == null || t == null;
			if (!flag)
			{
				bool flag2 = t.IsTypeOf<Thought_Memory>();
				if (flag2)
				{
					pawn.AddThought_Memory(t, otherPawn, stage, optDefName, moodPowerFactor, opinionOffset, age);
				}
				else
				{
					bool flag3 = t.IsTypeOf<Thought_Situational>();
					if (flag3)
					{
						ThoughtTool.AddThoughtSituational(pawn, t, otherPawn, stage, optDefName, moodPowerFactor, opinionOffset);
					}
				}
			}
		}

		// Token: 0x060002A0 RID: 672 RVA: 0x00014130 File Offset: 0x00012330
		private static void AddThought_Memory(this Pawn p, ThoughtDef t, Pawn otherPawn, int stage, string optDefName, float moodPowerFactor, float opinionOffset, int age)
		{
			IndividualThoughtToAdd individualThoughtToAdd = new IndividualThoughtToAdd(t, p, otherPawn, moodPowerFactor, opinionOffset);
			individualThoughtToAdd.thought.SetForcedStage(stage);
			bool flag = t.IsTypeOf<Thought_Memory>();
			if (flag)
			{
				individualThoughtToAdd.thought.age = age;
				individualThoughtToAdd.thought.moodPowerFactor = moodPowerFactor;
			}
			bool flag2 = t.IsTypeOf<Thought_MemorySocial>();
			if (flag2)
			{
				((Thought_MemorySocial)individualThoughtToAdd.thought).opinionOffset = opinionOffset;
			}
			bool flag3 = t.IsTypeOf<Thought_MemoryRoyalTitle>() && !optDefName.NullOrEmpty();
			if (flag3)
			{
				((Thought_MemoryRoyalTitle)individualThoughtToAdd.thought).titleDef = DefTool.GetDef<RoyalTitleDef>(optDefName);
			}
			bool flag4 = t.IsTypeOf<Thought_PsychicHarmonizer>() && p.HasHealthTracker();
			if (flag4)
			{
				List<BodyPartRecord> listOfAllowedBodyPartRecords = p.GetListOfAllowedBodyPartRecords(HediffDefOf.PsychicHarmonizer);
				p.AddHediff2(false, HediffDefOf.PsychicHarmonizer, 1f, listOfAllowedBodyPartRecords.FirstOrDefault<BodyPartRecord>(), false, -1, -1, -1, null);
				foreach (Hediff hediff in p.health.hediffSet.hediffs)
				{
					bool flag5 = hediff.def == HediffDefOf.PsychicHarmonizer;
					if (flag5)
					{
						((Thought_PsychicHarmonizer)individualThoughtToAdd.thought).harmonizer = hediff;
						break;
					}
				}
			}
			bool flag6 = t.IsTypeOf<Thought_WeaponTrait>() && p.HasEquipmentTracker();
			if (flag6)
			{
				List<ThingWithComps> allEquipmentListForReading = p.equipment.AllEquipmentListForReading;
				((Thought_WeaponTrait)individualThoughtToAdd.thought).weapon = allEquipmentListForReading.FirstOrDefault<ThingWithComps>();
			}
			bool flag7 = t.IsTypeOf<Thought_MemorySocial>() && otherPawn == null;
			if (!flag7)
			{
				individualThoughtToAdd.Add();
			}
		}

		// Token: 0x060002A1 RID: 673 RVA: 0x000142E8 File Offset: 0x000124E8
		private static void AddThoughtSituational(Pawn p, ThoughtDef t, Pawn otherPawn, int stage, string optDefName, float moodPowerFactor, float opinionOffset)
		{
			Thought thought = ThoughtMaker.MakeThought(t);
			thought.pawn = p;
			int num = ThoughtTool.FixStageValue(t, stage);
			thought.SetMemberValue("curStageIndex", num);
			bool flag = otherPawn != null;
			if (flag)
			{
				bool flag2 = t.IsTypeOf<Thought_SituationalSocial>();
				if (flag2)
				{
					((Thought_SituationalSocial)thought).otherPawn = otherPawn;
				}
				bool flag3 = t.IsTypeOf<Thought_BondedAnimalMaster>() || t.IsTypeOf<Thought_NotBondedAnimalMaster>() || t.SDefname().Equals("Jealous");
				if (flag3)
				{
					((Thought_Situational)thought).SetMemberValue("reason", otherPawn.LabelShort);
				}
			}
			try
			{
				object memberValue = p.needs.mood.thoughts.situational.GetMemberValue("cachedThoughts");
				memberValue.CallMethod("Add", new object[]
				{
					thought
				});
				p.needs.mood.thoughts.situational.SetMemberValue("cachedThoughts", memberValue);
			}
			catch (Exception ex)
			{
				ThoughtTool.RemoveThoughtSituational(p, thought);
				Log.Error(ex.Message + "\n" + ex.StackTrace);
				MessageTool.Show("failed to add thought. reason = this thought or thought stage cause an exception. action was rolled back.", null);
			}
		}

		// Token: 0x060002A2 RID: 674 RVA: 0x00014428 File Offset: 0x00012628
		public static void ClearAllThoughts(this Pawn p)
		{
			bool flag = p.HasMemoryTracker();
			if (flag)
			{
				p.needs.mood.thoughts.memories.Memories.Clear();
			}
			bool flag2 = p.HasSituationalTracker();
			if (flag2)
			{
				p.needs.mood.thoughts.situational.Notify_SituationalThoughtsDirty();
				p.needs.mood.thoughts.situational.SetMemberValue("lastMoodThoughtsRecalculationTick", Find.TickManager.TicksGame);
			}
		}

		// Token: 0x060002A3 RID: 675 RVA: 0x000144B8 File Offset: 0x000126B8
		public static void RemoveThought(this Pawn p, Thought t)
		{
			bool flag = p == null || t == null;
			if (!flag)
			{
				bool flag2 = t.def.IsTypeOf<Thought_Memory>();
				if (flag2)
				{
					ThoughtTool.RemoveThoughtMemory(p, t);
				}
				else
				{
					bool flag3 = t.def.IsTypeOf<Thought_Situational>();
					if (flag3)
					{
						ThoughtTool.RemoveThoughtSituational(p, t);
					}
				}
			}
		}

		// Token: 0x060002A4 RID: 676 RVA: 0x00014508 File Offset: 0x00012708
		private static void RemoveThoughtMemory(Pawn p, Thought t)
		{
			bool flag = !p.HasMemoryTracker() || t == null;
			if (!flag)
			{
				try
				{
					Thought_Memory th = t as Thought_Memory;
					p.needs.mood.thoughts.memories.RemoveMemory(th);
				}
				catch
				{
				}
			}
		}

		// Token: 0x060002A5 RID: 677 RVA: 0x00014568 File Offset: 0x00012768
		private static void RemoveThoughtSituational(Pawn p, Thought t)
		{
			bool flag = !p.HasSituationalTracker() || t == null;
			if (!flag)
			{
				try
				{
					Thought_Situational item = t as Thought_Situational;
					object memberValue = p.needs.mood.thoughts.situational.GetMemberValue("cachedThoughts");
					bool flag2 = memberValue != null;
					if (flag2)
					{
						List<Thought_Situational> list = memberValue as List<Thought_Situational>;
						bool flag3 = list.Contains(item);
						if (flag3)
						{
							list.Remove(item);
							p.needs.mood.thoughts.situational.SetMemberValue("cachedThoughts", list);
						}
					}
				}
				catch
				{
				}
			}
		}
	}
}
