﻿using System;
using RimWorld;
using Verse;
using Verse.AI;

namespace CharacterEditor
{
	// Token: 0x0200001E RID: 30
	public static class TrackerTool
	{
		// Token: 0x06000115 RID: 277 RVA: 0x00008775 File Offset: 0x00006975
		public static bool HasAbilityTracker(this Pawn p)
		{
			return p != null && p.abilities != null;
		}

		// Token: 0x06000116 RID: 278 RVA: 0x00008786 File Offset: 0x00006986
		public static bool HasPsyTracker(this Pawn p)
		{
			return p != null && p.psychicEntropy != null;
		}

		// Token: 0x06000117 RID: 279 RVA: 0x00008797 File Offset: 0x00006997
		public static bool HasAgeTracker(this Pawn p)
		{
			return p != null && p.ageTracker != null;
		}

		// Token: 0x06000118 RID: 280 RVA: 0x000087A8 File Offset: 0x000069A8
		public static bool HasApparelTracker(this Pawn pawn)
		{
			return pawn != null && pawn.apparel != null;
		}

		// Token: 0x06000119 RID: 281 RVA: 0x000087B9 File Offset: 0x000069B9
		public static bool HasForcedApparel(this Pawn pawn)
		{
			return pawn != null && pawn.outfits != null && pawn.outfits.forcedHandler != null && !pawn.outfits.forcedHandler.ForcedApparel.NullOrEmpty<Apparel>();
		}

		// Token: 0x0600011A RID: 282 RVA: 0x000087EE File Offset: 0x000069EE
		public static bool HasHealthTracker(this Pawn p)
		{
			return p != null && p.health != null && p.health.hediffSet != null;
		}

		// Token: 0x0600011B RID: 283 RVA: 0x0000880C File Offset: 0x00006A0C
		public static bool HasStoryTracker(this Pawn p)
		{
			return p != null && p.story != null;
		}

		public static bool HasGeneTracker(this Pawn p)
    {
			return p != null && p.genes != null;
    }
		// Token: 0x0600011C RID: 284 RVA: 0x0000881D File Offset: 0x00006A1D
		public static bool HasStyleTracker(this Pawn p)
		{
			return p != null && p.style != null;
		}

		// Token: 0x0600011D RID: 285 RVA: 0x0000882E File Offset: 0x00006A2E
		public static bool HasRoyalTitle(this Pawn p)
		{
			return p.HasRoyaltyTracker() && !p.royalty.AllTitlesForReading.NullOrEmpty<RoyalTitle>();
		}

		// Token: 0x0600011E RID: 286 RVA: 0x0000884E File Offset: 0x00006A4E
		public static bool HasRoyaltyTracker(this Pawn p)
		{
			return p != null && p.royalty != null;
		}

		// Token: 0x0600011F RID: 287 RVA: 0x0000885F File Offset: 0x00006A5F
		public static bool HasRecordsTracker(this Pawn p)
		{
			return p != null && p.records != null;
		}

		// Token: 0x06000120 RID: 288 RVA: 0x00008870 File Offset: 0x00006A70
		public static bool HasRelationTracker(this Pawn p)
		{
			return p != null && p.relations != null;
		}

		// Token: 0x06000121 RID: 289 RVA: 0x00008881 File Offset: 0x00006A81
		public static bool HasCarryTracker(this Pawn p)
		{
			return p != null && p.carryTracker != null;
		}

		// Token: 0x06000122 RID: 290 RVA: 0x00008892 File Offset: 0x00006A92
		public static bool HasSkillTracker(this Pawn p)
		{
			return p != null && p.skills != null;
		}

		// Token: 0x06000123 RID: 291 RVA: 0x000088A3 File Offset: 0x00006AA3
		public static bool HasInventoryTracker(this Pawn pawn)
		{
			return pawn != null && pawn.inventory != null;
		}

		// Token: 0x06000124 RID: 292 RVA: 0x000088B4 File Offset: 0x00006AB4
		public static bool HasMemoryTracker(this Pawn p)
		{
			return p.HasThoughtsTracker() && p.needs.mood.thoughts.memories != null;
		}

		// Token: 0x06000125 RID: 293 RVA: 0x000088D9 File Offset: 0x00006AD9
		public static bool HasThoughtsTracker(this Pawn p)
		{
			return p != null && p.needs != null && p.needs.mood != null && p.needs.mood.thoughts != null;
		}

		// Token: 0x06000126 RID: 294 RVA: 0x00008909 File Offset: 0x00006B09
		public static bool HasSituationalTracker(this Pawn p)
		{
			return p.HasThoughtsTracker() && p.needs.mood.thoughts.situational != null;
		}

		// Token: 0x06000127 RID: 295 RVA: 0x0000892E File Offset: 0x00006B2E
		public static bool HasNeedsTracker(this Pawn p)
		{
			return p != null && p.needs != null;
		}

		// Token: 0x06000128 RID: 296 RVA: 0x0000893F File Offset: 0x00006B3F
		public static bool HasTraitTracker(this Pawn p)
		{
			return p != null && p.story != null && p.story.traits != null;
		}

		// Token: 0x06000129 RID: 297 RVA: 0x0000895D File Offset: 0x00006B5D
		public static bool HasEquipmentTracker(this Pawn pawn)
		{
			return pawn != null && pawn.equipment != null;
		}

		// Token: 0x0600012A RID: 298 RVA: 0x0000896E File Offset: 0x00006B6E
		public static bool HasWorkTracker(this Pawn p)
		{
			return p != null && p.workSettings != null;
		}

		// Token: 0x0600012B RID: 299 RVA: 0x0000897F File Offset: 0x00006B7F
		public static bool HasTimeTracker(this Pawn p)
		{
			return p != null && p.timetable != null;
		}

		// Token: 0x0600012C RID: 300 RVA: 0x00008990 File Offset: 0x00006B90
		public static void FixInvalidTracker(this Pawn newPawn)
		{
			try
			{
				bool flag = newPawn == null || newPawn.Faction.IsCreature() || newPawn.kindDef.IsMechanoid() || newPawn.kindDef.IsZombie() || newPawn.kindDef.IsAbomination() || newPawn.kindDef.RaceProps.Animal;
				if (!flag)
				{
					bool flag2 = false;
					bool flag3 = newPawn.outfits == null;
					if (flag3)
					{
						newPawn.outfits = new Pawn_OutfitTracker(newPawn);
						bool flag4 = flag2;
						if (flag4)
						{
							MessageTool.Show("fixed outfits for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag5 = newPawn.outfits != null;
					if (flag5)
					{
						bool flag6 = newPawn.outfits.CurrentOutfit != null && newPawn.outfits.CurrentOutfit.filter != null;
						if (flag6)
						{
							newPawn.outfits.CurrentOutfit.filter.SetAllowAll(null, false);
						}
					}
					bool flag7 = newPawn.foodRestriction == null;
					if (flag7)
					{
						newPawn.foodRestriction = new Pawn_FoodRestrictionTracker(newPawn);
						bool flag8 = flag2;
						if (flag8)
						{
							MessageTool.Show("fixed food restriction for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag9 = newPawn.drugs == null;
					if (flag9)
					{
						newPawn.drugs = new Pawn_DrugPolicyTracker(newPawn);
						bool flag10 = flag2;
						if (flag10)
						{
							MessageTool.Show("fixed drugs for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag11 = newPawn.drafter == null;
					if (flag11)
					{
						newPawn.drafter = new Pawn_DraftController(newPawn);
						bool flag12 = flag2;
						if (flag12)
						{
							MessageTool.Show("fixed drafter for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag13 = newPawn.playerSettings == null;
					if (flag13)
					{
						newPawn.playerSettings = new Pawn_PlayerSettings(newPawn);
						bool flag14 = flag2;
						if (flag14)
						{
							MessageTool.Show("fixed player settings for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag15 = newPawn.timetable == null;
					if (flag15)
					{
						newPawn.timetable = new Pawn_TimetableTracker(newPawn);
						bool flag16 = flag2;
						if (flag16)
						{
							MessageTool.Show("fixed timetable for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag17 = newPawn.needs == null;
					if (flag17)
					{
						newPawn.needs = new Pawn_NeedsTracker(newPawn);
						bool flag18 = flag2;
						if (flag18)
						{
							MessageTool.Show("fixed needs for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag19 = newPawn.mindState == null;
					if (flag19)
					{
						newPawn.mindState = new Pawn_MindState(newPawn);
						bool flag20 = flag2;
						if (flag20)
						{
							MessageTool.Show("fixed mindstate for " + newPawn.kindDef.defName, null);
						}
						bool flag21 = newPawn.mindState.mentalStateHandler == null;
						if (flag21)
						{
							newPawn.mindState.mentalStateHandler = new MentalStateHandler(newPawn);
							bool flag22 = flag2;
							if (flag22)
							{
								MessageTool.Show("fixed mentatlstatehandler for " + newPawn.kindDef.defName, null);
							}
						}
						bool flag23 = newPawn.mindState.mentalBreaker == null;
						if (flag23)
						{
							newPawn.mindState.mentalBreaker = new MentalBreaker(newPawn);
							bool flag24 = flag2;
							if (flag24)
							{
								MessageTool.Show("fixed mentalbreaker for " + newPawn.kindDef.defName, null);
							}
						}
						bool flag25 = newPawn.mindState.inspirationHandler == null;
						if (flag25)
						{
							newPawn.mindState.inspirationHandler = new InspirationHandler(newPawn);
							bool flag26 = flag2;
							if (flag26)
							{
								MessageTool.Show("fixed inspirationhandler for " + newPawn.kindDef.defName, null);
							}
						}
						bool flag27 = newPawn.mindState.priorityWork == null;
						if (flag27)
						{
							newPawn.mindState.priorityWork = new PriorityWork(newPawn);
							bool flag28 = flag2;
							if (flag28)
							{
								MessageTool.Show("fixed prioritywork for " + newPawn.kindDef.defName, null);
							}
						}
					}
					bool flag29 = newPawn.jobs == null;
					if (flag29)
					{
						newPawn.jobs = new Pawn_JobTracker(newPawn);
						bool flag30 = flag2;
						if (flag30)
						{
							MessageTool.Show("fixed jobs for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag31 = newPawn.inventory == null;
					if (flag31)
					{
						newPawn.inventory = new Pawn_InventoryTracker(newPawn);
						bool flag32 = flag2;
						if (flag32)
						{
							MessageTool.Show("fixed inventory for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag33 = newPawn.interactions == null;
					if (flag33)
					{
						newPawn.interactions = new Pawn_InteractionsTracker(newPawn);
						bool flag34 = flag2;
						if (flag34)
						{
							MessageTool.Show("fixed interactions for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag35 = newPawn.health == null;
					if (flag35)
					{
						newPawn.health = new Pawn_HealthTracker(newPawn);
						bool flag36 = flag2;
						if (flag36)
						{
							MessageTool.Show("fixed health for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag37 = newPawn.guest == null;
					if (flag37)
					{
						newPawn.guest = new Pawn_GuestTracker(newPawn);
						bool flag38 = flag2;
						if (flag38)
						{
							MessageTool.Show("fixed guest for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag39 = newPawn.filth == null;
					if (flag39)
					{
						newPawn.filth = new Pawn_FilthTracker(newPawn);
						bool flag40 = flag2;
						if (flag40)
						{
							MessageTool.Show("fixed filth for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag41 = newPawn.equipment == null;
					if (flag41)
					{
						newPawn.equipment = new Pawn_EquipmentTracker(newPawn);
						bool flag42 = flag2;
						if (flag42)
						{
							MessageTool.Show("fixed equipment for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag43 = newPawn.carryTracker == null;
					if (flag43)
					{
						newPawn.carryTracker = new Pawn_CarryTracker(newPawn);
						bool flag44 = flag2;
						if (flag44)
						{
							MessageTool.Show("fixed carry tracker for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag45 = newPawn.apparel == null;
					if (flag45)
					{
						newPawn.apparel = new Pawn_ApparelTracker(newPawn);
						bool flag46 = flag2;
						if (flag46)
						{
							MessageTool.Show("fixed apparel for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag47 = newPawn.abilities == null;
					if (flag47)
					{
						newPawn.abilities = new Pawn_AbilityTracker(newPawn);
						bool flag48 = flag2;
						if (flag48)
						{
							MessageTool.Show("fixed abilities for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag49 = newPawn.pather == null;
					if (flag49)
					{
						newPawn.pather = new Pawn_PathFollower(newPawn);
						bool flag50 = flag2;
						if (flag50)
						{
							MessageTool.Show("fixed path follower for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag51 = newPawn.psychicEntropy == null;
					if (flag51)
					{
						newPawn.psychicEntropy = new Pawn_PsychicEntropyTracker(newPawn);
						bool flag52 = flag2;
						if (flag52)
						{
							MessageTool.Show("fixed psychic entropy for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag53 = newPawn.records == null;
					if (flag53)
					{
						newPawn.records = new Pawn_RecordsTracker(newPawn);
						bool flag54 = flag2;
						if (flag54)
						{
							MessageTool.Show("fixed records for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag55 = newPawn.relations == null;
					if (flag55)
					{
						newPawn.relations = new Pawn_RelationsTracker(newPawn);
						bool flag56 = flag2;
						if (flag56)
						{
							MessageTool.Show("fixed releations for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag57 = newPawn.rotationTracker == null;
					if (flag57)
					{
						newPawn.rotationTracker = new Pawn_RotationTracker(newPawn);
						bool flag58 = flag2;
						if (flag58)
						{
							MessageTool.Show("fixed rotation tracker for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag59 = newPawn.royalty == null;
					if (flag59)
					{
						newPawn.royalty = new Pawn_RoyaltyTracker(newPawn);
						bool flag60 = flag2;
						if (flag60)
						{
							MessageTool.Show("fixed royalty for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag61 = newPawn.stances == null;
					if (flag61)
					{
						newPawn.stances = new Pawn_StanceTracker(newPawn);
						bool flag62 = flag2;
						if (flag62)
						{
							MessageTool.Show("fixed stances for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag63 = newPawn.thinker == null;
					if (flag63)
					{
						newPawn.thinker = new Pawn_Thinker(newPawn);
						bool flag64 = flag2;
						if (flag64)
						{
							MessageTool.Show("fixed thinker for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag65 = newPawn.workSettings == null;
					if (flag65)
					{
						newPawn.workSettings = new Pawn_WorkSettings(newPawn);
						bool flag66 = flag2;
						if (flag66)
						{
							MessageTool.Show("fixed work setting for " + newPawn.kindDef.defName, null);
						}
					}
					bool flag67 = newPawn.Faction == Faction.OfPlayer;
					if (flag67)
					{
						newPawn.workSettings.EnableAndInitialize();
					}
				}
			}
			catch (Exception ex)
			{
				Log.Message(ex.Message + "\n" + ex.StackTrace);
      }
		}
	}
}
