﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000037 RID: 55
	public static class TraitTool
	{
		// Token: 0x060002A6 RID: 678 RVA: 0x0001461C File Offset: 0x0001281C
		public static string GetTraitAsSeparatedString(this Trait t)
		{
			bool flag = t == null || t.def.IsNullOrEmpty();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				text = text + t.def.defName + "|";
				text += t.Degree.ToString();
				result = text;
			}
			return result;
		}

		// Token: 0x060002A7 RID: 679 RVA: 0x00014680 File Offset: 0x00012880
		public static string GetAllTraitsAsSeparatedString(this Pawn p)
		{
			bool flag = !p.HasTraitTracker() || p.story.traits.allTraits.NullOrEmpty<Trait>();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				foreach (Trait t in p.story.traits.allTraits)
				{
					text += t.GetTraitAsSeparatedString();
					text += ":";
				}
				text = text.SubstringRemoveLast();
				result = text;
			}
			return result;
		}

		// Token: 0x060002A8 RID: 680 RVA: 0x00014734 File Offset: 0x00012934
		public static void SetTraitsFromSeparatedString(this Pawn p, string s)
		{
			bool flag = !p.HasTraitTracker();
			if (!flag)
			{
				p.story.traits.allTraits.Clear();
				bool flag2 = s.NullOrEmpty();
				if (!flag2)
				{
					string[] array = s.Split(new string[]
					{
						":"
					}, StringSplitOptions.None);
					foreach (string text in array)
					{
						string[] array3 = text.Split(new string[]
						{
							"|"
						}, StringSplitOptions.None);
						bool flag3 = array3.Length == 2;
						if (flag3)
						{
							TraitDef traitDef = DefTool.TraitDef(array3[0]);
							bool flag4 = traitDef != null;
							if (flag4)
							{
								p.AddTrait(traitDef, traitDef.GetDegreeDataOrFirst(array3[1].AsInt32()), false, false, null);
							}
						}
					}
					MeditationFocusTypeAvailabilityCache.ClearFor(p);
					StatsReportUtility.Reset();
				}
			}
		}

		// Token: 0x060002A9 RID: 681 RVA: 0x00014814 File Offset: 0x00012A14
		public static string GetTooltipForSkillpoints(this Pawn pawn, SkillDef skill)
		{
			bool flag = pawn == null || skill == null || !pawn.HasStoryTracker();
			string result;
			if (flag)
			{
				result = "";
			}
			else
			{
				string text = "";
				try
				{
					foreach (Trait trait in pawn.story.traits.allTraits)
					{
						bool flag2 = trait.CurrentData != null && !trait.CurrentData.skillGains.NullOrEmpty<SkillDef, int>();
						if (flag2)
						{
							foreach (SkillDef skillDef in trait.CurrentData.skillGains.Keys)
							{
								bool flag3 = skillDef.defName == skill.defName;
								if (flag3)
								{
									int num = trait.CurrentData.skillGains[skillDef];
									bool flag4 = num == 0;
									if (!flag4)
									{
										bool flag5 = num > 0;
										if (flag5)
										{
											text += "+";
										}
										else
										{
											text = (text ?? "");
										}
										text = text + num.ToString() + " ";
										string str = text;
										string str2;
										if ((str2 = trait.LabelCap) == null)
										{
											str2 = (trait.Label ?? trait.def.label);
										}
										text = str + str2;
										text += "\n";
									}
								}
							}
						}
					}
					BackstoryDef backstory = pawn.story.GetBackstory(BackstorySlot.Adulthood);
					BackstoryDef backstory2 = pawn.story.GetBackstory(BackstorySlot.Childhood);
					bool flag6 = backstory != null && !backstory.skillGains.NullOrEmpty<SkillDef, int>();
					if (flag6)
					{
						foreach (SkillDef skillDef2 in backstory.skillGains.Keys)
						{
							bool flag7 = skillDef2.defName == skill.defName;
							if (flag7)
							{
								int num2 = backstory.skillGains[skillDef2];
								bool flag8 = num2 == 0;
								if (!flag8)
								{
									bool flag9 = num2 > 0;
									if (flag9)
									{
										text += "+";
									}
									else
									{
										text = (text ?? "");
									}
									text = text + num2.ToString() + " ";
									text += backstory.TitleCapFor(pawn.gender);
									text += "\n";
								}
							}
						}
					}
					bool flag10 = backstory2 != null && !backstory2.skillGains.NullOrEmpty<SkillDef, int>();
					if (flag10)
					{
						foreach (SkillDef skillDef3 in backstory2.skillGains.Keys)
						{
							bool flag11 = skillDef3.defName == skill.defName;
							if (flag11)
							{
								int num3 = backstory2.skillGains[skillDef3];
								bool flag12 = num3 == 0;
								if (!flag12)
								{
									bool flag13 = num3 > 0;
									if (flag13)
									{
										text += "+";
									}
									else
									{
										text = (text ?? "");
									}
									text = text + num3.ToString() + " ";
									text += backstory2.TitleCapFor(pawn.gender);
									text += "\n";
								}
							}
						}
					}
				}
				catch
				{
				}
				result = text;
			}
			return result;
		}

		// Token: 0x060002AA RID: 682 RVA: 0x00014C2C File Offset: 0x00012E2C
		public static int GetTraitOffsetForSkill(this Pawn pawn, SkillDef skill)
		{
			bool flag = pawn == null || skill == null || !pawn.HasStoryTracker();
			int result;
			if (flag)
			{
				result = 0;
			}
			else
			{
				int num = 0;
				foreach (Trait trait in pawn.story.traits.allTraits)
				{
					bool flag2 = trait.CurrentData != null && !trait.CurrentData.skillGains.NullOrEmpty<SkillDef, int>();
					if (flag2)
					{
						foreach (SkillDef skillDef in trait.CurrentData.skillGains.Keys)
						{
							bool flag3 = skillDef.defName == skill.defName;
							if (flag3)
							{
								num += trait.CurrentData.skillGains[skillDef];
							}
						}
					}
				}
				result = num;
			}
			return result;
		}

		// Token: 0x060002AB RID: 683 RVA: 0x00014D58 File Offset: 0x00012F58
		public static void AddTrait(this Pawn pawn, TraitDef traitDef, TraitDegreeData degreeData, bool random = false, bool doChangeSkillValue = false, Trait oldTraitToReplace = null)
		{
			bool flag = pawn == null || pawn.story == null || (!random && traitDef == null);
			if (!flag)
			{
				if (random)
				{
					traitDef = TraitTool.GetRandomTrait(out degreeData);
					bool flag2 = pawn.HasTrait(traitDef, degreeData);
					if (flag2)
					{
						traitDef = TraitTool.GetRandomTrait(out degreeData);
					}
				}
				bool flag3 = pawn.story.traits == null;
				if (flag3)
				{
					pawn.story.traits = new TraitSet(pawn);
				}
				bool flag4 = pawn.story.traits.allTraits == null;
				if (flag4)
				{
					pawn.story.traits.allTraits = new List<Trait>();
				}
				Trait trait = new Trait(traitDef, (degreeData == null) ? traitDef.degreeDatas.FirstOrDefault<TraitDegreeData>().degree : degreeData.degree, false);
				bool flag5 = oldTraitToReplace != null;
				if (flag5)
				{
					bool flag6 = pawn.skills != null;
					if (flag6)
					{
						if (doChangeSkillValue)
						{
							TraitDegreeData currentData = oldTraitToReplace.CurrentData;
							bool flag7 = currentData != null && !currentData.skillGains.NullOrEmpty<SkillDef, int>();
							if (flag7)
							{
								foreach (SkillRecord skillRecord in pawn.skills.skills)
								{
									foreach (SkillDef skillDef in currentData.skillGains.Keys)
									{
										bool flag8 = skillRecord.def.defName == skillDef.defName;
										if (flag8)
										{
											skillRecord.Level -= currentData.skillGains[skillDef];
										}
									}
								}
							}
						}
					}
					pawn.story.traits.allTraits.Replace(oldTraitToReplace, trait);
				}
				else
				{
					pawn.story.traits.allTraits.Add(trait);
				}
				trait.pawn = pawn;
				pawn.Notify_DisabledWorkTypesChanged();
				bool flag9 = pawn.skills != null;
				if (flag9)
				{
					pawn.skills.Notify_SkillDisablesChanged();
					if (doChangeSkillValue)
					{
						bool flag10 = degreeData != null && !degreeData.skillGains.NullOrEmpty<SkillDef, int>();
						if (flag10)
						{
							foreach (SkillRecord skillRecord2 in pawn.skills.skills)
							{
								foreach (SkillDef skillDef2 in degreeData.skillGains.Keys)
								{
									bool flag11 = skillRecord2.def.defName == skillDef2.defName;
									if (flag11)
									{
										skillRecord2.Level += degreeData.skillGains[skillDef2];
									}
								}
							}
						}
					}
				}
				bool flag12 = !pawn.Dead && pawn.RaceProps.Humanlike && pawn.needs.mood != null;
				if (flag12)
				{
					pawn.needs.mood.thoughts.situational.Notify_SituationalThoughtsDirty();
				}
				MeditationFocusTypeAvailabilityCache.ClearFor(pawn);
				StatsReportUtility.Reset();
			}
		}

		// Token: 0x060002AC RID: 684 RVA: 0x00015100 File Offset: 0x00013300
		public static void RemoveTrait(this Pawn pawn, Trait t)
		{
			bool flag = t == null;
			if (!flag)
			{
				bool flag2 = t.CurrentData != null && !t.CurrentData.skillGains.NullOrEmpty<SkillDef, int>();
				if (flag2)
				{
					foreach (SkillRecord skillRecord in pawn.skills.skills)
					{
						foreach (SkillDef skillDef in t.CurrentData.skillGains.Keys)
						{
							bool flag3 = skillRecord.def.defName == skillDef.defName;
							if (flag3)
							{
								skillRecord.Level -= t.CurrentData.skillGains[skillDef];
							}
						}
					}
				}
				CEditor.API.Pawn.story.traits.allTraits.Remove(t);
				MeditationFocusTypeAvailabilityCache.ClearFor(pawn);
				StatsReportUtility.Reset();
			}
		}

		// Token: 0x060002AD RID: 685 RVA: 0x00015248 File Offset: 0x00013448
		public static TraitDegreeData GetDegreeDataOrFirst(this TraitDef t, int degree)
		{
			bool flag = t == null || t.degreeDatas.NullOrEmpty<TraitDegreeData>();
			TraitDegreeData result;
			if (flag)
			{
				result = null;
			}
			else
			{
				foreach (TraitDegreeData traitDegreeData in t.degreeDatas)
				{
					bool flag2 = traitDegreeData.degree == degree;
					if (flag2)
					{
						return traitDegreeData;
					}
				}
				result = t.degreeDatas.FirstOrDefault<TraitDegreeData>();
			}
			return result;
		}

		// Token: 0x060002AE RID: 686 RVA: 0x000152D8 File Offset: 0x000134D8
		public static TraitDef GetRandomTrait(out TraitDegreeData degreeData)
		{
			TraitDef traitDef = DefDatabase<TraitDef>.AllDefs.RandomElement<TraitDef>();
			degreeData = (traitDef.degreeDatas.NullOrEmpty<TraitDegreeData>() ? null : traitDef.degreeDatas.RandomElement<TraitDegreeData>());
			return traitDef;
		}

		// Token: 0x060002AF RID: 687 RVA: 0x00015314 File Offset: 0x00013514
		public static bool HasTrait(this Pawn pawn, TraitDef traitDef, TraitDegreeData degreeData)
		{
			bool flag = pawn == null || pawn.story == null || traitDef == null || pawn.story.traits == null || pawn.story.traits.allTraits.NullOrEmpty<Trait>();
			bool result;
			if (flag)
			{
				result = false;
			}
			else
			{
				int num = (degreeData == null) ? 0 : degreeData.degree;
				foreach (Trait trait in pawn.story.traits.allTraits)
				{
					bool flag2 = trait.def.defName == traitDef.defName && trait.Degree == num;
					if (flag2)
					{
						return true;
					}
				}
				result = false;
			}
			return result;
		}

		// Token: 0x060002B0 RID: 688 RVA: 0x000153F0 File Offset: 0x000135F0
		public static List<TraitDef> ListOfTraitDef(string modname)
		{
			bool bAll = modname.NullOrEmpty();
			return (from td in DefDatabase<TraitDef>.AllDefs
			where td != null && !td.defName.NullOrEmpty() && (bAll || td.IsFromMod(modname))
			orderby td.defName
			select td).ToList<TraitDef>();
		}

		// Token: 0x060002B1 RID: 689 RVA: 0x00015464 File Offset: 0x00013664
		public static List<StatModifier> ListOfTraitStatModifier(string modname, bool withNull)
		{
			List<TraitDef> list = TraitTool.ListOfTraitDef(modname);
      List<StatModifier> hashSet = new List<StatModifier>();
			List<string> list2 = new List<string>();
			if (withNull)
			{
				hashSet.Add(null);
			}
			bool flag = !list.NullOrEmpty<TraitDef>();
			if (flag)
			{
				foreach (TraitDef traitDef in list)
				{
					bool flag2 = traitDef != null && !traitDef.degreeDatas.NullOrEmpty<TraitDegreeData>();
					if (flag2)
					{
						foreach (TraitDegreeData traitDegreeData in traitDef.degreeDatas)
						{
							bool flag3 = traitDegreeData == null;
							if (!flag3)
							{
								bool flag4 = !traitDegreeData.statOffsets.NullOrEmpty<StatModifier>();
								if (flag4)
								{
									foreach (StatModifier statModifier in traitDegreeData.statOffsets)
									{
										bool flag5 = statModifier.stat != null && statModifier.stat.defName != null;
										if (flag5)
										{
											bool flag6 = !list2.Contains(statModifier.stat.defName);
											if (flag6)
											{
												list2.Add(statModifier.stat.defName);
												hashSet.Add(statModifier);
											}
										}
									}
								}
								bool flag7 = !traitDegreeData.statFactors.NullOrEmpty<StatModifier>();
								if (flag7)
								{
									foreach (StatModifier statModifier2 in traitDegreeData.statFactors)
									{
										bool flag8 = statModifier2.stat != null && statModifier2.stat.defName != null;
										if (flag8)
										{
											bool flag9 = !list2.Contains(statModifier2.stat.defName);
											if (flag9)
											{
												list2.Add(statModifier2.stat.defName);
												hashSet.Add(statModifier2);
											}
										}
									}
								}
							}
						}
					}
				}
			}
			return hashSet;
		}

		// Token: 0x060002B2 RID: 690 RVA: 0x0001570C File Offset: 0x0001390C
		public static List<KeyValuePair<TraitDef, TraitDegreeData>> ListOfTraitsKeyValuePair(string modname, StatModifier sm = null, string category = null)
		{
			List<TraitDef> list = TraitTool.ListOfTraitDef(modname);
			List<KeyValuePair<TraitDef, TraitDegreeData>> list2 = new List<KeyValuePair<TraitDef, TraitDegreeData>>();
			foreach (TraitDef traitDef in list)
			{
				bool flag = traitDef.degreeDatas.NullOrEmpty<TraitDegreeData>();
				if (flag)
				{
					list2.Add(new KeyValuePair<TraitDef, TraitDegreeData>(traitDef, null));
				}
				else
				{
					foreach (TraitDegreeData value in traitDef.degreeDatas)
					{
						list2.Add(new KeyValuePair<TraitDef, TraitDegreeData>(traitDef, value));
					}
				}
			}
			List<KeyValuePair<TraitDef, TraitDegreeData>> list3 = list2.OrderBy(delegate(KeyValuePair<TraitDef, TraitDegreeData> td)
			{
				KeyValuePair<TraitDef, TraitDegreeData> keyValuePair = td;
				string result;
				if (keyValuePair.Value == null)
				{
					keyValuePair = td;
					result = keyValuePair.Key.LabelCap.RawText;
				}
				else
				{
					keyValuePair = td;
					if ((result = keyValuePair.Value.LabelCap) == null)
					{
						keyValuePair = td;
						if ((result = keyValuePair.Value.label) == null)
						{
							keyValuePair = td;
							result = keyValuePair.Key.label;
						}
					}
				}
				return result;
			}).ToList<KeyValuePair<TraitDef, TraitDegreeData>>();
			bool flag2 = sm != null;
			if (flag2)
			{
				List<KeyValuePair<TraitDef, TraitDegreeData>> list4 = new List<KeyValuePair<TraitDef, TraitDegreeData>>();
				foreach (KeyValuePair<TraitDef, TraitDegreeData> item in list3)
				{
					bool flag3 = !item.Key.degreeDatas.NullOrEmpty<TraitDegreeData>();
					if (flag3)
					{
						foreach (TraitDegreeData traitDegreeData in item.Key.degreeDatas)
						{
							bool flag4 = traitDegreeData == null;
							if (!flag4)
							{
								bool flag5 = !traitDegreeData.statOffsets.NullOrEmpty<StatModifier>();
								if (flag5)
								{
									bool flag6 = traitDegreeData.statOffsets.Contains(sm);
									if (flag6)
									{
										list4.Add(item);
										break;
									}
								}
								bool flag7 = !traitDegreeData.statFactors.NullOrEmpty<StatModifier>();
								if (flag7)
								{
									bool flag8 = traitDegreeData.statFactors.Contains(sm);
									if (flag8)
									{
										list4.Add(item);
										break;
									}
								}
							}
						}
					}
				}
				list3 = list4;
			}
			bool flag9 = category != null;
			if (flag9)
			{
				List<KeyValuePair<TraitDef, TraitDegreeData>> list5 = new List<KeyValuePair<TraitDef, TraitDegreeData>>();
				foreach (KeyValuePair<TraitDef, TraitDegreeData> item2 in list3)
				{
					bool flag10 = !item2.Key.degreeDatas.NullOrEmpty<TraitDegreeData>();
					if (flag10)
					{
						foreach (TraitDegreeData traitDegreeData2 in item2.Key.degreeDatas)
						{
							bool flag11 = traitDegreeData2 == null;
							if (!flag11)
							{
								bool flag12 = category == Label.STAT;
								if (flag12)
								{
									bool flag13 = !traitDegreeData2.statFactors.NullOrEmpty<StatModifier>() || !traitDegreeData2.statOffsets.NullOrEmpty<StatModifier>();
									if (flag13)
									{
										list5.Add(item2);
										break;
									}
								}
								else
								{
									bool flag14 = category == Label.MENTAL;
									if (flag14)
									{
										bool flag15 = traitDegreeData2.mentalBreakInspirationGainChance != 0f || traitDegreeData2.forcedMentalStateMtbDays != -1f || traitDegreeData2.forcedMentalState != null || traitDegreeData2.randomMentalState != null || !traitDegreeData2.disallowedMentalStates.NullOrEmpty<MentalStateDef>() || !traitDegreeData2.theOnlyAllowedMentalBreaks.NullOrEmpty<MentalBreakDef>();
										if (flag15)
										{
											list5.Add(item2);
											break;
										}
									}
									else
									{
										bool flag16 = category == Label.THOUGHTS;
										if (flag16)
										{
											bool flag17 = !traitDegreeData2.disallowedThoughts.NullOrEmpty<ThoughtDef>() || !traitDegreeData2.disallowedThoughtsFromIngestion.NullOrEmpty<TraitIngestionThoughtsOverride>() || !traitDegreeData2.extraThoughtsFromIngestion.NullOrEmpty<TraitIngestionThoughtsOverride>();
											if (flag17)
											{
												list5.Add(item2);
												break;
											}
										}
										else
										{
											bool flag18 = category == Label.INSPIRATIONS;
											if (flag18)
											{
												bool flag19 = traitDegreeData2.mentalBreakInspirationGainChance != 0f || !traitDegreeData2.disallowedInspirations.NullOrEmpty<InspirationDef>() || !traitDegreeData2.mentalBreakInspirationGainSet.NullOrEmpty<InspirationDef>();
												if (flag19)
												{
													list5.Add(item2);
													break;
												}
											}
											else
											{
												bool flag20 = category == Label.FOCUS;
												if (flag20)
												{
													bool flag21 = !traitDegreeData2.allowedMeditationFocusTypes.NullOrEmpty<MeditationFocusDef>() || !traitDegreeData2.disallowedMeditationFocusTypes.NullOrEmpty<MeditationFocusDef>();
													if (flag21)
													{
														list5.Add(item2);
														break;
													}
												}
												else
												{
													bool flag22 = category == Label.SKILLGAINS;
													if (flag22)
													{
														bool flag23 = !traitDegreeData2.skillGains.NullOrEmpty<SkillDef, int>();
														if (flag23)
														{
															list5.Add(item2);
															break;
														}
													}
													else
													{
														bool flag24 = category == Label.ABILITIES;
														if (flag24)
														{
															bool flag25 = !traitDegreeData2.abilities.NullOrEmpty<AbilityDef>();
															if (flag25)
															{
																list5.Add(item2);
																break;
															}
														}
														else
														{
															bool flag26 = category == Label.NEEDS;
															if (flag26)
															{
																bool flag27 = !traitDegreeData2.needs.NullOrEmpty<NeedDef>();
																if (flag27)
																{
																	list5.Add(item2);
																	break;
																}
															}
															else
															{
																bool flag28 = category == Label.INGESTIBLEMOD;
																if (flag28)
																{
																	bool flag29 = !traitDegreeData2.ingestibleModifiers.NullOrEmpty<IngestibleModifiers>();
																	if (flag29)
																	{
																		list5.Add(item2);
																		break;
																	}
																}
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
				list3 = list5;
			}
			bool devMode = Prefs.DevMode;
			if (devMode)
			{
				MessageTool.Show("list count=" + list3.CountAllowNull<KeyValuePair<TraitDef, TraitDegreeData>>().ToString(), null);
			}
			return list3;
		}
	}
}
