﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using Verse;

namespace CharacterEditor
{
  // Token: 0x0200005F RID: 95
  public static class WeaponTool
  {
    // Token: 0x0600063E RID: 1598 RVA: 0x000416D8 File Offset: 0x0003F8D8
    public static bool HasAnyWeapon(this Pawn pawn)
    {
      return pawn.HasEquipmentTracker() && !pawn.equipment.AllEquipmentListForReading.NullOrEmpty<ThingWithComps>();
    }

    // Token: 0x0600063F RID: 1599 RVA: 0x000416F8 File Offset: 0x0003F8F8
    public static bool IsEquippedByPawn(this Pawn pawn, ThingWithComps w)
    {
      return pawn.HasAnyWeapon() && pawn.equipment.AllEquipmentListForReading.Contains(w);
    }

    // Token: 0x06000640 RID: 1600 RVA: 0x00041716 File Offset: 0x0003F916
    public static bool HasAnyWeaponTags(this ThingDef weapon)
    {
      return weapon != null && !weapon.weaponTags.NullOrEmpty<string>();
    }

    // Token: 0x06000641 RID: 1601 RVA: 0x0004172C File Offset: 0x0003F92C
    public static bool DoesExplosion(this ThingDef weapon)
    {
      return weapon.HasVerb() && weapon.Verbs[0].CausesExplosion;
    }

    // Token: 0x06000642 RID: 1602 RVA: 0x0004174A File Offset: 0x0003F94A
    public static bool HasVerb(this ThingDef weapon)
    {
      return weapon != null && !weapon.Verbs.NullOrEmpty<VerbProperties>();
    }

    // Token: 0x06000643 RID: 1603 RVA: 0x00041760 File Offset: 0x0003F960
    public static bool HasProjectile(this ThingDef weapon)
    {
      return weapon != null && !weapon.Verbs.NullOrEmpty<VerbProperties>() && weapon.Verbs[0].defaultProjectile != null && weapon.Verbs[0].defaultProjectile.projectile != null;
    }

    // Token: 0x06000644 RID: 1604 RVA: 0x000417AC File Offset: 0x0003F9AC
    public static bool HasSoundcast(this ThingDef weapon)
    {
      return weapon != null && !weapon.Verbs.NullOrEmpty<VerbProperties>() && weapon.Verbs[0].soundCast != null;
    }

    // Token: 0x06000645 RID: 1605 RVA: 0x000417D5 File Offset: 0x0003F9D5
    public static ThingWithComps RandomEquippedWeapon(this Pawn pawn)
    {
      return pawn.HasAnyWeapon() ? pawn.equipment.AllEquipmentListForReading.RandomElement<ThingWithComps>() : null;
    }

    // Token: 0x06000646 RID: 1606 RVA: 0x000417F2 File Offset: 0x0003F9F2
    public static ThingWithComps ThisOrFirstWeapon(this Pawn p, ThingWithComps w)
    {
      return p.HasAnyWeapon() ? (p.IsEquippedByPawn(w) ? w : p.equipment.AllEquipmentListForReading.FirstOrDefault<ThingWithComps>()) : null;
    }

    // Token: 0x06000647 RID: 1607 RVA: 0x0004181C File Offset: 0x0003FA1C
    public static string GetAllWeaponsAsSeparatedString(this Pawn p)
    {
      bool flag = !p.HasEquipmentTracker() || p.equipment.AllEquipmentListForReading.NullOrEmpty<ThingWithComps>();
      string result;
      if (flag)
      {
        result = "";
      }
      else
      {
        string text = "";
        foreach (ThingWithComps t in p.equipment.AllEquipmentListForReading)
        {
          text += t.GetAsSeparatedString();
          text += ":";
        }
        text = text.SubstringRemoveLast();
        result = text;
      }
      return result;
    }

    // Token: 0x06000648 RID: 1608 RVA: 0x000418C8 File Offset: 0x0003FAC8
    public static void SetWeaponsFromSeparatedString(this Pawn p, string s)
    {
      bool flag = !p.HasEquipmentTracker();
      if (!flag)
      {
        p.equipment.DestroyAllEquipment(DestroyMode.Vanish);
        bool flag2 = s.NullOrEmpty();
        if (!flag2)
        {
          string[] array = s.SplitNo(":");
          foreach (string s2 in array)
          {
            string[] array3 = s2.SplitNo("|");
            bool flag3 = array3.Length >= 6;
            if (flag3)
            {
              string styledefname = (array3.Length >= 7) ? array3[6] : "";
              ThingWithComps thingWithComps = WeaponTool.GenerateWeapon(Selected.ByName(array3[0], array3[1], styledefname, array3[2].HexStringToColor(), array3[3].AsInt32(), array3[4].AsInt32()));
              bool flag4 = thingWithComps != null;
              if (flag4)
              {
                thingWithComps.HitPoints = array3[5].AsInt32();
                p.equipment.AddEquipment(thingWithComps);
              }
            }
          }
        }
      }
    }

    // Token: 0x06000649 RID: 1609 RVA: 0x000419C4 File Offset: 0x0003FBC4
    public static void DestroyAllEquipment(this Pawn pawn)
    {
      bool flag = !pawn.HasEquipmentTracker();
      if (!flag)
      {
        pawn.equipment.DestroyAllEquipment(DestroyMode.Vanish);
      }
    }

    // Token: 0x0600064A RID: 1610 RVA: 0x000419F0 File Offset: 0x0003FBF0
    public static void DestroyEquipment(this Pawn pawn, ThingWithComps weapon)
    {
      bool flag = !pawn.HasEquipmentTracker();
      if (!flag)
      {
        pawn.equipment.DestroyEquipment(weapon);
      }
    }

    // Token: 0x0600064B RID: 1611 RVA: 0x00041A1A File Offset: 0x0003FC1A
    public static List<ThingWithComps> ListOfCopyWeapons(this Pawn pawn)
    {
      return pawn.HasAnyWeapon() ? pawn.equipment.AllEquipmentListForReading.ListFullCopy<ThingWithComps>() : null;
    }

    // Token: 0x0600064C RID: 1612 RVA: 0x00041A38 File Offset: 0x0003FC38
    public static void PasteCopyWeapons(this Pawn pawn, List<ThingWithComps> l)
    {
      bool flag = pawn.HasEquipmentTracker();
      if (flag)
      {
        pawn.DestroyAllEquipment();
        bool flag2 = !l.NullOrEmpty<ThingWithComps>();
        if (flag2)
        {
          bool firstWeapon = true;
          foreach (ThingWithComps thing in l)
          {
            ThingWithComps w = WeaponTool.GenerateWeapon(Selected.ByThing(thing));
            pawn.AddWeaponToEquipment(w, firstWeapon, true);
            firstWeapon = false;
          }
        }
      }
      CEditor.API.UpdateGraphics();
    }

    // Token: 0x0600064D RID: 1613 RVA: 0x00041AD0 File Offset: 0x0003FCD0
    public static void CreateAndWearEquipment(this Pawn pawn, Selected s, bool firstWeapon)
    {
      bool flag = !pawn.HasEquipmentTracker();
      if (!flag)
      {
        ThingWithComps w = WeaponTool.GenerateWeapon(s);
        pawn.AddWeaponToEquipment(w, firstWeapon, true);
      }
    }

    // Token: 0x0600064E RID: 1614 RVA: 0x00041B00 File Offset: 0x0003FD00
    public static void AddWeaponToEquipment(this Pawn pawn, ThingWithComps w, bool firstWeapon, bool destroyOld = true)
    {
      bool flag = !pawn.HasEquipmentTracker() || w == null;
      if (!flag)
      {
        if (firstWeapon)
        {
          bool flag2 = pawn.equipment.Primary != null;
          if (flag2)
          {
            if (destroyOld)
            {
              pawn.equipment.Primary.Destroy(DestroyMode.Vanish);
            }
            else
            {
              ThingWithComps primary = pawn.equipment.Primary;
              pawn.equipment.Remove(primary);
              pawn.AddItemToInventory(primary);
            }
          }
          pawn.equipment.AddEquipment(w);
        }
        else
        {
          bool isDualWieldActive = CEditor.IsDualWieldActive;
          if (isDualWieldActive)
          {
            try
            {
              Type atype = Reflect.GetAType("DualWield", "Ext_Pawn_EquipmentTracker");
              atype.CallMethod("MakeRoomForOffHand", new object[]
              {
                pawn.equipment,
                w
              });
              atype.CallMethod("AddOffHandEquipment", new object[]
              {
                pawn.equipment,
                w
              });
            }
            catch
            {
            }
          }
        }
      }
    }

    // Token: 0x0600064F RID: 1615 RVA: 0x00041C08 File Offset: 0x0003FE08
    public static ThingWithComps GenerateRandomWeapon(bool originalColors = true)
    {
      WeaponType weaponType = (CEditor.zufallswert.Next(0, 100) > 50) ? WeaponType.Ranged : WeaponType.Melee;
      List<ThingDef> l = WeaponTool.ListOfWeapons(null, null, weaponType);
      return WeaponTool.GenerateWeapon(Selected.Random(l, originalColors, false));
    }

    // Token: 0x06000650 RID: 1616 RVA: 0x00041C48 File Offset: 0x0003FE48
    public static ThingWithComps GenerateWeapon(Selected s)
    {
      bool flag = s == null || s.thingDef == null;
      ThingWithComps result;
      if (flag)
      {
        result = null;
      }
      else
      {
        ThingDef thingDef = DefTool.ThingDef(s.thingDef.defName);
        bool flag2 = thingDef == null || !thingDef.IsWeapon;
        if (flag2)
        {
          result = null;
        }
        else
        {
          s.Stuff = s.thingDef.ThisOrDefaultStuff(s.Stuff);
          bool flag3 = !s.thingDef.MadeFromStuff;
          if (flag3)
          {
            s.Stuff = null;
          }
          ThingWithComps thingWithComps = (ThingWithComps)ThingMaker.MakeThing(s.thingDef, s.Stuff);
          thingWithComps.HitPoints = thingWithComps.MaxHitPoints;
          thingWithComps.SetQuality(s.quality);
          thingWithComps.SetDrawColor(s.DrawColor);
          thingWithComps.stackCount = s.stackVal;
          bool flag4 = s.Style != null;
          if (flag4)
          {
            thingWithComps.StyleDef = s.Style;
            thingWithComps.StyleDef.color = s.Style.color;
          }
          result = thingWithComps;
        }
      }
      return result;
    }

    // Token: 0x06000651 RID: 1617 RVA: 0x00041D54 File Offset: 0x0003FF54
    public static bool IsTurret(this ThingDef t)
    {
      bool flag = t == null;
      return !flag && (t.thingClass == typeof(Building_TurretGun) || (t.thingClass != null && CEditor.IsCombatExtendedActive && t.thingClass.ToString().Contains("Building_TurretGun")) || (t.building != null && t.building.turretGunDef != null));
    }

    // Token: 0x06000652 RID: 1618 RVA: 0x00041DD4 File Offset: 0x0003FFD4
    public static bool IsTurretGun(this ThingDef t)
    {
      return t != null && t.weaponTags != null && t.weaponTags.Contains("TurretGun");
    }

    // Token: 0x06000653 RID: 1619 RVA: 0x00041E04 File Offset: 0x00040004
    public static bool IsBullet(this ThingDef t)
    {
      return t != null && t.projectile != null;
    }

    // Token: 0x06000654 RID: 1620 RVA: 0x00041E18 File Offset: 0x00040018
    public static void PasteWeaponTag(this ThingDef weapon, List<string> l)
    {
      bool flag = weapon == null || l.NullOrEmpty<string>();
      if (!flag)
      {
        bool flag2 = weapon.weaponTags == null;
        if (flag2)
        {
          weapon.weaponTags = new List<string>();
        }
        foreach (string tag in l)
        {
          weapon.AddWeaponTag(tag);
        }
      }
    }

    // Token: 0x06000655 RID: 1621 RVA: 0x00041E94 File Offset: 0x00040094
    public static bool HasWeaponTag(this ThingDef weapon, string tag)
    {
      return weapon.HasAnyWeaponTags() && weapon.weaponTags.Contains(tag);
    }

    // Token: 0x06000656 RID: 1622 RVA: 0x00041EB0 File Offset: 0x000400B0
    public static void AddWeaponTag(this ThingDef t, string tag)
    {
      bool flag = t == null || tag == null;
      if (!flag)
      {
        bool flag2 = t.weaponTags == null;
        if (flag2)
        {
          t.weaponTags = new List<string>();
        }
        bool flag3 = !t.HasWeaponTag(tag);
        if (flag3)
        {
          t.weaponTags.Add(tag);
        }
        t.ResolveReferences();
      }
    }

    // Token: 0x06000657 RID: 1623 RVA: 0x00041F08 File Offset: 0x00040108
    public static void RemoveWeaponTag(this ThingDef weapon, string tag)
    {
      bool flag = !weapon.HasAnyWeaponTags();
      if (!flag)
      {
        foreach (string a in weapon.weaponTags)
        {
          bool flag2 = a == tag;
          if (flag2)
          {
            weapon.weaponTags.Remove(tag);
            break;
          }
        }
        weapon.ResolveReferences();
      }
    }

    // Token: 0x06000658 RID: 1624 RVA: 0x00041F8C File Offset: 0x0004018C
    public static void SetBullet(this ThingDef weapon, ThingDef bullet)
    {
      bool flag = !weapon.HasVerb();
      if (!flag)
      {
        weapon.Verbs[0].defaultProjectile = bullet;
        bool flag2 = weapon.Verbs[0].defaultProjectile != null;
        if (flag2)
        {
          weapon.Verbs[0].defaultProjectile.ResolveReferences();
        }
      }
    }

    // Token: 0x06000659 RID: 1625 RVA: 0x00041FEA File Offset: 0x000401EA
    public static string GetBulletDefName(this ThingDef weapon)
    {
      return (!weapon.HasProjectile()) ? "" : weapon.Verbs[0].defaultProjectile.defName;
    }

    // Token: 0x0600065A RID: 1626 RVA: 0x00042011 File Offset: 0x00040211
    public static string GetSoundCastDefName(this ThingDef weapon)
    {
      return (!weapon.HasSoundcast()) ? "" : weapon.Verbs[0].soundCast.defName;
    }

    // Token: 0x0600065B RID: 1627 RVA: 0x00042038 File Offset: 0x00040238
    public static float GetBulletExplosionRadius(this ThingDef weapon)
    {
      return (!weapon.HasProjectile()) ? 0f : weapon.Verbs[0].defaultProjectile.projectile.explosionRadius;
    }

    // Token: 0x0600065C RID: 1628 RVA: 0x00042064 File Offset: 0x00040264
    public static float GetBulletStoppingPower(this ThingDef weapon)
    {
      return (!weapon.HasProjectile()) ? 0f : weapon.Verbs[0].defaultProjectile.projectile.stoppingPower;
    }

    // Token: 0x0600065D RID: 1629 RVA: 0x00042090 File Offset: 0x00040290
    public static float GetBulletSpeed(this ThingDef weapon)
    {
      return (!weapon.HasProjectile()) ? 0f : weapon.Verbs[0].defaultProjectile.projectile.speed;
    }

    // Token: 0x0600065E RID: 1630 RVA: 0x000420BC File Offset: 0x000402BC
    public static void SetBulletSpeed(this ThingDef weapon, float speed)
    {
      bool flag = !weapon.HasProjectile();
      if (!flag)
      {
        weapon.Verbs[0].defaultProjectile.projectile.speed = speed;
      }
    }

    // Token: 0x0600065F RID: 1631 RVA: 0x000420F8 File Offset: 0x000402F8
    public static void SetStoppingPower(this ThingDef weapon, float power)
    {
      bool flag = !weapon.HasProjectile();
      if (!flag)
      {
        weapon.Verbs[0].defaultProjectile.projectile.stoppingPower = power;
      }
    }

    // Token: 0x06000660 RID: 1632 RVA: 0x00042134 File Offset: 0x00040334
    public static void SetExplosionRadius(this ThingDef weapon, float radius)
    {
      bool flag = !weapon.HasProjectile();
      if (!flag)
      {
        weapon.Verbs[0].defaultProjectile.projectile.explosionRadius = radius;
      }
    }

    // Token: 0x06000661 RID: 1633 RVA: 0x0004216D File Offset: 0x0004036D
    public static bool GetWeaponTargetGround(this ThingDef weapon)
    {
      return weapon.HasVerb() && weapon.Verbs[0].targetParams != null && weapon.Verbs[0].targetParams.canTargetLocations;
    }

    // Token: 0x06000662 RID: 1634 RVA: 0x000421A4 File Offset: 0x000403A4
    public static int GetDmg(this ThingDef weapon)
    {
      bool flag = weapon == null;
      int result;
      if (flag)
      {
        result = 0;
      }
      else
      {
        bool isMeleeWeapon = weapon.IsMeleeWeapon;
        if (isMeleeWeapon)
        {
          float num = weapon.GetStatValue(StatDefOf.MeleeWeapon_DamageMultiplier);
          num = (weapon.HasStat(StatDefOf.MeleeWeapon_DamageMultiplier) ? num : 1f);
          result = ((weapon.tools != null) ? ((int)(weapon.tools[0].power * num)) : 0);
        }
        else
        {
          float num2 = weapon.GetStatValue(StatDefOf.RangedWeapon_DamageMultiplier);
          num2 = (weapon.HasStat(StatDefOf.RangedWeapon_DamageMultiplier) ? num2 : 1f);
          bool flag2 = weapon.IsTurret();
          if (flag2)
          {
            result = ((weapon.building.turretGunDef.Verbs.Count > 0 && weapon.building.turretGunDef.Verbs[0].defaultProjectile != null) ? weapon.building.turretGunDef.Verbs[0].defaultProjectile.projectile.GetDamageAmount(num2, null) : 0);
          }
          else
          {
            result = ((weapon.Verbs.Count > 0 && weapon.Verbs[0].defaultProjectile != null && weapon.Verbs[0].defaultProjectile.projectile != null) ? weapon.Verbs[0].defaultProjectile.projectile.GetDamageAmount(num2, null) : 0);
          }
        }
      }
      return result;
    }

    // Token: 0x06000663 RID: 1635 RVA: 0x00042308 File Offset: 0x00040508
    public static void SetTargetParams(this ThingDef weapon, bool canTargetGround)
    {
      bool flag = !weapon.HasVerb();
      if (!flag)
      {
        bool flag2 = weapon.Verbs[0].targetParams == null;
        if (flag2)
        {
          weapon.Verbs[0].targetParams = new TargetingParameters();
        }
        weapon.Verbs[0].targetParams.canTargetLocations = canTargetGround;
      }
    }

    // Token: 0x06000664 RID: 1636 RVA: 0x0004236C File Offset: 0x0004056C
    public static ThingDef GetTurretDef(this ThingDef gun)
    {
      bool flag = gun == null || gun.defName.NullOrEmpty();
      ThingDef result;
      if (flag)
      {
        result = null;
      }
      else
      {
        Dictionary<string, ThingDef> allTurrets = PresetWeapon.GetAllTurrets();
        bool flag2 = allTurrets != null;
        if (flag2)
        {
          string text = allTurrets.KeyByValue(gun);
          bool flag3 = !text.NullOrEmpty();
          if (flag3)
          {
            result = DefTool.ThingDef(text);
          }
          else
          {
            bool flag4 = allTurrets.ContainsKey(gun.defName);
            if (flag4)
            {
              result = gun;
            }
            else
            {
              result = null;
            }
          }
        }
        else
        {
          result = null;
        }
      }
      return result;
    }

    // Token: 0x06000665 RID: 1637 RVA: 0x000423E4 File Offset: 0x000405E4
    public static ThingDef GetRealWeaponDef(this ThingDef gun)
    {
      bool flag = gun == null || gun.defName.NullOrEmpty();
      ThingDef result;
      if (flag)
      {
        result = null;
      }
      else
      {
        Dictionary<string, ThingDef> allTurrets = PresetWeapon.GetAllTurrets();
        bool flag2 = allTurrets != null;
        if (flag2)
        {
          string str = allTurrets.KeyByValue(gun);
          bool flag3 = !str.NullOrEmpty();
          if (flag3)
          {
            result = gun;
          }
          else
          {
            bool flag4 = allTurrets.ContainsKey(gun.defName);
            if (flag4)
            {
              result = allTurrets[gun.defName];
            }
            else
            {
              result = gun;
            }
          }
        }
        else
        {
          result = null;
        }
      }
      return result;
    }

    // Token: 0x06000666 RID: 1638 RVA: 0x00042464 File Offset: 0x00040664
    public static Selected SelectorForPawnSpecificWeapon(Pawn pawn)
    {
      Selected result;
      try
      {
        string modname = pawn.kindDef.GetModName();
        WeaponType weaponType = (WeaponType)CEditor.zufallswert.Next(0, 2);
        List<ThingDef> source = WeaponTool.ListOfWeapons(null, null, weaponType);
        List<ThingDef> hashSet = WeaponTool.ListOfWeapons(null, modname, weaponType);
        bool flag = hashSet.NullOrEmpty<ThingDef>() || hashSet.Count < 4;
        List<ThingDef> l;
        if (flag)
        {
          l = (from td in source
               where td.IsFromMod(modname) || (td.weaponTags != null
               && (from t in td.weaponTags
                   select pawn.kindDef.weaponTags.Contains(t)) != null)
               select td).ToList<ThingDef>();
        }
        else
        {
          l = hashSet;
        }
        result = Selected.Random(l, true, false);
      }
      catch
      {
        result = null;
      }
      return result;
    }

    // Token: 0x06000667 RID: 1639 RVA: 0x00042530 File Offset: 0x00040730
    public static bool IsPrimaryWeapon(this Pawn pawn, ThingWithComps w)
    {
      return w != null && w == CEditor.API.Pawn.equipment.Primary;
    }

    // Token: 0x06000668 RID: 1640 RVA: 0x00042550 File Offset: 0x00040750
    public static void Reequip(this Pawn pawn, Selected selected, int primary = 0, bool pawnSpecific = false)
    {
      bool flag = !pawn.HasEquipmentTracker();
      if (!flag)
      {
        primary = ((primary < 0) ? CEditor.zufallswert.Next(0, CEditor.IsDualWieldActive ? 3 : 1) : primary);
        int num = pawn.equipment.AllEquipmentListForReading.CountAllowNull<ThingWithComps>();
        bool flag2 = primary != 0 && num > 0;
        if (flag2)
        {
          for (int i = 0; i < pawn.equipment.AllEquipmentListForReading.CountAllowNull<ThingWithComps>(); i++)
          {
            ThingWithComps thingWithComps = pawn.equipment.AllEquipmentListForReading[i];
            bool flag3 = pawn.equipment.Primary == thingWithComps && (primary == 0 || primary >= 2);
            if (flag3)
            {
              pawn.equipment.DestroyEquipment(thingWithComps);
            }
            bool flag4 = pawn.equipment.Primary != thingWithComps && (primary == 1 || primary >= 2);
            if (flag4)
            {
              pawn.equipment.DestroyEquipment(thingWithComps);
            }
          }
        }
        bool flag5 = pawnSpecific && selected == null;
        if (flag5)
        {
          selected = WeaponTool.SelectorForPawnSpecificWeapon(pawn);
        }
        ThingWithComps thingWithComps2 = null;
        int num2 = 3;
        ThingWithComps w;
        do
        {
          w = ((selected == null) ? WeaponTool.GenerateRandomWeapon(true) : WeaponTool.GenerateWeapon(selected));
          bool flag6 = primary <= 0 || (primary == 1 && pawn.equipment.Primary.DestroyedOrNull()) || CEditor.InStartingScreen;
          if (flag6)
          {
            pawn.AddWeaponToEquipment(w, true, true);
          }
          else
          {
            bool flag7 = primary == 1;
            if (flag7)
            {
              pawn.AddWeaponToEquipment(w, false, true);
            }
            else
            {
              bool flag8 = primary == 2;
              if (flag8)
              {
                pawn.AddWeaponToEquipment(w, true, true);
                thingWithComps2 = WeaponTool.GenerateRandomWeapon(true);
                pawn.AddWeaponToEquipment(thingWithComps2, false, true);
              }
            }
          }
          num2--;
          primary = CEditor.zufallswert.Next(0, CEditor.IsDualWieldActive ? 3 : 1);
        }
        while (!pawn.IsEquippedByPawn(w) && !pawn.IsEquippedByPawn(thingWithComps2) && num2 > 0);
        bool flag9 = selected != null;
        if (flag9)
        {
          bool flag10 = primary == 1 && pawn.equipment.Primary != null;
          if (flag10)
          {
            pawn.equipment.Primary.DrawColor = selected.DrawColor;
          }
          bool flag11 = primary == 2 && thingWithComps2 != null;
          if (flag11)
          {
            thingWithComps2.DrawColor = selected.DrawColor;
          }
        }
      }
    }

    // Token: 0x06000669 RID: 1641 RVA: 0x0004279C File Offset: 0x0004099C
    public static void SetCompRefuelable(ThingDef turretDef, ThingDef weapon, int value)
    {
      bool flag = !CEditor.IsCombatExtendedActive && turretDef != null && turretDef.comps != null;
      if (flag)
      {
        CompProperties compByType = turretDef.GetCompByType(typeof(CompProperties_Refuelable));
        bool flag2 = compByType != null;
        if (flag2)
        {
          (compByType as CompProperties_Refuelable).fuelCapacity = (float)value;
        }
      }
      else
      {
        bool flag3 = CEditor.IsCombatExtendedActive && weapon != null && weapon.comps != null;
        if (flag3)
        {
          CompProperties compByType2 = weapon.GetCompByType("CombatExtended.CompProperties_AmmoUser");
          bool flag4 = compByType2 != null;
          if (flag4)
          {
            compByType2.SetMemberValue("magazineSize", value);
          }
        }
      }
    }

    // Token: 0x0600066A RID: 1642 RVA: 0x00042838 File Offset: 0x00040A38
    public static void SetCompReloadTime(ThingDef turretDef, ThingDef weapon, float value)
    {
      bool flag = CEditor.IsCombatExtendedActive && weapon != null && weapon.comps != null;
      if (flag)
      {
        CompProperties compByType = weapon.GetCompByType("CombatExtended.CompProperties_AmmoUser");
        bool flag2 = compByType != null;
        if (flag2)
        {
          compByType.SetMemberValue("reloadTime", value);
        }
      }
    }

    // Token: 0x0600066B RID: 1643 RVA: 0x00042888 File Offset: 0x00040A88
    public static float GetCompReloadTime(ThingDef turretDef, ThingDef weapon)
    {
      bool flag = CEditor.IsCombatExtendedActive && weapon != null && weapon.comps != null;
      if (flag)
      {
        CompProperties compByType = weapon.GetCompByType("CombatExtended.CompProperties_AmmoUser");
        bool flag2 = compByType != null;
        if (flag2)
        {
          return (float)compByType.GetMemberValue("reloadTime");
        }
      }
      return 0f;
    }

    // Token: 0x0600066C RID: 1644 RVA: 0x000428E4 File Offset: 0x00040AE4
    public static int GetCompRefuelable(ThingDef turretDef, ThingDef weapon)
    {
      bool flag = !CEditor.IsCombatExtendedActive && turretDef != null && turretDef.comps != null;
      if (flag)
      {
        CompProperties compByType = turretDef.GetCompByType(typeof(CompProperties_Refuelable));
        bool flag2 = compByType != null;
        if (flag2)
        {
          return (int)(compByType as CompProperties_Refuelable).fuelCapacity;
        }
      }
      else
      {
        bool flag3 = CEditor.IsCombatExtendedActive && weapon != null && weapon.comps != null;
        if (flag3)
        {
          CompProperties compByType2 = weapon.GetCompByType("CombatExtended.CompProperties_AmmoUser");
          bool flag4 = compByType2 != null;
          if (flag4)
          {
            return (int)compByType2.GetMemberValue("magazineSize");
          }
        }
      }
      return 0;
    }

    // Token: 0x0600066D RID: 1645 RVA: 0x00042988 File Offset: 0x00040B88
    public static List<ThingDef> ListOfTurrets()
    {
      return (from td in DefDatabase<ThingDef>.AllDefs
              where !td.defName.NullOrEmpty() && !td.label.NullOrEmpty() && td.IsTurret()
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x0600066E RID: 1646 RVA: 0x000429F0 File Offset: 0x00040BF0
    public static List<ThingDef> ListOfWeapons(ThingCategoryDef tc, string modname, WeaponType weaponType)
    {
      Dictionary<string, ThingDef> dicTurrets = PresetWeapon.GetAllTurrets();
      bool bAll1 = modname.NullOrEmpty();
      bool bAll2 = tc == null;
      return (from td in DefDatabase<ThingDef>.AllDefs
              where !td.label.NullOrEmpty() && ((td.IsMeleeWeapon && weaponType == WeaponType.Melee) || (td.IsRangedWeapon && weaponType == WeaponType.Ranged && !dicTurrets.Values.Contains(td)) || (td.IsTurret() && weaponType == WeaponType.Turret)) && (bAll1 || td.IsFromMod(modname)) && (bAll2 || (!td.thingCategories.NullOrEmpty<ThingCategoryDef>() && td.thingCategories.Contains(tc)))
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x0600066F RID: 1647 RVA: 0x00042A8C File Offset: 0x00040C8C
    public static List<ThingDef> ListOfBullets(ThingCategoryDef tc, string modname)
    {
      bool bAll1 = modname.NullOrEmpty();
      bool bAll2 = tc == null;
      return (from td in DefDatabase<ThingDef>.AllDefs
              where td.IsBullet() && !td.label.NullOrEmpty() && (bAll1 || td.IsFromMod(modname)) && (bAll2 || (!td.thingCategories.NullOrEmpty<ThingCategoryDef>() && td.thingCategories.Contains(tc)))
              orderby td.label
              select td).ToList<ThingDef>();
    }

    // Token: 0x06000670 RID: 1648 RVA: 0x00042B14 File Offset: 0x00040D14
    public static List<SoundDef> ListOfSounds(bool all)
    {
      List<SoundDef> result;
      if (all)
      {
        List<SoundDef> list = (from td in DefDatabase<SoundDef>.AllDefs
                               where !td.sustain
                               orderby td.defName
                               select td).ToList<SoundDef>();
        list.RemoveDuplicates(null);
        result = list;
      }
      else
      {
        List<SoundDef> list2 = (from td in DefDatabase<ThingDef>.AllDefs
                                where td.IsWeapon && td.HasSoundcast()
                                orderby td.Verbs[0].soundCast.defName
                                select td.Verbs[0].soundCast).ToList<SoundDef>();
        list2.RemoveDuplicates(null);
        result = list2;
      }
      return result;
    }
  }
}
