﻿using System;
using System.Collections.Generic;
using UnityEngine;
using Verse;

namespace CharacterEditor
{
	// Token: 0x02000024 RID: 36
	public static class WindowTool
	{
		// Token: 0x0600018B RID: 395 RVA: 0x0000C7C4 File Offset: 0x0000A9C4
		public static void Open(Window w)
		{
			w.layer = CEditor.Layer;
			Find.WindowStack.Add(w);
		}

		// Token: 0x0600018C RID: 396 RVA: 0x0000C7E0 File Offset: 0x0000A9E0
		public static void TopLayerForWindowOf<T>() where T : Window
		{
			Window w = Find.WindowStack.WindowOfType<T>();
			WindowTool.BringToFront(w);
		}

		// Token: 0x0600018D RID: 397 RVA: 0x0000C808 File Offset: 0x0000AA08
		public static void TopLayerForWindowOfType(string type)
		{
			Window windowOfType = WindowTool.GetWindowOfType(type);
			WindowTool.BringToFront(windowOfType);
		}

		// Token: 0x0600018E RID: 398 RVA: 0x0000C824 File Offset: 0x0000AA24
		public static void BringToFront(Window w)
		{
			bool flag = w != null && w.layer != CEditor.Layer;
			if (flag)
			{
				w.layer = CEditor.Layer;
				GUI.BringWindowToFront(w.ID);
			}
		}

		// Token: 0x0600018F RID: 399 RVA: 0x0000C868 File Offset: 0x0000AA68
		public static void BringToFrontMulti(List<Window> l)
		{
			bool flag = l.NullOrEmpty<Window>();
			if (!flag)
			{
				l.Reverse();
				foreach (Window w in l)
				{
					WindowTool.BringToFront(w);
				}
			}
		}

		// Token: 0x06000190 RID: 400 RVA: 0x0000C8CC File Offset: 0x0000AACC
		public static Window GetWindowOf<T>() where T : Window
		{
			return Find.WindowStack.WindowOfType<T>();
		}

		// Token: 0x06000191 RID: 401 RVA: 0x0000C8DD File Offset: 0x0000AADD
		public static bool IsOpen<T>() where T : Window
		{
			return Find.WindowStack.IsOpen(typeof(T));
		}

		// Token: 0x06000192 RID: 402 RVA: 0x0000C8F3 File Offset: 0x0000AAF3
		public static void TryRemove<T>() where T : Window
		{
			Find.WindowStack.TryRemove(typeof(T), true);
		}

		// Token: 0x06000193 RID: 403 RVA: 0x0000C90C File Offset: 0x0000AB0C
		public static Window GetWindowOfType(string type)
		{
			foreach (Window window in Find.WindowStack.Windows)
			{
				bool flag = window.GetType().ToString() == type;
				if (flag)
				{
					return window;
				}
			}
			return null;
		}

		// Token: 0x06000194 RID: 404 RVA: 0x0000C97C File Offset: 0x0000AB7C
		public static void ShowOpenedWindows()
		{
			foreach (Window window in Find.WindowStack.Windows)
			{
				MessageTool.Show(window.GetType().ToString(), null);
			}
		}

		// Token: 0x06000195 RID: 405 RVA: 0x0000C9E0 File Offset: 0x0000ABE0
		public static List<Window> GetWindowOfStartsWithType(string type)
		{
			List<Window> list = new List<Window>();
			foreach (Window window in Find.WindowStack.Windows)
			{
				bool flag = window.GetType().ToString().StartsWith(type);
				if (flag)
				{
					list.Add(window);
				}
			}
			return list;
		}

		// Token: 0x06000196 RID: 406 RVA: 0x0000CA5C File Offset: 0x0000AC5C
		public static List<Window> GetWindowOfEndsWithType(string type)
		{
			List<Window> list = new List<Window>();
			foreach (Window window in Find.WindowStack.Windows)
			{
				bool flag = window.GetType().ToString().EndsWith(type);
				if (flag)
				{
					list.Add(window);
				}
			}
			return list;
		}

		// Token: 0x06000197 RID: 407 RVA: 0x0000CAD8 File Offset: 0x0000ACD8
		public static void Close(List<Window> l)
		{
			bool flag = l.NullOrEmpty<Window>();
			if (!flag)
			{
				foreach (Window window in l)
				{
					window.Close(true);
				}
			}
		}
	}
}
