﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using HarmonyLib;
using RimWorld;
using UnityEngine;
using Verse;
using Verse.Sound;
using Psychology;

namespace CharacterEditor
{
  [HarmonyPatch(typeof(CompatibilityTool), nameof(CompatibilityTool.GetPsychology))]
  public static class CharacterEditor_CompatibilityTools_GetPsychology_Patch
  {
    [HarmonyPostfix]
    public static void GetPsychology(Pawn pawn, string __result)
    {
      if (!PsycheHelper.PsychologyEnabled(pawn))
      {
        return;
      }
      //List<Pair<string, float>> psychologyList = new List<Pair<string, float>>();
      Dictionary<string, float> psychologyDict = new Dictionary<string, float>();
      CompPsychology comp = PsycheHelper.Comp(pawn);
      Pawn_SexualityTracker sexuality = comp.Sexuality;
      Pawn_PsycheTracker psyche = comp.Psyche;

      ////Log.Message("nameof(sexuality.kinseyRating) = " + nameof(sexuality.kinseyRating));
      ////Log.Message("nameof(sexuality.sexDrive) = " + nameof(sexuality.sexDrive));

      psychologyDict[nameof(sexuality.kinseyRating)] = sexuality.kinseyRating;
      psychologyDict[nameof(sexuality.sexDrive)] =  sexuality.sexDrive;
      psychologyDict[nameof(sexuality.romanticDrive)] = sexuality.romanticDrive;
      foreach (PersonalityNode node in comp.Psyche.PersonalityNodes)
      {
        psychologyDict[node.def.defName] = sexuality.kinseyRating;
      }
      List<string> psychologyList = new List<string>();
      foreach (KeyValuePair<string, float> kvp in psychologyDict)
      {
        psychologyList.Add(kvp.Key + ":" + kvp.Value);
      }
      __result = string.Join("|", psychologyList);
    }
  }

  [HarmonyPatch(typeof(CompatibilityTool), nameof(CompatibilityTool.SetPsychology))]
  public static class CharacterEditor_CompatibilityTools_SetPsychology_Patch
  {
    [HarmonyPostfix]
    public static void SetPsychology(Pawn pawn, string psychologyDataAsString)
    {
      if (!PsycheHelper.PsychologyEnabled(pawn))
      {
        return;
      }
      List<Pair<string, float>> psychologyList = new List<Pair<string, float>>();
      CompPsychology comp = PsycheHelper.Comp(pawn);
      Pawn_SexualityTracker sexuality = comp.Sexuality;
      Pawn_PsycheTracker psyche = comp.Psyche;

      Dictionary<string, float> psychologyDict = new Dictionary<string, float>();

      string[] array = psychologyDataAsString.Split('|');

      float parsedFloat;
      foreach (string entry in array)
      {
        string[] pair = entry.Split(':');
        if (pair.Count() < 2)
        {
          continue;
        }
        if (float.TryParse(pair[1], out parsedFloat))
        {
          psychologyDict[pair[0]] = parsedFloat;
        }
      }
      if (psychologyDict.TryGetValue(nameof(sexuality.kinseyRating), out parsedFloat))
      {
        sexuality.kinseyRating = Mathf.RoundToInt(parsedFloat);
      }
      if (psychologyDict.TryGetValue(nameof(sexuality.sexDrive), out parsedFloat))
      {
        sexuality.sexDrive = parsedFloat;
      }
      if (psychologyDict.TryGetValue(nameof(sexuality.romanticDrive), out parsedFloat))
      {
        sexuality.romanticDrive = parsedFloat;
      }
      foreach (PersonalityNode node in psyche.PersonalityNodes)
      {
        if (psychologyDict.TryGetValue(node.def.defName, out parsedFloat))
        {
          node.rawRating = parsedFloat;
        }
      }
    }
  }


}



